/* $Header: /Src/RhinoIO/3dm.h 21    10/05/99 6:31p Dalelear $      */
/* $NoKeywords: $  */
/*
//
// (c) 1997 Robert McNeel & Associates  All rights reserved
//
// THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
// ALL IMPLIED WARRANTIES OF FITNESS FOR ANY PARTICULAR PURPOSE AND OF
// MERCHANTABILITY ARE HEREBY DISCLAIMED.
//
//   Defines Rhino type codes
//
////////////////////////////////////////////////////////////////
*/

#if !defined(RHINO_THREEDM_INC_)
#define RHINO_THREEDM_INC_

/* 3dm defines, structs and typedefs */

/*  Typecode format 4 bytes long
   x xxxxxxxxxxxxxxx,x xxx xxxx xxxx x x xx
  | |               | |               | |  |
   |        |        |                 |
   |        |        |         |       +---  "stuff" bit
   |        |        |         |
   |        |        |         +-- specific codes
   |        |        |
   |        |        +-- RESERVED for possible CRC flag DO NOT USE
   |        |
   |        +-- category:000 0000 0000 0001  AG geometry        TCODE_AG       
   |                     000 0000 0000 0010  Rhino object       TCODE_RHINO_OBJECT
   |                     000 0000 0000 0100                      
   |                     000 0000 0000 1000                       
   |                     000 0000 0001 0000  Geometry           TCODE_GEOMETRY
   |                     000 0000 0010 0000
   |                     000 0000 0100 0000  Display Attributes TCODE_DISPLAY
   |                     000 0000 1000 0000  Rendering          TCODE_RENDER     
   |                     000 0001 0000 0000                         
   |                     000 0010 0000 0000  Interface          TCODE_INTERFACE 
   |                     000 0100 0000 0000
   |                     000 1000 0000 0000  Tolerances         TCODE_TOLERANCE
   |                     001 0000 0000 0000  Tables             TCODE_TABLE    
   |                     010 0000 0000 0000  Table References   TCODE_TABLEREF
   |                     100 0000 0000 0000  User information   TCODE_USER
   | 
   +-- format: 0 - data size in header  - data block follows    TCODE_SHORT
               1 - data in header - no data block follows

*/


#define TCODE_AG                  0x00010000
#define TCODE_RHINO_OBJECT        0x00020000
#define TCODE_GEOMETRY            0x00100000
#define TCODE_ANNOTATION          0x00200000  /* added 5/28/99 */
#define TCODE_DISPLAY             0x00400000
#define TCODE_RENDER              0x00800000
#define TCODE_INTERFACE           0x02000000
#define TCODE_TOLERANCE           0x08000000
#define TCODE_TABLE               0x10000000
#define TCODE_TABLEREF            0x20000000
#define TCODE_USER                0x40000000
#define TCODE_SHORT               0x80000000

#define TCODE_RHINO_OBJECT_DATA   (TCODE_RHINO_OBJECT | 0x0000FFFE)
#define TCODE_RHINO_OBJECT_END    (TCODE_RHINO_OBJECT | 0x0000FFFF)

#define TCODE_RHINO_OBJECT_NURBS_CURVE   (TCODE_RHINO_OBJECT | 0x00000008)
#define TCODE_RHINO_OBJECT_NURBS_SURFACE (TCODE_RHINO_OBJECT | 0x00000009)
#define TCODE_RHINO_OBJECT_BREP          (TCODE_RHINO_OBJECT | 0x0000000B)

/* Rhino file IO toolkit allows users to write NURBS curves and surfaces
// using TCODE_RHINO_OBJECT chunks.  In the .3dm file these
// TCODE_RHINO_OBJECT chunks are always contain a TCODE_RHINO_OBJECT_DATA
// chunk that defines the object, and end with a TCODE_RHINO_OBJECT_END
// chunk.
//
// TCODE_RHINO_OBJECT | 0x0000****  ( low 2 bytes specify type of object )
// length of Rhino object chunk
//
//   TCODE_RHINO_OBJECT_DATA
//   length of object data
//   ... basic definition of object (coordinates, knots, etc.)

//   optional attribute information like layer index, color, etc.
//   ...
//
//   TCODE_RHINO_OBJECT_END
//   0
//
// For example, a NURBS curve is stored as:
// The sub-chunks that are after the TCODE_RHINO_OBJECT_DATA chunk
// and before the TCODE_RHINO_OBJECT_END chunk may appear in any order.
//
//  32 bit unsigned int   TCODE_RHINO_OBJECT_NURBS_CURVE
//  32 bit unsigned int   length of chunk

//  32 bit unsigned int     TCODE_RHINO_OBJECT_DATA
//  32 bit unsigned int     length of chunk
//                            ... NURBS curve data (knots, CVs, etc.)
//     
//  ( optional index of layer in layer table )            
//  32 bit unsigned int     TCODE_LAYERREF
//  32 bit unsigned int     index of layer in layer table
//                          
//  ( optional text lable )            
//  32 bit unsigned int     TCODE_NAME
//  32 bit unsigned int     length of chunk
//  32 bit unsigned int       length of ASCII text string
//                            ... ASCII text string
//                          
//  ( optional display color (wire frame) )
//  32 bit unsigned int     TCODE_RGBDISPLAY
//  32 bit unsigned int     Microsoft COLORREF
//                          
//  ( optional )
//  32 bit unsigned int     TCODE_COMPRESSED_MESH_GEOMETRY
//  32 bit unsigned int     length of chunk
//                            ... faceted mesh used for shading
//  ( optional )
//  32 bit unsigned int     TCODE_XDATA
//  32 bit unsigned int     length of chunk
//                          ... "X" data
//                          
//  ... the next items really should be a single material chunk
//
//  ( optional "full render" color )
//  32 bit unsigned int     TCODE_RGB
//  32 bit unsigned int     Microsoft COLORREF
//                          
//  ( optional "full render" transparency )
//  32 bit unsigned int     TCODE_TRANSPARENCY
//  32 bit unsigned int     transparency
//                          
//  ( optional "full render" texture map )
//  32 bit unsigned int     TCODE_TEXTUREMAP
//  32 bit unsigned int     length of chunk
//                          ... texture map data
//                          
//  ( optional "full render" bump map )
//  32 bit unsigned int     TCODE_BUMPMAP
//  32 bit unsigned int     length of chunk
//                          ... bump map data
//  ... other optional chunks with attribute information
//
//  32 bit unsigned int     TCODE_RHINO_OBJECT_END
//  32 bit unsigned int     0
*/

#define TCODE_AG_ASSEMBLY         (TCODE_AG | 0x00000001)
#define TCODE_AG_PART             (TCODE_AG | 0x00000002)
#define TCODE_AG_SHELL            (TCODE_AG | 0x00000003)
#define TCODE_AG_FACE             (TCODE_AG | 0x00000004)
#define TCODE_AG_BOUNDARY         (TCODE_AG | 0x00000005)
#define TCODE_AG_TEDGE            (TCODE_AG | 0x00000006)
#define TCODE_AG_SURFACE          (TCODE_AG | 0x00000007)
#define TCODE_AG_CURVE            (TCODE_AG | 0x00000008)
#define TCODE_AG_SPLINE           (TCODE_AG | 0x00000009)
#define TCODE_AG_POINT            (TCODE_AG | 0x0000000A)

#define TCODE_STUFF               0x00000100

#define TCODE_AG_ASSEMBLYSTUFF    (TCODE_AG | TCODE_STUFF | TCODE_AG_ASSEMBLY)
#define TCODE_AG_PARTSTUFF        (TCODE_AG | TCODE_STUFF | TCODE_AG_PART)
#define TCODE_AG_SHELLSTUFF       (TCODE_AG | TCODE_STUFF | TCODE_AG_SHELL)
#define TCODE_AG_FACESTUFF        (TCODE_AG | TCODE_STUFF | TCODE_AG_FACE)
#define TCODE_AG_BOUNDARYSTUFF    (TCODE_AG | TCODE_STUFF | TCODE_AG_BOUNDARY)
#define TCODE_AG_TEDGESTUFF       (TCODE_AG | TCODE_STUFF | TCODE_AG_TEDGE)
#define TCODE_AG_SURFACESTUFF     (TCODE_AG | TCODE_STUFF | TCODE_AG_SURFACE)
#define TCODE_AG_CURVESTUFF       (TCODE_AG | TCODE_STUFF | TCODE_AG_CURVE)
#define TCODE_AG_SPLINESTUFF      (TCODE_AG | TCODE_STUFF | TCODE_AG_SPLINE)
#define TCODE_AG_POINTSTUFF       (TCODE_AG | TCODE_STUFF | TCODE_AG_POINT)

#define TCODE_RH_POINT            (TCODE_GEOMETRY | 0x00000001)

#define TCODE_RH_SPOTLIGHT        (TCODE_RENDER   | 0x00000001)

#define TCODE_OLD_RH_TRIMESH          (TCODE_GEOMETRY | 0x00000011)
#define TCODE_OLD_MESH_VERTEX_NORMALS (TCODE_GEOMETRY | 0x00000012)
#define TCODE_OLD_MESH_UV             (TCODE_GEOMETRY | 0x00000013)
#define TCODE_OLD_FULLMESH        (TCODE_GEOMETRY | 0x00000014)


#define TCODE_MESH_OBJECT         (TCODE_GEOMETRY | 0x00000015)
#define TCODE_COMPRESSED_MESH_GEOMETRY   (TCODE_GEOMETRY | 0x00000017)
#define TCODE_ANALYSIS_MESH       (TCODE_GEOMETRY | 0x00000018)

#define TCODE_ANNOTATION_SETTINGS (TCODE_ANNOTATION | 0x00000001)

#define TCODE_TEXT_BLOCK          (TCODE_ANNOTATION | 0x00000004)
#define TCODE_ANNOTATION_LEADER   (TCODE_ANNOTATION | 0x00000005)
#define TCODE_LINEAR_DIMENSION    (TCODE_ANNOTATION | 0x00000006)
#define TCODE_ANGULAR_DIMENSION   (TCODE_ANNOTATION | 0x00000007)
#define TCODE_RADIAL_DIMENSION    (TCODE_ANNOTATION | 0x00000008)


#define TCODE_NAME                (TCODE_INTERFACE | 0x00000001)
#define TCODE_VIEW                (TCODE_INTERFACE | 0x00000002)
#define TCODE_CPLANE              (TCODE_INTERFACE | 0x00000003)

#define TCODE_NAMED_CPLANE        (TCODE_INTERFACE | 0x00000004)
#define TCODE_NAMED_VIEW          (TCODE_INTERFACE | 0x00000005)
#define TCODE_VIEWPORT            (TCODE_INTERFACE | 0x00000006)

#define TCODE_SHOWGRID            (TCODE_SHORT | TCODE_INTERFACE | 0x00000007)
#define TCODE_SHOWGRIDAXES        (TCODE_SHORT | TCODE_INTERFACE | 0x00000008)
#define TCODE_SHOWWORLDAXES       (TCODE_SHORT | TCODE_INTERFACE | 0x00000009)

#define TCODE_VIEWPORT_POSITION   (TCODE_INTERFACE | 0x0000000A)
#define TCODE_VIEWPORT_TRACEINFO  (TCODE_INTERFACE | 0x0000000B)
#define TCODE_SNAPSIZE            (TCODE_INTERFACE | 0x0000000C)
#define TCODE_NEAR_CLIP_PLANE     (TCODE_INTERFACE | 0x0000000D)
#define TCODE_HIDE_TRACE          (TCODE_INTERFACE | 0x0000000E)

#define TCODE_NOTES               (TCODE_INTERFACE | 0x0000000F)
#define TCODE_UNIT_AND_TOLERANCES (TCODE_INTERFACE | 0x00000010)

#define TCODE_MAXIMIZED_VIEWPORT  (TCODE_SHORT | TCODE_INTERFACE | 0x00000011)
#define TCODE_VIEWPORT_WALLPAPER  (TCODE_INTERFACE | 0x00000012)


#define TCODE_SUMMARY             (TCODE_INTERFACE | 0x00000013)
#define TCODE_BITMAPPREVIEW       (TCODE_INTERFACE | 0x00000014)
#define TCODE_VIEWPORT_DISPLAY_MODE  (TCODE_SHORT | TCODE_INTERFACE | 0x00000015)


#define TCODE_LAYERTABLE          (TCODE_SHORT   | TCODE_TABLE    | 0x00000001)
#define TCODE_LAYERREF            (TCODE_SHORT   | TCODE_TABLEREF | 0x00000001)

#define TCODE_XDATA               (TCODE_USER | 0x00000001)


/* object material settings */
#define TCODE_RGB                 (TCODE_SHORT   | TCODE_DISPLAY | 0x00000001)
#define TCODE_TEXTUREMAP          (TCODE_DISPLAY | 0x00000002)
#define TCODE_BUMPMAP             (TCODE_DISPLAY | 0x00000003)
#define TCODE_TRANSPARENCY        (TCODE_SHORT   | TCODE_DISPLAY | 0x00000004)
#define TCODE_DISP_AM_RESOLUTION  (TCODE_SHORT   | TCODE_DISPLAY | 0x00000005)
#define TCODE_RGBDISPLAY          (TCODE_SHORT   | TCODE_DISPLAY | 0x00000006)  /* will be used for color by object */
#define TCODE_RENDER_MATERIAL_ID  (TCODE_DISPLAY | 0x00000007)                  /* id for render material */

#define TCODE_LAYER               (TCODE_DISPLAY | 0x00000010)

/* obsolete layer typecodes from earlier betas - not used anymore */
#define TCODE_LAYER_OBSELETE_1    (TCODE_SHORT   | TCODE_DISPLAY | 0x00000013)
#define TCODE_LAYER_OBSELETE_2    (TCODE_SHORT   | TCODE_DISPLAY | 0x00000014)
#define TCODE_LAYER_OBSELETE_3    (TCODE_SHORT   | TCODE_DISPLAY | 0x00000015)

/* these were only ever used by AccuModel and never by Rhino */
#define TCODE_LAYERON             (TCODE_SHORT   | TCODE_DISPLAY | 0x00000016)
#define TCODE_LAYERTHAWED         (TCODE_SHORT   | TCODE_DISPLAY | 0x00000017)
#define TCODE_LAYERLOCKED         (TCODE_SHORT   | TCODE_DISPLAY | 0x00000018)


#define TCODE_LAYERVISIBLE        (TCODE_SHORT   | TCODE_DISPLAY | 0x00000012)
#define TCODE_LAYERPICKABLE       (TCODE_SHORT   | TCODE_DISPLAY | 0x00000030)
#define TCODE_LAYERSNAPABLE       (TCODE_SHORT   | TCODE_DISPLAY | 0x00000031)
#define TCODE_LAYERRENDERABLE     (TCODE_SHORT   | TCODE_DISPLAY | 0x00000032)


/* use LAYERSTATE ( 0 = LAYER_ON, 1 = LAYER_OFF, 2 = LAYER_LOCKED ) instead of above individual toggles */
#define TCODE_LAYERSTATE          (TCODE_SHORT   | TCODE_DISPLAY | 0x00000033)

#define TCODE_RENDERMESHPARAMS    (TCODE_DISPLAY | 0x00000020) /* block of parameters for render meshes */



#define TCODE_DISP_CPLINES        (TCODE_SHORT   | TCODE_DISPLAY | 0x00000022)
#define TCODE_DISP_MAXLENGTH      (TCODE_DISPLAY | 0x00000023)

#define TCODE_CURRENTLAYER        (TCODE_SHORT   | TCODE_DISPLAY | 0x00000025 )




#define TCODE_LAYERNAME           (TCODE_DISPLAY | 0x00000011)

#define TCODE_TOL_AGFIT           (TCODE_TOLERANCE | 0x00000001)
#define TCODE_TOL_AGANGLE         (TCODE_TOLERANCE | 0x00000002)


#define TCODE_ENDOFTABLE          0xFFFFFFFF


#define TCODE_COMMENTBLOCK        0x00000001



#if defined (cplusplus) || defined(_cplusplus) || defined(__cplusplus)
extern "C" {
#endif

/* Function declatations */

#if defined (cplusplus) || defined(_cplusplus) || defined(__cplusplus)
}
#endif

#endif
