/* $Header: /src2/opennurbs/opennurbs_3dm_attributes.h 16    1/03/01 4:59p Dalelear $ */
/* $NoKeywords: $ */
/*
//
// Copyright (c) 1993-2001 Robert McNeel & Associates. All rights reserved.
// Rhinoceros is a registered trademark of Robert McNeel & Assoicates.
//
// THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
// ALL IMPLIED WARRANTIES OF FITNESS FOR ANY PARTICULAR PURPOSE AND OF
// MERCHANTABILITY ARE HEREBY DISCLAIMED.
//				
// For complete openNURBS copyright information see <http://www.opennurbs.org>.
//
////////////////////////////////////////////////////////////////
*/

////////////////////////////////////////////////////////////////
//
//   defines ON_3dmObjectAttributes
//
////////////////////////////////////////////////////////////////

#if !defined(OPENNURBS_3DM_ATTRIBUTES_INC_)
#define OPENNURBS_3DM_ATTRIBUTES_INC_

class ON_CLASS ON_3dmObjectAttributes
{
  // attributes of geometry and dimension table objects
public:
  ON_3dmObjectAttributes();
  ~ON_3dmObjectAttributes();
  ON_3dmObjectAttributes(const ON_3dmObjectAttributes&);
  ON_3dmObjectAttributes& operator=(const ON_3dmObjectAttributes&);

  void Default();

  BOOL Write(ON_BinaryArchive&) const;
  BOOL Read(ON_BinaryArchive&);

  void Dump( ON_TextLog& ) const;

  // Interface ////////////////////////////////////////////////////////

  // object mode  = normal/locked/hidden
  ON::object_mode Mode() const;
  void SetMode( ON::object_mode );

  // color source = from object/from layer/from material
  ON::object_color_source ColorSource() const;
  void SetColorSource( ON::object_color_source );

  // material source = from object/from layer
  ON::object_material_source MaterialSource() const;
  void SetMaterialSource( ON::object_material_source );

  // display mode
  ON::display_mode DisplayMode() const;
  void SetDisplayMode( ON::display_mode  );

  // line style source = from object/from layer/from material
  ON::object_line_style_source LineStyleSource() const;
  void SetLineStyleSource( ON::object_line_style_source );

  ON_UUID       m_uuid;
  ON_wString    m_name;
  ON_wString    m_url;            // e.g., if part came from a library this URL will check for updates
  int           m_layer_index;    // 
  int           m_material_index; // object render material index if ON::material_from_object == MaterialSource()
                                  // A material index of -1 indicates no material has been assigned
                                  // and the default material created by the default ON_Material constructor
                                  // should be used.
  ON_Color      m_color;          // object color if ON::color_from_object == ColorSource()
  ON_LineStyle  m_line_style;     // line style if ON::line_style_from_object == LineStyleSource()
  int           m_wire_density;   // surface wireframe density
                                  // 0    = boundary + "knot" wires 
                                  // 1    = boundary + "knot" wires + 1 interior wire if no interior "knots"
                                  // N>=2 = boundry  + "knot" wires + (N-1) interior wires

  // group interface

  //////////
  // returns number of groups object belongs to
  int GroupCount() const;

  //////////
  // returns an int array of GroupCount() zero based group indices
  // returns NULL if GroupCount() is zero.
  const int* GroupList() const;

  //////////
  // returns GroupCount() and puts list of zero based group indices int array
  int GetGroupList(ON_SimpleArray<int>&) const;

  //////////
  // returns the index of the last group in the group list
  // or -1 if the object is not in any groups
  int TopGroup() const;

  //////////
  // returns TRUE if object is in group with specified index
  BOOL IsInGroup(
    int // zero based group index
    ) const;

  //////////
  // returns TRUE if object is in any of the groups in the list
  BOOL IsInGroups(
    int,       // group_list_count
    const int* // group_list[] array
    ) const;

  //////////
  // returns TRUE if object is in any of the groups in the list
  BOOL IsInGroups(
    const ON_SimpleArray<int>& // group_list[] array
    ) const;

  //////////
  // Adds object to group with specified index by appending index to
  // group list (If object is already in group, nothing is changed.)
  void AddToGroup(
    int // zero based group index
    );

  //////////
  // Removes object from group with specified index.  If object is not
  // in the group, nothing is changed.
  void RemoveFromGroup(
    int // zero based group index
    );

  //////////
  // removes the object from the last group in the group list
  void RemoveFromTopGroup();

  //////////
  // Removes object from all groups.
  void RemoveFromAllGroups();

private:
  unsigned char m_mode;               // (m_mode % 16) = ON::object_mode values
                                      // (m_mode / 16) = ON::display_mode values
  unsigned char m_color_source;       // ON::object_color_source values
  unsigned char m_line_style_source;  // ON::object_line_style_source values
  unsigned char m_material_source;    // ON::object_material_source values

  ON_SimpleArray<int> m_group; // array of zero based group indices
};

#endif
