/* $Header: /src2/opennurbs/opennurbs_3dm_settings.cpp 35    7/26/01 11:06a Dalelear $ */
/* $NoKeywords: $ */
/*
//
// Copyright (c) 1993-2001 Robert McNeel & Associates. All rights reserved.
// Rhinoceros is a registered trademark of Robert McNeel & Assoicates.
//
// THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
// ALL IMPLIED WARRANTIES OF FITNESS FOR ANY PARTICULAR PURPOSE AND OF
// MERCHANTABILITY ARE HEREBY DISCLAIMED.
//				
// For complete openNURBS copyright information see <http://www.opennurbs.org>.
//
////////////////////////////////////////////////////////////////
*/

#include "opennurbs.h"

//////////////////////////////////////////////////////////////////////////////////////////
//
// ON_3dmUnitsAndTolerances
//

void ON_3dmUnitsAndTolerances::Default()
{
  m_unit_system = ON::millimeters;
  m_absolute_tolerance = 0.01;
  m_angle_tolerance = ON_PI/60.0;
  m_relative_tolerance = 0.01;
  m_distance_display_mode = ON::decimal;
  m_distance_display_precision = 3;
  m_custom_unit_scale = 1.0;
  m_custom_unit_name = "Units";
}

ON_3dmUnitsAndTolerances::ON_3dmUnitsAndTolerances()
                        : m_unit_system(ON::no_unit_system),
                          m_absolute_tolerance(0.0),
                          m_angle_tolerance(0.0),
                          m_relative_tolerance(0.0),
                          m_distance_display_mode(ON::decimal),
                          m_distance_display_precision(3),
                          m_custom_unit_scale(1.0)
{
  Default();
}

ON_3dmUnitsAndTolerances::~ON_3dmUnitsAndTolerances()
{}

ON_3dmUnitsAndTolerances::ON_3dmUnitsAndTolerances(const ON_3dmUnitsAndTolerances& src )
                        : m_unit_system(ON::no_unit_system),
                          m_absolute_tolerance(0.0),
                          m_angle_tolerance(0.0),
                          m_relative_tolerance(0.0),
                          m_distance_display_mode(ON::decimal),
                          m_distance_display_precision(3),
                          m_custom_unit_scale(1.0)
{
  Default();
  *this = src;
}

ON_3dmUnitsAndTolerances& ON_3dmUnitsAndTolerances::operator=(const ON_3dmUnitsAndTolerances& src )
{
  if ( this != &src ) 
  {
    m_unit_system = src.m_unit_system;
    m_absolute_tolerance = src.m_absolute_tolerance;
    m_angle_tolerance = src.m_angle_tolerance;
    m_relative_tolerance = src.m_relative_tolerance;
    m_distance_display_mode = src.m_distance_display_mode;
    m_distance_display_precision = src.m_distance_display_precision;
    m_custom_unit_scale = src.m_custom_unit_scale;
    // copy string this way to force separate memory pools
    const wchar_t* s = src.m_custom_unit_name;
    m_custom_unit_name = s;
  }
  return *this;
}

BOOL ON_3dmUnitsAndTolerances::Write( ON_BinaryArchive& file ) const
{
  const int version = 102;
  int i;

  // version 100 ON_3dmUnitsAndTolerances settings
  BOOL rc = file.WriteInt( version );
  i = m_unit_system;
  if ( rc ) rc = file.WriteInt( i );
  if ( rc ) rc = file.WriteDouble( m_absolute_tolerance );
  if ( rc ) rc = file.WriteDouble( m_angle_tolerance );
  if ( rc ) rc = file.WriteDouble( m_relative_tolerance );

  // added in version 101
  i = m_distance_display_mode;
  if ( rc ) rc = file.WriteInt( i );
  i = m_distance_display_precision;
  if ( i < 0 || i > 20 ) {
    ON_ERROR("ON_3dmUnitsAndTolerances::Write() - m_distance_display_precision out of range.");
    i = 3;
  }
  if ( rc ) rc = file.WriteInt( i );

  // added in version 102
  if ( rc ) rc = file.WriteDouble( m_custom_unit_scale );
  if ( rc ) rc = file.WriteString( m_custom_unit_name );
  return rc;
}

BOOL ON_3dmUnitsAndTolerances::Read( ON_BinaryArchive& file )
{
  Default();
  int version = 0;
  BOOL rc = file.ReadInt( &version );
  if ( rc && version >= 100 && version < 200 ) {
    int us = ON::no_unit_system;
    rc = file.ReadInt( &us );
    if ( rc )
      m_unit_system = ON::UnitSystem(us);
    if ( rc ) rc = file.ReadDouble( &m_absolute_tolerance );
    if ( rc ) rc = file.ReadDouble( &m_angle_tolerance );
    if ( rc ) rc = file.ReadDouble( &m_relative_tolerance );
    if ( version >= 101 ) {
      int dm = ON::decimal;
      if ( rc ) rc = file.ReadInt( &dm );
      if ( rc ) m_distance_display_mode = ON::DistanceDisplayMode(dm);
      if ( rc ) rc = file.ReadInt( &m_distance_display_precision );
      if ( m_distance_display_precision < 0 || m_distance_display_precision > 20 )
        m_distance_display_precision = 3; // some beta files had bogus values stored in file
      if ( version >= 102 ) {
        if ( rc ) rc = file.ReadDouble( &m_custom_unit_scale );
        if ( rc ) rc = file.ReadString( m_custom_unit_name );
      }
    }
  }
  return rc;
}

void ON_3dmUnitsAndTolerances::Dump( ON_TextLog& dump) const
{
  ON_wString sUnitSystem;
  switch( m_unit_system )
  {
  case ON::no_unit_system:
    sUnitSystem = "no units";
    break;
  case ON::microns:
    sUnitSystem = "microns";
    break;
  case ON::millimeters:
    sUnitSystem = "millimeters";
    break;
  case ON::centimeters:
    sUnitSystem = "centimeters";
    break;
  case ON::meters:
    sUnitSystem = "meters";
    break;
  case ON::kilometers:
    sUnitSystem = "kilometers";
    break;
  case ON::microinches:
    sUnitSystem = "microinches";
    break;
  case ON::mils:
    sUnitSystem = "mils (= 0.001 inches)";
    break;
  case ON::inches:
    sUnitSystem = "inches";
    break;
  case ON::feet:
    sUnitSystem = "feet";
    break;
  case ON::miles:
    sUnitSystem = "miles";
    break;
  case ON::custom_unit_system:
    if ( m_custom_unit_name.Length() > 0 )
      sUnitSystem.Format("%S (= %g meters)",
                         m_custom_unit_name.Array(),
                         m_custom_unit_scale);
    else
      sUnitSystem.Format("user defined unit (= %g meters)",m_custom_unit_scale);
    break;
  default:
    sUnitSystem = "unknown unit system";
    break;
  }
  dump.Print("Unit system: %S\n",sUnitSystem.Array());
  dump.Print("Absolute tolerance: %g\n",m_absolute_tolerance);
  dump.Print("Angle tolerance: %g\n",m_angle_tolerance);
}

double ON_3dmUnitsAndTolerances::Scale( ON::unit_system us ) const
{
  // Example: If us = meters and m_unit_system = centimeters,
  // then Scale() returns 100.
  return (m_unit_system == ON::custom_unit_system && m_custom_unit_scale > 0.0)
         ? ON::UnitScale(us,ON::meters)*m_custom_unit_scale
         : ON::UnitScale(us,m_unit_system);
}

//////////////////////////////////////////////////////////////////////////////////////////
//
// ON_3dmRenderSettings
//

void ON_3dmRenderSettings::Default()
{
  m_bCustomImageSize = FALSE;
  m_image_width  = 800;
  m_image_height = 600;
  m_image_dpi = 72.0;
  m_image_us = ON::inches;


  m_ambient_light.SetRGB( 0, 0, 0);

  m_background_style = 0;
  m_background_color.SetRGB(160,160,160);
  m_background_bitmap_filename.Empty();

  m_bUseHiddenLights = FALSE;

  m_bDepthCue = FALSE;
  m_bFlatShade = FALSE;

  m_bRenderBackfaces = TRUE;
  m_bRenderPoints = FALSE;
  m_bRenderCurves = FALSE;
  m_bRenderIsoparams = FALSE;
  m_bRenderMeshEdges = FALSE;
  m_bRenderAnnotation = FALSE;

  m_antialias_style = 1;

  m_shadowmap_style = 1;
  m_shadowmap_width = 1000;
  m_shadowmap_height = 1000;
  m_shadowmap_offset = 0.75;
}

ON_3dmRenderSettings::ON_3dmRenderSettings()
{
  Default();
}

ON_3dmRenderSettings::~ON_3dmRenderSettings()
{
  m_background_bitmap_filename.Empty();
}

ON_3dmRenderSettings::ON_3dmRenderSettings(const ON_3dmRenderSettings& src )
{
  Default();
  *this = src;
}

ON_3dmRenderSettings& ON_3dmRenderSettings::operator=(const ON_3dmRenderSettings& src )
{
  if ( this != &src ) {
    m_bCustomImageSize = src.m_bCustomImageSize;
    m_image_width = src.m_image_width;
    m_image_height = src.m_image_height;
    m_image_dpi = src.m_image_dpi;
    m_image_us = src.m_image_us;
    m_ambient_light = src.m_ambient_light;
    m_background_style = src.m_background_style;
    m_background_color = src.m_background_color;
    m_background_bitmap_filename = src.m_background_bitmap_filename;
    m_bUseHiddenLights = src.m_bUseHiddenLights;
    m_bDepthCue = src.m_bDepthCue;
    m_bFlatShade = src.m_bFlatShade;
    m_bRenderBackfaces = src.m_bRenderBackfaces;
    m_bRenderPoints = src.m_bRenderPoints;
    m_bRenderCurves = src.m_bRenderCurves;
    m_bRenderIsoparams = src.m_bRenderIsoparams;
    m_bRenderMeshEdges = src.m_bRenderMeshEdges;
    m_bRenderAnnotation = src.m_bRenderAnnotation;
    m_antialias_style = src.m_antialias_style;
    m_shadowmap_style = src.m_shadowmap_style;
    m_shadowmap_width = src.m_shadowmap_width;
    m_shadowmap_height = src.m_shadowmap_height;
    m_shadowmap_offset = src.m_shadowmap_offset;
  }
  return *this;
}

BOOL ON_3dmRenderSettings::Write( ON_BinaryArchive& file ) const
{
  int i;
  const int version = 101;
  BOOL rc = file.WriteInt( version );
  // version >= 100
  if (rc) rc = file.WriteInt( m_bCustomImageSize );
  if (rc) rc = file.WriteInt( m_image_width );
  if (rc) rc = file.WriteInt( m_image_height );
  if (rc) rc = file.WriteColor( m_ambient_light );
  if (rc) rc = file.WriteInt( m_background_style );
  if (rc) rc = file.WriteColor( m_background_color );
  if (rc) rc = file.WriteString( m_background_bitmap_filename );
  if (rc) rc = file.WriteInt( m_bUseHiddenLights );
  if (rc) rc = file.WriteInt( m_bDepthCue );
  if (rc) rc = file.WriteInt( m_bFlatShade );
  if (rc) rc = file.WriteInt( m_bRenderBackfaces );
  if (rc) rc = file.WriteInt( m_bRenderPoints );
  if (rc) rc = file.WriteInt( m_bRenderCurves );
  if (rc) rc = file.WriteInt( m_bRenderIsoparams );
  if (rc) rc = file.WriteInt( m_bRenderMeshEdges );
  if (rc) rc = file.WriteInt( m_bRenderAnnotation );
  if (rc) rc = file.WriteInt( m_antialias_style );
  if (rc) rc = file.WriteInt( m_shadowmap_style );
  if (rc) rc = file.WriteInt( m_shadowmap_width );
  if (rc) rc = file.WriteInt( m_shadowmap_height );
  if (rc) rc = file.WriteDouble( m_shadowmap_offset );
  // version >= 101 begins here
  if (rc) rc = file.WriteDouble( m_image_dpi );
  i = m_image_us;
  if (rc) rc = file.WriteInt( i );
  return rc;
}

BOOL ON_3dmRenderSettings::Read( ON_BinaryArchive& file )
{
  Default();
  int i;
  int version = 0;
  BOOL rc = file.ReadInt( &version );
  if ( rc && version >= 100 && version < 200 ) {
    if (rc) rc = file.ReadInt( &m_bCustomImageSize );
    if (rc) rc = file.ReadInt( &m_image_width );
    if (rc) rc = file.ReadInt( &m_image_height );
    if (rc) rc = file.ReadColor( m_ambient_light );
    if (rc) rc = file.ReadInt( &m_background_style );
    if (rc) rc = file.ReadColor( m_background_color );
    if (rc) rc = file.ReadString( m_background_bitmap_filename );
    if (rc) rc = file.ReadInt( &m_bUseHiddenLights );
    if (rc) rc = file.ReadInt( &m_bDepthCue );
    if (rc) rc = file.ReadInt( &m_bFlatShade );
    if (rc) rc = file.ReadInt( &m_bRenderBackfaces );
    if (rc) rc = file.ReadInt( &m_bRenderPoints );
    if (rc) rc = file.ReadInt( &m_bRenderCurves );
    if (rc) rc = file.ReadInt( &m_bRenderIsoparams );
    if (rc) rc = file.ReadInt( &m_bRenderMeshEdges );
    if (rc) rc = file.ReadInt( &m_bRenderAnnotation );
    if (rc) rc = file.ReadInt( &m_antialias_style );
    if (rc) rc = file.ReadInt( &m_shadowmap_style );
    if (rc) rc = file.ReadInt( &m_shadowmap_width );
    if (rc) rc = file.ReadInt( &m_shadowmap_height );
    if (rc) rc = file.ReadDouble( &m_shadowmap_offset );
    if (version >= 101) {
      if (rc) rc = file.ReadDouble( &m_image_dpi );
      if (rc) rc = file.ReadInt(&i);
      if (rc)
        m_image_us = ON::UnitSystem(i);
    }
  }
  return rc;
}

//////////////////////////////////////////////////////////////////////////////////////////
//
// ON_3dmAnnotationSettings
//

ON_3dmAnnotationSettings::ON_3dmAnnotationSettings()
{
  Default();
}

ON_3dmAnnotationSettings::~ON_3dmAnnotationSettings()
{
}

ON_3dmAnnotationSettings::ON_3dmAnnotationSettings(const ON_3dmAnnotationSettings& src)
{
  Default();
  *this = src;

}

ON_3dmAnnotationSettings& ON_3dmAnnotationSettings::operator=(const ON_3dmAnnotationSettings& src)
{
  if ( this != &src ) {
    m_dimscale = src.m_dimscale;
    m_textheight = src.m_textheight;
    m_dimexe = src.m_dimexe;
    m_dimexo = src.m_dimexo;
    m_arrowlength = src.m_arrowlength;
    m_arrowwidth = src.m_arrowwidth;
    m_centermark = src.m_centermark;
    m_dimunits = src.m_dimunits;;
    m_arrowtype = src.m_arrowtype;
    m_angularunits = src.m_angularunits;
    m_lengthformat = src.m_lengthformat;
    m_angleformat = src.m_angleformat;
    m_textalign = src.m_textalign;
    m_resolution = src.m_resolution;
    m_facename = src.m_facename;
  }
  return *this;
}

void ON_3dmAnnotationSettings::Default()
{
  m_dimscale = 1.0;       // model size / plotted size
  m_textheight = 1.0;
  m_dimexe = 1.0;
  m_dimexo = 1.0;
  m_arrowlength = 1.0;
  m_arrowwidth = 1.0;
  m_centermark = 1.0;

  m_dimunits = ON::no_unit_system;  // units used to measure the dimension
  m_arrowtype = 0;     // 0: filled narrow triangular arrow
  m_angularunits = 0;  // 0: degrees, 1: radians
  m_lengthformat = 0;  // 0: decimal, ...
  m_angleformat = 0;   // 0: decimal degrees, ...
  m_textalign = 0;     // 0: above line, 1: in line, 2: horizontal
  m_resolution = 0;    // depends on m_lengthformat
                       // for decimal, digits past the decimal point

  m_facename.Destroy(); // [LF_FACESIZE] // windows font name
}

BOOL ON_3dmAnnotationSettings::Read( ON_BinaryArchive& file )
{
  int major_version = 0;
  int minor_version = 0;
  BOOL rc = file.Read3dmChunkVersion(&major_version,&minor_version);
  if ( major_version == 1 ) {
    if ( minor_version >= 0 ) {
      if (rc) rc = file.ReadDouble(&m_dimscale);
      if (rc) rc = file.ReadDouble(&m_textheight);
      if (rc) rc = file.ReadDouble(&m_dimexe);
      if (rc) rc = file.ReadDouble(&m_dimexo);
      if (rc) rc = file.ReadDouble(&m_arrowlength);
      if (rc) rc = file.ReadDouble(&m_arrowwidth);
      if (rc) rc = file.ReadDouble(&m_centermark);

      int i;
      if (rc) rc = file.ReadInt( &i );
      m_dimunits = ON::UnitSystem(i);

      if (rc) rc = file.ReadInt( &m_arrowtype );
      if (rc) rc = file.ReadInt( &m_angularunits );
      if (rc) rc = file.ReadInt( &m_lengthformat );
      if (rc) rc = file.ReadInt( &m_angleformat );
      if (rc) rc = file.ReadInt( &m_textalign );
      if (rc) rc = file.ReadInt( &m_resolution );

      if (rc) rc = file.ReadString( m_facename );
    }
  }
  else {
    rc = FALSE;
  }
  return rc;
}

BOOL ON_3dmAnnotationSettings::Write( ON_BinaryArchive& file ) const
{
  int i;
  BOOL rc = file.Write3dmChunkVersion(1,0);
  if (rc) rc = file.WriteDouble(m_dimscale);
  if (rc) rc = file.WriteDouble(m_textheight);
  if (rc) rc = file.WriteDouble(m_dimexe);
  if (rc) rc = file.WriteDouble(m_dimexo);
  if (rc) rc = file.WriteDouble(m_arrowlength);
  if (rc) rc = file.WriteDouble(m_arrowwidth);
  if (rc) rc = file.WriteDouble(m_centermark);

  i = m_dimunits;
  if (rc) rc = file.WriteInt( i );
  if (rc) rc = file.WriteInt( m_arrowtype );
  if (rc) rc = file.WriteInt( m_angularunits );
  if (rc) rc = file.WriteInt( m_lengthformat );
  if (rc) rc = file.WriteInt( m_angleformat );
  if (rc) rc = file.WriteInt( m_textalign );
  if (rc) rc = file.WriteInt( m_resolution );

  if (rc) rc = file.WriteString( m_facename );

  return rc;
}

//////////////////////////////////////////////////////////////////////////////////////////
//
// ON_3dmConstructionPlane
//
ON_3dmConstructionPlane::ON_3dmConstructionPlane()
{
  Default();
}

ON_3dmConstructionPlane::~ON_3dmConstructionPlane()
{
}

ON_3dmConstructionPlane::ON_3dmConstructionPlane(const ON_3dmConstructionPlane& src)
{
  Default();
  *this = src;
}
ON_3dmConstructionPlane& ON_3dmConstructionPlane::operator=(const ON_3dmConstructionPlane& src)
{
  if ( this != &src ) {
    m_plane = src.m_plane;
    m_grid_spacing = src.m_grid_spacing;
    m_snap_spacing = src.m_snap_spacing;
    m_grid_line_count = src.m_grid_line_count;
    m_grid_thick_frequency = src.m_grid_thick_frequency;
    m_name = src.m_name;
  }
  return *this;
}

void ON_3dmConstructionPlane::Default()
{
  m_name.Destroy();
  m_plane = ON_xy_plane;

  // construction grid appearance
	m_grid_spacing = 1.0;   // distance between grid lines
	m_snap_spacing = 1.0;   // distance between grid snap points
	m_grid_line_count = 70;     // number of grid lines in each direction
  m_grid_thick_frequency = 5; // thick line frequency
}

BOOL ON_3dmConstructionPlane::Write( ON_BinaryArchive& file ) const
{
  BOOL rc = file.Write3dmChunkVersion(1,0);
  if (rc) rc = file.WritePlane(m_plane);
  if (rc) rc = file.WriteDouble(m_grid_spacing);
  if (rc) rc = file.WriteDouble(m_snap_spacing);
  if (rc) rc = file.WriteInt(m_grid_line_count);
  if (rc) rc = file.WriteInt(m_grid_thick_frequency);
  if (rc) rc = file.WriteString(m_name);
  return rc;
}

BOOL ON_3dmConstructionPlane::Read( ON_BinaryArchive& file )
{
  int major_version = 0;
  int minor_version = 0;
  BOOL rc = file.Read3dmChunkVersion(&major_version,&minor_version);
  if (rc && major_version==1) {
    if (rc) rc = file.ReadPlane(m_plane);
    if (rc) rc = file.ReadDouble(&m_grid_spacing);
    if (rc) rc = file.ReadDouble(&m_snap_spacing);
    if (rc) rc = file.ReadInt(&m_grid_line_count);
    if (rc) rc = file.ReadInt(&m_grid_thick_frequency);
    if (rc) rc = file.ReadString(m_name);
  }
  return rc;
}

//////////////////////////////////////////////////////////////////////////////////////////
//
// ON_3dmConstructionPlaneGridDefaults
//
ON_3dmConstructionPlaneGridDefaults::ON_3dmConstructionPlaneGridDefaults()
{
  Default();
}

ON_3dmConstructionPlaneGridDefaults::~ON_3dmConstructionPlaneGridDefaults()
{
}

ON_3dmConstructionPlaneGridDefaults::ON_3dmConstructionPlaneGridDefaults(const ON_3dmConstructionPlaneGridDefaults& src)
{
  Default();
  *this = src;
}
ON_3dmConstructionPlaneGridDefaults& ON_3dmConstructionPlaneGridDefaults::operator=(const ON_3dmConstructionPlaneGridDefaults& src)
{
  if ( this != &src ) {
    m_grid_spacing = src.m_grid_spacing;
    m_snap_spacing = src.m_snap_spacing;
    m_grid_line_count = src.m_grid_line_count;
    m_grid_thick_frequency = src.m_grid_thick_frequency;
    m_bShowGrid = src.m_bShowGrid;
    m_bShowGridAxes = src.m_bShowGridAxes;
    m_bShowWorldAxes = src.m_bShowWorldAxes;
  }
  return *this;
}

void ON_3dmConstructionPlaneGridDefaults::Default()
{
  // construction grid appearance
	m_grid_spacing = 1.0;   // distance between grid lines
	m_snap_spacing = 1.0;   // distance between grid snap points
	m_grid_line_count = 70;     // number of grid lines in each direction
  m_grid_thick_frequency = 5; // thick line frequency
  m_bShowGrid = TRUE;
  m_bShowGridAxes = TRUE;
  m_bShowWorldAxes = TRUE;
}

BOOL ON_3dmConstructionPlaneGridDefaults::Write( ON_BinaryArchive& file ) const
{
  BOOL rc = file.Write3dmChunkVersion(1,0);
  if (rc) rc = file.WriteDouble(m_grid_spacing);
  if (rc) rc = file.WriteDouble(m_snap_spacing);
  if (rc) rc = file.WriteInt(m_grid_line_count);
  if (rc) rc = file.WriteInt(m_grid_thick_frequency);
  if (rc) rc = file.WriteInt(m_bShowGrid);
  if (rc) rc = file.WriteInt(m_bShowGridAxes);
  if (rc) rc = file.WriteInt(m_bShowWorldAxes);
  return rc;
}

BOOL ON_3dmConstructionPlaneGridDefaults::Read( ON_BinaryArchive& file )
{
  int major_version = 0;
  int minor_version = 0;
  BOOL rc = file.Read3dmChunkVersion(&major_version,&minor_version);
  if (rc && major_version==1) {
    if (rc) rc = file.ReadDouble(&m_grid_spacing);
    if (rc) rc = file.ReadDouble(&m_snap_spacing);
    if (rc) rc = file.ReadInt(&m_grid_line_count);
    if (rc) rc = file.ReadInt(&m_grid_thick_frequency);
    if (rc) rc = file.ReadInt(&m_bShowGrid);
    if (rc) rc = file.ReadInt(&m_bShowGridAxes);
    if (rc) rc = file.ReadInt(&m_bShowWorldAxes);
  }
  return rc;
}



//////////////////////////////////////////////////////////////////////////////////////////
//
// ON_3dmViewPosition
//
ON_3dmViewPosition::ON_3dmViewPosition()
{
  Default();
}

ON_3dmViewPosition::~ON_3dmViewPosition()
{
}

ON_3dmViewPosition::ON_3dmViewPosition(const ON_3dmViewPosition& src)
{
  Default();
  *this = src;
}

ON_3dmViewPosition& ON_3dmViewPosition::operator=(const ON_3dmViewPosition& src)
{
  if ( this != &src ) {
    m_wnd_left   = src.m_wnd_left;
    m_wnd_right  = src.m_wnd_right;
    m_wnd_top    = src.m_wnd_top;
    m_wnd_bottom = src.m_wnd_bottom;
    m_bMaximized = src.m_bMaximized;
  }
  return *this;
}

void ON_3dmViewPosition::Default()
{
  m_wnd_left   = 0.0;
  m_wnd_right  = 1.0;
  m_wnd_top    = 0.0;
  m_wnd_bottom = 1.0;
  m_bMaximized = FALSE;
}

BOOL ON_3dmViewPosition::Write( ON_BinaryArchive& file ) const
{
  BOOL rc = file.Write3dmChunkVersion(1,0);
  if (rc) {
    if (rc) rc = file.WriteInt( m_bMaximized );
    if (rc) rc = file.WriteDouble( m_wnd_left );
    if (rc) rc = file.WriteDouble( m_wnd_right );
    if (rc) rc = file.WriteDouble( m_wnd_top );
    if (rc) rc = file.WriteDouble( m_wnd_bottom );
  }
  return rc;
}

BOOL ON_3dmViewPosition::Read( ON_BinaryArchive& file )
{
  int major_version = 0;
  int minor_version = 0;
  BOOL rc = file.Read3dmChunkVersion(&major_version,&minor_version);
  double x;
  Default();
  if (rc && major_version==1) {
    if (rc) rc = file.ReadInt( &m_bMaximized );
    if (rc) rc = file.ReadDouble( &m_wnd_left );
    if (rc) rc = file.ReadDouble( &m_wnd_right );
    if (rc) rc = file.ReadDouble( &m_wnd_top );
    if (rc) rc = file.ReadDouble( &m_wnd_bottom );
  }

  // if people put bogus values in a file, tune them up to something that will work
  if ( m_wnd_left > m_wnd_right ) {
    x = m_wnd_left; m_wnd_left = m_wnd_right; m_wnd_right = x;
  }
  if ( m_wnd_left  < 0.0 ) 
    m_wnd_left  = 0.0; 
  if ( m_wnd_right >= 1.0 ) 
    m_wnd_right = 1.0;
  if ( m_wnd_left >= m_wnd_right ) {
    m_wnd_left = 0.0;
    m_wnd_right = 1.0;
  }
  
  if ( m_wnd_top > m_wnd_bottom ) {
    x = m_wnd_top; m_wnd_top = m_wnd_bottom; m_wnd_bottom = x;
  }
  if ( m_wnd_top  < 0.0 ) 
    m_wnd_top  = 0.0; 
  if ( m_wnd_bottom >= 1.0 )
    m_wnd_bottom = 1.0;
  if ( m_wnd_top >= m_wnd_bottom ) {
    m_wnd_top = 0.0;
    m_wnd_bottom = 1.0;
  }

  return rc;
}

//////////////////////////////////////////////////////////////////////////////////////////
//
// ON_3dmViewTraceImage
//
ON_3dmViewTraceImage::ON_3dmViewTraceImage()
{
  Default();
}

ON_3dmViewTraceImage::~ON_3dmViewTraceImage()
{
}

ON_3dmViewTraceImage::ON_3dmViewTraceImage(const ON_3dmViewTraceImage& src)
{
  Default();
  *this = src;
}

ON_3dmViewTraceImage& ON_3dmViewTraceImage::operator=(const ON_3dmViewTraceImage& src)
{
  if ( this != &src ) {
    m_plane = src.m_plane;
    m_width = src.m_width;
    m_height = src.m_height;
    m_bitmap_filename = src.m_bitmap_filename;
  }
  return *this;
}

void ON_3dmViewTraceImage::Default()
{
  m_plane = ON_xy_plane;
  m_width = 0.0;
  m_height = 0.0;
  m_bitmap_filename.Destroy();
}

BOOL ON_3dmViewTraceImage::Write( ON_BinaryArchive& file ) const
{
  BOOL rc = file.Write3dmChunkVersion(1,0);
  if (rc) {
    if (rc) rc = file.WriteString( m_bitmap_filename );
    if (rc) rc = file.WriteDouble( m_width );
    if (rc) rc = file.WriteDouble( m_height );
    if (rc) rc = file.WritePlane( m_plane );
  }
  return rc;
}

BOOL ON_3dmViewTraceImage::Read( ON_BinaryArchive& file )
{
  int major_version = 0;
  int minor_version = 0;
  BOOL rc = file.Read3dmChunkVersion(&major_version,&minor_version);
  if (rc && major_version==1) {
    if (rc) rc = file.ReadString( m_bitmap_filename );
    if (rc) rc = file.ReadDouble( &m_width );
    if (rc) rc = file.ReadDouble( &m_height );
    if (rc) rc = file.ReadPlane( m_plane );
  }
  return rc;
}

//////////////////////////////////////////////////////////////////////////////////////////
//
// ON_3dmView
//
ON_3dmView::ON_3dmView()
{
  Default();
}

ON_3dmView::~ON_3dmView()
{
}

ON_3dmView::ON_3dmView(const ON_3dmView& src)
{
  Default();
  *this = src;
}

ON_3dmView& ON_3dmView::operator=(const ON_3dmView& src)
{
  if ( this != &src ) {
    m_name = src.m_name;
    m_vp = src.m_vp;
    m_display_mode = src.m_display_mode;
    m_target = src.m_target;
    m_position = src.m_position;
    m_cplane = src.m_cplane;
    m_bShowConstructionGrid = src.m_bShowConstructionGrid;
    m_bShowConstructionAxes = src.m_bShowConstructionAxes;
    m_bShowWorldAxes = src.m_bShowWorldAxes;
    m_trace_image = src.m_trace_image;
    m_wallpaper_bitmap_filename = src.m_wallpaper_bitmap_filename;
  }
  return *this;
}

void ON_3dmView::Default()
{
  m_name.Destroy();
  m_vp.Initialize();
  m_target = ON_origin;
  //m_vp.GetFrustumCenter(m_target);
  m_cplane.Default();
  m_display_mode = ON::wireframe_display;
  m_position.Default();
  if ( m_vp.Projection() == ON::parallel_view ) {
    m_cplane.m_plane.CreateFromFrame( m_cplane.m_plane.origin, m_vp.CameraX(), m_vp.CameraY() );
  }
  m_bShowConstructionGrid = TRUE;
  m_bShowConstructionAxes = TRUE;
  m_bShowWorldAxes = TRUE;
  m_trace_image.Default();
  m_wallpaper_bitmap_filename.Empty();
}

BOOL ON_3dmView::Write( ON_BinaryArchive& file ) const
{
  // Everything in a view is in a subchunk so new records can 
  // be added to a view and old I/O code will still
  // work.
  BOOL rc = TRUE;

  if(rc) {
    rc = file.BeginWrite3dmChunk( TCODE_VIEW_VIEWPORT, 0 );
    if(rc) {
      rc = m_vp.Write(file);
      if ( !file.EndWrite3dmChunk() )
        rc = FALSE;
    }
  }
  if(rc) {
    rc = file.BeginWrite3dmChunk( TCODE_VIEW_CPLANE, 0 );
    if(rc) {
      rc = m_cplane.Write(file);
      if ( !file.EndWrite3dmChunk() )
        rc = FALSE;
    }
  }
  if(rc) {
    rc = file.BeginWrite3dmChunk( TCODE_VIEW_TARGET, 0 );
    if(rc) {
      if(rc) rc = file.WritePoint(m_target);
      if ( !file.EndWrite3dmChunk() )
        rc = FALSE;
    }
  }
  if(rc) {
    rc = file.BeginWrite3dmChunk( TCODE_VIEW_DISPLAYMODE, m_display_mode );
    if(rc) {
      if ( !file.EndWrite3dmChunk() )
        rc = FALSE;
    }
  }
  if(rc) {
    rc = file.BeginWrite3dmChunk( TCODE_VIEW_POSITION, 0 );
    if(rc) {
      if(rc) rc = m_position.Write(file);
      if ( !file.EndWrite3dmChunk() )
        rc = FALSE;
    }
  }
  if(rc) {
    rc = file.BeginWrite3dmChunk( TCODE_VIEW_SHOWCONGRID, m_bShowConstructionGrid );
    if(rc) {
      if ( !file.EndWrite3dmChunk() )
        rc = FALSE;
    }
  }
  if(rc) {
    rc = file.BeginWrite3dmChunk( TCODE_VIEW_SHOWCONAXES, m_bShowConstructionAxes );
    if(rc) {
      if ( !file.EndWrite3dmChunk() )
        rc = FALSE;
    }
  }
  if(rc) {
    rc = file.BeginWrite3dmChunk( TCODE_VIEW_SHOWWORLDAXES, m_bShowWorldAxes );
    if(rc) {
      if ( !file.EndWrite3dmChunk() )
        rc = FALSE;
    }
  }
  if(rc) {
    rc = file.BeginWrite3dmChunk( TCODE_VIEW_NAME, 0 );
    if(rc) {
      if(rc) rc = file.WriteString(m_name);
      if ( !file.EndWrite3dmChunk() )
        rc = FALSE;
    }
  }
  if(rc) {
    rc = file.BeginWrite3dmChunk( TCODE_VIEW_TRACEIMAGE, 0 );
    if(rc) {
      if(rc) rc = m_trace_image.Write(file);
      if ( !file.EndWrite3dmChunk() )
        rc = FALSE;
    }
  }
  if(rc) {
    rc = file.BeginWrite3dmChunk( TCODE_VIEW_WALLPAPER, 0 );
    if(rc) {
      if(rc) rc = file.WriteString(m_wallpaper_bitmap_filename);
      if ( !file.EndWrite3dmChunk() )
        rc = FALSE;
    }
  }

  // required TCODE_ENDOFTABLE chunk - marks end of view table
  if ( rc ) {
    rc = file.BeginWrite3dmChunk( TCODE_ENDOFTABLE, 0 );
    if ( rc ) {
      if ( !file.EndWrite3dmChunk() )
        rc = FALSE;
    }
  }
  return rc;
}

BOOL ON_3dmView::Read( ON_BinaryArchive& file )
{
  // Everything in a view is in a subchunk so new records can 
  // be added to a view and old I/O code will still
  // work.
  unsigned int tcode = 0;
  int value = 0;
  BOOL rc = TRUE;
  while(rc) {
    rc = file.BeginRead3dmChunk(&tcode,&value);
    if (!rc)
      break;
    switch(tcode) {
    case TCODE_VIEW_CPLANE:
      rc = m_cplane.Read(file);
      break;
    case TCODE_VIEW_VIEWPORT:
      rc = m_vp.Read(file);
      break;
    case TCODE_VIEW_SHOWCONGRID:
      m_bShowConstructionGrid = value;
      break;
    case TCODE_VIEW_SHOWCONAXES:
      m_bShowConstructionAxes = value;
      break;
    case TCODE_VIEW_SHOWWORLDAXES:
      m_bShowWorldAxes = value;
      break;
    case TCODE_VIEW_TRACEIMAGE:
      rc = m_trace_image.Read(file);
      break;
    case TCODE_VIEW_WALLPAPER:
      rc = file.ReadString(m_wallpaper_bitmap_filename);
      break;
    case TCODE_VIEW_TARGET:
      rc = file.ReadPoint(m_target);
      break;
    case TCODE_VIEW_DISPLAYMODE:
      m_display_mode = ON::DisplayMode(value);
      break;
    case TCODE_VIEW_NAME:
      rc = file.ReadString(m_name);
      break;
    case TCODE_VIEW_POSITION:
      rc = m_position.Read(file);
      break;
    }
    if (!file.EndRead3dmChunk())
      rc = FALSE;
    if ( tcode == TCODE_ENDOFTABLE )
      break;
  }
  return rc;
}


//////////////////////////////////////////////////////////////////////////////////////////
//
// ON_3dmSettings
//

void ON_3dmSettings::Default()
{
  // default properties
  m_model_URL.Destroy();
  m_UnitsAndTolerances.Default();
  m_RenderMeshSettings.Default();
  m_AnnotationSettings.Default();
  m_named_cplanes.Empty();
  m_named_views.Empty();
  m_views.Empty();
  m_current_layer_index = 0;
  m_current_material_index = -1; // -1 = "default" material
  m_current_material_source = ON::material_from_layer;
  m_current_color.SetRGB(0,0,0);
  m_current_color_source = ON::color_from_layer;
  m_current_line_style.Default();
  m_current_line_style_source = ON::line_style_from_layer;
  m_current_wire_density = 1;
  m_RenderSettings.Default();
  m_GridDefaults.Default();
}

ON_3dmSettings::ON_3dmSettings()
{
  Default();
};

ON_3dmSettings::~ON_3dmSettings()
{
}

ON_3dmSettings::ON_3dmSettings(const ON_3dmSettings& src)
{
  *this = src;
}

ON_3dmSettings& ON_3dmSettings::operator=(const ON_3dmSettings& src)
{
  if ( this != &src ) {
    Default();
    m_model_URL = src.m_model_URL;
    m_UnitsAndTolerances   = src.m_UnitsAndTolerances;
    m_RenderMeshSettings   = src.m_RenderMeshSettings;
    m_AnalysisMeshSettings = src.m_AnalysisMeshSettings;
    m_AnnotationSettings   = src.m_AnnotationSettings;
    m_named_cplanes        = src.m_named_cplanes;
    m_named_views          = src.m_named_views;
    m_views                = src.m_views;
    m_current_layer_index       = src.m_current_layer_index;
    m_current_material_index    = src.m_current_material_index;
    m_current_material_source   = src.m_current_material_source;
    m_current_color             = src.m_current_color;
    m_current_color_source      = src.m_current_color_source;
    m_current_line_style        = src.m_current_line_style;
    m_current_line_style_source = src.m_current_line_style_source;
    m_current_wire_density      = src.m_current_wire_density;
    m_RenderSettings          = src.m_RenderSettings;
    m_GridDefaults            = src.m_GridDefaults;
  }
  return *this;
}

static BOOL ON_3dmSettings_Read_v1_TCODE_NAME(ON_BinaryArchive& file, ON_wString& str )
{
  // reads legacy 1.0 named view TCODE_NAME chunk
  str.Empty();
  int len = 0;
  BOOL rc = file.ReadInt( &len );
  if (rc && len > 0) {
    char* name = (char*)oncalloc( 1, len + 1);
    rc = file.ReadString( len, name );
    if (rc)
      str = name; // ASCII -> UNICODE
    if (name)
      onfree(name);
  }
  return rc;
}

static BOOL ON_3dmSettings_Read_v1_TCODE_CPLANE(ON_BinaryArchive& file, ON_3dmConstructionPlane& cplane)
{
  // reads legacy 1.0 named view TCODE_CPLANE chunk

  // do NOT call cplane.Default() here
  BOOL rc = TRUE;
	ON_3dPoint origin;
	ON_3dVector xaxis, yaxis;
	double gridsize;
	int gridsections, gridthicksections;
  if (rc) rc = file.ReadPoint( origin );
  if (rc) rc = file.ReadVector( xaxis );
  if (rc) rc = file.ReadVector( yaxis );
  if (rc) rc = file.ReadDouble(&gridsize);
  if (rc) rc = file.ReadInt(&gridsections);
  if (rc) rc = file.ReadInt(&gridthicksections);
  if (rc) {
    cplane.m_plane.CreateFromFrame(origin,xaxis,yaxis);
    cplane.m_grid_line_count = gridsections;
    cplane.m_grid_thick_frequency = gridthicksections;
    cplane.m_grid_spacing = gridsize;
    cplane.m_snap_spacing = gridsize;
  }
  return rc;
}

static BOOL ON_3dmSettings_Read_v1_TCODE_VIEW(ON_BinaryArchive& file, ON_3dmView& view)
{
  // reads legacy 1.0 named view TCODE_VIEW chunk
  // do NOT call view.Default() here
  BOOL rc = TRUE;

	int projection, valid;
	double angle1, angle2, angle3, viewsize, cameradist;
  if (rc) rc = file.ReadInt(&projection);
  if (rc) rc = file.ReadInt(&valid);
  if (rc) rc = file.ReadPoint( view.m_target );
  if (rc) rc = file.ReadDouble( &angle1 );
  if (rc) rc = file.ReadDouble( &angle2 );
  if (rc) rc = file.ReadDouble( &angle3 );
  if (rc) rc = file.ReadDouble( &viewsize );
  if (rc) rc = file.ReadDouble( &cameradist );

  if (rc) {
    if( cameradist <= 0.0 || cameradist >= ( DBL_MAX / 2.0 ))
      cameradist = 100.0;
    if( viewsize <= 0.0 || viewsize >= ( DBL_MAX / 2.0 ))
      viewsize = 0.125;
    ON_ViewportFromRhinoView(
          projection == 2 ? ON::perspective_view : ON::parallel_view,
          view.m_target,
          angle1,
          angle2,
          angle3,
          viewsize,
          cameradist,
          100, // screen_width, 
          100, // screen_height,
          view.m_vp
          );
  }

  return rc;
}

static BOOL ON_3dmSettings_Read_v1_TCODE_NAMED_VIEW(ON_BinaryArchive& file, ON_3dmView& view)
{
  // reads legacy 1.0 named view TCODE_NAMED_VIEW chunk
  view.Default();
  BOOL rc = TRUE;
  unsigned int tcode;
  int value;

  while(rc) {
    rc = file.BeginRead3dmChunk( &tcode, &value );
    if (!rc )
      break;
    switch(tcode) {

    case TCODE_NAME:
      rc = ON_3dmSettings_Read_v1_TCODE_NAME(file,view.m_name);
      break;

    case TCODE_CPLANE:
      rc = ON_3dmSettings_Read_v1_TCODE_CPLANE(file,view.m_cplane);
      break;

    case TCODE_VIEW:
      rc = ON_3dmSettings_Read_v1_TCODE_VIEW( file, view );
      break;

    case TCODE_SHOWGRID:
      view.m_bShowConstructionGrid = value;
      break;
						
    case TCODE_SHOWGRIDAXES:
      view.m_bShowConstructionAxes = value;
      break;
						
    case TCODE_SHOWWORLDAXES:
      view.m_bShowWorldAxes = value;
      break; 			
      
    }
    if ( !file.EndRead3dmChunk() )
      rc = FALSE;
    if ( tcode == TCODE_ENDOFTABLE )
      break;
  }
  return rc;
}

static BOOL ON_3dmSettings_Read_v1_TCODE_NAMED_CPLANE(ON_BinaryArchive& file, ON_3dmConstructionPlane& cplane)
{
  // reads legacy 1.0 named construction plane TCODE_NAMED_CPLANE chunk
  cplane.Default();

  BOOL rc = TRUE;
  unsigned int tcode;
  int value;

  while(rc) {
    rc = file.BeginRead3dmChunk( &tcode, &value );
    if (!rc )
      break;
    switch(tcode) {

    case TCODE_NAME:
      rc = ON_3dmSettings_Read_v1_TCODE_NAME(file, cplane.m_name );
      break;

    case TCODE_CPLANE:
      rc = ON_3dmSettings_Read_v1_TCODE_CPLANE(file, cplane );
      break;
    }
    if ( !file.EndRead3dmChunk() )
      rc = FALSE;
    if ( tcode == TCODE_ENDOFTABLE )
      break;
  }
  return rc;
}

static BOOL ON_3dmSettings_Read_v1_TCODE_UNIT_AND_TOLERANCES(ON_BinaryArchive& file, ON_3dmUnitsAndTolerances& UnitsAndTolerances )
{
  BOOL rc = TRUE;
  int v, us;
  if (rc) rc = file.ReadInt( &v ); // v = 1
  if (rc) rc = file.ReadInt( &us );
  switch (us) 
  {
  case 0: // NO_UNIT_SYSTEM:
    UnitsAndTolerances.m_unit_system = ON::no_unit_system;
    break;
  case 1: // MICRONS:
    UnitsAndTolerances.m_unit_system = ON::microns;
    break;
  case 2: // MILLIMETERS:
    UnitsAndTolerances.m_unit_system = ON::millimeters;
    break;
  case 3: // CENTIMETERS:
    UnitsAndTolerances.m_unit_system = ON::centimeters;
    break;
  case 4: // METERS:
    UnitsAndTolerances.m_unit_system = ON::meters;
    break;
  case 5: // KILOMETERS:
    UnitsAndTolerances.m_unit_system = ON::kilometers;
    break;
  case 6: // MICROINCHES:
    UnitsAndTolerances.m_unit_system = ON::microinches;
    break;
  case 7: // MILS:
    UnitsAndTolerances.m_unit_system = ON::mils;
    break;
  case 8: // INCHES:
    UnitsAndTolerances.m_unit_system = ON::inches;
    break;
  case 9: // FEET:
    UnitsAndTolerances.m_unit_system = ON::feet;
    break;
  case 10: // MILES:
    UnitsAndTolerances.m_unit_system = ON::miles;
    break;
  default: // NO_UNIT_SYSTEM:
    UnitsAndTolerances.m_unit_system = ON::no_unit_system;
    break;
  }
  if (rc) rc = file.ReadDouble( &UnitsAndTolerances.m_absolute_tolerance );
  if (rc) rc = file.ReadDouble( &UnitsAndTolerances.m_relative_tolerance );
  if (rc) rc = file.ReadDouble( &UnitsAndTolerances.m_angle_tolerance );
  return rc;
}

static BOOL ON_3dmSettings_Read_v1_TCODE_VIEWPORT(ON_BinaryArchive& file, ON_3dmView& view)
{
  // reads legacy 1.0 named construction plane TCODE_VIEWPORT chunk
  view.Default();
  BOOL rc = TRUE;
  unsigned int tcode;
  int value;

  double clipdist = 0.0;
  double snapsize = 0.0;

  while(rc) {
    rc = file.BeginRead3dmChunk( &tcode, &value );
    if (!rc )
      break;
    switch(tcode) {

    case TCODE_NEAR_CLIP_PLANE:
      rc = file.ReadDouble(&clipdist);
      break;

    case TCODE_SNAPSIZE:
      rc = file.ReadDouble(&snapsize);
      break;
      
    case TCODE_NAME:
      rc = ON_3dmSettings_Read_v1_TCODE_NAME(file,view.m_name);
      break;

    case TCODE_CPLANE:
      rc = ON_3dmSettings_Read_v1_TCODE_CPLANE(file,view.m_cplane);
      break;

    case TCODE_VIEW:
      rc = ON_3dmSettings_Read_v1_TCODE_VIEW( file, view );
      break;

    case TCODE_SHOWGRID:
      view.m_bShowConstructionGrid = value;
      break;
						
    case TCODE_SHOWGRIDAXES:
      view.m_bShowConstructionAxes = value;
      break;
						
    case TCODE_SHOWWORLDAXES:
      view.m_bShowWorldAxes = value;
      break; 			
      
    case TCODE_VIEWPORT_POSITION:
      rc = file.ReadDouble(&view.m_position.m_wnd_left);
      rc = file.ReadDouble(&view.m_position.m_wnd_top);
      rc = file.ReadDouble(&view.m_position.m_wnd_right);
      rc = file.ReadDouble(&view.m_position.m_wnd_bottom);
      break;
						
    case TCODE_VIEWPORT_TRACEINFO:
      {
        ON_3dPoint origin;
        ON_3dVector xaxis, yaxis;
        if (rc) rc = file.ReadPoint( origin );
        if (rc) rc = file.ReadVector( xaxis );
        if (rc) rc = file.ReadVector( yaxis );
        view.m_trace_image.m_plane.CreateFromFrame(origin,xaxis,yaxis);
        if (rc) rc = file.ReadDouble(&view.m_trace_image.m_width);
        if (rc) rc = file.ReadDouble(&view.m_trace_image.m_height);
        if (rc) rc = ON_3dmSettings_Read_v1_TCODE_NAME(file,view.m_trace_image.m_bitmap_filename);
      }
      break;
      
    case TCODE_VIEWPORT_WALLPAPER:
      rc = ON_3dmSettings_Read_v1_TCODE_NAME(file,view.m_wallpaper_bitmap_filename);
      break;
						
    case TCODE_HIDE_TRACE:
      // TCODE_HIDE_TRACE was used in early 1.0 betas.  
      // It should have add the short bit set and it is no longer used.
      // This case is here so that these old files will read correctly.
      tcode |= TCODE_SHORT; // so goo skip will work
      break;
      
    case TCODE_MAXIMIZED_VIEWPORT:
      if ( value )
        view.m_position.m_bMaximized = TRUE;
      break; 

    case TCODE_VIEWPORT_DISPLAY_MODE: // short TCODE with display mode value
      switch ( value ) {
      case 0: // wireframe working mode
        view.m_display_mode = ON::wireframe_display;
        break;
      case 1: // shaded working mode
        view.m_display_mode = ON::shaded_display;
        break;
      }
      break;
						
    }
    if ( !file.EndRead3dmChunk() )
      rc = FALSE;
    if ( tcode == TCODE_ENDOFTABLE )
      break;
  }
  return rc;
}

BOOL ON_3dmSettings::Read_v1( ON_BinaryArchive& file )
{
  BOOL bGotSomething = FALSE;
  BOOL rc = FALSE;
  // read settings from old version 1 file
  int pos0 = file.CurrentPosition();

  // need to start at the beginning of the file
  unsigned int tcode;
  int value;
  rc = file.SeekFromStart(32);
  while( rc ) // skip 32 byte header
  {
    rc = file.BeginRead3dmChunk( &tcode, &value );
    if ( !rc ) 
      break; // assume we are at the end of the file

    switch(tcode) {
    case TCODE_VIEWPORT:
      bGotSomething = TRUE;
      {
        ON_3dmView view;
        rc = ON_3dmSettings_Read_v1_TCODE_VIEWPORT(file, view);
        if (rc)
          m_views.Append(view);
      }
      break;

    case TCODE_NAMED_CPLANE:
      bGotSomething = TRUE;
      {
        ON_3dmConstructionPlane cplane;
        rc = ON_3dmSettings_Read_v1_TCODE_NAMED_CPLANE(file,cplane);
        if (rc)
          m_named_cplanes.Append(cplane);
      }
      break;

    case TCODE_NAMED_VIEW:
      bGotSomething = TRUE;
      {
        ON_3dmView view;
        rc = ON_3dmSettings_Read_v1_TCODE_NAMED_VIEW(file, view);
        if (rc)
          m_named_views.Append(view);
      }
      break;
    
    case TCODE_UNIT_AND_TOLERANCES:
      bGotSomething = TRUE;
      rc = ON_3dmSettings_Read_v1_TCODE_UNIT_AND_TOLERANCES(file,m_UnitsAndTolerances);
      break;
    }

    rc = file.EndRead3dmChunk();
  }

  file.SeekFromStart(pos0);
  return bGotSomething;
}

BOOL ON_3dmSettings::Read_v2(ON_BinaryArchive& file )
{
  BOOL rc = TRUE;
  unsigned int tcode;
  int value;

  while(rc) {

    rc = file.BeginRead3dmChunk( &tcode, &value );
    if ( !rc )
      break;

    switch(tcode) {
      
    case TCODE_SETTINGS_UNITSANDTOLS: // units and tolerances
      rc = m_UnitsAndTolerances.Read(file);
      break;
      
    case TCODE_SETTINGS_RENDERMESH:
      rc = m_RenderMeshSettings.Read(file);
      break;
      
    case TCODE_SETTINGS_ANALYSISMESH:
      rc = m_AnalysisMeshSettings.Read(file);
      break;
      
    case TCODE_SETTINGS_ANNOTATION:
      rc = m_AnnotationSettings.Read(file);
      break;
      
    case TCODE_SETTINGS_NAMED_CPLANE_LIST: // named cplanes
      {
        m_named_cplanes.Empty();
        unsigned int subtcode;
        int count, i, subvalue;
        rc = file.ReadInt(&count);
        for ( i = 0; i < count && rc ; i++ ) {
          rc = file.BeginRead3dmChunk( &subtcode, &subvalue );
          if (rc ) {
            if ( subtcode != TCODE_VIEW_CPLANE )
              rc = FALSE;
            else {
              ON_3dmConstructionPlane& cplane = m_named_cplanes.AppendNew();
              rc = cplane.Read(file);
            }
            if ( !file.EndRead3dmChunk() ) {
              rc = FALSE;
            }
          }
        }
      }
      break;
      
    case TCODE_SETTINGS_NAMED_VIEW_LIST: // named views
      {
        m_named_views.Empty();
        unsigned int subtcode;
        int count, i, subvalue;
        rc = file.ReadInt(&count);
        for ( i = 0; i < count && rc ; i++ ) {
          rc = file.BeginRead3dmChunk( &subtcode, &subvalue );
          if (rc ) {
            if ( subtcode != TCODE_VIEW_RECORD )
              rc = FALSE;
            else {
              ON_3dmView& cplane = m_named_views.AppendNew();
              rc = cplane.Read(file);
            }
            if ( !file.EndRead3dmChunk() ) {
              rc = FALSE;
            }
          }
        }
      }
      break;
      
    case TCODE_SETTINGS_VIEW_LIST: // active view is first in list
      {
        m_views.Empty();
        unsigned int subtcode;
        int count, i, subvalue;
        rc = file.ReadInt(&count);
        m_views.Reserve(count);
        for ( i = 0; i < count && rc ; i++ ) {
          rc = file.BeginRead3dmChunk( &subtcode, &subvalue );
          if (rc ) {
            if ( subtcode != TCODE_VIEW_RECORD )
              rc = FALSE;
            else {
              ON_3dmView& view = m_views.AppendNew();
              rc = view.Read(file);
            }
            if ( !file.EndRead3dmChunk() ) {
              rc = FALSE;
            }
          }
        }
      }
      break;
      
    case TCODE_SETTINGS_CURRENT_LAYER_INDEX:
      m_current_layer_index = value;
      break;
      
    case TCODE_SETTINGS_CURRENT_MATERIAL_INDEX:
      {
        if (rc) rc = file.ReadInt( &m_current_material_index );
        if (rc) rc = file.ReadInt( &value );
        if (rc) m_current_material_source = ON::ObjectMaterialSource(value);
      }
      break;
      
    case TCODE_SETTINGS_CURRENT_COLOR:
      {
        if (rc) rc = file.ReadColor( m_current_color );
        if (rc) rc = file.ReadInt( &value );
        if (rc) m_current_color_source = ON::ObjectColorSource(value);
      }
      break;
      
    case TCODE_SETTINGS_CURRENT_LINE_STYLE:
      {
        if (rc) rc = file.ReadLineStyle( m_current_line_style );
        if (rc) rc = file.ReadInt( &value );
        if (rc) m_current_line_style_source = ON::ObjectLineStyleSource(value);
      }
      break;
      
    case TCODE_SETTINGS_CURRENT_WIRE_DENSITY:
      m_current_wire_density = value;
      break;
      
    case TCODE_SETTINGS_RENDER:
      rc = m_RenderSettings.Read(file);
      break;

    case TCODE_SETTINGS_GRID_DEFAULTS:
      rc = m_GridDefaults.Read(file);
      break;

    case TCODE_SETTINGS_MODEL_URL: // added 21 June 2001
      rc = file.ReadString(m_model_URL);
      break;

    default:
      // information added in future will be skipped by file.EndRead3dmChunk()
      break;
    }

    if ( !file.EndRead3dmChunk() )
      rc = FALSE;
    if ( TCODE_ENDOFTABLE == tcode )
      break;
  }

  return rc;
}

BOOL ON_3dmSettings::Read(ON_BinaryArchive& file )
{
  BOOL rc = FALSE;

  Default();

  switch( file.Archive3dmVersion() ) {
  case 1: rc = Read_v1(file); break;
  case 2: rc = Read_v2(file); break;
  }

  return rc;
}


static BOOL ON_3dmSettings_Write_v1_TCODE_UNIT_AND_TOLERANCES(ON_BinaryArchive& file, const ON_3dmUnitsAndTolerances& UnitsAndTolerances )
{
  BOOL rc = TRUE;
  int v = 1, us = 0;
  if (rc) rc = file.WriteInt( v ); // v = 1
  switch (UnitsAndTolerances.m_unit_system) 
  {
  case ON::no_unit_system: 
    us=0; // NO_UNIT_SYSTEM
    break;
  case ON::microns: 
    us=1; // MICRONS
    break;
  case ON::millimeters:
    us=2; // MILLIMETERS
    break;
  case ON::centimeters:
    us=3; // CENTIMETERS
    break;
  case ON::meters:
    us=4; // METERS
    break;
  case ON::kilometers:
    us=5; // KILOMETERS
    break;
  case ON::microinches:
    us=6; // MICROINCHES
    break;
  case ON::mils:
    us=7; // MILS
    break;
  case ON::inches:
    us=8; // INCHES
    break;
  case ON::feet:
    us=9; // FEET
    break;
  case ON::miles:
    us=10; // MILES
    break;
  default:
    us=0; // NO_UNIT_SYSTEM
    break;
  }
  if (rc) rc = file.WriteInt( us );
  if (rc) rc = file.WriteDouble( UnitsAndTolerances.m_absolute_tolerance );
  if (rc) rc = file.WriteDouble( UnitsAndTolerances.m_relative_tolerance );
  if (rc) rc = file.WriteDouble( UnitsAndTolerances.m_angle_tolerance );

  return rc;
}

BOOL ON_3dmSettings::Write_v1(ON_BinaryArchive& file) const
{
  BOOL rc = TRUE;

  // version 1 units and tolerances chunk
  rc = file.BeginWrite3dmChunk(TCODE_UNIT_AND_TOLERANCES,0);
  if (rc) {
    rc = ON_3dmSettings_Write_v1_TCODE_UNIT_AND_TOLERANCES( file, m_UnitsAndTolerances );
    if (!file.EndWrite3dmChunk())
      rc = FALSE;
  }

  return rc;
}

BOOL ON_3dmSettings::Write_v2(ON_BinaryArchive& file) const
{
  int i;
  BOOL rc = TRUE;

  // TCODE_PROPERTIES_UNITSANDTOLS - units and tolerances
  if ( rc  ) {
    rc = file.BeginWrite3dmChunk(TCODE_SETTINGS_UNITSANDTOLS,0);
    if ( rc ) {
      rc = m_UnitsAndTolerances.Write(file);
      if ( !file.EndWrite3dmChunk() )
        rc = FALSE;
    }
  }

  // TCODE_SETTINGS_RENDERMESH - rendering defaults
  if ( rc  ) {
    rc = file.BeginWrite3dmChunk(TCODE_SETTINGS_RENDERMESH,0);
    if ( rc ) {
      rc = m_RenderMeshSettings.Write(file);
      if ( !file.EndWrite3dmChunk() )
        rc = FALSE;
    }
  }

  // TCODE_SETTINGS_ANALYSISMESH - analysis mesh defaults
  if ( rc  ) {
    rc = file.BeginWrite3dmChunk(TCODE_SETTINGS_ANALYSISMESH,0);
    if ( rc ) {
      rc = m_AnalysisMeshSettings.Write(file);
      if ( !file.EndWrite3dmChunk() )
        rc = FALSE;
    }
  }

  // TCODE_SETTINGS_ANNOTATION - annotation settings
  if ( rc  ) {
    rc = file.BeginWrite3dmChunk(TCODE_SETTINGS_ANNOTATION,0);
    if ( rc ) {
      rc = m_AnnotationSettings.Write(file);
      if ( !file.EndWrite3dmChunk() )
        rc = FALSE;
    }
  }

  // TCODE_SETTINGS_NAMED_CPLANE_LIST
  if ( rc  ) {
    rc = file.BeginWrite3dmChunk(TCODE_SETTINGS_NAMED_CPLANE_LIST,0);
    if ( rc ) {
      const int count = m_named_cplanes.Count();
      rc = file.WriteInt(count);
      for ( i = 0; i < count && rc; i++ ) {
        rc = file.BeginWrite3dmChunk( TCODE_VIEW_CPLANE, 0 );
        if (rc ) {
          rc = m_named_cplanes[i].Write(file);
          if ( !file.EndWrite3dmChunk() )
            rc = FALSE;
        }
      }
      if ( !file.EndWrite3dmChunk() )
        rc = FALSE;
    }
  }

  // TCODE_SETTINGS_NAMED_VIEW_LIST
  if ( rc  ) {
    rc = file.BeginWrite3dmChunk(TCODE_SETTINGS_NAMED_VIEW_LIST,0);
    if ( rc ) {
      const int count = m_named_views.Count();
      rc = file.WriteInt(count);
      for ( i = 0; i < count && rc; i++ ) {
        rc = file.BeginWrite3dmChunk( TCODE_VIEW_RECORD, 0 );
        if (rc ) {
          rc = m_named_views[i].Write(file);
          if ( !file.EndWrite3dmChunk() )
            rc = FALSE;
        }
      }
      if ( !file.EndWrite3dmChunk() )
        rc = FALSE;
    }
  }

  // TCODE_SETTINGS_VIEW_LIST
  if ( rc  ) {
    rc = file.BeginWrite3dmChunk(TCODE_SETTINGS_VIEW_LIST,0);
    if ( rc ) {
      const int count = m_views.Count();
      rc = file.WriteInt(count);
      for ( i = 0; i < count && rc; i++ ) {
        rc = file.BeginWrite3dmChunk( TCODE_VIEW_RECORD, 0 );
        if (rc ) {
          rc = m_views[i].Write(file);
          if ( !file.EndWrite3dmChunk() )
            rc = FALSE;
        }
      }
      if ( !file.EndWrite3dmChunk() )
        rc = FALSE;
    }
  }

  // TCODE_SETTINGS_CURRENT_LAYER_INDEX
  if (rc) {
    rc = file.BeginWrite3dmChunk( TCODE_SETTINGS_CURRENT_LAYER_INDEX, m_current_layer_index );
    if ( !file.EndWrite3dmChunk() )
      rc = FALSE;
  }

  // TCODE_SETTINGS_CURRENT_MATERIAL_INDEX
  if (rc) {
    rc = file.BeginWrite3dmChunk( TCODE_SETTINGS_CURRENT_MATERIAL_INDEX, 0 );
    if (rc) {
      rc = file.WriteInt( m_current_material_index );
      i = m_current_material_source;
      if (rc) rc = file.WriteInt( i );
      if ( !file.EndWrite3dmChunk() )
        rc = FALSE;
    }
  }

  // TCODE_SETTINGS_CURRENT_COLOR
  if (rc) {
    rc = file.BeginWrite3dmChunk( TCODE_SETTINGS_CURRENT_COLOR, 0 );
    if (rc) {
      rc = file.WriteColor( m_current_color );
      i = m_current_color_source;
      if (rc) rc = file.WriteInt( i );
      if ( !file.EndWrite3dmChunk() )
        rc = FALSE;
    }
  }

  // TCODE_SETTINGS_CURRENT_LINE_STYLE
  if (rc) {
    rc = file.BeginWrite3dmChunk( TCODE_SETTINGS_CURRENT_LINE_STYLE, 0 );
    if (rc) {
      rc = file.WriteLineStyle( m_current_line_style );
      i = m_current_line_style_source;
      if (rc) rc = file.WriteInt( i );
      if ( !file.EndWrite3dmChunk() )
        rc = FALSE;
    }
  }

  // TCODE_SETTINGS_CURRENT_WIRE_DENSITY
  if (rc) {
    rc = file.BeginWrite3dmChunk( TCODE_SETTINGS_CURRENT_WIRE_DENSITY, m_current_wire_density );
    if (rc) {
      if ( !file.EndWrite3dmChunk() )
        rc = FALSE;
    }
  }

  // TCODE_SETTINGS_RENDER
  if (rc) {
    rc = file.BeginWrite3dmChunk( TCODE_SETTINGS_RENDER, 0 );
    if (rc) {
      rc = m_RenderSettings.Write(file);
      if ( !file.EndWrite3dmChunk() )
        rc = FALSE;
    }
  }

  // TCODE_SETTINGS_GRID_DEFAULTS
  if (rc) {
    rc = file.BeginWrite3dmChunk( TCODE_SETTINGS_GRID_DEFAULTS, 0 );
    if (rc) {
      rc = m_GridDefaults.Write(file);
      if ( !file.EndWrite3dmChunk() )
        rc = FALSE;
    }
  }

  // TCODE_SETTINGS_MODEL_URL - added 21 June 2001
  if (rc && m_model_URL.Length() > 0 ) {
    rc = file.BeginWrite3dmChunk( TCODE_SETTINGS_MODEL_URL, 0 );
    if (rc) {
      rc = file.WriteString(m_model_URL);
      if ( !file.EndWrite3dmChunk() )
        rc = FALSE;
    }
  }

  // required TCODE_ENDOFTABLE chunk - marks end of settings table
  if ( rc ) {
    rc = file.BeginWrite3dmChunk( TCODE_ENDOFTABLE, 0 );
    if ( rc ) {
      if ( !file.EndWrite3dmChunk() )
        rc = FALSE;
    }
  }

  return rc;
}

BOOL ON_3dmSettings::Write(ON_BinaryArchive& file) const
{
  BOOL rc = FALSE;
  switch( file.Archive3dmVersion() ) {
  case 1: rc = Write_v1(file); break;
  case 2: rc = Write_v2(file); break;
  }
  return rc;
}

void ON_3dmSettings::Dump( ON_TextLog& dump ) const
{
  const wchar_t* model_URL = m_model_URL;
  if ( model_URL && *model_URL ) {
    dump.Print("Model URL: %S\n",model_URL);
  }
  dump.Print("Units and tolerances:\n");
  dump.PushIndent();
  m_UnitsAndTolerances.Dump(dump);
  dump.PopIndent();
}

