/* $Header: /src2/opennurbs/opennurbs_3dm_settings.h 21    6/22/01 11:04a Dalelear $ */
/* $NoKeywords: $ */
/*
//
// Copyright (c) 1993-2001 Robert McNeel & Associates. All rights reserved.
// Rhinoceros is a registered trademark of Robert McNeel & Assoicates.
//
// THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
// ALL IMPLIED WARRANTIES OF FITNESS FOR ANY PARTICULAR PURPOSE AND OF
// MERCHANTABILITY ARE HEREBY DISCLAIMED.
//				
// For complete openNURBS copyright information see <http://www.opennurbs.org>.
//
////////////////////////////////////////////////////////////////
*/

#if !defined(OPENNURBS_3DM_SETTINGS_INC_)
#define OPENNURBS_3DM_SETTINGS_INC_

///////////////////////////////////////////////////////////////////////
//
// units and tolerances
//

class ON_CLASS ON_3dmUnitsAndTolerances
{
public:
  ON_3dmUnitsAndTolerances();
  ON_3dmUnitsAndTolerances(const ON_3dmUnitsAndTolerances&);
  ~ON_3dmUnitsAndTolerances();
  ON_3dmUnitsAndTolerances& operator=(const ON_3dmUnitsAndTolerances&);

  void Default();

  BOOL Read( ON_BinaryArchive& );
  BOOL Write( ON_BinaryArchive& ) const;

  void Dump( ON_TextLog& ) const;

  //////////
  // Returns scale factor that needs to be applied to change from
  // the argument's unit system to m_unit_system.  
  // When m_unit_system is not ON::custom_unit_system,
  // Scale(us) = ON::UnitScale(us,m_unit_system).  When Scale(us)
  // When m_unit_system is ON::custom_unit_system,
  // Scale(us) = ON::UnitScale(us,ON::meters)*m_custom_unit_scale.
  double Scale( ON::unit_system ) const;

  ON::unit_system m_unit_system;
  double          m_absolute_tolerance;  // in units (default = 1/100)
  double          m_angle_tolerance;     // in radians (default = 3 degrees)
  double          m_relative_tolerance;  // fraction >= 0 and < 1 (default = 1%)

  ON::distance_display_mode m_distance_display_mode;
  int m_distance_display_precision; // decimal mode: number of decimal places
                                    // fractional modes:
                                    //    denominator = (1/2)^m_distance_display_precision

  ////////////
  // These settings apply when m_unit_system is ON::custom_unit_system
  //
  double m_custom_unit_scale;      // 1 meter = m_custom_unit_scale custom units
  ON_wString m_custom_unit_name;   // name of custom units
};

///////////////////////////////////////////////////////////////////////
//
// Model settings
//   render mesh defaults
//   viewports
//   construction planes
//

class ON_CLASS ON_3dmAnnotationSettings
{
public:
  ON_3dmAnnotationSettings();
  ~ON_3dmAnnotationSettings();
  ON_3dmAnnotationSettings(const ON_3dmAnnotationSettings&);
  ON_3dmAnnotationSettings& operator=(const ON_3dmAnnotationSettings&);

  void Default();

  BOOL Read( ON_BinaryArchive& );
  BOOL Write( ON_BinaryArchive& ) const;

  // these are the running defaults for making dimensions
  // they are also the things written to the 3dm file as dimension settings
  double m_dimscale;       // model size / plotted size
  double m_textheight;
  double m_dimexe;
  double m_dimexo;
  double m_arrowlength;
  double m_arrowwidth;
  double m_centermark;

  ON::unit_system m_dimunits;  // units used to measure the dimension
  int m_arrowtype;     // 0: filled narrow triangular arrow
  int m_angularunits;  // 0: degrees, 1: radians
  int m_lengthformat;  // 0: decimal, ...
  int m_angleformat;   // 0: decimal degrees, ...
  int m_textalign;     // 0: above line, 1: in line, 2: horizontal
  int m_resolution;    // depends on m_lengthformat
                       // for decimal, digits past the decimal point

  ON_wString m_facename; // [LF_FACESIZE] // windows font name
};

//////////////////////////////////////////////////////////////////////////////////////////
//
// ON_3dmConstructionPlaneGridDefaults
//
// Default settings used for construction plane grids
class ON_CLASS ON_3dmConstructionPlaneGridDefaults
{
public:
  ON_3dmConstructionPlaneGridDefaults();
  ~ON_3dmConstructionPlaneGridDefaults();
  ON_3dmConstructionPlaneGridDefaults(const ON_3dmConstructionPlaneGridDefaults&);
  ON_3dmConstructionPlaneGridDefaults& operator=(const ON_3dmConstructionPlaneGridDefaults&);

  void Default();

  BOOL Write( ON_BinaryArchive& ) const;
  BOOL Read( ON_BinaryArchive& );

	double m_grid_spacing;   // distance between grid lines
  double m_snap_spacing;   // when "grid snap" is enabled, the
                           // distance between snap points.  Typically
                           // this is the same distance as grid spacing.
	int m_grid_line_count;   // number of grid lines in each direction
  int m_grid_thick_frequency; // thick line frequency
                            // 0: none, 
                            // 1: all lines are thick, 
                            // 2: every other is thick, ...

  BOOL m_bShowGrid;
  BOOL m_bShowGridAxes;
  BOOL m_bShowWorldAxes;
};

//////////////////////////////////////////////////////////////////////////////////////////
//
// ON_3dmConstructionPlane
//
class ON_CLASS ON_3dmConstructionPlane
{
public:
  ON_3dmConstructionPlane();
  ~ON_3dmConstructionPlane();
  ON_3dmConstructionPlane(const ON_3dmConstructionPlane&);
  ON_3dmConstructionPlane& operator=(const ON_3dmConstructionPlane&);

  void Default();

  BOOL Write( ON_BinaryArchive& ) const;
  BOOL Read( ON_BinaryArchive& );

  ON_Plane    m_plane;

  // construction grid appearance
	double m_grid_spacing;   // distance between grid lines
  double m_snap_spacing;   // when "grid snap" is enabled, the
                           // distance between snap points.  Typically
                           // this is the same distance as grid spacing.
	int m_grid_line_count;   // number of grid lines in each direction
  int m_grid_thick_frequency; // thick line frequency
                            // 0: none, 
                            // 1: all lines are thick, 
                            // 2: every other is thick, ...

  ON_wString  m_name;
};

#if defined(ON_DLL_EXPORTS) || defined(ON_DLL_IMPORTS)
// This stuff is here because of a limitation in the way Microsoft
// handles templates and DLLs.  See Microsoft's knowledge base 
// article ID Q168958 for details.
#pragma warning( push )
#pragma warning( disable : 4231 )
ON_TEMPLATE template class ON_CLASS ON_ClassArray<ON_3dmConstructionPlane>;
#pragma warning( pop )
#endif

//////////////////////////////////////////////////////////////////////////////////////////
//
// ON_3dmViewPos
//
class ON_CLASS ON_3dmViewPosition
{
public:
  // view window relative position and state in parent frame
  ON_3dmViewPosition();
  ~ON_3dmViewPosition();
  ON_3dmViewPosition(const ON_3dmViewPosition&);
  ON_3dmViewPosition& operator=(const ON_3dmViewPosition&);

  void Default();

  BOOL Write( ON_BinaryArchive& ) const;
  BOOL Read( ON_BinaryArchive& );

  // relative position of view window in main frame
  double m_wnd_left;    // 0.0 to 1.0
  double m_wnd_right;
  double m_wnd_top;
  double m_wnd_bottom;
  BOOL m_bMaximized;    // TRUE if view window is maximized
};

//////////////////////////////////////////////////////////////////////////////////////////
//
// ON_3dmViewTraceImage
//
class ON_CLASS ON_3dmViewTraceImage
{
public:
  // view window relative position and state in parent frame
  ON_3dmViewTraceImage();
  ~ON_3dmViewTraceImage();
  ON_3dmViewTraceImage(const ON_3dmViewTraceImage&);
  ON_3dmViewTraceImage& operator=(const ON_3dmViewTraceImage&);

  void Default();

  BOOL Write( ON_BinaryArchive& ) const;
  BOOL Read( ON_BinaryArchive& );

  ON_Plane m_plane;
  double   m_width;
  double   m_height;
  ON_wString m_bitmap_filename;
};

//////////////////////////////////////////////////////////////////////////////////////////
//
// ON_3dmView
//

class ON_CLASS ON_3dmView
{
public:
  ON_3dmView();
  ~ON_3dmView();
  ON_3dmView(const ON_3dmView&);
  ON_3dmView& operator=(const ON_3dmView&);

  void Default();

  BOOL Write( ON_BinaryArchive& ) const;
  BOOL Read( ON_BinaryArchive& );

  ON_wString  m_name;              // name on window
  ON_Viewport m_vp;                // projection information
  ON::display_mode m_display_mode; // wireframe, shaded, or render preview
  ON_3dPoint  m_target;            // target point

  // position of view in main frame
  ON_3dmViewPosition m_position;

  // construction plane
  ON_3dmConstructionPlane m_cplane;
  BOOL m_bShowConstructionGrid;
  BOOL m_bShowConstructionAxes;

  // world axes icon
  BOOL m_bShowWorldAxes;

  // tracing image
  ON_3dmViewTraceImage m_trace_image;

  // wallpaper image
  ON_wString m_wallpaper_bitmap_filename;
};

#if defined(ON_DLL_EXPORTS) || defined(ON_DLL_IMPORTS)
// This stuff is here because of a limitation in the way Microsoft
// handles templates and DLLs.  See Microsoft's knowledge base 
// article ID Q168958 for details.
#pragma warning( push )
#pragma warning( disable : 4231 )
ON_TEMPLATE template class ON_CLASS ON_ClassArray<ON_3dmView>;
#pragma warning( pop )
#endif

//////////////////////////////////////////////////////////////////////////////////////////
//
// ON_3dmRenderSettings
//

class ON_CLASS ON_3dmRenderSettings
{
public:
  ON_3dmRenderSettings();
  ~ON_3dmRenderSettings();
  ON_3dmRenderSettings(const ON_3dmRenderSettings&);
  ON_3dmRenderSettings& operator=(const ON_3dmRenderSettings&);

  void Default();

  BOOL Write( ON_BinaryArchive& ) const;
  BOOL Read( ON_BinaryArchive& );

  //////////
  // FALSE: image pixel size = current viewport size
  // TRUE:  image pixel size = m_image_width X m_image_height pixels
  BOOL m_bCustomImageSize;
  int  m_image_width;   // image width in pixels
  int  m_image_height;  // image height in pixels
  ////////
  // Number of dots/inch (dots=pixels) to use when printing and 
  // saving bitmaps. The default is 72.0 dots/inch.
  double m_image_dpi; 
  //////////
  // unit system to use when converting image pixel size and dpi
  // information into a print size.  Default = inches
  ON::unit_system m_image_us;

  ON_Color m_ambient_light;
  
  int m_background_style; // 0 = solid color, 1 = "wallpaper" image
  ON_Color m_background_color;
  ON_wString m_background_bitmap_filename;

  BOOL m_bUseHiddenLights;

  BOOL m_bDepthCue;
  BOOL m_bFlatShade;

  BOOL m_bRenderBackfaces;
  BOOL m_bRenderPoints;
  BOOL m_bRenderCurves;
  BOOL m_bRenderIsoparams;
  BOOL m_bRenderMeshEdges;
  BOOL m_bRenderAnnotation;

  int m_antialias_style; // 0 = none, 1 = normal, 2 = best

  int m_shadowmap_style;    // 0 = none, 1 = normal, 2 = best
  int m_shadowmap_width;
  int m_shadowmap_height;
  double m_shadowmap_offset;
};

//////////////////////////////////////////////////////////////////////////////////////////
//
// ON_3dmSettings
//

class ON_CLASS ON_3dmSettings
{
public:
  ON_3dmSettings();
  ~ON_3dmSettings();
  ON_3dmSettings(const ON_3dmSettings&);
  ON_3dmSettings& operator=(const ON_3dmSettings&);

  void Default();

  BOOL Read(ON_BinaryArchive&);
  BOOL Write(ON_BinaryArchive&) const;

  void Dump( ON_TextLog& ) const;

  //////////
  // model URL (can be empty)
  ON_wString m_model_URL;

  //////////
  // model tolerances and unit system
  ON_3dmUnitsAndTolerances m_UnitsAndTolerances;

  // settings used for automatically created rendering meshes
  ON_MeshParameters        m_RenderMeshSettings;

  // settings used for automatically created analysis meshes
  ON_MeshParameters        m_AnalysisMeshSettings;

  // settings used when annotation objects are created
  ON_3dmAnnotationSettings m_AnnotationSettings;

  ON_ClassArray<ON_3dmConstructionPlane> m_named_cplanes;
  ON_ClassArray<ON_3dmView>              m_named_views;
  ON_ClassArray<ON_3dmView>              m_views; // current viewports

  // These fields determine what layer, material, color, line style, and
  // wire density are used for new objects.
  int m_current_layer_index;

  int m_current_material_index;
  ON::object_material_source m_current_material_source;
  
  ON_Color m_current_color;
  ON::object_color_source m_current_color_source;

  ON_LineStyle  m_current_line_style;     // line style if ON::line_style_from_object == LineStyleSource()
  ON::object_line_style_source m_current_line_style_source;
 
  int m_current_wire_density;   // surface wireframe density
        // 0    = boundary + "knot" wires 
        // 1    = boundary + "knot" wires + 1 interior wire if no interior "knots"
        // N>=2 = boundry  + "knot" wires + (N-1) interior wires

  ON_3dmRenderSettings m_RenderSettings;

  // default settings for construction plane grids
  ON_3dmConstructionPlaneGridDefaults m_GridDefaults;
private:
  BOOL Read_v1(ON_BinaryArchive&);
  BOOL Read_v2(ON_BinaryArchive&);
  BOOL Write_v1(ON_BinaryArchive&) const;
  BOOL Write_v2(ON_BinaryArchive&) const;
};

#endif
