/* $Header: /src2/opennurbs/opennurbs_arc.h 13    6/19/01 11:44a Dalelear $ */
/* $NoKeywords: $ */
/*
//
// Copyright (c) 1993-2001 Robert McNeel & Associates. All rights reserved.
// Rhinoceros is a registered trademark of Robert McNeel & Assoicates.
//
// THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
// ALL IMPLIED WARRANTIES OF FITNESS FOR ANY PARTICULAR PURPOSE AND OF
// MERCHANTABILITY ARE HEREBY DISCLAIMED.
//				
// For complete openNURBS copyright information see <http://www.opennurbs.org>.
//
////////////////////////////////////////////////////////////////
*/

#if !defined(ON_ARC_INC_)
#define ON_ARC_INC_

class ON_CLASS ON_Arc : public ON_Circle
{
public:
  ON_Arc();

  ON_Arc(
    const ON_Circle&,
    double           // angle in radians
    );

  ON_Arc(
    const ON_Plane&, // circle is in this plane with center at m_origin
    double,          // radius
    double           // angle in radians
    );

  ON_Arc( // arc is parallel to XY plane
    const ON_3dPoint&, // center
    double,            // radius
    double             // angle in radians
    );

  ON_Arc( // arc parallel to a plane
    const ON_Plane&,   // circle will be parallel to this plane
    const ON_3dPoint&, // center
    double,            // radius
    double             // angle in radians
    );

  ON_Arc( // arc through 3 2d points
    const ON_2dPoint&, // point P
    const ON_2dPoint&, // point Q
    const ON_2dPoint&  // point R
    );

  ON_Arc( // arc through 3 3d points
    const ON_3dPoint&, // point P
    const ON_3dPoint&, // point Q
    const ON_3dPoint&  // point R
    );

  BOOL Create(
    const ON_Circle&,
    double           // angle in radians
    );

  BOOL Create(
    const ON_Plane&, // circle is in this plane with center at m_origin
    double,          // radius
    double           // angle in radians
    );

  BOOL Create( // arc is parallel to XY plane
    const ON_3dPoint&, // center
    double,            // radius
    double             // angle in radians
    );

  BOOL Create( // arc parallel to a plane
    const ON_Plane&,   // circle will be parallel to this plane
    const ON_3dPoint&, // center
    double,            // radius
    double             // angle in radians
    );

  BOOL Create( // arc through 3 2d points
    const ON_2dPoint&, // point P
    const ON_2dPoint&, // point Q
    const ON_2dPoint&  // point R
    );

  BOOL Create( // arc through 3 3d points
    const ON_3dPoint&, // point P
    const ON_3dPoint&, // point Q
    const ON_3dPoint&  // point R
    );

  ON_Arc& operator=( const ON_Circle& );


  ~ON_Arc();

  void Dump(ON_TextLog&) const;

  BOOL IsValid() const;

  ON_BoundingBox BoundingBox() const;

  BOOL UpdatePoints();  // sets m_point[] to have valid points on the arc

  BOOL IsCircle() const; // TRUE if arc is a complete circle

  double AngleRadians() const;
  double AngleDegrees() const;

  ON_Interval ON_Arc::DomainRadians() const;
  ON_Interval ON_Arc::DomainDegrees() const;

  BOOL SetAngleRadians(double); // 0 < angle <= 2*pi
  BOOL SetAngleDegrees(double); // 0 < angle <= 360

  ON_3dPoint StartPoint() const;
  ON_3dPoint MidPoint() const;
  ON_3dPoint EndPoint() const;

  // returns parameters of point on arc that is closest to given point
  BOOL ClosestPointTo( 
         const ON_3dPoint&, 
         double*
         ) const;
  // returns point on circle that is arc to given point
  ON_3dPoint ClosestPointTo( 
         const ON_3dPoint& 
         ) const;

  double Length() const;

  BOOL Reverse();

  // parameterization of NURBS curve does not match arc's transcendtal paramaterization
  int GetNurbForm( ON_NurbsCurve& ) const; // returns 0=failure, 2=success


private:
  friend BOOL ON_BinaryArchive::ReadArc( ON_Arc& );
  friend BOOL ON_BinaryArchive::WriteArc( const ON_Arc& );
  // circle is in the plane with center at m_plane.m_origin.
  ON_Interval m_angle; // increasing interval with start and end angle in radians;
};

#endif

