/* $Header: /src2/opennurbs/opennurbs_archive.h 31    2/13/01 11:37a Dalelear $ */
/* $NoKeywords: $ */
/*
//
// Copyright (c) 1993-2001 Robert McNeel & Associates. All rights reserved.
// Rhinoceros is a registered trademark of Robert McNeel & Assoicates.
//
// THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
// ALL IMPLIED WARRANTIES OF FITNESS FOR ANY PARTICULAR PURPOSE AND OF
// MERCHANTABILITY ARE HEREBY DISCLAIMED.
//				
// For complete openNURBS copyright information see <http://www.opennurbs.org>.
//
////////////////////////////////////////////////////////////////
*/

#if !defined(ON_ARCHIVE_INC_)
#define ON_ARCHIVE_INC_

#include "opennurbs_zlib.h" // for z_ztream

/////////////////////////////////////////////////////////////////////
//
// ON_BinaryArchive 
//      virtual class for CPU independent serialization
//
// ON_BinaryFile
//      simple class for CPU independent binary file I/O
//      includes optional CRC support
//

class ON_Object;
class ON_Group;
class ON_Arc;
class ON_ObjectAttributes;

struct ON_3DM_CHUNK
{
  int          m_offset;    // In read or write_using_fseek mode, this is the
                            // file position of first byte after chunk's length.
                            // In write_using_buffer mode, this of the m_buffer[]
                            // position of first byte after chunk's length.
  unsigned int m_typecode;
  int m_value;
  int m_do_length; // TRUE if chunk is a long chunk with length
  unsigned short m_do_crc16; // 16 bit CRC using CCITT polynomial
  unsigned short m_crc16;
  unsigned long  m_do_crc32; // 32 bit CRC
  unsigned long  m_crc32;
};

#if defined(ON_DLL_EXPORTS) || defined(ON_DLL_IMPORTS)
// This stuff is here because of a limitation in the way Microsoft
// handles templates and DLLs.  See Microsoft's knowledge base 
// article ID Q168958 for details.
#pragma warning( push )
#pragma warning( disable : 4231 )
ON_TEMPLATE template class ON_CLASS ON_SimpleArray<ON_3DM_CHUNK>;
#pragma warning( pop )
#endif

class ON_Light;
class ON_Bitmap;
class ON_Material;
class ON_Layer;
class ON_3dmProperties;
class ON_3dmSettings;
class ON_3dmObjectAttributes;
class ON_3dmGoo;

class ON_BinaryArchive;

// Used int ON_3dmProperties::Read() to set ON_BinaryArchive.m_3dm_opennurbs_version
// Do not call directly. 
void ON_SetBinaryArchiveOpenNURBSVersion(ON_BinaryArchive&,int);

class ON_CLASS ON_BinaryArchive // use for generic serialization of binary data
{
public:
  ON_BinaryArchive( ON::archive_mode );
  virtual ~ON_BinaryArchive();

  virtual int  CurrentPosition( // current offset (in bytes) into archive ( like ftell() )
                ) const = 0; 
  virtual BOOL SeekFromCurrentPosition( // seek from current position ( like fseek( ,SEEK_CUR) )
                int // byte offset ( >= -CurrentPostion() )
                ) = 0; 
  virtual BOOL SeekFromStart(  // seek from current position ( like fseek( ,SEEK_SET) )
                int // byte offset ( >= 0 )
                ) = 0;
  virtual BOOL AtEnd() const = 0; // TRUE if at end of file

  static
  BOOL ToggleByteOrder( // tool for swapping bytes when doing I/O on
                        // big endian CPUs.  See Endian().
    int, // number of elements
    int, // size of element (2,4, or 8)
    const void*,  // source buffer
    void*         // destination buffer (can be same a source buffer)
    );

  BOOL ReadMode() const;  // TRUE if reading is permitted
  BOOL WriteMode() const; // TRUE if writing is permitted
  ON::endian Endian() const; // endian-ness of cpu
  int BadCRCCount() const; // number of chunks read with bad CRC 

  BOOL ReadByte( size_t, void* ); // must fail if mode is not read or readwrite

  BOOL WriteByte( size_t, const void* ); // must fail if mode is not write or readwrite

  // ReadCompressedBuffer()/WriteCompressedBuffer() use zlib 1.3.3
  // to inflate/deflate the data buffer.
  // Care must be used to get an endian independent file.  
  // See ON_Mesh::Read()/ON_Mesh::Write() for an example of an endian
  // independent use of compression. See also ToggleByteOrder() and Endian().
  //
  // To read data archived by WriteCompressedBuffer( sizeof_buffer, buffer )
  // do something like:
  //
  //   int sizeof_buffer = 0;
  //   ReadCompressedBufferSize(&sizeof_buffer);
  //   buffer = something with sizeof_buffer bytes.
  //   BOOL bFailedCRC = FALSE;
  //   ReadCompressedBuffer( sizeof_buffer, buffer, &bFailedCRC );
  //
  BOOL ReadCompressedBufferSize( int* ); // gets uncompressed size in bytes
  BOOL ReadCompressedBuffer( // read compressed buffer and uncompress
    int,   // number of uncomressed bytes
    void*, // uncompressed data
    BOOL*  // bFailedCRC - set to TRUE if cyclic redundancy check fails
    );
  BOOL WriteCompressedBuffer( // compress block and write to archive
    int,        // number of uncomressed bytes
    const void* // uncompressed data
    );

	BOOL ReadChar(    // Read an array of 8 bit chars
			size_t,       // number of chars to read
			char*    
			);  
	BOOL ReadChar(    // Read an array of 8 bit unsigned chars
			size_t,       // number of unsigned chars to read
			unsigned char*    
			);  
	BOOL ReadChar(    // Read a single 8 bit char
			char*    
			);  
	BOOL ReadChar(    // Read a single 8 bit unsigned char
			unsigned char*    
			);  

	BOOL ReadShort(   // Read an array of 16 bit shorts
			size_t,       // number of shorts to read
			short*    
			);  
	BOOL ReadShort(   // Read an array of 16 bit unsigned shorts
			size_t,       // number of shorts to read
			unsigned short*    
			);  
	BOOL ReadShort(   // Read a single 16 bit short
			short*    
			);  
	BOOL ReadShort(   // Read a single 16 bit unsigned short
			unsigned short*    
			);  

	BOOL ReadInt( // Read an array of 32 bit integers
			size_t,	      // number of ints to read
			int*      
			); 
	BOOL ReadInt( // Read an array of 32 bit integers
			size_t,	      // number of ints to read
			unsigned int*      
			); 
	BOOL ReadInt( // Read a single 32 bit integer
			int*      
			); 
	BOOL ReadInt( // Read a single 32 bit unsigned integer
			unsigned int*      
			); 

	BOOL ReadLong( // Read an array of 32 bit integers
			size_t,	      // number of ints to read
			long*      
			); 
	BOOL ReadLong( // Read an array of 32 bit integers
			size_t,	      // number of ints to read
			unsigned long*      
			); 
	BOOL ReadLong( // Read a single 32 bit integer
			long*      
			); 
	BOOL ReadLong( // Read a single 32 bit unsigned integer
			unsigned long*      
			); 

	BOOL ReadFloat(   // Read an array of floats
			size_t,       // number of floats
			float*
			);
	BOOL ReadFloat(   // Read a single float
			float*
			);
	BOOL ReadDouble(  // Read an array of IEEE doubles
			size_t,       // number of doubles
			double*
			);
	BOOL ReadDouble(  // Read a single double
			double*
			);

  BOOL ReadColor(
    ON_Color&
    );

  BOOL ReadLineStyle(
    ON_LineStyle&
    );

  BOOL ReadPoint (
    ON_2dPoint&
    );
  BOOL ReadPoint (
    ON_3dPoint&
    );
  BOOL ReadPoint (
    ON_4dPoint&
    );
  BOOL ReadVector (
    ON_2dVector&
    );
  BOOL ReadVector (
    ON_3dVector&
    );

  BOOL ReadBoundingBox(ON_BoundingBox&);

  BOOL ReadXform(ON_Xform&);

  BOOL ReadPlane(ON_Plane&);

  BOOL ReadLine(ON_Line&);

  BOOL ReadArc(ON_Arc&);

  BOOL ReadCircle(ON_Circle&);

  BOOL ReadInterval( ON_Interval& );

  BOOL ReadUuid( ON_UUID& );

  // All times are stored in coordinated universal time
  // ( a.k.a GMT, UTC ).  Use ANSI C time() and gmtime() calls.
  BOOL ReadTime( struct tm& );


  BOOL ReadStringSize( // Read size of NULL terminated string
      size_t*          // (returned size includes NULL terminator)
      );
  BOOL ReadString(     // Read NULL terminated string
      size_t,          // length = value from ReadStringSize()
      char*            // array[length]
      );
  BOOL ReadString(     // Read NULL terminated string
      size_t,          // length = value from ReadStringSize()
      unsigned char*   // array[length]
      );
  BOOL ReadString(    // Read NULL terminated wide (unicode) string
      size_t,          // length = value from ReadStringSize()
      unsigned short*  // array[length]
      );
  BOOL ReadString( ON_String& );
  BOOL ReadString( ON_wString& );

  BOOL ReadArray( ON_SimpleArray<char>& );
  BOOL ReadArray( ON_SimpleArray<short>& );
  BOOL ReadArray( ON_SimpleArray<int>& );
  BOOL ReadArray( ON_SimpleArray<float>& );
  BOOL ReadArray( ON_SimpleArray<double>& );

  BOOL ReadArray( ON_SimpleArray<ON_Color>& );

  BOOL ReadArray( ON_SimpleArray<ON_2dPoint>& );
  BOOL ReadArray( ON_SimpleArray<ON_3dPoint>& );
  BOOL ReadArray( ON_SimpleArray<ON_4dPoint>& );
  BOOL ReadArray( ON_SimpleArray<ON_2dVector>& );
  BOOL ReadArray( ON_SimpleArray<ON_3dVector>& );

  BOOL ReadArray( ON_SimpleArray<ON_2fPoint>& );
  BOOL ReadArray( ON_SimpleArray<ON_3fPoint>& );
  BOOL ReadArray( ON_SimpleArray<ON_4fPoint>& );
  BOOL ReadArray( ON_SimpleArray<ON_2fVector>& );
  BOOL ReadArray( ON_SimpleArray<ON_3fVector>& );

  BOOL ReadArray( ON_SimpleArray<ON_SurfaceCurvature>& );

  BOOL WriteChar(    // Write an array of 8 bit chars
			size_t,       // number of chars to write
			const char*    
			);  
	BOOL WriteChar(    // Write an array of 8 bit unsigned chars
			size_t,       // number of unsigned chars to write
			const unsigned char*    
			);  
	BOOL WriteChar(    // Write a single 8 bit char
			char
			);  
	BOOL WriteChar(    // Write a single 8 bit unsigned char
			unsigned char
			);  

	BOOL WriteShort(   // Write an array of 16 bit shorts
			size_t,       // number of shorts to write
			const short*    
			);  
	BOOL WriteShort(   // Write an array of 16 bit unsigned shorts
			size_t,       // number of shorts to write
			const unsigned short*    
			);  
	BOOL WriteShort(   // Write a single 16 bit short
			short
			);  
	BOOL WriteShort(   // Write a single 16 bit unsigned short
			unsigned short
			);  

	BOOL WriteInt( // Write an array of 32 bit integers
			size_t,	      // number of ints to write
			const int*      
			); 
	BOOL WriteInt( // Write an array of 32 bit integers
			size_t,	      // number of ints to write
			const unsigned int*      
			); 
	BOOL WriteInt( // Write a single 32 bit integer
			int    
			); 
	BOOL WriteInt( // Write a single 32 bit unsigned integer
			unsigned int
			); 

	BOOL WriteLong( // Write an array of 32 bit integers
			size_t,	      // number of ints to write
			const long*      
			); 
	BOOL WriteLong( // Write an array of 32 bit integers
			size_t,	      // number of ints to write
			const unsigned long*      
			); 
	BOOL WriteLong( // Write a single 32 bit integer
			long    
			); 
	BOOL WriteLong( // Write a single 32 bit unsigned integer
			unsigned long
			); 

	BOOL WriteFloat(   // Write a number of IEEE floats
			size_t,       // number of doubles
			const float*
			);
	BOOL WriteFloat(   // Write a single float
			float
			);
	BOOL WriteDouble(  // Write a single double
      size_t,
			const double*
			);
	BOOL WriteDouble(  // Write a single double
			double
			);

  BOOL WriteColor (
    const ON_Color&
    );

  BOOL WriteLineStyle (
    const ON_LineStyle&
    );

  BOOL WritePoint (
    const ON_2dPoint&
    );
  BOOL WritePoint (
    const ON_3dPoint&
    );
  BOOL WritePoint (
    const ON_4dPoint&
    );
  BOOL WriteVector (
    const ON_2dVector&
    );
  BOOL WriteVector (
    const ON_3dVector&
    );

  BOOL WriteBoundingBox(const ON_BoundingBox&);

  BOOL WriteXform(const ON_Xform&);

  BOOL WritePlane(const ON_Plane&);

  BOOL WriteLine(const ON_Line&);

  BOOL WriteArc(const ON_Arc&);

  BOOL WriteCircle(const ON_Circle&);

  BOOL WriteInterval( const ON_Interval& );

  BOOL WriteUuid( const ON_UUID& );

  // All times are stored in coordinated universal time
  // ( a.k.a GMT, UTC ).  Use ANSI C time() and gmtime() calls.
  BOOL WriteTime( const struct tm& );

  // To read strings written with WriteString(), first call
  // ReadStringLength() to get the length of the string (including
  // the NULL terminator, then call ReadString() with enought memory
  // to load the string.
  BOOL WriteString( // Write NULL terminated string
      const char*         
      );
  BOOL WriteString( // Write NULL terminated string
      const unsigned char*
      );
  BOOL WriteString(  // Write NULL terminated wide (unicode) string
      const unsigned short* // array[length]
      );
  BOOL WriteString( const ON_String& );
  BOOL WriteString( const ON_wString& );

  BOOL WriteArray( const ON_SimpleArray<char>& );
  BOOL WriteArray( const ON_SimpleArray<short>& );
  BOOL WriteArray( const ON_SimpleArray<int>& );
  BOOL WriteArray( const ON_SimpleArray<float>& );
  BOOL WriteArray( const ON_SimpleArray<double>& );

  BOOL WriteArray( const ON_SimpleArray<ON_Color>& );

  BOOL WriteArray( const ON_SimpleArray<ON_2dPoint>& );
  BOOL WriteArray( const ON_SimpleArray<ON_3dPoint>& );
  BOOL WriteArray( const ON_SimpleArray<ON_4dPoint>& );
  BOOL WriteArray( const ON_SimpleArray<ON_2dVector>& );
  BOOL WriteArray( const ON_SimpleArray<ON_3dVector>& );

  BOOL WriteArray( const ON_SimpleArray<ON_2fPoint>& );
  BOOL WriteArray( const ON_SimpleArray<ON_3fPoint>& );
  BOOL WriteArray( const ON_SimpleArray<ON_4fPoint>& );
  BOOL WriteArray( const ON_SimpleArray<ON_2fVector>& );
  BOOL WriteArray( const ON_SimpleArray<ON_3fVector>& );

  BOOL WriteArray( const ON_SimpleArray<ON_SurfaceCurvature>& );
  /////////////////////////////////////////////////////
  //
  // Read/Write classes derived from ON_Object
  //
  BOOL ReadObject( 
         ON_Object** // object created in ReadObject() and returned here
         );
  BOOL WriteObject( const ON_Object* ); // writes object definition
  BOOL WriteObject( const ON_Object& ); // writes object definition


  ///////////////////////////////////////////////////////////////////
  ///////////////////////////////////////////////////////////////////
  //
  // 3DM Interface - ignore if not reading/writing a 3DM file
  //                 this is here so that the infrastructure
  //                 for writing 3dm archives is available for
  //                 any type of serialization device.
  //
  BOOL EnableSave3dmRenderMeshes( BOOL = TRUE ); // returns previous state
  BOOL Save3dmRenderMeshes() const;

  BOOL EnableSave3dmAnalysisMeshes( BOOL = TRUE ); // returns previous state
  BOOL Save3dmAnalysisMeshes() const;
  
  ///////////////////////////////////////////////////////////////////
  // Step 1: REQUIRED - Write/Read Start Section
  //
  BOOL Write3dmStartSection( 
        int,        // version = 1 or 2
        const char* // NULL or ASCII string with application name, etc.
                    // This information is primarily used when debugging files
                    // that contain problems.  McNeel and Associates stores
                    // application name, application version, compile date, 
                    // OS in use when file was written.
        );
  BOOL Read3dmStartSection( 
        int*, // returns version (1 or 2)
        ON_String& 
        );

  ///////////////////////////////////////////////////////////////////
  // Step 2: REQUIRED - Write/Read properties table
  //
  BOOL Write3dmProperties(
        const ON_3dmProperties&
        );
  BOOL Read3dmProperties(
        ON_3dmProperties&
        );

  ///////////////////////////////////////////////////////////////////
  // Step 3: REQUIRED - Write/Read settings table
  //
  BOOL Write3dmSettings(
        const ON_3dmSettings&
        );
  BOOL Read3dmSettings(
        ON_3dmSettings&
        );

  ///////////////////////////////////////////////////////////////////
  // Step 4: REQUIRED - Write/Read bitmap table (it can be empty)
  //
  BOOL BeginWrite3dmBitmapTable();
  BOOL Write3dmBitmap( const ON_Bitmap& );
  BOOL EndWrite3dmBitmapTable();

  BOOL BeginRead3dmBitmapTable();
  int  Read3dmBitmap(   // returns 0 at end of light table
                        //         1 bitmap successfully read
            ON_Bitmap** // bitmap returned here
            );
  BOOL EndRead3dmBitmapTable();

  ///////////////////////////////////////////////////////////////////
  // Step 5: REQUIRED - Write/Read render material table (it can be empty)
  //
  BOOL BeginWrite3dmMaterialTable();
  BOOL Write3dmMaterial( const ON_Material& );
  BOOL EndWrite3dmMaterialTable();

  BOOL BeginRead3dmMaterialTable();
  int  Read3dmMaterial( // returns 0 at end of table
            ON_Material** // layer returned here
            );
  BOOL EndRead3dmMaterialTable();

  ///////////////////////////////////////////////////////////////////
  // Step 6: REQUIRED - Write/Read layer table (it can be empty)
  //
  BOOL BeginWrite3dmLayerTable();
  BOOL Write3dmLayer( const ON_Layer& );
  BOOL EndWrite3dmLayerTable();

  BOOL BeginRead3dmLayerTable();
  int  Read3dmLayer( // returns 0 at end of table
            ON_Layer** // layer returned here
            );
  BOOL EndRead3dmLayerTable();

  ///////////////////////////////////////////////////////////////////
  // Step 7: REQUIRED - Write/Read group table (it can be empty)
  //
  BOOL BeginWrite3dmGroupTable();
  BOOL Write3dmGroup( const ON_Group& );
  BOOL EndWrite3dmGroupTable();

  BOOL BeginRead3dmGroupTable();

  //////////
  // returns 0 at the end of the group table
  //         1 if the group definition was successfully read
  //           In this case a group is created by calling new ON_Group(),
  //           initialized with values stored in the archive, and returned.
  //        -1 if the archive is corrupt at this point
  // 
  int  Read3dmGroup( // returns 0 at end of table
            ON_Group** // group returned here
            );
  BOOL EndRead3dmGroupTable();

  ///////////////////////////////////////////////////////////////////
  // Step 8: REQUIRED - Write/Read render light table (it can be empty)
  //
  BOOL BeginWrite3dmLightTable();
  BOOL Write3dmLight( 
         const ON_Light&,
         const ON_3dmObjectAttributes* // optional
         );
  BOOL EndWrite3dmLightTable();

  BOOL BeginRead3dmLightTable();
  int  Read3dmLight(  // returns 0 at end of light table
                      //         1 light successfully read
                      //        -1 if file is corrupt
            ON_Light**, // light returned here
            ON_3dmObjectAttributes* // optional - if NOT NULL, object attributes are
                                    //            returned here
            );
  BOOL EndRead3dmLightTable();

  ///////////////////////////////////////////////////////////////////
  // Step 9: REQUIRED - Write/Read geometry and annotation table (it can be empty)
  //
  BOOL BeginWrite3dmObjectTable();
  BOOL Write3dmObject( 
         const ON_Object&,
         const ON_3dmObjectAttributes* // optional
         );
  BOOL EndWrite3dmObjectTable();

  BOOL BeginRead3dmObjectTable();
  int  Read3dmObject( // returns 0 at end of object table
                      //         1 if object is read
                      //         2 if object is skipped because it does not match filter
                      //        -1 if file is corrupt
          ON_Object**, // object returned here (NULL if skipped)
          ON_3dmObjectAttributes*, // optional - if NOT NULL, object attributes are
                                   //            returned here
          unsigned int = 0 // optional filter made by setting ON::object_type bits
          );  // returns NULL at end of object table
  BOOL EndRead3dmObjectTable();

  ///////////////////////////////////////////////////////////////////
  // Step 10: OPTIONAL - Write/Read 0 or more user tables
  //
  BOOL BeginWrite3dmUserTable(
    const ON_UUID& // user table id, (e.g., a plugin id) - cannot be nil
    );
  // Use Write3dmAnonymousUserTable() to put back what Read3dmAnonymousUserTable() read.
  // (Generally a plugin or some other app will be called to write
  //  it's information to the archive.)
  BOOL Write3dmAnonymousUserTable( const ON_3dmGoo& );
  BOOL EndWrite3dmUserTable();

  // BeginRead3dmUserTable returns FALSE when there are no more user tables
  BOOL BeginRead3dmUserTable(
    ON_UUID& // user table id returned here 
             //  - simply call EndRead3dmUserTable() to skip this information
    );
  // Use Read3dmAnonymousUserTableRecord() if you don't know how to decipher 
  // the contents of a user table but you need to rewrite it for some reason.
  // If you simple want to skip over the table, just call EndRead3dmUserTable().
  BOOL Read3dmAnonymousUserTable( ON_3dmGoo& );
  BOOL EndRead3dmUserTable();

  ///////////////////////////////////////////////////////////////////
  // Step 11: REQUIRED when writing / OPTIONAL when reading
  //         Write end of file marker.  This information is primarily
  //         used when debugging files to make sure the end of the file
  //         hasn't been cut off.
  //
  BOOL Write3dmEndMark();
  BOOL Read3dmEndMark( 
           int* // length of file is returned here
           );

  ///////////////////////////////////////////////////////////////////
  ///////////////////////////////////////////////////////////////////
  // Low level tools to  Write/Read chunks. See opennurbs_3dm.h for details
  //         about the structure of chunks.  Every chunk must begin with a
  //         call to BeginWrite/ReadChunk().
  //         If BeginWriteChunk()/BeginReadChunk() returns TRUE, then
  //         you must call EndWrite/ReadChunk() or cease using the archive.
  BOOL BeginWrite3dmChunk(
        unsigned int,   // typecode from opennurbs_3dm.h
        int             // 0 or value
        );
  BOOL EndWrite3dmChunk(); // updates length

  BOOL Write3dmGoo( const ON_3dmGoo& ); // call to write "goo"

  // When the end of the 3dm file is reached, BeginReadChunk() will
  // return TRUE with a typecode of TCODE_ENDOFFILE.
  BOOL BeginRead3dmChunk(
        unsigned int*,   // typecode from opennurbs_3dm.h
        int*             // value
        );
  BOOL EndRead3dmChunk(); // Calling this will skip rest of stuff in chunk if it was only partially read.
  BOOL Read3dmGoo( ON_3dmGoo& ); // Call to read "goo"

  BOOL PeekAt3dmChunkType( // does not change file position
        unsigned int*,   // typecode from opennurbs_3dm.h
        int*             // value
        );
  BOOL Seek3dmChunkFromStart( 
        // beginning at the start of the active chunk, search portion of
        // archive included in active chunk for the start of a subchunk 
        // with the specified type.
        // if TRUE is returned, then the position is set so the next call to
        // BeginRead3dmChunk() will read a chunk with the specified typecode
        unsigned int    // typecode from opennurbs_3dm.h
        );
  BOOL Seek3dmChunkFromCurrentPosition( 
        // beginning at the current position, search portion of archive
        // included in active chunk for the start of a subchunk with the
        // specified type.
        // if TRUE is returned, then the position is set so the next call to
        // BeginRead3dmChunk() will read a chunk with the specified typecode
        unsigned int    // typecode from opennurbs_3dm.h
        );

  // A chunk version is a single byte that encodes a major.minor 
  // version number.  Useful when creating I/O code for 3dm chunks
  // that may change in the future.  Increment the minor version 
  // number if new information is added to the end of the chunk. 
  // Increment the major version if the format of the chunk changes
  // in some other way.
  BOOL Write3dmChunkVersion(
    int, // major // 0 to 15
    int  // minor // 0 to 16
    );
  BOOL Read3dmChunkVersion(
    int*, // major // 0 to 15
    int*  // minor // 0 to 16
    );

  int Archive3dmVersion() const; // returns 0 if a 3dm archive is not being read/written
                          // returns 1 if a version 1 3dm archive is being read/written
                          // returns 2 if a version 2 3dm archive is being read/written

  enum table_type
  {
    no_active_table = 0,
    properties_table = 1,
    settings_table = 2,
    bitmap_table = 3,
    material_table = 4,
    layer_table = 5,
    light_table = 6,
    object_table = 7,
    group_table = 8,
    user_table = 9
  };

protected:
  virtual
  int Read( size_t, void* ) = 0; // return actual number of bytes read (like fread())
  virtual
  int Write( size_t, const void* ) = 0; // return actual number of bytes written (like fwrite())
  virtual
  BOOL Flush() = 0; // Force Write() to flush any buffered data to physical archive

  BOOL SetArchive3dmVersion(int);

private:
  BOOL BeginWrite3dmTable( 
    unsigned int // tcode
    );
  BOOL EndWrite3dmTable( 
    unsigned int // tcode
    );
  BOOL BeginRead3dmTable( 
    unsigned int // tcode
    );
  BOOL EndRead3dmTable( 
    unsigned int // tcode
    );
  
  BOOL Read3dmV1Layer( ON_Layer*& );
  int  Read3dmV1Light(  // returns 0 at end of light table
                      //         1 light successfully read
                      //        -1 if file is corrupt
            ON_Light**, // light returned here
            ON_3dmObjectAttributes* // optional - if NOT NULL, object attributes are
                                    //            returned here
            );
  int Read3dmV1Material( ON_Material** );
  int  Read3dmV1Object( // returns 0 at end of object table
                      //         1 if object is read
                      //         2 if object is skipped because it does not match filter
                      //        -1 if file is corrupt
          ON_Object**, // object returned here (NULL if skipped)
          ON_3dmObjectAttributes*, // optional - if NOT NULL, object attributes are
                                   //            returned here
          unsigned int = 0 // optional filter made by setting ON::object_type bits
          );  // returns NULL at end of object table

  BOOL Read3dmV1AttributesOrMaterial( 
            ON_3dmObjectAttributes*,    // attributes,
            ON_Material*,      // material,
            BOOL&,             // bHaveMat
            unsigned int       // end_mark_tcode 
            );
  BOOL Read3dmV1String( ON_String& );

  BOOL ON_BinaryArchive::ReadV1_TCODE_RH_POINT(ON_Object**,ON_3dmObjectAttributes*);
  BOOL ON_BinaryArchive::ReadV1_TCODE_MESH_OBJECT(ON_Object**,ON_3dmObjectAttributes*);
  BOOL ON_BinaryArchive::ReadV1_TCODE_AG_CURVE(ON_Object**,ON_3dmObjectAttributes*);
  BOOL ON_BinaryArchive::ReadV1_TCODE_AG_FACE(ON_Object**,ON_3dmObjectAttributes*);
  BOOL ON_BinaryArchive::ReadV1_TCODE_AG_SHELL(ON_Object**,ON_3dmObjectAttributes*);
  BOOL ON_BinaryArchive::ReadV1_TCODE_RHINOIO_OBJECT_NURBS_CURVE(ON_Object**,ON_3dmObjectAttributes*);
  BOOL ON_BinaryArchive::ReadV1_TCODE_RHINOIO_OBJECT_NURBS_SURFACE(ON_Object**,ON_3dmObjectAttributes*);
  BOOL ON_BinaryArchive::ReadV1_TCODE_RHINOIO_OBJECT_BREP(ON_Object**,ON_3dmObjectAttributes*);

  ON::archive_mode Mode() const; // current read/write mode
  void UpdateCRC( size_t, const void* );

  int m_3dm_version;
  int m_3dm_v1_layer_index;
  int m_3dm_v1_material_index;

  // * V1 files do not have a table structure and are read using
  //   multiple passes and there are valid situations where a 
  //   read is attempted at the end of a file.  During those 
  //   situations, the following flag has bit 0x01 set.
  //
  // * Some v1 files do not have an end mark.  When reading
  //   these v1 files the following flag has bit 0x02 set.
  unsigned int m_3dm_v1_suppress_error_message;

  // When a 3DM archive is read, m_3dm_opennurbs_version records the version of
  // OpenNURBS used to create the archive.  Otherwise, m_3dm_opennurbs_version
  // is zero.
  //
  // Read3dmProperties() sets this to the version of OpenNURBS that was
  // used to write file file.  If the file was created using a version
  // of OpenNURBS before 200012210, this number will be zero.
  //
  // Write3dmProperties() stores the value returned by ON::Version() in
  // the archive's properties table.
  friend void ON_SetBinaryArchiveOpenNURBSVersion(ON_BinaryArchive&,int);
  int m_3dm_opennurbs_version;

  table_type m_active_table;

  table_type TableTypeFromTypecode( unsigned int ); // table type from tcode


  ON_SimpleArray<ON_3DM_CHUNK> m_chunk;

  // stack of chunks
  BOOL PushChunk( 
        unsigned int, // typecode
        int          // value
        );

  BOOL m_bDoChunkCRC; // TRUE if active chunk crc status should be checked
                      // and updated.
  int m_bad_CRC_count; // number of chunks that have a bad crc


private:
  // compressed buffer I/O uses zlib 1.3.3 inflate()/deflate()
  struct
  {
    ON::archive_mode mode; // ON::read = read and inflate,  ON::write = deflate and write
    int              sizeof_buffer;
    unsigned char    buffer[16384];
    z_stream         strm;
  } m_zlib;

  BOOL WriteDeflate(
        int,         // sizeof uncompressed input data
        const void*  // uncompressed input data
        );
  BOOL ReadInflate(
        int,  // sizeof uncompressed input data
        void* // buffer to hold uncompressed data
        );
  BOOL CompressionInit();
  void CompressionEnd();

private:
  ON::endian m_endian;
  ON::archive_mode m_mode;

  // 3dm write options
  BOOL m_bSavePreviewImage;    // TRUE to save 200x200 preview bitmap (increases file size)
  BOOL m_bEmbedTextureBitmaps; // TRUE to embed texture, bump, trace, and wallpaper bitmaps (increases file size)
  BOOL m_bSaveRenderMeshes;    // TRUE to save meshes used to render B-rep objects (increases file size)
  BOOL m_bSaveAnalysisMeshes;  // TRUE to save meshes used in surface analysis (increases file size)

  // prohibit default construction, copy construction, and operator=
  ON_BinaryArchive();
  ON_BinaryArchive( const ON_BinaryArchive& ); // no implementation
  ON_BinaryArchive& operator=( const ON_BinaryArchive& ); // no implementation

};

class ON_CLASS ON_3dmGoo
{
  // used to store goo
public:
  ON_3dmGoo();
  ~ON_3dmGoo();
  ON_3dmGoo( const ON_3dmGoo& );
  ON_3dmGoo& operator=( const ON_3dmGoo& );

  void Dump(ON_TextLog&) const;

  unsigned int m_typecode;
  int m_value;
  unsigned char* m_goo;
  ON_3dmGoo* m_next_goo;
  ON_3dmGoo* m_prev_goo;
};


class ON_CLASS ON_BinaryFile : public ON_BinaryArchive
{
public:
  ON_BinaryFile( ON::archive_mode );
  ON_BinaryFile( ON::archive_mode, FILE* ); // pointer from fopen(...,"rb") or fopoen(...,"wb")
  virtual ~ON_BinaryFile();

  // ON_BinaryArchive overrides
  int  CurrentPosition() const; // ftell
  BOOL AtEnd() const; // TRUE if at end of file
  BOOL SeekFromCurrentPosition( int ); // fseek from current position

  BOOL SeekFromEnd( int ); // fseek from end
  BOOL SeekFromStart( int ); // fseek from start

  //////////
  // To use custom memory buffering instead of relying
  // on fread()/fwrite()'s build in buffering, call
  // EnableMemoryBuffer() with the buffer size immediately
  // after constructing the ON_BinaryFile.  There appear
  // to be enough bugs in existing Windows NT/2000 NETWORK
  // I/O that using this hack will speed up I/O by factors
  // of 10 to 100.
  void EnableMemoryBuffer(
         int=16384 // capacity of memory buffer
         );

protected:
  int Read( size_t, void* );
  int Write( size_t, const void* );
  BOOL Flush();

private:
  // Implementation
  FILE* m_fp;

  // if m_memory_buffer_capacity is zero, then Write() uses
  // fwrite() directly.  If m_memory_buffer_capacity is
  // greater than zero, then Write() buffers its results
  // into m_memory_buffer.  This is provided to work around
  // bugs in some networks that result in extremely slow
  // performance when seeking is used.
  int m_memory_buffer_capacity;
  int m_memory_buffer_size;
  int m_memory_buffer_ptr;
  unsigned char* m_memory_buffer;

private:
  // prohibit default construction, copy construction, and operator=
  ON_BinaryFile( ); // no implementation
  ON_BinaryFile( const ON_BinaryFile& ); // no implementation
  ON_BinaryFile& operator=( const ON_BinaryFile& ); // no implementation
};

#endif

