/* $Header: /src2/opennurbs/opennurbs_array.cpp 10    1/03/01 4:59p Dalelear $ */
/* $NoKeywords: $ */
/*
//
// Copyright (c) 1993-2001 Robert McNeel & Associates. All rights reserved.
// Rhinoceros is a registered trademark of Robert McNeel & Assoicates.
//
// THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
// ALL IMPLIED WARRANTIES OF FITNESS FOR ANY PARTICULAR PURPOSE AND OF
// MERCHANTABILITY ARE HEREBY DISCLAIMED.
//				
// For complete openNURBS copyright information see <http://www.opennurbs.org>.
//
////////////////////////////////////////////////////////////////
*/

#include "opennurbs.h"   // openNURBS declarations of functions that perform simple calculations

ON_2dPointArray::ON_2dPointArray() 
{}

ON_2dPointArray::ON_2dPointArray(int c) 
                : ON_SimpleArray<ON_2dPoint>(c) 
{}

ON_2dPointArray::ON_2dPointArray(const ON_2dPointArray& src) 
                : ON_SimpleArray<ON_2dPoint>(src)
{}

BOOL ON_2dPointArray::GetBBox( // returns TRUE if successful
       double* boxmin,    // boxmin[2]
       double* boxmax,    // boxmax[2]
       BOOL bGrowBox
       ) const
{
  return ON_GetPointListBoundingBox( 2, FALSE, Count(), 2, (m_a) ? &m_a[0].x : 0, boxmin, boxmax, bGrowBox );
}

BOOL ON_2dPointArray::Transform( const ON_Xform& xform )
{
  return ON_TransformPointList( 2, FALSE, Count(), 2, (m_a) ? &m_a[0].x : 0, xform );
}

BOOL ON_2dPointArray::SwapCoordinates( int i, int j )
{
  return ON_SwapPointListCoordinates( Count(), 2, (m_a) ? &m_a[0].x : 0, i, j );
}

ON_2dPointArray& ON_2dPointArray::operator=(const ON_2dPointArray& src)
{
  if ( this != &src ) {
    ON_SimpleArray<ON_2dPoint>::operator=(src);
  }
  return *this;
}

ON_3dPointArray::ON_3dPointArray() {}
ON_3dPointArray::ON_3dPointArray(int c) : ON_SimpleArray<ON_3dPoint>(c) {}
ON_3dPointArray::ON_3dPointArray(const ON_3dPointArray& src) : ON_SimpleArray<ON_3dPoint>(src){}

BOOL ON_3dPointArray::GetBBox( // returns TRUE if successful
       double* boxmin,    // boxmin[3]
       double* boxmax,    // boxmax[3]
       BOOL bGrowBox
       ) const
{
  return ON_GetPointListBoundingBox( 3, FALSE, Count(), 3, (m_a) ? &m_a[0].x : 0, boxmin, boxmax, bGrowBox );
}

BOOL ON_3dPointArray::Transform( const ON_Xform& xform )
{
  return ON_TransformPointList( 3, FALSE, Count(), 3, (m_a) ? &m_a[0].x : 0, xform );
}

BOOL ON_3dPointArray::SwapCoordinates( int i, int j )
{
  return ON_SwapPointListCoordinates( Count(), 3, (m_a) ? &m_a[0].x : 0, i, j );
}

ON_3dPointArray& ON_3dPointArray::operator=(const ON_3dPointArray& src)
{
  if ( this != &src ) {
    ON_SimpleArray<ON_3dPoint>::operator=(src);
  }
  return *this;
}

ON_4dPointArray::ON_4dPointArray() 
{}

ON_4dPointArray::ON_4dPointArray(int c) : ON_SimpleArray<ON_4dPoint>(c) 
{}

ON_4dPointArray::ON_4dPointArray(const ON_4dPointArray& src) : ON_SimpleArray<ON_4dPoint>(src)
{}

BOOL ON_4dPointArray::Transform( const ON_Xform& xform )
{
  return ON_TransformPointList( 3, TRUE, Count(), 4, (m_a) ? &m_a[0].x : 0, xform );
}

BOOL ON_4dPointArray::SwapCoordinates( int i, int j )
{
  return ON_SwapPointListCoordinates( Count(), 4, (m_a) ? &m_a[0].x : 0, i, j );
}

ON_4dPointArray& ON_4dPointArray::operator=(const ON_4dPointArray& src)
{
  if ( this != &src ) {
    ON_SimpleArray<ON_4dPoint>::operator=(src);
  }
  return *this;
}

ON_2dVectorArray::ON_2dVectorArray() 
{}

ON_2dVectorArray::ON_2dVectorArray(int c) : ON_SimpleArray<ON_2dVector>(c) 
{}

ON_2dVectorArray::ON_2dVectorArray(const ON_2dVectorArray& src) : ON_SimpleArray<ON_2dVector>(src)
{}

BOOL ON_2dVectorArray::GetBBox( // returns TRUE if successful
       double* boxmin,    // boxmin[2]
       double* boxmax,    // boxmax[2]
       BOOL bGrowBox
       ) const
{
  return ON_GetPointListBoundingBox( 2, FALSE, Count(), 2, (m_a) ? &m_a[0].x : 0, boxmin, boxmax, bGrowBox );
}

BOOL ON_2dVectorArray::Transform( const ON_Xform& xform )
{
  return ON_TransformPointList( 2, FALSE, Count(), 2, (m_a) ? &m_a[0].x : 0, xform );
}

BOOL ON_2dVectorArray::SwapCoordinates( int i, int j )
{
  return ON_SwapPointListCoordinates( Count(), 2, (m_a) ? &m_a[0].x : 0, i, j );
}

ON_2dVectorArray& ON_2dVectorArray::operator=(const ON_2dVectorArray& src)
{
  if ( this != &src ) {
    ON_SimpleArray<ON_2dVector>::operator=(src);
  }
  return *this;
}

ON_3dVectorArray::ON_3dVectorArray() 
{}

ON_3dVectorArray::ON_3dVectorArray(int c) : ON_SimpleArray<ON_3dVector>(c) 
{}

ON_3dVectorArray::ON_3dVectorArray(const ON_3dVectorArray& src) : ON_SimpleArray<ON_3dVector>(src)
{}

BOOL ON_3dVectorArray::GetBBox( // returns TRUE if successful
       double* boxmin,    // boxmin[3]
       double* boxmax,    // boxmax[3]
       BOOL bGrowBox
       ) const
{
  return ON_GetPointListBoundingBox( 3, FALSE, Count(), 3, (m_a) ? &m_a[0].x : 0, boxmin, boxmax, bGrowBox );
}

BOOL ON_3dVectorArray::Transform( const ON_Xform& xform )
{
  return ON_TransformPointList( 3, FALSE, Count(), 3, (m_a) ? &m_a[0].x : 0, xform );
}

BOOL ON_3dVectorArray::SwapCoordinates( int i, int j )
{
  return ON_SwapPointListCoordinates( Count(), 3, (m_a) ? &m_a[0].x : 0, i, j );
}

ON_3dVectorArray& ON_3dVectorArray::operator=(const ON_3dVectorArray& src)
{
  if ( this != &src ) {
    ON_SimpleArray<ON_3dVector>::operator=(src);
  }
  return *this;
}

////
////
////
ON_2fPointArray::ON_2fPointArray() 
{}

ON_2fPointArray::ON_2fPointArray(int c) 
                : ON_SimpleArray<ON_2fPoint>(c) 
{}

ON_2fPointArray::ON_2fPointArray(const ON_2fPointArray& src) 
                : ON_SimpleArray<ON_2fPoint>(src)
{}

BOOL ON_2fPointArray::GetBBox( // returns TRUE if successful
       float* boxmin,    // boxmin[2]
       float* boxmax,    // boxmax[2]
       BOOL bGrowBox
       ) const
{
  return ON_GetPointListBoundingBox( 2, FALSE, Count(), 2, (m_a) ? &m_a[0].x : 0, boxmin, boxmax, bGrowBox );
}

BOOL ON_2fPointArray::Transform( const ON_Xform& xform )
{
  return ON_TransformPointList( 2, FALSE, Count(), 2, (m_a) ? &m_a[0].x : 0, xform );
}

BOOL ON_2fPointArray::SwapCoordinates( int i, int j )
{
  return ON_SwapPointListCoordinates( Count(), 2, (m_a) ? &m_a[0].x : 0, i, j );
}

ON_2fPointArray& ON_2fPointArray::operator=(const ON_2fPointArray& src)
{
  if ( this != &src ) {
    ON_SimpleArray<ON_2fPoint>::operator=(src);
  }
  return *this;
}


ON_3fPointArray::ON_3fPointArray() 
{}

ON_3fPointArray::ON_3fPointArray(int c) : ON_SimpleArray<ON_3fPoint>(c)
{}

ON_3fPointArray::ON_3fPointArray(const ON_3fPointArray& src) : ON_SimpleArray<ON_3fPoint>(src)
{}

BOOL ON_3fPointArray::GetBBox( // returns TRUE if successful
       float* boxmin,    // boxmin[3]
       float* boxmax,    // boxmax[3]
       BOOL bGrowBox
       ) const
{
  return ON_GetPointListBoundingBox( 3, FALSE, Count(), 3, (m_a) ? &m_a[0].x : 0, boxmin, boxmax, bGrowBox );
}

BOOL ON_3fPointArray::Transform( const ON_Xform& xform )
{
  return ON_TransformPointList( 3, FALSE, Count(), 3, (m_a) ? &m_a[0].x : 0, xform );
}

BOOL ON_3fPointArray::SwapCoordinates( int i, int j )
{
  return ON_SwapPointListCoordinates( Count(), 3, (m_a) ? &m_a[0].x : 0, i, j );
}

ON_3fPointArray& ON_3fPointArray::operator=(const ON_3fPointArray& src)
{
  if ( this != &src ) {
    ON_SimpleArray<ON_3fPoint>::operator=(src);
  }
  return *this;
}

ON_4fPointArray::ON_4fPointArray() 
{}

ON_4fPointArray::ON_4fPointArray(int c) : ON_SimpleArray<ON_4fPoint>(c) 
{}

ON_4fPointArray::ON_4fPointArray(const ON_4fPointArray& src) : ON_SimpleArray<ON_4fPoint>(src)
{}

BOOL ON_4fPointArray::Transform( const ON_Xform& xform )
{
  return ON_TransformPointList( 3, TRUE, Count(), 4, (m_a) ? &m_a[0].x : 0, xform );
}

BOOL ON_4fPointArray::SwapCoordinates( int i, int j )
{
  return ON_SwapPointListCoordinates( Count(), 4, (m_a) ? &m_a[0].x : 0, i, j );
}

ON_4fPointArray& ON_4fPointArray::operator=(const ON_4fPointArray& src)
{
  if ( this != &src ) {
    ON_SimpleArray<ON_4fPoint>::operator=(src);
  }
  return *this;
}

ON_2fVectorArray::ON_2fVectorArray() 
{}

ON_2fVectorArray::ON_2fVectorArray(int c) : ON_SimpleArray<ON_2fVector>(c) 
{}

ON_2fVectorArray::ON_2fVectorArray(const ON_2fVectorArray& src) : ON_SimpleArray<ON_2fVector>(src)
{}

BOOL ON_2fVectorArray::GetBBox( // returns TRUE if successful
       float* boxmin,    // boxmin[2]
       float* boxmax,    // boxmax[2]
       BOOL bGrowBox
       ) const
{
  return ON_GetPointListBoundingBox( 2, FALSE, Count(), 2, (m_a) ? &m_a[0].x : 0, boxmin, boxmax, bGrowBox );
}

BOOL ON_2fVectorArray::Transform( const ON_Xform& xform )
{
  return ON_TransformPointList( 2, FALSE, Count(), 2, (m_a) ? &m_a[0].x : 0, xform );
}

BOOL ON_2fVectorArray::SwapCoordinates( int i, int j )
{
  return ON_SwapPointListCoordinates( Count(), 2, (m_a) ? &m_a[0].x : 0, i, j );
}

ON_2fVectorArray& ON_2fVectorArray::operator=(const ON_2fVectorArray& src)
{
  if ( this != &src ) {
    ON_SimpleArray<ON_2fVector>::operator=(src);
  }
  return *this;
}

ON_3fVectorArray::ON_3fVectorArray() 
{}

ON_3fVectorArray::ON_3fVectorArray(int c) : ON_SimpleArray<ON_3fVector>(c) 
{}

ON_3fVectorArray::ON_3fVectorArray(const ON_3fVectorArray& src) : ON_SimpleArray<ON_3fVector>(src)
{}

BOOL ON_3fVectorArray::GetBBox( // returns TRUE if successful
       float* boxmin,    // boxmin[3]
       float* boxmax,    // boxmax[3]
       BOOL bGrowBox
       ) const
{
  return ON_GetPointListBoundingBox( 3, FALSE, Count(), 3, (m_a) ? &m_a[0].x : 0, boxmin, boxmax, bGrowBox );
}

BOOL ON_3fVectorArray::Transform( const ON_Xform& xform )
{
  return ON_TransformPointList( 3, FALSE, Count(), 3, (m_a) ? &m_a[0].x : 0, xform );
}

BOOL ON_3fVectorArray::SwapCoordinates( int i, int j )
{
  return ON_SwapPointListCoordinates( Count(), 3, (m_a) ? &m_a[0].x : 0, i, j );
}

ON_3fVectorArray& ON_3fVectorArray::operator=(const ON_3fVectorArray& src)
{
  if ( this != &src ) {
    ON_SimpleArray<ON_3fVector>::operator=(src);
  }
  return *this;
}

