/* $Header: /src2/opennurbs/opennurbs_bitmap.h 10    1/03/01 4:59p Dalelear $ */
/* $NoKeywords: $ */
/*
//
// Copyright (c) 1993-2001 Robert McNeel & Associates. All rights reserved.
// Rhinoceros is a registered trademark of Robert McNeel & Assoicates.
//
// THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
// ALL IMPLIED WARRANTIES OF FITNESS FOR ANY PARTICULAR PURPOSE AND OF
// MERCHANTABILITY ARE HEREBY DISCLAIMED.
//				
// For complete openNURBS copyright information see <http://www.opennurbs.org>.
//
////////////////////////////////////////////////////////////////
*/

////////////////////////////////////////////////////////////////
//
//  Defines ON_WindowsBITMAPINFO class that is used to provide OS independent
//  serialization of Windows device independent bitmaps (BITMAPINFO) used
//  to store preview images.
//
////////////////////////////////////////////////////////////////

#if !defined(OPENNURBS_BITMAP_INC_)
#define OPENNURBS_BITMAP_INC_

class ON_CLASS ON_Bitmap : public ON_Object
{
  // virtual base class for bitmap objects
  ON_OBJECT_DECLARE(ON_Bitmap);
public:
  ON_Bitmap();
  ON_Bitmap(const ON_Bitmap&);
  ON_Bitmap& operator=(const ON_Bitmap&);
  ~ON_Bitmap();

  void Dump( ON_TextLog& ) const; // for debugging

  virtual
  int Width() const = 0;
  virtual
  int Height() const = 0; // >0 means it's a bottom-up bitmap with origin at lower right
                          // <0 means it's a top-down bitmap with origin at upper left
  virtual
  int BitsPerPixel() const = 0; // bits per pixel
  virtual
  int SizeofScan() const = 0;  // number of bytes per scan line
  virtual
  int SizeofImage() const = 0; // size of current map in bytes

  virtual
  unsigned char* Bits(
    int // index of scan line 
    ) = 0;
  virtual
  const unsigned char* Bits(
    int // index of scan line 
    ) const = 0;

  ON_wString m_name;
};


#define BI_RGB        0L
#define BI_RLE8       1L
#define BI_RLE4       2L
#define BI_BITFIELDS  3L

struct ON_WindowsBITMAPINFOHEADER
{
  // Mimics Windows BITMAPINFOHEADER structure.
  // For details searh for "BITMAPINFOHEADER" at http://msdn.microsoft.com/default.asp 
  unsigned int   biSize;          // DWORD = sizeof(BITMAPINFOHEADER)
  int            biWidth;         // LONG  = width (in pixels) of (decompressed) bitmap
  int            biHeight;        // LONG  = height (in pixels) of (decompressed) bitmap
                                  //         >0 means it's a bottom-up bitmap with origin
                                  //            in the lower left corner.
                                  //         <0 means it's a top-down bitmap with origin
                                  //            in the upper left corner.
  unsigned short biPlanes;        // WORD  = number of planes 
                                  //         (always 1 in current Windows versions)
  unsigned short biBitCount;      // WORD  = bits per pixel (0,1,4,8,16,24,32 are valid)
                                  //         1 See http://msdn.microsoft.com/default.asp  
                                  //         4 See http://msdn.microsoft.com/default.asp  
                                  //         8 The bitmap has a maximum of 256 colors, 
                                  //           and the bmiColors member contains up 
                                  //           to 256 entries. In this case, each byte
                                  //           in the array represents a single pixel. 
                                  //        16 See http://msdn.microsoft.com/default.asp  
                                  //        24 If biClrUsed=0 and biCompression=BI_RGB(0),
                                  //           then each 3-byte triplet in the bitmap 
                                  //           array represents the relative intensities
                                  //           of blue, green, and red, respectively, for
                                  //           a pixel. For other possibilities, see
                                  //           http://msdn.microsoft.com/default.asp  
                                  //        32 If biClrUsed=0 and biCompression=BI_RGB(0),
                                  //           then each 4-byte DWORD in the bitmap 
                                  //           array represents the relative intensities
                                  //           of blue, green, and red, respectively, for
                                  //           a pixel. The high byte in each DWORD is not
                                  //           used.  
                                  //           If biClrUsed=3, biCompression=BITFIELDS(3),
                                  //           biColors[0] = red mask (0x00FF0000), 
                                  //           biColors[1] = green mask (0x0000FF00), and
                                  //           biColors[2] = blue mask (0x000000FF),
                                  //           then tese masks are used with each 4-byte
                                  //           DWORD in the bitmap array to determine
                                  //           the pixel's relative intensities.                                 //           
                                  //           For other possibilities, see
                                  //           http://msdn.microsoft.com/default.asp  
  unsigned int   biCompression;   // DWORD   Currently, Windows defines the following
                                  //         types of compression.
                                  //         =0  BI_RGB (no compression)
                                  //         =1  BI_RLE8 (run length encoded used for 8 bpp)
                                  //         =2  BI_RLE4 (run length encoded used for 4 bpp)
                                  //         =3  BI_BITFIELDS  Specifies that the bitmap is
                                  //             not compressed and that the color table 
                                  //             consists of three DWORD color masks that
                                  //             specify the red, green, and blue components,
                                  //             respectively, of each pixel. This is valid
                                  //             when used with 16- and 32-bit-per-pixel
                                  //             bitmaps.
                                  //         =4  BI_JPEG (not supported in Win 95/NT4)
                                  //
  unsigned int   biSizeImage;     // DWORD = bytes in image
  int            biXPelsPerMeter; // LONG
  int            biYPelsPerMeter; // LONG
  unsigned int   biClrUsed;       // DWORD = 0 or true length of bmiColors[] array.  If 0,
                                  //           then the value of biBitCount determines the
                                  //           length of the bmiColors[] array.
  unsigned int   biClrImportant;  // DWORD
};

struct ON_WindowsRGBQUAD {
  // Mimics Windows RGBQUAD structure.
  // For details searh for "RGBQUAD" at http://msdn.microsoft.com/default.asp 
  unsigned char rgbBlue;      // BYTE
  unsigned char rgbGreen;     // BYTE
  unsigned char rgbRed;       // BYTE
  unsigned char rgbReserved;  // BYTE
};

struct ON_WindowsBITMAPINFO
{
  // Mimics Windows BITMAPINFO structure.
  // For details searh for "BITMAPINFO" at http://msdn.microsoft.com/default.asp 
  ON_WindowsBITMAPINFOHEADER bmiHeader;
  ON_WindowsRGBQUAD bmiColors[1]; //   The "[1]" is for the compiler.  In
                                  //   practice this array commonly has
                                  //   length 0, 3, or 256 and a BITMAPINFO*
                                  //   points to a contiguous piece of memory
                                  //   that contains
                                  //     BITMAPINFOHEADER
                                  //     RGBQUAD[length determined by flags]
                                  //     unsigned char[biSizeImage]
                                  //
                                  //   See the ON_WindowsBITMAPINFOHEADER comments
                                  //   and http://msdn.microsoft.com/default.asp
                                  //   for more details.
};

class ON_OpenGLBitmap;

class ON_CLASS ON_WindowsBitmap : public ON_Bitmap
{
  ON_OBJECT_DECLARE(ON_WindowsBitmap);
  // Uncompressed 8 bpp, 24 bpp, or 32 bpp Windows device independent bitmaps (DIB)
public:
  // The ON_WindowsBITMAPINFO, ON_WindowsBITMAPINFOHEADER, ON_WindowsRGBQUAD
  // structures mimic the ones defined in Windows header files and are
  // provided so that non-Windows OSs can read/write 3DM preview bitmaps.
  // 

  ON_WindowsBitmap();
  ON_WindowsBitmap( const ON_WindowsBitmap& );
  ON_WindowsBitmap( const ON_OpenGLBitmap& );
  ~ON_WindowsBitmap();

  ON_WindowsBitmap& operator=( const ON_WindowsBitmap& );
  ON_WindowsBitmap& operator=( const ON_OpenGLBitmap& );

  virtual 
  ON_WindowsBITMAPINFO* Realloc(ON_WindowsBITMAPINFO*,size_t); // default uses onrealloc()

  void EmergencyDestroy();
  void Destroy();

  BOOL Create( 
         int, // width
         int, // height
         int  // bits per pixel ( 1, 2, 4, 8, 16, 24, or 32 )
         );

  BOOL IsValid() const;

  BOOL Write( ON_BinaryArchive& ) const; // writes compressed image
  BOOL Read( ON_BinaryArchive& );        // reads compressed image
  BOOL WriteCompressed( ON_BinaryArchive& ) const;
  BOOL ReadCompressed( ON_BinaryArchive& );
  BOOL WriteUncompressed( ON_BinaryArchive& ) const;
  BOOL ReadUncompressed( ON_BinaryArchive& );

  int Width() const;
  int Height() const; // >0 means it's a bottom-up bitmap with origin at lower right
                      // <0 means it's a top-down bitmap with origin at upper left

  int PaletteColorCount() const; // number of colors in palette
  int SizeofPalette() const;     // number of bytes in palette
  int BitsPerPixel() const;
  int SizeofPixel() const;       // number of bytes per pixel
  int SizeofScan() const;        // number of bytes per scan line
  int SizeofImage() const;       // number of bytes in image

  unsigned char* Bits(
    int // index of scan line 
    );
  const unsigned char* Bits(
    int // index of scan line 
    ) const;
  int PaletteIndex( ON_Color ) const; // for 8bpp bitmaps
  ON_Color Pixel( 
    int, // 0 <= i < width
    int  // 0 <= j < height
    ) const;
  ON_Color Pixel( 
    int,  // 0 <= i < width
    const unsigned char* // value of Bits( j )
    ) const;

  BOOL SetColor( // sets entire map to specified color 
         ON_Color
         );

#if defined(ON_OS_WINDOWS) && !defined(NOGDI)
  // NOTE: ON_WindowsBitmap bitmaps must be uncompressed 8 bpp, 24 bpp, or 32 bpp
  //       bitmaps.
  ON_WindowsBitmap( const BITMAPINFO& );
  ON_WindowsBitmap& operator=( const BITMAPINFO& );

  // If the current Windows BITMAPINFO is identical to ON_WindowsBITMAPINFO,
  // then these two members simply cast one form to the other and NULL out the
  // original pointer.  Since this class is intended to be used purely for I/O
  // of Windows device independent bitmaps (BITMAPINFO), this is a way to avoid
  // frivolous duplication and memcpy() calls.
  ON_WindowsBitmap( const BITMAPINFO*& );
  BITMAPINFO* Convert();
#endif

  struct ON_WindowsBITMAPINFO* m_bmi;
  unsigned char*               m_bits;

private:
  BOOL m_bFreeBMI;
};


class ON_CLASS ON_OpenGLBitmap : public ON_Bitmap
{
  ON_OBJECT_DECLARE(ON_OpenGLBitmap);
  // Unpacked one byte per channel RGB or RGBA Open GL bitmap
public:
  ON_OpenGLBitmap();
  ON_OpenGLBitmap( const ON_OpenGLBitmap& );
  ON_OpenGLBitmap( const ON_WindowsBitmap& );
  ~ON_OpenGLBitmap();

  ON_OpenGLBitmap& operator=( const ON_OpenGLBitmap& );
  ON_OpenGLBitmap& operator=( const ON_WindowsBitmap& );

  virtual 
  unsigned char* Realloc(unsigned char*,size_t); // default uses onrealloc()

  void EmergencyDestroy();
  void Destroy();

  BOOL Create( 
         int, // width
         int, // height
         int  // sizeof pixel ( 3 = rgb, 4 = rgba)
         );

  BOOL IsValid() const;

  BOOL Write( ON_BinaryArchive& ) const;
  BOOL Read( ON_BinaryArchive& );

  int Width() const;
  int Height() const; // >0 means it's a bottom-up bitmap with origin at lower right
                      // <0 means it's a top-down bitmap with origin at upper left
  int SizeofPixel() const; // ( 3 for RGB, 4 for RGBA )
  int BitsPerPixel() const;
  int SizeofScan() const;  // number of bytes per scan line
  int SizeofImage() const; // size of current map

  unsigned char* Bits(
    int // index of scan line 
    );
  const unsigned char* Bits(
    int // index of scan line 
    ) const;
  ON_Color Pixel( 
    int, // 0 <= i < width
    int  // 0 <= j < height
    ) const;
  ON_Color Pixel( 
    int,  // 0 <= i < width
    const unsigned char* // value of Bits( j )
    ) const;

  BOOL SetColor( // sets entire map to specified color 
                 // (use SetApha() for alpha channel)
         ON_Color
         );
  BOOL SetAlpha( // sets entire map alpha to specified value
         int // 0 = transparent, 255 = opaque
         );

  BOOL Convert( 
    const ON_WindowsBitmap&, 
    int = 3 // OpenGL pixel size in bytes (3=RGB, 4=RGBA)
    );
  BOOL ConvertToTextureMap();
  BOOL ConvertToRGB();
  BOOL ConvertToRGBA();
  BOOL Stretch( 
         int, // new width
         int  // new height
         );

  unsigned char* m_bits;
private:
  BOOL GetTextureSize( // get valid texture size that is closest to current size
        int*, // texture width  - will be 2^m for some integer m >= 0
        int*  // texture height - will be 2^n for some integer n >= 0
        ) const;
  int SizeofImage( // size of a map with specified width, height, and pixel size
         int, // width
         int, // height
         int  // sizeof pixel ( 3 for RGB, 4 for RGBA )
    ) const;
  int m_width;
  int m_height;
  int m_sizeof_pixel; // 3 = rgb, 4 = rgba
};

#endif
