/* $Header: /src2/opennurbs/opennurbs_brep.cpp 56    7/24/01 3:57p Dalelear $ */
/* $NoKeywords: $ */
/*
//
// Copyright (c) 1993-2001 Robert McNeel & Associates. All rights reserved.
// Rhinoceros is a registered trademark of Robert McNeel & Assoicates.
//
// THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
// ALL IMPLIED WARRANTIES OF FITNESS FOR ANY PARTICULAR PURPOSE AND OF
// MERCHANTABILITY ARE HEREBY DISCLAIMED.
//				
// For complete openNURBS copyright information see <http://www.opennurbs.org>.
//
////////////////////////////////////////////////////////////////
*/

#include "opennurbs.h"

////////////////////////////////////////////////////////////////
//   Class ON_BrepVertex
////////////////////////////////////////////////////////////////

ON_OBJECT_IMPLEMENT(ON_BrepVertex,ON_Geometry,"60B5DBC0-E660-11d3-BFE4-0010830122F0");

ON_BrepVertex::ON_BrepVertex()
              : m_vertex_user_i(0),
                m_vertex_index(-1),
                m_tolerance(ON_UNSET_VALUE)
{}

ON_BrepVertex::ON_BrepVertex( int vertex_index ) 
              : m_vertex_user_i(0),
                m_vertex_index(vertex_index),
                m_tolerance(ON_UNSET_VALUE)
{}

ON_BrepVertex& ON_BrepVertex::operator=(const ON_BrepVertex& src)
{
  if ( &src != this ) {
    ON_Point::operator=(src);
    m_vertex_user_i = src.m_vertex_user_i;
    m_vertex_index  = src.m_vertex_index;
    m_ei            = src.m_ei;
    m_tolerance     = src.m_tolerance;
  }
  return *this;
}

BOOL 
ON_BrepVertex::IsValid() const
{
  BOOL rc = (m_vertex_index >= 0) ? TRUE : FALSE;
  const int ve_count = EdgeCount();
  int vei, ei;
  for ( vei = 0; vei < ve_count; vei++ ) {
    ei = m_ei[vei];
    if ( ei < 0 )
      rc = FALSE;
  }
  return rc;
}

void 
ON_BrepVertex::Dump( ON_TextLog& dump ) const
{
  dump.Print("ON_BrepVertex[%d]: ",m_vertex_index);
  dump.Print( point );
  dump.Print("\n");
}

BOOL
ON_BrepVertex::Write( ON_BinaryArchive& file ) const
{
  BOOL rc = file.WriteInt( m_vertex_index );
  if ( rc )
    rc = file.WritePoint( point );
  if ( rc )
    rc = file.WriteArray( m_ei );
  if ( rc )
    rc = file.WriteDouble( m_tolerance );
  return rc;
}

BOOL
ON_BrepVertex::Read( ON_BinaryArchive& file )
{
  BOOL rc = file.ReadInt( &m_vertex_index );
  if ( rc )
    rc = file.ReadPoint( point );
  if ( rc )
    rc = file.ReadArray( m_ei );
  if ( rc )
    rc = file.ReadDouble( &m_tolerance );
  return rc;
}

BOOL
ON_BrepVertex::SetPoint( const ON_3dPoint& p )
{
  point = p;
  m_tolerance = ON_UNSET_VALUE;
  return TRUE;
}

ON_3dPoint
ON_BrepVertex::Point() const
{
  return point;
}

double
ON_BrepVertex::Tolerance() const
{
  return m_tolerance;
}

int
ON_BrepVertex::EdgeCount() const
{
  return m_ei.Count();
}


////////////////////////////////////////////////////////////////
//   Class ON_BrepEdge
////////////////////////////////////////////////////////////////

ON_OBJECT_IMPLEMENT(ON_BrepEdge,ON_CurveProxy,"60B5DBC1-E660-11d3-BFE4-0010830122F0");

ON_BrepEdge::ON_BrepEdge() : ON_CurveProxy(0),
                         m_edge_user_i(0),
                         m_edge_index(-1),
                         m_c3i(-1),
                         m_tolerance(ON_UNSET_VALUE),
                         m_selected_trim(0), m_brep(0)
{
  m_vi[0] = m_vi[1] = -1;
}

ON_BrepEdge::ON_BrepEdge(int edge_index ) : ON_CurveProxy(0),
                         m_edge_user_i(0),
                         m_edge_index(edge_index),
                         m_c3i(-1),
                         m_tolerance(ON_UNSET_VALUE),
                         m_selected_trim(0), m_brep(0)
{
  m_vi[0] = m_vi[1] = -1;
}

ON_BrepEdge& ON_BrepEdge::operator=(const ON_BrepEdge& src)
{
  if ( &src != this ) {
    ON_CurveProxy::operator=(src);
    m_edge_user_i = src.m_edge_user_i;
    m_edge_index  = src.m_edge_index;
    m_c3i         = src.m_c3i;
    m_vi[0]       = src.m_vi[0];
    m_vi[1]       = src.m_vi[1];
    m_ti          = src.m_ti;
    m_tolerance   = src.m_tolerance;
  }
  return *this;
}

BOOL ON_BrepEdge::IsValid() const
{
  BOOL rc = ON_CurveProxy::IsValid();
  if (m_edge_index < 0)
    rc = FALSE;
  if ( m_c3i < 0 )
    rc = FALSE;
  if ( m_vi[0] < 0 )
    rc = FALSE;
  if ( m_vi[1] < 0 )
    rc = FALSE;
  if ( !m_brep )
    rc = FALSE;
  return rc;
}

void ON_BrepEdge::Dump( ON_TextLog& dump ) const
{
  dump.Print("ON_BrepEdge[%d]: ",m_edge_index);
}

BOOL ON_BrepEdge::Write( ON_BinaryArchive& file ) const
{
  BOOL rc = file.WriteInt( m_edge_index );
  if (rc) rc = file.WriteInt( m_c3i );
  if (rc) rc = file.WriteInt( m_bReversed );
  if (rc) rc = file.WriteInterval( m_domain );
  if (rc) rc = file.WriteInt( 2, m_vi );
  if (rc) rc = file.WriteArray( m_ti );
  if (rc) rc = file.WriteDouble( m_tolerance );
  return rc;
}

BOOL ON_BrepEdge::Read( ON_BinaryArchive& file )
{
  BOOL rc = file.ReadInt( &m_edge_index );
  if (rc) rc = file.ReadInt( &m_c3i );
  if (rc) rc = file.ReadInt( &m_bReversed );
  if (rc) rc = file.ReadInterval( m_domain );
  if (rc) rc = file.ReadInt( 2, m_vi );
  if (rc) rc = file.ReadArray( m_ti );
  if (rc) rc = file.ReadDouble( &m_tolerance );
  return rc;
}

////////////////////////////////////////////////////////////////
//   Class ON_BrepTrim
////////////////////////////////////////////////////////////////

ON_OBJECT_IMPLEMENT(ON_BrepTrim,ON_Object,"60B5DBC2-E660-11d3-BFE4-0010830122F0");

ON_BrepTrim::ON_BrepTrim()
              : m_trim_user_i(0),
                m_trim_index(-1), 
                m_c2i(-1), 
                m_ei(-1), 
                m_bRev3d(FALSE), 
                m_type(ON_BrepTrim::unknown), 
                m_iso(ON_Surface::not_iso),
                m_li(-1),
                m__legacy_2d_tol(ON_UNSET_VALUE), 
                m__legacy_3d_tol(ON_UNSET_VALUE),
                m__legacy_flags(0)
{
  m_vi[0] = m_vi[1] = -1; 
  m_tolerance[0] = m_tolerance[1] = ON_UNSET_VALUE;
  //m_P[0] = ON_unset_point;
  //m_P[1] = ON_unset_point;
}

ON_BrepTrim::ON_BrepTrim(int trim_index) 
              : m_trim_user_i(0),
                m_trim_index(trim_index), 
                m_c2i(-1), 
                m_ei(-1), 
                m_bRev3d(FALSE), 
                m_type(ON_BrepTrim::unknown), 
                m_iso(ON_Surface::not_iso),
                m_li(-1),
                m__legacy_2d_tol(ON_UNSET_VALUE), 
                m__legacy_3d_tol(ON_UNSET_VALUE),
                m__legacy_flags(0)
{
  m_vi[0] = m_vi[1] = -1; 
  m_tolerance[0] = m_tolerance[1] = ON_UNSET_VALUE;
  //m_P[0] = ON_unset_point;
  //m_P[1] = ON_unset_point;
}

ON_BrepTrim& ON_BrepTrim::operator=(const ON_BrepTrim& src)
{
  if ( &src != this ) {
    ON_Object::operator=(src);
    m_trim_user_i = src.m_trim_user_i;
    m_trim_index  = src.m_trim_index;
    m_c2i    = src.m_c2i;
    m_t      = src.m_t;
    m_ei     = src.m_ei;
    m_vi[0]  = src.m_vi[0];
    m_vi[1]  = src.m_vi[1];
    m_bRev3d = src.m_bRev3d;
    m_type   = src.m_type;
    m_iso    = src.m_iso;
    m_li     = src.m_li;
    m_tolerance[0] = src.m_tolerance[0];
    m_tolerance[1] = src.m_tolerance[1];
    //m_P[0] = src.m_P[0];
    //m_P[1] = src.m_P[1];
    m__legacy_2d_tol = src.m__legacy_2d_tol;
    m__legacy_3d_tol = src.m__legacy_3d_tol;
    m__legacy_flags = src.m__legacy_flags;
    m_pline = src.m_pline;
    m_pbox = src.m_pbox;
  }
  return *this;
}

BOOL ON_BrepTrim::IsValid() const
{
  BOOL rc = (m_trim_index >= 0) ? TRUE : FALSE;
  if ( m_c2i < 0 )
    rc = FALSE;
  if ( m_t[0] == m_t[1] )
    rc = FALSE;
  if ( m_ei < 0 ) {
    if ( m_type != singular )
      rc = FALSE;
  }
  if ( m_vi[0] < 0 )
    rc = FALSE;
  if ( m_vi[1] < 0 )
    rc = FALSE;
  int i = m_type;
  if ( i < 0 || i >= type_count )
    rc = FALSE;
  i = m_iso;
  if ( i < 0 || i >= ON_Surface::iso_count )
    rc = FALSE;
  if ( m_li < 0 )
    rc = FALSE;
  // m_P[0], m_P[1] invalid since we now give away evaluators
  //if ( m_vi[0] == m_vi[1] ) {
  //  if ( m_P[0] != m_P[1] )
  //    rc = FALSE;
  //}
  //else if ( m_P[0] == m_P[1] )
  //  rc = FALSE;
  return rc;
}

void ON_BrepTrim::Dump( ON_TextLog& dump ) const
{
  dump.Print("ON_BrepTrim[%d]:\n",m_trim_index);
}

BOOL ON_BrepTrim::Write( ON_BinaryArchive& file ) const
{
  ON_3dPoint P(0.0,0.0,0.0);
  BOOL rc = file.WriteInt( m_trim_index );
  int i;
  if ( rc )
    rc = file.WriteInt( m_c2i );
  if ( rc )
    rc = file.WriteDouble( 2, m_t.m_t );
  if ( rc )
    rc = file.WriteInt( m_ei );
  if ( rc )
    rc = file.WriteInt( 2, m_vi );
  if ( rc )
    rc = file.WriteInt( m_bRev3d );
  i = m_type;
  if ( rc )
    rc = file.WriteInt( i );
  i = m_iso;
  if ( rc )
    rc = file.WriteInt( i );
  if ( rc )
    rc = file.WriteInt( m_li );
  if ( rc )
    rc = file.WriteDouble( 2, m_tolerance );
  if ( rc )
    rc = file.WritePoint( P ); // m_P[0] );
  if ( rc )
    rc = file.WritePoint( P ); // m_P[1] );
  if ( rc )
    rc = file.WriteDouble( m__legacy_2d_tol );
  if ( rc )
    rc = file.WriteDouble( m__legacy_3d_tol );
  return rc;
}

BOOL ON_BrepTrim::Read( ON_BinaryArchive& file )
{
  ON_3dPoint P[2];
  int i;
  BOOL rc = file.ReadInt( &m_trim_index );
  if ( rc )
    rc = file.ReadInt( &m_c2i );
  if ( rc )
    rc = file.ReadDouble( 2, m_t.m_t );
  if ( rc )
    rc = file.ReadInt( &m_ei );
  if ( rc )
    rc = file.ReadInt( 2, m_vi );
  if ( rc )
    rc = file.ReadInt( &m_bRev3d );
  if ( rc )
    rc = file.ReadInt( &i );
  switch (i) {
  case unknown:
    m_type = unknown;
    break;
  case boundary:
    m_type = boundary;
    break;
  case mated:
    m_type = mated;
    break;
  case seam:
    m_type = seam;
    break;
  case singular:
    m_type = singular;
    break;
  }
  if ( rc )
    rc = file.ReadInt( &i );
  switch(i) {
  case ON_Surface::not_iso:
    m_iso = ON_Surface::not_iso;
    break;
  case ON_Surface::x_iso:
    m_iso = ON_Surface::x_iso;
    break;
  case ON_Surface::y_iso:
    m_iso = ON_Surface::y_iso;
    break;
  case ON_Surface::W_iso:
    m_iso = ON_Surface::W_iso;
    break;
  case ON_Surface::S_iso:
    m_iso = ON_Surface::S_iso;
    break;
  case ON_Surface::E_iso:
    m_iso = ON_Surface::E_iso;
    break;
  case ON_Surface::N_iso:
    m_iso = ON_Surface::N_iso;
    break;
  }
  if ( rc )
    rc = file.ReadInt( &m_li );
  if ( rc )
    rc = file.ReadDouble( 2, m_tolerance );
  if ( rc )
    rc = file.ReadPoint( P[0] ); //m_P[0] );
  if ( rc )
    rc = file.ReadPoint( P[1] ); //m_P[1] );
  if ( rc )
    rc = file.ReadDouble( &m__legacy_2d_tol );
  if ( rc )
    rc = file.ReadDouble( &m__legacy_3d_tol );
  return rc;
}

////////////////////////////////////////////////////////////////
//   Class ON_BrepLoop
////////////////////////////////////////////////////////////////

ON_OBJECT_IMPLEMENT(ON_BrepLoop,ON_BrepEntity,"60B5DBC3-E660-11d3-BFE4-0010830122F0");

ON_BrepLoop::ON_BrepLoop()
              : m_loop_user_i(0),
                m_loop_index(-1), 
                m_type(ON_BrepLoop::unknown), 
                m_fi(-1)
{}

ON_BrepLoop::ON_BrepLoop(int loop_index) 
              : m_loop_user_i(0),
                m_loop_index(loop_index), 
                m_type(ON_BrepLoop::unknown), 
                m_fi(-1)
{}

ON_BrepLoop& ON_BrepLoop::operator=(const ON_BrepLoop& src)
{
  if ( &src != this ) {
    ON_Object::operator=(src);
    m_loop_user_i = src.m_loop_user_i;
    m_loop_index  = src.m_loop_index;
    m_ti    = src.m_ti;
    m_type  = src.m_type;
    m_fi    = src.m_fi;
    m_pbox  = src.m_pbox;
  }
  return *this;
}

BOOL ON_BrepLoop::IsValid() const
{
  BOOL rc = (m_loop_index >= 0) ? TRUE : FALSE;
  if ( m_ti.Count() < 1 )
    rc = FALSE;
  int i = m_type;
  if ( i < 0 || i > type_count )
    rc = FALSE;
  if ( m_fi < 0 )
    rc = FALSE;
  return rc;
}

void ON_BrepLoop::Dump( ON_TextLog& dump ) const
{
  dump.Print("ON_BrepLoop[%d]:\n",m_loop_index);
}

BOOL ON_BrepLoop::Write( ON_BinaryArchive& file ) const
{
  int i;
  BOOL rc = file.WriteInt( m_loop_index );
  if (rc)
    rc = file.WriteArray( m_ti );
  i = m_type;
  if (rc)
    rc = file.WriteInt( i );
  if (rc)
    rc = file.WriteInt( m_fi );
  return rc;
}

BOOL ON_BrepLoop::Read( ON_BinaryArchive& file )
{
  int i;
  BOOL rc = file.ReadInt( & m_loop_index );
  if (rc)
    rc = file.ReadArray( m_ti );
  if (rc)
    rc = file.ReadInt( &i );
  switch(i) {
  case unknown:
    m_type = unknown;
    break;
  case outer:
    m_type = outer;
    break;
  case inner:
    m_type = inner;
    break;
  case slit:
    m_type = slit;
    break;
  }
  if (rc)
    rc = file.ReadInt( &m_fi );
  return rc;
}

int ON_BrepLoop::IndexOfTrim( const ON_BrepTrim& trim ) const
{
  const int count = m_ti.Count();
  int lti;
  for ( lti = 0; lti < count; lti++ )
  {
    if ( m_ti[lti] == trim.m_trim_index )
      return lti;
  }
  return -1;
}

////////////////////////////////////////////////////////////////
//   Class ON_BrepFace
////////////////////////////////////////////////////////////////

ON_OBJECT_IMPLEMENT(ON_BrepFace,ON_BrepEntity,"60B5DBC4-E660-11d3-BFE4-0010830122F0");

ON_BrepFace::ON_BrepFace() : ON_SurfaceProxy(0),
                m_face_user_i(0),
                m_face_index(-1), 
                m_si(-1), 
                m_bRev(FALSE),
                m_render_mesh(0),
                m_analysis_mesh(0),
                m_preview_mesh(0),
                m_material_index(-1),
                m_brep(0)
{}

ON_BrepFace::ON_BrepFace(int face_index) : ON_SurfaceProxy(0),
                m_face_user_i(0),
                m_face_index(face_index),
                m_si(-1), 
                m_bRev(FALSE),
                m_render_mesh(0),
                m_analysis_mesh(0),
                m_preview_mesh(0),
                m_material_index(-1),
                m_brep(0)
{}

ON_BrepFace& ON_BrepFace::operator=(const ON_BrepFace& src)
{
  if ( &src != this ) {
    ON_SurfaceProxy::operator=(src);
    m_face_user_i = src.m_face_user_i;
    m_face_index  = src.m_face_index;
    m_li    = src.m_li;
    m_si    = src.m_si;
    m_bRev  = src.m_bRev;
    if ( m_render_mesh ) {
      delete m_render_mesh;
      m_render_mesh = 0;
    }
    if ( src.m_render_mesh ) {
      m_render_mesh = new ON_Mesh(*src.m_render_mesh);
    }    
    if ( m_analysis_mesh ) {
      delete m_analysis_mesh;
      m_analysis_mesh = 0;
    }
    if ( src.m_analysis_mesh ) {
      m_analysis_mesh = new ON_Mesh(*src.m_analysis_mesh);
    }    
    if ( m_preview_mesh ) {
      delete m_preview_mesh;
      m_preview_mesh = 0;
    }
    if ( src.m_preview_mesh ) {
      m_preview_mesh = new ON_Mesh(*src.m_preview_mesh);
    }    
    m_material_index = src.m_material_index;
  }
  return *this;
}

ON_BrepFace::~ON_BrepFace()
{
  DestroyMesh(ON::any_mesh);
  m_li.Destroy();
}

const ON_Brep* ON_BrepFace::Brep() const
{
  return m_brep;
}


BOOL ON_BrepFace::IsValid() const
{
  BOOL rc = (m_face_index >= 0) ? TRUE : FALSE;
  if ( m_li.Count() < 1 )
    rc = FALSE;
  if ( m_si < 0 )
    rc = FALSE;
  return rc;
}

void ON_BrepFace::Dump( ON_TextLog& dump ) const
{
  dump.Print("ON_BrepFace[%d]:\n",m_face_index);
}

BOOL ON_BrepFace::Write( ON_BinaryArchive& file ) const
{
  BOOL rc = file.WriteInt( m_face_index );
  if ( rc )
    rc = file.WriteArray( m_li );
  if ( rc )
    rc = file.WriteInt( m_si );
  if ( rc )
    rc = file.WriteInt( m_bRev );
  if ( rc )
    rc = file.WriteInt( m_material_index );
  return rc;
}

BOOL ON_BrepFace::Read( ON_BinaryArchive& file )
{
  BOOL rc = file.ReadInt( &m_face_index );
  if ( rc )
    rc = file.ReadArray( m_li );
  if ( rc )
    rc = file.ReadInt( &m_si );
  if ( rc )
    rc = file.ReadInt( &m_bRev );
  if ( rc )
    rc = file.ReadInt( &m_material_index );
  return rc;
}

int ON_BrepFace::MaterialIndex() const
{
  return m_material_index;
}

void ON_BrepFace::SetMaterialIndex(int mi)
{
  m_material_index = (mi>0) ? mi : -1;
}

ON_Mesh* ON_BrepFace::Mesh( ON::mesh_type mt ) const
{
  ON_Mesh* m = 0;
  switch(mt) {
  case ON::render_mesh:
    m = m_render_mesh;
    break;
  case ON::analysis_mesh:
    m = m_analysis_mesh;
    break;
  case ON::preview_mesh:
    m = m_preview_mesh;
    break;
  default:
    m = m_render_mesh ? m_render_mesh : m_analysis_mesh;
    if ( !m )
      m = m_preview_mesh;
    break;
  }
  if ( m ) {
    m->m_parent = this;
    m->m_material_index = m_material_index;
  }
  return m;
}

void ON_BrepFace::DestroyMesh( ON::mesh_type mt )
{
  switch(mt) {
  case ON::render_mesh:
    if ( m_render_mesh ) {
      delete m_render_mesh;
      m_render_mesh = 0;
    }
    break;
  case ON::analysis_mesh:
    if (m_analysis_mesh) {
      delete m_analysis_mesh;
      m_analysis_mesh = 0;
    }
    break;
  case ON::preview_mesh:
    if (m_preview_mesh) {
      delete m_preview_mesh;
      m_preview_mesh = 0;
    }
    break;
  default:
    DestroyMesh( ON::render_mesh );
    DestroyMesh( ON::analysis_mesh );
    DestroyMesh( ON::preview_mesh );
    break;
  }
}

////////////////////////////////////////////////////////////////
//   Class ON_Brep
////////////////////////////////////////////////////////////////

ON_BrepVertexArray::ON_BrepVertexArray()
{}

ON_BrepVertexArray::~ON_BrepVertexArray()
{}

BOOL ON_BrepVertexArray::Read( ON_BinaryArchive& file )
{
  Empty();
  unsigned int tcode = 0;
  int count = 0;
  int i;
  int major_version = 0;
  int minor_version = 0;
  BOOL rc = file.BeginRead3dmChunk( &tcode, &i );
  if (rc) {
    if (tcode != TCODE_ANONYMOUS_CHUNK)
      rc = FALSE;
    if (rc) rc = file.Read3dmChunkVersion(&major_version,&minor_version);
    if (rc) {
      if ( major_version==1 ) {
        if (rc) rc = file.ReadInt(&count);
        SetCapacity(count);
        for ( i = 0; i < count && rc ; i++ ) {
          ON_BrepVertex& vertex = AppendNew();
          rc = vertex.Read(file);
        }    
      }
      else {
        rc = 0;
      }
    }
    if ( !file.EndRead3dmChunk() )
      rc = FALSE;
  }
  return rc;
}

BOOL ON_BrepVertexArray::Write( ON_BinaryArchive& file ) const
{
  int i;
  BOOL rc = file.BeginWrite3dmChunk( TCODE_ANONYMOUS_CHUNK, 0 );
  if (rc) {
    rc = file.Write3dmChunkVersion(1,0);
    const int count = Count();
    if (rc) rc = file.WriteInt( count );
    for ( i = 0; rc && i < count; i++ ) {
      if (rc) rc = m_a[i].Write(file);
    }
    if ( !file.EndWrite3dmChunk() )
      rc = FALSE;
  }
  return rc;
}


ON_BrepEdgeArray::ON_BrepEdgeArray()
{}

ON_BrepEdgeArray::~ON_BrepEdgeArray()
{}

BOOL ON_BrepEdgeArray::Read( ON_BinaryArchive& file )
{
  Empty();
  unsigned int tcode = 0;
  int count = 0;
  int i;
  int major_version = 0;
  int minor_version = 0;
  BOOL rc = file.BeginRead3dmChunk( &tcode, &i );
  if (rc) {
    if (tcode != TCODE_ANONYMOUS_CHUNK)
      rc = FALSE;
    if (rc) rc = file.Read3dmChunkVersion(&major_version,&minor_version);
    if (rc) {
      if ( major_version==1 ) {
        if (rc) rc = file.ReadInt(&count);
        SetCapacity(count);
        for ( i = 0; i < count && rc ; i++ ) {
          ON_BrepEdge& edge = AppendNew();
          rc = edge.Read(file);
        }    
      }
      else {
        rc = 0;
      }
    }
    if ( !file.EndRead3dmChunk() )
      rc = FALSE;
  }
  return rc;
}

BOOL ON_BrepEdgeArray::Write( ON_BinaryArchive& file ) const
{
  int i;
  BOOL rc = file.BeginWrite3dmChunk( TCODE_ANONYMOUS_CHUNK, 0 );
  if (rc) {
    rc = file.Write3dmChunkVersion(1,0);
    const int count = Count();
    if (rc) rc = file.WriteInt( count );
    for ( i = 0; rc && i < count; i++ ) {
      if (rc) rc = m_a[i].Write(file);
    }
    if ( !file.EndWrite3dmChunk() )
      rc = FALSE;
  }
  return rc;
}

ON_BrepTrimArray::ON_BrepTrimArray()
{}

ON_BrepTrimArray::~ON_BrepTrimArray()
{}

BOOL ON_BrepTrimArray::Read( ON_BinaryArchive& file )
{
  Empty();
  unsigned int tcode = 0;
  int count = 0;
  int i;
  int major_version = 0;
  int minor_version = 0;
  BOOL rc = file.BeginRead3dmChunk( &tcode, &i );
  if (rc) {
    if (tcode != TCODE_ANONYMOUS_CHUNK)
      rc = FALSE;
    if (rc) rc = file.Read3dmChunkVersion(&major_version,&minor_version);
    if (rc) {
      if ( major_version==1 ) {
        if (rc) rc = file.ReadInt(&count);
        SetCapacity(count);
        for ( i = 0; i < count && rc ; i++ ) {
          ON_BrepTrim& trim = AppendNew();
          rc = trim.Read(file);
        }    
      }
      else {
        rc = 0;
      }
    }
    if ( !file.EndRead3dmChunk() )
      rc = FALSE;
  }
  return rc;
}

BOOL ON_BrepTrimArray::Write( ON_BinaryArchive& file ) const
{
  int i;
  BOOL rc = file.BeginWrite3dmChunk( TCODE_ANONYMOUS_CHUNK, 0 );
  if (rc) {
    rc = file.Write3dmChunkVersion(1,0);
    const int count = Count();
    if (rc) rc = file.WriteInt( count );
    for ( i = 0; rc && i < count; i++ ) {
      if (rc) rc = m_a[i].Write(file);
    }
    if ( !file.EndWrite3dmChunk() )
      rc = FALSE;
  }
  return rc;
}

ON_BrepLoopArray::ON_BrepLoopArray()
{}

ON_BrepLoopArray::~ON_BrepLoopArray()
{}

BOOL ON_BrepLoopArray::Read( ON_BinaryArchive& file )
{
  Empty();
  unsigned int tcode = 0;
  int count = 0;
  int i;
  int major_version = 0;
  int minor_version = 0;
  BOOL rc = file.BeginRead3dmChunk( &tcode, &i );
  if (rc) {
    if (tcode != TCODE_ANONYMOUS_CHUNK)
      rc = FALSE;
    if (rc) rc = file.Read3dmChunkVersion(&major_version,&minor_version);
    if (rc) {
      if ( major_version==1 ) {
        if (rc) rc = file.ReadInt(&count);
        SetCapacity(count);
        for ( i = 0; i < count && rc ; i++ ) {
          ON_BrepLoop& loop = AppendNew();
          rc = loop.Read(file);
        }    
      }
      else {
        rc = 0;
      }
    }
    if ( !file.EndRead3dmChunk() )
      rc = FALSE;
  }
  return rc;
}

BOOL ON_BrepLoopArray::Write( ON_BinaryArchive& file ) const
{
  int i;
  BOOL rc = file.BeginWrite3dmChunk( TCODE_ANONYMOUS_CHUNK, 0 );
  if (rc) {
    rc = file.Write3dmChunkVersion(1,0);
    const int count = Count();
    if (rc) rc = file.WriteInt( count );
    for ( i = 0; rc && i < count; i++ ) {
      if (rc) rc = m_a[i].Write(file);
    }
    if ( !file.EndWrite3dmChunk() )
      rc = FALSE;
  }
  return rc;
}

ON_BrepFaceArray::ON_BrepFaceArray()
{}

ON_BrepFaceArray::~ON_BrepFaceArray()
{}

BOOL ON_BrepFaceArray::Read( ON_BinaryArchive& file )
{
  Empty();
  unsigned int tcode = 0;
  int count = 0;
  int i;
  int major_version = 0;
  int minor_version = 0;
  BOOL rc = file.BeginRead3dmChunk( &tcode, &i );
  if (rc) {
    if (tcode != TCODE_ANONYMOUS_CHUNK)
      rc = FALSE;
    if (rc) rc = file.Read3dmChunkVersion(&major_version,&minor_version);
    if (rc) {
      if ( major_version==1 ) {
        if (rc) rc = file.ReadInt(&count);
        SetCapacity(count);
        for ( i = 0; i < count && rc ; i++ ) {
          ON_BrepFace& face = AppendNew();
          rc = face.Read(file);
        }    
      }
      else {
        rc = 0;
      }
    }
    if ( !file.EndRead3dmChunk() )
      rc = FALSE;
  }
  return rc;
}

BOOL ON_BrepFaceArray::Write( ON_BinaryArchive& file ) const
{
  int i;
  BOOL rc = file.BeginWrite3dmChunk( TCODE_ANONYMOUS_CHUNK, 0 );
  if (rc) {
    rc = file.Write3dmChunkVersion(1,0);
    const int count = Count();
    if (rc) rc = file.WriteInt( count );
    for ( i = 0; rc && i < count; i++ ) {
      if (rc) rc = m_a[i].Write(file);
    }
    if ( !file.EndWrite3dmChunk() )
      rc = FALSE;
  }
  return rc;
}

ON_OBJECT_IMPLEMENT(ON_Brep,ON_Geometry,"60B5DBC5-E660-11d3-BFE4-0010830122F0");

void ON_Brep::Initialize()
{
  m_bbox.Destroy();
}

ON_Brep::ON_Brep()
{ 
  Initialize();
}

ON_Brep::~ON_Brep()
{ 
  DestroyMesh(ON::any_mesh);
  // everything is in array classes that destroy themselves.
}

void ON_Brep::DestroyMesh( ON::mesh_type mt )
{
  const int fcnt = m_F.Count();
  int fi;
  for ( fi = 0; fi < fcnt; fi++ ) {
    m_F[fi].DestroyMesh(mt);
  }
}


int ON_Brep::Dimension() const
{
  return (m_V.Count() > 0) ? 3 : 0;
}

BOOL ON_Brep::Transform( 
       const ON_Xform& xform
       )
{
  BOOL rc = TRUE;

  int i, count;
  TransformUserData(xform);

  count = m_C3.Count();
  for ( i = 0; i < count; i++ ) {
    if ( m_C3[i] ) {
      if ( !m_C3[i]->Transform(xform) )
        rc = FALSE;
    }
  }

  count = m_S.Count();
  for ( i = 0; i < count; i++ ) {
    if ( m_S[i] ) {
      if ( !m_S[i]->Transform(xform) )
        rc = FALSE;
    }
  }

  count = m_V.Count();
  for ( i = 0; i < count; i++ ) {
    if ( !m_V[i].Transform(xform) )
      rc = FALSE;
  }

  count = m_E.Count();
  for ( i = 0; i < count; i++ ) {
    m_E[i].TransformUserData(xform);
  }

  count = m_F.Count();
  for ( i = 0; i < count; i++ ) {
    m_F[i].TransformUserData(xform);
  }
  return rc;

}

/////////////////////////////////////////////////////////////////
// ON_Brep Creation Interface

int 
ON_Brep::AddTrimCurve( ON_Curve* pC )
{
  int c2i = -1;
  if ( pC && pC->Dimension() == 2 ) {
    c2i = m_C2.Count();
    m_C2.Append(pC);
  }
  return c2i;
}

int 
ON_Brep::AddEdgeCurve( ON_Curve* pC )
{
  int c3i = -1;
  if ( pC && pC->Dimension() == 3 ) {
    c3i = m_C3.Count();
    m_C3.Append(pC);
  }
  return c3i;
}

int 
ON_Brep::AddSurface( ON_Surface* pS )
{
  int si = -1;
  if ( pS && pS->Dimension() == 3 ) {
    si = m_S.Count();
    m_S.Append(pS);
  }
  return si;
}

ON_BrepVertex& 
ON_Brep::NewVertex()
{
  int vi = m_V.Count();
  m_V.Reserve(vi+1);
  m_V.SetCount(vi+1);
  ON_BrepVertex& vertex = m_V.Array()[vi];
  vertex.m_vertex_index = vi;
  return vertex;
}

ON_BrepVertex& 
ON_Brep::NewVertex( const ON_3dPoint& pt )
{
  ON_BrepVertex& vertex = NewVertex();
  vertex.point = pt;
  return vertex;
}

ON_BrepEdge& 
ON_Brep::NewEdge( int c3i )
{
  int ei = m_E.Count();
  ON_BrepEdge& edge = m_E.AppendNew();
  edge.m_tolerance = ON_UNSET_VALUE;
  edge.m_edge_index = ei;
  edge.m_c3i = c3i;
  if ( edge.m_c3i >= 0 && edge.m_c3i < m_C3.Count() ) {
    edge.m_curve = m_C3[edge.m_c3i];
    if ( edge.m_curve )
      edge.m_domain = edge.m_curve->Domain();
  }
  edge.m_brep = this;
  return edge;
}

ON_BrepEdge& 
ON_Brep::NewEdge( ON_BrepVertex& v0, ON_BrepVertex& v1, int c3i )
{
  ON_BrepEdge& edge = NewEdge(c3i);
  edge.m_vi[0] = v0.m_vertex_index;
  edge.m_vi[1] = v1.m_vertex_index;
  v0.m_ei.Append(edge.m_edge_index);
  v1.m_ei.Append(edge.m_edge_index);
  return edge;
}

ON_BrepTrim& 
ON_Brep::NewTrim( int c2i )
{
  int ti = m_T.Count();
  ON_BrepTrim& trim = m_T.AppendNew();
  trim.m_trim_index = ti;
  trim.m_ei = -1;
  trim.m_type = ON_BrepTrim::unknown;
  trim.m_bRev3d = FALSE;
  trim.m_c2i = c2i;
  trim.m_iso = ON_Surface::not_iso;
  trim.m_li = -1;
  trim.m_tolerance[0] = ON_UNSET_VALUE;
  trim.m_tolerance[1] = ON_UNSET_VALUE;
  trim.m__legacy_2d_tol = ON_UNSET_VALUE;
  trim.m__legacy_3d_tol = ON_UNSET_VALUE;
  trim.m__legacy_flags = 0;
  const ON_Curve* c2 = TrimCurveOf(trim);
  if ( c2 ) {
    trim.m_t = c2->Domain();
  }

  return trim;
}

ON_BrepTrim& 
ON_Brep::NewTrim( ON_BrepEdge& edge, BOOL bRev3d, int c2i )
{
  ON_BrepTrim& trim = NewTrim( c2i );
  trim.m_ei = edge.m_edge_index;
  edge.m_ti.Append(trim.m_trim_index);
  trim.m_vi[0] = edge.m_vi[bRev3d?1:0];
  trim.m_vi[1] = edge.m_vi[bRev3d?0:1];
  trim.m_bRev3d = bRev3d;
  return trim;
}

ON_BrepTrim& 
ON_Brep::NewTrim( ON_BrepEdge& edge, BOOL bRev3d, ON_BrepLoop& loop, int c2i )
{
  ON_BrepTrim& trim = NewTrim( edge, bRev3d, c2i );
  trim.m_li = loop.m_loop_index;
  loop.m_ti.Append(trim.m_trim_index);
  //const ON_Curve* c2 = TrimCurveOf(trim);
  //if ( c2 ) {
  //  const ON_Surface* srf = SurfaceOf(trim);
    //if ( srf ) {
    //  ON_3dPoint p0, p1;
    //  p0 = c2->PointAt( trim.m_t[0] );
    //  p1 = c2->PointAt( trim.m_t[1] );
    //  trim.m_P[0] = srf->PointAt( p0.x, p0.y );
    //  trim.m_P[1] = srf->PointAt( p1.x, p1.y );
    //}
  //}
  return trim;
}

ON_BrepTrim& 
ON_Brep::NewTrim( BOOL bRev3d, ON_BrepLoop& loop, int c2i )
{
  ON_BrepTrim& trim = NewTrim( c2i );
  trim.m_bRev3d = bRev3d ? TRUE : FALSE;
  trim.m_li = loop.m_loop_index;
  loop.m_ti.Append(trim.m_trim_index);
  //const ON_Curve* c2 = TrimCurveOf(trim);
  //if ( c2 ) {
  //  const ON_Surface* srf = SurfaceOf(trim);
    //if ( srf ) {
    //  ON_3dPoint p0, p1;
    //  p0 = c2->PointAt( trim.m_t[0] );
    //  p1 = c2->PointAt( trim.m_t[1] );
    //  trim.m_P[0] = srf->PointAt( p0.x, p0.y );
    //  trim.m_P[1] = srf->PointAt( p1.x, p1.y );
    //}
  //}
  return trim;
}

void ON_Brep::Append( const ON_Brep& b )
{
  int i, j, jcnt;

  const int vcount0  = m_V.Count();
  const int ecount0  = m_E.Count();
  const int fcount0  = m_F.Count();
  const int tcount0  = m_T.Count();
  const int lcount0  = m_L.Count();
  const int c2count0 = m_C2.Count();
  const int c3count0 = m_C3.Count();
  const int scount0  = m_S.Count();

  const int vcount1  = b.m_V.Count();
  const int ecount1  = b.m_E.Count();
  const int fcount1  = b.m_F.Count();
  const int tcount1  = b.m_T.Count();
  const int lcount1  = b.m_L.Count();
  const int c2count1 = b.m_C2.Count();
  const int c3count1 = b.m_C3.Count();
  const int scount1  = b.m_S.Count();

  // need to duplicate geometry
  ON_Object* obj;
  ON_Curve* c;
  ON_Surface* s;
  for ( i = 0; i < scount1; i++ ) {
    s = b.m_S[i];
    if ( s ) {
      obj = s->Duplicate();
      s = ON_Surface::Cast(obj);
      if ( !s )
        delete obj;
    }
    m_S.Append(s);
  }
  for ( i = 0; i < c2count1; i++ ) {
    c = b.m_C2[i];
    if ( c ) {
      obj = c->Duplicate();
      c = ON_Curve::Cast(obj);
      if ( !c )
        delete obj;
    }
    m_C2.Append(c);
  }
  for ( i = 0; i < c3count1; i++ ) {
    c = b.m_C3[i];
    if ( c ) {
      obj = c->Duplicate();
      c = ON_Curve::Cast(obj);
      if ( !c )
        delete obj;
    }
    m_C3.Append(c);
  }

  // copy topology info
  m_V.Append( b.m_V.Count(), b.m_V.Array() );
  m_E.Append( b.m_E.Count(), b.m_E.Array() );
  m_F.Append( b.m_F.Count(), b.m_F.Array() );
  m_T.Append( b.m_T.Count(), b.m_T.Array() );
  m_L.Append( b.m_L.Count(), b.m_L.Array() );

  // update indices
  for ( i = 0; i < vcount1; i++ ) {
    ON_BrepVertex& vertex = m_V[vcount0+i];
    if ( vertex.m_vertex_index >= 0 )
      vertex.m_vertex_index += vcount0;
    else
      vertex.m_vertex_index = -1;
    jcnt = vertex.m_ei.Count();
    for ( j = 0; j < jcnt; j++ ) {
      if ( vertex.m_ei[j] >=0 )
        vertex.m_ei[j] += ecount0;
    }
  }

  for ( i = 0; i < ecount1; i++ ) {
    ON_BrepEdge& edge = m_E[ecount0+i];
    if ( edge.m_edge_index >= 0 )
      edge.m_edge_index += ecount0;
    else
      edge.m_edge_index = -1;
    if ( edge.m_c3i >= 0 ) 
      edge.m_c3i += c3count0;
    if ( edge.m_vi[0] >= 0 )
      edge.m_vi[0] += vcount0;
    if ( edge.m_vi[1] >= 0 )
      edge.m_vi[1] += vcount0;
    jcnt = edge.m_ti.Count();
    for ( j = 0; j < jcnt; j++ ) {
      if ( edge.m_ti[j] >= 0 )
        edge.m_ti[j] += tcount0;
    }
    edge.m_brep = this;
    if (edge.m_c3i >= 0)
      edge.m_curve = m_C3[edge.m_c3i];
    else edge.m_curve = NULL;

  }

  for ( i = 0; i < tcount1; i++ ) {
    ON_BrepTrim& trim = m_T[tcount0+i];
    if ( trim.m_trim_index == i )
      trim.m_trim_index = tcount0+i;
    else
      trim.m_trim_index = -1;
    if ( trim.m_c2i >= 0 )
      trim.m_c2i += c2count0;
    if ( trim.m_ei >= 0 )
      trim.m_ei += ecount0;
    if ( trim.m_vi[0] >= 0 )
      trim.m_vi[0] += vcount0;
    if ( trim.m_vi[1] >= 0 )
      trim.m_vi[1] += vcount0;
    if ( trim.m_li >= 0 )
      trim.m_li += lcount0;
  }

  for ( i = 0; i < lcount1; i++ ) {
    ON_BrepLoop& loop = m_L[lcount0+i];
    if ( loop.m_loop_index >= 0 )
      loop.m_loop_index += lcount0;
    else
      loop.m_loop_index = -1;
    jcnt = loop.m_ti.Count();
    for ( j = 0; j < jcnt; j++ ) {
      if ( loop.m_ti[j] >= 0)
        loop.m_ti[j] += tcount0;
    }
    if ( loop.m_fi >= 0 )
      loop.m_fi += fcount0;
  }

  for ( i = 0; i < fcount1; i++ ) {
    ON_BrepFace& face = m_F[fcount0+i];
    if ( face.m_face_index >= 0 )
      face.m_face_index += fcount0;
    else
      face.m_face_index = -1;
    jcnt = face.m_li.Count();
    for ( j = 0; j < jcnt; j++ ) {
      if ( face.m_li[j] >= 0 )
        face.m_li[j] += lcount0;
    }
    if ( face.m_si >= 0 )
      face.m_si += scount0;
    face.m_brep = this;
    face.m_surface = m_S[face.m_si];
  }
}

ON_BrepLoop& 
ON_Brep::NewLoop( ON_BrepLoop::TYPE looptype )
{
  int li = m_L.Count();
  m_L.Reserve(li+1);
  m_L.SetCount(li+1);
  ON_BrepLoop& loop =  m_L.Array()[li];
  loop.m_loop_index = li;
  loop.m_type = looptype;
  return loop;
}

ON_BrepLoop& 
ON_Brep::NewLoop( ON_BrepLoop::TYPE looptype, ON_BrepFace& face )
{
  ON_BrepLoop& loop = NewLoop( looptype );
  loop.m_fi = face.m_face_index;
  face.m_li.Append(loop.m_loop_index);
  return loop;
}

ON_BrepFace& 
ON_Brep::NewFace( int si )
{
  int fi = m_F.Count();
  m_F.Reserve(fi+1);
  m_F.SetCount(fi+1);
  ON_BrepFace& face = m_F.Array()[fi];
  face.m_face_index = fi;
  face.m_si = si;
  face.m_brep = this;
  return face;
}

BOOL
ON_Brep::SetTrimIsoFlags()
{
  BOOL rc = TRUE;
  int fi;
  const int fcnt = m_F.Count();
  for ( fi = 0; fi < fcnt; fi++ ) {
    if ( !SetTrimIsoFlags( m_F[fi] ) )
      rc = FALSE;
  }
  return rc;
}

BOOL
ON_Brep::SetTrimIsoFlags( ON_BrepFace& face )
{
  BOOL rc = TRUE;
  int fli;
  const int face_loop_count = face.m_li.Count();
  for ( fli = 0; fli < face_loop_count; fli++ ) {
    if ( !SetTrimIsoFlags( m_L[face.m_li[fli]] ) )
      rc = FALSE;
  }
  return rc;
}

BOOL
ON_Brep::SetTrimIsoFlags( ON_BrepLoop& loop )
{
  BOOL rc = TRUE;
  int lti;
  const int loop_trim_count = loop.m_ti.Count();
  for ( lti = 0; lti < loop_trim_count; lti++ ) {
    if ( !SetTrimIsoFlags( m_T[loop.m_ti[lti]] ) )
      rc = FALSE;
  }
  return rc;
}

BOOL
ON_Brep::SetTrimIsoFlags( ON_BrepTrim& trim )
{
  BOOL rc = FALSE;
  const ON_Surface* pS = SurfaceOf( trim );
  const ON_Curve* pC = TrimCurveOf( trim );
  if ( pC && pS ) {
    trim.m_iso = pS->IsIsoparametric( *pC );
    rc = TRUE;
  }
  return rc;
}

BOOL
ON_Brep::SetTrimTypeFlags( BOOL bLazy )
{
  BOOL rc = TRUE;
  int fi;
  const int fcnt = m_F.Count();
  for ( fi = 0; fi < fcnt; fi++ ) {
    if ( !SetTrimTypeFlags( m_F[fi], bLazy ) )
      rc = FALSE;
  }
  return rc;
}

BOOL
ON_Brep::SetTrimTypeFlags( ON_BrepFace& face, BOOL bLazy )
{
  BOOL rc = TRUE;
  int fli;
  const int face_loop_count = face.m_li.Count();
  for ( fli = 0; fli < face_loop_count; fli++ ) {
    if ( !SetTrimTypeFlags( m_L[face.m_li[fli]], bLazy ) )
      rc = FALSE;
  }
  return rc;
}

BOOL
ON_Brep::SetTrimTypeFlags( ON_BrepLoop& loop, BOOL bLazy )
{
  BOOL rc = TRUE;
  int lti;
  const int loop_trim_count = loop.m_ti.Count();
  for ( lti = 0; lti < loop_trim_count; lti++ ) {
    if ( !SetTrimTypeFlags( m_T[loop.m_ti[lti]], bLazy ) )
      rc = FALSE;
  }
  return rc;
}

BOOL
ON_Brep::SetTrimTypeFlags( ON_BrepTrim& trim, BOOL bLazy )
{
  int lti, twin_ti;
  BOOL rc = bLazy;

  if ( !bLazy )
    trim.m_type = ON_BrepTrim::unknown;
  if ( trim.m_type == ON_BrepTrim::unknown ) {
    if ( trim.m_ei < 0 ) {
      trim.m_type = ON_BrepTrim::singular;
      rc = TRUE;
    }
    else if ( trim.m_ei >= 0 && trim.m_ei < m_E.Count() ) {
      const ON_BrepEdge& edge = m_E[trim.m_ei];
      if ( edge.m_ti.Count() == 1 && edge.m_ti[0] == trim.m_trim_index ) {
        trim.m_type = ON_BrepTrim::boundary;
        rc = TRUE;
      }
      else if ( edge.m_ti.Count() > 1 ) {
        if ( trim.m_li >= 0 && trim.m_li < m_L.Count() ) {
          // trim is in a valid loop;
          const ON_BrepLoop& loop = m_L[trim.m_li];
          if ( loop.m_fi >= 0 && loop.m_fi < m_F.Count() ) {
            rc = TRUE;
            trim.m_type = ON_BrepTrim::mated;
            // check for seam
            for ( lti = 0; lti < loop.m_ti.Count(); lti++ ) {
              twin_ti = loop.m_ti[lti];
              if ( twin_ti != trim.m_trim_index && twin_ti >= 0 && twin_ti < m_T.Count() ) {
                if ( m_T[twin_ti].m_ei == trim.m_ei ) {
                  trim.m_type = ON_BrepTrim::seam;
                  break;
                }
              }
            }
          }
        }
      }
    }
  }
  
  return rc;
}

ON_BrepLoop::TYPE 
ON_Brep::ComputeLoopType( const ON_BrepLoop& loop ) const
{
  ON_BrepLoop::TYPE loop_type = ON_BrepLoop::unknown;

  /*
  int ugap_count = 0;
  int vgap_count = 0;
  double d, utol0, vtol0, loop_start_utol, loop_start_vtol;
  ON_3dPoint p0, p1, loop_start;
  ON_Interval srf_domain[2];
  if ( loop.m_fi >= 0 && loop.m_fi < m_F.Count() ) {
    const ON_BrepFace& face = m_F[loop.m_fi];
    if ( face.m_si >= 0 && face.m_si < m_S.Count() ) {
      ON_Surface* srf = m_S[face.m_si];
      srf_domain[0] = srf->Domain(0);
      srf_domain[1] = srf->Domain(1);
    }
  }
  const ON_2dPoint basePt( srf_domain[0].ParameterAt(0.5), srf_domain[1].ParameterAt(0.5) );

  const int trim_count = loop.m_ti.Count();
  for ( lti = 0; lti < trim_count; lti++ ) {
    ti = loop.m_ti[lti];
    const ON_BrepTrim& trim = m_T[ti];
    ON_Curve* c2 = m_C2[trim.m_c2i];
    p0 = c2->PointAtStart();
    u_tol0 = trim.m_tolerance[0];
    v_tol0 = trim.m_tolerance[1];
    if ( !lti ) {
      loop_start = p0;
      loop_start_utol = trim.m_tolerance[0];
      loop_start_vtol = trim.m_tolerance[1];
    }
    else {
      d = fabs(p0.x-p1.x);
      if ( d > utol0 + trim.m_tolerance[0] )
        ugap_count++;
      d = fabs(p0.y-p1.y);
      if ( d > vtol0 + trim.m_tolerance[1] )
        vgap_count++;
    }
    p1 = c2->PointAtEnd();
  }
  */

  return loop_type;
}


BOOL
ON_Brep::IsValidTrim( int trim_index ) const
{
  if ( trim_index < 0 || trim_index >= m_T.Count() )
    return FALSE;
  const ON_BrepTrim& trim = m_T[trim_index];
  if ( trim.m_trim_index != trim_index )
    return FALSE;
  if ( !trim.IsValid() )
    return FALSE;
  if ( trim.m_c2i < 0 || trim.m_c2i >= m_C2.Count() )
    return FALSE;
  const ON_Curve* pC = m_C2[trim.m_c2i];
  if ( !pC )
    return FALSE;
  if ( pC->Dimension() != 2 )
    return FALSE;
  double t0, t1;
  if ( !pC->GetDomain(&t0,&t1) )
    return FALSE;
  if ( t0 != trim.m_t[0] || t1 != trim.m_t[1] )
    return FALSE;
  int vi0 = trim.m_vi[0];
  int vi1 = trim.m_vi[1];
  if ( vi0 < 0 || vi0 >= m_V.Count() )
    return FALSE;
  if ( vi1 < 0 || vi1 >= m_V.Count() )
    return FALSE;
  const int ei = trim.m_ei;
  if ( trim.m_type == ON_BrepTrim::singular ) {
    // singular trim - no edge and 3d v0 = 3d v1
    if ( ei != -1 )
      return FALSE;
    if ( vi0 != vi1 )
      return FALSE;
    if ( pC->IsClosed() )
      return FALSE;
  }
  else {
    // non-singular trim should have valid edge
    if ( ei < 0 || ei >= m_E.Count() )
      return FALSE;
    const ON_BrepEdge& edge = m_E[ei];
    if ( edge.m_vi[trim.m_bRev3d?1:0] != vi0 )
      return FALSE;
    if ( edge.m_vi[trim.m_bRev3d?0:1] != vi1 )
      return FALSE;
    if ( pC->IsClosed() ) {
      // (open 2d trims can still have vi0 = vi1 on closed surfaces)
      if ( vi0 != vi1 )
        return FALSE;
    }
    BOOL bFoundIt = FALSE;
    int i;
    for ( i = 0; i < edge.m_ti.Count() && !bFoundIt; i++ ) {
      bFoundIt  = ( edge.m_ti[i] == trim_index );
    }
    if ( !bFoundIt )
      return FALSE;
  }
  if ( trim.m_li < 0 || trim.m_li >= m_L.Count() )
    return FALSE;
  return TRUE;
}

BOOL
ON_Brep::IsValidLoop( int loop_index ) const
{
  if ( loop_index < 0 || loop_index >= m_L.Count() )
    return FALSE;
  const ON_BrepLoop& loop = m_L[loop_index];
  if ( loop.m_loop_index != loop_index )
    return FALSE;
  if ( loop.m_fi < 0 || loop.m_fi >= m_F.Count() )
    return FALSE;
  const int loop_trim_count = loop.m_ti.Count();
  if ( loop_trim_count <= 0 )
    return FALSE;

  if ( loop.m_type != ON_BrepLoop::outer && loop.m_type != ON_BrepLoop::inner )
    return FALSE;

  // make sure trims are valid
  int i, lti, ti;
  for ( lti = 0; lti < loop_trim_count; lti++ ) {
    ti = loop.m_ti[lti];
    for ( i = 0; i < lti; i++ ) {
      if ( loop.m_ti[i] == ti )
        return FALSE;
    }
    if ( !IsValidTrim( ti ) )
      return FALSE;
    if ( m_T[ti].m_li != loop_index )
      return FALSE;
  }

  // make sure end of trims jibe
  int ci0, ci1;
  ON_3dPoint P0, P1;
  const ON_Curve *pC0, *pC1;
  for ( lti = 0; lti < loop_trim_count; lti++ ) {
    ci0 = m_T[loop.m_ti[lti]].m_c2i;
    ci1 = m_T[loop.m_ti[(lti+1)%loop_trim_count]].m_c2i;
    pC0 = m_C2[ci0];
    pC1 = m_C2[ci1];
    P0 = pC0->PointAtEnd(); // end of this 2d trim
    P1 = pC1->PointAtStart(); // start of next 2d trim
    if ( P0 != P1 )
      return FALSE;
  }

  return TRUE; 
}


BOOL
ON_Brep::IsValidFace( int face_index ) const
{
  if ( face_index < 0 || face_index >= m_F.Count() )
    return FALSE;
  const ON_BrepFace& face = m_F[face_index];
  if ( face.m_face_index != face_index )
    return FALSE;
  if ( face.m_brep != this )
    return FALSE;

  const int face_loop_count = face.m_li.Count();
  if ( face_loop_count <= 0 )
    return FALSE;

  int i, fli, li;
  for ( fli = 0; fli < face_loop_count; fli++ ) {
    li = face.m_li[fli];
    for ( i = 0; i < fli; i++ ) {
      if ( face.m_li[i] == li )
        return FALSE;
    }
    if ( !IsValidLoop( li ) )
      return FALSE;
    const ON_BrepLoop& loop = m_L[li];
    if ( loop.m_fi != face_index )
      return FALSE;
    if ( fli == 0 ) {
      if ( loop.m_type != ON_BrepLoop::outer )
        return FALSE;
    }
    else {
      if ( loop.m_type != ON_BrepLoop::inner )
        return FALSE;
    }
  }

  const int si = face.m_si;
  if ( si < 0 || si >= m_S.Count() )
    return FALSE;
  if ( !m_S[si] )
    return FALSE;

  return TRUE; 
}

BOOL
ON_Brep::IsValidEdge( int edge_index ) const
{
  if ( edge_index < 0 || edge_index >= m_E.Count() )
    return FALSE;
  const ON_BrepEdge& edge = m_E[edge_index];
  if ( edge.m_brep != this )
    return FALSE;
  if ( !edge.IsValid() )
    return FALSE;
  if ( edge.m_edge_index != edge_index )
    return FALSE;
  const int ci = edge.m_c3i;
  if ( ci < 0 || ci >= m_C3.Count() )
    return FALSE;
  if ( m_C3[ci] != edge.m_curve )
    return FALSE;
  double t0, t1;
  if ( !edge.GetDomain( &t0, &t1 ) )
    return FALSE;
  const int vi0 = edge.m_vi[0];
  const int vi1 = edge.m_vi[1];
  if ( vi0 < 0 || vi0 >= m_V.Count() )
    return FALSE;
  if ( vi1 < 0 || vi1 >= m_V.Count() )
    return FALSE;
  int evi;
  for ( evi = 0; evi < 2; evi++ ) {
    const ON_BrepVertex& vertex = m_V[edge.m_vi[evi]];
    const int vertex_edge_count = vertex.m_ei.Count();
    BOOL bFoundIt = FALSE;
    int vei;
    for ( vei = 0; vei < vertex_edge_count && !bFoundIt; vei++ ) {
      bFoundIt = (vertex.m_ei[vei] == edge_index);
    }
    if ( !bFoundIt )
      return FALSE;
  }

  if ( edge.IsClosed() ) {
    if ( vi0 != vi1 )
      return FALSE;
  }
  else {
    if ( vi0 == vi1 )
      return FALSE;
  }

  const int edge_trim_count = edge.m_ti.Count();
  if ( edge_trim_count < 0 )
    return FALSE;
  int i, eti, ti;
  for (eti = 0; eti < edge_trim_count; eti++ ) {
    ti = edge.m_ti[eti];
    if ( ti < 0 || ti >= m_T.Count() )
      return FALSE;
    for ( i = 0; i < eti; i++ ) {
      if ( edge.m_ti[i] == ti )
        return FALSE;
    }
    const ON_BrepTrim& trim = m_T[ti];
    if ( trim.m_ei != edge_index )
      return FALSE;
  }

  return TRUE;
}

BOOL
ON_Brep::IsValidVertex( int vertex_index ) const
{
  if ( vertex_index < 0 || vertex_index >= m_V.Count() )
    return FALSE;
  const ON_BrepVertex& vertex = m_V[vertex_index];
  if ( vertex.m_vertex_index != vertex_index )
    return FALSE;

  const int vertex_edge_count = vertex.m_ei.Count();
  int i, vei, ei;
  for ( vei = 0; vei < vertex_edge_count; vei++ ) {
    ei = vertex.m_ei[vei];
    if ( ei < 0 || ei >= m_E.Count() )
      return FALSE;
    const ON_BrepEdge& edge = m_E[ei];
    for ( i = 0; i < vei; i++ ) {
      if ( vertex.m_ei[i] == ei ) {
        // edge should be closed
        if ( edge.m_vi[0] != vertex_index || edge.m_vi[1] != vertex_index )
          return FALSE;
        for (i++; i < vei; i++ ) {
          if ( vertex.m_ei[i] == ei )
            return FALSE;
        }
      }
    }
    if ( edge.m_vi[0] != vertex_index && edge.m_vi[1] != vertex_index )
      return FALSE;
  }
  return TRUE;
}

BOOL
ON_Brep::IsValid() const
{
  const int curve2d_count = m_C2.Count();
  const int curve3d_count = m_C3.Count();
  const int surface_count = m_S.Count();
  const int vertex_count  = m_V.Count();
  const int edge_count    = m_E.Count();
  const int trim_count    = m_T.Count();
  const int loop_count    = m_L.Count();
  const int face_count    = m_F.Count();

  int c2i, c3i, si, vi, ei, fi, ti, li;

  // check element indices match array positions
  for ( vi = 0; vi < vertex_count; vi++ ) {
    if ( m_V[vi].m_vertex_index != vi )
      return FALSE;
  }
  for ( ei = 0; ei < edge_count; ei++ ) {
    if ( m_E[ei].m_edge_index != ei )
      return FALSE;
  }
  for ( ti = 0; ti < trim_count; ti++ ) {
    if ( m_T[ti].m_trim_index != ti )
      return FALSE;
  }
  for ( li = 0; li < loop_count; li++ ) {
    if ( m_L[li].m_loop_index != li )
      return FALSE;
  }
  for ( fi = 0; fi < face_count; fi++ ) {
    if ( m_F[fi].m_face_index != fi )
      return FALSE;
  }

  // check 2d curve geometry
  for ( c2i = 0; c2i < curve2d_count; c2i++ ) {
    if ( !m_C2[c2i] )
      return FALSE;
    if ( !m_C2[c2i]->IsValid() )
      return FALSE;
    if ( m_C2[c2i]->Dimension() != 2 )
      return FALSE;
  }

  // check 3d curve geometry
  for ( c3i = 0; c3i < curve3d_count; c3i++ ) {
    if ( !m_C3[c3i] )
      return FALSE;
    if ( !m_C3[c3i]->IsValid() )
      return FALSE;
    if ( m_C3[c3i]->Dimension() != 3 )
      return FALSE;
  }

  // check 3d surface geometry
  for ( si = 0; si < surface_count; si++ ) {
    if ( !m_S[si] )
      return FALSE;
    if ( !m_S[si]->IsValid() )
      return FALSE;
    int dim = m_S[si]->Dimension();
    if ( dim != 3 )
      return FALSE;
  }

  // check vertices
  for ( vi = 0; vi < vertex_count; vi++ ) {
    if ( !IsValidVertex( vi ) ) {
      return FALSE;
    }
  }

  // check edges
  for ( ei = 0; ei < edge_count; ei++ ) {
    if ( !IsValidEdge( ei ) ) {
      return FALSE;
    }
  }

  // check faces
  for ( fi = 0; fi < face_count; fi++ ) {
    if ( !IsValidFace( fi ) ) {
      return FALSE;
    }
  }

  return TRUE;
}

BOOL ON_Brep::SetEdgeVertex( const int ei, const int evi, const int vi )
{
  if ( ei < 0 || vi < 0 || evi < 0 || evi > 1 )
    return FALSE;
  ON_BrepEdge& edge = m_E[ei];
  if ( edge.m_vi[evi] != vi ) {
    edge.m_vi[evi] = vi;
    ON_BrepVertex& vertex = m_V[vi];
    vertex.m_ei.Append(ei);
  }
  const int trim_count = edge.m_ti.Count();
  int eti, ti, tvi;
  for ( eti = 0; eti < trim_count; eti++ ) {
    ti = edge.m_ti[eti];
    if ( ti < 0 )
      continue;
    ON_BrepTrim& trim = m_T[ti];
    tvi = trim.m_bRev3d ? 1-evi : evi;
    trim.m_vi[tvi] = vi;
  }
  return TRUE;
}

BOOL ON_Brep::HopAcrossEdge( int& ti, int& tvi ) const
{
  // Tf ti is a trim associated with an interior manifold edge,
  // then ti is set to twin.
  int ei, evi, new_ti, new_tvi;
  if ( ti < 0 )
    return FALSE;
  ei = m_T[ti].m_ei;
  if ( ei < 0 )
    return FALSE;
  const ON_BrepEdge& edge = m_E[ei];
  if ( edge.m_ti.Count() < 2 )
    return FALSE;
  evi = (m_T[ti].m_bRev3d) ? 1-tvi : tvi;
  new_ti = edge.m_ti[(edge.m_ti[0] == ti)?1:0];
  if ( new_ti < 0 )
    return FALSE;
  new_tvi = (m_T[new_ti].m_bRev3d) ? 1-evi : evi;
  ti = new_ti;
  tvi = new_tvi;
  return TRUE;
}

BOOL ON_Brep::SetTrimStartVertex( const int ti0, const int vi )
{
  // Do not use NextEdge(), PrevEdge() because they require
  // the information we are in the act of creating.
  if ( ti0 < 0 || vi < 0 )
    return FALSE;
  int next_ti, ti, ei, evi, tvi, counter;

  // Step counter clockwise around vertex until we hit a boundary
  // or we get back to where we started.
  for ( ti = ti0, tvi = 0, counter = 0; ti >= 0 && counter < 512; counter++ ) {
    if ( counter > 0 ) {
      if ( ti == ti0 && tvi == 0 )
        return TRUE; // vertex was interior
    }
    ON_BrepTrim& trim = m_T[ti];
    if ( trim.m_type == ON_BrepTrim::singular ) {
      trim.m_vi[0] = trim.m_vi[1] = vi;
      tvi = 1-tvi;
      next_ti = (tvi) ? NextTrim(ti) : PrevTrim(ti);
      ti = next_ti;
      tvi = 1-tvi;
      if ( ti == ti0 && tvi == 0 )
        return TRUE; // vertex was interior
      if ( m_T[ti].m_type != ON_BrepTrim::singular )
        HopAcrossEdge( ti, tvi ); // OK if hop fails because ti is a boundary
      continue;
    }

    ei = trim.m_ei;
    evi = (trim.m_bRev3d) ? 1-tvi : tvi;
    if ( !SetEdgeVertex( ei, evi, vi ) )
      return FALSE;
    next_ti = (tvi) ? NextTrim(ti) : PrevTrim(ti);
    ti = next_ti;
    tvi = 1-tvi;
    if ( ti < 0 )
      return FALSE; // should not happen

    if ( m_T[ti].m_type == ON_BrepTrim::singular )
      continue;
    ei = m_T[ti].m_ei;
    if ( ei < 0 )
      return FALSE; // should not happen
    evi = (m_T[ti].m_bRev3d) ? 1-tvi : tvi;
    const int edge_trim_count = m_E[ei].m_ti.Count();
    if ( edge_trim_count < 1 )
      break; // should not happen
    if ( edge_trim_count == 1 ) {
      SetEdgeVertex( ei, evi, vi );
      break; // ran into boundary
    }
    if ( !HopAcrossEdge( ti, tvi ) )
      return FALSE;
  }


  // Get ready to step counter clockwise around vertex until
  // we hit a boundary.
  ti = ti0;
  tvi = 0;
  if ( m_T[ti].m_type == ON_BrepTrim::singular ) {
    // back up until we get to a non-singular trim
    while ( m_T[ti].m_type == ON_BrepTrim::singular ) {
      if ( ti != ti0 ) {
        m_T[ti].m_vi[0] = vi;
        m_T[ti].m_vi[1] = vi;
      }
      ti = PrevTrim(ti);
      tvi = 1;
      if ( ti == ti0 )
        break;
    }
    ei = m_T[ti].m_ei;
    if ( ei >= 0 ) {
      evi = (m_T[ti].m_bRev3d) ? 1-tvi : tvi;
      SetEdgeVertex( ei, evi, vi );
    }
  }
  else {
    ei = m_T[ti].m_ei;
  }
  if ( ei < 0 ) {
    // did the best we could - return TRUE so setter keeps going
    // but the fact we are here means the brep is bogus.
    return TRUE;
  }
  if ( m_E[ei].m_ti.Count() < 2 )
    return TRUE; // ti0 is a boundary - we're done.
  if ( !HopAcrossEdge( ti, tvi ) )
    return FALSE;
  next_ti = (tvi) ? NextTrim(ti) : PrevTrim(ti);
  if ( next_ti < 0 )
    return FALSE;
  ti = next_ti;
  tvi = 1-tvi;
  if ( m_T[ti].m_type != ON_BrepTrim::singular ) {
    ei = m_T[ti].m_ei;
    if ( ei < 0 )
      return FALSE;
    if ( m_E[ei].m_ti.Count() == 1 ) {
      evi = (m_T[ti].m_bRev3d)? 1-tvi : tvi;
      SetEdgeVertex( ei, evi, vi );
      return TRUE;
    }
    if ( !HopAcrossEdge( ti, tvi ) )
      return FALSE;
  }

  const int ti1 = ti;
  const int tvi1 = tvi;
  
  for ( ti = ti1, tvi = tvi1, counter = 0; ti >= 0 && counter < 512; counter++ ) {
    if ( counter > 0 ) {
      if ( ti == ti1 && tvi == tvi1 )
        return FALSE; // vertex is not interior - so this should not happen
    }
    ON_BrepTrim& trim = m_T[ti];
    if ( trim.m_type == ON_BrepTrim::singular ) {
      trim.m_vi[0] = trim.m_vi[1] = vi;
      tvi = 1-tvi;
      next_ti = (tvi) ? NextTrim(ti) : PrevTrim(ti);
      ti = next_ti;
      tvi = 1-tvi;
      if ( ti == ti1 && tvi == tvi1 )
        return FALSE; // vertex is not interior - so this should not happen
      if ( m_T[ti].m_type != ON_BrepTrim::singular )
        HopAcrossEdge( ti, tvi );  // OK if hop fails because ti is a boundary
      continue;
    }

    ei = trim.m_ei;
    evi = (trim.m_bRev3d) ? 1-tvi : tvi;
    if ( !SetEdgeVertex( ei, evi, vi ) )
      return FALSE;
    next_ti = (tvi) ? NextTrim(ti) : PrevTrim(ti);
    ti = next_ti;
    tvi = 1-tvi;
    if ( ti < 0 )
      return FALSE; // should not happen
    
    if ( m_T[ti].m_type == ON_BrepTrim::singular )
      continue;
    ei = m_T[ti].m_ei;
    if ( ei < 0 )
      return FALSE; // should not happen
    evi = (m_T[ti].m_bRev3d) ? 1-tvi : tvi;
    const int edge_trim_count = m_E[ei].m_ti.Count();
    if ( edge_trim_count < 1 )
      break; // should not happen
    if ( edge_trim_count == 1 ) {
      SetEdgeVertex( ei, evi, vi );
      return TRUE; // ran into boundary - expected
    }
    if ( !HopAcrossEdge( ti, tvi ) )
      return FALSE;
  }

  return FALSE; // should have exited by hitting "expected" boundary ~10 lines above
}

void ON_Brep::SetLoopVertices( const int li )
{
  ON_BrepLoop& loop = m_L[li];
  const int loop_trim_count = loop.m_ti.Count();
  int lti;
  for ( lti = 0; lti < loop_trim_count; lti++ ) {
    const int ti = loop.m_ti[lti];
    ON_BrepTrim& trim = m_T[ti];
    int vi = trim.m_vi[0];
    if ( vi >= 0 )
      continue;
    ON_BrepVertex& v = NewVertex();
    SetTrimStartVertex( ti, v.m_vertex_index );
  }
}

void ON_Brep::ClearTrimVertices()
{
  int ti;
  const int tcnt = m_T.Count();
  for ( ti = 0; ti < tcnt; ti++ ) {
    ON_BrepTrim& trim = m_T[ti];
    trim.m_vi[0] = -1;
    trim.m_vi[1] = -1;
  }
}

void ON_Brep::ClearEdgeVertices()
{
  int ei;
  const int ecnt = m_E.Count();
  for ( ei = 0; ei < ecnt; ei++ ) {
    ON_BrepEdge& edge = m_E[ei];
    edge.m_vi[0] = -1;
    edge.m_vi[1] = -1;
  }
}

int ON_Brep::NextTrim(int ti) const
{
  const ON_BrepTrim& trim = m_T[ti];
  const int li = trim.m_li;
  const ON_BrepLoop& loop = m_L[li];
  const int trim_count = loop.m_ti.Count();
  int lti;
  for ( lti = 0; loop.m_ti[lti] != ti && lti < trim_count; lti++)
    ;/* empty for*/
  if ( lti < 0 || lti >= trim_count )
    return -1;
  return loop.m_ti[(lti+1)%trim_count];
}

int ON_Brep::PrevTrim(int ti) const
{
  const ON_BrepTrim& trim = m_T[ti];
  const int li = trim.m_li;
  const ON_BrepLoop& loop = m_L[li];
  const int trim_count = loop.m_ti.Count();
  int lti;
  for ( lti = 0; loop.m_ti[lti] != ti && lti < trim_count; lti++)
    ;/* empty for*/
  if ( lti < 0 || lti >= trim_count )
    return -1;
  return loop.m_ti[(lti+trim_count-1)%trim_count];
}

int ON_Brep::NextEdge(int ei, int endi, int* next_endi ) const
{
  const ON_BrepEdge& edge = m_E[ei];
  const int vi = edge.m_vi[endi];
  const ON_BrepVertex& vertex = m_V[vi];
  const int edge_count = vertex.m_ei.Count();
  int vei;
  if ( edge_count < 2 )
    return -1;
  if ( next_endi )
    *next_endi = 0;
  for ( vei = 0; vertex.m_ei[vei] != ei && vei < edge_count; vei++)
    ;/* empty for*/
  if ( edge.m_vi[0] == edge.m_vi[1]  && endi ) {
    // get next occurance of edge index
    //
    // On closed edges, edge.m_vi[0] = edge.m_vi[1] and edge.edge_index 
    // appears TWICE in vertex.m_ei[].  The first occurance of edge.edge_index
    // in vertex.m_ei[] corresponds to edge.m_vi[0].  The second occurance
    // of edge.edge_index in vertex.m_ei[] corresponds to edge.m_vi[1].
    vei++;
    while ( vei < edge_count && vertex.m_ei[vei] != ei )
      vei++;
  }
  if ( vei < 0 || vei >= edge_count )
    return -1;
  vei = (vei+1)%edge_count;
  const int next_ei = vertex.m_ei[vei];
  if ( next_endi ) {
    if ( m_E[next_ei].m_vi[0] == m_E[next_ei].m_vi[1] ) {
      *next_endi = 1;
      for ( vei++; vei < edge_count; vei++ ) {
        if ( vertex.m_ei[vei] == next_ei ) {
          *next_endi = 0;
          break;
        }
      }
    }
    else if ( m_E[next_ei].m_vi[1] == vi )
      *next_endi = 1;
  }
  return next_ei;
}

int ON_Brep::PrevEdge(int ei, int endi, int* prev_endi ) const
{
  const ON_BrepEdge& edge = m_E[ei];
  const int vi = edge.m_vi[endi];
  const ON_BrepVertex& vertex = m_V[vi];
  const int edge_count = vertex.m_ei.Count();
  if ( edge_count < 2 )
    return -1;
  int vei;
  if ( prev_endi )
    *prev_endi = 0;
  for ( vei = 0; vertex.m_ei[vei] != ei && vei < edge_count; vei++)
    ;/* empty for*/
  if ( edge.m_vi[0] == edge.m_vi[1] && endi ) {
    // get next occurance of edge index
    //
    // On closed edges, edge.m_vi[0] = edge.m_vi[1] and edge.edge_index 
    // appears TWICE in vertex.m_ei[].  The first occurance of edge.edge_index
    // in vertex.m_ei[] corresponds to edge.m_vi[0].  The second occurance
    // of edge.edge_index in vertex.m_ei[] corresponds to edge.m_vi[1].
    vei++;
    while ( vei < edge_count && vertex.m_ei[vei] != ei )
      vei++;
  }
  if ( vei < 0 || vei >= edge_count )
    return -1;
  vei = (vei+edge_count-1)%edge_count;
  const int prev_ei = vertex.m_ei[(vei+edge_count-1)%edge_count];
  if ( prev_endi ) {
    if ( m_E[prev_ei].m_vi[0] == m_E[prev_ei].m_vi[1] ) {
      *prev_endi = 1;
      for ( vei++; vei < edge_count; vei++ ) {
        if ( vertex.m_ei[vei] == prev_ei ) {
          *prev_endi = 0;
          break;
        }
      }
    }
    else if ( m_E[prev_ei].m_vi[1] == vi ) {
      *prev_endi = 1;
    }
  }
  return prev_ei;
}


void ON_Brep::SetVertices(void)
{
  const int face_count = m_F.Count();
  int fi;
  
  ClearEdgeVertices();
  ClearTrimVertices();
  m_V.Empty();
  m_V.Shrink();
  fi = m_E.Count() - m_F.Count() + 8;
  if ( fi < 32 )
    fi = 32;
  m_V.Reserve( fi );
  for ( fi = 0; fi < face_count; fi++ ) {
    ON_BrepFace& face = m_F[fi];
    const int loop_count = face.m_li.Count();
    int fli;
    for ( fli = 0; fli < loop_count; fli++ ) {
      SetLoopVertices( face.m_li[fli] );
    }
  }

  // average edges' end location to get vertex location
  const int vertex_count = m_V.Count();
  int vi;
  ON_3dPoint VP, EP;
  for ( vi = 0; vi < vertex_count; vi++ ) {
    VP.Zero();
    double d = 0.0;
    ON_BrepVertex& vertex = m_V[vi];
    const int edge_count = vertex.m_ei.Count();
    int vei;
    for ( vei = 0; vei < edge_count; vei++ ) {
      const int ei = vertex.m_ei[vei];
      if ( ei < 0 ) 
        continue;
      const ON_BrepEdge& edge = m_E[ei];
      if ( edge.m_c3i < 0 )
        continue;
      const ON_Curve* pC = EdgeCurveOf(edge);
      if ( !pC )
        continue;
      int evi;
      if ( edge.m_vi[0] == vi )
        evi = 0;
      else if ( edge.m_vi[1] == vi )
        evi = 1;
      else
        continue;
      EP = pC->PointAt(pC->Domain()[evi]);
      VP.x += EP.x;
      VP.y += EP.y;
      VP.z += EP.z;
      d += 1.0;
    }
    if ( d > 0.0 ) {
      d = 1.0/d;
      vertex.point = d*VP;
    }
  }
}

BOOL ON_Brep::Write( ON_BinaryArchive& file ) const
{
  BOOL rc = file.Write3dmChunkVersion(3,0); // serialization version

  // 2d curves
  if (rc) rc = m_C2.Write(file);

  // 3d curves
  if (rc) rc = m_C3.Write(file);

  // untrimmed surfaces
  if (rc) rc = m_S.Write(file);

  // vertices
  if (rc) rc = m_V.Write(file);

  // edges
  if (rc) rc = m_E.Write(file);

  // trims
  if (rc) rc = m_T.Write(file);

  // loops
  if (rc) rc = m_L.Write(file);

  // faces
  if (rc) rc = m_F.Write(file);

  // bounding box
  if (rc) rc = file.WritePoint( m_bbox.m_min );
  if (rc) rc = file.WritePoint( m_bbox.m_max );

  return TRUE;
}

BOOL ON_Brep::Read( ON_BinaryArchive& file )
{
  int i;
  int C3_count = 0;
  int S_count = 0;
  int major_version = 0;
  int minor_version = 0;
  BOOL rc = file.Read3dmChunkVersion( &major_version, &minor_version );
  if ( rc && major_version == 2 ) {
    rc = ReadOld200(file,minor_version); // legacy trimmed face
  }
  else if ( rc && major_version == 3 ) {
    // 2d curves
    if (rc) rc = m_C2.Read(file);

    // 3d curves
    if (rc) rc = m_C3.Read(file);
    C3_count = m_C3.Count();

    // untrimmed surfaces
    if (rc) rc = m_S.Read(file);
    S_count = m_C3.Count();

    // vertices
    if (rc) rc = m_V.Read(file);

    // edges
    if (rc) rc = m_E.Read(file);
    if (rc) {
      for ( i = 0; i < m_E.Count(); i++ ) {
        ON_BrepEdge& e = m_E[i];
        e.m_brep = this;
        if ( e.m_c3i >= 0 && e.m_c3i < C3_count )
          e.m_curve = m_C3[e.m_c3i];
      }
    }

    // trims
    if (rc) rc = m_T.Read(file);

    // loops
    if (rc) rc = m_L.Read(file);

    // faces
    if (rc) rc = m_F.Read(file);
    if (rc) {
      for ( i = 0; i < m_F.Count(); i++ ) {
        ON_BrepFace& f = m_F[i];
        f.m_brep = this;
        if ( f.m_si >= 0 && f.m_si < S_count )
          f.m_surface = m_S[f.m_si];
      }
    }

    // bounding box
    if (rc) rc = file.ReadPoint( m_bbox.m_min );
    if (rc) rc = file.ReadPoint( m_bbox.m_max );
  }
  return rc;
}

void ON_BrepTrim::m__legacy_flags_Set(int gcon, int mono)
{
  m__legacy_flags = 0;
  switch(gcon)
  {
  case -1:
    m__legacy_flags |= 1;
    break;
  case 0:
    m__legacy_flags |= 2;
    break;
  case 1:
    m__legacy_flags |= 3;
    break;
  case 2:
    m__legacy_flags |= 4;
    break;
  }
  if (mono)
    m__legacy_flags |= 8;
  else
    m__legacy_flags |= 16;
}

BOOL ON_BrepTrim::m__legacy_flags_Get(int* gcon, int* mono) const
{
  if ( gcon ) {
    switch ( m__legacy_flags & 7 )
    {
    case 1:
      *gcon = -1;
      break;
    case 2:
      *gcon = 0;
      break;
    case 3:
      *gcon = 1;
      break;
    case 4:
      *gcon = 2;
      break;
    default:
      *gcon = -1;
    }
  }
  if ( mono ) {
    if ( 0 != (m__legacy_flags&8) )
      *mono = 1;
    else
      *mono = 0;
  }
  return m__legacy_flags ? TRUE : FALSE;
}

BOOL ON_Brep::ReadOld200( ON_BinaryArchive& file, int minor_version )
{
  BOOL rc = TRUE;

  // read legacy trimmed surface collection from Rhino 2.0

  int face_count = 0;
  int edge_count = 0;
  int loop_count = 0;
  int trim_count = 0;
  int outer_flag = 0;

  ON_BoundingBox bnd_2d_bbox;
  int i, fi, fbi, fbcnt, bti, btcnt, twin_index;
  int ftype_flag, btype_flag, gcon_flag, mono_flag;
  char b;

  if (rc) rc = file.ReadInt( &face_count );
  if (rc) rc = file.ReadInt( &edge_count );
  if (rc) rc = file.ReadInt( &loop_count );
  if (rc) rc = file.ReadInt( &trim_count );

  if ( face_count < 1 || edge_count < 1 || loop_count < 1 || trim_count < 1 )
    rc = FALSE;

  if (rc) rc = file.ReadInt( &outer_flag );
  if (rc) rc = file.ReadPoint( m_bbox.m_min );
  if (rc) rc = file.ReadPoint( m_bbox.m_max );

  // 2d curves
  m_C2.Reserve(trim_count);
  for ( i = 0; rc && i < trim_count; i++ ) {
    ON_PolyCurve* curve = new ON_PolyCurve();
    rc = curve->Read( file );
    if ( curve->Count() == 1 ) {
      m_C2.Append( curve->HarvestSegment(0) );
      delete curve;
    }
    else
      m_C2.Append( curve );
  }
  const int c2_count = m_C2.Count();

  // 3d curves
  m_C3.Reserve(edge_count);
  for ( i = 0; rc && i < edge_count; i++ ) {
    ON_PolyCurve* curve = new ON_PolyCurve();
    rc = curve->Read( file );
    if ( curve->Count() == 1 ) {
      m_C3.Append( curve->HarvestSegment(0) );
      delete curve;
    }
    else
      m_C3.Append( curve );
  }
  const int c3_count = m_C3.Count();

  m_E.Reserve(c3_count);
  for ( i = 0; i < c3_count && rc; i++ ) {
    ON_BrepEdge& edge = m_E.AppendNew();
    edge.m_edge_index = i;
    edge.m_c3i = i;
    edge.m_brep = this;
  }

  // 3d surfaces
  m_S.Reserve(face_count);
  for ( i = 0; rc && i < face_count; i++ ) {
    ON_NurbsSurface* surface = new ON_NurbsSurface();
    rc = surface->Read( file );
    m_S.Append( surface );
  }

  ON_SimpleArray<int> te_index(trim_count);
  ON_SimpleArray<int> te_twin_index(trim_count);

  m_F.Reserve(face_count);
  m_L.Reserve(loop_count);
  m_T.Reserve(trim_count);

  for ( fi = 0; rc && fi < face_count; fi++ ) {
    ftype_flag = 0;
    fbcnt = 0;
    ON_BrepFace& f = m_F.AppendNew();
    f.m_brep = this;
    f.m_face_index = m_F.Count()-1;
    f.m_si = fi;
    if (rc) rc = file.ReadInt( &i ); // legacy face index
    if (rc) rc = file.ReadInt( &f.m_material_index );
    if (rc) rc = file.ReadInt( &f.m_bRev );
    if (rc) rc = file.ReadInt( &ftype_flag );
    if (rc) rc = file.ReadPoint( f.m_bbox.m_min );
    if (rc) rc = file.ReadPoint( f.m_bbox.m_max );
    if (rc) rc = file.ReadInt( &fbcnt);
    if (fbcnt < 1 )
      rc = FALSE;
    for ( fbi = 0; rc && fbi < fbcnt; fbi++ ) {
      btype_flag = 0;
      ON_BrepLoop& bnd = m_L.AppendNew();
      bnd.m_loop_index = m_L.Count()-1;
      bnd.m_fi = f.m_face_index;
      f.m_li.Append(bnd.m_loop_index);
      if (rc) rc = file.ReadInt( &i ); // legacy loop index
      if (rc) rc = file.ReadInt( &btype_flag );
      switch (btype_flag)
      {
      case 0:
        bnd.m_type = ON_BrepLoop::outer;
        break;
      case 1:
        bnd.m_type = ON_BrepLoop::inner;
        break;
      case -1:
        bnd.m_type = ON_BrepLoop::slit;
        break;
      default:
        bnd.m_type = ON_BrepLoop::unknown;
        break;
      }
      if (rc) rc = file.ReadDouble( 2, &bnd_2d_bbox.m_min.x );
      if (rc) rc = file.ReadDouble( 2, &bnd_2d_bbox.m_max.x );
      if (rc) rc = file.ReadInt( &btcnt );
      if (btcnt < 1 )
        rc = FALSE;
      for ( bti = 0; rc && bti < btcnt; bti++ ) {
        ON_BrepTrim& trim = m_T.AppendNew();
        trim.m_trim_index = m_T.Count()-1;
        trim.m_li = bnd.m_loop_index;
        bnd.m_ti.Append(trim.m_trim_index);
        te_index.Append(trim.m_trim_index);
        if (rc) rc = file.ReadInt( &i ); // legacy trim index
        if ( trim.m_trim_index != i )
        {
          ON_ERROR("ON_Brep::ReadOld200 - trim.m_trim_index out of synch.");
          //rc = FALSE;
          //break;
        }
        if (rc) rc = file.ReadInt( &twin_index );
        te_twin_index.Append(twin_index);
        if (rc) rc = file.ReadChar( &b ); // true if legacy trim managed 3d edge
        if (rc) rc = file.ReadInt( &trim.m_ei );
        if (b) {
          if ( trim.m_ei < 0 || trim.m_ei >= c3_count )
          {
            trim.m_ei = -1;
            ON_ERROR("ON_Brep::ReadOld201 - trim.m_ei out of range.");
            rc = FALSE;
            break;
          }
        }
        if ( trim.m_trim_index >= 0 && trim.m_trim_index < c2_count )
          trim.m_c2i = trim.m_trim_index;
        else {
          ON_ERROR("ON_Brep::ReadOld200 - trim.m_trim_index out of range.");
          rc = FALSE;
          break;
        }
        if (rc) rc = file.ReadInt(&trim.m_bRev3d);
        if (rc) rc = file.ReadInt(&gcon_flag);
        if (rc) rc = file.ReadInt(&mono_flag);
        if (rc) rc = file.ReadDouble(&trim.m__legacy_3d_tol);
        if (rc) rc = file.ReadDouble(&trim.m__legacy_2d_tol);
      }
    }
  }

  // finish hooking trims to edges
  if (rc) {
    int trim_index;
    for ( i = 0; i < trim_count; i++ ) {
      trim_index = te_index[i];
      if ( trim_index >= 0 && trim_index < m_T.Count() )
        continue;
      twin_index = te_twin_index[i];
      if ( twin_index >= 0 && twin_index < m_T.Count() )
        continue;
      ON_BrepTrim& trim1 = m_T[trim_index];
      ON_BrepTrim& trim2 = m_T[twin_index];
      if ( trim1.m_ei >= 0 && trim1.m_ei < c2_count && trim2.m_ei < 0 )
        trim2.m_ei = trim1.m_ei;
      else if ( trim2.m_ei >= 0 && trim2.m_ei < c2_count && trim1.m_ei < 0 )
        trim1.m_ei = trim2.m_ei;
    }
    for ( i = 0; i < m_T.Count(); i++ ) {
      ON_BrepTrim& trim = m_T[i];
      if ( trim.m_ei >= 0 && trim.m_ei < c3_count )
        m_E[trim.m_ei].m_ti.Append(trim.m_trim_index);
    }


    // finish setting flags
    SetTrimIsoFlags();
    SetTrimTypeFlags();

    // create 3d vertex information
    SetVertices();

    // set tols from values in file
    SetTolsFromLegacyValues();

  }
  else {
    Destroy();
  }

  if (rc) {
    // 3d render mesh geometry
    ON_Object* obj;
    for ( i = 0; rc && i < face_count; i++ ) {
      ON_BrepFace& f = m_F[i];
      file.ReadChar(&b);
      if (b) {
        rc = file.ReadObject(&obj);
        f.m_render_mesh = ON_Mesh::Cast(obj);
        if ( !f.m_render_mesh )
          delete obj;
      }
    }
    if ( !rc ) {
      // delete render mesh geometry
      for ( i = 0; i < face_count; i++ ) {
        ON_BrepFace& f = m_F[i];
        if ( f.m_render_mesh ) {
          delete f.m_render_mesh;
          f.m_render_mesh = 0;
        }
      }
    }

    if (rc && minor_version >= 1) {
      // 3d analysis mesh geometry
      for ( i = 0; rc && i < face_count; i++ ) {
        ON_BrepFace& f = m_F[i];
        file.ReadChar(&b);
        if (b) {
          rc = file.ReadObject(&obj);
          f.m_analysis_mesh = ON_Mesh::Cast(obj);
          if ( !f.m_analysis_mesh )
            delete obj;
        }
      }
      if ( !rc ) {
        // delete analysis mesh geometry
        for ( i = 0; i < face_count; i++ ) {
          ON_BrepFace& f = m_F[i];
          if ( f.m_analysis_mesh ) {
            delete f.m_analysis_mesh;
            f.m_analysis_mesh = 0;
          }
        }
      }
    }

    if (!rc ) {
      ON_ERROR("ON_Brep::ReadOld201() - trouble reading render/analysis meshes");
      rc = TRUE;
    }
  }
  return rc;

}

BOOL ON_Brep::ReadOld100( ON_BinaryArchive& file )
{
  // b-rep was written by old Rhino I/O tookit
  int sz, i;

  // 2d curve geometry
  file.ReadInt( &sz );
  if ( sz < 1 ) {
    return FALSE;
  }
  m_C2.Reserve(sz);
  for ( i = 0; i < sz; i++ ) {
    m_C2.Append( Read100_BrepCurve( file ) );
  }

  // 3d curve geomery
  file.ReadInt( &sz );
  if ( sz < 1 ) {
    return FALSE;
  }
  m_C3.Reserve(sz);
  for ( i = 0; i < sz; i++ ) {
    m_C3.Append( Read100_BrepCurve( file ) );
  }

  // surface geometry
  file.ReadInt( &sz );
  if ( sz < 1 ) {
    return FALSE;
  }
  m_S.Reserve(sz);
  for ( i = 0; i < sz; i++ ) {
    m_S.Append( Read100_BrepSurface( file ) );
  }
  /*
    CRhinoChunk* pChunk;
    CRhinoNurbsSurface* pSurface;
    const int typecode, length;
    for ( i = 0; i < sz; i++ ) {
      file.ReadInt( &typecode );
      if ( typecode == TCODE_RHINO_OBJECT_NURBS_SURFACE && length > 0 ) {
        pSurface = (CRhinoNurbsSurface*)pChunk;
      }
      m_S.Append( pSurface );
    }
  */

  // read topology

  // vertices
  file.ReadInt( &sz );
  m_V.Reserve(sz);
  for ( i = 0; i < sz; i++ ) {
    m_V.Append(ON_BrepVertex());
    m_V[i].Read(file);
  }

  // edges
  file.ReadInt( &sz );
  m_E.Reserve(sz);
  for ( i = 0; i < sz; i++ ) {
    m_E.Append(ON_BrepEdge());
    m_E[i].Read(file);
  }

  // trims
  file.ReadInt( &sz );
  m_T.Reserve(sz);
  for ( i = 0; i < sz; i++ ) {
    m_T.Append(ON_BrepTrim());
    m_T[i].Read(file);
  }

  // loops
  file.ReadInt( &sz );
  m_L.Reserve(sz);
  for ( i = 0; i < sz; i++ ) {
    m_L.Append(ON_BrepLoop());
    m_L[i].Read(file);
  }

  // faces
  file.ReadInt( &sz );
  m_F.Reserve(sz);
  for ( i = 0; i < sz; i++ ) {
    m_F.Append(ON_BrepFace());
    m_F[i].Read(file);
  }

  // bounding box
  file.ReadPoint( m_bbox.m_min );
  file.ReadPoint( m_bbox.m_max );

  return TRUE;
}

BOOL ON_Brep::ReadOld101( ON_BinaryArchive& file )
{
  ON_Object*  pO = NULL;
  ON_Curve*   pC = NULL;
  ON_Surface* pS = NULL;
  int i, count;

  // 2d curves
  file.ReadInt( &count );
  m_C2.Reserve(count);
  for ( i = 0; i < count; i++ ) {
    pO = NULL;
    file.ReadObject( &pO );
    pC = ON_Curve::Cast(pO);
    if ( !pC )
      delete pO; // ERROR!
    m_C2.Append( pC );
    pC = NULL;
    pO = NULL;
  }

  // 3d curves
  file.ReadInt( &count );
  m_C3.Reserve(count);
  for ( i = 0; i < count; i++ ) {
    pO = NULL;
    file.ReadObject( &pO );
    pC = ON_Curve::Cast(pO);
    if ( !pC )
      delete pO; // ERROR!
    m_C3.Append( pC );
    pC = NULL;
    pO = NULL;
  }

  // untrimmed surfaces
  file.ReadInt( &count );
  m_S.Reserve(count);
  for ( i = 0; i < count; i++ ) {
    pO = NULL;
    file.ReadObject( &pO );
    pS = ON_Surface::Cast(pO);
    if ( !pS )
      delete pO; // ERROR!
    m_S.Append( pS );
    pS = NULL;
    pO = NULL;
  }

  // vertices
  file.ReadInt( &count );
  m_V.Reserve(count);
  m_V.SetCount(count);
  for ( i = 0; i < count; i++ ) {
    m_V[i].Read(file);
  }

  // edges
  file.ReadInt( &count );
  m_E.Reserve(count);
  m_E.SetCount(count);
  for ( i = 0; i < count; i++ ) {
    ON_BrepEdge& edge = m_E[i];
    edge.Read(file);
    edge.m_curve = edge.m_c3i >= 0 ? m_C3[edge.m_c3i] : 0;
    edge.m_brep = this;
  }

  // trims
  file.ReadInt( &count );
  m_T.Reserve(count);
  m_T.SetCount(count);
  for ( i = 0; i < count; i++ ) {
    m_T[i].Read(file);
  }

  // loops
  file.ReadInt( &count );
  m_L.Reserve(count);
  m_L.SetCount(count);
  for ( i = 0; i < count; i++ ) {
    m_L[i].Read(file);
  }

  // faces
  file.ReadInt( &count );
  m_F.Reserve(count);
  m_F.SetCount(count);
  for ( i = 0; i < count; i++ ) {
    ON_BrepFace& face = m_F[i];
    face.Read(file);
    face.m_surface = face.m_si >= 0 ? m_S[face.m_si] : 0;
    face.m_brep = this;
  }

  // bounding box
  file.ReadPoint( m_bbox.m_min );
  file.ReadPoint( m_bbox.m_max );

  return TRUE;
}

BOOL ON_Brep::Read_1( ON_BinaryArchive& file )
{
  // b-rep was written by openNURBS code
  ON_ERROR("Dale - write this code now!!");
  
  return FALSE;
}

ON_Curve* ON_Brep::Read100_BrepCurve( ON_BinaryArchive& ) const
{
  // TODO - look at old Rhino I/O tookit code and read b-rep curves
  return NULL;
}

ON_Surface* ON_Brep::Read100_BrepSurface( ON_BinaryArchive& ) const
{
  // TODO - look at old Rhino I/O tookit code and read b-rep surfaces
  return NULL;
}

void ON_Brep::SetTolsFromLegacyValues()
{
  // use m_2d_tol and m_3d_tol read from file to set public tolerances
  const int vcnt = m_V.Count();
  const int tcnt = m_T.Count();
  ON_3dPoint endP;
  double d;
  int vi, ti, vei, evi, vecnt;
  
  // set trim and edge tolerances from values in file
  for ( ti = 0; ti < tcnt; ti++ ) {
    ON_BrepTrim& trim = m_T[ti];
    trim.m_tolerance[0] = trim.m__legacy_2d_tol; // "pe_tol"
    trim.m_tolerance[1] = trim.m__legacy_2d_tol; // "pe_tol"
    if ( trim.m_ei >= 0 ) {
      ON_BrepEdge& edge = m_E[trim.m_ei];
      if ( edge.m_tolerance < trim.m__legacy_3d_tol )
        edge.m_tolerance = trim.m__legacy_3d_tol; // "e_tol"
    }
  }

  // set vertex tolerances from edge tols and end evaluations
  for ( vi = 0; vi < vcnt; vi++ ) {
    ON_BrepVertex& vertex = m_V[vi];
    vecnt = vertex.m_ei.Count();
    for ( vei = 0; vei < vecnt; vei++ ) {
      const ON_BrepEdge& edge = m_E[vertex.m_ei[vei]];
      if ( vertex.m_tolerance < edge.m_tolerance )
        vertex.m_tolerance = edge.m_tolerance;
      const ON_Curve* c = m_C3[edge.m_c3i];
      evi = 0;
      if ( edge.m_vi[0] != vi )
        evi = 1;
      if ( edge.m_vi[evi] == vi ) {
        endP = c->PointAt( c->Domain()[evi] );
        d = vertex.point.DistanceTo( endP );
        if ( d > vertex.m_tolerance ) {
          vertex.m_tolerance = d;      
        }
      }
    }
  }
}

ON::object_type ON_Brep::ObjectType() const
{
  return ON::brep_object;
}


BOOL
ON_Brep::GetBBox(
          double* box_min, // [3],
          double* box_max, // [3],
          BOOL bGrowBox     // = FALSE
          ) const
{
  BOOL rc = TRUE;
  const int srf_count = m_S.Count();
  int i;
  for ( i = 0; i < srf_count; i++ ) {
    if ( !m_S[i] )
      continue;
    if ( m_S[i]->GetBBox( box_min, box_max, bGrowBox ) )
      bGrowBox = TRUE;
    else
      rc = FALSE;
  }
  return rc;
}

BOOL
ON_Brep::SwapCoordinates( int i, int j )
{
  BOOL rc = FALSE;
  // swap surface coordinates
  const int srf_count = m_S.Count();
  int si;
  for ( si = 0; si < srf_count; si++ ) {
    if ( !m_S[si] ) 
      continue;
    rc = m_S[si]->SwapCoordinates(i,j);
    if ( !rc ) {
      while ( --si >= 0 ) {
        // undo any changes;
        if ( m_S[si] )
          m_S[si]->SwapCoordinates(i,j);
      }
      return FALSE;
    }
  }
  // swap 3d curve coordinates
  const int crv_count = m_S.Count();
  int ci;
  for ( ci = 0; ci < crv_count; ci++ ) {
    if ( !m_C3[ci] ) 
      continue;
    rc = m_C3[ci]->SwapCoordinates(i,j);
    if ( !rc ) {
      // undo any changes;
      while ( --ci >= 0 ) {
        if ( m_C3[ci] )
          m_C3[ci]->SwapCoordinates(i,j);
        for ( si = 0; si < srf_count; si++ ) {
          if ( m_S[si] )
            m_S[si]->SwapCoordinates(i,j);
        }
      }
      return FALSE;
    }
  }
  return rc;
}

BOOL
ON_Brep::SwapTrimParameters(
        int trim_index
        )
{
  // helper for SwapLoopParameters
  if ( trim_index < 0 || trim_index >= m_T.Count() )
    return FALSE;
  ON_BrepTrim& trim = m_T[trim_index];
  const int ci = trim.m_c2i;
  if ( ci < 0 || ci >= m_C2.Count() )
    return FALSE;
  ON_Curve* pC = m_C2[ci];
  if ( !pC )
    return FALSE;

  BOOL rc = pC->SwapCoordinates(0,1); // "u" <-> "v"
  if ( !rc )
    return FALSE;

  // reverse 2d curve to keep active region on the left
  rc = pC->Reverse();
  if ( !rc ) {
    // undo changes
    rc = pC->SwapCoordinates(0,1); // "u" <-> "v"
    return FALSE;
  }

  // reverse end information
  trim.m_t.Reverse();
  int vi = trim.m_vi[0]; trim.m_vi[0] = trim.m_vi[1]; trim.m_vi[1] = vi;
  //double x;
  //x = trim.m_P[0][0]; trim.m_P[0][0] = trim.m_P[1][0]; trim.m_P[1][0]= x;
  //x = trim.m_P[0][1]; trim.m_P[0][1] = trim.m_P[1][1]; trim.m_P[1][1]= x;
  //x = trim.m_P[0][2]; trim.m_P[0][2] = trim.m_P[1][2]; trim.m_P[1][2]= x;

  switch ( trim.m_iso ) {
  case ON_Surface::not_iso:
    trim.m_iso = ON_Surface::not_iso;
    break;
  case ON_Surface::x_iso:
    trim.m_iso = ON_Surface::y_iso;
    break;
  case ON_Surface::y_iso:
    trim.m_iso = ON_Surface::x_iso;
    break;
  case ON_Surface::W_iso:
    trim.m_iso = ON_Surface::S_iso;
    break;
  case ON_Surface::S_iso:
    trim.m_iso = ON_Surface::W_iso;
    break;
  case ON_Surface::E_iso:
    trim.m_iso = ON_Surface::N_iso;
    break;
  case ON_Surface::N_iso:
    trim.m_iso = ON_Surface::E_iso;
    break;
  default:
    trim.m_iso = ON_Surface::not_iso;
    break;
  }

  if ( trim.m_ei >= 0 )
    trim.m_bRev3d = (trim.m_bRev3d) ? FALSE : TRUE;
  return TRUE;
}

BOOL
ON_Brep::SwapLoopParameters(
        int loop_index
        )
{
  BOOL rc = FALSE;
  if ( loop_index < 0 || loop_index >= m_L.Count() )
    return FALSE;
  ON_BrepLoop& L = m_L[loop_index];
  const int loop_trim_count = L.m_ti.Count();
  if ( loop_trim_count < 1 )
    return FALSE;
  int lti, ti;
  for ( lti = 0; lti < loop_trim_count; lti++ ) {
    ti = L.m_ti[lti];
    rc = SwapTrimParameters( ti );
    if ( !rc ) {
      while ( --lti >= 0 ) {
        // undo any changes
        ti = L.m_ti[lti];
        SwapTrimParameters( ti );
      }
      return FALSE;
    }
  }

  // reverse order of trimming curves
  if ( rc )
    L.m_ti.Reverse();
  return rc;
}

BOOL
ON_Brep::IsSolid() const
{
  BOOL bIsOriented = FALSE;
  BOOL bHasBoundary = TRUE;
  BOOL bIsManifold = IsManifold( &bIsOriented, &bHasBoundary );
  return (bIsManifold && bIsOriented && !bHasBoundary) ? TRUE : FALSE;
}

BOOL
ON_Brep::IsManifold( BOOL* pbIsOriented, BOOL* pbHasBoundary ) const
{
  const int fcnt = m_F.Count();
  BOOL bIsManifold = (fcnt > 0) ? TRUE : FALSE;
  BOOL bIsOriented = bIsManifold;
  BOOL bHasBoundary = FALSE;
  int fi, li, ti, other_ti, lcnt, tcnt;
  if ( pbIsOriented )
    *pbIsOriented = bIsOriented;
  if ( pbHasBoundary )
    *pbHasBoundary = bHasBoundary;
  for ( fi = 0; fi < fcnt && bIsManifold; fi++ ) {
    const ON_BrepFace& face = m_F[fi];
    lcnt = face.m_li.Count();
    if ( lcnt < 1 ) {
      bIsManifold = FALSE;
    }
    for ( li = 0; li < lcnt && bIsManifold; li++ ) {
      const ON_BrepLoop& loop = m_L[face.m_li[li]];
      tcnt = loop.m_ti.Count();
      if (tcnt < 1 ) {
        bIsManifold = FALSE;
      }
      for ( ti = 0; ti < tcnt && bIsManifold; ti++ ) {
        const ON_BrepTrim& trim = m_T[ti];
        switch ( trim.m_type ) {
        case ON_BrepTrim::boundary:
          bHasBoundary = TRUE;
          break;
        case ON_BrepTrim::mated:
        case ON_BrepTrim::seam:
          // make sure we have a manifold join
          {
            const ON_BrepEdge& edge = m_E[trim.m_ei];
            if ( edge.m_ti.Count() != 2 ) {
              bIsManifold = FALSE;
            }
            else {
              other_ti = edge.m_ti[0];
              if ( other_ti == ti )
                other_ti = edge.m_ti[1];
              if ( other_ti == ti ) {
                bIsManifold = FALSE;
              }
              else {
                const ON_BrepTrim& other_trim = m_T[other_ti];

                BOOL bFlipTrim = trim.m_bRev3d;
                if ( m_F[m_L[trim.m_li].m_fi].m_bRev )
                  bFlipTrim = !bFlipTrim;

                BOOL bFlipOther = other_trim.m_bRev3d;
                if ( m_F[m_L[other_trim.m_li].m_fi].m_bRev )
                  bFlipOther = !bFlipOther;

                if ( bFlipTrim && bFlipOther ) {
                  bIsOriented = FALSE;
                }
                else if ( !bFlipTrim && !bFlipOther ) {
                  bIsOriented = FALSE;
                }
              }
            }
          }
          break;
        case ON_BrepTrim::singular:
          // nothing to check here
          break;
        default:
          bIsManifold = FALSE;
          break;
        }
      }
    }
  }
  if ( !bIsManifold ) {
    bIsOriented = FALSE;
    bHasBoundary = FALSE;
  }
  if ( pbIsOriented )
    *pbIsOriented = bIsOriented;
  if ( pbHasBoundary )
    *pbHasBoundary = bHasBoundary;
  return bIsManifold;
}


BOOL
ON_Brep::IsSurface() const
{
  // returns TRUE if the b-rep has a single face
  // and that face is geometrically the same
  // as the underlying surface.  I.e., the face
  // has trivial trimming.  In this case, the
  // surface is m_S[0].
  return (m_F.Count() == 1 && FaceIsSurface(0));
}

BOOL 
ON_Brep::FaceIsSurface( int face_index ) const
{
  // returns TRUE if the face has a single
  // outer boundary and that boundary runs
  // along the edges of the underlying surface.
  // In this case the geometry of the surface
  // is the same as the geometry of the face.

  BOOL bTrivialFace = FALSE;
  if ( face_index >= 0 && face_index < m_F.Count() ) {
    const ON_BrepFace& face = m_F[face_index];
    if ( face.m_li.Count() == 1 ) {
      bTrivialFace = LoopIsSurfaceBoundary( face.m_li[0] );
    }
  }
  return bTrivialFace;
}

BOOL 
ON_Brep::LoopIsSurfaceBoundary( int loop_index ) const
{
  // returns TRUE if the loop's trims run along the underlying surface boundary
  BOOL bTrivialLoop = FALSE;
  if ( loop_index >= 0 && loop_index < m_L.Count() ) {
    const ON_BrepLoop& loop = m_L[loop_index];
    const int trim_count = loop.m_ti.Count();
    if ( trim_count > 0 ) {
      bTrivialLoop = TRUE;
      int ti;
      for ( ti = 0; ti < trim_count && bTrivialLoop; ti++ ) {
        const ON_BrepTrim& trim = m_T[loop.m_ti[ti]];
        if ( trim.m_iso == ON_Surface::W_iso )
          continue;
        if ( trim.m_iso == ON_Surface::S_iso )
          continue;
        if ( trim.m_iso == ON_Surface::N_iso )
          continue;
        if ( trim.m_iso == ON_Surface::E_iso )
          continue;
        bTrivialLoop = FALSE;
      }
    }
  }
  return bTrivialLoop;
}

BOOL
ON_Brep::FlipReversedSurfaces()
{
  // Clears all ON_BrepFace.m_bRev flags
  // by calling SwapFaceParameters() on each
  // face with a TRUE m_bRev.
  //
  // Returns TRUE if successful.

  const int face_count = m_F.Count();

  BOOL rc = TRUE;
  int fi;
  for ( fi = 0; fi < face_count; fi++ ) {
    if ( m_F[fi].m_bRev )
      rc &= SwapFaceParameters( fi );
  }
  return rc;
}

////////
// Change the domain of a trim
BOOL ON_Brep::SetTrimDomain(
       int trim_index, // index of trim in m_T[] array
       const ON_Interval& domain
       )
{
  ON_Curve* c;
  int ti;
  BOOL rc = FALSE;
  BOOL bDupCurve = FALSE;
  const int trim_count = m_T.Count();
  if ( trim_index >= 0 && trim_index < trim_count && domain.IsIncreasing() ) {
    ON_BrepTrim& trim = m_T[trim_index];
    c = TrimCurveOf(trim);
    if ( c ) {
      ON_Interval cdom = c->Domain();
      if ( cdom == domain && trim.m_t == domain ) {
        // no change is required
        rc = TRUE;
      }
      else {
        // make sure this is the only ON_BrepTrim using this 2d curve
        for ( ti = 0; ti < trim_count; ti++ ) {
          if ( ti == trim_index )
            continue;
          if ( c == TrimCurveOf( m_T[ti] ) ) {
            bDupCurve = TRUE;
            break;
          }
        }
        if ( bDupCurve ) {
          // make a copy of the 2d curve because it is referenced by
          // multiple ON_BrepTrims
          ON_Curve* dup = ON_Curve::Cast(c->Duplicate());
          if ( dup ) {
            c = dup;
            trim.m_c2i = m_C2.Count();
            m_C2.Append(c);
          }
          else {
            c = NULL;
          }
        }
        if ( c ) {
          rc = TRUE;
          if ( trim.m_t != cdom ) {
            // the trim was only using a portion of the 2d curve
            rc = c->Trim( trim.m_t );
          }
          if ( rc ) {
            // change the domain of the 2d curve
            rc = c->SetDomain( domain[0], domain[1] );
            if ( rc )
              trim.m_t = domain;
          }
        }
      }
    }
  }
  return rc;
}

////////
// Change the domain of an edge
BOOL ON_Brep::SetEdgeDomain(
       int edge_index, // index of edge in m_E[] array
       const ON_Interval& domain
       )
{
  BOOL rc = FALSE;
  int ei;
  BOOL bDupCurve = FALSE;
  const int edge_count = m_E.Count();
  if ( edge_index >= 0 && edge_index < edge_count && domain.IsIncreasing() ) {
    ON_BrepEdge& edge = m_E[edge_index];
    ON_Curve* c = EdgeCurveOf( edge );
    if ( c ) {
      ON_Interval edom = edge.Domain();
      ON_Interval cdom = c->Domain();
      if ( edom == domain && cdom == domain ) {
        // no changes required
        rc = TRUE;
      }
      else {
        // make sure this is the only ON_BrepEdge using this 3d curve
        for ( ei = 0; ei < edge_count; ei++ ) {
          if ( ei == edge_count )
            continue;
          if ( c == EdgeCurveOf( m_E[ei] ) ) {
            bDupCurve = TRUE;
            break;
          }
        }
        if ( bDupCurve ) {
          // make a copy of the 3d curve because it is referenced by
          // multiple ON_BrepEdges
          ON_Curve* dup = ON_Curve::Cast(c->Duplicate());
          if ( dup ) {
            c = dup;
            edge.m_c3i = m_C3.Count();
            edge.m_curve = c;
            m_C3.Append(c);
          }
          else {
            c = NULL;
          }
          if ( c ) {
            rc = TRUE;
            if ( edge.m_domain != cdom ) {
              // the edge was only using a portion of the 3d curve
              rc = c->Trim( edge.m_domain );
            }
            if ( rc ) {
              // change the domain of the 3d curve
              rc = c->SetDomain( domain[0], domain[1] );
              if ( rc )
                edge.m_domain = domain;
            }
          }
        }
      }
    }
  }
  return rc;
}

BOOL ON_Brep::ReverseFaceParameter(   // swaps "u" and "v" parameters of face's
      int, // index of face
      int // dir = 0 reverse "u", 1 reverse "v"
      )
{
//#error TODO
  return FALSE;
}

////////
// Change the domain of a face
// This also transforms the "u" and "v" coordinates of all the 
// face's parameter space trimming curves.
BOOL ON_Brep::SetFaceDomain(
       int, // index of face in m_F[] array
       const ON_Interval&,
       const ON_Interval&
       )
{
//#error TODO
  return FALSE;
}

BOOL
ON_Brep::SwapFaceParameters( int face_index )
{
  // Returns TRUE if successful

  if ( face_index < 0 || face_index >= m_F.Count() )
    return FALSE;
  ON_BrepFace& face = m_F[face_index];

  const int si = face.m_si;
  if ( si < 0 || si >= m_S.Count() )
    return FALSE;
  ON_Surface* pS = m_S[si];
  if ( !pS )
    return FALSE;

  const int face_loop_count = face.m_li.Count();
  if ( face_loop_count < 0 || face_loop_count > m_L.Count() )
    return FALSE;

  // swap surface "u" and "v"
  BOOL rc = pS->Transpose();
  if ( !rc )
    return FALSE;

  // reverse trimming loops
  int fli, li;
  for ( fli = 0; fli < face_loop_count; fli++ ) {
    li = face.m_li[fli];
    rc = SwapLoopParameters(li);
    if ( !rc ) {
      // undo any changes
      pS->Transpose();
      while ( --fli >= 0 ) {
        li = face.m_li[fli];
        SwapLoopParameters(li);
      }
      return FALSE;
    }
  }

  // toggle m_bRev flag
  face.m_bRev = (face.m_bRev) ? FALSE : TRUE;

  return TRUE;
}

void
ON_Brep::Flip()
{
  const int fcnt = m_F.Count();
  int fi;
  for ( fi = 0; fi < fcnt; fi++ ) {
    FlipFace(m_F[fi]);
  }
}

void
ON_Brep::FlipEdge( ON_BrepEdge& edge )
{
  const int tcount = m_T.Count();
  int ti, vi, eti;
  edge.Reverse(); // reverse use of 3d curve
  vi = edge.m_vi[0]; edge.m_vi[0] = edge.m_vi[1]; edge.m_vi[1] = vi;
  for ( eti = edge.m_ti.Count()-1; eti >= 0; eti-- ) {
    ti = edge.m_ti[eti];
    if ( ti >= 0 && ti < tcount ) {
      ON_BrepTrim& trim = m_T[ti];
      trim.m_bRev3d = trim.m_bRev3d ? FALSE : TRUE;
    }
  }
}

void
ON_Brep::FlipFace( ON_BrepFace& face )
{
  face.m_bRev = (face.m_bRev) ? FALSE : TRUE;
}

void 
ON_Brep::FlipTrim(ON_BrepTrim& trim)
{
  ON_Interval d0, d1;
  double t0, t1;
  //ON_3dPoint pt;
  int i;
  ON_Curve* c2;
  if ( trim.m_c2i >= 0 && trim.m_c2i < m_C2.Count() ) {
    c2 = m_C2[trim.m_c2i];
    if ( c2 ) {
      // reverse 2d trimming curve
      d0 = c2->Domain();
      c2->Reverse();
      d1 = c2->Domain();
      // update trim's m_t
      if ( d0 == trim.m_t )
        trim.m_t = d1;
      else {
        t0 = d1.ParameterAt(1.0-d0.NormalizedParameterAt(trim.m_t[1]));
        t1 = d1.ParameterAt(1.0-d0.NormalizedParameterAt(trim.m_t[0]));
        trim.m_t.Reverse();
        if ( fabs(t0 - trim.m_t[0]) > ON_ZERO_TOLERANCE || fabs(t1 - trim.m_t[1]) > ON_ZERO_TOLERANCE ) {
          // some curves do not reverse their domain when the orientation is
          // reversed.  This test traps those cases.
          trim.m_t.Set(t0,t1);
        }
      }
    }
    // swap end vertices
    i = trim.m_vi[0];
    trim.m_vi[0] = trim.m_vi[1];
    trim.m_vi[1] = i;
    // swap end points
    //pt = trim.m_P[0];
    //trim.m_P[0] = trim.m_P[1];
    //trim.m_P[1] = pt; 
    
    // If trim has a 3d edge, then toggle m_bRev3d.
    if ( trim.m_ei >= 0 ) {
      trim.m_bRev3d = trim.m_bRev3d ? FALSE : TRUE;
    }
  }
}

void 
ON_Brep::FlipLoop(ON_BrepLoop& loop)
{
  int ti, lti;
  const int brep_trim_count = m_T.Count();
  const int loop_trim_count = loop.m_ti.Count();

  // reverse order of trimming curves
  loop.m_ti.Reverse();
  // reverse direction of individual trimming curves
  for ( lti = 0; lti < loop_trim_count; lti++ ) {
    ti = loop.m_ti[lti];
    if ( ti >= 0 && ti < brep_trim_count ) {
      FlipTrim(m_T[ti]);
    }
  }
}

static int curve_area( ON_3dPoint& start_point, const ON_Curve* curve, const ON_Interval& curve_domain, double *area )
{
  ON_Workspace ws;
  ON_Interval span_domain;
  double *span_vector, *t, twice_area = 0.0;
  ON_3dPoint p0, p1;
  int degree, span_count, span_i, j;
  if ( !area )
    return FALSE;
  *area = 0;
  if ( !curve )
    return FALSE;
  const ON_PolyCurve* polycurve = ON_PolyCurve::Cast(curve);
  if ( polycurve ) {
    span_count = polycurve->Count();  
    for ( span_i = 0; span_i < span_count; span_i++ ) {
      if ( !curve_area( start_point, polycurve->SegmentCurve(span_i), curve_domain, &twice_area ) ) {
        *area = 0.0;
        return FALSE;
      }
      *area += twice_area;
    }
  }
  else {
    span_count = curve->SpanCount();
    if ( span_count < 1 )
      return FALSE;
    degree = curve->Degree();
    if ( degree < 1 )
      degree = 1;
    span_vector = ws.GetDoubleMemory(span_count+1+degree);
    t = span_vector+(span_count+1);
    t[0] = 0.0;
    for ( j = 1; j < degree; j++ ) {
      t[j] = ((double)(j))/(1.0+degree);
    }
    if ( !curve->GetSpanVector( span_vector ) )
      return FALSE;

    p1 = start_point;
    for ( span_i = 0; span_i < span_count; span_i++ ) {
      span_domain.Set( span_vector[span_i], span_vector[span_i+1] );
      if ( span_domain[1] <= curve_domain[0] )
        continue;
      if ( span_domain[0] >= curve_domain[1] )
        break;
      if ( span_domain[1] > curve_domain[1] )
        span_domain.m_t[1] = curve_domain[1];
      if ( span_domain[0] < curve_domain[0] )
        span_domain.m_t[0] = curve_domain[0];
      if ( span_domain[0] >= span_domain[1] )
        continue;
      for ( j = 0; j < degree; j++ ) {
        p0 = p1;
        p1 = curve->PointAt(span_domain.ParameterAt(t[j]));
        twice_area += (p0.x-p1.x)*(p0.y+p1.y);
        if ( !span_i && !j ) {
          // check gap
        }
      }
    }
    p0 = p1;
    p1 = curve->PointAtEnd();
    twice_area += (p0.x-p1.x)*(p0.y+p1.y);
    start_point = p1;
    *area = 0.5*twice_area;
  }  
  
  return TRUE;
}

int 
ON_Brep::LoopDirection( const ON_BrepLoop& loop ) const
{
  ON_3dPoint start_point;
  double d, a = 0.0;
  int ti, lti, c2i;
  const int brep_trim_count = m_T.Count();
  const int brep_C2_count = m_C2.Count();
  const int loop_trim_count = loop.m_ti.Count();

  // reverse direction of individual trimming curves
  for ( lti = 0; lti < loop_trim_count; lti++ ) {
    ti = loop.m_ti[lti];
    if ( ti < 0 || ti >= brep_trim_count ) {
      a = 0.0;
      break;
    }
    c2i =  m_T[ti].m_c2i;
    if ( c2i < 0 || c2i >= brep_C2_count ) {
      a = 0.0;
      break;
    }
    if ( lti == 0 ) {
      // evaluate start of first trim
      if ( m_C2[c2i] )
        start_point = m_C2[c2i]->PointAt(m_T[ti].m_t[0]);
    }
    if ( !curve_area( start_point, m_C2[c2i], m_T[ti].m_t, &d ) ) {
      a = 0.0;
      break;
    }
    a += d;
  }
  if (a > 0.0)
    return 1;
  else if (a < 0.0)
    return -1;
  return 0;
}


BOOL
ON_Brep::SetTolerance( ON_BrepVertex& vertex,
  BOOL bLazySet // default = FALSE
                // FALSE: recompute tolerance even if
                //        its current value is positive
                // TRUE:  recompute tolerance only if
                //        its current value is nonpositive
  ) const
{
  if ( vertex.m_tolerance < 0.0 || !bLazySet ) {
    const int vertex_edge_count = vertex.EdgeCount();
    if ( vertex_edge_count < 1 ) {
      vertex.m_tolerance = 0.0;
    }
    else {
      vertex.m_tolerance = ON_UNSET_VALUE;
      double tolerance = 0.0;
      double d;
      ON_3dPoint uv;
      ON_Interval edge_domain;
      const ON_Curve* c=0;
      const ON_Surface* s=0;
      int vei, ei, eti, ti, endi;
      const int vertex_index = vertex.m_vertex_index;
      for ( vei = 0; vei < vertex_edge_count; vei++ ) {
        ei = vertex.m_ei[vei];
        if ( ei < 0 )
          return FALSE;
        const ON_BrepEdge& edge = m_E[ei];
        if ( !edge.m_curve )
          return FALSE;
        edge_domain = edge.m_curve->Domain();
        for ( endi = 0; endi < 2; endi++ ) {
          if ( edge.m_vi[endi] == vertex_index ) {
            d = vertex.point.DistanceTo( edge.m_curve->PointAt(edge_domain[endi]) );
            if ( tolerance < d )
              tolerance = d;
          }
        }
        const int edge_trim_count = edge.m_ti.Count();
        for ( eti = 0; eti < edge_trim_count; eti++ ) {
          ti = edge.m_ti[eti];
          if ( ti < 0 )
            return FALSE;
          const ON_BrepTrim& trim = m_T[ti];
          c = TrimCurveOf( trim );
          if ( !c )
            return FALSE;
          s = SurfaceOf( trim );
          if ( !s )
            return FALSE;
          for ( endi = 0; endi < 2; endi++ ) {
            if ( edge.m_vi[endi] == vertex_index ) {
              uv = c->PointAt( trim.m_t[trim.m_bRev3d?1-endi:endi] );
              d = vertex.point.DistanceTo( s->PointAt(uv.x,uv.y) );
              if ( tolerance < d )
                tolerance = d;
            }
          }
        }
      }
      vertex.m_tolerance = (tolerance <= ON_ZERO_TOLERANCE) ? 0.0 : 1.01*tolerance;
    }
  }
  return (vertex.m_tolerance >= 0.0) ? TRUE : FALSE;
}

BOOL
ON_Brep::SetTolerance( ON_BrepEdge& edge,
  BOOL bLazySet // default = FALSE
                // FALSE: recompute tolerance
                // TRUE:  recompute tolerance only if
                //        its current value is nonpositive
  ) const
{
  //int eti;
  if ( edge.m_tolerance < 0.0 || !bLazySet ) {
    const int edge_trim_count = edge.m_ti.Count();
    if ( edge_trim_count < 1 ) {
      edge.m_tolerance = 0.0;
    }
    else {
      edge.m_tolerance = ON_UNSET_VALUE;
      /*
      double tolerance = 0.0;
      const ON_Curve* c3 = m_C3[edge.m_c3i];
      for ( eti = 0; eti < edge_trim_count; eti++ ) {
        const ON_BrepTrim& trim = m_T[edge.m_ti[eti]];
        const ON_Surface* srf = SurfaceOf(trim);
        const ON_Curve* c2 = m_C2[trim.m_c2i];
        //ON_CurveOnSurface cs(c2,srf);

        // TODO - compute overlap distance


      }
      */
    }
  }
  return (edge.m_tolerance >= 0.0) ? TRUE : FALSE;
}

void ON_Brep::Dump( ON_TextLog& dump ) const
{
  dump.Print("ON_Brep:\n");

  if ( IsSurface() ) {
    dump.Print("(B-rep geometry is the same as underlying surface.)\n");
  }

  dump.Print("surfaces:  %d\n",m_S.Count());
  dump.Print("3d curve:  %d\n",m_C3.Count());
  dump.Print("2d curves: %d\n",m_C2.Count());
  dump.Print("vertices:  %d\n",m_V.Count());
  dump.Print("edges:     %d\n",m_E.Count());
  dump.Print("trims:     %d\n",m_T.Count());
  dump.Print("loops:     %d\n",m_L.Count());
  dump.Print("faces:     %d\n",m_F.Count());

  int vi;
  for ( vi = 0; vi < m_V.Count(); vi++ ) {
    const ON_BrepVertex& vertex = m_V[vi];
    dump.Print("vertex[%2d]: (%f %f %f) tolerance(%g)\n",
               vi,vertex.point.x,vertex.point.y,vertex.point.z,
               vertex.m_tolerance);
    if ( vertex.m_ei.Count() > 0 ) {
      int vei;
      dump.PushIndent();
      dump.Print("edges (");
      for ( vei = 0; vei < vertex.m_ei.Count(); vei++ ) {
        dump.Print( (vei)?",%d":"%d", vertex.m_ei[vei] );
      }
      dump.PopIndent();
      dump.Print(")\n");
    }
  }

  int ei,ti;
  for ( ei = 0; ei < m_E.Count(); ei++ ) {
    const ON_BrepEdge& edge = m_E[ei];
    dump.Print("edge[%2d]: v0(%2d) v1(%2d) 3d_curve(%d) tolerance(%g)\n",
               ei,edge.m_vi[0],edge.m_vi[1],edge.m_c3i,edge.m_tolerance);
    dump.PushIndent();
    if ( edge.m_ti.Count() > 0 ) {
      dump.Print("trims (",edge.m_ti.Count());
      int eti;
      for ( eti = 0; eti < edge.m_ti.Count(); eti++ ) {
        ti = edge.m_ti[eti];
        const char* sign = "?";
        if ( ti >= 0 && ti < m_T.Count() ) {
          sign = m_T[ti].m_bRev3d ? "-" : "+";
        }
        dump.Print( (eti)?",%s%d":"%s%d", sign,edge.m_ti[eti]);
      }
      dump.PopIndent();
      dump.Print(")\n");
    }
  }

  int fi;
  for ( fi = 0; fi < m_F.Count(); fi++ ) {
    const ON_BrepFace& face = m_F[fi];
    dump.Print("face[%2d]: surface(%d) reverse(%d) loops(",
               fi,face.m_si,face.m_bRev);
    int fli;
    for ( fli = 0; fli < face.m_li.Count(); fli++ ) {
      dump.Print( (fli)?",%d":"%d", face.m_li[fli]);
    }
    dump.Print(")\n");
    dump.PushIndent();
    if ( face.m_render_mesh ) {
      dump.Print("Render mesh: %d polygons\n",face.m_render_mesh->FaceCount());
    }
    if ( face.m_analysis_mesh ) {
      dump.Print("Analysis mesh: %d polygons\n",face.m_render_mesh->FaceCount());
    }
    if ( FaceIsSurface(fi) ) {
      dump.Print("(Face geometry is the same as underlying surface.)\n");
    }


    for ( fli = 0; fli < face.m_li.Count(); fli++ ) {
      const int li = face.m_li[fli];
      const ON_BrepLoop& loop = m_L[li];
      const char* sLoopType = 0;
      switch( loop.m_type ) {
      case ON_BrepLoop::unknown:
        sLoopType = "unknown";
        break;
      case ON_BrepLoop::outer:
        sLoopType = "outer";
        break;
      case ON_BrepLoop::inner:
        sLoopType = "inner";
        break;
      case ON_BrepLoop::slit:
        sLoopType = "slit";
        break;
      case ON_BrepLoop::crvonsrf:
        sLoopType = "crvonsrf";
        break;
      default:
        sLoopType = "unknown";
        break;
      }
      dump.Print("loop[%2d]: type(%s) %d trims(",
                 li, sLoopType, loop.m_ti.Count());
      int lti;
      for ( lti = 0; lti < loop.m_ti.Count(); lti++ ) {
        dump.Print( (lti)?",%d":"%d", loop.m_ti[lti]);
      }
      dump.Print(")\n");
      dump.PushIndent();
      for ( lti = 0; lti < loop.m_ti.Count(); lti++ ) {
        const int ti = loop.m_ti[lti];
        const ON_BrepTrim& trim = m_T[ti];
        const char* sTrimType = "?";
        const char* sTrimIso = "-?";
        switch( trim.m_type ) {
        case ON_BrepTrim::unknown:
          sTrimType = "unknown ";
          break;
        case ON_BrepTrim::boundary:
          sTrimType = "boundary";
          break;
        case ON_BrepTrim::mated:
          sTrimType = "mated   ";
          break;
        case ON_BrepTrim::seam:
          sTrimType = "seam    ";
          break;
        case ON_BrepTrim::singular:
          sTrimType = "singular";
          break;
        case ON_BrepTrim::crvonsrf:
          sTrimType = "crvonsrf";
          break;
        default:
          sTrimType = "unknown";
          break;
        }
        switch( trim.m_iso ) {
        case ON_Surface::not_iso:
          sTrimIso = "";
          break;
        case ON_Surface::x_iso:
          sTrimIso = "-u iso";
          break;
        case ON_Surface::W_iso:
          sTrimIso = "-west side iso";
          break;
        case ON_Surface::E_iso:
          sTrimIso = "-east side iso";
          break;
        case ON_Surface::y_iso:
          sTrimIso = "-v iso";
          break;
        case ON_Surface::S_iso:
          sTrimIso = "-south side iso";
          break;
        case ON_Surface::N_iso:
          sTrimIso = "-north side iso";
          break;
        default:
          sTrimIso = "-unknown_iso_flag";
          break;
        }
        dump.Print("trim[%2d]: edge(%2d) v0(%2d) v1(%2d) tolerance(%g,%g)\n"
                   "          type(%s%s) rev3d(%d) 2d_curve(%d)\n",
               ti,
               trim.m_ei,trim.m_vi[0],trim.m_vi[1],
               trim.m_tolerance[0],trim.m_tolerance[1],
               sTrimType,sTrimIso,trim.m_bRev3d,trim.m_c2i);
      }
      dump.PopIndent();
    }
    dump.PopIndent();
  }

  //int si;
  //for ( si = 0; si < m_S.Count(); si++ ) {
  //  dump.Print("surface[%d]:\n",si);
  //  dump.PushIndent();
  //  if ( m_S[si] )
  //    m_S[si]->Dump(dump);
  //  else
  //    dump.Print("NULL\n");
  //  dump.PopIndent();
  //}

}

int ON_Brep::SurfaceIndexOf( const ON_BrepTrim& trim ) const
{
  return ((trim.m_li>=0)?SurfaceIndexOf(m_L[trim.m_li]):-1);
}

int ON_Brep::SurfaceIndexOf( const ON_BrepLoop& loop ) const
{
  return ((loop.m_fi>=0)?SurfaceIndexOf(m_F[loop.m_fi]):-1);
}

int ON_Brep::SurfaceIndexOf( const ON_BrepFace& face ) const
{
  return face.m_si;
}

ON_Surface* ON_Brep::SurfaceOf( const ON_BrepTrim& trim ) const
{
  const int si = SurfaceIndexOf(trim);
  return (si>=0)?m_S[si]:0;
}

ON_Surface* ON_Brep::SurfaceOf( const ON_BrepLoop& loop ) const
{
  const int si = SurfaceIndexOf(loop);
  return (si>=0)?m_S[si]:0;
}

ON_Surface* ON_Brep::SurfaceOf( const ON_BrepFace& face ) const
{
  return (face.m_si>=0&&face.m_si<m_S.Count())?m_S[face.m_si]:0;
}

int ON_Brep::EdgeCurveIndexOf( const ON_BrepTrim& trim ) const
{
  return ((trim.m_ei>=0)?EdgeCurveIndexOf(m_E[trim.m_ei]):-1);
}

int ON_Brep::EdgeCurveIndexOf( const ON_BrepEdge& edge ) const
{
  return edge.m_c3i;
}

ON_Curve* ON_Brep::EdgeCurveOf( const ON_BrepTrim& trim ) const
{
  const int c3i = EdgeCurveIndexOf(trim);
  return ((c3i>=0)?m_C3[c3i]:0);
}

ON_Curve* ON_Brep::EdgeCurveOf( const ON_BrepEdge& edge ) const
{
  return ((edge.m_c3i>=0&&edge.m_c3i<m_C3.Count())?m_C3[edge.m_c3i]:0);
}

int ON_Brep::TrimCurveIndexOf( const ON_BrepTrim& trim ) const
{
  return trim.m_c2i;
}

ON_Curve* ON_Brep::TrimCurveOf( const ON_BrepTrim& trim ) const
{
  return ((trim.m_c2i>=0&&trim.m_c2i<m_C2.Count())?m_C2[trim.m_c2i]:0);
}

void ON_Brep::DeleteVertex(ON_BrepVertex& vertex)
{
  const int vi = vertex.m_vertex_index;
  vertex.m_vertex_index = -1;
  if ( vi >= 0 && vi < m_V.Count() ) {
    int vei, ei;
    for ( vei = vertex.m_ei.Count()-1; vei>=0; vei-- ) {
      ei = vertex.m_ei[vei];
      if ( ei >= 0 && ei < m_E.Count() ) {
        ON_BrepEdge& edge = m_E[ei];
        if ( edge.m_vi[0] == vi )
          edge.m_vi[0] = -1;
        if ( edge.m_vi[1] == vi )
          edge.m_vi[1] = -1;
        DeleteEdge( edge, FALSE );
      }
    }
  }
  vertex.m_ei.Empty();
  vertex.m_tolerance = ON_UNSET_VALUE;
}

void ON_Brep::DeleteEdge(ON_BrepEdge& edge, BOOL bDeleteEdgeVertices )
{
  const int ei = edge.m_edge_index;
  edge.m_edge_index = -1;

  if ( ei >= 0 && ei < m_E.Count() ) {
    int eti, ti, evi, vei, vi;
    for ( eti = edge.m_ti.Count()-1; eti >= 0; eti-- ) {
      ti = edge.m_ti[eti];
      if ( ti >= 0 && ti < m_T.Count() ) {
        ON_BrepTrim& trim = m_T[ti];
        trim.m_ei = -1;
        if ( trim.m_li >= 0 && trim.m_li < m_L.Count() ) {
          ON_BrepLoop& loop = m_L[trim.m_li];
          if ( loop.m_fi >= 0 && loop.m_fi < m_F.Count() ) {
            DeleteFace( m_F[loop.m_fi], bDeleteEdgeVertices );
          }
        }
        DeleteTrim(trim,FALSE);
      }
    }

    for (evi = 0; evi < 2; evi++ ) {
      vi = edge.m_vi[evi];
      if ( vi >= 0 && vi < m_V.Count() ) {
        ON_BrepVertex& v = m_V[vi];
        if ( bDeleteEdgeVertices && v.m_ei.Count() == 1 && v.m_ei[0] == ei ) {
          v.m_ei.Empty();
          DeleteVertex(v);
        }
        else {
          for ( vei = v.m_ei.Count()-1; vei >= 0; vei-- ) {
            if ( v.m_ei[vei] == ei )
              v.m_ei.Remove(vei);
          }
        }
      }
    }
  }

  edge.m_c3i = -1;
  edge.m_vi[0] = -1;
  edge.m_vi[1] = -1;
  edge.m_ti.Empty();
  edge.m_tolerance = ON_UNSET_VALUE;
  edge.m_brep = 0;
  edge.m_curve = 0;
}

void ON_Brep::DeleteTrim(ON_BrepTrim& trim, BOOL bDeleteTrimEdges )
{
  const int ti = trim.m_trim_index;
  trim.m_trim_index = -1;

  if ( ti >= 0 && ti < m_T.Count() ) {
    const int ei = trim.m_ei;
    if ( ei >= 0 && ei < m_E.Count() ) {
      ON_BrepEdge& edge = m_E[ei];
      if ( bDeleteTrimEdges && edge.m_ti.Count() == 1 && edge.m_ti[0] == ti ) {
        edge.m_ti.Empty();
        DeleteEdge(edge,bDeleteTrimEdges);
      }
      else {
        int eti;
        for ( eti = edge.m_ti.Count()-1; eti >= 0; eti-- ) {
          if ( edge.m_ti[eti] == ti ) {
            edge.m_ti.Remove(eti);
            if ( edge.m_ti.Count() == 0 )
              edge.m_tolerance = 0.0;
          }
        }
      }
    }

    const int li = trim.m_li;
    if ( li >= 0 && li < m_L.Count() ) {
      ON_BrepLoop& loop = m_L[li];
      int lti;
      for ( lti = loop.m_ti.Count()-1; lti >= 0; lti-- ) {
        if ( loop.m_ti[lti] == ti )
          loop.m_ti.Remove(lti);
      }
    }
  }

  trim.m_c2i = -1;
  trim.m_ei = -1;
  trim.m_vi[0] = -1;
  trim.m_vi[1] = -1;
  trim.m_bRev3d = 0;
  trim.m_type = ON_BrepTrim::unknown;
  trim.m_iso = ON_Surface::not_iso;
  trim.m_li = -1;
  trim.m_tolerance[0] = ON_UNSET_VALUE;
  trim.m_tolerance[1] = ON_UNSET_VALUE;
  trim.m__legacy_2d_tol = ON_UNSET_VALUE;
  trim.m__legacy_3d_tol = ON_UNSET_VALUE;
  trim.m__legacy_flags = 0;
}

void ON_Brep::DeleteLoop(ON_BrepLoop& loop,  BOOL bDeleteLoopEdges  )
{
  const int li = loop.m_loop_index;
  loop.m_loop_index = -1;

  if ( li >= 0 && li < m_L.Count() ) {
    const int tcount = m_T.Count();
    int lti, ti;
    for ( lti = loop.m_ti.Count()-1; lti >= 0; lti-- ) {
      ti = loop.m_ti[lti];
      if ( ti >= 0 && ti < tcount ) {
        ON_BrepTrim& trim = m_T[ti];
        trim.m_li = -1;
        DeleteTrim(trim,bDeleteLoopEdges);
      }
    }

    const int fi = loop.m_fi;
    if ( fi >= 0 && fi < m_F.Count() ) {
      ON_BrepFace& face = m_F[fi];
      int fli;
      for ( fli = face.m_li.Count()-1; fli >= 0; fli-- ) {
        if ( face.m_li[fli] == li ) {
          face.m_li.Remove(fli);
        }
      }
    }
  }

  loop.m_type = ON_BrepLoop::unknown;
  loop.m_ti.Empty();
  loop.m_fi = -1;
}

void ON_Brep::DeleteFace(ON_BrepFace& face, BOOL bDeleteFaceEdges )
{
  const int fi = face.m_face_index;
  face.m_face_index = -1;

  if ( fi >= 0 && fi < m_F.Count() ) {
    const int lcount = m_L.Count();
    int fli, li;
    for ( fli = face.m_li.Count()-1; fli >= 0; fli-- ) {
      li = face.m_li[fli];
      if ( li >= 0 && li < lcount ) {
        ON_BrepLoop& loop = m_L[li];
        loop.m_fi = -1;
        DeleteLoop(loop,bDeleteFaceEdges);
      }
    }
  }

  face.m_si = -1;
  face.m_li.Empty();
  face.m_surface = 0;
  face.m_brep = 0;
}


void ON_Brep::DeleteSurface(int si)
{
  if ( si >= 0 && si < m_S.Count() ) {
    delete m_S[si];
    m_S[si] = 0;
  }
}

void ON_Brep::Delete2dCurve(int c2i)
{
  if ( c2i >= 0 && c2i < m_C2.Count() ) {
    delete m_C2[c2i];
    m_C2[c2i] = 0;
  }
}

void ON_Brep::Delete3dCurve(int c3i)
{
  if ( c3i >= 0 && c3i < m_C3.Count() ) {
    delete m_C3[c3i];
    m_C3[c3i] = 0;
  }
}


BOOL ON_Brep::CullUnusedFaces()
{
  BOOL rc = TRUE;
  int fcount = m_F.Count();
  if (fcount > 0 ) {
    ON_Workspace ws;
    int *fmap = ws.GetIntMemory(fcount+1);
    *fmap++ = -1;
    memset( fmap, 0, fcount*sizeof(*fmap) );
    const int lcount = m_L.Count();
    int fi, li, mi = 0;

    // if face.m_face_index is -1, cull face
    for ( fi = 0; fi < fcount; fi++ ) {
      ON_BrepFace& face = m_F[fi];
      if ( face.m_face_index == -1)
        fmap[fi] = -1;
      else if ( face.m_face_index == fi )
        fmap[fi] = face.m_face_index = mi++;
      else {
        ON_ERROR("Brep face has illegal m_face_index.");
        rc = FALSE;
        fmap[fi] = face.m_face_index;
      }
    }

    if ( mi == 0 ) {
      m_F.Destroy();
    }
    else if ( mi < fcount ) {
      // set new face indices
      mi = 0;
      for ( fi = fcount-1; fi >= 0; fi-- ) {
        if ( m_F[fi].m_face_index  == -1 )
          m_F.Remove(fi);
        else
          m_F[fi].m_face_index = fmap[fi];
      }

      // remap loop.m_fi indices
      for ( li = 0; li < lcount; li++ ) {
        ON_BrepLoop& loop = m_L[li];
        fi = loop.m_fi;
        if ( fi < -1 || fi >= fcount ) {
          ON_ERROR("Brep loop has illegal m_fi.");
          rc = FALSE;
        }
        else 
          loop.m_fi = fmap[fi];
      }

    }
  }
  m_F.Shrink();
  return rc;
}

BOOL ON_Brep::CullUnusedSurfaces()
{
  // remove unused surfaces
  BOOL rc = TRUE;
  const int fcount = m_F.Count();
  int scount = m_S.Count();
  int si, fi, mi;

  if ( scount > 0 ) {
    ON_Workspace ws;
    int* smap = ws.GetIntMemory(scount+1);
    *smap++ = -1;
    memset(smap,0,scount*sizeof(*smap));
    mi = 0;
    for ( fi = 0; fi < fcount; fi++ ) {
      ON_BrepFace& face = m_F[fi];
      if ( face.m_face_index == -1 ) {
        face.m_si = -1;
        continue;
      }
      si = face.m_si;
      if ( si == -1 )
        continue;
      if ( si < 0 || si >= scount ) {
        ON_ERROR("Brep face has illegal m_si.");
        rc = FALSE;
      }
      else {
        if ( !smap[si] )
          mi++;
        smap[si]++;
      }
    }

    if ( mi == 0 ) {
      m_S.Destroy();
    }
    else if ( mi < scount ) {
      mi = 0;
      for ( si = 0; si < scount; si++ ) {
        if ( smap[si] )
          smap[si] = mi++;
        else {
          delete m_S[si];
          m_S[si] = 0;
          smap[si] = -1;
        }
      }

      for ( fi = 0; fi < fcount; fi++ ) {
        ON_BrepFace& face = m_F[fi];
        si = face.m_si;
        if ( si >= 0 && si < scount )
          face.m_si = smap[si];
      }

      for ( si = scount-1; si >= 0; si-- ) {
        if ( smap[si] < 0 ) {
          m_S.Remove(si);
          scount--;
        }
      }
    }
  }
  m_S.Shrink();
  return rc;
}

BOOL ON_Brep::CullUnused3dCurves()
{
  // remove unused surfaces
  BOOL rc = TRUE;
  const int ecount = m_E.Count();
  int c3count = m_C3.Count();
  int c3i, ei, mi;

  if ( c3count > 0 ) {
    ON_Workspace ws;
    int* c3map = ws.GetIntMemory(c3count+1);
    *c3map++ = -1;
    memset(c3map,0,c3count*sizeof(*c3map));
    mi = 0;
    for ( ei = 0; ei < ecount; ei++ ) {
      ON_BrepEdge& edge = m_E[ei];
      if ( edge.m_edge_index == -1 ) {
        edge.m_c3i = -1;
        continue;
      }
      c3i = edge.m_c3i;
      if ( c3i == -1 )
        continue;
      if ( c3i < -1 || c3i >= c3count ) {
        ON_ERROR("Brep edge has illegal m_c3i.");
        rc = FALSE;
      }
      else {
        if ( !c3map[c3i] )
          mi++;
        c3map[c3i]++;
      }
    }

    if ( mi == 0 ) {
      m_C3.Destroy();
    }
    else if ( mi < c3count ) {
      mi = 0;
      for ( c3i = 0; c3i < c3count; c3i++ ) {
        if ( c3map[c3i] )
          c3map[c3i] = mi++;
        else {
          delete m_S[c3i];
          m_S[c3i] = 0;
          c3map[c3i] = -1;
        }
      }

      for ( ei = 0; ei < ecount; ei++ ) {
        ON_BrepEdge& edge = m_E[ei];
        c3i = edge.m_c3i;
        if ( c3i >= 0 && c3i < c3count )
          edge.m_c3i = c3map[c3i];
      }

      for ( c3i = c3count-1; c3i >= 0; c3i-- ) {
        if ( c3map[c3i] < 0 ) {
          m_S.Remove(c3i);
          c3count--;
        }
      }
    }
  }
  m_C3.Shrink();
  return rc;
}


BOOL ON_Brep::CullUnused2dCurves()
{
  // remove unused surfaces
  BOOL rc = TRUE;
  const int tcount = m_T.Count();
  int c2count = m_C2.Count();
  int c2i, ti, mi;

  if ( c2count > 0 ) {
    ON_Workspace ws;
    int* c2map = ws.GetIntMemory(c2count+1);
    *c2map++ = -1;
    memset(c2map,0,c2count*sizeof(*c2map));
    mi = 0;
    for ( ti = 0; ti < tcount; ti++ ) {
      ON_BrepTrim& trim = m_T[ti];
      if ( trim.m_trim_index == -1 ) {
        trim.m_c2i = -1;
        continue;
      }
      c2i = trim.m_c2i;
      if ( c2i == -1 )
        continue;
      if ( c2i < -1 || c2i >= c2count ) {
        ON_ERROR("Brep trim has illegal m_c2i.");
        rc = FALSE;
      }
      else {
        if ( !c2map[c2i] )
          mi++;
        c2map[c2i]++;
      }
    }

    if ( mi == 0 ) {
      m_C2.Destroy();
    }
    else if ( mi < c2count ) {
      mi = 0;
      for ( c2i = 0; c2i < c2count; c2i++ ) {
        if ( c2map[c2i] )
          c2map[c2i] = mi++;
        else {
          delete m_S[c2i];
          m_S[c2i] = 0;
          c2map[c2i] = -1;
        }
      }

      for ( ti = 0; ti < tcount; ti++ ) {
        ON_BrepTrim& trim = m_T[ti];
        c2i = trim.m_c2i;
        if ( c2i >= 0 && c2i < c2count )
          trim.m_c2i = c2map[c2i];
      }

      for ( c2i = c2count-1; c2i >= 0; c2i-- ) {
        if ( c2map[c2i] < 0 ) {
          m_S.Remove(c2i);
          c2count--;
        }
      }
    }
  }
  m_C2.Shrink();
  return rc;
}


BOOL ON_Brep::CullUnusedLoops()
{
  BOOL rc = TRUE;
  const int lcount = m_L.Count();
  if ( lcount > 0 ) {
    ON_Workspace ws;
    int* lmap = ws.GetIntMemory(lcount+1);
    *lmap++ = -1;
    memset( lmap, 0, lcount*sizeof(*lmap) );
    const int fcount = m_F.Count();
    const int tcount = m_T.Count();
    int li, fli, flcnt, fi, ti, mi;

    mi = 0;
    for ( li = 0; li < lcount; li++ ) {
      ON_BrepLoop& loop = m_L[li];
      if ( loop.m_loop_index == -1)
        lmap[li] = -1;
      else if ( loop.m_loop_index == li )
        lmap[li] = loop.m_loop_index = mi++;
      else {
        ON_ERROR("Brep loop has illegal m_loop_index.");
        rc = FALSE;
        lmap[li] = loop.m_loop_index;
      }
    }

    if ( mi == 0 ) {
      m_L.Destroy();
    }
    else if ( mi < lcount ) {

      // remap loops
      for ( li = lcount-1; li >= 0; li-- ) {
        if ( m_L[li].m_loop_index == -1 )
          m_L.Remove(li);
        else
          m_L[li].m_loop_index = lmap[li];
      }

      // remap ON_BrepFace.m_li[] indices
      for ( fi = 0; fi < fcount; fi++ ) {
        ON_BrepFace& face = m_F[fi];
        flcnt = face.m_li.Count();
        for ( fli = flcnt-1; fli >= 0; fli-- ) {
          li = face.m_li[fli];
          if ( li < -1 || li >= lcount ) {
            ON_ERROR("Brep face m_li[] has illegal loop index.");
            rc = FALSE;
          }
          else {
            li = lmap[li];
            if (li >= 0 ) {
              face.m_li[fli] = li;
            }
            else {
              face.m_li.Remove(fli);
            }
          }
        }
      }

      // remap ON_BrepTrim.m_li indices
      for ( ti = 0; ti < tcount; ti++ ) {
        ON_BrepTrim& trim = m_T[ti];
        li = trim.m_li;
        if ( li < -1 || li >= lcount ) {
          ON_ERROR("Brep trim has illegal m_li.");
          rc = FALSE;
        }
        else {
          trim.m_li = lmap[li];
        }
      }
    }
  }
  m_L.Shrink();
  return rc;
}

BOOL ON_Brep::CullUnusedTrims()
{
  BOOL rc = TRUE;
  const int tcount = m_T.Count();
  if ( tcount > 0 ) {
    ON_Workspace ws;
    int *tmap = ws.GetIntMemory(tcount+1);
    *tmap++ = -1;
    memset( tmap, 0, tcount*sizeof(*tmap));
    const int lcount = m_L.Count();
    const int ecount = m_E.Count();
    int ti, li, ei, mi, ltcnt, lti, etcnt, eti;

    mi = 0;
    for ( ti = 0; ti < lcount; ti++ ) {
      ON_BrepTrim& trim = m_T[ti];
      if ( trim.m_trim_index == -1)
        tmap[ti] = -1;
      else if ( trim.m_trim_index == ti )
        tmap[ti] = trim.m_trim_index = mi++;
      else {
        ON_ERROR("Brep trim has illegal m_trim_index.");
        rc = FALSE;
        tmap[ti] = trim.m_trim_index;
      }
    }

    if ( mi == 0 ) {
      m_T.Destroy();
    }
    else if ( mi < tcount ) {
      // remap trim indices
      for ( ti = tcount-1; ti >= 0; ti-- ) {
        if ( m_T[ti].m_trim_index == -1 ) {
          m_T.Remove(ti);
        }
        else {
          m_T[ti].m_trim_index = tmap[ti];
        }
      }

      // remap loop.m_ti[] indicies
      for ( li = 0; li < lcount; li++ ) {
        ON_BrepLoop& loop = m_L[li];
        ltcnt = loop.m_ti.Count();
        for ( lti = ltcnt-1; lti >= 0; lti-- ) {
          ti = loop.m_ti[lti];
          if ( ti < -1 || ti >= tcount ) {
            ON_ERROR("Brep loop.m_ti[] has illegal index.");
            rc = FALSE;
          }
          else {
            ti = tmap[ti];
            if (ti >= 0 ) {
              loop.m_ti[lti] = ti;
            }
            else {
              loop.m_ti.Remove(lti);
            }
          }
        }
      }

      // remap edge.m_ti[] indicies
      for ( ei = 0; ei < ecount; ei++ ) {
        ON_BrepEdge& edge = m_E[ei];
        etcnt = edge.m_ti.Count();
        for ( eti = etcnt-1; eti >= 0; eti-- ) {
          ti = edge.m_ti[eti];
          if ( ti < -1 || ti >= tcount ) {
            ON_ERROR("Brep edge.m_ti[] has illegal index.");
            rc = FALSE;
          }
          else {
            ti = tmap[ti];
            if (ti >= 0 ) {
              edge.m_ti[eti] = ti;
            }
            else {
              edge.m_ti.Remove(eti);
            }
          }
        }
      }
    }
  }
  m_T.Shrink();
  return rc;
}

BOOL ON_Brep::CullUnusedEdges()
{
  BOOL rc = TRUE;
  const int ecount = m_E.Count();
  if ( ecount > 0 ) {
    ON_Workspace ws;
    int* emap = ws.GetIntMemory(ecount+1);
    *emap++ = -1;
    memset( emap, 0, ecount*sizeof(*emap) );
    const int vcount = m_V.Count();
    const int tcount = m_T.Count();
    int ei, ti, vi, mi, vecnt, vei;

    mi = 0;
    for ( ei = 0; ei < ecount; ei++ ) {
      ON_BrepEdge& edge = m_E[ei];
      if ( edge.m_edge_index == -1)
        emap[ei] = -1;
      else if ( edge.m_edge_index == ei )
        emap[ei] = edge.m_edge_index = mi++;
      else {
        ON_ERROR("Brep edge has illegal m_edge_index.");
        rc = FALSE;
        emap[ei] = edge.m_edge_index;
      }
    }

    if ( mi == 0 ) {
      m_E.Destroy();
    }
    else if ( mi < ecount ) {
      // remap edge indices
      for ( ei = ecount-1; ei >= 0; ei-- ) {
        if ( m_E[ei].m_edge_index == -1 ) {
          m_E.Remove(ei);
        }
        else {
          m_E[ei].m_edge_index = emap[ei];
        }
      }

      // remap trim.m_ei
      for ( ti = 0; ti < tcount; ti++ ) {
        ON_BrepTrim& trim = m_T[ti];
        ei = trim.m_ei;
        if ( ei < -1 || ei >= ecount ) {
          ON_ERROR("Brep trim.m_ei has illegal index.");
          rc = FALSE;
        }
        else {
          trim.m_ei = emap[ei];
        }
      }

      // remap vertex.m_ei[]
      for ( vi = 0; vi < vcount; vi++ ) {
        ON_BrepVertex& vertex = m_V[vi];
        vecnt = vertex.m_ei.Count();
        for ( vei = vecnt-1; vei >= 0; vei-- ) {
          ei = vertex.m_ei[vei];
          if ( ei < -1 || ei >= ecount ) {
            ON_ERROR("Brep vertex.m_ei[] has illegal index.");
            rc = FALSE;
          }
          else {
            ei = emap[ei];
            if (ei >= 0 ) {
              vertex.m_ei[vei] = ei;
            }
            else {
              vertex.m_ei.Remove(vei);
            }
          }
        }
      }
    }
  }
  m_E.Shrink();
  return rc;
}


BOOL ON_Brep::CullUnusedVertices()
{
  BOOL rc = TRUE;
  const int vcount = m_V.Count();
  if ( vcount > 0 ) {
    ON_Workspace ws;
    int* vmap = ws.GetIntMemory(vcount+1);
    *vmap++ = -1;
    memset(vmap,0,vcount*sizeof(*vmap));
    const int tcount = m_T.Count();
    const int ecount = m_E.Count();
    int vi, ei, ti, mi, j;

    mi = 0;
    for ( vi = 0; vi < vcount; vi++ ) {
      ON_BrepVertex& vertex = m_V[vi];
      if ( vertex.m_vertex_index == -1)
        vmap[vi] = -1;
      else if ( vertex.m_vertex_index == vi )
        vmap[vi] = vertex.m_vertex_index = mi++;
      else {
        ON_ERROR("Brep vertex has illegal m_vertex_index.");
        rc = FALSE;
        vmap[vi] = vertex.m_vertex_index;
      }
    }

    if ( mi == 0 ) {
      m_V.Destroy();
    }
    else if ( mi < ecount ) {
      // remap vertex indices
      for ( vi = vcount-1; vi >= 0; vi-- ) {
        if ( m_V[vi].m_vertex_index == -1 ) {
          m_V.Remove(vi);
        }
        else {
          m_V[vi].m_vertex_index = vmap[vi];
        }
      }

      // remap edge indices
      for ( ei = 0; ei < ecount; ei++ ) {
        ON_BrepEdge& edge = m_E[ei];
        for ( j = 0; j < 2; j++ ) {
          vi = edge.m_vi[j];
          if ( vi < -1 || vi >= vcount ) {
            ON_ERROR("Brep edge.m_vi[] has illegal index.");
            rc = FALSE;
          }
          else {
            edge.m_vi[j] = vmap[vi];
          }
        }
      }

      // remap trim indices
      for ( ti = 0; ti < tcount; ti++ ) {
        ON_BrepTrim& trim = m_T[ti];
        for ( j = 0; j < 2; j++ ) {
          vi = trim.m_vi[j];
          if ( vi < -1 || vi >= vcount ) {
            ON_ERROR("Brep trim.m_vi[] has illegal index.");
            rc = FALSE;
          }
          else {
            trim.m_vi[j] = vmap[vi];
          }
        }
      }

    }
  }
  m_V.Shrink();
  return rc;
}

BOOL ON_Brep::Compact()
{
  // Removes any unreferenced objects from arrays,
  // reindexes as needed, and shrinks arrays to
  // minimum required size.

  BOOL rc = TRUE;
  if (!CullUnusedFaces())
    rc = FALSE;
  if (!CullUnusedEdges())
    rc = FALSE;
  if (!CullUnusedVertices())
    rc = FALSE;
  if (!CullUnusedLoops())
    rc = FALSE;
  if (!CullUnusedTrims())
    rc = FALSE;

  if (!CullUnusedSurfaces())
    rc = FALSE;
  if (!CullUnused3dCurves())
    rc = FALSE;
  if (!CullUnused2dCurves())
    rc = FALSE;

  return rc;
}

ON_Brep::ON_Brep(const ON_Brep& src) : ON_Geometry(src)
{
  Initialize();
  *this = src;
}

ON_Brep& ON_Brep::operator=(const ON_Brep& src)
{
  if ( this != &src ) {
    Destroy();
    ON_Geometry::operator=(src);

    m_V.SetCapacity(src.m_V.Count());
    m_E.SetCapacity(src.m_E.Count());
    m_F.SetCapacity(src.m_F.Count());
    m_T.SetCapacity(src.m_T.Count());
    m_L.SetCapacity(src.m_L.Count());

    m_V.SetCount(src.m_V.Count());
    m_E.SetCount(src.m_E.Count());
    m_F.SetCount(src.m_F.Count());
    m_T.SetCount(src.m_T.Count());
    m_L.SetCount(src.m_L.Count());

    src.m_C2.Duplicate( m_C2 );
    src.m_C3.Duplicate( m_C3 );
    src.m_S.Duplicate( m_S );

    int count = m_V.Count();
    int i;
    for ( i = 0; i < count; i++ ) {
      m_V[i] = src.m_V[i];
    }
    count = m_E.Count();
    for ( i = 0; i < count; i++ ) {
      m_E[i] = src.m_E[i];
      ON_BrepEdge& e = m_E[i];
      e.m_brep = this;
      e.m_curve = ( e.m_c3i >= 0 ) ? m_C3[e.m_c3i] : 0;
    }
    count = m_F.Count();
    for ( i = 0; i < count; i++ ) {
      m_F[i] = src.m_F[i];
      ON_BrepFace& f = m_F[i];
      f.m_brep = this;
      f.m_surface = ( f.m_si >= 0 ) ? m_S[f.m_si] : 0;
    }
    count = m_T.Count();
    for ( i = 0; i < count; i++ ) {
      m_T[i] = src.m_T[i];
    }
    count = m_L.Count();
    for ( i = 0; i < count; i++ ) {
      m_L[i] = src.m_L[i];
    }

    m_bbox = src.m_bbox;
  }
  return *this;
}

void ON_Brep::Destroy()
{
  m_V.Empty();
  m_E.Empty();
  m_F.Empty();
  m_T.Empty();
  m_L.Empty();

  int i, count = m_C2.Count();
  for ( i = 0; i < count; i++ ) {
    delete m_C2[i];
    m_C2[i] = 0;
  }
  m_C2.Empty();
  m_C2.Zero();

  count = m_C3.Count();
  for ( i = 0; i < count; i++ ) {
    delete m_C3[i];
    m_C3[i] = 0;
  }
  m_C3.Empty();
  m_C3.Zero();

  count = m_S.Count();
  for ( i = 0; i < count; i++ ) {
    delete m_S[i];
    m_S[i] = 0;
  }
  m_S.Empty();
  m_S.Zero();

  m_bbox.Destroy();
  // returns Brep to state it has after default construction
}

void ON_Brep::EmergencyDestroy()
{ // call if memory pool used by b-rep members becomes invalid
  m_V.EmergencyDestroy();
  m_E.EmergencyDestroy();
  m_F.EmergencyDestroy();
  m_T.EmergencyDestroy();
  m_L.EmergencyDestroy();
  m_C2.EmergencyDestroy();
  m_C3.EmergencyDestroy();
  m_S.EmergencyDestroy();
  m_bbox.Destroy();
}

BOOL ON_Brep::CombineCoincidentVertices(ON_BrepVertex& vertex0, ON_BrepVertex& vertex1)
{

  BOOL rc = FALSE;
  if (&vertex0 == &vertex1) {
    ON_ERROR("ON_Brep::CombineCoincidentVertices - vertex0 = vertex1.");
    return rc;
  }
  // moves information to vertex0 and deletes vertex1
  int runaway, vei, vecnt, ei, eti, etcnt, ti, prev_ti, next_ti;

  if ( vertex0.m_vertex_index >= 0 && vertex1.m_vertex_index != vertex0.m_vertex_index ) {
    rc = TRUE;
    // update edges and trim references from vertex0 to vertex1
    vecnt = vertex1.m_ei.Count();
    for ( vei = 0; vei < vecnt; vei++ ) {
      ei = vertex1.m_ei[vei];
      if ( ei >= 0 ) {
        // update edge vertex indices
        ON_BrepEdge& edge = m_E[ei];
        if ( edge.m_vi[0] == vertex1.m_vertex_index )
          edge.m_vi[0] = vertex0.m_vertex_index;
        if ( edge.m_vi[1] == vertex1.m_vertex_index )
          edge.m_vi[1] = vertex0.m_vertex_index;

        // update trim vertex indices
        etcnt = edge.m_ti.Count();
        for (eti = 0; eti < etcnt; eti++ ) {
          ti = edge.m_ti[eti];
          if (ti >= 0 ) {
            ON_BrepTrim& trim = m_T[ti];
            if ( trim.m_vi[0] == vertex1.m_vertex_index ) {
              trim.m_vi[0] = vertex0.m_vertex_index;
              // check for previous singular trims using vertex0
              for (prev_ti = PrevTrim(ti), runaway=0;prev_ti >= 0 && prev_ti != ti && runaway < 1024;prev_ti=PrevTrim(prev_ti),runaway++) {
                ON_BrepTrim& prevtrim = m_T[prev_ti];
                if ( prevtrim.m_ei >= 0 )
                  break;
                if ( prevtrim.m_vi[0] == vertex1.m_vertex_index )
                  prevtrim.m_vi[0] = vertex0.m_vertex_index;
                if ( prevtrim.m_vi[1] == vertex1.m_vertex_index )
                  prevtrim.m_vi[1] = vertex0.m_vertex_index;
              }
            }
            if ( trim.m_vi[1] == vertex1.m_vertex_index ) {
              trim.m_vi[1] = vertex0.m_vertex_index;
               // check for previous singular trims using vertex0
              for (next_ti = NextTrim(ti), runaway=0;next_ti >= 0 && next_ti != ti && runaway < 1024;next_ti=NextTrim(next_ti),runaway++) {
                ON_BrepTrim& nexttrim = m_T[next_ti];
                if ( nexttrim.m_ei >= 0 )
                  break;
                if ( nexttrim.m_vi[0] == vertex1.m_vertex_index )
                  nexttrim.m_vi[0] = vertex0.m_vertex_index;
                if ( nexttrim.m_vi[1] == vertex1.m_vertex_index )
                  nexttrim.m_vi[1] = vertex0.m_vertex_index;
              }
            }
          }
        }
        vertex0.m_ei.Append(ei);
      }
    }
  }

  // update vertex tolerances
  if ( vertex0.m_tolerance != ON_UNSET_VALUE && vertex1.m_tolerance != ON_UNSET_VALUE && vertex0.m_tolerance < vertex1.m_tolerance )
    vertex0.m_tolerance = vertex1.m_tolerance + vertex0.point.DistanceTo(vertex1.point);

  vertex1.m_vertex_index = -1;
  vertex1.m_ei.Destroy();
  DeleteVertex(vertex1);

  return rc;
}

BOOL ON_Brep::CombineCoincidentEdges(ON_BrepEdge& edge0, ON_BrepEdge& edge1)
{
  BOOL rc = FALSE;
  if (&edge0 == &edge1) {
    ON_ERROR("ON_Brep::CombineCoincidentEdges - edge0 = edge1.");
    return rc;
  }
  int ti, eti, etcnt;
  if ( edge0.m_edge_index >= 0 && edge1.m_edge_index >= 0 && edge0.m_vi[0] == edge1.m_vi[0] && edge0.m_vi[1] == edge1.m_vi[1] ) {
    etcnt = edge1.m_ti.Count();
    int tcount = m_T.Count();
    for ( eti = 0; eti < etcnt; eti++ ) {
      ti = edge1.m_ti[eti];
      if ( ti >= 0 && ti < tcount ) {
        ON_BrepTrim& trim = m_T[ti];
        trim.m_ei = edge0.m_edge_index;
        edge0.m_ti.Append(ti);
        // TODO - tolerances ?
      }
    }
    edge1.m_ti.Destroy();
    DeleteEdge( edge1, FALSE );
  }
  return rc;
}

BOOL ON_Brep::Create( ON_NurbsSurface*& pS )
{
  Destroy();

  if (!pS) 
    return FALSE;
  double u[2], v[2];
  if (!pS->GetDomain(0, &u[0], &u[1])) 
      return FALSE;
  if (!pS->GetDomain(1, &v[0], &v[1])) 
      return FALSE;
  int vid[2][2];
  ON_3dPoint P;

  if (!pS->EvPoint(u[0], v[0], P)) return FALSE;
  vid[0][0] = NewVertex(P).m_vertex_index;

  if (pS->IsSingular(0) || pS->IsClosed(0)) vid[1][0] = vid[0][0];
  else {
    if (!pS->EvPoint(u[1], v[0], P)) return FALSE;
    vid[1][0] = NewVertex(P).m_vertex_index;
  }

  if (pS->IsSingular(1) || pS->IsClosed(1)) vid[1][1] = vid[1][0];
  else {
    if (!pS->EvPoint(u[1], v[1], P)) return FALSE;
    vid[1][1] = NewVertex(P).m_vertex_index;
  }

  if (pS->IsSingular(2) || pS->IsClosed(0)) vid[0][1] = vid[1][1];
  else if (pS->IsSingular(3) || pS->IsClosed(1)) vid[0][1] = vid[0][0];
  else {
    if (!pS->EvPoint(u[0], v[1], P)) return FALSE;
    vid[0][1] = NewVertex(P).m_vertex_index;
  }

  int sid = AddSurface(pS);
  ON_BrepFace& face = NewFace(sid);
  ON_BrepLoop& loop = NewLoop(ON_BrepLoop::outer, face);

  ON_Curve* pC3;
  ON_LineCurve* pC2;
  int id3[4], eid[4], id2;

  {//south side
    ON_3dPoint Start(u[0], v[0], 0.0);
    ON_3dPoint End(u[1], v[0], 0.0);
    ON_Line line(Start, End);
    pC2 = new ON_LineCurve(line);
    pC2->m_dim = 2;
    id2 = AddTrimCurve(pC2);
    if (pS->IsSingular(0)) {
      id3[0] = -1;
      ON_BrepTrim& trim = NewTrim(id2);
      trim.m_li = loop.m_loop_index;
      eid[0] = -1;
      trim.m_iso = ON_Surface::S_iso;
      trim.m_type = ON_BrepTrim::singular;
      trim.m_tolerance[0] = trim.m_tolerance[1] = 0.0;
      trim.m__legacy_2d_tol = 0.0;
      trim.m__legacy_3d_tol = 0.0;
      trim.m__legacy_flags_Set(-1,1);
      //trim.m_P[0] = pS->PointAt(Start.x,Start.y);
      //trim.m_P[1] = pS->PointAt(End.x,End.y);
    }
    else {
      pC3 = pS->IsoCurve(0, v[0]);
      id3[0] = AddEdgeCurve(pC3);
      ON_BrepEdge& edge = NewEdge(m_V[vid[0][0]], m_V[vid[1][0]], id3[0]);
      eid[0] = edge.m_edge_index;
      ON_BrepTrim& trim = NewTrim(edge, FALSE, loop, id2);
      trim.m_iso = ON_Surface::S_iso;
      if (pS->IsClosed(1)) trim.m_type = ON_BrepTrim::seam;
      else trim.m_type = ON_BrepTrim::boundary;
      trim.m_tolerance[0] = trim.m_tolerance[1] = 0.0;
      trim.m__legacy_2d_tol = 0.0;
      trim.m__legacy_3d_tol = 0.0;
      trim.m__legacy_flags_Set(-1,1);
      //trim.m_P[0] = pS->PointAt(Start.x,Start.y);
      //trim.m_P[1] = pS->PointAt(End.x,End.y);
    }
  }

  { //east side
    ON_3dPoint Start(u[1], v[0], 0.0);
    ON_3dPoint End(u[1], v[1], 0.0);
    ON_Line line(Start, End);
    pC2 = new ON_LineCurve(line);
    pC2->m_dim = 2;
    id2 = AddTrimCurve(pC2);
    if (pS->IsSingular(1)) {
      id3[1] = -1;
      ON_BrepTrim& trim = NewTrim(id2);
      trim.m_li = loop.m_loop_index;
      eid[1] = -1;
      trim.m_iso = ON_Surface::E_iso;
      trim.m_type = ON_BrepTrim::singular;
      trim.m_tolerance[0] = trim.m_tolerance[1] = 0.0;
      trim.m__legacy_2d_tol = 0.0;
      trim.m__legacy_3d_tol = 0.0;
      trim.m__legacy_flags_Set(-1,1);
      //trim.m_P[0] = pS->PointAt(Start.x,Start.y);
      //trim.m_P[1] = pS->PointAt(End.x,End.y);
    }
    else {
      pC3 = pS->IsoCurve(1, u[1]);
      id3[1] = AddEdgeCurve(pC3);
      ON_BrepEdge& edge = NewEdge(m_V[vid[1][0]], m_V[vid[1][1]], id3[1]);
      eid[1] = edge.m_edge_index;
      ON_BrepTrim& trim = NewTrim(edge, FALSE, loop, id2);
      trim.m_iso = ON_Surface::E_iso;
      if (pS->IsClosed(0)) trim.m_type = ON_BrepTrim::seam;
      else trim.m_type = ON_BrepTrim::boundary;
      trim.m_tolerance[0] = trim.m_tolerance[1] = 0.0;
      trim.m__legacy_2d_tol = 0.0;
      trim.m__legacy_3d_tol = 0.0;
      trim.m__legacy_flags_Set(-1,1);
      //trim.m_P[0] = pS->PointAt(Start.x,Start.y);
      //trim.m_P[1] = pS->PointAt(End.x,End.y);
    }
  }

  { //north side
    ON_3dPoint Start(u[1], v[1], 0.0);
    ON_3dPoint End(u[0], v[1], 0.0);
    ON_Line line(Start, End);
    pC2 = new ON_LineCurve(line);
    pC2->m_dim = 2;
    int id2 = AddTrimCurve(pC2);
    BOOL rev = FALSE;
    if (pS->IsSingular(2)) {
      id3[2] = -1;
      ON_BrepTrim& trim = NewTrim(id2);
      trim.m_li = loop.m_loop_index;
      eid[2] = -1;
      trim.m_iso = ON_Surface::N_iso;
      trim.m_type = ON_BrepTrim::singular;
      trim.m_tolerance[0] = trim.m_tolerance[1] = 0.0;
      trim.m__legacy_2d_tol = 0.0;
      trim.m__legacy_3d_tol = 0.0;
      trim.m__legacy_flags_Set(-1,1);
      //trim.m_P[0] = pS->PointAt(Start.x,Start.y);
      //trim.m_P[1] = pS->PointAt(End.x,End.y);
    }
    else{
      if (pS->IsClosed(1)) {
        id3[2] = id3[0];
        eid[2] = eid[0];
        rev = TRUE;
      }
      else {
        pC3 = pS->IsoCurve(0, v[1]);
        if (pC3) pC3->Reverse();
        id3[2] = AddEdgeCurve(pC3);
        ON_BrepEdge& edge = NewEdge(m_V[vid[1][1]], m_V[vid[0][1]], id3[2]);
        eid[2] = edge.m_edge_index;
      }

      ON_BrepTrim& trim = NewTrim(m_E[eid[2]], rev, loop, id2);
      trim.m_iso = ON_Surface::N_iso;
      if (pS->IsClosed(1)) trim.m_type = ON_BrepTrim::seam;
      else trim.m_type = ON_BrepTrim::boundary;
      trim.m_tolerance[0] = trim.m_tolerance[1] = 0.0;
      trim.m__legacy_2d_tol = 0.0;
      trim.m__legacy_3d_tol = 0.0;
      trim.m__legacy_flags_Set(-1,1);
      //trim.m_P[0] = pS->PointAt(Start.x,Start.y);
      //trim.m_P[1] = pS->PointAt(End.x,End.y);
    }
  }

  { //west side
    ON_3dPoint Start(u[0], v[1], 0.0);
    ON_3dPoint End(u[0], v[0], 0.0);
    ON_Line line(Start, End);
    pC2 = new ON_LineCurve(line);
    pC2->m_dim = 2;
    id2 = AddTrimCurve(pC2);
    BOOL rev = FALSE;
    if (pS->IsSingular(3)){
      id3[3] = -1;
      ON_BrepTrim& trim = NewTrim(id2);
      trim.m_li = loop.m_loop_index;
      eid[3] = -1;
      trim.m_iso = ON_Surface::W_iso;
      trim.m_type = ON_BrepTrim::singular;
      trim.m_tolerance[0] = trim.m_tolerance[1] = 0.0;
      trim.m__legacy_2d_tol = 0.0;
      trim.m__legacy_3d_tol = 0.0;
      trim.m__legacy_flags_Set(-1,1);
      //trim.m_P[0] = pS->PointAt(Start.x,Start.y);
      //trim.m_P[1] = pS->PointAt(End.x,End.y);
    }
    else {
      if (pS->IsClosed(0)){
      id3[3] = id3[1];
        eid[3] = eid[1];
        rev = TRUE;
      }
      else {
        pC3 = pS->IsoCurve(1, u[0]);
        if (pC3) pC3->Reverse();
        id3[3] = AddEdgeCurve(pC3);
        ON_BrepEdge& edge = NewEdge(m_V[vid[0][1]], m_V[vid[0][0]], id3[3]);
        eid[3] = edge.m_edge_index;
      }
      ON_BrepTrim& trim = NewTrim(m_E[eid[3]], rev, loop, id2);
      trim.m_iso = ON_Surface::W_iso;
      if (pS->IsClosed(0)) trim.m_type = ON_BrepTrim::seam;
      else trim.m_type = ON_BrepTrim::boundary;
      trim.m_tolerance[0] = trim.m_tolerance[1] = 0.0;
      trim.m__legacy_2d_tol = 0.0;
      trim.m__legacy_3d_tol = 0.0;
      trim.m__legacy_flags_Set(-1,1);
      //trim.m_P[0] = pS->PointAt(Start.x,Start.y);
      //trim.m_P[1] = pS->PointAt(End.x,End.y);
    }
  }

  pS = 0;

  return TRUE;
}


BOOL ON_Brep::HasBrepForm() const
{
  return TRUE;
}

ON_Brep* ON_Brep::BrepForm() const
{
  ON_Brep* brep = new ON_Brep(*this);
  return brep;
}
