/* $Header: /src2/opennurbs/opennurbs_circle.h 10    1/03/01 4:59p Dalelear $ */
/* $NoKeywords: $ */
/*
//
// Copyright (c) 1993-2001 Robert McNeel & Associates. All rights reserved.
// Rhinoceros is a registered trademark of Robert McNeel & Assoicates.
//
// THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
// ALL IMPLIED WARRANTIES OF FITNESS FOR ANY PARTICULAR PURPOSE AND OF
// MERCHANTABILITY ARE HEREBY DISCLAIMED.
//				
// For complete openNURBS copyright information see <http://www.opennurbs.org>.
//
////////////////////////////////////////////////////////////////
*/

#if !defined(ON_CIRCLE_INC_)
#define ON_CIRCLE_INC_

class ON_NurbsCurve;

class ON_CLASS ON_Circle
{
public:
  ON_Circle();

  ON_Circle(
    const ON_Plane&, // circle is in this plane with center at m_origin
    double            // radius
    );

  ON_Circle( // circle is parallel to XY plane
    const ON_3dPoint&, // center
    double               // radius
    );

  ON_Circle( // circle parallel to a plane
    const ON_Plane&,   // circle will be parallel to this plane
    const ON_3dPoint&, // center
    double               // radius
    );

  ON_Circle( // circle through 3 2d points
    const ON_2dPoint&, // point P
    const ON_2dPoint&, // point Q
    const ON_2dPoint&  // point R
    );

  ON_Circle( // circle through 3 3d points
    const ON_3dPoint&, // point P
    const ON_3dPoint&, // point Q
    const ON_3dPoint&  // point R
    );

  ~ON_Circle();

  BOOL Create(
    const ON_Plane&, // circle is in this plane with center at m_origin
    double            // radius
    );

  BOOL Create( // circle is parallel to XY plane
    const ON_3dPoint&, // center
    double               // radius
    );

  BOOL Create( // circle is parallel to plane
    const ON_Plane&,   // circle will be parallel to this plane
    const ON_3dPoint&, // center
    double               // radius
    );

  BOOL Create( // circle through 3 2d points
    const ON_2dPoint&, // point P
    const ON_2dPoint&, // point Q
    const ON_2dPoint&  // point R
    );

  BOOL Create( // circle through 3 3d points
    const ON_3dPoint&, // point P
    const ON_3dPoint&, // point Q
    const ON_3dPoint&  // point R
    );

  BOOL IsValid() const;

  BOOL UpdatePoints();  // sets m_point[] to have valid points

  BOOL IsInPlane( const ON_Plane&, double = ON_ZERO_TOLERANCE ) const;

  double Radius() const;
  double Diameter() const;
  double Circumference() const;
  const ON_3dPoint& Center() const;
  const ON_3dVector& Normal() const;
  const ON_Plane& Plane() const; // plane containing circle
  ON_BoundingBox BoundingBox() const;
  BOOL Transform( const ON_Xform& );

  // Circles use trigonometric parameterization
  // t -> center + cos(t)*radius*xaxis + sin(t)*radius*yaxis
  ON_3dPoint PointAt( 
    double // evaluation parameter
    ) const;
  ON_3dVector DerivativeAt( 
    int,   // derivative (>=0)
    double // evaluation parameter
    ) const;

  ON_3dVector TangentAt(double) const;

  // returns parameters of point on circle that is closest to given point
  BOOL ClosestPointTo( 
         const ON_3dPoint&, 
         double*
         ) const;
  // returns point on circle that is closest to given point
  ON_3dPoint ClosestPointTo( 
         const ON_3dPoint& 
         ) const;

  // evaluate circle's implicit equation in plane
  double EquationAt( const ON_2dPoint& ) const;
  ON_2dVector GradientAt( const ON_2dPoint& ) const;

  // rotate circle about its center
  BOOL Rotate(
        double,               // sin(angle)
        double,               // cos(angle)
        const ON_3dVector&  // axis of rotation
        );
  BOOL Rotate(
        double,               // angle in radians
        const ON_3dVector&  // axis of rotation
        );

  // rotate circle about a point and axis
  BOOL Rotate(
        double,               // sin(angle)
        double,               // cos(angle)
        const ON_3dVector&, // axis of rotation
        const ON_3dPoint&   // center of rotation
        );
  BOOL Rotate(
        double,              // angle in radians
        const ON_3dVector&, // axis of rotation
        const ON_3dPoint&   // center of rotation
        );

  BOOL Translate(
        const ON_3dVector&
        );

  BOOL Reverse();

  // parameterization of NURBS curve does not match circle's transcendental paramaterization
  int GetNurbForm( ON_NurbsCurve& ) const; // returns 0=failure, 2=success

public:
  // circle is in the plane with center at plane.m_origin.
  ON_Plane   plane;  
  double     radius;   // radius
  ON_3dPoint m_point[3]; // 3 points on the circle
};


#endif

