/* $Header: /src2/opennurbs/opennurbs_circlecurve.cpp 10    4/27/01 10:21a Dalelear $ */
/* $NoKeywords: $ */
/*
//
// Copyright (c) 1993-2001 Robert McNeel & Associates. All rights reserved.
// Rhinoceros is a registered trademark of Robert McNeel & Assoicates.
//
// THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
// ALL IMPLIED WARRANTIES OF FITNESS FOR ANY PARTICULAR PURPOSE AND OF
// MERCHANTABILITY ARE HEREBY DISCLAIMED.
//				
// For complete openNURBS copyright information see <http://www.opennurbs.org>.
//
////////////////////////////////////////////////////////////////
*/

#include "opennurbs.h"

ON_OBJECT_IMPLEMENT(ON_CircleCurve,ON_Curve,"CF33BE29-09B4-11d4-BFFB-0010830122F0");

ON_CircleCurve::ON_CircleCurve()
{
  m_circle.Create(ON_xy_plane,1.0);
  m_t.m_t[0] = 0.0;
  m_t.m_t[1] = m_circle.Circumference();
  m_dim = 3;
}

ON_CircleCurve::ON_CircleCurve( const ON_Circle& C )
{
  m_circle = C;
  m_t.m_t[0] = 0.0;
  m_t.m_t[1] = m_circle.Circumference();
  if ( m_t.m_t[1] == 0.0 )
    m_t.m_t[1] = 1.0;
  m_dim = 3;
}

ON_CircleCurve::ON_CircleCurve( const ON_Circle& C, double t0, double t1 )
{
  m_circle = C;
  m_t.m_t[0] = t0;
  m_t.m_t[1] = t1;
  m_dim = 3;
}

ON_CircleCurve::ON_CircleCurve( const ON_CircleCurve& src ) : ON_Curve(src)
{
  m_circle = src.m_circle;
  m_t = src.m_t;
  m_dim  = src.m_dim;
}

ON_CircleCurve::~ON_CircleCurve()
{
}

ON_CircleCurve& ON_CircleCurve::operator=( const ON_CircleCurve& src )
{
  if ( this != &src ) {
    ON_Curve::operator=(src);
    m_circle = src.m_circle;
    m_t = src.m_t;
    m_dim  = src.m_dim;
  }
  return *this;
}

ON_CircleCurve& ON_CircleCurve::operator=( const ON_Circle& C )
{
  m_circle = C;
  m_t.m_t[0] = 0.0;
  m_t.m_t[1] = C.Circumference();
  if ( m_t.m_t[1] == 0.0 )
    m_t.m_t[1] = 1.0;
  m_dim = 3;
  return *this;
}

int ON_CircleCurve::Dimension() const
{
  return m_dim;
}

BOOL 
ON_CircleCurve::GetBBox( // returns TRUE if successful
         double* boxmin,    // minimum
         double* boxmax,    // maximum
         BOOL bGrowBox
         ) const
{
  BOOL rc = m_circle.IsValid();
  if (rc) {
    ON_BoundingBox bbox = m_circle.BoundingBox();
    if ( bGrowBox ) {
      if ( boxmin[0] > bbox.m_min.x ) boxmin[0] = bbox.m_min.x;
      if ( boxmin[1] > bbox.m_min.y ) boxmin[1] = bbox.m_min.y;
      if ( boxmax[0] < bbox.m_max.x ) boxmax[0] = bbox.m_max.x;
      if ( boxmax[1] < bbox.m_max.y ) boxmax[1] = bbox.m_max.y;
      if ( m_dim > 2 ) {
        if ( boxmin[2] > bbox.m_min.z ) boxmin[2] = bbox.m_min.z;
        if ( boxmax[2] < bbox.m_max.z ) boxmax[2] = bbox.m_max.z;
      }
    }
    else {
      boxmin[0] = bbox.m_min.x;
      boxmin[1] = bbox.m_min.y;
      boxmax[0] = bbox.m_max.x;
      boxmax[1] = bbox.m_max.y;
      if ( m_dim > 2 ) {
        boxmin[2] = bbox.m_min.z;
        boxmax[2] = bbox.m_max.z;
      }
    }
  }
  return rc;
}

BOOL
ON_CircleCurve::Transform( const ON_Xform& xform )
{
  TransformUserData(xform);
  return m_circle.Transform( xform );
}

BOOL ON_CircleCurve::IsValid() const
{
  return ( m_t[0] < m_t[1] && m_circle.IsValid() > 0.0 ) ? TRUE : FALSE;
}

void ON_CircleCurve::Dump( ON_TextLog& dump ) const
{
  dump.Print( "ON_CircleCurve:  domain = [%g,%g]\n",m_t[0],m_t[1]);
  dump.PushIndent();
  dump.Print( "normal = ");
  dump.Print( m_circle.plane.zaxis );
  dump.Print("\n");
  dump.Print( "start/end = ");
  dump.Print( m_circle.plane.origin + m_circle.radius*m_circle.plane.xaxis );
  dump.Print("\n");
  dump.Print( "center = ");
  dump.Print( m_circle.plane.origin );
  dump.Print("\n");
  dump.Print( "radius = %g\n",m_circle.radius);
  dump.PopIndent();
}

BOOL ON_CircleCurve::Write(
       ON_BinaryArchive& file // open binary file
     ) const
{
  BOOL rc = file.Write3dmChunkVersion(1,0);
  if (rc) {
    rc = file.WriteCircle( m_circle );
    if (rc) rc = file.WriteInterval( m_t );
    if (rc) rc = file.WriteInt(m_dim);
  }
  return rc;
}

BOOL ON_CircleCurve::Read(
       ON_BinaryArchive& file // open binary file
     )
{
  int major_version = 0;
  int minor_version = 0;
  BOOL rc = file.Read3dmChunkVersion(&major_version,&minor_version);
  if (rc && major_version==1) {
    // common to all 1.x versions
    rc = file.ReadCircle( m_circle );
    if (rc) rc = file.ReadInterval( m_t );
    if (rc) rc = file.ReadInt(&m_dim);
  }
  return rc;
}

ON_Interval ON_CircleCurve::Domain() const
{
  return m_t;
}

int ON_CircleCurve::SpanCount() const
{
  return 1;
}

BOOL ON_CircleCurve::GetSpanVector( // span "knots" 
       double* s// array of length SpanCount() + 1 
       ) const
{
  s[0] = m_t[0];
  s[1] = m_t[1];
  return TRUE;
}

int ON_CircleCurve::Degree() const
{
  return 2;
}


BOOL
ON_CircleCurve::IsLinear(  // TRUE if curve locus is a line segment
      double tolerance   // tolerance to use when checking linearity
      ) const
{
  return FALSE;
}

BOOL
ON_CircleCurve::IsArc( // TRUE if curve locus in an arc or circle
      const ON_Plane* plane, // if not NULL, test is performed in this plane
      ON_Arc* arc,         // if not NULL and TRUE is returned, then arc
                              // arc parameters are filled in
      double tolerance // tolerance to use when checking linearity
      ) const
{
  if ( plane ) {
    if ( !IsInPlane(*plane,tolerance) )
      return FALSE;
  }
  if ( arc )
    *arc = m_circle;
  return IsValid();
}

BOOL
ON_CircleCurve::IsPlanar(
      ON_Plane* plane, // if not NULL and TRUE is returned, then plane parameters
                         // are filled in
      double tolerance // tolerance to use when checking linearity
      ) const
{
  if ( plane )
    *plane = m_circle.plane;
  return TRUE;
}

BOOL
ON_CircleCurve::IsInPlane(
      const ON_Plane& plane, // plane to test
      double tolerance // tolerance to use when checking linearity
      ) const
{
  return m_circle.IsInPlane( plane, tolerance );
  return FALSE;
}

BOOL 
ON_CircleCurve::IsClosed() const
{
  return TRUE;
}

BOOL 
ON_CircleCurve::IsPeriodic() const
{
  return FALSE;
}

BOOL
ON_CircleCurve::Reverse()
{
  BOOL rc = m_circle.Reverse();
  if (rc)
    m_t.Reverse();
  return TRUE;
}

BOOL ON_CircleCurve::Evaluate( // returns FALSE if unable to evaluate
       double t,       // evaluation parameter
       int der_count,  // number of derivatives (>=0)
       int v_stride,   // v[] array stride (>=Dimension())
       double* v,      // v[] array of length stride*(ndir+1)
       int side,       // optional - determines which side to evaluate from
                       //         0 = default
                       //      <  0 to evaluate from below, 
                       //      >  0 to evaluate from above
       int* hint       // optional - evaluation hint (int) used to speed
                       //            interative evaluations
       ) const
{
  BOOL rc = FALSE;
  if ( m_t[0] < m_t[1] ) {
    double s = (t == m_t[1]) ? 1.0 : (t-m_t[0])/(m_t[1]-m_t[0]);
    const ON_3dPoint p = m_circle.PointAt(s);
    v[0] = p.x;
    v[1] = p.y;
    if ( m_dim == 3 )
      v[2] = p.z;
    for ( int di = 1; di <= der_count; di++ ) {
      v += v_stride;
      v[0] = 0.0;
      v[1] = 0.0;
      if ( m_dim == 3 )
        v[2] = 0.0;
    }
    rc = TRUE;
  }
  return rc;
}

BOOL ON_CircleCurve::GetClosestPoint( const ON_3dPoint& test_point,
        double* t,       // parameter of local closest point returned here
        double maximum_distance,
        const ON_Interval* sub_domain
        ) const
{
  double s, d;
  BOOL rc = m_circle.ClosestPointTo( test_point, &s );
  if ( rc ) {
    s = m_t.ParameterAt(s*0.5/ON_PI);
    if ( sub_domain ) {
      if ( s < sub_domain->Min() )
        s = sub_domain->Min();
      else if ( s > sub_domain->Max() )
        s = sub_domain->Max();
    }
    if ( maximum_distance > 0.0 ) {
      d = test_point.DistanceTo(PointAt(s));
      if ( d > maximum_distance )
        rc = FALSE;
    }
    if (rc && t)
      *t = s;
  }
  return rc;
}

BOOL ON_CircleCurve::Length(
        double* length,               // length returned here
        double fractional_tolerance,  // default = 1.0e-8
        const ON_Interval* sub_domain // default = NULL
        ) const
{
  double a0, a1;
  if ( sub_domain ) {
    a0 = m_t.NormalizedParameterAt(sub_domain->Min())*2.0*ON_PI;
    a1 = m_t.NormalizedParameterAt(sub_domain->Max())*2.0*ON_PI;
  }
  else {
    a0 = 0.0;
    a1 = 2.0*ON_PI;
  }
  *length = fabs((a1-a0)*m_circle.radius);
  return TRUE;
}


BOOL ON_CircleCurve::GetLocalClosestPoint( const ON_3dPoint& test_point,
        double seed_parameter,
        double* t,
        const ON_Interval* sub_domain
        ) const
{
  return GetClosestPoint( test_point, t, 0.0, sub_domain );
}

int ON_CircleCurve::GetNurbForm( // returns 0: unable to create NURBS representation
                 //            with desired accuracy.
                 //         1: success - returned NURBS parameterization
                 //            matches the curve's to wthe desired accuracy
                 //         2: success - returned NURBS point locus matches
                 //            the curve's to the desired accuracy but, on
                 //            the interior of the curve's domain, the 
                 //            curve's parameterization and the NURBS
                 //            parameterization may not match to the 
                 //            desired accuracy.
      ON_NurbsCurve& c,
      double tolerance,
      const ON_Interval* subdomain  // OPTIONAL subdomain of circle
      ) const
{
  int rc = 0;
  if ( m_t.IsIncreasing() && m_circle.IsValid() ) {
    ON_ArcCurve arc;
    arc.m_arc = m_circle;
    arc.m_t = m_t;
    arc.m_dim = m_dim;
    if ( subdomain )
      arc.Trim(*subdomain);
    rc = arc.GetNurbForm(c,tolerance,NULL);
  }
  return rc;
}

