/* $Header: /src2/opennurbs/opennurbs_color.h 4     1/03/01 4:59p Dalelear $ */
/* $NoKeywords: $ */
/*
//
// Copyright (c) 1993-2001 Robert McNeel & Associates. All rights reserved.
// Rhinoceros is a registered trademark of Robert McNeel & Assoicates.
//
// THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
// ALL IMPLIED WARRANTIES OF FITNESS FOR ANY PARTICULAR PURPOSE AND OF
// MERCHANTABILITY ARE HEREBY DISCLAIMED.
//				
// For complete openNURBS copyright information see <http://www.opennurbs.org>.
//
////////////////////////////////////////////////////////////////
*/

////////////////////////////////////////////////////////////////
//
//   defines ON_Color and ON_Material
//
////////////////////////////////////////////////////////////////

#if !defined(OPENNURBS_COLOR_INC_)
#define OPENNURBS_COLOR_INC_

///////////////////////////////////////////////////////////////////////////////
//
// Class ON_Color
// 
class ON_CLASS ON_Color
{
public:
	// Constructors & Conversions -     also default copy and assignment	
	ON_Color();              // Default is black
	ON_Color(
    int, // red   ( 0 to 255 )
    int, // green ( 0 to 255 )
    int  // blue  ( 0 to 255 )
    );
	ON_Color(
    int, // red   ( 0 to 255 )
    int, // green ( 0 to 255 )
    int, // blue  ( 0 to 255 )
    int  // alpha ( 0 to 255 )
    );
	ON_Color(unsigned int);     // construct from COLORREF

	operator unsigned int() const;	// Conversion to COLORREF

  int Compare( const ON_Color& ) const; // < 0 if this < arg, 0 ir this==arg, > 0 if this > arg

	int Red()   const; // ( 0 to 255 )
	int Green() const; // ( 0 to 255 )
	int Blue()  const; // ( 0 to 255 )
  int Alpha() const; // ( 0 to 255 )

	double FractionRed()   const; // ( 0.0 to 1.0 )
	double FractionGreen() const; // ( 0.0 to 1.0 )
	double FractionBlue()  const; // ( 0.0 to 1.0 )
	double FractionAlpha() const; // ( 0.0 to 1.0 )

  void SetRGB(int,int,int); // 0 to 255
  void SetFractionalRGB(double,double,double); // 0.0 to 1.0

  void SetAlpha(int); // 0 to 255
  void SetFractionalAlpha(double); // 0.0 to 1.0

  void SetRGBA(int,int,int,int); // 0 to 255
  void SetFractionalRGBA(double,double,double,double); // 0.0 to 1.0

  double Hue() const;        // 0 to 2*pi 
                             // 0    = red,  pi/3   = yellow, 2*pi/3 = green, 
                             // pi   = cyan, 4*pi/3 = blue,   5*pi/3 = magenta,
                             // 2*pi = red
  double Saturation() const; // 0.0 to 1.0    0.0 = gray,  1.0 = saturated
  double Value() const;      // 0.0 to 1.0    0.0 = black, 1.0 = white
  void SetHSV( 
         double, // hue in radians 
         double, // satuation 0.0 = gray, 1.0 = saturated
         double  // value     
         );

private:
	unsigned int m_color;    // Stored in Windows COLORREF format
							 //  0x00bbggrr,  rr= red component 0-255, etc. (little endian order)
};

#endif
