/* $Header: /src2/opennurbs/opennurbs_cone.h 4     1/03/01 4:59p Dalelear $ */
/* $NoKeywords: $ */
/*
//
// Copyright (c) 1993-2001 Robert McNeel & Associates. All rights reserved.
// Rhinoceros is a registered trademark of Robert McNeel & Assoicates.
//
// THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
// ALL IMPLIED WARRANTIES OF FITNESS FOR ANY PARTICULAR PURPOSE AND OF
// MERCHANTABILITY ARE HEREBY DISCLAIMED.
//				
// For complete openNURBS copyright information see <http://www.opennurbs.org>.
//
////////////////////////////////////////////////////////////////
*/

#if !defined(ON_CONE_INC_)
#define ON_CONE_INC_

class ON_CLASS ON_Cone
{
public:
  ON_Cone(); // creates world XY cone with origin at 0,0,0

  ON_Cone(
    const ON_Ellipse&,  // base ellipse
    double              // height
    );
  ON_Cone(
    const ON_Circle&,   // base circle
    double              // height
    );

  ~ON_Cone();

  BOOL Create(
    const ON_Circle&,   // base ellipse
    double              // height
    );
  BOOL Create(
    const ON_Ellipse&,  // base ellipse
    double              // height
    );

  BOOL IsValid() const; // returns TRUE if all fields contain reasonable
                        // information and equation jibes with point and Z.

  const ON_3dPoint& BasePoint() const;
  const ON_3dVector& Axis() const; // length = |height|
  double MajorAngleInRadians(); // largest angle between axis and side (same sign as height)
  double MajorAngleInDegrees();           
  double MinorAngleInRadians(); // smallest angle between axis and side (same sign as height)
  double MinorAngleInDegrees();           

  // equation of cone: e[0]*X*X + e[1]*Y*Y + e[2]*Z + e[3] = 0
  const double* Equation() const; 

  // evaluate parameters and return point
  ON_3dPoint PointAt(
    double, // radial parameter 0 to 2*pi
    double  // height 0 = base 1 = apex
    ) const;

  ON_3dVector NormalAt(
    double, // radial parameter 0 to 2*pi
    double  // height 0 = base 1 = apex
    ) const;

  ON_Ellipse IsoEllipse( double ) const; // isoparametric ellipse at specified height
  ON_Line IsoLine( double ) const; // isoparametric circle at specified angle

  // evaluate cone's implicit equation
  double EquationAt( const ON_3dPoint& ) const;
  double EquationAt( const ON_4dPoint& ) const;

  // returns parameters of point on cone that is closest to given point
  BOOL ClosestPointTo( 
         const ON_3dPoint&, 
         double*, double*
         ) const;

  // returns point on cone that is closest to given point
  ON_3dPoint ClosestPointTo( 
         const ON_3dPoint& 
         ) const;

  BOOL Transform( const ON_Xform& );

  // rotate cone about its origin
  BOOL Rotate(
        double,               // sin(angle)
        double,               // cos(angle)
        const ON_3dVector&  // axis of rotation
        );
  BOOL Rotate(
        double,               // angle in radians
        const ON_3dVector&  // axis of rotation
        );

  // rotate cone about a point and axis
  BOOL Rotate(
        double,               // sin(angle)
        double,               // cos(angle)
        const ON_3dVector&, // axis of rotation
        const ON_3dPoint&   // center of rotation
        );
  BOOL Rotate(
        double,              // angle in radians
        const ON_3dVector&, // axis of rotation
        const ON_3dPoint&   // center of rotation
        );

  BOOL Translate(
        const ON_3dVector&
        );

  BOOL GetNurbForm( ON_NurbsSurface& ) const;

public: // members left public
  ON_Ellipse m_ellipse;
  double     m_height;  // distance from 
  // equation of cone: e[0]*X*X + e[1]*Y*Y + e[2]*Z + e[3] = 0 in ellipse plane coordinates
  double m_equation[4];
};

#endif
