/* $Header: /src2/opennurbs/opennurbs_curveproxy.cpp 9     4/27/01 10:21a Dalelear $ */
/* $NoKeywords: $ */
/*
//
// Copyright (c) 1993-2001 Robert McNeel & Associates. All rights reserved.
// Rhinoceros is a registered trademark of Robert McNeel & Assoicates.
//
// THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
// ALL IMPLIED WARRANTIES OF FITNESS FOR ANY PARTICULAR PURPOSE AND OF
// MERCHANTABILITY ARE HEREBY DISCLAIMED.
//				
// For complete openNURBS copyright information see <http://www.opennurbs.org>.
//
////////////////////////////////////////////////////////////////
*/

#include "opennurbs.h"

ON_OBJECT_IMPLEMENT(ON_CurveProxy,ON_Curve,"4ED7D4D9-E947-11d3-BFE5-0010830122F0");

ON_CurveProxy::ON_CurveProxy() : m_curve(0), m_bReversed(0)
{}

ON_CurveProxy::ON_CurveProxy( const ON_CurveProxy& src ) : ON_Curve(src), m_curve(0), m_bReversed(0)
{
  *this = src;
}

ON_CurveProxy::ON_CurveProxy( const ON_Curve* c ) : m_curve(c), m_bReversed(0)
{
  if ( m_curve )
    m_domain = m_curve->Domain();
}

ON_CurveProxy::ON_CurveProxy( const ON_Curve* c, ON_Interval domain ) : m_curve(c), m_bReversed(0), m_domain(domain)
{}

ON_CurveProxy& ON_CurveProxy::operator=( const ON_CurveProxy& src )
{
  if ( this != &src ) {
    ON_Curve::operator=(src);
    m_curve = src.m_curve;
    m_bReversed = src.m_bReversed;
    m_domain = src.m_domain;
  }
  return *this;
}

ON_CurveProxy::~ON_CurveProxy()
{
  m_curve = 0;
}

BOOL
ON_CurveProxy::IsValid() const
{
  BOOL rc = ( m_curve ) ? m_curve->IsValid() : FALSE;
  if ( rc ) {
    if ( !m_domain.IsIncreasing() || !m_curve->Domain().Includes( m_domain ) ) {
      rc = FALSE;
    }
  }
  return rc;
}

void
ON_CurveProxy::Dump( ON_TextLog& dump ) const
{
  dump.Print("ON_CurveProxy uses %x on [%g,%g]\n",m_curve,m_domain[0],m_domain[1]);
}

BOOL 
ON_CurveProxy::Write(
       ON_BinaryArchive&  // open binary file
     ) const
{
  return FALSE;
}

BOOL 
ON_CurveProxy::Read(
       ON_BinaryArchive&  // open binary file
     )
{
  return FALSE;
}

int 
ON_CurveProxy::Dimension() const
{
  return ( m_curve ) ? m_curve->Dimension() : FALSE;
}

BOOL 
ON_CurveProxy::GetBBox( // returns TRUE if successful
         double* boxmin,    // minimum
         double* boxmax,    // maximum
         BOOL bGrowBox
         ) const
{
  return ( m_curve ) ? m_curve->GetBBox(boxmin,boxmax,bGrowBox) : FALSE;
}

BOOL
ON_CurveProxy::Transform( const ON_Xform& )
{
  return FALSE; // cannot modify proxy objects
}

ON_Interval ON_CurveProxy::Domain() const
{
  BOOL rc = FALSE;
  ON_Interval d;
  if ( m_curve ) {
    d = m_curve->Domain();
    if ( d.IsIncreasing() ) {
      rc = TRUE;
      if ( m_bReversed ) {
        d.Reverse();
      }
    }
  }
  return d;
}

int ON_CurveProxy::SpanCount() const
{
  return m_curve ? m_curve->SpanCount() : 0;
}

BOOL ON_CurveProxy::GetSpanVector( double* d ) const
{
  BOOL rc = m_curve ? m_curve->GetSpanVector(d) : FALSE;
  if (rc && m_bReversed ) {
    double x;
    int i, j;
    for ( i = 0, j = SpanCount()-1; i <= j; i++, j-- ) {
      x = d[i];
      d[i] = -d[j];
      d[j] = -x;
    }
  }
  return rc;
}

int ON_CurveProxy::Degree() const
{
  return m_curve ? m_curve->Degree() : 0;
}

BOOL 
ON_CurveProxy::GetParameterTolerance(
         double t,  // t = parameter in domain
         double* tminus, // tminus
         double* tplus   // tplus
         ) const
{
  return ( m_curve ) ? m_curve->GetParameterTolerance(m_bReversed?-t:t,tminus,tplus) : FALSE;
}


BOOL
ON_CurveProxy::IsLinear( // TRUE if curve locus is a line segment
      double tolerance // tolerance to use when checking linearity
      ) const
{
  return ( m_curve ) ? m_curve->IsLinear(tolerance) : FALSE;
}

BOOL
ON_CurveProxy::IsArc( // TRUE if curve locus in an arc or circle
      const ON_Plane* plane, // if not NULL, test is performed in this plane
      ON_Arc* arc,         // if not NULL and TRUE is returned, then arc
                              // arc parameters are filled in
      double tolerance // tolerance to use when checking linearity
      ) const
{
  BOOL rc = ( m_curve ) ? m_curve->IsArc(plane,arc,tolerance) : FALSE;
  if ( rc && arc && m_bReversed ) {
    arc->Reverse();
  }
  return rc;
}

BOOL
ON_CurveProxy::IsPlanar(
      ON_Plane* plane, // if not NULL and TRUE is returned, then plane parameters
                         // are filled in
      double tolerance // tolerance to use when checking linearity
      ) const
{
  return ( m_curve ) ? m_curve->IsPlanar(plane,tolerance) : FALSE;
}

BOOL
ON_CurveProxy::IsInPlane(
      const ON_Plane& plane, // plane to test
      double tolerance // tolerance to use when checking linearity
      ) const
{
  return ( m_curve ) ? m_curve->IsInPlane(plane,tolerance) : FALSE;
}

BOOL 
ON_CurveProxy::IsClosed() const
{
  return ( m_curve ) ? m_curve->IsClosed() : FALSE;
}

BOOL 
ON_CurveProxy::IsPeriodic() const
{
  return ( m_curve ) ? m_curve->IsPeriodic() : FALSE;
}

BOOL
ON_CurveProxy::Reverse()
{
  m_bReversed = (m_bReversed) ? FALSE : TRUE;
  return TRUE;
}

BOOL 
ON_CurveProxy::Evaluate( // returns FALSE if unable to evaluate
       double t,       // evaluation parameter
       int der_count,  // number of derivatives (>=0)
       int v_stride,   // v[] array stride (>=Dimension())
       double* v,      // v[] array of length stride*(ndir+1)
       int side,       // optional - determines which side to evaluate from
                       //         0 = default
                       //      <  0 to evaluate from below, 
                       //      >  0 to evaluate from above
       int* hint       // optional - evaluation hint (int) used to speed
                       //            interative evaluations
       ) const
{
  if ( m_bReversed )
    t = -t;
  BOOL rc = ( m_curve ) ? m_curve->Evaluate(t,der_count,v_stride,v,side,hint) : FALSE;
  if ( rc && m_bReversed ) {
    // negate odd derivatives
    const int dim = m_curve->Dimension();
    int di, i;
    for ( di = 1; di <= der_count; di++ ) {
      v += v_stride;
      for ( i = 0; i < dim; i++ ) {
        v[i] = -v[i];
      }
      v += v_stride;
    }
  }
  return rc;
}

BOOL ON_CurveProxy::GetClosestPoint( const ON_3dPoint& test_point,
        double* t,       // parameter of local closest point returned here
        double maximum_distance,
        const ON_Interval* sub_domain
        ) const
{
  BOOL rc = FALSE;
  if ( m_curve ) {
    ON_Interval scratch_domain;
    if ( sub_domain && m_bReversed ) {
      scratch_domain = *sub_domain;
      scratch_domain.Reverse();
      sub_domain = &scratch_domain;
    }
    rc = m_curve->GetClosestPoint( test_point, t, maximum_distance, sub_domain );
    if ( rc && m_bReversed && t )
      *t = -(*t);
  }
  return rc;
}

BOOL ON_CurveProxy::Length(
        double* length,               // length returned here
        double fractional_tolerance,  // default = 1.0e-8
        const ON_Interval* sub_domain // default = NULL
        ) const
{
  // override when possible
  // TODO: add simple integration routine that works with C1 curves
  if ( length )
    *length = 0;
  ON_Interval scratch_domain;
  if ( sub_domain && m_bReversed ) {
    scratch_domain = *sub_domain;
    scratch_domain.Reverse();
    sub_domain = &scratch_domain;
  }
  return m_curve->Length( length, fractional_tolerance, sub_domain );
}

BOOL ON_CurveProxy::GetLocalClosestPoint( const ON_3dPoint& test_point,
        double seed_parameter,
        double* t,
        const ON_Interval* sub_domain
        ) const
{
  BOOL rc = FALSE;
  if ( m_curve ) {
    ON_Interval scratch_domain;
    if ( sub_domain && m_bReversed ) {
      scratch_domain = *sub_domain;
      scratch_domain.Reverse();
      sub_domain = &scratch_domain;
    }
    if ( m_bReversed )
      seed_parameter = -seed_parameter;
    rc = m_curve->GetLocalClosestPoint( test_point, seed_parameter, t, sub_domain );
    if ( rc && m_bReversed && t )
      *t = -(*t);
  }
  return rc;
}


int 
ON_CurveProxy::GetNurbForm( // returns 0: unable to create NURBS representation
                 //            with desired accuracy.
                 //         1: success - returned NURBS parameterization
                 //            matches the curve's to wthe desired accuracy
                 //         2: success - returned NURBS point locus matches
                 //            the curve's to the desired accuracy but, on
                 //            the interior of the curve's domain, the 
                 //            curve's parameterization and the NURBS
                 //            parameterization may not match to the 
                 //            desired accuracy.
      ON_NurbsCurve& nurbs,
      double tolerance,  // (>=0)
      const ON_Interval* subdomain  // OPTIONAL subdomain of ON::ProxyCurve::Domain()
      ) const
{
  BOOL rc = FALSE;
  if ( m_curve ) {
    ON_Interval dom = subdomain ? *subdomain : m_domain;
    if ( m_bReversed )
      dom.Reverse();
    rc = m_curve->GetNurbForm(nurbs,tolerance,&dom);
    if ( rc && m_bReversed )
      rc = nurbs.Reverse();
  }
  return rc;
}
