/* $Header: /src2/opennurbs/opennurbs_defines.cpp 27    1/03/01 4:59p Dalelear $ */
/* $NoKeywords: $ */
/*
//
// Copyright (c) 1993-2001 Robert McNeel & Associates. All rights reserved.
// Rhinoceros is a registered trademark of Robert McNeel & Assoicates.
//
// THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
// ALL IMPLIED WARRANTIES OF FITNESS FOR ANY PARTICULAR PURPOSE AND OF
// MERCHANTABILITY ARE HEREBY DISCLAIMED.
//				
// For complete openNURBS copyright information see <http://www.opennurbs.org>.
//
////////////////////////////////////////////////////////////////
*/

#include "opennurbs.h"

// on_stricmp() is a wrapper for case insensitive string compare
// and calls one of _stricmp(), stricmp(), or strcasecmp()
// depending on OS.
int on_stricmp(const char * s1, const char * s2)
{
#if defined(ON_OS_WINDOWS)
  //return stricmp(s1,s2);
  return _stricmp(s1,s2);
#else
  return strcasecmp(s1,s2);
#endif
}

// on_strupr() calls _strupr() or strupr() depending on OS
char* on_strupr(char* s)
{
#if defined(ON_OS_WINDOWS)
  return _strupr(s); // ANSI name
#else
  if (s) {
    while (*s) {
      *s = toupper(*s);
      s++;
    }
  }
  return s;
#endif
}

// on_strlwr() calls _strlwr() or strlwr() depending on OS
char* on_strlwr(char* s)
{
#if defined(ON_OS_WINDOWS)
  return _strlwr(s); // ANSI name
#else
  if (s) {
    while (*s) {
      *s = tolower(*s);
      s++;
    }
  }
  return s;
#endif
}

// on_strrev() calls _strrev() or strrev() depending on OS
char* on_strrev(char* s)
{
#if defined(ON_OS_WINDOWS)
  return _strrev(s); // ANSI name
#else
  int i, j;
  char c;
  for ( i = 0, j = strlen(s)-1; i < j; i++, j-- ) {
    c = s[i];
    s[i] = s[j];
    s[j] = c;
  }
  return s;
#endif
}

int on_wcsicmp( const wchar_t* s1, const wchar_t* s2)
{
  if ( !s1 ) {
    return s2 ? -1 : 0;
  }
  else if ( !s2 ) {
    return 1;
  }

#if defined(ON_OS_WINDOWS)
  return wcsicmp(s1,s2);
#else
  return wcscasecmp(s1,s2);
#endif
}

wchar_t* on_wcsupr(wchar_t* s)
{
#if defined(ON_OS_WINDOWS)
  return _wcsupr(s);
#else
  if (s) {
    while (*s) {
      *s = toupper(*s);
      s++;
    }
  }
  return s;
#endif
}

// on_wcslwr() calls _wcslwr() or wcslwr() depending on OS
wchar_t* on_wcslwr(wchar_t* s)
{
#if defined(ON_OS_WINDOWS)
  return _wcslwr(s);
#else
  if (s) {
    while (*s) {
      *s = tolower(*s);
      s++;
    }
  }
  return s;
#endif
}

// on_wcsrev() calls _wcsrev() or wcsrev() depending on OS
wchar_t* on_wcsrev(wchar_t* s)
{
  if ( !s )
    return 0;
#if defined(ON_OS_WINDOWS)
  return _wcsrev(s);
#else
  int i, j;
  wchar_t w;
  for ( i = 0, j = wcslen(s)-1; i < j; i++, j-- ) {
    w = s[i];
    s[i] = s[j];
    s[j] = w;
  }
  return s;
#endif
}

int on_WideCharToMultiByte(
    const wchar_t* lpWideCharStr,
    int cchWideChar,
    char* lpMultiByteStr,
    int cchMultiByte
    )
{
#if defined(ON_OS_WINDOWS)
  return ::WideCharToMultiByte( CP_ACP, 0, 
                        lpWideCharStr, cchWideChar, 
                        lpMultiByteStr, cchMultiByte,
                        NULL, NULL );
#else
  // use simple wchar_t -> char conversion since 
  // wcsnrtombs() and wcsrtombs() tend to crash on various
  // UNIX platforms.

  // this union is used to get around issues invovling signed/unsigned
  // behavior of "char" on various platforms and with various
  // compiler flags.
  union
  {
    char c;
    unsigned char u;
  } u;
  int i;

  if ( cchMultiByte > 0 && lpMultiByteStr ) {
    for (i = 0; i < cchWideChar && i < cchMultiByte; i++ ) {
      unsigned int w = lpWideCharStr[i];
      if ( w >= 256 )
        w = '_'; // default is underbar
      u.u = (unsigned char)w;
      lpMultiByteStr[i] = u.c;
    }
    if ( i < cchMultiByte )
      lpMultiByteStr[i] = 0;
  }
  return cchWideChar; // return number of characters required for conversion

  /*
#if defined(__USE_GNU)
  // gcc extension is what we really need
  return wcsnrtombs( lpMultiByteStr, &lpWideCharStr, cchWideChar, cchMultiByte, NULL );
#else
  // see http://www.datafocus.com/docs/man3/wcsrtombs.3.asp
  return wcsrtombs( lpMultiByteStr, &lpWideCharStr, cchMultiByte, NULL );
#endif
  */

#endif
}

int on_MultiByteToWideChar(
    const char* lpMultiByteStr,
    int cchMultiByte,
    wchar_t* lpWideCharStr,
    int cchWideChar
    )
{
#if defined(ON_OS_WINDOWS)
  return ::MultiByteToWideChar(CP_ACP, 0, lpMultiByteStr, cchMultiByte, lpWideCharStr, cchWideChar);
#else
  // use simple char -> wchar_t conversion since 
  // mbsnrtowcs() and mbsrtowcs() tend to crash on various
  // UNIX platforms.
  union
  {
    char c;
    unsigned char u;
  } u;
  int i;

  if ( cchWideChar > 0 && lpWideCharStr ) {
    for (i = 0; i < cchMultiByte && i < cchWideChar; i++ ) {
      u.c = lpMultiByteStr[i];
      lpWideCharStr[i] = u.u;
    }
    if ( i < cchWideChar )
      lpWideCharStr[i] = 0;
  }
  return cchMultiByte; // return number of characters required for conversion

  /*
#if defined(__USE_GNU)
  // gcc extension is what we really need
  return mbsnrtowcs(lpWideCharStr, &lpMultiByteStr, cchMultiByte, cchWideChar, NULL );
#else
  // see http://www.datafocus.com/docs/man3/mbsrtowcs.3.asp
  return mbsrtowcs(lpWideCharStr, &lpMultiByteStr, cchWideChar, NULL );
#endif
  */
#endif
}


int on_vsnprintf( char *buffer, size_t count, const char *format, va_list argptr )
{
#if defined(ON_OS_WINDOWS)
  return _vsnprintf( buffer, count, format, argptr );
#else
  return vsnprintf( buffer, count, format, argptr );
#endif
}

int on_vsnwprintf( wchar_t *buffer, size_t count, const wchar_t *format, va_list argptr )
{
#if defined(ON_OS_WINDOWS)
  return _vsnwprintf( buffer, count, format, argptr );
#else
  // if an OS doesn't support vsnwprintf(), use ASCII and hope for the best

  ON_String aformat = format; // convert format from UNICODE to ASCII

  // format an ASCII buffer
  char* abuffer = (char*)onmalloc(2*count*sizeof(*abuffer));
  int rc = on_vsnprintf( abuffer, 2*count, aformat.Array(), argptr );

  // convert formatted ASCII buffer to UNICODE
  on_MultiByteToWideChar( abuffer, strlen(abuffer), buffer, count );
  onfree(abuffer);  
  return rc;
#endif
}

int ON::Version()
{
#define OPENNURBS_VERSION_DEFINITION
#include "opennurbs_version.h"
  return OPENNURBS_VERSION;
#undef OPENNURBS_VERSION
#undef OPENNURBS_VERSION_DEFINITION
}


FILE* ON::OpenFile( // like fopen() - needed when OpenNURBS is used as a DLL
        const char* filename, // file name
        const char* filemode // file mode
        )
{
  return (filename && filename[0] && filemode && filemode[0]) ? fopen(filename,filemode) : 0;
}

int ON::CloseFile( // like fclose() - needed when OpenNURBS is used as a DLL
        FILE* fp // pointer returned by OpenFile()
        )
{
  return fp ? fclose(fp) : EOF;
}

int ON::CloseAllFiles()
{
  // returns number of files closed or EOF for error
#if defined(ON_OS_WINDOWS)
  return _fcloseall(); // ANSI C name
#else
  // I can't find an fcloseall() or _fcloseall() in
  // gcc version egcs-2.91.66 19990314/Linux (egcs-1.1.2 release)
  return EOF;
#endif
}

ON::unit_system ON::UnitSystem(int i)
{
  unit_system us = no_unit_system;
  switch(i) {
  case no_unit_system: us = no_unit_system; break;
  case microns: us = microns; break;
  case millimeters: us = millimeters; break;
  case centimeters: us = centimeters; break;
  case meters: us = meters; break;
  case kilometers: us = kilometers; break;
  case microinches: us = microinches; break;
  case mils: us = mils; break;
  case inches: us = inches; break;
  case feet: us = feet; break;
  case miles: us = miles; break;
  case custom_unit_system: us = custom_unit_system; break;
  default: us = no_unit_system; break;
  }
  return us;
}

double ON::UnitScale(
            ON::unit_system u0, // from
            ON::unit_system u1  // to
            )
{
  // Scale factor for changing unit systems
  // example 
  //  2.54 = UnitScale( ON::inches, ON::centimeters ) 
  //  12.0 = UnitScale( ON::feet, ON::inches ) 

  // the default cases are here to keep lint quiet
  double scale = 1.0;
  
  switch( u0 ) 
  {
  case microns:
    switch( u1 ) {
    case microns:     scale = 1.0;    break;
    case millimeters: scale = 1.0e-3; break;
    case centimeters: scale = 1.0e-4; break;
    case meters:      scale = 1.0e-6; break;
    case kilometers:  scale = 1.0e-9; break;
    case microinches:
    case mils:
    case inches:
    case feet:
    case miles:
      scale = UnitScale( inches, u1 )*1.0e-4/2.54;
      break;
    default:
      break;
    }
    break;

  case millimeters:
    switch( u1 ) {
    case microns:     scale = 1.0e+3; break;
    case millimeters: scale = 1.0;    break;
    case centimeters: scale = 1.0e-1; break;
    case meters:      scale = 1.0e-3; break;
    case kilometers:  scale = 1.0e-6; break;
    case microinches:
    case mils:
    case inches:
    case feet:
    case miles:
      scale = UnitScale( inches, u1 )*1.0e-1/2.54;
      break;
    default:
      break;
    }
    break;

  case centimeters:
    switch( u1 ) {
    case microns:     scale = 1.0e+4; break;
    case millimeters: scale = 1.0e+1; break;
    case centimeters: scale = 1.0;    break;
    case meters:      scale = 1.0e-2; break;
    case kilometers:  scale = 1.0e-5; break;
    case microinches:
    case mils:
    case inches:
    case feet:
    case miles:
      scale = UnitScale( inches, u1 )/2.54;
      break;
    default:
      break;
    }
    break;

  case meters:
    switch( u1 ) {
    case microns:     scale = 1.0e+6; break;
    case millimeters: scale = 1.0e+3; break;
    case centimeters: scale = 1.0e+2; break;
    case meters:      scale = 1.0;    break;
    case kilometers:  scale = 1.0e-3; break;
    case microinches:
    case mils:
    case inches:
    case feet:
    case miles:
      scale = UnitScale( inches, u1 )*100.0/2.54;
      break;
    default:
      break;
    }
    break;

  case kilometers:
    switch( u1 ) {
    case microns:     scale = 1.0e+9; break;
    case millimeters: scale = 1.0e+6; break;
    case centimeters: scale = 1.0e+5; break;
    case meters:      scale = 1.0e+3; break;
    case kilometers:  scale = 1.0;    break;
    case microinches:
    case mils:
    case inches:
    case feet:
    case miles:
      scale = UnitScale( inches, u1 )*1.0e5/2.54;
      break;
    default:
      break;
    }
    break;

  case microinches:
    switch( u1 ) {
    case microns:
    case millimeters:
    case centimeters:
    case meters:
    case kilometers:
      scale = UnitScale( centimeters, u1 )*2.54e-6;
      break;
    case microinches: scale = 1.0; break;
    case mils:        scale = 1.0e-3; break;
    case inches:      scale = 1.0e-6; break;
    case feet:        scale = 1.0e-6/12.0; break;
    case miles:       scale = 1.0e-6/(12.0*5280.0); break;
    default:
      break;
    }
    break;

  case mils:
    switch( u1 ) {
    case microns:
    case millimeters:
    case centimeters:
    case meters:
    case kilometers:
      scale = UnitScale( centimeters, u1 )*2.54e-3;
      break;
    case microinches: scale = 1.0e+3; break;
    case mils:        scale = 1.0; break;
    case inches:      scale = 1.0e-3; break;
    case feet:        scale = 1.0e-3/12.0; break;
    case miles:       scale = 1.0e-3/(12.0*5280.0); break;
    default:
      break;
    }
    break;

  case inches:
    switch( u1 ) {
    case microns:
    case millimeters:
    case centimeters:
    case meters:
    case kilometers:
      scale = UnitScale( centimeters, u1 )*2.54;
      break;
    case microinches: scale = 1.0e+6;            break;
    case mils:        scale = 1.0e+3;            break;
    case inches:      scale = 1.0;               break;
    case feet:        scale = 1.0/12.0;          break;
    case miles:       scale = 1.0/(12.0*5280.0); break;
    default:
      break;
    }
    break;

  case feet:
    switch( u1 ) {
    case microns:
    case millimeters:
    case centimeters:
    case meters:
    case kilometers:
      scale = UnitScale( centimeters, u1 )*2.54*12.0;
      break;
    case microinches: scale = 1.0e+6*12.0; break;
    case mils:        scale = 1.0e+3*12.0; break;
    case inches:      scale = 12.0; break;
    case feet:        scale = 1.0; break;
    case miles:       scale = 1.0/5280.0; break;
    default:
      break;
    }
    break;

  case miles:
    switch( u1 ) {
    case microns:
    case millimeters:
    case centimeters:
    case meters:
    case kilometers:
      scale = UnitScale( centimeters, u1 )*2.54*12.0*5280.0;
      break;
    case microinches: scale = 1.0e+6*12.0*5280.0; break;
    case mils:        scale = 1.0e+3*12.0*5280.0; break;
    case inches:      scale = 12.0*5280.0;        break;
    case feet:        scale = 5280.0;             break;
    case miles:       scale = 1.0;                break;
    default:
      break;
    }
    break;

  default:
    break;
  }

  return scale;
}


  //// distance_display_mode ///////////////////////////////////
  enum distance_display_mode
  {
    decimal     = 0, 
    fractional  = 1,
    feet_inches = 2
  };

ON::distance_display_mode ON::DistanceDisplayMode(int i)
{
  distance_display_mode dm = decimal;
  switch (i) 
  {
  case decimal:
    dm = decimal;
    break;
  case fractional:
    dm = fractional;
    break;
  case feet_inches:
    dm = feet_inches;
    break;
  }
  return dm;
}


ON::point_style ON::PointStyle(int i)
{
  //convertintegertopoint_styleenum
  point_style ps = unknown_point_style;
  switch (i) {
  case not_rational: ps = not_rational; break;
  case homogeneous_rational: ps = homogeneous_rational; break;
  case euclidean_rational: ps = euclidean_rational; break;
  default: ps = unknown_point_style; break;
  }
  return ps;
}


ON::knot_style ON::KnotStyle(int i)
{
  //convertintegertoknot_styleenum
  knot_style ks = unknown_knot_style;
  switch (i) {
  case uniform_knots: ks = uniform_knots; break;
  case quasi_uniform_knots: ks = quasi_uniform_knots; break;
  case piecewise_bezier_knots: ks = piecewise_bezier_knots; break;
  case clamped_end_knots: ks = clamped_end_knots; break;
  case non_uniform_knots: ks = non_uniform_knots; break;
  default: ks = unknown_knot_style; break;
  }
  return ks;
}

ON::curve_style ON::CurveStyle(int i)
{
  //convertintegertocurve_styleenum
  curve_style cs = unknown_curve_style;
  switch (i) {
  case line: cs = line; break;
  case circle: cs = circle; break;
  case ellipse: cs = ellipse; break;
  case parabola: cs = parabola; break;
  case hyperbola: cs = hyperbola; break;
  case planar_polyline: cs = planar_polyline; break;
  case polyline: cs = polyline; break;
  case planar_freeform_curve: cs = planar_freeform_curve; break;
  case freeform_curve: cs = freeform_curve; break;
  default: cs = unknown_curve_style; break;
  }
  return cs;
}

ON::surface_style ON::SurfaceStyle(int i)
{
  //convertintegertosurface_styleenum
  surface_style ss = unknown_surface_style;
  
  switch (i) {
  case plane: ss = plane; break;
  case circular_cylinder: ss = circular_cylinder; break;
  case elliptical_cylinder: ss = elliptical_cylinder; break;
  case circular_cone: ss = circular_cone; break;
  case elliptical_cone: ss = elliptical_cone; break;
  case sphere: ss = sphere; break;
  case torus: ss = torus; break;
  case surface_of_revolution: ss = surface_of_revolution; break;
  case ruled_surface: ss = ruled_surface; break;
  case freeform_surface: ss = freeform_surface; break;
  default: ss = unknown_surface_style; break;
  }
  return ss;
}

ON::sort_algorithm ON::SortAlgorithm(int i)
{
  sort_algorithm sa = heap_sort;
  
  switch (i) {
  case heap_sort: sa = heap_sort; break;
  case quick_sort: sa = quick_sort; break;
  default: sa = heap_sort; break;
  }
  return sa;
}

ON::endian ON::Endian(int i)
{ // convert integer to endian enum
  endian e = (i<=0) ? little_endian : big_endian;
  return e;
}

ON::endian ON::Endian()
{
  // returns endian-ness of cpu.
  union {
    int i;
    unsigned char b[sizeof(int)];
  } u;
  u.i = 1;
  return (u.b[0] == 1) ? little_endian : big_endian;
}

ON::archive_mode ON::ArchiveMode(int i)
{
  // convert integer to endian enum
  archive_mode a = read;
  switch(i) {
  case read:      a = read; break;
  case write:     a = write; break;
  case readwrite: a = readwrite; break;
  case read3dm:   a = read3dm; break;
  case write3dm:  a = write3dm; break;
  }
  return a;
}

ON::view_projection ON::ViewProjection(int i)
{
  // convert integer to view_projection enum
  view_projection v = unknown_view;
  switch(i) {
  case parallel_view: v = parallel_view; break;
  case perspective_view: v = perspective_view; break;
  }
  return v;

}

ON::coordinate_system ON::CoordinateSystem(int i)
{
  // convert integer to coordinate_system enum
  coordinate_system cs = world_cs;
  switch(i) {
  case world_cs:  cs = world_cs; break;
  case camera_cs: cs = camera_cs; break;
  case clip_cs:   cs = clip_cs; break;
  case screen_cs: cs = screen_cs; break;
  }
  return cs;
}

ON::exception_type ON::ExceptionType(int i)
{
  // convert integer to exception_type enum
  ON::exception_type e = unknown_exception;
  switch(i) {
  case out_of_memory:               e = out_of_memory; break;
  case unable_to_write_archive:     e = unable_to_write_archive; break;
  case unable_to_read_archive:      e = unable_to_read_archive; break;
  case unable_to_seek_archive:      e = unable_to_seek_archive; break;
  case unexpected_end_of_archive:   e = unexpected_end_of_archive; break;
  case unexpected_value_in_archive: e = unexpected_value_in_archive; break;
  };
  return e;
}

ON::layer_mode ON::LayerMode(int i)
{
  ON::layer_mode m = normal_layer;
	switch(i)
  {
    case normal_layer:    m = normal_layer;    break;
    case hidden_layer:    m = hidden_layer;    break;
    case locked_layer:    m = locked_layer;    break;
    case reference_layer: m = reference_layer; break;
  }
  return m;
}

ON::object_mode ON::ObjectMode(int i)
{
  ON::object_mode m = normal_object;
	switch(i)
  {
    case normal_object:  m = normal_object;  break;
    case hidden_object:  m = hidden_object;  break;
    case locked_object:  m = locked_object;  break;
  }
  return m;
}

ON::object_color_source ON::ObjectColorSource(int i)
{
  // convert integer to object_mode enum
  ON::object_color_source cs = color_from_layer;
  switch (i) {
  case color_from_layer: // use color assigned to layer
    cs = color_from_layer;
    break;
  case color_from_object: // use color assigned to object
    cs = color_from_object;
    break;
  case color_from_material:  // use diffuse render material color
    cs = color_from_material;
    break;
  }
  return cs;
}

ON::object_line_style_source ON::ObjectLineStyleSource(int i)
{
  // convert integer to object_mode enum
  ON::object_line_style_source ls = line_style_from_layer;
  switch (i) {
  case line_style_from_layer: // use line_style assigned to layer
    ls = line_style_from_layer;
    break;
  case line_style_from_object: // use line_style assigned to object
    ls = line_style_from_object;
    break;
  case line_style_from_material:  // use diffuse render material line_style
    ls = line_style_from_material;
    break;
  }
  return ls;
}

ON::object_material_source ON::ObjectMaterialSource(int i)
{
  ON::object_material_source ms = material_from_layer;
  switch(i) {
  case material_from_layer:
    ms = material_from_layer;
    break;
  case material_from_object:
    ms = material_from_object;
    break;
  }
  return ms;
}

ON::light_style ON::LightStyle(int i)
{
  // convert integer to light_style enum
  light_style ls = unknown_light_style;
  switch(i)
  {
    case unknown_light_style: ls = unknown_light_style; break;
    case view_directional_light: ls = view_directional_light; break;
    case view_point_light: ls = view_point_light; break;
    case view_spot_light: ls = view_spot_light; break;
    case camera_directional_light: ls = camera_directional_light; break;
    case camera_point_light: ls = camera_point_light; break;
    case camera_spot_light: ls = camera_spot_light; break;
    case world_directional_light: ls = world_directional_light; break;
    case world_point_light: ls = world_point_light; break;
    case world_spot_light: ls = world_spot_light; break;
    case ambient_light: ls = ambient_light; break;
    case world_linear_light: ls = world_linear_light; break;
    case world_rectangular_light: ls = world_rectangular_light; break;
  }
  return ls;
}

ON::curvature_style ON::CurvatureStyle(int i)
{
  // convert integer to light_style enum
  ON::curvature_style cs = unknown_curvature_style;
  switch(i) {
  case gaussian_curvature:
    cs = gaussian_curvature;
    break;
  case mean_curvature:
    cs = mean_curvature;
    break;
  case min_curvature: 
    // min unsigned principal curvature = maximum unsigned radius of curvature
    cs = min_curvature;
    break;
  case max_curvature: 
    // max unsigned principal curvature = minimum unsigned radius of curvature
    cs = max_curvature;
    break;
  //case section_curvature_x:
    // unsigned normal curvature with respect to sections cut perp to world x axis
    //cs = section_curvature_x;
    //break;
  //case section_curvature_y:
    // unsigned normal curvature with respect to sections cut perp to world y axis
    //cs = section_curvature_y;
    //break;
  //case section_curvature_z:
    // unsigned normal curvature with respect to sections cut perp to world z axis
    //cs = section_curvature_z;
    //break;
  }
  return cs;
}

ON::display_mode ON::DisplayMode(int i)
{
  // convert integer to light_style enum
  ON::display_mode dm = default_display;
  switch(i) {
  case default_display:
    dm = default_display;
    break;
  case wireframe_display:
    dm = wireframe_display;
    break;
  case shaded_display:
    dm = shaded_display;
    break;
  case renderpreview_display: 
    dm = renderpreview_display;
    break;
  }
  return dm;
}

ON::texture_mode ON::TextureMode(int i)
{
  // convert integer to texture_mode enum
  ON::texture_mode tm;
  switch (i) {
  case no_texture:
    tm = no_texture;
    break;
  case modulate_texture:
    tm = modulate_texture;
    break;
  case decal_texture:
    tm = decal_texture;
    break;
  default:
    tm = no_texture;
    break;
  }
  return tm;
}

ON::object_type ON::ObjectType(int i)
{
  // convert integer to object_type enum
  object_type ot = unknown_object_type;
  switch(i) {
  case unknown_object_type: ot = unknown_object_type; break;
  case point_object: ot = point_object; break;
  case pointset_object: ot = pointset_object; break;
  case curve_object: ot = curve_object; break;
  case surface_object: ot = surface_object; break;
  case brep_object: ot = brep_object; break;
  case mesh_object: ot = mesh_object; break;
  case layer_object: ot = layer_object; break;
  case material_object: ot = material_object; break;
  case light_object: ot = light_object; break;
  case userdata_object: ot = userdata_object; break;
  default: ot = unknown_object_type; break;
  }
  return ot;
}

ON::bitmap_type ON::BitmapType(int i)
{
  // convert integer to object_type enum
  bitmap_type bt = unknown_bitmap_type;
  switch(i) {
  case unknown_bitmap_type: bt = unknown_bitmap_type; break;
  case windows_bitmap:      bt = windows_bitmap; break;
  case opengl_bitmap:       bt = opengl_bitmap; break;
  case png_bitmap:          bt = png_bitmap; break;
  default: bt = unknown_bitmap_type; break;
  }
  return bt;
}

ON::line_pattern ON::LinePattern(int i)
{
  line_pattern lp = solid_line;
  switch(i)
  {
  case solid_line:          lp = solid_line; break;
  case custom_line_pattern: lp = custom_line_pattern; break;
  case hidden_line:         lp = hidden_line; break;
  case center_line_classic: lp = center_line_classic; break;
  case center_line_modern:  lp = center_line_modern; break;
  case phantom_line:        lp = phantom_line; break;
  default:                  lp = solid_line; break;
  }
  return lp;
}

ON::mesh_type ON::MeshType(int i)
{
  mesh_type mt = default_mesh;
  switch(i)
  {
  case default_mesh:  mt = default_mesh;  break;
  case render_mesh:   mt = render_mesh;   break;
  case analysis_mesh: mt = analysis_mesh; break;
  case preview_mesh:  mt = preview_mesh; break;
  case any_mesh:      mt = any_mesh;      break;
  default:            mt = default_mesh;  break;
  }
  return mt;
}


ON::eAnnotationType ON::AnnotationType(int i)
{
  // convert integer to eAnnotationType enum
  eAnnotationType at = dtNothing;
  switch(i) {
  case dtNothing:
    at = dtNothing;
    break;
  case dtDimLinear:
    at = dtDimLinear;
    break;
  case dtDimAligned:
    at = dtDimAligned;
    break;
  case dtDimAngular:
    at = dtDimAngular;
    break;
  case dtDimDiameter:
    at = dtDimDiameter;
    break;
  case dtDimRadius:
    at = dtDimRadius;
    break;
  case dtLeader:
    at = dtLeader;
    break;
  case dtTextBlock:
    at = dtTextBlock;
    break;
  }
  return at;
}
