/* $Header: /src2/opennurbs/opennurbs_geometry.h 12    6/28/01 8:17a Dalelear $ */
/* $NoKeywords: $ */
/*
//
// Copyright (c) 1993-2001 Robert McNeel & Associates. All rights reserved.
// Rhinoceros is a registered trademark of Robert McNeel & Assoicates.
//
// THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
// ALL IMPLIED WARRANTIES OF FITNESS FOR ANY PARTICULAR PURPOSE AND OF
// MERCHANTABILITY ARE HEREBY DISCLAIMED.
//				
// For complete openNURBS copyright information see <http://www.opennurbs.org>.
//
////////////////////////////////////////////////////////////////
*/

////////////////////////////////////////////////////////////////
//
//   virtual base class for all geomtric objects
//
////////////////////////////////////////////////////////////////

#if !defined(OPENNURBS_GEOMETRY_INC_)
#define OPENNURBS_GEOMETRY_INC_

class ON_Brep;

////////////////////////////////////////////////////////////////
//
// ON_Geometry: base class for all geometry classes that must
// provide runtime class id.
//
class ON_CLASS ON_Geometry : public ON_Object
{
  // Any object derived from ON_Geometry should have a
  //   ON_OBJECT_DECLARE(ON_...);
  // as the last line of its class definition and a
  //   ON_OBJECT_IMPLEMENT( ON_..., ON_baseclass );
  // in a .cpp file.
  //
  // See the definition of ON_Object for details.
  ON_OBJECT_DECLARE(ON_Geometry);

public:
  ON_Geometry();
  ON_Geometry(const ON_Geometry&);
  ON_Geometry& operator=(const ON_Geometry&);
  virtual ~ON_Geometry();

  ON_BoundingBox BoundingBox() const;

  BOOL GetBoundingBox( // returns TRUE if successful
         ON_BoundingBox&,
         BOOL = FALSE  // TRUE means grow box
         ) const;

  BOOL GetBoundingBox( // returns TRUE if successful
         ON_3dPoint&, // minimum
         ON_3dPoint&, // maximum
         BOOL = FALSE  // TRUE means grow box
         ) const;

  BOOL Rotate(
        double,               // sin(angle)
        double,               // cos(angle)
        const ON_3dVector&, // axis of rotation
        const ON_3dPoint&   // center of rotation
        );

  BOOL Rotate(
        double,              // angle in radians
        const ON_3dVector&, // axis of rotation
        const ON_3dPoint&   // center of rotation
        );

  BOOL Translate( const ON_3dVector& );

  virtual 
  int Dimension() const = 0;

  // work horse bounding box getter
  virtual
  BOOL GetBBox( // returns TRUE if successful
         double*,    // boxmin[dim]
         double*,    // boxmax[dim]
         BOOL = FALSE  // TRUE means grow box
         ) const = 0;

  // Some objects cache bounding box information.
  // If you modify an object, then call ClearBoundingBox()
  // to inform the object that any cached bounding boxes
  // are invalid.  Generally, ClearBoundingBox() overrides
  // simply invalidate a cached bounding box and then wait
  // for a call to GetBBox() before recomputing the bounding box.
  virtual
  void ClearBoundingBox();

  // transform geometry - be sure to call
  // ON_Object::TransformUserData() in your override.
  virtual
  BOOL Transform( 
         const ON_Xform&
         ) = 0;

  virtual // override if using Transform() is not appropriate or fast enough
  BOOL SwapCoordinates(
        int, int        // indices of coords to swap
        );

  //////////
  // HasBrepForm() returns TRUE if the BrepForm() can create
  // an ON_Brep representation of the ON_Geometry.
  virtual
  BOOL HasBrepForm() const;

  //////////
  // If possible, BrepForm() creates a brep representation of the
  // ON_Geometry.  The caller is responsible for deleting this
  // brep.
  virtual
  ON_Brep* BrepForm() const;
};

#endif

