/* $Header: /src2/opennurbs/opennurbs_group.cpp 4     1/03/01 4:59p Dalelear $ */
/* $NoKeywords: $ */
/*
//
// Copyright (c) 1993-2001 Robert McNeel & Associates. All rights reserved.
// Rhinoceros is a registered trademark of Robert McNeel & Assoicates.
//
// THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
// ALL IMPLIED WARRANTIES OF FITNESS FOR ANY PARTICULAR PURPOSE AND OF
// MERCHANTABILITY ARE HEREBY DISCLAIMED.
//				
// For complete openNURBS copyright information see <http://www.opennurbs.org>.
//
////////////////////////////////////////////////////////////////
*/

#include "opennurbs.h"

ON_OBJECT_IMPLEMENT( ON_Group, ON_Object, "721D9F97-3645-44c4-8BE6-B2CF697D25CE" );

ON_Group::ON_Group() : m_group_index(-1)
{
}

ON_Group::~ON_Group()
{
}

ON_Group::ON_Group( const ON_Group& src ) : ON_Object(src),
          m_group_name(src.m_group_name),
          m_group_index(src.m_group_index)
{
}

ON_Group& ON_Group::operator=(const ON_Group& src)
{
  if (this != &src) {
    ON_Object::operator=(src);
    m_group_index = src.m_group_index;
    m_group_name = src.m_group_name;
  }
  return *this;
}

//////////////////////////////////////////////////////////////////////
//
// ON_Object overrides

BOOL ON_Group::IsValid() const
{
  return ( m_group_name.Length() > 0 && m_group_index >= 0 );
}

void ON_Group::Dump( ON_TextLog& dump ) const
{
  const wchar_t* name = GroupName();
  if ( !name )
    name = L"";
  dump.Print("group index = %d\n",m_group_index);
  dump.Print("group name = \"%S\"\n",name);
}

BOOL ON_Group::Write(
       ON_BinaryArchive& file // serialize definition to binary archive
     ) const
{
  BOOL rc = file.Write3dmChunkVersion(1,0);
  if (rc) rc = file.WriteInt(m_group_index);
  if (rc) rc = file.WriteString(m_group_name);
  return rc;
}

BOOL ON_Group::Read(
       ON_BinaryArchive& file // restore definition from binary archive
     )
{
  m_group_index = -1;
  m_group_name.Empty();
  int major_version = 0;
  int minor_version = 0;
  BOOL rc = file.Read3dmChunkVersion(&major_version,&minor_version);
  if ( major_version == 1 ) {
    if (rc) rc = file.ReadInt( &m_group_index );
    if (rc) rc = file.ReadString( m_group_name );
  }
  else
    rc = FALSE;
  return rc;
}

//////////////////////////////////////////////////////////////////////
//
// Interface
void ON_Group::SetGroupName( const wchar_t* s )
{
  m_group_name = s;
}

void ON_Group::SetGroupName( const char* s )
{
  m_group_name = s;
}

void ON_Group::GetGroupName( ON_wString& s ) const
{
  s = m_group_name;
}

const wchar_t* ON_Group::GroupName() const
{
  const wchar_t* s = m_group_name;
  return s;
}

void ON_Group::SetGroupIndex(int i )
{
  m_group_index = i;
}

int ON_Group::GroupIndex() const
{
  return m_group_index;
}

