#if !defined(OPENNURBS_LAYER_INC_)
#define OPENNURBS_LAYER_INC_

class ON_CLASS ON_Layer : public ON_Object
{
public:
  ON_Layer();
  ON_Layer( const ON_Layer& );
  ~ON_Layer();
	ON_Layer& operator=(const ON_Layer&);
	BOOL operator==(const ON_Layer&) const;	// Comparison tests if names are equal
	BOOL operator!=(const ON_Layer&) const;	// Comparison tests if names are equal

  //////////////////////////////////////////////////////////////////////
  //
  // ON_Object overrides

  BOOL IsValid() const;

  void Dump( ON_TextLog& ) const; // for debugging

  BOOL Write(
         ON_BinaryArchive&  // serialize definition to binary archive
       ) const;

  BOOL Read(
         ON_BinaryArchive&  // restore definition from binary archive
       );

  ON::object_type ObjectType() const;

  //////////////////////////////////////////////////////////////////////
  //
  // Interface

  BOOL SetLayerName( const char* );
  BOOL SetLayerName( const wchar_t* );
	const ON_wString& LayerName() const;

	void SetColor( ON_Color ); // layer display color
	ON_Color Color() const;

	void SetLineStyle( ON_LineStyle ); // layer line style
	ON_LineStyle LineStyle() const;

  // 4 mutually exclusive layer modes: normal, hidden, locked, hidden, reference
  BOOL SetMode( ON::layer_mode );
  ON::layer_mode Mode() const;

  // status determined by mode
  BOOL IsVisible() const;    // TRUE if objects on layer are visible (normal,locked,and reference)
  BOOL IsChangeable() const; // TRUE if objects on layer can be changed (normal)
  BOOL IsSelectable() const; // TRUE if objects on layer are selectable (normal and reference)

  //////////
  // Index of render material for objects on this layer that have
  // MaterialSource() == ON::material_from_layer.
  // A material index of -1 indicates no material has been assigned
  // and the material created by the default ON_Material constructor
  // should be used.
  BOOL SetRenderMaterialIndex( int ); // index of layer's rendering material
  int RenderMaterialIndex() const;

  BOOL SetLayerIndex( int ); // index of this layer;
  int LayerIndex() const;

  BOOL SetIgesLevel( int ); // IGES level for this layer
  int IgesLevel() const;

  BOOL SetModelIndex( int ); // index of model during a multi-model worksession
  int ModelIndex() const;

private:

  int m_layer_index;    // index of this layer
  int m_iges_level;     // IGES level number if this layer was made during IGES import
  int m_material_index; // index of render material
  int m_model_index;    // index of model during a multi-model worksessions
  ON_Color m_color;     // layer color
  ON_LineStyle m_line_style; // layer line style
  ON_wString m_name;
  ON::layer_mode m_mode;

  ON_OBJECT_DECLARE(ON_Layer);
};

#endif

