/* $Header: /src2/opennurbs/opennurbs_linestyle.cpp 4     1/03/01 4:59p Dalelear $ */
/* $NoKeywords: $ */
/*
//
// Copyright (c) 1993-2001 Robert McNeel & Associates. All rights reserved.
// Rhinoceros is a registered trademark of Robert McNeel & Assoicates.
//
// THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
// ALL IMPLIED WARRANTIES OF FITNESS FOR ANY PARTICULAR PURPOSE AND OF
// MERCHANTABILITY ARE HEREBY DISCLAIMED.
//				
// For complete openNURBS copyright information see <http://www.opennurbs.org>.
//
////////////////////////////////////////////////////////////////
*/

#include "opennurbs.h"

ON_LineStyle::ON_LineStyle() : m_ls(0)
{}

ON_LineStyle::~ON_LineStyle()
{}

ON_LineStyle::ON_LineStyle( const ON_LineStyle& src ) : m_ls(0)
{
  *this = src;
}

ON_LineStyle& ON_LineStyle::operator=( const ON_LineStyle& src )
{
  if ( this != &src ) {
    m_ls = src.m_ls;
  }
  return *this;
}

void ON_LineStyle::Default()
{
  m_ls = 0;
}

int ON_LineStyle::Compare( const ON_LineStyle& src ) const
{
  const unsigned int pattern0 = Pattern();
  const unsigned int pattern1 = src.Pattern();
  if ( pattern0 < pattern1 )
    return -1;
  if ( pattern0 > pattern1 )
    return 1;

  const int pattern_index0 = PatternIndex();
  const int pattern_index1 = src.PatternIndex();
  if ( pattern_index0 < pattern_index1 )
    return -1;
  if ( pattern_index0 > pattern_index1 )
    return 1;

  double thick0 = Thickness();
  double thick1 = src.Thickness();
  if ( thick0 < thick1 )
    return -1;
  if ( thick0 > thick1 )
    return 1;

  double scale0 = Scale();
  double scale1 = src.Scale();
  if ( scale0 < scale1 )
    return -1;
  if ( scale0 > scale1 )
    return 1;

  return 0;
}

ON::line_pattern ON_LineStyle::Pattern() const
{
  int pattern = m_ls & 0xFFF;
  return ( pattern & 0x800 ) ? ON::custom_line_pattern : ON::LinePattern( pattern );
}

void ON_LineStyle::SetPattern( ON::line_pattern pattern )
{
  if ( pattern == ON::custom_line_pattern )
    SetPatternIndex(0);
  else {
    int p = pattern;
    p &= 0x7FFF;
    m_ls &= 0xFFFFF000;
    m_ls |= p;
  }
}

int ON_LineStyle::PatternIndex() const
{
  int pattern = m_ls & 0xFFF;
  return ( pattern & 0x800 ) ? (pattern&0x7FF) : 0;
}

void ON_LineStyle::SetPatternIndex( int pattern_index )
{
  if ( pattern_index < 0 || pattern_index > 0x7FF )
    pattern_index = 0;
  m_ls &= 0xFFFFF000;
  m_ls |= 0x800;
  m_ls |= pattern_index;
}

double ON_LineStyle::Thickness() const
{
  int thickness = (m_ls >> 12) & 0x3FF;
  return ((double)thickness)/100.0;
}

void ON_LineStyle::SetThickness( double thickness )
{
  thickness *= 100.0;
  int th = 0;
  if ( thickness > 0x3FF )
    th = 0x3FF;
  else if (thickness > 0.0 )
    th = (int)thickness;
  m_ls &= 0xFFD00FFF;
  m_ls |= (th<<12);
}

double ON_LineStyle::Scale() const
{
  int scale = (m_ls >> 24);
  if ( scale == 0 )
    return 1.0;
  return (((double)scale) - 512.0)/32.0;
}

void ON_LineStyle::SetScale( double scale )
{
  int s = 0;
  if ( scale > 0.0 ) {
    scale = scale*32.0 + 512.0;
    if ( scale < 0.0 )
      scale = 0.0;
    else if ( scale > 0x3FF )
      scale = 0x3FF;
    s = (int)scale;
  }
  m_ls &= 0x3FFFFF;
  m_ls |= (s << 24);
}

