/* $Header: /src2/opennurbs/opennurbs_material.cpp 22    6/19/01 11:45a Dalelear $ */
/* $NoKeywords: $ */
/*
//
// Copyright (c) 1993-2001 Robert McNeel & Associates. All rights reserved.
// Rhinoceros is a registered trademark of Robert McNeel & Assoicates.
//
// THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
// ALL IMPLIED WARRANTIES OF FITNESS FOR ANY PARTICULAR PURPOSE AND OF
// MERCHANTABILITY ARE HEREBY DISCLAIMED.
//				
// For complete openNURBS copyright information see <http://www.opennurbs.org>.
//
////////////////////////////////////////////////////////////////
*/

#include "opennurbs.h"


////////////////////////////////////////////////////////////////
//   Class ON_Material
////////////////////////////////////////////////////////////////

ON_OBJECT_IMPLEMENT(ON_Material,ON_Object,"60B5DBBC-E660-11d3-BFE4-0010830122F0");

double ON_Material::m_max_shine = 255.0f;

double ON_Material::MaxShine()
{
  return m_max_shine;
}

void ON_Material::Default()
{
  m_ambient = ON_Color( 128, 128, 128 ),
  m_diffuse = ON_Color( 128, 128, 128 ),
  m_emission = ON_Color( 0, 0, 0 ),
  m_specular = ON_Color( 255, 255, 255 ),
  m_shine = 0.0;
  m_transparency = 0.0;

  m_casts_shadows = 1;
  m_shows_shadows = 1;

  m_wire_mode = 0;
  m_wire_density = 2;
  m_wire_color = ON_Color(0,0,0);
  m_wire_style.Default();

  m_material_index = 0;

  m_sTextureBitmapFileName.Destroy();
  m_texture_mode = ON::no_texture;
  m_texture_bitmap_index = 0;
  
  m_sBumpBitmapFileName.Destroy();
  m_bump_mode = ON::no_texture;
  m_bump_bitmap_index = 0;
  m_bump_scale = 1.0;

  m_sEmapBitmapFileName.Destroy();
  m_emap_mode = ON::no_texture;
  m_emap_bitmap_index = 0;

  m_uuid = ON_nil_uuid;
  m_sMaterialLibrary.Destroy();
  m_sMaterialName.Destroy();
}

// Default constructor
ON_Material::ON_Material() 
{
  Default();
}

// Copy Constructor
ON_Material::ON_Material(const ON_Material& src)
{
  Default();
  *this = src;
}

// operator =
ON_Material& ON_Material::operator=(const ON_Material& src)
{
  if ( this != &src ) {
    ON_Object::operator=(src);
	  SetAmbient ( src.Ambient() );
	  SetDiffuse ( src.Diffuse() );
	  SetEmission ( src.Emission() );
	  SetSpecular ( src.Specular() );
	  SetShine ( src.Shine() );
	  SetTransparency ( src.Transparency() );

    m_wire_mode    = src.m_wire_mode;
    m_wire_density = src.m_wire_density;
    m_wire_color   = src.m_wire_color;
    m_wire_style   = src.m_wire_style;

    SetMaterialIndex( src.MaterialIndex() );

	  SetTextureBitmapFileName(  src.TextureBitmapFileName()  );
    SetTextureMode( src.TextureMode() );
    m_texture_bitmap_index = src.m_texture_bitmap_index;

	  SetBumpBitmapFileName(  src.BumpBitmapFileName()  );
    SetBumpMode( src.BumpMode() );
    m_bump_bitmap_index = src.m_bump_bitmap_index;
    m_bump_scale = src.m_bump_scale;

	  SetEmapBitmapFileName(  src.EmapBitmapFileName()  );
    SetEmapMode( src.EmapMode() );
    m_emap_bitmap_index = src.m_emap_bitmap_index;

    SetMaterialPlugInUuid( src.MaterialPlugInUuid() );
	  SetMaterialLibrary(  src.MaterialLibrary()  );
	  SetMaterialName(  src.MaterialName()  );
  }
	return *this;
}


ON_Material::~ON_Material()
{}

BOOL
ON_Material::IsValid() const
{
  return TRUE;
}


void
ON_Material::Dump( ON_TextLog& dump ) const
{
  const wchar_t* s;
  dump.Print("index = %d\n",MaterialIndex());
  
  s = MaterialName();
  if ( s && *s )
    dump.Print("name = \"%S\"\n",s);
  
  s = MaterialLibrary();
  if ( s && *s )
    dump.Print("library = %S\n",s);

  dump.Print("ambient rgb = "); dump.PrintRGB( Ambient() ); dump.Print("\n");
  dump.Print("diffuse rgb = "); dump.PrintRGB( Diffuse() ); dump.Print("\n");
  dump.Print("emmisive rgb = "); dump.PrintRGB( Emission() ); dump.Print("\n");
  dump.Print("specular rgb = "); dump.PrintRGB( Specular() ); dump.Print("\n");
  dump.Print("shine = %g%%\n",100.0*Shine()/ON_Material::MaxShine() );
  dump.Print("transparency = %g%%\n",100.0*Transparency());

  s = TextureBitmapFileName();
  if ( s && *s )
    dump.Print("texture bitmap = %S\n",s);
  s = BumpBitmapFileName();
  if ( s && *s )
    dump.Print("bump bitmap = %S\n",s);
  s = EmapBitmapFileName();
  if ( s && *s )
    dump.Print("emap bitmap = %S\n",s);
}

BOOL ON_Material::Write( ON_BinaryArchive& file ) const
{
  int i;
  BOOL rc = file.Write3dmChunkVersion(1,0);
  if ( rc ) rc = file.WriteColor( m_ambient );
  if ( rc ) rc = file.WriteColor( m_diffuse );
  if ( rc ) rc = file.WriteColor( m_emission );
  if ( rc ) rc = file.WriteColor( m_specular );
  if ( rc ) rc = file.WriteDouble( Shine() );
  if ( rc ) rc = file.WriteDouble( Transparency() );

  if ( rc ) rc = file.WriteChar( m_casts_shadows );
  if ( rc ) rc = file.WriteChar( m_shows_shadows );

  if ( rc ) rc = file.WriteChar( m_wire_mode );
  if ( rc ) rc = file.WriteChar( m_wire_density );

  if ( rc ) rc = file.WriteColor( m_wire_color );
  if ( rc ) rc = file.WriteLineStyle( m_wire_style );

  if ( rc ) rc = file.WriteString( TextureBitmapFileName() );
  i = TextureMode();
  if ( rc ) rc = file.WriteInt( i );
  if ( rc ) rc = file.WriteInt( m_texture_bitmap_index );

  if ( rc ) rc = file.WriteString( BumpBitmapFileName() );
  i = BumpMode();
  if ( rc ) rc = file.WriteInt( i );
  if ( rc ) rc = file.WriteInt( m_bump_bitmap_index );
  if ( rc ) rc = file.WriteDouble( m_bump_scale );

  if ( rc ) rc = file.WriteString( EmapBitmapFileName() );
  i = EmapMode();
  if ( rc ) rc = file.WriteInt( i );
  if ( rc ) rc = file.WriteInt( m_emap_bitmap_index );

  if ( rc ) rc = file.WriteInt( m_material_index );

  if ( rc ) rc = file.WriteUuid( m_uuid );
  if ( rc ) rc = file.WriteString( MaterialLibrary() );
  if ( rc ) rc = file.WriteString( MaterialName() );
  return rc;
}

BOOL ON_Material::Read( ON_BinaryArchive& file )
{
  Default();
  int major_version = 0;
  int minor_version = 0;
  double shine = 0.0, transparency = 0.0;
  int i;
  BOOL rc = file.Read3dmChunkVersion(&major_version,&minor_version);
  if ( rc && major_version == 1 ) {
    // common to all version 1.x formats
    if ( rc ) rc = file.ReadColor( m_ambient );
    if ( rc ) rc = file.ReadColor( m_diffuse );
    if ( rc ) rc = file.ReadColor( m_emission );
    if ( rc ) rc = file.ReadColor( m_specular );
    if ( rc ) rc = file.ReadDouble( &shine );
    if ( rc ) SetShine(shine);
    if ( rc ) rc = file.ReadDouble( &transparency );
    if ( rc ) SetTransparency(transparency);

    if ( rc ) rc = file.ReadChar( &m_casts_shadows );
    if ( rc ) rc = file.ReadChar( &m_shows_shadows );

    if ( rc ) rc = file.ReadChar( &m_wire_mode );
    if ( rc ) rc = file.ReadChar( &m_wire_density );
    if ( rc ) rc = file.ReadColor( m_wire_color );
    if ( rc ) rc = file.ReadLineStyle( m_wire_style );

    if ( rc ) rc = file.ReadString( m_sTextureBitmapFileName );
    if ( rc ) rc = file.ReadInt( &i );
    if ( rc ) SetTextureMode( ON::TextureMode(i) );
    if ( rc ) rc = file.ReadInt( &m_texture_bitmap_index );

    if ( rc ) rc = file.ReadString( m_sBumpBitmapFileName );
    if ( rc ) rc = file.ReadInt( &i );
    if ( rc ) SetBumpMode( ON::TextureMode(i) );
    if ( rc ) rc = file.ReadInt( &m_bump_bitmap_index );
    if ( rc ) rc = file.ReadDouble( &m_bump_scale );

    if ( rc ) rc = file.ReadString( m_sEmapBitmapFileName );
    if ( rc ) rc = file.ReadInt( &i );
    if ( rc ) SetEmapMode( ON::TextureMode(i) );
    if ( rc ) rc = file.ReadInt( &m_emap_bitmap_index );

    if ( rc ) rc = file.ReadInt( &m_material_index );

    if ( rc ) rc = file.ReadUuid( m_uuid );
    if ( rc ) rc = file.ReadString( m_sMaterialLibrary );
    if ( rc ) rc = file.ReadString( m_sMaterialName );
  }
  return rc;
}

ON::object_type ON_Material::ObjectType() const
{
   return ON::material_object;
}


// Shine values are in range 0.0 to ON_Material::GetMaxShine()
double ON_Material::Shine() const
{
  return m_shine;
}

void ON_Material::SetShine( double shine )
{
  if ( shine < 0.0 )
    m_shine = 0.0;
  else if ( shine > m_max_shine)
    m_shine = m_max_shine;
  else
    m_shine = (float)shine;
}

  // Transparency values are in range 0.0 = opaque to 1.0 = transparent
double ON_Material::Transparency( ) const
{
  return  m_transparency;
}

void ON_Material::SetTransparency( double transparency )
{
  if ( transparency < 0.0 )
    m_transparency = 0.0f;
  else if ( transparency > 1.0)
    m_transparency = 1.0f;
  else
    m_transparency = (float)transparency;
}

BOOL ON_Material::operator==( const ON_Material& src ) const
{
  return Compare(src) ? FALSE : TRUE;
}

BOOL ON_Material::operator!=( const ON_Material& src ) const
{
  return !ON_Material::operator==(src);
}

int ON_Material::Compare( const ON_Material& src ) const
{
  int rc = 0;
  if ( !rc )
    rc = Emission().Compare( src.Emission() );
  if ( !rc )
    rc = Diffuse().Compare( src.Diffuse() );
  if ( !rc )
    rc = Specular().Compare( src.Specular() );
  if ( !rc )
    rc = Ambient().Compare( src.Ambient() );
  if ( !rc ) {
    if ( Shine() < src.Shine() ) 
      rc = -1;
    else if ( Shine() > src.Shine() )
      rc = 1;
  }
  if ( !rc ) {
    if ( Transparency() < src.Transparency() ) 
      rc = -1;
    else if ( Transparency() > src.Transparency() )
      rc = 1;
  }

  if ( !rc ) {
    rc = m_sTextureBitmapFileName.CompareNoCase( src.TextureBitmapFileName() );
  }

  if ( !rc ) {
    rc = m_sBumpBitmapFileName.CompareNoCase( src.BumpBitmapFileName() );
  }

  if ( !rc ) {
    rc = m_sEmapBitmapFileName.CompareNoCase( src.EmapBitmapFileName() );
  }

  if ( !rc )
    rc = ON_UuidCompare( &m_uuid, &src.MaterialPlugInUuid() );
  if ( !rc ) {
    rc = m_sMaterialLibrary.CompareNoCase( src.MaterialLibrary() );
  }
  if ( !rc ) {
    rc = m_sMaterialName.CompareNoCase( src.MaterialName() );
  }

  return rc;  
}


ON_Color ON_Material::Ambient() const
{
  return m_ambient;
}

ON_Color ON_Material::Diffuse( ) const
{
  return m_diffuse;
}

ON_Color ON_Material::Emission( ) const
{
  return m_emission;
}

ON_Color ON_Material::Specular() const
{
  return m_specular;
}

void ON_Material::SetAmbient( ON_Color  c )
{
  m_ambient = c;
}

void ON_Material::SetDiffuse(  ON_Color c )
{
  m_diffuse = c ;
}

void ON_Material::SetEmission( ON_Color c )
{
  m_emission = c ;
}

void ON_Material::SetSpecular( ON_Color c )
{
  m_specular = c;
}

int ON_Material::MaterialIndex() const
{
  return m_material_index;
}

void ON_Material::SetMaterialIndex( int i )
{
  m_material_index = i;
}


ON::texture_mode ON_Material::TextureMode() const
{
  return m_texture_mode;
}

void ON_Material::SetTextureMode(ON::texture_mode tm)
{
  m_texture_mode = ON::TextureMode(tm);
}

const ON_wString& ON_Material::TextureBitmapFileName( ) const
{
	return m_sTextureBitmapFileName;
}

void ON_Material::SetTextureBitmapFileName( const char* sFileName )
{
  // Use SetTextureMode() to prescribe if the texture map is active and how it
  // should be rendered.
	m_sTextureBitmapFileName = sFileName;
}

void ON_Material::SetTextureBitmapFileName( const wchar_t* sFileName )
{
  // Use SetTextureMode() to prescribe if the texture map is active and how it
  // should be rendered.
	m_sTextureBitmapFileName = sFileName;
}


ON::texture_mode ON_Material::BumpMode() const
{
  return m_bump_mode;
}

void ON_Material::SetBumpMode(ON::texture_mode tm)
{
  m_bump_mode = ON::TextureMode(tm);
}

const ON_wString& ON_Material::BumpBitmapFileName( ) const
{
	return m_sBumpBitmapFileName;
}

void ON_Material::SetBumpBitmapFileName( const char* sFileName )
{
  // Use SetBumpMode() to prescribe if the bump map is active and how it
  // should be rendered.
	m_sBumpBitmapFileName = sFileName;
}

void ON_Material::SetBumpBitmapFileName( const wchar_t* sFileName )
{
  // Use SetBumpMode() to prescribe if the bump map is active and how it
  // should be rendered.
	m_sBumpBitmapFileName = sFileName;
}


double ON_Material::BumpScale() const
{
  return m_bump_scale;
}
void ON_Material::SetBumpScale(double s)
{
  m_bump_scale = s;
}


ON::texture_mode ON_Material::EmapMode() const
{
  return m_emap_mode;
}

void ON_Material::SetEmapMode(ON::texture_mode tm)
{
  m_emap_mode = ON::TextureMode(tm);
}

const ON_wString& ON_Material::EmapBitmapFileName( ) const
{
	return m_sEmapBitmapFileName;
}

void ON_Material::SetEmapBitmapFileName( const char* sFileName )
{
  // Use SetEmapMode() to prescribe if the environment map is active and how it
  // should be rendered.
	m_sEmapBitmapFileName = sFileName;
}

void ON_Material::SetEmapBitmapFileName( const wchar_t* sFileName )
{
  // Use SetEmapMode() to prescribe if the environment map is active and how it
  // should be rendered.
	m_sEmapBitmapFileName = sFileName;
}


const ON_wString& ON_Material::MaterialName( ) const
{
	return m_sMaterialName;
}

void ON_Material::SetMaterialName( const char* sMaterialName )
{
  m_sMaterialName = sMaterialName;
}

void ON_Material::SetMaterialName( const wchar_t* sMaterialName )
{
  m_sMaterialName = sMaterialName;
}

void ON_Material::SetWireMode( int m )
{
  // 0 = no wire, 1 = wire+shade, 2 = wire + no shade
  if ( m < 0 )
    m = 0;
  else if ( m > 2 )
    m = 2;
  m_wire_mode = m;
}

void ON_Material::SetWireDensity( int d )
{
  // 0 = boundry only, 1 = boundary + knots, ...
  if ( d < 0 )
    d = 0;
  m_wire_density = d;
}

void ON_Material::SetWireColor(ON_Color c)
{
  m_wire_color = c;
}

void ON_Material::SetWireLineStyle(ON_LineStyle ls)
{
  m_wire_style = ls;
}

int ON_Material::WireMode() const
{
  // 0 = no wire, 1 = wire+shade, 2 = wire + no shade
  return m_wire_mode;
}

int ON_Material::WireDensity() const
{
  // 0 = boundry only, 1 = boundary + knots, ...
  return m_wire_density;
}

ON_Color ON_Material::WireColor() const
{
  return m_wire_color;
}

ON_LineStyle ON_Material::WireLineStyle() const
{
  return m_wire_style;
}


const ON_wString& ON_Material::MaterialLibrary( ) const
{
	return m_sMaterialLibrary;
}

void ON_Material::SetMaterialLibrary( const char* sMaterialLibrary )
{
  m_sMaterialLibrary = sMaterialLibrary;
}

void ON_Material::SetMaterialLibrary( const wchar_t* sMaterialLibrary )
{
  m_sMaterialLibrary = sMaterialLibrary;
}


void ON_Material::SetMaterialPlugInUuid( const ON_UUID& uuid )
{
  memcpy( &m_uuid, &uuid, 16 );
}



const ON_UUID& ON_Material::MaterialPlugInUuid(void) const
{
  return m_uuid;
}
