/* $Header: /src2/opennurbs/opennurbs_material.h 13    6/19/01 11:45a Dalelear $ */
/* $NoKeywords: $ */
/*
//
// Copyright (c) 1993-2001 Robert McNeel & Associates. All rights reserved.
// Rhinoceros is a registered trademark of Robert McNeel & Assoicates.
//
// THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
// ALL IMPLIED WARRANTIES OF FITNESS FOR ANY PARTICULAR PURPOSE AND OF
// MERCHANTABILITY ARE HEREBY DISCLAIMED.
//				
// For complete openNURBS copyright information see <http://www.opennurbs.org>.
//
////////////////////////////////////////////////////////////////
*/

////////////////////////////////////////////////////////////////
//
//   defines ON_Color and ON_Material
//
////////////////////////////////////////////////////////////////

#if !defined(OPENNURBS_MATERIAL_INC_)
#define OPENNURBS_MATERIAL_INC_


///////////////////////////////////////////////////////////////////////////////
//
// Class ON_Material
// 
class ON_CLASS ON_Material : public ON_Object
{
public:
  static double MaxShine();			// maximum value of shine exponent

  ON_Material();							// Default grey color
  ON_Material(const ON_Material&);    // copy constructor
  ~ON_Material();						// destructor

  ON_Material& operator=(const ON_Material&);	// assignment operator

  BOOL operator==(const ON_Material&) const; // ignores m_material_index
  BOOL operator!=(const ON_Material&) const; // ignores m_material_index

  void Default();

  /////////////////////////////////////////////////////////////////
  // ON_Object overrides

  BOOL IsValid() const;

  void Dump( ON_TextLog& ) const; // for debugging

  BOOL Write(
         ON_BinaryArchive&  // open binary file
       ) const;

  BOOL Read(
         ON_BinaryArchive&  // open binary file
       );

  ON::object_type ObjectType() const;

  /////////////////////////////////////////////////////////////////
  // Interface

  int Compare( const ON_Material& ) const; // ignores m_material_index

  ON_Color Ambient() const;
  ON_Color Diffuse() const;
  ON_Color Emission() const;
  ON_Color Specular() const;

  void SetAmbient(  ON_Color );
  void SetDiffuse(  ON_Color );
  void SetEmission( ON_Color );
  void SetSpecular( ON_Color );

  // Shine values are in range 0.0 to ON_Material::MaxShine()
  double Shine() const;
  void SetShine( double );         // 0 to ON_Material::MaxShine()

  // Transparency values are in range 0.0 = opaque to 1.0 = transparent
  double Transparency() const;
  void SetTransparency( double );  // 0.0 = opaque, 1.0 = transparent

  int MaterialIndex() const;
  void SetMaterialIndex( int );

  //////////
  // Texture bitmap file name
  // Use SetTextureBitmapFileName() to set the name of the texture bitmap.
  // Use TextureMode() to determine if the texture is active and how it
  // should be rendered.
  const ON_wString& TextureBitmapFileName() const;

  //////////
  // Set texture bitmap file name
  // Use SetTextureMode() to prescribe if the texture is active and how it
  // should be rendered.
  void SetTextureBitmapFileName( const char* );

  //////////
  // Set texture bitmap file name
  // Use SetTextureMode() to prescribe if the texture is active and how it
  // should be rendered.
  void SetTextureBitmapFileName( const wchar_t* );

  //////////
  // TextureMode() determines if the texture map is active and how it
  // should be displayed.  Use TextureBitmapFileName() to get the name
  // of the texture bitmap.
  ON::texture_mode TextureMode() const;

  //////////
  // Use SetTextureMode() to prescribe how the texture map should be rendered.
  void SetTextureMode( ON::texture_mode );

  //////////
  // Bump bitmap file name
  // Use SetBumpBitmapFileName() to set the name of the bump bitmap.
  // Use BumpMode() to determine if the bump map is active and how it
  // should be rendered.  Use BumpScale() to get the bump map scale.
  const ON_wString& BumpBitmapFileName() const;

  //////////
  // Set bump map bitmap file name
  // Use SetBumpMode() to prescribe if the bump map is active and how it
  // should be rendered.  Use SetBumpScale() to set the bump map scale.
  void SetBumpBitmapFileName( const char* );

  //////////
  // Set bump map bitmap file name
  // Use SetBumpMode() to prescribe if the bump map is active and how it
  // should be rendered.  Use SetBumpScale() to set the bump map scale.
  void SetBumpBitmapFileName( const wchar_t* );

  //////////
  // BumpMode() determines if the bump map is active and how it
  // should be displayed.  Use BumpBitmapFileName() to get the name
  // of the bump map bitmap.
  ON::texture_mode BumpMode() const;

  //////////
  // Use SetBumpMode() to prescribe how the bump map should be rendered.
  void SetBumpMode( ON::texture_mode );

  //////////
  // Use BumpScale() to get the bump map scale
  double BumpScale() const;

  //////////
  // Use SetBumpScale() to set the bump map scale
  void SetBumpScale(double);

  //////////
  // Environment map bitmap file name
  // Use SetEmapBitmapFileName() to set the name of the environment map.
  // Use EmapMode() to determine if the environment map is active and how it
  // should be rendered.
  const ON_wString& EmapBitmapFileName() const;

  //////////
  // Set environment map bitmap file name
  // Use SetEmapMode() to prescribe if the environment is active and how it
  // should be rendered.
  void SetEmapBitmapFileName( const char* );

  //////////
  // Set environment map bitmap file name
  // Use SetEmapMode() to prescribe if the environment is active and how it
  // should be rendered.
  void SetEmapBitmapFileName( const wchar_t* );

  //////////
  // EmapMode() determines if the environment map is active and how it
  // should be displayed.  Use EmapBitmapFileName() to get the name
  // of the environment map bitmap.
  ON::texture_mode EmapMode() const;

  //////////
  // Use SetEmapMode() to prescribe how the environment map should be rendered.
  void SetEmapMode( ON::texture_mode );

  // Render Material Name
  const ON_wString& MaterialName() const;
  void SetMaterialName( const char* );    // render material name used in RIB, POV, OBJ, ...
  void SetMaterialName( const wchar_t* );    // render material name used in RIB, POV, OBJ, ...
  
  const ON_wString& MaterialLibrary() const;
  void SetMaterialLibrary( const char* );    // render material name used in RIB, POV, OBJ, ...
  void SetMaterialLibrary( const wchar_t* );    // render material name used in RIB, POV, OBJ, ...

  const ON_UUID& MaterialPlugInUuid() const;
  void SetMaterialPlugInUuid( const ON_UUID& );

  // wire frame attributes
  int WireMode() const;    // 0 = no wire, 1 = wire+shade, 2 = wire + no shade
  int WireDensity() const; // 0 = boundry only, 1 = boundary + knots, ...
  ON_Color WireColor() const;
  ON_LineStyle WireLineStyle() const;
  void SetWireMode( int );    // 0 = no wire, 1 = wire+shade, 2 = wire + no shade
  void SetWireDensity( int ); // 0 = boundry only, 1 = boundary + knots, ...
  void SetWireColor(ON_Color);
  void SetWireLineStyle(ON_LineStyle);

private:
  static double m_max_shine;

  ON_Color   m_ambient;
  ON_Color   m_diffuse;
  ON_Color   m_emission;
  ON_Color   m_specular;
  double     m_shine;        // 0.0 = none to GetMaxShine()=maximum
  double     m_transparency; // 0.0 = opaque to 1.0 = transparent

  // shadow attributes
  unsigned char m_casts_shadows; // 0 = no, 1 = yes
  unsigned char m_shows_shadows; // 0 = no, 1 = yes

  // wire attributes
  unsigned char m_wire_mode;    // 0 = no wire, 1 = wire+shade 2 = wire + no shade
  unsigned char m_wire_density; // 0 = boundry only, 1 = boundary + knots, ...
  ON_Color      m_wire_color;   // color for wire frame
  ON_LineStyle  m_wire_style;   // wire line pattern, thickness, scale

  // texture map
  ON_wString    m_sTextureBitmapFileName;
  ON::texture_mode m_texture_mode;
  int m_texture_bitmap_index;

  // bump map
  ON_wString    m_sBumpBitmapFileName;
  ON::texture_mode m_bump_mode;
  int m_bump_bitmap_index;
  double m_bump_scale;

  // environment map
  ON_wString    m_sEmapBitmapFileName;
  ON::texture_mode m_emap_mode;
  int m_emap_bitmap_index;

  int m_material_index;           // material index in matrial table

  ON_UUID       m_uuid;        // plug-in uuid for material name references
  ON_wString    m_sMaterialLibrary;
  ON_wString    m_sMaterialName;

  ON_OBJECT_DECLARE(ON_Material);
};

#endif
