/* $Header: /src2/opennurbs/opennurbs_math.h 15    5/11/01 3:36p Dalelear $ */
/* $NoKeywords: $ */
/*
//
// Copyright (c) 1993-2001 Robert McNeel & Associates. All rights reserved.
// Rhinoceros is a registered trademark of Robert McNeel & Assoicates.
//
// THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
// ALL IMPLIED WARRANTIES OF FITNESS FOR ANY PARTICULAR PURPOSE AND OF
// MERCHANTABILITY ARE HEREBY DISCLAIMED.
//				
// For complete openNURBS copyright information see <http://www.opennurbs.org>.
//
////////////////////////////////////////////////////////////////
*/

#if !defined(ON_MATH_INC_)
#define ON_MATH_INC_

class ON_3dVector;
class ON_Interval;
class ON_Line;
class ON_Arc;
class ON_Plane;

ON_DECL
float ON_ArrayDotProduct( // returns AoB
          int,           // size of arrays (can be zero)
          const float*, // A[]
          const float*  // B[]
          );

ON_DECL
void   ON_ArrayScale( 
          int,           // size of arrays (can be zero)
          float,        // a
          const float*, // A[]
          float*        // returns a*A[]
          );

ON_DECL
void   ON_Array_aA_plus_B( 
          int,           // size of arrays (can be zero)
          float,        // a
          const float*, // A[]
          const float*, // B[]
          float*        // returns a*A[] + B[]
          );

ON_DECL
double ON_ArrayDotProduct( // returns AoB
          int,           // size of arrays (can be zero)
          const double*, // A[]
          const double*  // B[]
          );

ON_DECL
double ON_ArrayDotDifference( // returns A o ( B - C )
          int,           // size of arrays (can be zero)
          const double*, // A[]
          const double*, // B[]
          const double*  // C[]
          );

ON_DECL
double ON_ArrayMagnitude( // returns sqrt(AoA)
          int,           // size of arrays (can be zero)
          const double*  // A[]
          );

ON_DECL
double ON_ArrayMagnitudeSquared( // returns AoA
          int,           // size of arrays (can be zero)
          const double*  // A[]
          );

ON_DECL
double ON_ArrayDistance( // returns sqrt((A-B)o(A-B))
          int,           // size of arrays (can be zero)
          const double*, // A[]
          const double*  // B[]
          );

ON_DECL
double ON_ArrayDistanceSquared( // returns (A-B)o(A-B)
          int,           // size of arrays (can be zero)
          const double*, // A[]
          const double*  // B[]
          );

ON_DECL
void   ON_ArrayScale( 
          int,           // size of arrays (can be zero)
          double,        // a
          const double*, // A[]
          double*        // returns a*A[]
          );

ON_DECL
void   ON_Array_aA_plus_B( 
          int,           // size of arrays (can be zero)
          double,        // a
          const double*, // A[]
          const double*, // B[]
          double*        // returns a*A[] + B[]
          );

ON_DECL
int    ON_SearchMonotoneArray( // find a value in an increasing array
          // returns  -1: t < array[0]
          //           i: array[i] <= t < array[i+1] ( 0 <= i < length-1 )
          //    length-1: t == array[length-1]
          //      length: t >= array[length-1]
          const double*, // array[]
          int,           // length of array
          double         // t = value to search for
          );

ON_DECL
double ON_BinomialCoefficient( 
                    // returns (i+j)!/(i! * j!)  if 0 <= i and 0 <= j
                    //         0 if i < 0 or j < 0
                    int, int // i,j
                    );

ON_DECL
BOOL ON_GetParameterTolerance(
        double, double, // domain
        double,          // parameter in domain
        double*, double* // parameter tolerance (tminus, tplus) returned here
        );


ON_DECL
BOOL ON_IsValidPointList(
        int,  // dim
        BOOL, // TRUE for homogeneous rational points
        int,  // count
        int,  // stride
        const float*
        );

ON_DECL
BOOL ON_IsValidPointList(
        int,  // dim
        BOOL, // TRUE for homogeneous rational points
        int,  // count
        int,  // stride
        const double*
        );

ON_DECL
BOOL ON_IsValidPointGrid(
        int,  // dim
        BOOL, // TRUE for homogeneous rational points
        int, int, // point_count0, point_count1,
        int, int, // point_stride0, point_stride1,
        const double*
        );

ON_DECL
BOOL ON_ReversePointList(
        int,  // dim
        BOOL, // TRUE for homogeneous rational points
        int,  // count
        int,  // stride
        double*
        );

ON_DECL
BOOL ON_ReversePointGrid(
        int,  // dim
        BOOL, // TRUE for homogeneous rational points
        int, int, // point_count0, point_count1,
        int, int, // point_stride0, point_stride1,
        double*,
        int       // dir = 0 or 1
        );

ON_DECL
BOOL ON_SwapPointListCoordinates( 
        int, // count
        int, // stride
        float*,
        int, int // coordinates to swap
        );

ON_DECL
BOOL ON_SwapPointListCoordinates( 
        int, // count
        int, // stride
        double*,
        int, int // coordinates to swap
        );

ON_DECL
BOOL ON_SwapPointGridCoordinates(
        int, int, // point_count0, point_count1,
        int, int, // point_stride0, point_stride1,
        double*,
        int, int // coordinates to swap
        );

ON_DECL
BOOL ON_TransformPointList(
        int,  // dim
        BOOL, // TRUE for homogeneous rational points
        int,  // count
        int,  // stride
        float*,
        const ON_Xform&
        );

ON_DECL
BOOL ON_TransformPointList(
        int,  // dim
        BOOL, // TRUE for homogeneous rational points
        int,  // count
        int,  // stride
        double*,
        const ON_Xform&
        );

ON_DECL
BOOL ON_TransformPointGrid(
        int,      // dim
        BOOL,     // TRUE for homogeneous rational points
        int, int, // point_count0, point_count1,
        int, int, // point_stride0, point_stride1,
        double*,
        const ON_Xform&
        );

ON_DECL
BOOL ON_TransformVectorList(
       int,  // dim
       int,  // count
       int,  // stride
       float*,
       const ON_Xform&
       );

ON_DECL
BOOL ON_TransformVectorList(
       int,  // dim
       int,  // count
       int,  // stride
       double*,
       const ON_Xform&
       );

ON_DECL
int ON_ComparePoint( // returns 
                              // -1: first < second
                              //  0: first == second
                              // +1: first > second
          int,           // dim (>=0)
          BOOL,          // TRUE for rational CVs
          const double*, // first CV
          const double*  // secont CV
          );

ON_DECL
int ON_ComparePointList( // returns 
                              // -1: first < second
                              //  0: first == second
                              // +1: first > second
          int,           // dim (>=0)
          BOOL,          // TRUE for rational CVs
          int,           // count
          // first point list
          int,           // stride
          const double*, // point
          // second point list
          int,           // stride
          const double*  // point
          );

ON_DECL
BOOL ON_IsPointListClosed(
       int,  // dim
       int,  // TRUE for homogeneos rational points
       int,  // count
       int,  // stride
       const double*
       );

ON_DECL
BOOL ON_IsPointGridClosed(
        int,  // dim
        BOOL, // TRUE for homogeneous rational points
        int, int, // point_count0, point_count1,
        int, int, // point_stride0, point_stride1,
        const double*,
        int       // dir = 0 or 1
       );

ON_DECL
int ON_SolveQuadraticEquation( // solve a*X^2 + b*X + c = 0
        // returns 0: two distinct real roots (r0 < r1)
        //         1: one real root (r0 = r1)
        //         2: two complex conjugate roots (r0 +/- (r1)*sqrt(-1))
        //        -1: failure - a = 0, b != 0        (r0 = r1 = -c/b)
        //        -2: failure - a = 0, b  = 0 c != 0 (r0 = r1 = 0.0)
        //        -3: failure - a = 0, b  = 0 c  = 0 (r0 = r1 = 0.0)
       double, double, double, // a, b, c
       double*, double*        // roots r0 and r1 returned here
       );

ON_DECL
BOOL ON_SolveTriDiagonal( // solve TriDiagMatrix( a,b,c )*X = d
        int,               // dimension of d and X (>=1)
        int,               // number of equations (>=2)
        double*,           // a[n-1] = sub-diagonal (a is modified)
        const double*,     // b[n] = diagonal
        double*,           // c[n-1] = supra-diagonal
        const double*,     // d[n*dim]
        double*            // X[n*dim] = unknowns
        );

ON_DECL
int ON_Solve2x2( // returns rank - if rank != 2, system is under determined
        // If rank = 2, then solution to a00*x0 + a01*x1 = b0, a10*x0 + a11*x1 = b1 is returned
        double, double,   // a00 a01 = first row of 2x2 matrix
        double, double,   // a10 a11 = second row of 2x2 matrix
        double, double,   // b0 b1
        double*, double*, // x0, x1 if not NULL, then solution is returned here
        double*           // if not NULL, then pivot_ratio returned here
        );

ON_DECL
int ON_Solve3x2( // returns rank - if rank != 2, system is under determined
        //      If rank = 2, then the solution is
        //        (*x_addr)*{col0} + (*y_addr)*{col1}
        //        + (*err_addr)*({col0 X col1}/|col0 X col1|)
        //        = {d0,d1,d2}.
        const double[3], // col0[3], 
        const double[3], // col1[3], 
        double,  // d0, 
        double,  // d1, 
        double,  // d2,
        double*, // x_addr, 
        double*, // y_addr, 
        double*, // err_addr, 
        double*  // pivot_ratio
        );

ON_DECL
int ON_Solve3x3( // returns rank - if rank != 3, system is under determined
        const double row0[3], 
        const double row1[3], 
        const double row2[3],
        double d0, double d1, double d2,
        double* x_addr, double* y_addr, double* z_addr,
        double* pivot_ratio
        );

ON_DECL
BOOL ON_EvJacobian( // return FALSE if determinant is (nearly) singular
        double, // ds o ds
        double, // ds o dt
        double, // dt o dt
        double* // jacobian = determinant ( ds_o_ds ds_o_dt / ds_o_dt dt_o_dt )
        );

ON_DECL
BOOL ON_EvNormal(
        int, // limit_dir 0=default,1=from quadrant I, 2 = from quadrant II, ...
        const ON_3dVector&, const ON_3dVector&, // first partials (Du,Dv)
        const ON_3dVector&, const ON_3dVector&, const ON_3dVector&, // optional second partials (Duu, Duv, Dvv)
        ON_3dVector& // unit normal returned here
        );

ON_DECL
BOOL ON_EvCurvature( // returns FALSE if first derivtive is zero
        const ON_3dVector&, // first derivative
        const ON_3dVector&, // second derivative
        ON_3dVector&,       // Unit tangent returned here
        ON_3dVector&        // Curvature returned here
        );

ON_DECL
BOOL ON_EvPrincipalCurvatures( 
        const ON_3dVector&, // Ds,
        const ON_3dVector&, // Dt,
        const ON_3dVector&, // Dss,
        const ON_3dVector&, // Dst,
        const ON_3dVector&, // Dtt,
        const ON_3dVector&, // N,   // unit normal to surface (use ON_EvNormal())
        double*, // gauss,  // = Gaussian curvature = kappa1*kappa2
        double*, // mean,   // = mean curvature = (kappa1+kappa2)/2
        double*, // kappa1, // = largest principal curvature value (may be negative)
        double*, // kappa2, // = smallest principal curvature value (may be negative)
        ON_3dVector&, // K1,     // kappa1 unit principal curvature direction
        ON_3dVector&  // K2      // kappa2 unit principal curvature direction
                        // output K1,K2,N is right handed frame
        );

ON_DECL
ON_3dVector ON_NormalCurvature( 
        const ON_3dVector&, // surface 1rst partial (Ds)
        const ON_3dVector&, // surface 1rst partial (Dt)
        const ON_3dVector&, // surface 1rst partial (Dss)
        const ON_3dVector&, // surface 1rst partial (Dst)
        const ON_3dVector&, // surface 1rst partial (Dtt)
        const ON_3dVector&, // surface unit normal
        const ON_3dVector&  // unit tangent direction
        );


ON_DECL
void ON_Sort( // sort an index
        ON::sort_algorithm,          // ON::heap_sort or ON::quick_sort, 
        int*,                        // index[] array of count integers, 
        const void*,                 // data buffer
        size_t,                      // count 
        size_t,                      // sizeof_element width,
        int (*)(const void*,const void*) // int compar(const void*,const void*)
        );

ON_DECL
void ON_hsort( // heap sort analogue to qsort()
        void*,                                // data buffer
        size_t,                               // number of elements
        size_t,                               // sizeof_element width,
        int (*)(const void*,const void*)      // int compar(const void*,const void*)
        );

ON_DECL
void ON_SortDoubleArray( // heap sort array of doubles in place
        ON::sort_algorithm, // ON::heap_sort or ON::quick_sort, 
        double*,  // array of doubles
        size_t    // length of array
        );

ON_DECL
void ON_SortIntArray(
        ON::sort_algorithm, // ON::heap_sort or ON::quick_sort, 
        int*,     // array of doubles
        size_t    // length of array
        );

ON_DECL
void ON_SortStringArray(
        ON::sort_algorithm, // ON::heap_sort or ON::quick_sort, 
        char**,   // array of doubles
        size_t    // length of array
        );


// These simple intersectors are fast and detect transverse intersections.
// If the intersection is not a simple transverse case, then they
// return FALSE and you will have to use one of the slower but fancier
// models.

ON_DECL
BOOL ON_Intersect( // returns closest points between the two lines
          const ON_Line&, 
          const ON_Line&, 
          double*, // parameter on first line
          double*  // parameter on second line
          );

ON_DECL
BOOL ON_Intersect( // Returns FALSE unless intersection is a single point
                   // If returned parameter is < 0 or > 1, then the line
                   // segment between line.m_point[0] and line.m_point[1]
                   // does not intersect the plane
          const ON_Line&, 
          const ON_Plane&, 
          double* // parameter on line
          );

ON_DECL
BOOL ON_Intersect( const ON_Plane&, const ON_Plane&, 
                   ON_Line& // intersection line is returned here
                   );

ON_DECL
BOOL ON_Intersect( const ON_Plane&, const ON_Plane&, const ON_Plane&,
                  ON_3dPoint& // intersection point is returned here
                  );

ON_DECL
int ON_Intersect( // returns 0 = no intersections, 
                  // 1 = intersection = single point, 
                  // 2 = intersection = circle
                  // If 0 is returned, returned circle has radius=0
                  // and center = point on sphere closest to plane.
                  // If 1 is returned, intersection is a single
                  // point and returned circle has radius=0
                  // and center = intersection point on sphere.
                 const ON_Plane&, const ON_Sphere&, ON_Circle&
                  );

ON_DECL
int ON_Intersect( // returns 0 = no intersections, 
                  // 1 = one intersection, 
                  // 2 = 2 intersections
                  // If 0 is returned, first point is point 
                  // on line closest to sphere and 2nd point is the point
                  // on the sphere closest to the line.
                  // If 1 is returned, first point is obtained by evaluating
                  // the line and the second point is obtained by evaluating
                  // the sphere.
                 const ON_Line&, const ON_Sphere&,
                  ON_3dPoint&, ON_3dPoint& // intersection point(s) returned here
                  );

ON_DECL
int ON_Intersect( // returns 0 = no intersections, 
                  // 1 = one intersection, 
                  // 2 = 2 intersections
                  // 3 = line lies on cylinder
                  // If 0 is returned, first point is point 
                  // on line closest to cylinder and 2nd point is the point
                  // on the cylinder closest to the line.
                  // If 1 is returned, first point is obtained by evaluating
                  // the line and the second point is obtained by evaluating
                  // the cylinder.
                  //
                  // The value of cylinder.IsFinite() determines if the
                  // intersection is performed on the finite or infinite cylinder.
                  const ON_Line&, const ON_Cylinder&,
                  ON_3dPoint&, ON_3dPoint& // intersection point(s) returned here
                  );

ON_DECL
int ON_Intersect( // returns 0 = line is coincident with circle's axis
                  // 1 = one intersection, 
                  // 2 = 2 intersections
                  //
                  // If the line misses the circle, the closest point(s)
                  // are returned.
                  const ON_Line&, const ON_Circle&,
                  double*,     // line paramter of first intersection point
                  ON_3dPoint&, // first intersection point on circle
                  double*,     // line parameter of second intersection point
                  ON_3dPoint&  // second intersection point on circle
                  );

ON_DECL
int ON_Intersect( // returns 0 = line is coincident with arc's axis
                  // 1 = one intersection, 
                  // 2 = 2 intersections
                  //
                  // If the line misses the arc, the closest point(s)
                  // are returned.
                  const ON_Line&, const ON_Arc&,
                  double*,     // line paramter of first intersection point
                  ON_3dPoint&, // first intersection point on arc
                  double*,     // line parameter of second intersection point
                  ON_3dPoint&  // second intersection point on arc
                  );

ON_DECL
int ON_ArePointsOnLine(  // 0 = no, 1 = yes, 2 = points are coincident and on line
        int, // dimension of points
        int, // is_rat = TRUE if homogeneous rational
        int, // count = number of points
        int, // stride ( >= is_rat?(dim+1) :dim)
        const double*, // point array
        const ON_BoundingBox&, // if needed, use ON_GetBoundingBox(dim,is_rat,count,stride,point)
        const ON_Line&,
        double         // tolerance (if 0.0, a tolerance based on bounding box size is used)
        );

ON_DECL
int ON_ArePointsOnPlane(  // 0 = no, 1 = yes, 2 = points are coincident and on line
        int, // dimension of points
        int, // is_rat = TRUE if homogeneous rational
        int, // count = number of points
        int, // stride ( >= is_rat?(dim+1) :dim)
        const double*, // point array
        const ON_BoundingBox&, // if needed, use ON_GetBoundingBox(dim,is_rat,count,stride,point)
        const ON_Plane&,
        double         // tolerance (if 0.0, a tolerance based on bounding box size is used)
        );

ON_DECL
BOOL ON_EvaluateQuotientRule(
                  int,           // dimension
                  int,           // number of derivatives to compute (>=0)
                  int,           // answer_stride (>=dimension)
                  double*        // answer[] array of length (ndir+1)*answer_stride
                  );

ON_DECL
BOOL ON_EvaluateQuotientRule2( 
                  int,           // dimension
                  int,           // number of derivatives to compute (>=0)
                  int,           // answer_stride (>=dimension)
                  double*        // answer[] array of length (ndir+1)*answer_stride
                  );

ON_DECL
BOOL ON_GetPolylineLength(
        int,           // dimension of points
        BOOL,          // bIsRational TRUE if points are homogeneous rational
        int,           // number of points
        int,           // stride between points
        const double*, // points
        double*        // length returned here
        );


#endif
