/* $Header: /src2/opennurbs/opennurbs_matrix.h 6     1/03/01 4:59p Dalelear $ */
/* $NoKeywords: $ */
/*
//
// Copyright (c) 1993-2001 Robert McNeel & Associates. All rights reserved.
// Rhinoceros is a registered trademark of Robert McNeel & Assoicates.
//
// THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
// ALL IMPLIED WARRANTIES OF FITNESS FOR ANY PARTICULAR PURPOSE AND OF
// MERCHANTABILITY ARE HEREBY DISCLAIMED.
//				
// For complete openNURBS copyright information see <http://www.opennurbs.org>.
//
////////////////////////////////////////////////////////////////
*/

#if !defined(ON_MATRIX_INC_)
#define ON_MATRIX_INC_

class ON_Xform;

class ON_CLASS ON_Matrix
{
public:
  ON_Matrix();
  ON_Matrix( 
     int, // number of rows
     int  // number of columns
     );
  ON_Matrix( // see ON_Matrix::Create(int,int,int,int) for details
     int, // first valid row index
     int, // last valid row index
     int, // first valid column index
     int  // last valid column index
     );
  ON_Matrix( const ON_Xform& );
  ON_Matrix( const ON_Matrix& );
  virtual ~ON_Matrix();
  void EmergencyDestroy(); // call if memory pool used matrix by becomes invalid

  // ON_Matrix[i][j] = value at row i and column j
  //           0 <= i < RowCount()
  //           0 <= j < ColCount()
  double* operator[](int);
  const double* operator[](int) const;

  ON_Matrix& operator=(const ON_Matrix&);
  ON_Matrix& operator=(const ON_Xform&);

  BOOL IsValid() const;
  int IsSquare() const; // returns 0 for no and m_row_count (= m_col_count) for yes
  int RowCount() const;
  int ColCount() const;
  int MinCount() const; // smallest of row and column count
  int MaxCount() const; // largest of row and column count

  void RowScale(int,double); 
  void ColScale(int,double);
  void RowOp(int,double,int);
  void ColOp(int,double,int);

  BOOL Create(
     int, // number of rows
     int  // number of columns
     );

  BOOL Create( // E.g., Create(1,5,1,7) creates a 5x7 sized matrix that with
               // "top" row = m[1][1],...,m[1][7] and "bottom" row
               // = m[5][1],...,m[5][7].  The result of Create(0,m,0,n) is
               // identical to the result of Create(m+1,n+1).
     int, // first valid row index
     int, // last valid row index
     int, // first valid column index
     int  // last valid column index
     );

  void Destroy();

  void Zero();

  void SetDiagonal(double); // sets diagonal value and zeros off diagonal values
  void SetDiagonal(const double*); // sets diagonal values and zeros off diagonal values
  void SetDiagonal(int, const double*); // sets size to count x count and diagonal values and zeros off diagonal values
  void SetDiagonal(const ON_SimpleArray<double>&); // sets size to length X lengthdiagonal values and zeros off diagonal values

  BOOL Transpose();

  BOOL SwapRows( int, int ); // ints are row indices to swap
  BOOL SwapCols( int, int ); // ints are col indices to swap
  BOOL Invert( 
          double // zero tolerance
          );

  // safe arithmetic
  BOOL Multiply( const ON_Matrix&, const ON_Matrix& );
  BOOL Add( const ON_Matrix&, const ON_Matrix& );
  BOOL Scale( double );


  // row reduction
  int RowReduce( // returns rank
    double,   // >= 0.0 (zero tolerance for pivot test)
    double&,  // determinant is returned
    double&   // pivot is returned
    ); 

  int RowReduce( // returns rank
    double,        // >= 0.0 (zero tolerance for pivot test)
    double*,       // B[m_row_count] array that is reduced with matrix
    double* = NULL // if not NULL, then smallest pivot is returned
    ); 

  int RowReduce( // returns rank
    double,        // >= 0.0 (zero tolerance for pivot test)
    ON_3dPoint*, // B[m_row_count] array that is reduced with matrix
    double* = NULL // if not NULL, then smallest pivot is returned
    ); 

  int RowReduce( // returns rank
    double,        // >= 0.0 (zero tolerance for pivot test)
    int,         // pt_dim
    int,         // pt_stride
    double*,     // pt[m_row_count*pt_stride] array of pt_dim pts
                 // that is reduced with the matrix
    double* = NULL // if not NULL, then smallest pivot is returned
    ); 

  BOOL
  BackSolve( // matrix must be upper triangular, have unit diagonal,
             // and m_col_count <= m_row_count.  Rows below upper square
             // are ignored.
    double,        // >= 0.0 (zero tolerance)
    int,           // size of B ( >= m_col_count ) B[m_col_count,...,Bsize-1]
                   // are tested to make sure fabs() <= zero tolerance.
    const double*, // B[]
    double*        // X[m_col_count]
      ) const;

  BOOL
  BackSolve( // matrix must be upper triangular, have unit diagonal,
             // and m_col_count <= m_row_count.  Rows below upper square
             // are ignored.
    double,        // >= 0.0 (zero tolerance)
    int,           // size of B ( >= m_col_count ) B[m_col_count,...,Bsize-1]
                   // are tested to make sure fabs() <= zero tolerance.
    const ON_3dPoint*, // B[]
    ON_3dPoint*        // X[m_col_count]
      ) const;

  BOOL IsRowOrthoganal() const;
  BOOL IsRowOrthoNormal() const;

  BOOL IsColOrthoganal() const;
  BOOL IsColOrthoNormal() const;

  double** m; // m[i][j] = value at row i and column j
              //           0 <= i < RowCount()
              //           0 <= j < ColCount()
private:
  int m_row_count;
  int m_col_count;
  ON_SimpleArray<double*> m_row; // m_M[i][j] = row i and column j
  ON_SimpleArray<double> m_a; // array of at least doubles
};

#endif
