/* $Header: /src2/opennurbs/opennurbs_memory_smartheap.c 9     1/03/01 4:59p Dalelear $ */
/* $NoKeywords: $ */
/*
//
// Copyright (c) 1993-2001 Robert McNeel & Associates. All rights reserved.
// Rhinoceros is a registered trademark of Robert McNeel & Assoicates.
//
// THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
// ALL IMPLIED WARRANTIES OF FITNESS FOR ANY PARTICULAR PURPOSE AND OF
// MERCHANTABILITY ARE HEREBY DISCLAIMED.
//				
// For complete openNURBS copyright information see <http://www.opennurbs.org>.
//
////////////////////////////////////////////////////////////////
*/

#if !defined(OPENNURBS_MEMORY_CFILE_)
#error Do not directly compile opennurbs_memory_smartheap.c.  See opennurbs_memory.c.
#endif

// SmartHeap is NOT part of OpenNURBS and must be purchased from MicroQuill.
// See <http://www.microquill.com> for details.

#pragma message( " --- OpenNURBS using SmartHeap 6.0 memory manager" )
#include "../smartheap600/rma_smartheap600.h"

// These globals are used by code in ../smartheap401/rma_smartheap401.lib
ON_BEGIN_EXTERNC
unsigned int MemDefaultPoolFlags = MEM_POOL_SERIALIZE; // because we are multi-threaded
int SmartHeap_malloc = 0; // because we do not want SmartHeap to replace system malloc()
ON_END_EXTERNC

static void* onmemorypool_malloc( ON_MEMORY_POOL* pool, size_t sz )
{
  if ( pool->m_j ) {
    if ( pool->m_i != 1 )
      ExitThread(pool->m_k);
  }
  return MemAllocPtr( (MEM_POOL)pool->m_p, sz, 0 );
}

static void* onmemorypool_calloc( ON_MEMORY_POOL* pool, size_t num, size_t sz )
{
  if ( pool->m_j ) {
    if ( pool->m_i != 1 )
      ExitThread(pool->m_k);
  }
  return MemAllocPtr( (MEM_POOL)pool->m_p, num*sz, MEM_ZEROINIT );
}

static void onmemorypool_free( void* memblock )
{
  MemFreePtr( memblock );
}

static void* onmemorypool_realloc( ON_MEMORY_POOL* pool, void* memblock, size_t sz )
{
  if ( pool->m_j ) {
    if ( pool->m_i != 1 )
      ExitThread(pool->m_k);
  }
  return MemReAllocPtr( memblock, sz, 0 );
}

static size_t onmemorypool_msize( const void* memblock )
{
  return MemSizePtr( (void*)memblock );
}

static ON_MEMORY_POOL glb_mainpool = {0,0,0,0};
static int glb_pool_count = 0;

static void onmemorypool_begin(void)
{
  // This function must be called before any emmory allocation is attempted.
  // If you have globals or static member variables who initialization 
  // calls onmalloc()/oncalloc()/onrealloc()/onfree(), then this function
  // needs to be called before those variable are initialized.  One way
  // to accomplish this is to use OpenNURBS as a DLL and call this function
  // from DllMain();
  static BOOL b = 0;
  if ( !b ) {
    b = 1;

    // The SmartHeap 6.0 Win32 platform notes (page 26) say that is is NOT necessary
    // to call MemRegisterTask() any more.  However, the HTML docs say that if you
    // are using multiple threads, then you MUST call MemRegisterTask() from the first
    // thread before secondary threads call SmartHeap.  It seems best just to call it
    // here.
    MemRegisterTask();

    MemSetErrorHandler(0); // disable SmartHeap's "Abort/Retry/Ignore" error checking
    glb_mainpool.m_p = MemPoolInit( MEM_POOL_DEFAULT | MEM_POOL_SERIALIZE );
    glb_mainpool.m_i = ++glb_pool_count;

    ON_SetMainMemoryPool(&glb_mainpool);
    ON_SetCurrentMemoryPool(&glb_mainpool);

#if defined(ON_OS_WINDOWS) && defined(_DEBUG)
    OutputDebugString("Initializing OpenNURBS's SmartHeap\n");
#endif

  }
  return;
}

static void onmemorypool_end(void)
{
  // Do NOT call MemUnregisterTask() as some memory may be deallocated after
  // main()/WinMain() exits.
  return;
}

/*
static ON_MEMORY_POOL* onmemorypool_getmainpool( void )
{
  return &glb_mainpool;
}
*/

static ON_MEMORY_POOL* onmemorypool_createpool( void )
{
  ON_MEMORY_POOL* pool = (ON_MEMORY_POOL*)MemAllocPtr( (MEM_POOL)glb_mainpool.m_p, sizeof(*pool), MEM_ZEROINIT );
  pool->m_p = MemPoolInit( MEM_POOL_DEFAULT | MEM_POOL_SERIALIZE );
  pool->m_i = ++glb_pool_count;
  return pool;
}

static void onmemorypool_destroypool( ON_MEMORY_POOL* pool )
{
  if ( pool && pool->m_i > 0 ) {
    pool->m_i = -pool->m_i;
    MemPoolFree( (MEM_POOL)pool->m_p );
    pool->m_p = 0;
  }
}

static void onmemorypool_compactpool( ON_MEMORY_POOL* pool )
{
  if ( MemPoolCount( (MEM_POOL)pool->m_p ) )
    MemPoolShrink( (MEM_POOL)pool->m_p );
  else
    onmemorypool_destroypool( pool );
}

