/* $Header: /src2/opennurbs/opennurbs_mesh.cpp 53    7/24/01 4:02p Dalelear $ */
/* $NoKeywords: $ */
/*
//
// Copyright (c) 1993-2001 Robert McNeel & Associates. All rights reserved.
// Rhinoceros is a registered trademark of Robert McNeel & Assoicates.
//
// THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
// ALL IMPLIED WARRANTIES OF FITNESS FOR ANY PARTICULAR PURPOSE AND OF
// MERCHANTABILITY ARE HEREBY DISCLAIMED.
//				
// For complete openNURBS copyright information see <http://www.opennurbs.org>.
//
////////////////////////////////////////////////////////////////
*/

#include "opennurbs.h"

ON_OBJECT_IMPLEMENT(ON_Mesh,ON_Geometry,"4ED7D4E4-E947-11d3-BFE5-0010830122F0");

/*
ON_MeshEdge& ON_MeshEdge::operator=( const ON_MeshEdge& src )
{
  int fc = src.m_fcount+(src.m_fcount%1);
  if ( fc <= 2 ) {
    if ( m_fi && m_fi != m_private_fi ) onfree((void*)m_fi);
    m_fi = m_private_fi;
    fc = 2;
  }
  else {
    if ( (m_fcount+(m_fcount%1)) != fc ) {
      if ( m_fi == m_private_fi )
        m_fi = 0;
      m_fi = (int*)onrealloc(m_fi,fc*sizeof(*m_fi));
    }
  }
  memcpy(m_fi,src.m_fi,fc*sizeof(*m_fi));
  m_fcount=src.m_fcount;
  return *this;
}

void ON_MeshEdge::AppendFaceIndex(int face_index)
{
  if ( m_fcount>0 && !(m_fcount%1) ) {
    if ( m_fi == m_private_fi ) {
      m_fi = (int*)onmalloc((m_fcount+2)*sizeof(*m_fi));
      m_fi[0] = m_private_fi[0];
      m_fi[1] = m_private_fi[1];
    }
    else {
      m_fi = (int*)onrealloc(m_fi,(m_fcount+2)*sizeof(*m_fi));
    }
  }
  m_fi[m_fcount++] = face_index;
}
*/

BOOL 
ON_MeshFace::IsValid(int vertex_count) const
{
  return (    vi[0] >= 0 && vi[0] < vertex_count
           && vi[1] >= 0 && vi[1] < vertex_count
           && vi[2] >= 0 && vi[2] < vertex_count
           && vi[3] >= 0 && vi[3] < vertex_count
           && vi[0] != vi[1] && vi[1] != vi[2] && vi[2] != vi[0] 
           && (vi[2]==vi[3]||(vi[0] != vi[3] && vi[1] != vi[3])) );
}


ON_Mesh::ON_Mesh() : m_srf_tex_rotate(0), 
                     m_parent(0), m_material_index(-1),
                     m_mesh_parameters(0), 
                     m_invalid_count(0), m_quad_count(0), m_triangle_count(0),
                     m_closed(-1)

{
  m_top.m_mesh = this;
  m_texture_scale[0] = 1.0;
  m_texture_scale[1] = 1.0;

  m_kstat[0] = 0;
  m_kstat[1] = 0;
  m_kstat[2] = 0;
  m_kstat[3] = 0;
  //m_kstat[4] = 0;
  //m_kstat[5] = 0;
  //m_kstat[6] = 0;
  InvalidateBoundingBoxes();
}

ON_Mesh::ON_Mesh(
    int initial_facet_capacity,  // initial facet array capacity
    int initial_vertex_capacity,  // initial vertex array capacity
    BOOL bHasVertexNormals, // TRUE if mesh has unit vertex normals
    BOOL bHasTextureCoordinates // TRUE if mesh has texture coordinates
    )
    : m_V(initial_vertex_capacity),
      m_F(initial_facet_capacity),
      m_N(bHasVertexNormals?initial_vertex_capacity:0),
      m_T(bHasTextureCoordinates?initial_vertex_capacity:0),
      m_srf_tex_rotate(0),
      m_parent(0), m_material_index(-1),
      m_mesh_parameters(0),
      m_invalid_count(0), m_quad_count(0), m_triangle_count(0),
      m_closed(-1)
{
  m_top.m_mesh = this;
  m_texture_scale[0] = 1.0;
  m_texture_scale[1] = 1.0;

  m_kstat[0] = 0;
  m_kstat[1] = 0;
  m_kstat[2] = 0;
  m_kstat[3] = 0;
  //m_kstat[4] = 0;
  //m_kstat[5] = 0;
  //m_kstat[6] = 0;
  InvalidateBoundingBoxes();
}

ON_Mesh::ON_Mesh( const ON_Mesh& src ) 
        : m_srf_tex_rotate(0),
          m_parent(0), m_material_index(-1),
          m_mesh_parameters(0), 
          m_invalid_count(0), m_quad_count(0), m_triangle_count(0),
          m_closed(-1)
{
  m_top.m_mesh = this;
  m_texture_scale[0] = 1.0;
  m_texture_scale[1] = 1.0;

  m_kstat[0] = 0;
  m_kstat[1] = 0;
  m_kstat[2] = 0;
  m_kstat[3] = 0;
  //m_kstat[4] = 0;
  //m_kstat[5] = 0;
  //m_kstat[6] = 0;
  InvalidateBoundingBoxes();
  *this = src;
}

ON_Mesh& ON_Mesh::operator=( const ON_Mesh& src )
{
  if ( this != &src ) {
    Destroy();
    ON_Geometry::operator=(src);

    m_V = src.m_V;
    m_F = src.m_F;
    m_N = src.m_N;
    m_FN =src.m_FN;
    m_T = src.m_T;

    m_tex_domain[0] = src.m_tex_domain[0];
    m_tex_domain[1] = src.m_tex_domain[1];
    m_srf_domain[0] = src.m_srf_domain[0];
    m_srf_domain[1] = src.m_srf_domain[1];
    m_texture_scale[0] = src.m_texture_scale[0];
    m_texture_scale[1] = src.m_texture_scale[1];
    m_srf_tex_rotate = src.m_srf_tex_rotate;

    m_K = src.m_K;
    m_C = src.m_C;

    m_parent         = src.m_parent;
    m_material_index = src.m_material_index;

    if ( m_mesh_parameters ) {
      delete m_mesh_parameters;
      m_mesh_parameters = 0;
    }
    if ( src.m_mesh_parameters )
      m_mesh_parameters = new ON_MeshParameters(*src.m_mesh_parameters);

    m_invalid_count = src.m_invalid_count;
    m_quad_count = src.m_quad_count;
    m_triangle_count = src.m_triangle_count;

    m_closed = src.m_closed;

    memcpy(m_vbox,src.m_vbox,sizeof(m_vbox));
    memcpy(m_nbox,src.m_nbox,sizeof(m_nbox));
    memcpy(m_tbox,src.m_tbox,sizeof(m_tbox));

    int i;
    for ( i = 0; i < 4; i++ ) {
      if ( m_kstat[i] ) {
        delete m_kstat[i];
        m_kstat[i] = 0;
      }
      if ( src.m_kstat[i] ) {
        m_kstat[i] = new ON_MeshCurvatureStats(*src.m_kstat[i]);
      }
    }
  }
  return *this;
}


ON_Mesh::~ON_Mesh()
{
  Destroy();
  m_top.m_mesh = 0;
}

void ON_Mesh::Destroy()
{
  m_V.Destroy();
  m_F.Destroy();
  m_N.Destroy();
  m_FN.Destroy();
  m_T.Destroy();
  m_K.Destroy();
  m_C.Destroy();
  m_top.Destroy();
  m_top.m_mesh = this;
  InvalidateBoundingBoxes();
  DeleteMeshParameters();
  m_parent = 0;
  m_material_index = -1;
  m_closed = 0;
}

void ON_Mesh::EmergencyDestroy()
{
  int i;
  m_mesh_parameters = 0;
  for ( i = 0; i < 4; i++ ) {
    m_kstat[i] = 0;
  }
  m_V.EmergencyDestroy();
  m_F.EmergencyDestroy();
  m_N.EmergencyDestroy();
  m_FN.EmergencyDestroy();
  m_T.EmergencyDestroy();
  m_K.EmergencyDestroy();
  m_C.EmergencyDestroy();
  m_top.EmergencyDestroy();
}

BOOL ON_Mesh::IsValid() const
{
  const int facet_count = FaceCount();
  const int vertex_count = VertexCount();
  int fi, vi;

  if (facet_count < 1) 
    return FALSE;
  if ( vertex_count < 3 )
    return FALSE;

  if ( m_N.Capacity() > 0 && m_N.Count() != vertex_count )
    return FALSE;
  if ( m_T.Capacity() > 0 && m_T.Count() != vertex_count )
    return FALSE;

  for ( fi = 0; fi < facet_count; fi++ ) {
    if ( !m_F[fi].IsValid( vertex_count ) ) {
      ON_ERROR("ON_MESH has invalid face.");
      return FALSE;
    }
 }

  if ( HasVertexNormals() ) {
    float x;
    for ( vi = 0; vi < vertex_count; vi++ ) {
      x = m_N[vi][0]*m_N[vi][0] + m_N[vi][1]*m_N[vi][1] + m_N[vi][2]*m_N[vi][2];
      if ( x < 0.99 || x > 1.01 )
        return FALSE;
    }
  }

  return TRUE;
}

void ON_Mesh::Dump( ON_TextLog& dump ) const
{
  dump.Print("ON_Mesh: vertex count = %d  facet count = %d\n",VertexCount(), FaceCount());
}

BOOL ON_Mesh::WriteFaceArray( int vcount, int fcount, ON_BinaryArchive& file ) const
{
  unsigned char  cvi[4];
  unsigned short svi[4];
  const int* vi;
  int i_size = 0;
  if ( vcount < 256 ) {
    i_size = 1; // unsigned chars
  }
  else if (vcount < 65536 ) {
    i_size = 2; // unsigned shorts
  }
  else {
    i_size = 4; // 4 byte ints
  }

  BOOL rc = file.WriteInt( i_size );
  int i;
  switch(i_size) {
  case 1:
    for ( i = 0; i < fcount && rc ; i++ ) {
      vi = m_F[i].vi;
      cvi[0] = (unsigned char)vi[0];
      cvi[1] = (unsigned char)vi[1];
      cvi[2] = (unsigned char)vi[2];
      cvi[3] = (unsigned char)vi[3];
      rc = file.WriteChar( 4, cvi );
    }
    break;
  case 2:
    for ( i = 0; i < fcount && rc ; i++ ) {
      vi = m_F[i].vi;
      svi[0] = (unsigned short)vi[0];
      svi[1] = (unsigned short)vi[1];
      svi[2] = (unsigned short)vi[2];
      svi[3] = (unsigned short)vi[3];
      rc = file.WriteShort( 4, svi );
    }
    break;
  case 4:
    for ( i = 0; i < fcount && rc ; i++ ) {
      rc = file.WriteInt( 4, m_F[i].vi );
    }
    break;
  }

  return rc;
}

BOOL ON_Mesh::ReadFaceArray( int vcount, int fcount, ON_BinaryArchive& file )
{
  unsigned char  cvi[4];
  unsigned short svi[4];
  unsigned int* vi;
  int i_size = 0;

  if ( m_F.Capacity() < fcount )
    m_F.SetCapacity(fcount);
  BOOL rc = file.ReadInt( &i_size );
  int i = 0;
  switch(i_size) {
  case 1:
    for ( i = 0; i < fcount && rc ; i++ ) {
      rc = file.ReadChar( 4, cvi );
      vi = (unsigned int*)m_F[i].vi;
      vi[0] = cvi[0];
      vi[1] = cvi[1];
      vi[2] = cvi[2];
      vi[3] = cvi[3];
    }
    break;
  case 2:
    for ( i = 0; i < fcount && rc ; i++ ) {
      rc = file.ReadShort( 4, svi );
      vi = (unsigned int*)m_F[i].vi;
      vi[0] = svi[0];
      vi[1] = svi[1];
      vi[2] = svi[2];
      vi[3] = svi[3];
    }
    break;
  case 4:
    for ( i = 0; i < fcount && rc ; i++ ) {
      rc = file.ReadInt( 4, m_F[i].vi );
    }
    break;
  }
  m_F.SetCount(i);

  return rc;
}


BOOL ON_Mesh::Write_1( ON_BinaryArchive& file ) const
{
  // ver 1.0 uncompressed format

  BOOL rc = file.WriteArray( m_V );
  if (rc) rc = file.WriteArray( m_N );
  if (rc) rc = file.WriteArray( m_T );
  if (rc) rc = file.WriteArray( m_K );
  if (rc) rc = file.WriteArray( m_C );

  return rc;
}

BOOL ON_Mesh::Read_1( ON_BinaryArchive& file )
{
  // common to all 1.x formats (uncompressed)

  BOOL rc = file.ReadArray( m_V );
  if (rc) rc = file.ReadArray( m_N );
  if (rc) rc = file.ReadArray( m_T );
  if (rc) rc = file.ReadArray( m_K );
  if (rc) rc = file.ReadArray( m_C );

  return rc;
}

BOOL ON_Mesh::Write_2( int vcount, ON_BinaryArchive& file ) const
{
  // ver 2.0 uncompressed format
  const ON::endian e = file.Endian();

  BOOL rc = TRUE;

  if ( vcount > 0 ) {
    if ( e == ON::big_endian ) {
      file.ToggleByteOrder( m_V.Count()*3, 4, m_V.Array(), (void*)m_V.Array() );
      file.ToggleByteOrder( m_N.Count()*3, 4, m_N.Array(), (void*)m_N.Array() );
      file.ToggleByteOrder( m_T.Count()*2, 4, m_T.Array(), (void*)m_T.Array() );
      file.ToggleByteOrder( m_K.Count()*2, 8, m_K.Array(), (void*)m_K.Array() );
      file.ToggleByteOrder( m_C.Count()*3, 4, m_C.Array(), (void*)m_C.Array() );
    }
    if (rc) rc = file.WriteCompressedBuffer( m_V.Count()*sizeof(ON_3fPoint),m_V.Array() );
    if (rc) rc = file.WriteCompressedBuffer( m_N.Count()*sizeof(ON_3fVector),m_N.Array() );
    if (rc) rc = file.WriteCompressedBuffer( m_T.Count()*sizeof(ON_2fPoint),m_T.Array() );
    if (rc) rc = file.WriteCompressedBuffer( m_K.Count()*sizeof(ON_SurfaceCurvature),m_K.Array() );
    if (rc) rc = file.WriteCompressedBuffer( m_C.Count()*sizeof(ON_Color),m_C.Array() );
    if ( e == ON::big_endian ) {
      file.ToggleByteOrder( m_V.Count()*3, 4, m_V.Array(), (void*)m_V.Array() );
      file.ToggleByteOrder( m_N.Count()*3, 4, m_N.Array(), (void*)m_N.Array() );
      file.ToggleByteOrder( m_T.Count()*2, 4, m_T.Array(), (void*)m_T.Array() );
      file.ToggleByteOrder( m_K.Count()*2, 8, m_K.Array(), (void*)m_K.Array() );
      file.ToggleByteOrder( m_C.Count()*3, 4, m_C.Array(), (void*)m_C.Array() );
    }
  }

  return rc;
}

BOOL ON_Mesh::Read_2( int vcount, ON_BinaryArchive& file )
{
  // common to all 2.x formats (compressed)
  const ON::endian e = file.Endian();

  BOOL rc = TRUE;


  if ( vcount > 0 ) {
    int sz = 0;
    BOOL bFailedCRC;

    if (rc) rc = file.ReadCompressedBufferSize( &sz );
    if (rc && sz) m_V.SetCapacity(vcount);
    if (rc) rc = file.ReadCompressedBuffer( sz,m_V.Array(),&bFailedCRC);
    if (rc) m_V.SetCount(vcount);

    if (rc) rc = file.ReadCompressedBufferSize( &sz );
    if (rc && sz) {
      m_N.SetCapacity(vcount);
      if (rc) rc = file.ReadCompressedBuffer( sz,m_N.Array(),&bFailedCRC );
      if (rc) m_N.SetCount(vcount);
    }
    
    if (rc) rc = file.ReadCompressedBufferSize( &sz );
    if (rc && sz) {
      m_T.SetCapacity(vcount);
      if (rc) rc = file.ReadCompressedBuffer( sz,m_T.Array(),&bFailedCRC );
      if (rc) m_T.SetCount(vcount);
    }
    
    if (rc) rc = file.ReadCompressedBufferSize( &sz );
    if (rc && sz) {
      m_K.SetCapacity(vcount);
      if (rc) rc = file.ReadCompressedBuffer( sz,m_K.Array(),&bFailedCRC );
      if (rc) m_K.SetCount(vcount);
    }
    
    if (rc) rc = file.ReadCompressedBufferSize( &sz );
    if (rc && sz) {
      m_C.SetCapacity(vcount);
      if (rc) rc = file.ReadCompressedBuffer( sz,m_C.Array(),&bFailedCRC );
      if (rc) m_C.SetCount(vcount);
    }
    
    if ( e == ON::big_endian ) {
      file.ToggleByteOrder( m_V.Count()*3, 4, m_V.Array(), (void*)m_V.Array() );
      file.ToggleByteOrder( m_N.Count()*3, 4, m_N.Array(), (void*)m_N.Array() );
      file.ToggleByteOrder( m_T.Count()*2, 4, m_T.Array(), (void*)m_T.Array() );
      file.ToggleByteOrder( m_K.Count()*2, 8, m_K.Array(), (void*)m_K.Array() );
      file.ToggleByteOrder( m_C.Count()*3, 4, m_C.Array(), (void*)m_C.Array() );
    }
  }


  return rc;
}


BOOL ON_Mesh::Write( ON_BinaryArchive& file ) const
{
  int i;
  //const int major_version = 1; // uncompressed
  //const int major_version = 2; // beta format (never used)
  const int major_version = 3; // compressed
  BOOL rc = file.Write3dmChunkVersion(major_version,2);

  const int vcount = VertexCount();
  const int fcount = FaceCount();

  if (rc) rc = file.WriteInt( vcount );
  if (rc) rc = file.WriteInt( fcount );
  if (rc) rc = file.WriteInterval( m_tex_domain[0] );
  if (rc) rc = file.WriteInterval( m_tex_domain[1] );
  if (rc) rc = file.WriteInterval( m_srf_domain[0] );
  if (rc) rc = file.WriteInterval( m_srf_domain[1] );
  if (rc) rc = file.WriteDouble( 2, m_texture_scale );
  if (rc) rc = file.WriteFloat( 6, &m_vbox[0][0] );
  if (rc) rc = file.WriteFloat( 6, &m_nbox[0][0] );
  if (rc) rc = file.WriteFloat( 4, &m_tbox[0][0] );

  if (rc) rc = file.WriteInt( m_closed );

  unsigned char b = m_mesh_parameters ? 1 : 0;
  if (rc) rc = file.WriteChar(b);
  if (rc && b) {
    if (rc) rc = file.BeginWrite3dmChunk( TCODE_ANONYMOUS_CHUNK, 0 );
    if (rc) {
      rc = m_mesh_parameters->Write(file);
      if ( !file.EndWrite3dmChunk() )
        rc = FALSE;
    }
  }

  for ( i = 0; rc && i < 4; i++ ) {
    b = m_kstat[i] ? 1 : 0;
    rc = file.WriteChar(b);
    if (b) {
      rc = file.BeginWrite3dmChunk( TCODE_ANONYMOUS_CHUNK, 0 );
      if (rc) {
        rc = m_kstat[i]->Write(file);
        if ( !file.EndWrite3dmChunk() )
          rc = FALSE;
      }
    }
  }

  if (rc) rc = WriteFaceArray( vcount, fcount, file );

  if (rc) {
    if ( major_version == 1 )
      rc = Write_1(file);
    else if ( major_version == 3 )
      rc = Write_2(vcount,file);
    else
      rc = FALSE;
  }

  // added for minor version 1.2 and 3.2
  if (rc) rc = file.WriteInt( m_srf_tex_rotate );


  return rc;
}


BOOL ON_Mesh::Read( ON_BinaryArchive& file )
{
  Destroy();

  int major_version = 0;
  int minor_version = 0;
  int i;
  BOOL rc = file.Read3dmChunkVersion(&major_version,&minor_version);
  if (rc && (major_version == 1 || major_version == 3) ) {
    int vcount = 0;
    int fcount = 0;

    if (rc) rc = file.ReadInt( &vcount );
    if (rc) rc = file.ReadInt( &fcount );
    if (rc) rc = file.ReadInterval( m_tex_domain[0] );
    if (rc) rc = file.ReadInterval( m_tex_domain[1] );
    if (rc) rc = file.ReadInterval( m_srf_domain[0] );
    if (rc) rc = file.ReadInterval( m_srf_domain[1] );
    if (rc) rc = file.ReadDouble( 2, m_texture_scale );
    if (rc) rc = file.ReadFloat( 6, &m_vbox[0][0] );
    if (rc) rc = file.ReadFloat( 6, &m_nbox[0][0] );
    if (rc) rc = file.ReadFloat( 4, &m_tbox[0][0] );

    if (rc) rc = file.ReadInt( &m_closed );

    unsigned char b = 0;
    unsigned int tcode;
    int value;
    if (rc) rc = file.ReadChar(&b);
    if (rc && b) {
      rc = file.BeginRead3dmChunk(&tcode,&value);
      if (rc) {
        m_mesh_parameters = new ON_MeshParameters();
        rc = m_mesh_parameters->Read( file );
        if (!file.EndRead3dmChunk())
          rc = FALSE;
      }
    }

    for ( i = 0; rc && i < 4; i++ ) {
      rc = file.ReadChar(&b);
      if (rc && b) {
        rc = file.BeginRead3dmChunk( &tcode, &value );
        if (rc) {
          m_kstat[i] = new ON_MeshCurvatureStats();
          rc = m_kstat[i]->Read(file);
          if ( !file.EndRead3dmChunk() )
            rc = FALSE;
        }
      }
    }

    if (rc) rc = ReadFaceArray( vcount, fcount, file );

    if (rc) {
      if ( major_version==1) {
        rc = Read_1(file);
      }
      else if ( major_version == 3 ) {
        rc = Read_2(vcount,file);
      }
      else
        rc = FALSE;
    }

    if ( minor_version >= 2 ) {
      if (rc) rc = file.ReadInt( &m_srf_tex_rotate );
    }
  }
  return rc;
}

ON::object_type ON_Mesh::ObjectType() const
{
  return ON::mesh_object;
}

int ON_Mesh::Dimension() const
{
  return 3;
}

BOOL ON_Mesh::GetBBox( // returns TRUE if successful
       double* boxmin, // minimum
       double* boxmax, // maximum
       BOOL bGrowBox
       ) const
{
  BOOL rc = FALSE;
  const int facet_count  = FaceCount();
  const int vertex_count = VertexCount();
  if ( facet_count >= 1 && vertex_count >= 3 ) {
    ON_BoundingBox vbox;
    if ( m_vbox[0][0] > m_vbox[1][0] ) {
      rc = ON_GetPointListBoundingBox( 3, FALSE, 
              vertex_count, 3, &m_V[0].x,
              (float*)m_vbox[0], (float*)m_vbox[1], FALSE );
    }
    else
      rc = TRUE;
    if ( rc ) {
      vbox[0][0] = m_vbox[0][0];
      vbox[0][1] = m_vbox[0][1];
      vbox[0][2] = m_vbox[0][2];
      vbox[1][0] = m_vbox[1][0];
      vbox[1][1] = m_vbox[1][1];
      vbox[1][2] = m_vbox[1][2];
      if ( bGrowBox )
        vbox.Union( ON_BoundingBox(boxmin,boxmax) );

      boxmin[0] = vbox.m_min.x;
      boxmin[1] = vbox.m_min.y;
      boxmin[2] = vbox.m_min.z;

      boxmax[0] = vbox.m_max.x;
      boxmax[1] = vbox.m_max.y;
      boxmax[2] = vbox.m_max.z;
    }
  }
  return rc;
}

BOOL ON_Mesh::Transform( 
       const ON_Xform& xform
       )
{
  TransformUserData(xform);
  double d = xform.Determinant();
  const int vertex_count = VertexCount();
  BOOL rc = ON_TransformPointList( 3, FALSE, vertex_count, 3, &m_V[0][0], xform );
  if ( rc && HasVertexNormals() ) {
    rc = ON_TransformVectorList( 3, vertex_count, 3, &m_N[0][0], xform );
    if ( d < 0.0 )
      FlipVertexNormals();
    if ( rc )
      UnitizeVertexNormals();
  }
  if ( rc && HasFaceNormals() ) {
    ComputeFaceNormals();
  }
  if ( rc && HasPrincipalCurvatures() ) {
    if ( fabs(fabs(d) - 1.0) > ON_SQRT_EPSILON ) {
      ON_ERROR("ON_Mesh::Transform() cannot apply this transform to curvatures.\n");
      rc = FALSE;
    }
  }
  InvalidateVertexBoundingBox();
  InvalidateVertexNormalBoundingBox();
  if ( fabs(d) <= ON_ZERO_TOLERANCE )
    DestroyTopology(); // transform may not be one-to-one on vertices
  return rc;
}

BOOL ON_Mesh::SwapCoordinates(
      int i, int j        // indices of coords to swap
      )
{
  const int vertex_count = VertexCount();
  BOOL rc = ON_SwapPointListCoordinates( vertex_count, 3, &m_V[0][0], i, j );
  if ( rc && HasVertexNormals() ) {
    rc = ON_SwapPointListCoordinates( vertex_count, 3, &m_N[0][0], i, j );
  }
  if ( rc && i != j ) {
    float x;
    if ( m_vbox[0][0] <= m_vbox[1][0] ) {
      x = m_vbox[0][i]; m_vbox[0][i] = m_vbox[0][j]; m_vbox[0][j] = x;
      x = m_vbox[1][i]; m_vbox[1][i] = m_vbox[1][j]; m_vbox[1][j] = x;
    }
    if ( m_nbox[0][0] <= m_nbox[1][0] ) {
      x = m_nbox[0][i]; m_nbox[0][i] = m_nbox[0][j]; m_nbox[0][j] = x;
      x = m_nbox[1][i]; m_nbox[1][i] = m_nbox[1][j]; m_nbox[1][j] = x;
    }
  }
  return rc;
}

void ON_Mesh::SetClosed(BOOL b)
{
  m_closed = (b)?1:0;
}

int ON_Mesh::IsClosed() const
{
  if ( m_closed < 0 ) {
    // lazy evaluation
    int closed = 0;
    const ON_MeshTopology& top = Topology();
    const int ecnt = top.m_tope.Count();
    int ei, efcnt;
    if ( ecnt >= 6 ) {
      closed = 1;
      for ( ei = 0; ei < ecnt; ei++ ) {
        efcnt = top.m_tope[ei].m_topf_count;
        if ( efcnt == 1 ) {
          closed = 0;
          break;
        }
        if ( efcnt > 2 ) {
          closed = 2;
        }
      }
    }
    ((ON_Mesh*)this)->m_closed = closed;
  }
  return (m_closed>0) ? TRUE : FALSE;
}

BOOL ON_Mesh::SetVertex(
       int vertex_index,
       const ON_3dPoint& vertex_location
       )
{
  ON_3fPoint p(vertex_location.x,vertex_location.y,vertex_location.z);
  return SetVertex( vertex_index, p );
}

BOOL ON_Mesh::SetVertex(
       int vertex_index,
       const ON_3fPoint& vertex_location
       )
{
  BOOL rc = FALSE;
  int vertex_count = m_V.Count();
  if ( vertex_index >= 0 ) {
    if ( vertex_index < vertex_count ) {
      m_V[vertex_index] = vertex_location;
      rc = TRUE;
    }
    else if ( vertex_index == vertex_count ) {
      m_V.Append(vertex_location);
      rc = TRUE;
    }
  }
  return rc;
}

BOOL ON_Mesh::SetVertexNormal(
       int vertex_index,
       const ON_3dVector& normal
       )
{
  BOOL rc = FALSE;
  // use double precision for unitizing normal
  ON_3dVector unit_vector = normal;
  const BOOL bUnitVector = unit_vector.Unitize();
  ON_3fVector v(unit_vector.x,unit_vector.y,unit_vector.z);
  int normal_count = m_N.Count();
  if ( vertex_index >= 0 ) {
    if ( vertex_index < normal_count ) {
      m_N[vertex_index] = v;
      rc = bUnitVector;
    }
    else if ( vertex_index == normal_count ) {
      m_N.Append(v);
      rc = bUnitVector;
    }
  }
  return rc;
}

BOOL ON_Mesh::SetVertexNormal(
       int vertex_index,
       const ON_3fVector& normal
       )
{
  ON_3dVector v(normal.x,normal.y,normal.z);
  return SetVertexNormal(vertex_index,v);
}

BOOL ON_Mesh::SetTextureDomain( 
      const ON_Interval& d0, // second texture parameter domain
      const ON_Interval& d1  // second texture parameter domain
      )
{
  m_tex_domain[0] = d0;
  m_tex_domain[1] = d1;
  return (d0.IsIncreasing() && d1.IsIncreasing());
}

BOOL ON_Mesh::SetTextureCoord(
       int vertex_index,
       double s, double t    // texture coordinates
       )
{
  ON_2fPoint tc(s,t);
  BOOL rc = FALSE;
  int vertex_count = m_T.Count();
  if ( vertex_index >= 0 ) {
    if ( vertex_index < vertex_count ) {
      m_T[vertex_index] = tc;
      rc = TRUE;
    }
    else if ( vertex_index == vertex_count ) {
      m_T.Append(tc);
      rc = TRUE;
    }
  }
  return rc;
}


BOOL ON_Mesh::SetTriangle(
       int face_index,
       int a,int b ,int c // vertex indices
       )
{
  return SetQuad( face_index, a,b,c,c );
}

BOOL ON_Mesh::SetQuad(
       int face_index,
       int a, int b, int c, int d // vertex indices
       )
{
  BOOL rc = FALSE;
  int face_count = m_F.Count();
  if ( face_index >= 0 ) {
    ON_MeshFace f;
    f.vi[0] = a;
    f.vi[1] = b;
    f.vi[2] = c;
    f.vi[3] = d;
    if ( face_index < face_count ) {
      m_F[face_index] = f;
      rc = TRUE;
    }
    else if ( face_index == face_count ) {
      m_F.Append(f);
      rc = TRUE;
    }
    if ( rc )
      rc = f.IsValid(m_V.Count());
  }
  return rc;
}



int ON_Mesh::FaceCount() const
{
  return m_F.Count();
}

int ON_Mesh::QuadCount()
{
  // number of faces that are quads
  if ( m_quad_count + m_triangle_count + m_invalid_count != FaceCount() ) {
    CountQuads();
  }
  return m_quad_count;
}

int ON_Mesh::TriangleCount()
{
  // number of faces that are triangles
  if ( m_quad_count + m_triangle_count + m_invalid_count != FaceCount() ) {
    CountQuads();
  }
  return m_triangle_count;
}

int ON_Mesh::VertexCount() const
{
  return m_V.Count();
}

BOOL ON_Mesh::HasVertexNormals() const
{
  const int vertex_count = VertexCount();
  return ( vertex_count > 0 && m_N.Count() == vertex_count ) ? TRUE : FALSE;
}

BOOL ON_Mesh::HasFaceNormals() const
{
  const int face_count = FaceCount();
  return ( face_count > 0 && m_FN.Count() == face_count ) ? TRUE : FALSE;
}

BOOL ON_Mesh::HasTextureCoordinates() const
{
  const int vertex_count = VertexCount();
  return ( vertex_count > 0 && m_T.Count() == vertex_count ) ? TRUE : FALSE;
}

BOOL ON_Mesh::HasPrincipalCurvatures() const
{
  const int vertex_count = VertexCount();
  return ( vertex_count > 0 && m_K.Count() == vertex_count ) ? TRUE : FALSE;
}

BOOL ON_Mesh::HasVertexColors() const
{
  const int vertex_count = VertexCount();
  return ( vertex_count > 0 && m_C.Count() == vertex_count ) ? TRUE : FALSE;
}

void ON_Mesh::InvalidateBoundingBoxes()
{
  InvalidateVertexBoundingBox();
  InvalidateVertexNormalBoundingBox();
  InvalidateTextureCoordinateBoundingBox();
  InvalidateCurvatureStats();
}

void ON_Mesh::InvalidateVertexBoundingBox()
{
  m_vbox[0][0] = m_vbox[0][1] = m_vbox[0][2] =  1.0;
  m_vbox[1][0] = m_vbox[1][1] = m_vbox[1][2] = -1.0;
}

void ON_Mesh::InvalidateVertexNormalBoundingBox()
{
  m_nbox[0][0] = m_nbox[0][1] = m_nbox[0][2] =  1.0;
  m_nbox[1][0] = m_nbox[1][1] = m_nbox[1][2] = -1.0;
}

void ON_Mesh::InvalidateTextureCoordinateBoundingBox()
{
  m_tbox[0][0] = m_tbox[0][1] =  1.0;
  m_tbox[1][0] = m_tbox[1][1] = -1.0;
}

void ON_Mesh::InvalidateCurvatureStats()
{
  int i;
  for ( i = 0; i < 4; i++ ) {
    if ( m_kstat[i] ) {
      delete m_kstat[i];
      m_kstat[i] = 0;
    }
  }
}

BOOL ON_Mesh::UnitizeVertexNormals()
{
  BOOL rc = HasVertexNormals();
  if ( rc ) {
    const int vertex_count = VertexCount();
    float* n = &m_N[0][0];
    int i;
    ON_3dVector N;
    for ( i = 0; i < vertex_count; i++ ) {
      N.x = n[0];
      N.y = n[1];
      N.z = n[2];
      if ( !N.Unitize() )
        rc = FALSE;
      *n++ = (float)N.x;
      *n++ = (float)N.y;
      *n++ = (float)N.z;
    }
  }
  return rc;
}

BOOL ON_Mesh::UnitizeFaceNormals()
{
  BOOL rc = HasFaceNormals();
  if ( rc ) {
    const int face_count = FaceCount();
    float* n = &m_FN[0][0];
    int i;
    ON_3dVector N;
    for ( i = 0; i < face_count; i++ ) {
      N.x = n[0];
      N.y = n[1];
      N.z = n[2];
      if ( !N.Unitize() )
        rc = FALSE;
      *n++ = (float)N.x;
      *n++ = (float)N.y;
      *n++ = (float)N.z;
    }
  }
  return rc;
}


BOOL ON_Mesh::GetCurvatureStats( // returns TRUE if successful
       ON::curvature_style kappa_style,
       ON_MeshCurvatureStats& stats
       ) const
{
  BOOL rc = FALSE;
  stats.Destroy();
  int ksi;
  switch ( kappa_style ) {
    case ON::gaussian_curvature:
      ksi = 0;
      break;
    case ON::mean_curvature:
      ksi = 1;
      break;
    case ON::min_curvature:
      ksi = 2;
      break;
    case ON::max_curvature:
      ksi = 3;
      break;
    //case ON::section_curvature_x:
    //  ksi = 4;
    //  break;
    //case ON::section_curvature_y:
    //  ksi = 5;
    //  break;
    //case ON::section_curvature_z:
    //  ksi = 6;
    //  break;
    default:
      ksi = -1;
      break;
  }
  if ( ksi >= 0 && ksi <= 3 && HasPrincipalCurvatures() ) {
    ON_Mesh* p = (ON_Mesh*)this; // const lie 
    if ( !m_kstat[ksi] ) {
      p->m_kstat[ksi] = new ON_MeshCurvatureStats();
      p->m_kstat[ksi]->Set( kappa_style, m_K.Count(), m_K.Array(), m_N.Array() );
    }
    if ( p->m_kstat[ksi] ) {
      stats = *p->m_kstat[ksi];
      rc = TRUE;
    }
  }
  return rc;
}


const ON_MeshTopology& ON_Mesh::Topology() const
{
  if ( !m_top.m_bIsValid ) {
    // const lie since volitile members are not universally supported
    ON_MeshTopology* top = (ON_MeshTopology*)&m_top;
    top->m_mesh = this;
    top->Create();
  }
  return m_top;
}

void ON_Mesh::DestroyTopology()
{
  m_top.Destroy();
}

BOOL
ON_MeshFace::IsTriangle() const 
{
  return vi[2]==vi[3];
}

BOOL
ON_MeshFace::IsQuad() const 
{
  return vi[2]!=vi[3];
}

void
ON_MeshFace::Flip()
{
  int i;
  if ( vi[2] == vi[3] ) {
    i = vi[1];
    vi[1] = vi[2];
    vi[2] = i;
    vi[3] = i;
  }
  else {
    i = vi[1];
    vi[1] = vi[3];
    vi[3] = i;
  }
}

/*
ON_MeshEdge::ON_MeshEdge() : m_fcount(0)
{
  m_fi = m_private_fi;
  m_private_fi[0] = 0;
  m_private_fi[0] = 0;
}

ON_MeshEdge::~ON_MeshEdge()
{
  if ( m_fi && m_fi != m_private_fi ) onfree((void*)m_fi);
}

ON_MeshEdge::ON_MeshEdge( const ON_MeshEdge& src ) : m_fcount(0), m_fi(0)
{
  *this = src;
}

int ON_MeshEdge::FaceIndexCount() const
{
  return m_fcount;
}

const int* ON_MeshEdge::FaceIndexArray() const
{
  return m_fi;
}

int& ON_MeshEdge::operator[](int i)
{
  return (i>=0&&i<m_fcount) ? m_fi[i] : m_private_fi[0];
}

int ON_MeshEdge::operator[](int i) const
{
  return (i>=0&&i<m_fcount) ? m_fi[i] : m_private_fi[0];
}
*/

void 
ON_Mesh::Flip()
{
  FlipFaceOrientation();
  FlipFaceNormals();
  FlipVertexNormals();
}

void 
ON_Mesh::FlipVertexNormals()
{
  int i;
  const int vcount = VertexCount();
  if ( HasVertexNormals() ) {
    for ( i = 0; i < vcount; i++ ) {
      m_N[i].Reverse();
    }
  }
}

void 
ON_Mesh::FlipFaceNormals()
{
  int i;
  const int fcount = FaceCount();
  if ( HasFaceNormals() ) {
    for( i = 0; i < fcount; i++ ) {
      m_FN[i].Reverse();
    }
  }
}

void 
ON_Mesh::FlipFaceOrientation()
{
  int i;
  const int fcount = FaceCount();
  for( i = 0; i < fcount; i++ ) {
    m_F[i].Flip();
  }
  if ( fcount > 0 )
    DestroyTopology(); // flipping changes order of face corners
}

BOOL
ON_Mesh::ComputeFaceNormal( int fi )
{
  BOOL rc = FALSE;
  const int fcount = FaceCount();
  if ( fi >= 0 && fi < fcount && HasFaceNormals() ) {
    ON_3fVector a, b, n;
    const int* vi = m_F[fi].vi;
    a = m_V[vi[2]] - m_V[vi[0]];
    b = m_V[vi[3]] - m_V[vi[1]];
    n = ON_CrossProduct( a, b ); // works for triangles, quads, and nonplanar quads
    rc = n.Unitize();
    m_FN[fi] = n;
  }
  return rc;
}

BOOL
ON_Mesh::ComputeFaceNormals()
{
  BOOL rc = FALSE;
  const int fcount = FaceCount();
  if ( fcount > 0 ) {
    ON_3fVector a, b, n;
    int fi;
    const int* vi;
    if ( m_FN.Capacity() < fcount )
      m_FN.SetCapacity(fcount);
    m_FN.SetCount(0);
    rc = TRUE;
    for ( fi = 0; fi < fcount; fi++ ) {
      vi = m_F[fi].vi;
      a = m_V[vi[2]] - m_V[vi[0]];
      b = m_V[vi[3]] - m_V[vi[1]];
      n = ON_CrossProduct( a, b ); // works for triangles, quads, and nonplanar quads
      n.Unitize();
      m_FN.Append(n);
    }
  }
  else {
    m_FN.Destroy();
  }
  return rc;
}


static int compareRadial3fPoint( const ON_3fPoint* a, const ON_3fPoint* b )
{
  double ar = a->x+a->y+a->z;
  double br = b->x+b->y+b->z;
  if ( ar < br )
    return -1;
  if ( ar > br )
    return 1;
  return 0;
}

BOOL ON_Mesh::CombineCoincidentVertices( 
        const ON_3fVector tolerance,
        double cos_normal_angle // = -1.0  // cosine(break angle) -1.0 will merge all coincident vertices
        )
{

  BOOL rc = FALSE;
  const int vcount = VertexCount();
  if ( vcount > 0 && rc ) {
    ON_Workspace ws;
    int* index = ws.GetIntMemory(vcount);
    rc = m_V.Sort( ON::quick_sort, index, compareRadial3fPoint );
    int i, j;
    ON_3fPoint p0, p1, pmin, pmax;
    for ( i = 0; i < vcount; i++ ) {
      p0 = m_V[i];
      pmin = p0 - tolerance;
      pmax = p0 + tolerance;
      for ( j = i+1; j < vcount; j++ ) {
        p1 = m_V[j];
        // TODO        
      }
    }
  }
  return FALSE;
}

void ON_Mesh::Append( const ON_Mesh& m )
{ // appends a copy of mesh to this and updates
  const int vcount0 = VertexCount();
  const int vcount1 = m.VertexCount();
  //const int vcount = vcount0 + vcount1;
  const int fcount0 = FaceCount();
  const int fcount1 = m.FaceCount();
  const int fcount = fcount0+fcount1;
  int* vi;
  int j;

  m_top.Destroy();

  // The calls to Has*() must happen before the m_V[] and m_F[] arrays get enlarged
  const BOOL bHasVertexNormals = HasVertexNormals() && m.HasVertexNormals();
  const BOOL bHasFaceNormals   = HasFaceNormals() && m.HasFaceNormals();
  const BOOL bHasTextureCoordinates = HasTextureCoordinates() && m.HasTextureCoordinates();
  const BOOL bHasPrincipalCurvatures = HasPrincipalCurvatures() && m.HasPrincipalCurvatures();
  const BOOL bHasVertexColors = HasVertexColors() && m.HasVertexColors();

  m_V.Append( vcount1, m.m_V.Array() );
  m_F.Append( fcount1, m.m_F.Array() );
  for ( j = fcount0; j < fcount; j++ ) {
    vi = m_F[j].vi;
    vi[0] += vcount0;
    vi[1] += vcount0;
    vi[2] += vcount0;
    vi[3] += vcount0;
  }
  if ( bHasVertexNormals ) {
    m_N.Append(vcount1, m.m_N.Array());
  }
  else {
    m_N.Destroy();
  }
  if ( bHasFaceNormals ) {
    m_FN.Append( fcount1, m.m_FN.Array() );
  }
  else {
    m_FN.Destroy();
  }
  if ( bHasTextureCoordinates ) {
    m_T.Append( vcount1, m.m_T.Array() );
  }
  else {
    m_T.Destroy();
  }
  if ( bHasPrincipalCurvatures ) {
    m_K.Append( vcount1, m.m_K.Array() );
  }
  else {
    m_K.Destroy();
  }
  if ( bHasVertexColors ) {
    m_C.Append( vcount1, m.m_C.Array() );
  }
  else {
    m_C.Destroy();
  }


  if ( m_mesh_parameters && m.m_mesh_parameters ) {
    if ( *m_mesh_parameters != *m.m_mesh_parameters ) {
      delete m_mesh_parameters;
      m_mesh_parameters = 0;
    }
  }
  else {
    delete m_mesh_parameters;
    m_mesh_parameters = 0;
  }

  for ( j = 0; j < 4; j++ ) {
    if ( m_kstat[j] ) {
      // will be recomputed if required
      delete m_kstat[j];
      m_kstat[j] = 0;
    }
  }

}

ON_MeshParameters::ON_MeshParameters()
{
  Default();
}

ON_MeshParameters::~ON_MeshParameters()
{
}

ON_MeshParameters::ON_MeshParameters( const ON_MeshParameters& src )
{
  *this = src;
}

ON_MeshParameters& ON_MeshParameters::operator=(const ON_MeshParameters& src )
{
  if ( this != &src ) {
    m_bComputeCurvature = src.m_bComputeCurvature;
    m_bSimplePlanes = src.m_bSimplePlanes;
    m_bRefine = src.m_bRefine;
    m_bJaggedSeams = src.m_bJaggedSeams;
    m_bWeld = src.m_bWeld;
    m_texture_range = src.m_texture_range;
    m_tolerance = src.m_tolerance;
    m_min_edge_length = src.m_min_edge_length;
    m_max_edge_length = src.m_max_edge_length;
    m_grid_aspect_ratio = src.m_grid_aspect_ratio;
    m_grid_min_count = src.m_grid_min_count;
    m_grid_max_count = src.m_grid_max_count;
    m_grid_angle = src.m_grid_angle;
    m_grid_amplification = src.m_grid_amplification;
    m_refine_angle = src.m_refine_angle;
    m_combine_angle = src.m_combine_angle;
    m_face_type = src.m_face_type;
  }
  return *this;
}

BOOL ON_MeshParameters::operator==(const ON_MeshParameters& m2) const
{
  return (Compare(m2)==0) ? TRUE : FALSE;
}

BOOL ON_MeshParameters::operator!=(const ON_MeshParameters& m2) const
{
  return (Compare(m2)) ? TRUE : FALSE;
}

BOOL ON_MeshParameters::operator==(const ON_Mesh& mesh) const
{
  const ON_MeshParameters* mp1 = mesh.MeshParameters();
  return mp1 ? (*this == *mp1) : FALSE;
}

BOOL ON_MeshParameters::operator!=(const ON_Mesh& mesh) const
{
  const ON_MeshParameters* mp1 = mesh.MeshParameters();
  return mp1 ? (*this != *mp1) : TRUE;
}

void ON_MeshParameters::Default()
{
  m_bComputeCurvature = FALSE;
  m_bSimplePlanes = FALSE;
  m_bRefine = TRUE;
  m_bJaggedSeams = FALSE;
  m_bWeld = FALSE;
  m_texture_range = 2; // packed normalized textures
  m_tolerance = 0.0;
  m_min_edge_length = 0.0001;
  m_max_edge_length = 0.0;
  m_grid_aspect_ratio = 6.0;
  m_grid_min_count = 0; // 16;
  m_grid_max_count = 0;
  m_grid_angle = 20.0*ON_PI/180.0;
  m_grid_amplification = 1.0;
  m_refine_angle = 20.0*ON_PI/180.0;
  m_combine_angle = 0.5*ON_PI/180.0;
  m_face_type = 0;
}

void ON_MeshParameters::Set( double density, double min_edge_length )
{
  double t0, t1;

  Default();

  m_bComputeCurvature = FALSE;
  m_bWeld = FALSE;
  m_bRefine = TRUE;
  m_bSimplePlanes = FALSE;
  m_bJaggedSeams  = FALSE;
  m_texture_range = 0;
  m_tolerance = 0.0;
  m_min_edge_length = min_edge_length;
  m_max_edge_length = 0.0;
  m_grid_aspect_ratio = 0.0;
  m_grid_min_count = 16;
  m_grid_max_count = 0;
  m_grid_angle = 20.0*ON_PI/180.0;
  m_grid_amplification = 1.0;
  m_refine_angle = 20.0*ON_PI/180.0;
  m_combine_angle = 5.0*ON_PI/180.0;
 
  if ( density <= 0.0) {
    m_bSimplePlanes = TRUE;
    m_bRefine       = FALSE;

    m_grid_min_count     = 0;
    m_grid_aspect_ratio  = 0.0;
    m_grid_angle         = 0.0;
    m_grid_amplification = 1.0;

    m_refine_angle       = m_grid_angle;
  }
  else if ( density < 0.5)
  {
    t1 = 2.0*density;
    t0 = 1.0-t1;
    
    m_bRefine = density < 0.10 ? FALSE : TRUE;

    m_grid_min_count = 0;
    m_grid_aspect_ratio  = 6.0;
    m_grid_angle         = 20.0*ON_PI/180.0;
    m_grid_amplification = (t1 > 0.0 ) ? t1 : 1.0;

    m_refine_angle = (t0*65.0 + t1*20.0)*ON_PI/180.0;
  }
  else if ( density == 0.5 ) {
    m_bRefine       = TRUE;

    m_grid_min_count     = 0;
    m_grid_aspect_ratio  = 6.0;
    m_grid_angle         = 20.0*ON_PI/180.0;
    m_grid_amplification = 1.0;

    m_refine_angle        = m_grid_angle;
  }
  else {
    t1 = 2.0*(density - 0.5);
    if ( t1 > 1.0 )
      t1 = 1.0;
    t0 = 1.0-t1;
    
    m_bRefine = TRUE;

    m_grid_min_count     = 0;
    m_grid_aspect_ratio  = t0*6.0 + t1;
    m_grid_angle         = (t0*20.0 + t1*5.0)*ON_PI/180.0;
    m_grid_amplification = 1.0 + t1;

    m_refine_angle       = m_grid_angle;
  }
}


int ON_MeshParameters::Compare( const ON_MeshParameters& src ) const
{
  if ( !m_bSimplePlanes && src.m_bSimplePlanes )
    return -1;
  if ( m_bSimplePlanes && !src.m_bSimplePlanes )
    return 1;
  if ( !m_bRefine && src.m_bRefine )
    return -1;
  if ( m_bRefine && !src.m_bRefine )
    return 1;
  if ( m_bJaggedSeams && !src.m_bJaggedSeams )
    return -1;
  if ( !m_bJaggedSeams && src.m_bJaggedSeams )
    return 1;
  if ( m_texture_range < src.m_texture_range )
    return -1;
  if ( m_texture_range > src.m_texture_range )
    return 1;
  if ( m_bWeld && !src.m_bWeld )
    return -1;
  if ( !m_bWeld && src.m_bWeld )
    return 1;

  if ( m_tolerance <= 0.0 && src.m_tolerance > 0.0 )
    return -1;
  if ( m_tolerance > 0.0 && src.m_tolerance <= 0.0 )
    return 1;
  if ( 0.0 < m_tolerance && m_tolerance < src.m_tolerance )
    return 1;
  if ( 0.0 < src.m_tolerance && src.m_tolerance < m_tolerance )
    return -1;

  if ( m_min_edge_length <= 0.0 && src.m_min_edge_length > 0.0 )
    return 1;
  if ( m_min_edge_length > 0.0 && src.m_min_edge_length <= 0.0 )
    return -1;
  if ( 0.0 < m_min_edge_length && m_min_edge_length < src.m_min_edge_length )
    return 1;
  if ( 0.0 < src.m_min_edge_length && src.m_min_edge_length < m_min_edge_length )
    return 1;

  if ( m_max_edge_length <= 0.0 && src.m_max_edge_length > 0.0 )
    return -1;
  if ( m_max_edge_length > 0.0 && src.m_max_edge_length <= 0.0 )
    return 1;
  if ( 0.0 < m_max_edge_length && m_max_edge_length < src.m_max_edge_length )
    return 1;
  if ( 0.0 < src.m_max_edge_length && src.m_max_edge_length < m_max_edge_length )
    return -1;

  if ( m_grid_aspect_ratio <= 0.0 && src.m_grid_aspect_ratio > 0.0 )
    return -1;
  if ( m_grid_aspect_ratio > 0.0 && src.m_grid_aspect_ratio <= 0.0 )
    return 1;
  if ( 0.0 < m_grid_aspect_ratio && m_grid_aspect_ratio < src.m_grid_aspect_ratio )
    return 1;
  if ( 0.0 < src.m_grid_aspect_ratio && src.m_grid_aspect_ratio < m_grid_aspect_ratio )
    return -1;

  if ( m_grid_min_count <= 0 && src.m_grid_min_count > 0 )
    return -1;
  if ( m_grid_min_count > 0 && src.m_grid_min_count <= 0 )
    return 1;
  if ( 0 < m_grid_min_count && m_grid_min_count < src.m_grid_min_count )
    return 1;
  if ( 0 < src.m_grid_min_count && src.m_grid_min_count < m_grid_min_count )
    return -1;

  if ( m_grid_max_count <= 0 && src.m_grid_max_count > 0 )
    return 1;
  if ( m_grid_max_count > 0 && src.m_grid_max_count <= 0 )
    return -1;
  if ( 0 < m_grid_max_count && m_grid_max_count < src.m_grid_max_count )
    return -1;
  if ( 0 < src.m_grid_max_count && src.m_grid_max_count < m_grid_max_count )
    return 1;

  if ( m_grid_angle <= 0.0 && src.m_grid_angle > 0.0 )
    return -1;
  if ( m_grid_angle > 0.0 && src.m_grid_angle <= 0.0 )
    return 1;
  if ( 0.0 < m_grid_angle && m_grid_angle < src.m_grid_angle )
    return 1;
  if ( 0.0 < src.m_grid_angle && src.m_grid_angle < m_grid_angle )
    return -1;

  if ( m_refine_angle <= 0.0 && src.m_refine_angle > 0.0 )
    return -1;
  if ( m_refine_angle > 0.0 && src.m_refine_angle <= 0.0 )
    return 1;
  if ( 0.0 < m_refine_angle && m_refine_angle < src.m_refine_angle )
    return 1;
  if ( 0.0 < src.m_refine_angle && src.m_refine_angle < m_refine_angle )
    return -1;

  double g0 = ( m_grid_amplification > 0.0 ) ? m_grid_amplification : 1.0;
  double g1 = ( src.m_grid_amplification > 0.0 ) ? src.m_grid_amplification : 1.0;
  if ( g0 < g1 )
    return -1;
  if ( g0 > g1 )
    return 1;

  if ( m_face_type < src.m_face_type )
    return -1;
  if ( m_face_type > src.m_face_type )
    return 1;

  return 0;
}

BOOL ON_MeshParameters::Write( ON_BinaryArchive& file ) const
{
  BOOL rc = file.Write3dmChunkVersion(1,1);
  if (rc) {
    if (rc) rc = file.WriteInt(m_bComputeCurvature);
    if (rc) rc = file.WriteInt(m_bSimplePlanes);
    if (rc) rc = file.WriteInt(m_bRefine);
    if (rc) rc = file.WriteInt(m_bJaggedSeams);
    if (rc) rc = file.WriteInt(m_bWeld);
    if (rc) rc = file.WriteDouble(m_tolerance);
    if (rc) rc = file.WriteDouble(m_min_edge_length);
    if (rc) rc = file.WriteDouble(m_max_edge_length);
    if (rc) rc = file.WriteDouble(m_grid_aspect_ratio);
    if (rc) rc = file.WriteInt(m_grid_min_count);
    if (rc) rc = file.WriteInt(m_grid_max_count);
    if (rc) rc = file.WriteDouble(m_grid_angle);
    if (rc) rc = file.WriteDouble(m_grid_amplification);
    if (rc) rc = file.WriteDouble(m_refine_angle);
    if (rc) rc = file.WriteDouble(m_combine_angle);
    int mft = m_face_type;
    if ( mft < 0 || mft > 2 ) {
      ON_ERROR("ON_MeshParameters::Read() - m_face_type out of bounds.");
      mft = 0;
    }
    if (rc) rc = file.WriteInt(mft);

    // added for chunk version 1.1
    if (rc) rc = file.WriteInt( m_texture_range );
  }
  return rc;
}

BOOL ON_MeshParameters::Read( ON_BinaryArchive& file )
{
  Default();
  int major_version = 0;
  int minor_version = 0;
  BOOL rc = file.Read3dmChunkVersion(&major_version,&minor_version);
  if ( rc && major_version == 1 ) {
    if (rc) rc = file.ReadInt(&m_bComputeCurvature);
    if (rc) rc = file.ReadInt(&m_bSimplePlanes);
    if (rc) rc = file.ReadInt(&m_bRefine);
    if (rc) rc = file.ReadInt(&m_bJaggedSeams);
    if (rc) rc = file.ReadInt(&m_bWeld);
    if (rc) rc = file.ReadDouble(&m_tolerance);
    if (rc) rc = file.ReadDouble(&m_min_edge_length);
    if (rc) rc = file.ReadDouble(&m_max_edge_length);
    if (rc) rc = file.ReadDouble(&m_grid_aspect_ratio);
    if (rc) rc = file.ReadInt(&m_grid_min_count);
    if (rc) rc = file.ReadInt(&m_grid_max_count);
    if (rc) rc = file.ReadDouble(&m_grid_angle);
    if (rc) rc = file.ReadDouble(&m_grid_amplification);
    if (rc) rc = file.ReadDouble(&m_refine_angle);
    if (rc) rc = file.ReadDouble(&m_combine_angle);
    if (rc) rc = file.ReadInt(&m_face_type);
    if ( m_face_type < 0 || m_face_type > 2 ) {
      ON_ERROR("ON_MeshParameters::Read() - m_face_type out of bounds.");
      m_face_type = 0;
    }

    if ( minor_version >= 1 ) {
      if (rc) rc = file.ReadInt( &m_texture_range );
    }
  }
  return rc;
}



ON_MeshCurvatureStats::ON_MeshCurvatureStats()
{
  Destroy(); // initializes defaults
}

ON_MeshCurvatureStats::~ON_MeshCurvatureStats()
{}

void ON_MeshCurvatureStats::Destroy()
{
  m_style = ON::unknown_curvature_style;
  m_infinity = 0.0;
  m_count_infinite = 0;
  m_count = 0;
  m_mode = 0.0;
  m_average = 0.0;
  m_adev = 0.0;
  m_range.Set(0.0,0.0);
}

void ON_MeshCurvatureStats::EmergencyDestroy()
{
  Destroy(); // - no memory is freed so Destroy() will work
}

ON_MeshCurvatureStats::ON_MeshCurvatureStats(const ON_MeshCurvatureStats& src)
{
  *this = src;
}

ON_MeshCurvatureStats& ON_MeshCurvatureStats::operator=(const ON_MeshCurvatureStats& src)
{
  if ( this != &src ) {
    m_style          = src.m_style;
    m_infinity       = src.m_infinity;
    m_count_infinite = src.m_count_infinite;
    m_count          = src.m_count;
    m_mode           = src.m_mode;
    m_average        = src.m_average;
    m_adev           = src.m_adev;
    m_range          = src.m_range;
  }
  return *this;
}

BOOL ON_MeshCurvatureStats::Write( ON_BinaryArchive& file ) const
{
  int i;
  BOOL rc = file.Write3dmChunkVersion(1,1);
  if (rc) {
    i = m_style;
    if (rc) rc = file.WriteInt(i);
    if (rc) rc = file.WriteDouble(m_infinity);
    if (rc) rc = file.WriteInt(m_count_infinite);
    if (rc) rc = file.WriteInt(m_count);
    if (rc) rc = file.WriteDouble(m_mode);
    if (rc) rc = file.WriteDouble(m_average);
    if (rc) rc = file.WriteDouble(m_adev);
    if (rc) rc = file.WriteInterval(m_range);
  }
  return rc;
}


BOOL ON_MeshCurvatureStats::Read( ON_BinaryArchive& file )
{
  int major_version = 0;
  int minor_version = 0;
  Destroy();
  BOOL rc = file.Read3dmChunkVersion(&major_version,&minor_version);
  if (rc && major_version == 1) {
    int i;
    if (rc) rc = file.ReadInt(&i);
    if (rc) m_style = ON::CurvatureStyle(i);
    if (rc) rc = file.ReadDouble(&m_infinity);
    if (rc) rc = file.ReadInt(&m_count_infinite);
    if (rc) rc = file.ReadInt(&m_count);
    if (rc) rc = file.ReadDouble(&m_mode);
    if (rc) rc = file.ReadDouble(&m_average);
    if (rc) rc = file.ReadDouble(&m_adev);
    if (rc) rc = file.ReadInterval(m_range);
  }
  return rc;
}

BOOL ON_MeshCurvatureStats::Set( ON::curvature_style kappa_style,
                                 int Kcount,
                                 const ON_SurfaceCurvature* K,
                                 const ON_3fVector* N, // needed for normal sectional curvatures
                                 double infinity
                                 )
{
  BOOL rc = (Kcount > 0 && K != NULL);

  Destroy();

  if (rc) {
    ON_Workspace ws;
    //ON_3dVector tangent;
    double k;
    double* kappa = ws.GetDoubleMemory(Kcount);
    int i;

    switch( kappa_style ) {
    case ON::gaussian_curvature:
      m_style = kappa_style;
      m_infinity = ( infinity > 0.0 ) ? infinity : 1.0e20;
      break;
    case ON::mean_curvature: // unsigned mean
      m_style = kappa_style;
      m_infinity = ( infinity > 0.0 ) ? infinity : 1.0e10;
      break;
    case ON::min_curvature:
      m_style = kappa_style;
      m_infinity = ( infinity > 0.0 ) ? infinity : 1.0e10;
      break;
    case ON::max_curvature:
      m_style = kappa_style;
      m_infinity = ( infinity > 0.0 ) ? infinity : 1.0e10;
      break;
    //case ON::section_curvature_x:
    //  if ( !N )
    //    kappa_style = ON::unknown_curvature_style;
    //  m_style = kappa_style;
    //  m_infinity = ( infinity > 0.0 ) ? infinity : 1.0e10;
    //  break;
    //case ON::section_curvature_y:
    //  if ( !N )
    //    kappa_style = ON::unknown_curvature_style;
    //  m_style = kappa_style;
    //  m_infinity = ( infinity > 0.0 ) ? infinity : 1.0e10;
    //  break;
    //case ON::section_curvature_z:
    //  if ( !N )
    //    kappa_style = ON::unknown_curvature_style;
    //  m_style = kappa_style;
    //  m_infinity = ( infinity > 0.0 ) ? infinity : 1.0e10;
    //  break;
    default:
      rc = FALSE;
      break;
    }

    for ( i = 0; i < Kcount; i++ ) {
      switch( kappa_style ) {
      case ON::gaussian_curvature:
        k = K[i].GaussianCurvature();
        break;
      case ON::mean_curvature: // unsigned mean
        k = fabs(K[i].MeanCurvature());
        break;
      case ON::min_curvature:
        k = fabs(K[i].MinimumRadius());
        break;
      case ON::max_curvature:
        k = fabs(K[i].MaximumRadius());
        break;
      //case ON::section_curvature_x:
      //  tangent.x = 0.0; tangent.y = -N[i].z; tangent.z = N[i].y;
      //  if ( fabs(tangent.y) <= ON_SQRT_EPSILON && fabs(tangent.z) <= ON_SQRT_EPSILON )
      //    tangent.Zero();
      //  else
      //    tangent.Unitize();
      //  k = fabs(K[i].NormalCurvature(tangent));
      //  break;
      //case ON::section_curvature_y:
      //  tangent.x = N[i].z; tangent.y = 0.0; tangent.z = -N[i].x;
      //  if ( fabs(tangent.x) <= ON_SQRT_EPSILON && fabs(tangent.z) <= ON_SQRT_EPSILON )
      //    tangent.Zero();
      //  else
      //    tangent.Unitize();
      //  k = fabs(K[i].NormalCurvature(tangent));
      //  break;
      //case ON::section_curvature_z:
      //  tangent.x = -N[i].y; tangent.y = N[i].x; tangent.z = 0.0;
      //  if ( fabs(tangent.x) <= ON_SQRT_EPSILON && fabs(tangent.y) <= ON_SQRT_EPSILON )
      //    tangent.Zero();
      //  else
      //    tangent.Unitize();
      //  k = fabs(K[i].NormalCurvature(tangent));
      //  break;
      default:
        k=0.0;
        break;
      }
      if ( fabs(k) >= m_infinity ) {
        m_count_infinite++;
        continue;
      }
      if ( m_count ) {
        if ( k < m_range.m_t[0] )
          m_range.m_t[0] = k;
        else if ( k > m_range.m_t[1] )
          m_range.m_t[1] = k;
      }
      else {
        m_range.m_t[0] = m_range.m_t[1] = k;
      }
      kappa[m_count++] = k;
    }


    if ( m_count == 0 )
      rc = FALSE;
    else {
      // sum curvatures
      ON_SortDoubleArray( ON::quick_sort, kappa, m_count );

      // mode
      m_mode = kappa[m_count/2];
      if ( 0 == (m_count % 2) ) {
        m_mode += kappa[(m_count/2)-1];
        m_mode *= 0.5;
      }

      // average
      for ( i = 0; i < m_count; i++ ) {
        m_average += kappa[i];
      }
      m_average = m_average/m_count;
      
      // average deviation
      for ( i = 0; i < m_count; i++ ) {
        m_adev += fabs(kappa[i] - m_average);
      }
      m_adev = m_adev/m_count;
    }  
  }
  
  return rc;  
}

BOOL ON_Mesh::ConvertQuadsToTriangles()
{
  const BOOL bHasFaceNormals = HasFaceNormals();
  const int fcount = FaceCount();
  const int vcount = VertexCount();
  int fi;
  double d0, d1;
  if ( fcount > 0 && QuadCount() > 0 ) {
    // use SetCapacity() instead of Reserve() because it's unlikely
    // this mesh will have anything else added and we don't want to
    // waste memory.
    if ( m_F.Capacity() < fcount + m_quad_count )
      m_F.SetCapacity( fcount + m_quad_count );
    if ( bHasFaceNormals && m_FN.Capacity() < fcount + m_quad_count )
      m_FN.SetCapacity( fcount + m_quad_count );

    for ( fi = 0; fi < fcount; fi++ ) {
      ON_MeshFace& f0 = m_F[fi];
      if ( f0.IsValid(vcount) && f0.IsQuad() ) {
        // split quad into two triangles
        ON_MeshFace& f1 = m_F.AppendNew();
        d0 = m_V[f0.vi[0]].DistanceTo(m_V[f0.vi[2]]);
        d1 = m_V[f0.vi[1]].DistanceTo(m_V[f0.vi[3]]);
        if  ( d0 <= d1 ) {
          f1.vi[0] = f0.vi[0];
          f1.vi[1] = f0.vi[2];
          f1.vi[2] = f0.vi[3];
          f1.vi[3] = f1.vi[2];
          f0.vi[3] = f0.vi[2];
        }
        else {
          f1.vi[0] = f0.vi[1];
          f1.vi[1] = f0.vi[2];
          f1.vi[2] = f0.vi[3];
          f1.vi[3] = f1.vi[2];
          f0.vi[2] = f0.vi[3];
        }
        m_quad_count--;
        m_triangle_count++;
        if ( HasFaceNormals() ) {
          m_FN.AppendNew();
          ComputeFaceNormal(fi);
          ComputeFaceNormal(m_F.Count()-1);
        }
      }
    }
    if ( fcount != m_F.Count() )
      DestroyTopology(); // we added some triangles
  }
  return ( QuadCount() == 0 && TriangleCount() == FaceCount() ) ? TRUE : FALSE;
}

BOOL ON_Mesh::CountQuads()
{
  const int fcount = FaceCount();
  const int vcount = VertexCount();
  int fi;
  m_quad_count = 0;
  m_triangle_count = 0;
  m_invalid_count = 0;
  for ( fi = 0; fi < fcount; fi++ ) {
    const ON_MeshFace& f = m_F[fi];
    if ( f.IsValid(vcount) ) {
      if ( f.IsTriangle() )
        m_triangle_count++;
      else
        m_quad_count++;
    }
    else
      m_invalid_count++;
  }
  return TRUE;
}

BOOL ON_Mesh::ComputeVertexNormals()
{
  BOOL rc = FALSE;
  const int fcount = FaceCount();
  const int vcount = VertexCount();
  int vi, fi, j;
  ON_3fVector n;

  if ( fcount > 0 && vcount > 0 ) {
    rc = HasFaceNormals();
    if ( !rc )
      rc = ComputeFaceNormals();
    if ( rc ) {
      ON_Workspace ws;
      //double* face_area = ws.GetDoubleMemory(fcount);
      int* vfcount = ws.GetIntMemory( vcount );
      memset ( vfcount, 0, vcount*sizeof(*vfcount) );

      // count number of faces that use each vertex
      for ( fi = 0; fi < fcount; fi++ ) {
        ON_MeshFace& f = m_F[fi];
        if ( f.IsValid(vcount) ) {
          vfcount[f.vi[0]]++;
          vfcount[f.vi[1]]++;
          vfcount[f.vi[2]]++;
          if ( f.IsQuad() )
            vfcount[f.vi[3]]++;
        }
      }

      // set vfi[vi][] = array of face indices that use vertex vi
      int** vfi = (int**)ws.GetMemory( vcount*sizeof(vfi[0] ) );
      {
        int scratch_sz = 0;
        for ( vi = 0; vi < vcount; vi++ ) {
          scratch_sz += vfcount[vi];
        }
        int* scratch = ws.GetIntMemory(scratch_sz);
        for ( vi = 0; vi < vcount; vi++ ) {
          if ( vfcount[vi] ) {
            vfi[vi] = scratch;
            scratch += vfcount[vi];
          }
          vfcount[vi] = 0;
        }
      }
      for ( fi = 0; fi < fcount; fi++ ) {
        ON_MeshFace& f = m_F[fi];
        if ( f.IsValid(vcount) ) {
          vi = f.vi[0]; vfi[vi][vfcount[vi]++] = fi;
          vi = f.vi[1]; vfi[vi][vfcount[vi]++] = fi;
          vi = f.vi[2]; vfi[vi][vfcount[vi]++] = fi;
          if ( f.IsQuad() ) {
            vi = f.vi[3]; vfi[vi][vfcount[vi]++] = fi;
          }
        }
      }

      // average face normals to get an estimate for a vertex normal
      m_N.SetCapacity(vcount);
      m_N.SetCount(0);
      for ( vi = 0; vi < vcount; vi++ ) {
        n.Zero();
        for ( j = vfcount[vi]-1; j >= 0; j-- ) {
          n += m_FN[vfi[vi][j]];
        }
        if ( !n.Unitize() )
          rc = FALSE;
        m_N.Append(n);
      }
    }
  }
  return rc;
}

BOOL ON_Mesh::NormalizeTextureCoordinates()
{
  BOOL rc = HasTextureCoordinates() 
           && (m_tex_domain[0].IsIncreasing() || m_tex_domain[0].IsDecreasing() )
           && (m_tex_domain[1].IsIncreasing() || m_tex_domain[1].IsDecreasing() );
  if ( rc ) {
    if ( m_srf_tex_rotate ||
         m_tex_domain[0].m_t[0] != 0.0 || m_tex_domain[0].m_t[1] != 1.0 ||
         m_tex_domain[0].m_t[1] != 0.0 || m_tex_domain[1].m_t[1] != 1.0 ) {
      const int vertex_count = m_T.Count();
      int ti;
      ON_2fPoint t0, t1;
      for (ti = 0; ti < vertex_count; ti++ ) {
        t0 = m_T[ti];
        if ( m_srf_tex_rotate ) {
          t1.y = m_tex_domain[0].NormalizedParameterAt(t0.x);
          t1.x = 1.0-m_tex_domain[1].NormalizedParameterAt(t0.y);
        }
        else {
          t1.x = m_tex_domain[0].NormalizedParameterAt(t0.x);
          t1.y = m_tex_domain[1].NormalizedParameterAt(t0.y);
        }
        m_T[ti] = t1;
      }
      m_tex_domain[0].Set( 0.0, 1.0 );
      m_tex_domain[1].Set( 0.0, 1.0 );
      m_srf_tex_rotate = 0;
    }
    if ( m_mesh_parameters )
      m_mesh_parameters->m_texture_range = 1;
  }
  return rc;
}

BOOL ON_Mesh::GetSurfaceParameters( double tex_x, double tex_y, double* srf_s, double* srf_t ) const
{
  // convert texture coordinates to surface parameters
  double tex_s, tex_t;

  if ( m_srf_tex_rotate ) {
    // undo rotation and normalize
    tex_s = 1.0 - m_tex_domain[1].NormalizedParameterAt( tex_y );
    tex_t = m_tex_domain[0].NormalizedParameterAt( tex_x );
  }
  else {
    // normalize
    tex_s = m_tex_domain[0].NormalizedParameterAt( tex_x );
    tex_t = m_tex_domain[1].NormalizedParameterAt( tex_y );
  }
  *srf_s = m_srf_domain[0].ParameterAt(tex_s);
  *srf_t = m_srf_domain[1].ParameterAt(tex_t);
  return TRUE;
}


BOOL ON_Mesh::EvaluateMeshGeometry( const ON_Surface& srf )
{
  BOOL rc = FALSE;
  const int vcount = VertexCount();
  if ( HasTextureCoordinates() && m_srf_domain[0].IsIncreasing() && m_srf_domain[1].IsIncreasing() ) {
    const BOOL bHasVertexNormals = HasVertexNormals();
    m_N.SetCapacity(vcount);
    int vi, side, hint[2];
    ON_3dPoint point;
    ON_3dVector normal, Ds, Dt, Dss, Dst, Dtt, K1, K2;
    double s, t, kgauss, kmean;
    side = 0;
    hint[0] = 0;
    hint[1] = 0;
    if ( HasPrincipalCurvatures() ) {
      for ( vi = 0; vi < vcount; vi++ ) {
        ON_2fPoint& tex = m_T[vi];
        // convert texture coordinates to normalizied texture coordinates
        GetSurfaceParameters( tex.x, tex.y, &s, &t );
        srf.Ev2Der( s, t, point, Ds, Dt, Dss, Dst, Dtt, side, hint );
        ON_EvNormal( side, Ds, Dt, Dss, Dst, Dtt, normal );
        ON_EvPrincipalCurvatures( Ds, Dt, Dss, Dst, Dtt, normal,
                                  &kgauss, &kmean, 
                                  &m_K[vi].k1, &m_K[vi].k2, 
                                  K1, K2 ); //m_K[vi].e1, m_K[vi].e2 );
        m_V[vi] = point;
        if ( bHasVertexNormals )
          m_N[vi] = normal;
      }
      InvalidateCurvatureStats();
    }
    else if ( bHasVertexNormals ) {
      for ( vi = 0; vi < vcount; vi++ ) {
        ON_2fPoint& tex = m_T[vi];
        GetSurfaceParameters( tex.x, tex.y, &s, &t );
        srf.EvNormal( s, t, point, normal, side, hint );
        m_V[vi] = point;
        m_N[vi] = normal;
      }
    }
    else {
      for ( vi = 0; vi < vcount; vi++ ) {
        ON_2fPoint& tex = m_T[vi];
        GetSurfaceParameters( tex.x, tex.y, &s, &t );
        srf.EvPoint( s, t, point, side, hint );
        m_V[vi] = point;
      }
    }
    if ( HasFaceNormals() )
      ComputeFaceNormals();
    rc = TRUE;
    InvalidateVertexBoundingBox();
    InvalidateVertexNormalBoundingBox();
    DeleteMeshParameters();
  }
  return rc;
}

void ON_Mesh::SetMeshParameters( const ON_MeshParameters& mp )
{
  DeleteMeshParameters();
  m_mesh_parameters = new ON_MeshParameters(mp);
}

const ON_MeshParameters* ON_Mesh::MeshParameters() const
{
  return m_mesh_parameters;
}

void ON_Mesh::DeleteMeshParameters()
{
  if ( m_mesh_parameters ) {
    delete m_mesh_parameters;
    m_mesh_parameters = 0;
  }
}

static int compare3fPoint( const ON_3fPoint* a, const ON_3fPoint* b )
{
  if ( a->x < b->x ) return -1;
  if ( a->x > b->x ) return  1;
  if ( a->y < b->y ) return -1;
  if ( a->y > b->y ) return  1;
  if ( a->z < b->z ) return -1;
  if ( a->z > b->z ) return  1;
  return 0;
}

typedef int (*ON_COMPAR_LPVOID_LPVOID)(const void*,const void*);

static 
int GetPointMap( int pt_count, const ON_3fPoint* pt, ON_SimpleArray<int>& pt_map )
{
  // Builds a mapping array, pt_map[], such that the length of pt_map[] 
  // is pt_count and pt_map[i] == pt_map[j] if and only if pt[i] == pt[j]
  // as 3d points.  The values in map[] run from 0 to max_pt_index.
  int vt0, vt1;
  ON_3fPoint p0;
  int* map;
  int* index;
  int max_pt_index = 0;
  if ( pt_count > 0 && pt ) {
    index = (int*)onmalloc(pt_count*sizeof(*index));
    ON_Sort( ON::quick_sort, index, pt, pt_count, sizeof(*pt), (ON_COMPAR_LPVOID_LPVOID)compare3fPoint );
    pt_map.SetCapacity( pt_count );
    pt_map.SetCount( pt_count );
    map = pt_map.Array();
    for ( vt0 = 0; vt0 < pt_count; vt0++ )
      map[vt0] = -1;
    for (vt0 = 0; vt0 < pt_count; vt0 = vt1, max_pt_index++) {
      p0 = pt[index[vt0]];
      for ( vt1=vt0+1; vt1<pt_count && 0==compare3fPoint(&p0,pt+index[vt1]); vt1++ ) {
        // empty
      }
      while ( vt0 < vt1 ) {
        map[index[vt0++]] = max_pt_index;
      }
    }
    onfree(index);
  }
  if ( max_pt_index == 0 )
    pt_map.Destroy();
  return max_pt_index;
}

int ON_Mesh::CullDegenerateFaces()
{
  int bad_count = 0;
  int degenerate_count = 0;
  const int fcount = m_F.Count();
  const int vcount = m_V.Count();
  ON_MeshFace f;
  int fi;

  if ( fcount > 0 ) {
    // use GetTopologyVertexMap() instead of calling Topology() because this mesh
    // may have lots of bogus faces that need to be culled.
    ON_SimpleArray<int> topv_map;
    const int topv_count = GetPointMap( m_V.Count(), m_V.Array(), topv_map );
    if ( topv_count > 0 && topv_map.Count() == m_V.Count() ) {
      ON_Workspace ws;
      const int* vtop = topv_map.Array();
      unsigned char* bBadFace = (unsigned char*)ws.GetMemory(fcount*sizeof(*bBadFace));
      memset( bBadFace, 0, fcount*sizeof(*bBadFace) );
      for ( fi = 0; fi < fcount; fi++ ) {
        ON_MeshFace& f0 = m_F[fi];
        f.vi[0] = vtop[f0.vi[0]];
        f.vi[1] = vtop[f0.vi[1]];
        f.vi[2] = vtop[f0.vi[2]];
        f.vi[3] = vtop[f0.vi[3]];
        if ( !f.IsValid(topv_count) ) {
          degenerate_count++;
          f = m_F[fi];
          if ( f.vi[0] == f.vi[1] || f.vi[0] < 0 || f.vi[0] >= vcount ) {
            f0.vi[0] = f0.vi[1];
            f0.vi[1] = f0.vi[2];
            f0.vi[2] = f0.vi[3];
            f.vi[0] = f.vi[1];
            f.vi[1] = f.vi[2];
            f.vi[2] = f.vi[3];
          }
          if ( f.vi[1] == f.vi[2] || f.vi[1] < 0 || f.vi[1] >= vcount ) {
            f0.vi[1] = f0.vi[2];
            f0.vi[2] = f0.vi[3];
            f.vi[1] = f.vi[2];
            f.vi[2] = f.vi[3];
          }
          if ( f.vi[2] < 0 || f.vi[2] >= vcount ) {
            f0.vi[2] = f0.vi[3];
            f.vi[2] = f.vi[3];
          }
          if ( f.vi[3] < 0 || f.vi[3] >= vcount ) {
            f0.vi[3] = f0.vi[2];
            f.vi[3] = f.vi[2];
          }
          if ( !f0.IsValid(vcount) || !f.IsValid(vcount) ) {
            bBadFace[fi] = 1;
            bad_count++;
          }
        }
      }
      if ( bad_count > 0 ) {
        int fcnt = 0;
        for ( fi = fcnt = 0; fi < fcount; fi++ ) {
          if ( !bBadFace[fi] ) {
            if ( fcnt < fi )
              m_F[fcnt] = m_F[fi];
            fcnt++;
          }
        }
        m_F.SetCount(fcnt);
      }

      if ( degenerate_count > 0 ) {
        DestroyTopology();
        m_invalid_count = 0;
        m_quad_count = 0;
        m_triangle_count = 0;
      }
    }
  }
  return degenerate_count;
}

////////////////////////////////////////////////////////////////
//
//   ON_SurfaceCurvature
//
double ON_SurfaceCurvature::GaussianCurvature() const
{
  return k1*k2;
}

double ON_SurfaceCurvature::MeanCurvature() const
{
  return 0.5*(k1+k2);
}

double ON_SurfaceCurvature::MinimumRadius() const
{
  double k;
  k = (fabs(k1)>=fabs(k2)) ? fabs(k1) : fabs(k2); // k = maximum directional curvature
  k = ( k > 1.0e-300 ) ? 1.0/k : 1.0e300;         // 1/k = minimum radius of curvature
  return k;
}

double ON_SurfaceCurvature::MaximumRadius() const
{
  double k;
  if ( k1*k2 <= 0.0 ) {
    // if principal curvatures have opposite signs, there
    // is a direction with zero directional curvature
    k = 0.0;
  }
  else {
    k = (fabs(k1)<=fabs(k2)) ? fabs(k1) : fabs(k2);
  }
  // k = minimum directional curvature
  k = ( k > 1.0e-300 ) ? 1.0/k : 1.0e300; // 1/k = maximum radius of curvature
  return k;
}

//double ON_SurfaceCurvature::NormalCurvature(const ON_3dVector& tangent) const
//{
//  double c = tangent*e1;
//  double s = tangent*e2;
//  return k1*c*c + k2*s*s;
//}

//double ON_SurfaceCurvature::NormalSectionCurvature( const ON_3dVector& section_normal, const ON_3dVector& surface_normal ) const
//{
//  ON_3dVector tangent = ON_CrossProduct( section_normal, surface_normal );
//  if ( fabs(tangent.x) <= ON_SQRT_EPSILON && fabs(tangent.y) <= ON_SQRT_EPSILON && fabs(tangent.z) <= ON_SQRT_EPSILON )
//    tangent.Zero();
//  else
//    tangent.Unitize();
//  return NormalCurvature(tangent);
//}

ON_MeshTopology::ON_MeshTopology() 
: m_mesh(0), m_memchunk(0), m_bIsValid(0)
{
}

ON_MeshTopology::~ON_MeshTopology()
{
  Destroy();
}

void ON_MeshTopology::Destroy()
{
  m_topv_map.Destroy();
  m_topv.Destroy();
  m_tope.Destroy();
  m_topf.Destroy();
  struct memchunk *p, *n;
  n = m_memchunk;
  while(n)
  {
    p = n;
    n = n->next;
    onfree(p);
  }
  m_memchunk = 0;
  m_bIsValid = 0;
}

void ON_MeshTopology::EmergencyDestroy()
{
  m_mesh = 0;
  m_topv_map.EmergencyDestroy();
  m_topv.EmergencyDestroy();
  m_tope.EmergencyDestroy();
  m_topf.EmergencyDestroy();
  m_memchunk = 0;
  m_bIsValid = 0;
}

int ON_MeshTopology::TopVertexCount() const
{
  return m_topv.Count();
}

//////////
// number of topoligical edges
int ON_MeshTopology::TopEdgeCount() const
{
  return m_tope.Count();
}

//////////
// number of topoligical faces (same as m_mesh.FaceCount())
int ON_MeshTopology::TopFaceCount() const
{
  return m_topf.Count();
}

const ON_3fPoint& ON_MeshTopology::TopVertexPoint( int vtopi ) const
{
  return m_mesh->m_V[m_topv[vtopi].m_vi[0]];
}

////////
// returns index of edge that connects topological vertices
// returns -1 if no edge is found.
int ON_MeshTopology::TopEdge( int vtopi0, int vtopi1 ) const
{
  int i0, i1, ei, vi0;
  if ( vtopi0 > vtopi1 ) {vi0 = vtopi0; vtopi0 = vtopi1; vtopi1 = vi0;}
  if ( vtopi0 < vtopi1 ) {
    const int tope_count = TopEdgeCount();
    const ON_MeshTopologyEdge* tope = m_tope.Array(); // to speed array access
    i0 = 0;
    i1 = tope_count;
    while ( i0 < i1 )
    {
      ei = (i0+i1)/2;
      vi0 = tope[ei].m_topvi[0];
      if ( vi0 < vtopi0 ) {
        if ( i0 == ei )
          break;
        i0 = ei;
      }
      else if ( vi0 > vtopi0 ) {
        if ( i1 == ei )
          break;
        i1 = ei;
      }
      else {
        while (ei > 0 && tope[ei-1].m_topvi[0] == vtopi0 )
          ei--;
        while ( ei < tope_count && tope[ei].m_topvi[0] == vtopi0 ) {
          if ( tope[ei].m_topvi[1] == vtopi1 )
            return ei;
          ei++;
        }
        break;
      }
    }
  }
  return -1;
}

BOOL ON_MeshTopology::GetTopFaceVertices( int fi, int topvi[4] ) const
{
  if ( fi >= 0 && fi < m_mesh->m_F.Count() ) {
    const int* fvi = m_mesh->m_F[fi].vi;
    topvi[0] = m_topv_map[fvi[0]];
    topvi[1] = m_topv_map[fvi[1]];
    topvi[2] = m_topv_map[fvi[2]];
    topvi[3] = m_topv_map[fvi[3]];
  }
  return TRUE;
}


struct TMPEDGE
{
  int vi[2];
  int fi;
};

static int ecomp( const struct TMPEDGE* ea, const struct TMPEDGE* eb )
{
  if ( ea->vi[0] < eb->vi[0] )
    return -1;
  if ( ea->vi[0] > eb->vi[0] )
    return 1;
  if ( ea->vi[1] < eb->vi[1] )
    return -1;
  if ( ea->vi[1] > eb->vi[1] )
    return 1;
  if ( ea->fi < eb->fi )
    return -1;
  if ( ea->fi > eb->fi )
    return 1;
  return 0;
}

int* ON_MeshTopology::GetIntArray(int length)
{
  int* a = 0;
  if ( length > 0 ) {
    struct memchunk* pm;
    pm = (struct memchunk*)onmalloc(length*sizeof(*a) + sizeof(*pm));
    if ( pm ) {
      pm->next = m_memchunk;
      m_memchunk = pm++;
      a = (int*)pm;
    }
  }
  return a;
}

BOOL ON_MeshTopologyFace::IsTriangle() const
{
  return ( m_topei[2] == m_topei[3] && m_topei[0] != m_topei[1] )
         ? TRUE : FALSE;
}

BOOL ON_MeshTopologyFace::IsQuad() const
{
  return ( m_topei[2] != m_topei[3] ) ? TRUE : FALSE;
}

BOOL ON_MeshTopologyFace::IsValid() const
{
  return ( m_topei[0] != m_topei[1] ) ? TRUE : FALSE;
}


BOOL ON_MeshTopology::IsValid() const
{
  ON_Workspace ws;
  int topvi, topei, topfi, vi, fi, j, jmax, k, tfvi[4];
  ON_3fPoint p;

  // simple checks
  if ( !m_bIsValid )
    return FALSE;
  if ( !m_mesh )
    return FALSE;
  if ( this != &(m_mesh->Topology()) )
    return FALSE;
  const int v_count = m_mesh->VertexCount();
  const int f_count = m_mesh->FaceCount();
  const int topv_count = TopVertexCount();
  const int tope_count = TopEdgeCount();
  const int topf_count = TopFaceCount();
  if ( topv_count > v_count || topv_count < 0 )
    return FALSE;
  if ( topv_count == 0 && v_count > 0 )
    return FALSE;
  if ( topf_count != f_count )
    return FALSE;
  if ( f_count > 0 && tope_count < 3 )
    return FALSE;
  if ( m_topv_map.Count() != v_count )
    return FALSE;

  // check vertex information
  for ( vi = 0; vi < v_count; vi++ ) {
    topvi = m_topv_map[vi];
    if ( topvi < 0 || topvi >= topv_count )
      return FALSE;
  }

  char* vCheck = (char*)ws.GetMemory( v_count*sizeof(*vCheck) );
  memset( vCheck, 0, v_count*sizeof(*vCheck) );
  for ( topvi = 0; topvi < topv_count; topvi++ )
  {
    const ON_MeshTopologyVertex& topv = m_topv[topvi];
    if ( topv.m_v_count <= 0 )
      return FALSE;
    if ( !topv.m_vi )
      return FALSE;
    p = TopVertexPoint(topvi);
    for ( j = 0; j < topv.m_v_count; j++ ) {
      vi = topv.m_vi[j];
      if ( vi < 0 )
        return FALSE;
      if ( vi >= v_count )
        return FALSE;
      if ( vCheck[vi] != 0 )
        return FALSE; // mesh.m_V[vi] is referenced multiple times
      if ( compare3fPoint( &p, &m_mesh->m_V[vi] ) )
        return FALSE; // mesh.m_V[vi] not at same location as topv
      if ( m_topv_map[vi] != topvi )
        return FALSE; 
      vCheck[vi] = 1;
    }

    // check that edges in m_topei[] list have topvi has a start/end
    if ( topv.m_tope_count < 0 )
      return FALSE;
    if ( topv.m_tope_count == 0 && topv.m_topei )
      return FALSE; // array should be NULL
    if ( topv.m_tope_count > 0 && !topv.m_topei )
      return FALSE; // array should not be NULL
    for ( j = 0; j < topv.m_tope_count; j++ ) {
      topei = topv.m_topei[j];
      if ( topei < 0 )
        return FALSE;
      if ( topei >= tope_count )
        return FALSE;
      const ON_MeshTopologyEdge& tope = m_tope[topei];
      if ( tope.m_topvi[0] != topvi && tope.m_topvi[1] != topvi )
        return FALSE; // edge doesn't reference this top vtx
      for ( k = 0; k < j; k++ ) {
        if ( topv.m_topei[k] == topei )
          return FALSE; // edge listed multiple times
      }
    }
  }

  // make sure every mesh.m_V[] maps to a topoligical vertex
  for ( vi = 0; vi < v_count; vi++ ) {
    if ( vCheck[vi] != 1 )
      return FALSE; // mesh.m_V[vi] is not referenced
    topvi = m_topv_map[vi];
    if ( topvi < 0 )
      return FALSE;
    if ( topvi >= topv_count )
      return FALSE;
    const ON_MeshTopologyVertex& topv = m_topv[topvi];
    for ( j = 0; j < topv.m_v_count; j++ ) {
      if ( topv.m_vi[j] == vi )
        break;
    }
    if ( j >= topv.m_v_count )
      return FALSE; // bogus m_topv_map[] array
  }

  // check edges
  for ( topei = 0; topei < tope_count; topei++ ) {
    const ON_MeshTopologyEdge& tope = m_tope[topei];
    if ( tope.m_topvi[0] < 0 || tope.m_topvi[0] >= topv_count )
      return FALSE;
    if ( tope.m_topvi[1] < 0 || tope.m_topvi[1] >= topv_count )
      return FALSE;
    if ( tope.m_topvi[0] == tope.m_topvi[1] )
      return FALSE;

    const ON_MeshTopologyVertex& topv0 = m_topv[tope.m_topvi[0]];
    for ( j = 0; j < topv0.m_tope_count; j++ ) {
      if ( topv0.m_topei[j] == topei )
        break;
    }
    if ( j >= topv0.m_tope_count )
      return FALSE; // start vtx not linked to edge

    const ON_MeshTopologyVertex& topv1 = m_topv[tope.m_topvi[1]];
    for ( j = 0; j < topv1.m_tope_count; j++ ) {
      if ( topv1.m_topei[j] == topei )
        break;
    }
    if ( j >= topv1.m_tope_count )
      return FALSE; // end vtx not linked to edge

    if ( tope.m_topf_count < 0 )
      return FALSE;
    if ( tope.m_topf_count == 0 && tope.m_topfi )
      return FALSE; // array should be NULL
    if ( tope.m_topf_count > 0 && !tope.m_topfi )
      return FALSE; // array should not be NULL
    for ( j = 0; j < tope.m_topf_count; j++ ) {
      fi = tope.m_topfi[j];
      if ( fi < 0 || fi >= f_count )
        return FALSE;
      const ON_MeshTopologyFace& topf = m_topf[fi];
      for ( k = 0; k < 4; k++ ) {
        if ( topf.m_topei[k] == topei )
          break;
      }
      if ( k >= 4 )
        return FALSE; // edge not in face's list
    }
  }

  // check faces
  for ( fi = 0; fi < f_count; fi++ ) {
    topfi = fi;
    const ON_MeshTopologyFace& topf = m_topf[topfi];
    const ON_MeshFace& f = m_mesh->m_F[fi];
    if ( topf.m_topei[0] < 0 || topf.m_topei[0] >= tope_count )
      return FALSE;
    if ( topf.m_topei[1] < 0 || topf.m_topei[1] >= tope_count )
      return FALSE;
    if ( topf.m_topei[2] < 0 || topf.m_topei[2] >= tope_count )
      return FALSE;
    if ( topf.m_topei[3] < 0 || topf.m_topei[3] >= tope_count )
      return FALSE;
    tfvi[0] = m_topv_map[f.vi[0]];
    tfvi[1] = m_topv_map[f.vi[1]];
    tfvi[2] = m_topv_map[f.vi[2]];
    tfvi[3] = m_topv_map[f.vi[3]];
    if (    topf.m_topei[0] != 0 || topf.m_topei[1] != 0 
         || topf.m_topei[2] != 0 || topf.m_topei[3] != 0 ) {
      if ( !f.IsValid(v_count) )
        return FALSE;
      if ( f.IsTriangle() ) {
        if (topf.m_topei[2] != topf.m_topei[3] )
          return FALSE;
        jmax = 3;
      }
      else {
        if (topf.m_topei[2] == topf.m_topei[3] )
          return FALSE;
        jmax = 4;
      }
      for ( j = 0; j < jmax; j++ ) {
        const ON_MeshTopologyEdge& tope = m_tope[topf.m_topei[j]];
        for ( k = 0; k < tope.m_topf_count; k++ ) {
          if ( tope.m_topfi[k] == topfi )
            break;
        }
        if ( k >= tope.m_topf_count )
          return FALSE;

        // topedge m_tope[topf.m_topei[j]] ENDS at topvtx m_topv[tfvi[j]]
        if ( topf.m_reve[j] ) {
          if ( tope.m_topvi[1] != tfvi[(j+3)%4] )
            return FALSE;
          if ( tope.m_topvi[0] != tfvi[j] )
            return FALSE;
        }
        else {
          if ( tope.m_topvi[0] != tfvi[(j+3)%4] )
            return FALSE;
          if ( tope.m_topvi[1] != tfvi[j] )
            return FALSE;
        }
      }      
    }
    else {
      // all topf.m_topei[] must be zeros
      if (    topf.m_topei[0] != 0 || topf.m_topei[1] != 0 
           || topf.m_topei[2] != 0 || topf.m_topei[3] != 0 )
        return FALSE;
    }
  }
  return TRUE;
}

void ON_MeshTopology::Dump( ON_TextLog& dump ) const
{
  ON_3fPoint p;
  int vi, ei, fi, j;
  const int topv_count = m_topv.Count();
  const int tope_count = m_tope.Count();
  const int topf_count = m_topf.Count();

  // topological vertex information
  for ( vi = 0; vi < topv_count; vi++ ) {
    const ON_MeshTopologyVertex& v = m_topv[vi];
    dump.Print("topv %d: ",vi);
    if (m_mesh) {
      // dump geometric location of this vertex
      p = m_mesh->m_V[v.m_vi[0]];
      dump.Print("{%g,%g,%g} ", p.x, p.y, p.z);
    }

    // list all mesh geometry viertices that are coincident with this
    // topological vertex
    dump.Print("(");
    for ( j = 0; j < v.m_v_count; j++ ) {
      if ( j )
        dump.Print(",");
      dump.Print("m_V[%d]",v.m_vi[j]);
    }

    // list all toplogical edges that begin/end at this topological vertex
    dump.Print(") (");
    for ( j = 0; j < v.m_tope_count; j++ ) {
      if ( j )
        dump.Print(",");
      dump.Print("%d",v.m_topei[j]);
    }
    dump.Print(")\n");
  }

  // topological edge information
  for ( ei = 0; ei < tope_count; ei++ ) {
    const ON_MeshTopologyEdge& e = m_tope[ei];
    dump.Print("tope %d: topv%d to topvv%d (", ei, e.m_topvi[0], e.m_topvi[1] );
    // list all mesh topolical faces attached to this topolical edge
    for ( j = 0; j < e.m_topf_count; j++ ) {
      if (j)
        dump.Print(",");
      dump.Print("f%d",e.m_topfi[j]);
    }
    dump.Print(")\n");
  }

  // topological face information
  // mesh m_F[] index = mesh topology m_topf[] index
  for ( fi = 0; fi < topf_count; fi++ ) {
    const ON_MeshTopologyFace& f = m_topf[fi];
    dump.Print("topf %d: (",fi);
    for ( j = 0; j < 4; j++ ) {
      if ( j == 3 && f.m_topei[3] == f.m_topei[2] )
        break; // triangular face
      if (j)
        dump.Print(",");
      dump.Print("%ce%d",f.m_reve[j]?'-':'+',f.m_topei[j]);
    }
    dump.Print(")\n");
  }
}


BOOL ON_MeshTopology::Create()
{
  if ( !m_bIsValid ) {
    Destroy();

    // build vertex topology information
    const int fcount = m_mesh->FaceCount();
    const int vcount = m_mesh->VertexCount();
    int* vindex = GetIntArray(vcount);
    m_bIsValid = m_mesh->m_V.Sort( ON::quick_sort, vindex, compare3fPoint );
    if ( m_bIsValid ) {
      m_topv_map.SetCapacity( vcount );
      m_topv_map.SetCount( vcount );
      m_topv.SetCapacity( vcount );
      int vt0, vt1, topvi;
      ON_3fPoint p0;
      for (vt0 = 0; vt0 < vcount; vt0 = vt1) {
        topvi = m_topv.Count();
        ON_MeshTopologyVertex& topv = m_topv.AppendNew();
        topv.m_vi = vindex+vt0;
        p0 = m_mesh->m_V[vindex[vt0]];
        for ( vt1=vt0+1; vt1<vcount && 0==compare3fPoint(&p0,&m_mesh->m_V[vindex[vt1]]); vt1++ ) {
          // empty
        }
        topv.m_v_count = vt1-vt0;
        while ( vt0 < vt1 ) {
          m_topv_map[vindex[vt0++]] = topvi;
        }
      }
      m_topv.Shrink();
    }

    // build edge topology information
    const int topv_count = m_topv.Count();
    if ( m_bIsValid && topv_count >= 2 ) {
      BOOL rc = FALSE;
      int ei, ecnt, fi, fvi, vi0, vi1, efi, topfvi[4];
      const int* vi;
      ON_MeshFace f;

      if ( fcount > 0 && vcount > 0 ) {
        ON_Workspace ws;
        struct TMPEDGE* e = (struct TMPEDGE*)ws.GetMemory( 4*fcount*sizeof(*e) );
        memset(e,0,4*fcount*sizeof(*e));
        ecnt = 0;
        for ( fi = 0; fi < fcount; fi++ ) {
          f = m_mesh->m_F[fi];
          f.vi[0] = m_topv_map[f.vi[0]];
          f.vi[1] = m_topv_map[f.vi[1]];
          f.vi[2] = m_topv_map[f.vi[2]];
          f.vi[3] = m_topv_map[f.vi[3]];
          if ( f.IsValid(topv_count) ) {
            vi = f.vi;
            vi1 = vi[3];
            for ( fvi = 0; fvi < 4; fvi++ ) {
              vi0 = vi1;
              vi1 = vi[fvi];
              if ( vi0 == vi1 )
                continue;
              e[ecnt].fi = fi;
              if ( vi0 < vi1 ) {
                e[ecnt].vi[0] = vi0;
                e[ecnt++].vi[1] = vi1;
              }
              else if ( vi0 > vi1 ) {
                e[ecnt].vi[0] = vi1;
                e[ecnt++].vi[1] = vi0;
              }
            }
          }
        }
        if ( ecnt > 0 ) {
          rc = TRUE;
          qsort( e, ecnt, sizeof(*e), (ON_COMPAR_LPVOID_LPVOID)ecomp );
          {
            int ii, kk = 0, jj = 0;
            int badsort = 0;
            for ( ii = 1; ii < ecnt; ii++ ) {
              if ( e[ii].vi[0] == 500 || e[ii-1].vi[0] == 500 )
                jj++;
              if ( e[ii].vi[0] >= e[ii].vi[1] )
                kk++;
              if ( ecomp( &e[ii-1], &e[ii] ) != -1 )
                badsort++;
            }
          }
          int etop_count = 0;
          ei = 0;
          while( ei < ecnt ) {
            etop_count++;
            vi0 = e[ei].vi[0];
            vi1 = e[ei].vi[1];
            ei++;
            while (ei < ecnt && e[ei].vi[0] == vi0 && e[ei].vi[1] == vi1 )
              ei++;
          }
          m_tope.SetCapacity(etop_count);
          int* efindex = GetIntArray(ecnt);
          for ( ei = 0; ei < ecnt; /*empty*/ ) {
            ON_MeshTopologyEdge& tope = m_tope.AppendNew();
            tope.m_topvi[0] = e[ei].vi[0];
            tope.m_topvi[1] = e[ei].vi[1];
            tope.m_topfi = efindex;
            tope.m_topf_count = 0;
            *efindex++ = e[ei].fi;
            tope.m_topf_count++;
            ei++;
            while( ei < ecnt && e[ei].vi[0] == tope.m_topvi[0] && e[ei].vi[1] == tope.m_topvi[1] ) {
              *efindex++ = e[ei].fi;
              tope.m_topf_count++;
              ei++;
            }
          }
          efindex = 0; // memory deallocated by ~ON_MeshTopology()

          // connect vertices to edges;
          ecnt = m_tope.Count();
          int* ve_count = (int*)onmalloc(topv_count*sizeof(*ve_count));
          memset( ve_count, 0, topv_count*sizeof(*ve_count));
          for ( ei = 0; ei < ecnt; ei++ ) {
            ON_MeshTopologyEdge& tope = m_tope[ei];
            ve_count[tope.m_topvi[0]]++;
            ve_count[tope.m_topvi[1]]++;
          }
          int* vei = GetIntArray(2*ecnt);
          for ( vi0 = 0; vi0 < topv_count; vi0++ ) {
            ON_MeshTopologyVertex& topv = m_topv[vi0];
            if ( ve_count[vi0] > 0 ) {
              topv.m_topei = vei;
              vei += ve_count[vi0];
            }
          }
          onfree(ve_count); ve_count = 0;
          vei = 0; // memory deallocated by ~ON_MeshTopology()
          for ( ei = 0; ei < ecnt; ei++ ) {
            ON_MeshTopologyEdge& tope = m_tope[ei];
            ON_MeshTopologyVertex& topv0 = m_topv[tope.m_topvi[0]];
            ON_MeshTopologyVertex& topv1 = m_topv[tope.m_topvi[1]];
            vei = (int*)topv0.m_topei;
            vei[topv0.m_tope_count++] = ei;
            vei = (int*)topv1.m_topei;
            vei[topv1.m_tope_count++] = ei;
          }


          // build face topology information
          m_topf.SetCapacity(fcount);
          m_topf.SetCount(fcount);
          memset( m_topf.Array(), 0, fcount*sizeof(ON_MeshTopologyFace) );
          for ( fi = 0; fi < fcount; fi++ ) {
            ON_MeshTopologyFace& f = m_topf[fi];
            f.m_topei[0] = -1;
            f.m_topei[1] = -1;
            f.m_topei[2] = -1;
            f.m_topei[3] = -1;
          }
          for ( ei = 0; ei < ecnt; ei++ ) {
            ON_MeshTopologyEdge& tope = m_tope[ei];
            for (efi = 0; efi < tope.m_topf_count; efi++ ) {
              // Because ON_MeshFace.vi[2] == ON_MeshFace.vi[3] for triangles,
              // we have topf.m_topei[j] BEGIN at ON_MeshFace.vi[(j+3)%4] and END at ON_MeshFace.vi[j]
              fi = tope.m_topfi[efi];
              f = m_mesh->m_F[fi];
              topfvi[0] = m_topv_map[f.vi[0]];
              topfvi[1] = m_topv_map[f.vi[1]];
              topfvi[2] = m_topv_map[f.vi[2]];
              topfvi[3] = m_topv_map[f.vi[3]];
              ON_MeshTopologyFace& topf = m_topf[fi];
              vi0 = tope.m_topvi[0];
              vi1 = tope.m_topvi[1];
              // unroll loop for speed
              if      ( vi0 == topfvi[3] && vi1 == topfvi[0] ) {
                topf.m_topei[0] = ei;
                topf.m_reve[0] = 0;
              }
              else if ( vi0 == topfvi[0] && vi1 == topfvi[1] ) {
                topf.m_topei[1] = ei;
                topf.m_reve[1] = 0;
              }
              else if ( vi0 == topfvi[1] && vi1 == topfvi[2] ) {
                topf.m_topei[2] = ei;
                topf.m_reve[2] = 0;
              }
              else if ( vi0 == topfvi[2] && vi1 == topfvi[3] ) {
                topf.m_topei[3] = ei;
                topf.m_reve[3] = 0;
              }
              else if ( vi1 == topfvi[3] && vi0 == topfvi[0] ) {
                topf.m_topei[0] = ei;
                topf.m_reve[0] = 1;
              }
              else if ( vi1 == topfvi[0] && vi0 == topfvi[1] ) {
                topf.m_topei[1] = ei;
                topf.m_reve[1] = 1;
              }
              else if ( vi1 == topfvi[1] && vi0 == topfvi[2] ) {
                topf.m_topei[2] = ei;
                topf.m_reve[2] = 1;
              }
              else if ( vi1 == topfvi[2] && vi0 == topfvi[3] ) {
                topf.m_topei[3] = ei;
                topf.m_reve[3] = 1;
              }
            }
          }
          for ( fi = 0; fi < fcount; fi++ ) {
            ON_MeshTopologyFace& f = m_topf[fi];
            BOOL bIsGood = FALSE;
            if (    f.m_topei[0] >= 0 && f.m_topei[1] >= 0 && f.m_topei[2] >=0 
                 && f.m_topei[0] != f.m_topei[1] 
                 && f.m_topei[1] != f.m_topei[2] 
                 && f.m_topei[2] != f.m_topei[0] 
                 ) {
              if ( m_mesh->m_F[fi].IsTriangle() ) {
                bIsGood = TRUE;
                f.m_topei[3] = f.m_topei[2];
              }
              else if (   f.m_topei[3] >= 0 
                       && f.m_topei[0] != f.m_topei[3] 
                       && f.m_topei[1] != f.m_topei[3] 
                       && f.m_topei[2] != f.m_topei[3] ) {
                bIsGood = TRUE;
              }
            }
            if ( !bIsGood ) {
              f.m_topei[0] = 0;
              f.m_topei[1] = 0;
              f.m_topei[2] = 0;
              f.m_topei[3] = 0;
              f.m_reve[0] = 0;
              f.m_reve[1] = 0;
              f.m_reve[2] = 0;
              f.m_reve[3] = 0;
            }
          }

        }
      }
    }
  }

  if ( !m_bIsValid )
    Destroy();
  return m_bIsValid;
}
