/* $Header: /src2/opennurbs/opennurbs_plane.h 11    1/03/01 4:59p Dalelear $ */
/* $NoKeywords: $ */
/*
//
// Copyright (c) 1993-2001 Robert McNeel & Associates. All rights reserved.
// Rhinoceros is a registered trademark of Robert McNeel & Assoicates.
//
// THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
// ALL IMPLIED WARRANTIES OF FITNESS FOR ANY PARTICULAR PURPOSE AND OF
// MERCHANTABILITY ARE HEREBY DISCLAIMED.
//				
// For complete openNURBS copyright information see <http://www.opennurbs.org>.
//
////////////////////////////////////////////////////////////////
*/

#if !defined(ON_PLANE_INC_)
#define ON_PLANE_INC_

class ON_CLASS ON_Plane
{
public:
  ON_Plane(); // creates world XY plane with origin at 0,0,0

  ON_Plane(
    const ON_3dPoint&,  // point on the plane
    const ON_3dVector&  // non-zero normal to the plane
    );

  ON_Plane(
    const ON_3dPoint&,  // point on the plane
    const ON_3dVector&, // non-zero vector in plane
    const ON_3dVector&  // another vector in the plane not parallel to X
    );

  ON_Plane(
    const ON_3dPoint&,  // point on the plane
    const ON_3dPoint&,  // point on the plane
    const ON_3dPoint&   // point on the plane
    );

  ON_Plane(
    const double[4]     // equation of plane
    );

  ~ON_Plane();

  BOOL CreateFromNormal(
    const ON_3dPoint&,  // point on the plane
    const ON_3dVector&  // non-zero normal to the plane
    );

  BOOL CreateFromFrame(
    const ON_3dPoint&,  // point on the plane
    const ON_3dVector&, // non-zero vector in plane
    const ON_3dVector&  // another vector in the plane not parallel to X
    );

  BOOL CreateFromPoints(
    const ON_3dPoint&,  // point on the plane
    const ON_3dPoint&,  // point on the plane
    const ON_3dPoint&   // point on the plane
    );

  BOOL CreateFromEquation( 
    const double[4]  // equation of plane
    );

  BOOL IsValid() const; // returns TRUE if all fields contain reasonable
                        // information and equation jibes with point and Z.

  const ON_3dPoint& Origin() const;
  const ON_3dVector& Xaxis() const;
  const ON_3dVector& Yaxis() const;
  const ON_3dVector& Normal() const;

  // equation of plane: e[0]*X + e[1]*Y + e[2]*Z + e[3] = 0
  const double* Equation() const; 

  // evaluate parameters and return point
  ON_3dPoint PointAt( // returns origin + a*xaxis + b*yaxis
    double, double // a, b
    ) const;
  ON_3dPoint PointAt( // returns origin + a*xaxis + b*yaxis + c*zaxis
    double, double, double // a, b, c
    ) const;

  ON_Line IsoLine( // iso parametric line
         int,              // 0 first parameter varies and second parameter is constant
                           //   e.g., line(t) = plane(t,c)
                           // 1 first parameter is constant and second parameter varies
                           //   e.g., line(t) = plane(c,t)
         double            // c = value of constant parameter 
         );

  // evaluate plane's implicit equation
  double DistanceTo( const ON_3dPoint& ) const; // signed distance
  double EquationAt( const ON_3dPoint& ) const;
  double EquationAt( const ON_4dPoint& ) const;

  BOOL UpdateEquation();  // computes equation[] from origin and zaxis.

  // returns parameters of point on plane that is closest to given point
  BOOL ClosestPointTo( 
         ON_3dPoint, 
         double*, double*
         ) const;
  // returns point on plane that is closest to given point
  ON_3dPoint ClosestPointTo( 
         ON_3dPoint
         ) const;

  // For intersections see ON_Intersect();

  BOOL Transform( const ON_Xform& );
  BOOL SwapCoordinates(int,int);

  // rotate plane about its origin
  BOOL Rotate(
        double,               // sin(angle)
        double,               // cos(angle)
        const ON_3dVector&  // axis of rotation
        );

  BOOL Rotate(
        double,               // angle in radians
        const ON_3dVector&  // axis of rotation
        );

  // rotate plane about a point and axis
  BOOL Rotate(
        double,               // sin(angle)
        double,               // cos(angle)
        const ON_3dVector&, // axis of rotation
        const ON_3dPoint&   // center of rotation
        );
  BOOL Rotate(
        double,              // angle in radians
        const ON_3dVector&, // axis of rotation
        const ON_3dPoint&   // center of rotation
        );

  BOOL Translate(
        const ON_3dVector&
        );

  BOOL Flip();

public: // members left public
  ON_3dPoint  origin;
  ON_3dVector xaxis;  // unit X
  ON_3dVector yaxis;  // unit Y
  ON_3dVector zaxis;  // unit normal 

  double equation[4];      // equation of plane
};

ON_EXTERN_DECL const ON_Plane ON_xy_plane;
ON_EXTERN_DECL const ON_Plane ON_yz_plane;
ON_EXTERN_DECL const ON_Plane ON_zx_plane;



#endif
