/* $Header: /src2/opennurbs/opennurbs_polyline.h 5     1/03/01 4:59p Dalelear $ */
/* $NoKeywords: $ */
/*
//
// Copyright (c) 1993-2001 Robert McNeel & Associates. All rights reserved.
// Rhinoceros is a registered trademark of Robert McNeel & Assoicates.
//
// THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
// ALL IMPLIED WARRANTIES OF FITNESS FOR ANY PARTICULAR PURPOSE AND OF
// MERCHANTABILITY ARE HEREBY DISCLAIMED.
//				
// For complete openNURBS copyright information see <http://www.opennurbs.org>.
//
////////////////////////////////////////////////////////////////
*/

#if !defined(ON_POLYLINE_INC_)
#define ON_POLYLINE_INC_

class ON_CLASS ON_Polyline
{
public:
  ON_Polyline();
  ON_Polyline(const ON_Polyline&);
  ~ON_Polyline();

  void EmergencyDestroy(); // call if memory used by ON_Polyline becomes invalid

  ON_Polyline& operator=(const ON_Polyline&);

  int PointCount() const; // number of points in polyline
  int SegmentCount() const; // number of segments in polyline

  // polyline[0] = start point
  ON_3dPoint& operator[](int);
  const ON_3dPoint& operator[](int) const;

  double Length() const;
  ON_3dVector Direction
    ( int // segment index
    ) const;
  ON_3dVector Tangent // returns unit vector
    ( int // segment index
    ) const;
  ON_3dPoint PointAt( double ) const; // i-th segment goes from i to i+1

  BOOL ClosestPointTo( const ON_3dPoint&, double* ) const;
  BOOL ClosestPointTo( const ON_3dPoint&, double*, 
                       int, // index of segment where search begins
                       int  // index + 1 of segment where search stops
                       ) const;
  ON_3dPoint ClosestPointTo( const ON_3dPoint& ) const;

  BOOL Transform( const ON_Xform& );

  BOOL SwapCoordinates(int,int);

  // rotate line about a point and axis
  BOOL Rotate(
        double,               // sin(angle)
        double,               // cos(angle)
        const ON_3dVector&, // axis of rotation
        const ON_3dPoint&   // center of rotation
        );
  BOOL Rotate(
        double,              // angle in radians
        const ON_3dVector&, // axis of rotation
        const ON_3dPoint&   // center of rotation
        );

  BOOL Translate(
        const ON_3dVector&
        );

public:
  ON_3dPointArray m_point;
};

#endif
