/* $Header: /src2/opennurbs/opennurbs_polylinecurve.cpp 15    4/27/01 10:21a Dalelear $ */
/* $NoKeywords: $ */
/*
//
// Copyright (c) 1993-2001 Robert McNeel & Associates. All rights reserved.
// Rhinoceros is a registered trademark of Robert McNeel & Assoicates.
//
// THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
// ALL IMPLIED WARRANTIES OF FITNESS FOR ANY PARTICULAR PURPOSE AND OF
// MERCHANTABILITY ARE HEREBY DISCLAIMED.
//				
// For complete openNURBS copyright information see <http://www.opennurbs.org>.
//
////////////////////////////////////////////////////////////////
*/

#include "opennurbs.h"

ON_OBJECT_IMPLEMENT(ON_PolylineCurve,ON_Curve,"4ED7D4E6-E947-11d3-BFE5-0010830122F0");

ON_PolylineCurve::ON_PolylineCurve()
{
  m_dim = 3;
}

ON_PolylineCurve::ON_PolylineCurve( const ON_PolylineCurve& L )
{
  *this = L;
}

ON_PolylineCurve::ON_PolylineCurve( const ON_Polyline& L )
{
  *this = L;
}

ON_PolylineCurve::~ON_PolylineCurve()
{
}

void ON_PolylineCurve::EmergencyDestroy()
{
  m_pline.EmergencyDestroy();
  m_t.EmergencyDestroy();
}

ON_PolylineCurve& ON_PolylineCurve::operator=( const ON_PolylineCurve& src )
{
  if ( this != &src ) {
    ON_Curve::operator=(src);
    m_pline = src.m_pline;
    m_t     = src.m_t;
    m_dim   = src.m_dim;
  }
  return *this;
}

ON_PolylineCurve& ON_PolylineCurve::operator=( const ON_Polyline& src )
{
  m_pline = src;
  m_dim   = 3;
  const int count = src.PointCount();
  m_t.Reserve(count);
  m_t.SetCount(count);
  int i;
  for (i = 0; i < count; i++) {
    m_t[i] = (double)i;
  }
  return *this;
}

int ON_PolylineCurve::Dimension() const
{
  return m_dim;
}

BOOL 
ON_PolylineCurve::GetBBox( // returns TRUE if successful
         double* boxmin,    // minimum
         double* boxmax,    // maximum
         BOOL bGrowBox
         ) const
{
  return ON_GetPointListBoundingBox( m_dim, FALSE, PointCount(), 3, m_pline.m_point[0], 
                                      boxmin, boxmax, bGrowBox );
}

BOOL
ON_PolylineCurve::Transform( const ON_Xform& xform )
{
  TransformUserData(xform);
  return m_pline.Transform( xform );
}

BOOL
ON_PolylineCurve::SwapCoordinates( int i, int j )
{
  return m_pline.SwapCoordinates(i,j);
}

BOOL ON_PolylineCurve::IsValid() const
{
  BOOL rc = FALSE;
  const int count = PointCount();
  if ( count >= 2 && count == m_t.Count() ) {
    rc = TRUE;
    for ( int i = 1; rc && i < count; i++ ) {
      if ( m_t[i] <= m_t[i-1] )
        rc = FALSE;
    }
  }
  return rc;
}

void ON_PolylineCurve::Dump( ON_TextLog& dump ) const
{
  dump.Print( "ON_PolylineCurve:  domain = [%g,%g]\n",m_t[0],m_t[1]);
  for ( int i = 0; i < PointCount(); i++ ) {
    dump.Print( "  point[%2d] = ",i);
    dump.Print( m_pline.m_point[i] );
    dump.Print( ", %g\n",m_t[i]);
  }
}

BOOL ON_PolylineCurve::Write(
       ON_BinaryArchive&  // open binary file
     ) const
{
  // TODO
  return FALSE;
}

BOOL ON_PolylineCurve::Read(
       ON_BinaryArchive&  // open binary file
     )
{
  // TODO
  return FALSE;
}

ON_Interval ON_PolylineCurve::Domain() const
{
  ON_Interval d;
  //BOOL rc = FALSE;
  const int count = PointCount();
  if ( count >= 2 && m_t[0] < m_t[count-1] ) {
    d.Set(m_t[0],m_t[count-1]);
  }
  return d;
}

int ON_PolylineCurve::SpanCount() const
{
  return m_pline.SegmentCount();
}

BOOL ON_PolylineCurve::GetSpanVector( // span "knots" 
       double* s // array of length SpanCount() + 1 
       ) const
{
  BOOL rc = FALSE;
  const int count = PointCount();
  if ( count >= 1 ) {
    memcpy( s, m_t, count*sizeof(*s) );
    rc = TRUE;
  }
  return rc;
}

int ON_PolylineCurve::Degree() const
{
  return 1;
}


BOOL
ON_PolylineCurve::IsLinear( // TRUE if curve locus is a line segment
      double tolerance // tolerance to use when checking linearity
      ) const
{
  return ON_Curve::IsLinear(tolerance);
}

BOOL
ON_PolylineCurve::IsArc( // TRUE if curve locus in an arc or circle
      const ON_Plane* plane, // if not NULL, test is performed in this plane
      ON_Arc* arc,         // if not NULL and TRUE is returned, then arc
                              // arc parameters are filled in
      double tolerance // tolerance to use when checking linearity
      ) const
{
  return FALSE;
}

BOOL
ON_PolylineCurve::IsPlanar(
      ON_Plane* plane, // if not NULL and TRUE is returned, then plane parameters
                         // are filled in
      double tolerance // tolerance to use when checking linearity
      ) const
{
  return FALSE;
}

BOOL
ON_PolylineCurve::IsInPlane(
      const ON_Plane& plane, // plane to test
      double tolerance // tolerance to use when checking linearity
      ) const
{
  return FALSE;
}

BOOL 
ON_PolylineCurve::IsClosed() const
{
  BOOL rc = FALSE;
  const int count = PointCount();
  if ( count >= 4 && 0 == ON_ComparePoint( m_dim, FALSE, m_pline.m_point[0], m_pline.m_point[count-1] ) ) {
    int i1, i2;
    for ( i1 = 1; i1 < count; i1++ ) {
      if ( ON_ComparePoint(m_dim,FALSE,m_pline.m_point[0],m_pline.m_point[i1]) ) {
        for ( i2 = i1+1; i2 < count-1; i2++ ) {
          if ( ON_ComparePoint(m_dim,FALSE,m_pline.m_point[i2],m_pline.m_point[count-1]) ) {
            rc = TRUE;
            break;
          }
        }
        break;
      }
    }
  }
  return rc;
}

BOOL 
ON_PolylineCurve::IsPeriodic() const
{
  return FALSE;
}

BOOL
ON_PolylineCurve::Reverse()
{
  BOOL rc = FALSE;
  const int count = PointCount();
  if ( count >= 2 ) {
    m_pline.m_point.Reverse();
    m_t.Reverse();
    double* t = m_t.Array();
    for ( int i = 0; i < count; i++ ) {
      t[i] = -t[i];
    }
    rc = TRUE;
  }
  return rc;
}

BOOL 
ON_PolylineCurve::Evaluate( // returns FALSE if unable to evaluate
       double t,       // evaluation parameter
       int der_count,  // number of derivatives (>=0)
       int v_stride,   // v[] array stride (>=Dimension())
       double* v,      // v[] array of length stride*(ndir+1)
       int side,       // optional - determines which side to evaluate from
                       //         0 = default
                       //      <  0 to evaluate from below, 
                       //      >  0 to evaluate from above
       int* hint       // optional - evaluation hint (int) used to speed
                       //            interative evaluations
       ) const
{
  BOOL rc = FALSE;
  const int count = PointCount();
  if ( count >= 2 ) {
    const int segment_index = ON_SpanIndex(2,count,m_t,t,side,(hint)?*hint:0);
    const double t0 = m_t[segment_index];
    const double t1 = m_t[segment_index+1];
    double s = (t == t1) ? 1.0 : (t-t0)/(t1-t0);
    const ON_3dPoint p = (1.0-s)*m_pline.m_point[segment_index] + s*m_pline.m_point[segment_index+1];
    v[0] = p.x;
    v[1] = p.y;
    if ( m_dim == 3 )
      v[2] = p.z;
    if ( der_count >= 1 ) {
      v += v_stride;
      ON_3dVector d = 1.0/(t1-t0)*(m_pline.m_point[segment_index+1] - m_pline.m_point[segment_index]);
      v[0] = d.x;
      v[1] = d.y;
      if ( m_dim == 3 )
        v[2] = d.z;
    }
    for ( int di = 2; di <= der_count; di++ ) {
      v += v_stride;
      v[0] = 0.0;
      v[1] = 0.0;
      if ( m_dim == 3 )
        v[2] = 0.0;
    }
    if ( hint )
      *hint = segment_index;
    rc = TRUE;
  }
  return rc;
}

BOOL 
ON_PolylineCurve::PointCount() const
{
  return m_pline.PointCount();
}

BOOL ON_PolylineCurve::GetClosestPoint( const ON_3dPoint& test_point,
        double* t,       // parameter of local closest point returned here
        double maximum_distance,
        const ON_Interval* sub_domain
        ) const
{
  double s, d;
  int segment_index0 = 0;
  int segment_index1 = m_pline.SegmentCount();
  if ( sub_domain ) {
    segment_index0 = ON_SpanIndex(2,m_pline.PointCount(),m_t,sub_domain->Min(),1,0);
    segment_index1 = ON_SpanIndex(2,m_pline.PointCount(),m_t,sub_domain->Max(),-1,0)+1;
  }
  BOOL rc = m_pline.ClosestPointTo( test_point, &s, segment_index0, segment_index1 );
  if ( rc ) {
    int i = (int)floor(s);
    if ( i < 0 )
      i = 0;
    else if ( i > m_pline.PointCount() )
      i = m_pline.PointCount();
    ON_Interval in(m_t[i],m_t[i+1]);
    s = in.ParameterAt(s);
    if ( sub_domain ) {
      if ( s < sub_domain->Min() )
        s = sub_domain->Min();
      else if ( s > sub_domain->Max() )
        s = sub_domain->Max();
    }
    if ( maximum_distance > 0.0 ) {
      d = test_point.DistanceTo(PointAt(s));
      if ( d > maximum_distance )
        rc = FALSE;
    }
    if (rc && t)
      *t = s;
  }
  return rc;
}

BOOL ON_PolylineCurve::GetLocalClosestPoint( const ON_3dPoint& test_point,
        double seed_parameter,
        double* t,
        const ON_Interval* sub_domain
        ) const
{
  // TODO: this one requires a real local search instead of this quick fix
  // that works for lines
  return GetClosestPoint( test_point, t, 0.0, sub_domain );
}

BOOL ON_PolylineCurve::Length(
        double* length,               // length returned here
        double fractional_tolerance,  // default = 1.0e-8
        const ON_Interval* sub_domain // default = NULL
        ) const
{
  // override when possible
  // TODO: add simple integration routine that works with C1 curves
  if ( sub_domain ) {
    *length = 0.0;
    int i, count = m_t.Count();
    double s0 = sub_domain->Min();
    double s1 = sub_domain->Max();
    ON_3dPoint p0, p1;
    if ( s0 < m_t[0] )
      s0 = m_t[0];
    if ( s1 > m_t[count-1] )
      s1 = m_t[count-1];
    p1 = m_pline.m_point[0];
    for ( i = 1; i < count; i++ ) {
      if ( s0 < m_t[i] ) {
        p1 = PointAt(s0);
        break;
      }
    }
    for (/*empty*/; i < count; i++ ) {
      p0 = p1;
      if ( s1 < m_t[i] ) {
        p1 = PointAt(s1);
        *length += p0.DistanceTo(p1);
        break;
      }
      else {
        p1 = m_pline.m_point[i];
        *length += p0.DistanceTo(m_pline.m_point[i]);
      }
    }
  }
  else {
    *length = m_pline.Length();
  }
  return TRUE;
}

int ON_PolylineCurve::GetNurbForm( 
                                  ON_NurbsCurve& nurb, 
                                  double tol,
                                  const ON_Interval* subdomain  // OPTIONAL subdomain of ON::ProxyCurve::Domain()
                                  ) const
{
  int rc = 0;
  const int count = PointCount();
  if ( count < 2 )
    nurb.Destroy();
  else  if ( nurb.Create( Dimension(), FALSE, 2, count) ) {
    int i;
    for ( i = 0; i < count; i++ ) {
      nurb.SetKnot( i, m_t[i] );
      nurb.SetCV( i, m_pline[i] );
    }
    if ( nurb.IsValid() )
      rc = 1;
  }
  return rc;
}
