/* $Header: /src2/opennurbs/opennurbs_sphere.h 5     1/03/01 4:59p Dalelear $ */
/* $NoKeywords: $ */
/*
//
// Copyright (c) 1993-2001 Robert McNeel & Associates. All rights reserved.
// Rhinoceros is a registered trademark of Robert McNeel & Assoicates.
//
// THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
// ALL IMPLIED WARRANTIES OF FITNESS FOR ANY PARTICULAR PURPOSE AND OF
// MERCHANTABILITY ARE HEREBY DISCLAIMED.
//				
// For complete openNURBS copyright information see <http://www.opennurbs.org>.
//
////////////////////////////////////////////////////////////////
*/

#if !defined(ON_SPHERE_INC_)
#define ON_SPHERE_INC_

class ON_CLASS ON_Sphere
{
public:
  
  ON_Plane plane; // equitorial plane
  double radius;  // > 0

  ON_Sphere();
  ON_Sphere( const ON_3dPoint&, double ); // center, radius
  ~ON_Sphere();

  BOOL IsValid() const;

  BOOL Create( const ON_3dPoint&, double ); // center radius

  ON_Circle LatitudeRadians(double) const;
  ON_Circle LatitudeDegrees(double) const;
  ON_Circle LongitudeRadians(double) const;
  ON_Circle LongitudeDegrees(double) const;

  ON_3dPoint Center() const;
  ON_3dPoint NorthPole() const;
  ON_3dPoint SouthPole() const;
  double Diameter() const;
  double Radius() const;

  ON_3dPoint PointAt(double,double) const;   // longitude [0,2pi], latitude [-pi/2,pi/2] in radians
  ON_3dVector NormalAt(double,double) const; // longitude, latitude in radians

  // returns parameters of point on sphere that is closest to given point
  BOOL ClosestPointTo( 
         ON_3dPoint, 
         double*, // longitude  [0,2pi)
         double*  // latitude   [-pi/2,pi/2]
         ) const;
  // returns point on sphere that is closest to given point
  ON_3dPoint ClosestPointTo( 
         ON_3dPoint 
         ) const;

  // For intersections see ON_Intersect();

  // rotate sphere about its origin
  BOOL Rotate(
        double,               // sin(angle)
        double,               // cos(angle)
        const ON_3dVector&  // axis of rotation
        );
  BOOL Rotate(
        double,               // angle in radians
        const ON_3dVector&  // axis of rotation
        );

  // rotate sphere about a point and axis
  BOOL Rotate(
        double,               // sin(angle)
        double,               // cos(angle)
        const ON_3dVector&, // axis of rotation
        const ON_3dPoint&   // center of rotation
        );
  BOOL Rotate(
        double,              // angle in radians
        const ON_3dVector&, // axis of rotation
        const ON_3dPoint&   // center of rotation
        );

  BOOL Translate(
        const ON_3dVector&
        );

  BOOL Transform( const ON_Xform& );

  // parameterization of NURBS surface does not match sphere's transcendental paramaterization
  int GetNurbForm( ON_NurbsSurface& ) const; // returns 0=failure, 2=success

};

#endif
