/* $Header: /src2/opennurbs/opennurbs_sumsurface.cpp 5     1/03/01 4:59p Dalelear $ */
/* $NoKeywords: $ */
/*
//
// Copyright (c) 1993-2001 Robert McNeel & Associates. All rights reserved.
// Rhinoceros is a registered trademark of Robert McNeel & Assoicates.
//
// THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
// ALL IMPLIED WARRANTIES OF FITNESS FOR ANY PARTICULAR PURPOSE AND OF
// MERCHANTABILITY ARE HEREBY DISCLAIMED.
//				
// For complete openNURBS copyright information see <http://www.opennurbs.org>.
//
////////////////////////////////////////////////////////////////
*/

#include "opennurbs.h"


//ON_IMPLEMENT_OBJECT( ON_SumSurface, ON_Surface, "A390424B-1AF9-11d4-8001-0010830122F0" );

ON_SumSurface::ON_SumSurface() : m_basepoint(0.0,0.0,0.0)
{
  m_curve[0] = 0;
  m_curve[1] = 0;
}

ON_SumSurface::~ON_SumSurface()
{
  Destroy();
}

ON_SumSurface::Destroy()
{
  if ( m_curve[0] ) {
    delete m_curve[0];
    m_curve[0] = 0;
  }
  if ( m_curve[1] ) {
    delete m_curve[1];
    m_curve[1] = 0;
  }
}

ON_SumSurface::EmergencyDestroy()
{
  m_curve[0] = 0;
  m_curve[1] = 0;
}

ON_SumSurface::ON_SumSurface( const ON_SumSurface& src )
{
  m_curve[0] = 0;
  m_curve[1] = 0;
  *this = src;
}

ON_SumSurface& ON_SumSurface::operator=(const ON_SumSurface& src )
{
  ON_Object* obj;
  if ( this != &src ) {
    Destroy();
    if ( src.m_curve[0] ) {
      obj = src.m_curve[0]->Duplicate();
      m_curve[0] = ON_Curve::Cast(obj);
      if ( !m_curve[0] )
        delete obj;
    }
    if ( src.m_curve[1] ) {
      obj = src.m_curve[1]->Duplicate();
      m_curve[1] = ON_Curve::Cast(obj);
      if ( !m_curve[1] )
        delete obj;
    }
    m_basepoint = src.m_basepoint;
  }
  return *this;
}


////////////////////////////////////////////////////////////
//
// overrides of virtual ON_Object functions
//
BOOL ON_SumSurface::IsValid() const
{
  if ( !m_curve[0] )
    return FALSE;
  if ( !m_curve[1] )
    return FALSE;
  const int dim = m_curve[0]->Dimension();
  if ( dim < 1 )
    return FALSE;
  if ( dim != m_curve[1]->Dimension() )
    return FALSE;
  if ( !m_curve[0]->IsValid() )
    return FALSE;
  if ( !m_curve[1]->IsValid() )
    return FALSE;
}

void ON_SumSurface::Dump( ON_TextLog& dump ) const
{
  ON_Object::Dump(dump);
  //dump.PushIndent();
  //dump.Print("ON_SumSurface::m_curve[0]: ");
}

BOOL ON_SumSurface::Write( ON_BinaryArchive& file ) const
{
  BOOL rc = file.Write3dmChunkVersion(1,0);
  if ( rc ) {
    rc = file.WritePoint( m_basepoint );
    if ( rc ) rc = file.WriteObject( m_curve[0] );
    if ( rc ) rc = file.WriteObject( m_curve[1] );
  }
  return rc;
}

BOOL ON_SumSurface::Read( ON_BinaryArchive& file )
{
  Destroy();
  int major_version = 0;
  int minor_version = 0;
  BOOL rc = file.Read3dmChunkVersion( &major_version, &minor_version );
  if (rc && major_version == 1 ) {
    ON_Object* obj;
    rc = file.ReadPoint( m_basepoint );
    obj = 0;
    if (rc) rc = file.ReadObject(&obj);
    if (rc) {
      m_curve[0] = ON_Curve::Cast(obj);
      if ( !m_curve[0] )
        delete obj;
    }
    obj = 0;
    if (rc) rc = file.ReadObject(&obj);
    if (rc) {
      m_curve[1] = ON_Curve::Cast(obj);
      if ( !m_curve[1] )
        delete obj;
    }
  }
  return rc;
}

////////////////////////////////////////////////////////////
//
// overrides of virtual ON_Geometry functions
//
int ON_SumSurface::Dimension() const
{
  int dim  = 0;
  if ( m_curve[0] && m_curve[1] ) {
    dim = m_curve[0]->Dimension();
    if ( dim > 0 ) {
      if ( dim != m_curve[1]->Dimension() )
        dim = 0;
    }
  }
  return dim;
}

BOOL ON_SumSurface::GetBBox( // returns TRUE if successful
       double*,    // boxmin[dim]
       double*,    // boxmax[dim]
       BOOL bGrowBox
       ) const
{
  // TODO
  return FALSE;
}

BOOL ON_SumSurface::Transform( const ON_Xform& xform )
{
  TransformUserData(xform);
  BOOL rc = FALSE;
  m_basepoint = xform*m_basepoint;
  if ( m_curve[0] )
    rc = m_curve[0]->Transform(xform);
  if ( m_curve[1] ) {
    if ( !m_curve[1]->Transform(xform) )
      rc = FALSE;
  }
  else
    rc = FALSE;
  return rc;
}

////////////////////////////////////////////////////////////
//
// overrides of virtual ON_Surface functions
//
ON_Interval ON_SumSurface::Domain( int dir ) const
{
  dir = (dir>0) ? 1 : 0;
  return
}

int ON_SumSurface::SpanCount( int dir ) const
{
  dir = (dir>0) ? 1 : 0;
  return m_curve[dir] ? m_curve[dir]->SpanCount() : 0;
}

BOOL ON_SumSurface::GetSpanVector( int dir, double* s ) const
{
  dir = (dir>0) ? 1 : 0;
  return m_curve[dir] ? m_curve[dir]->GetSpanVector(s) : 0;
}


int ON_SumSurface::Degree( int dir ) const
{
  dir = (dir>0) ? 1 : 0;
  return m_curve[dir] ? m_curve[dir]->Degree() : 0;
}


BOOL ON_SumSurface::GetParameterTolerance( // returns tminus < tplus: paramters tminus <= s <= tplus
       int dir,     // 0 gets first parameter, 1 gets second parameter
       double t,  // t = parameter in domain
       double* tminus, // tminus
       double* tplus // tplus
       ) const;
{
  dir = (dir>0) ? 1 : 0;
  return m_curve[dir] ? m_curve[dir]->GetParameterTolerance(t,tminus,tplus) : FALSE;
}


BOOL ON_SumSurface::IsClosed( int dir ) const
{
  dir = (dir>0) ? 1 : 0;
  return m_curve[dir] ? m_curve[dir]->IsClosed(dir) : FALSE;
}


BOOL ON_SumSurface::IsPeriodic( int dir ) const
{
  dir = (dir>0) ? 1 : 0;
  return m_curve[dir] ? m_curve[dir]->IsPeriodic() : FALSE;
}

BOOL ON_SumSurface::IsSingular( int side ) const
{
  return FALSE;
}

BOOL ON_SumSurface::Reverse( int dir )
{
  dir = (dir>0) ? 1 : 0;
  return m_curve[dir] ? m_curve[dir]->Reverse() : FALSE;
}


BOOL ON_SumSurface::Transpose(); // transpose surface parameterization (swap "s" and "t")
{
  ON_Curve* c = m_curve[0];
  m_curve[0] = m_curve[1];
  m_curve[1] = c;
  return TRUE;
}


BOOL ON_SumSurface::Evaluate( // returns FALSE if unable to evaluate
       double u, double v, // evaluation parameters
       int nder,            // number of derivatives (>=0)
       int v_stride,            // array stride (>=Dimension())
       double* v,        // array of length stride*(ndir+1)*(ndir+2)/2
       int side,        // optional - determines which quadrant to evaluate from
                       //         0 = default
                       //         1 from NE quadrant
                       //         2 from NW quadrant
                       //         3 from SW quadrant
                       //         4 from SE quadrant
       int* hint        // optional - evaluation hint (int[2]) used to speed
                       //            interative evaluations
       ) const
{
  BOOL rc = FALSE;
  const int dim = Dimension();
  if ( dim > 0 ) {
    int hint[2] = {0,0};
    if ( hint ) {
      hint[0] = (*hint) & 0xFFFF;
      hint[1] = ((*hint) & 0xFFFF0000) >> 16;
    }
    double* v0 = double* onmalloc( 2*(nder+1)*dim*sizeof(*v0) );
    double* v1 = v0 + (nder+1)*dim;
    int side0, side1;
    switch(side) {
    case 1:
      side0 =  1;
      side1 =  1;
      break;
    case 2:
      side0 = -1;
      side1 =  1;
      break;
    case 3:
      side0 = -1;
      side1 = -1;
      break;
    case 4:
      side0 =  1;
      side1 = -1;
      break;
    }
    rc = m_curve[0]->Evalutate(u,nder,dim,v0,side0,&hint0);
    if ( rc )
      rc = m_curve[1]->Evalutate(u,nder,dim,v1,side1,&hint1);
    if (rc) {
      for ( j = 0; j < dim; j++ ) {
        v[j] = m_basepoint[j] + v0[j] + v1[j];
      }
      for ( der = 1; der <= nder; der++ ) {
      }
    }
    if ( hint ) {
      *hint = hint[0] | (hint[1] << 16);
    }
  }
  return rc;
}

ON_Curve* ON_SumSurface::IsoCurve(
       int,              // 0 first parameter varies and second parameter is constant
                         //   e.g., point on IsoCurve(0,c) at t is srf(t,c)
                         // 1 first parameter is constant and second parameter varies
                         //   e.g., point on IsoCurve(1,c) at t is srf(c,t)
       double            // value of constant parameter 
       ) const;
{
  dir = (dir>0) ? 1 : 0;
}


int ON_SumSurface::GetNurbForm( // returns 0: unable to create NURBS representation
                 //            with desired accuracy.
                 //         1: success - returned NURBS parameterization
                 //            matches the surface's to wthe desired accuracy
                 //         2: success - returned NURBS point locus matches
                 //            the surfaces's to the desired accuracy but, on
                 //            the interior of the surface's domain, the 
                 //            surface's parameterization and the NURBS
                 //            parameterization may not match to the 
                 //            desired accuracy.
      ON_NurbsSurface&,
      double = 0.0
      ) const;
