/* $Header: /src2/opennurbs/opennurbs_surfaceproxy.cpp 7     1/03/01 4:59p Dalelear $ */
/* $NoKeywords: $ */
/*
//
// Copyright (c) 1993-2001 Robert McNeel & Associates. All rights reserved.
// Rhinoceros is a registered trademark of Robert McNeel & Assoicates.
//
// THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
// ALL IMPLIED WARRANTIES OF FITNESS FOR ANY PARTICULAR PURPOSE AND OF
// MERCHANTABILITY ARE HEREBY DISCLAIMED.
//				
// For complete openNURBS copyright information see <http://www.opennurbs.org>.
//
////////////////////////////////////////////////////////////////
*/

#include "opennurbs.h"

ON_OBJECT_IMPLEMENT(ON_SurfaceProxy,ON_Surface,"4ED7D4E2-E947-11d3-BFE5-0010830122F0");

ON_SurfaceProxy::ON_SurfaceProxy() : m_surface(0), m_bTransposed(0)
{}

ON_SurfaceProxy::ON_SurfaceProxy( const ON_Surface* s ) : m_surface(s), m_bTransposed(0)
{}

ON_SurfaceProxy::ON_SurfaceProxy( const ON_SurfaceProxy& src ) : ON_Surface(src), m_surface(0), m_bTransposed(0)
{
  *this = src;
}

ON_SurfaceProxy& ON_SurfaceProxy::operator=( const ON_SurfaceProxy& src )
{
  if ( this != &src ) {
    ON_Surface::operator=(src);
    m_surface = src.m_surface;
    m_bTransposed = src.m_bTransposed;
  }
  return *this;
}

ON_SurfaceProxy::~ON_SurfaceProxy()
{
  m_surface = 0;
}

BOOL
ON_SurfaceProxy::IsValid() const
{
  return ( m_surface ) ? m_surface->IsValid() : FALSE;
}

void
ON_SurfaceProxy::Dump( ON_TextLog& dump ) const
{
  dump.Print("ON_SurfaceProxy uses %x\n",m_surface);
  if (m_surface ) 
    m_surface->Dump(dump);
}

BOOL 
ON_SurfaceProxy::Write(
       ON_BinaryArchive&  // open binary file
     ) const
{
  return FALSE;
}

BOOL 
ON_SurfaceProxy::Read(
       ON_BinaryArchive&  // open binary file
     )
{
  return FALSE;
}

int 
ON_SurfaceProxy::Dimension() const
{
  return ( m_surface ) ? m_surface->Dimension() : 0;
}

BOOL 
ON_SurfaceProxy::GetBBox( // returns TRUE if successful
         double* boxmin,    // minimum
         double* boxmax,    // maximum
         BOOL bGrowBox
         ) const
{
  return ( m_surface ) ? m_surface->GetBBox(boxmin,boxmax,bGrowBox) : FALSE;
}

BOOL
ON_SurfaceProxy::Transform( const ON_Xform& xform )
{
  return FALSE; // cannot modify m_surface
}

ON_Interval
ON_SurfaceProxy::Domain( int dir ) const
{
  ON_Interval d;
  if ( m_bTransposed ) {
    dir = (dir) ? 0 : 1;
  }
  if ( m_surface ) 
    d = m_surface->Domain(dir);
  return d;
}


int
ON_SurfaceProxy::SpanCount( int dir ) const
{
  if ( m_bTransposed ) {
    dir = (dir) ? 0 : 1;
  }
  return ( m_surface ) ? m_surface->SpanCount(dir) : FALSE;
}

BOOL
ON_SurfaceProxy::GetSpanVector( int dir, double* s ) const
{
  if ( m_bTransposed ) {
    dir = (dir) ? 0 : 1;
  }
  return ( m_surface ) ? m_surface->GetSpanVector(dir,s) : FALSE;
}

int
ON_SurfaceProxy::Degree( int dir ) const
{
  if ( m_bTransposed ) {
    dir = (dir) ? 0 : 1;
  }
  return ( m_surface ) ? m_surface->Degree(dir) : FALSE;
}


BOOL 
ON_SurfaceProxy::GetParameterTolerance(
         int dir,
         double t,  // t = parameter in domain
         double* tminus, // tminus
         double* tplus   // tplus
         ) const
{
  if ( m_bTransposed ) {
    dir = (dir) ? 0 : 1;
  }
  return ( m_surface ) ? m_surface->GetParameterTolerance(dir,t,tminus,tplus) : FALSE;
}

BOOL 
ON_SurfaceProxy::IsClosed( int dir ) const
{
  if ( m_bTransposed ) {
    dir = (dir) ? 0 : 1;
  }
  return ( m_surface ) ? m_surface->IsClosed( dir ) : FALSE;
}

BOOL 
ON_SurfaceProxy::IsPeriodic( int dir ) const
{
  if ( m_bTransposed ) {
    dir = (dir) ? 0 : 1;
  }
  return ( m_surface ) ? m_surface->IsPeriodic( dir ) : FALSE;
}

BOOL 
ON_SurfaceProxy::IsSingular( int side ) const
{
  if ( m_bTransposed ) {
    switch(side) {
    case 0:
      side = 3;
      break;
    case 1:
      side = 2;
      break;
    case 2:
      side = 1;
      break;
    case 3:
      side = 0;
      break;
    }
  }
  return ( m_surface ) ? m_surface->IsSingular( side ) : FALSE;
}

BOOL
ON_SurfaceProxy::Reverse( int dir )
{
  return FALSE; // cannot modify m_surface
}

BOOL
ON_SurfaceProxy::Transpose()
{
  m_bTransposed = (m_bTransposed) ? FALSE : TRUE;
  return TRUE;
}

BOOL 
ON_SurfaceProxy::Evaluate( // returns FALSE if unable to evaluate
       double s, double t, // evaluation parameters
       int der_count,  // number of derivatives (>=0)
       int v_stride,   // v[] array stride (>=Dimension())
       double* v,      // v[] array of length stride*(ndir+1)
       int side,       // optional - determines which side to evaluate from
                       //         0 = default
                       //      <  0 to evaluate from below, 
                       //      >  0 to evaluate from above
       int* hint       // optional - evaluation hint (int) used to speed
                       //            interative evaluations
       ) const
{
  if ( m_bTransposed ) {
    double x = s; s = t; t = x;
  }
  return ( m_surface ) ? m_surface->Evaluate(s,t,der_count,v_stride,v,side,hint) : FALSE;
}


BOOL ON_SurfaceProxy::GetClosestPoint( const ON_3dPoint& test_point,
        double* s, double* t,
        double maximum_distance,     // maximum_distance
        const ON_Interval* sdomain, // first parameter sub_domain
        const ON_Interval* tdomain  // second parameter sub_domain
        ) const
{
  BOOL rc = FALSE;
  if ( m_surface ) {
    if ( m_bTransposed ) {
      rc = m_surface->GetClosestPoint( test_point, t, s, maximum_distance, 
                                       tdomain, sdomain );
    }
    else {
      rc = m_surface->GetClosestPoint( test_point, s, t, maximum_distance, 
                                       sdomain, tdomain );
    }
  }
  return rc;
}


//////////
// Find parameters of the point on a surface that is locally closest to 
// the test_point.  The search for a local close point starts at 
// seed parameters. If a sub_domain parameter is not NULL, then
// the search is restricted to the specified portion of the surface.
//
// TRUE if returned if the search is successful.  FALSE is returned if
// the search fails.
BOOL ON_SurfaceProxy::GetLocalClosestPoint( const ON_3dPoint& test_point,
        double s0,double t0,     // seed_parameters
        double* s, double* t,   // parameters of local closest point returned here
        const ON_Interval* sdomain, // first parameter sub_domain
        const ON_Interval* tdomain  // second parameter sub_domain
        ) const
{
  BOOL rc = FALSE;
  if ( m_surface ) {
    if ( m_bTransposed ) {
      rc = m_surface->GetLocalClosestPoint( test_point, t0, s0, t, s, 
                                       tdomain, sdomain );
    }
    else {
      rc = m_surface->GetLocalClosestPoint( test_point, s0, t0, s, t, 
                                       sdomain, tdomain );
    }
  }
  return rc;
}


int 
ON_SurfaceProxy::GetNurbForm( // returns 0: unable to create NURBS representation
                   //            with desired accuracy.
                   //         1: success - returned NURBS parameterization
                   //            matches the surface's to wthe desired accuracy
                   //         2: success - returned NURBS point locus matches
                   //            the surfaces's to the desired accuracy but, on
                   //            the interior of the surface's domain, the 
                   //            surface's parameterization and the NURBS
                   //            parameterization may not match to the 
                   //            desired accuracy.
        ON_NurbsSurface& nurbs,
        double tolerance
        ) const
{
  BOOL rc = ( m_surface ) ? m_surface->GetNurbForm(nurbs,tolerance) : FALSE;
  if ( rc && m_bTransposed ) {
    rc = nurbs.Transpose();
  }
  return rc;
}
