/* $Header: /src2/opennurbs/opennurbs_userdata.cpp 9     3/13/01 7:58a Dalelear $ */
/* $NoKeywords: $ */
/*
//
// Copyright (c) 1993-2001 Robert McNeel & Associates. All rights reserved.
// Rhinoceros is a registered trademark of Robert McNeel & Assoicates.
//
// THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
// ALL IMPLIED WARRANTIES OF FITNESS FOR ANY PARTICULAR PURPOSE AND OF
// MERCHANTABILITY ARE HEREBY DISCLAIMED.
//				
// For complete openNURBS copyright information see <http://www.opennurbs.org>.
//
////////////////////////////////////////////////////////////////
*/

#include "opennurbs.h"

ON_VIRTUAL_OBJECT_IMPLEMENT(ON_UserData,ON_Object,"850324A7-050E-11d4-BFFA-0010830122F0");

ON_UserData::ON_UserData()
            : m_userdata_uuid(ON_nil_uuid), 
              m_userdata_copycount(1),
              m_userdata_xform(1),
              m_userdata_owner(0),
              m_userdata_next(0)
{}

ON_UserData::ON_UserData(const ON_UserData& src) 
            : ON_Object(src),
              m_userdata_uuid(ON_nil_uuid), 
              m_userdata_copycount(0),
              m_userdata_xform(1),
              m_userdata_owner(0),
              m_userdata_next(0)
{
  if ( src.m_userdata_copycount) {
    m_userdata_uuid = src.m_userdata_uuid;
    m_userdata_copycount = src.m_userdata_copycount+1;
  }
}

BOOL ON_UserData::Transform(const ON_Xform& x )
{
  m_userdata_xform = x*m_userdata_xform;
  return TRUE;
}

ON_UserData& ON_UserData::operator=(const ON_UserData& src)
{
  if ( this != &src ) {
    ON_Object::operator=(src);
    if ( src.m_userdata_copycount ) {
      m_userdata_uuid = src.m_userdata_uuid;
      m_userdata_copycount = src.m_userdata_copycount+1;
      if ( !m_userdata_copycount )
        m_userdata_copycount = 1;
    }
    else {
      m_userdata_uuid = ON_nil_uuid;
      m_userdata_copycount = 0;
    }
  }
  return *this;
}

ON_UserData::~ON_UserData()
{
  ON_Object* owner = m_userdata_owner;
  if ( owner ) {
    // remove this piece of user data from owner->m_userdata_list
    ON_UserData* prev = 0;
    ON_UserData* p;
    for ( p = owner->m_userdata_list; p; prev = p, p = p->m_userdata_next ) {
      if ( p == this ) {
        if ( prev ) {
          prev->m_userdata_next = p->m_userdata_next;
        }
        else {
          owner->m_userdata_list = p->m_userdata_next;
        }
        p->m_userdata_next = 0;
        p->m_userdata_owner = 0;
        break;
      }
    }
  }
}

BOOL ON_UserData::IsValid() const
{
  return ON_UuidCompare( &m_userdata_uuid, &ON_nil_uuid ) ? TRUE : FALSE;
}

ON_Object* ON_UserData::Owner() const
{
  return m_userdata_owner;
}

ON_UserData* ON_UserData::Next() const
{
  return m_userdata_next;
}

ON_UUID ON_UserData::UserDataClassUuid() const
{
  const ON_ClassId* cid = ClassId();
  return ( cid == &ON_UnknownUserData::m_ON_UnknownUserData_class_id )
          ? ((ON_UnknownUserData*)this)->m_unknownclass_uuid
          : cid->Uuid();
}

BOOL ON_UserData::IsUnknownUserData() const
{
  return (ClassId() == &ON_UnknownUserData::m_ON_UnknownUserData_class_id)?TRUE:FALSE;
}


ON_OBJECT_IMPLEMENT(ON_UnknownUserData,ON_UserData,"850324A8-050E-11d4-BFFA-0010830122F0");

ON_UnknownUserData::ON_UnknownUserData() 
                   : m_unknownclass_uuid(ON_nil_uuid), 
                     m_sizeof_buffer(0),
                     m_buffer(0),
                     m_3dm_version(0)
{}

ON_UnknownUserData::ON_UnknownUserData(const ON_UnknownUserData& src) : ON_UserData(src), 
                   m_unknownclass_uuid(ON_nil_uuid), 
                   m_sizeof_buffer(0), 
                   m_buffer(0),
                   m_3dm_version(0)
{
  if ( m_userdata_copycount > 0 && src.m_sizeof_buffer > 0 && src.m_buffer ) {
    m_unknownclass_uuid = src.m_unknownclass_uuid;
    m_sizeof_buffer = src.m_sizeof_buffer;
    m_buffer = onmemdup( src.m_buffer, src.m_sizeof_buffer);
    m_3dm_version = src.m_3dm_version;
  }
}

ON_UnknownUserData& ON_UnknownUserData::operator=(const ON_UnknownUserData& src)
{
  if ( this != &src ) {
    ON_UserData::operator=(src);
    if ( m_userdata_copycount > 0 && src.m_sizeof_buffer > 0 && src.m_buffer ) {
      m_unknownclass_uuid = src.m_unknownclass_uuid;
      m_sizeof_buffer = src.m_sizeof_buffer;
      m_buffer = onmemdup( src.m_buffer, src.m_sizeof_buffer);
      m_3dm_version = src.m_3dm_version;
    }
    else {
      m_unknownclass_uuid = ON_nil_uuid;
      m_sizeof_buffer = 0;
      m_buffer = 0;
      m_3dm_version = 0;
    }
  }
  return *this;
}

ON_UnknownUserData::~ON_UnknownUserData()
{
  if ( m_buffer )
    onfree(m_buffer);
}

BOOL ON_UnknownUserData::GetDescription( ON_wString& s )
{
  s = "Unknown user data - user application was not available when object was read.";
  return TRUE;
}

BOOL ON_UnknownUserData::IsValid() const
{
  BOOL rc = ON_UserData::IsValid();
  
  // valid unknown user data must have something in it
  if (rc) 
    rc = (m_sizeof_buffer>0);
  if (rc) 
    rc = (m_buffer != NULL);
  
  // the unknown class uuid cannot be nil
  if (rc) 
    rc = ON_UuidCompare( &m_unknownclass_uuid, &ON_nil_uuid );
  
  // the unknown class uuid cannot be the ON_UnknownUserData class uuid
  if (rc) {
    ON_UUID ON_UnknownUserData_classuuid = ON_UnknownUserData::m_ON_UnknownUserData_class_id.Uuid();
    rc = ON_UuidCompare( &m_unknownclass_uuid, &ON_UnknownUserData_classuuid );
  }
  return rc?TRUE:FALSE;
}

void ON_UnknownUserData::Dump( ON_TextLog& dump ) const
{
  ON_UserData::Dump(dump);
  dump.PushIndent();
  dump.Print( "unknown class id: ");
  dump.Print( m_unknownclass_uuid );
  dump.Print( "  Buffer size: %d\n",m_sizeof_buffer);
  dump.PopIndent();
}

BOOL ON_UnknownUserData::Write( ON_BinaryArchive& file ) const
{
  return file.WriteByte(m_sizeof_buffer,m_buffer);
}

BOOL ON_UnknownUserData::Read( ON_BinaryArchive& file )
{
  m_buffer = onrealloc( m_buffer, m_sizeof_buffer );
  m_3dm_version = file.Archive3dmVersion();
  return file.ReadByte(m_sizeof_buffer,m_buffer);
}


class ON_UnknownUserDataArchive : public ON_BinaryArchive
{
  // This class is used to define an ON_BinaryArchive that can be used
  // in ON_UnknownUserData::Convert() to initialize a ON_UserData class
  // from a memory buffer.
public:
  ON_UnknownUserDataArchive( const ON_UnknownUserData& );
  ~ON_UnknownUserDataArchive();

  // ON_BinaryArchive overrides
  int  CurrentPosition( // current offset (in bytes) into archive ( like ftell() )
                ) const; 
  BOOL SeekFromCurrentPosition( // seek from current position ( like fseek( ,SEEK_CUR) )
                int // byte offset ( >= -CurrentPostion() )
                ); 
  BOOL SeekFromStart(  // seek from current position ( like fseek( ,SEEK_SET) )
                int // byte offset ( >= 0 )
                );
  BOOL AtEnd() const; // TRUE if at end of file

protected:
  int Read( size_t, void* ); // return actual number of bytes read (like fread())
  int Write( size_t, const void* );
  BOOL Flush();

private:
  ON_UnknownUserDataArchive();

  int m_sizeof_buffer;
  const unsigned char* m_buffer;
  int m_buffer_position;
};

ON_UnknownUserDataArchive::ON_UnknownUserDataArchive( const ON_UnknownUserData& ud ) : ON_BinaryArchive( ON::read3dm )
{
  SetArchive3dmVersion(ud.m_3dm_version);
  m_sizeof_buffer = ud.m_sizeof_buffer;
  m_buffer = (const unsigned char*)ud.m_buffer;
  m_buffer_position = 0;
}

ON_UnknownUserDataArchive::~ON_UnknownUserDataArchive()
{
}

int  ON_UnknownUserDataArchive::CurrentPosition() const
{
  return m_buffer_position;
}

BOOL ON_UnknownUserDataArchive::SeekFromCurrentPosition( int offset )
{
  BOOL rc = FALSE;
  int newpos = m_buffer_position+offset;
  if ( newpos >= 0 && newpos < m_sizeof_buffer ) {
    m_buffer_position = newpos;
    rc = TRUE;
  }
  return rc;
}

BOOL ON_UnknownUserDataArchive::SeekFromStart( int offset )
{
  BOOL rc = FALSE;
  if ( offset >= 0 && offset < m_sizeof_buffer ) {
    m_buffer_position = offset;
    rc = TRUE;
  }
  return rc;
}

BOOL ON_UnknownUserDataArchive::AtEnd() const
{
  return (m_buffer_position >= m_sizeof_buffer) ? TRUE : FALSE;
}

int ON_UnknownUserDataArchive::Read( size_t count, void* buffer )
{
  size_t maxcount = 0;
  if ( m_sizeof_buffer > m_buffer_position ) {
    maxcount = m_sizeof_buffer - m_buffer_position;
  }
  if ( count > maxcount )
    count = maxcount;
  if ( count > 0 ) {
    memcpy( buffer, m_buffer+m_buffer_position, count );
    m_buffer_position += count;
  }
  return count;
}

int ON_UnknownUserDataArchive::Write( size_t, const void* )
{
  // ON_UnknownUserDataArchive does not support Write() and Flush()
  return FALSE;
}

BOOL ON_UnknownUserDataArchive::Flush()
{
  // ON_UnknownUserDataArchive does not support Write() and Flush()
  return FALSE;
}

ON_UserData* ON_UnknownUserData::Convert() const
{
  ON_UserData* ud = NULL;
  if ( IsValid() ) {
    const ON_ClassId* pID = ON_ClassId::ClassId( m_unknownclass_uuid );
    // if pID is NULL, it means the definiton of the unknown user data
    // is still not available
    if ( pID ) {
      // The class definition has been dynamically loaded since the
      // user data was read from an archive.  Use the class's Read()
      // to convert the buffer into a valid class
      ON_Object* pObject = pID->Create();
      if ( pObject ) {
        ud = ON_UserData::Cast(pObject);
        if ( !ud )
          delete pObject;
        else {
          // use class's Read() function to initialize class members from buffer
          ON_UnknownUserDataArchive file(*this);
          ud->Read(file);
        }
      }
    }
  }
  return ud;
}
