/* $Header: /src2/opennurbs/opennurbs_userdata.h 7     3/14/01 2:22p Dalelear $ */
/* $NoKeywords: $ */
/*
//
// Copyright (c) 1993-2001 Robert McNeel & Associates. All rights reserved.
// Rhinoceros is a registered trademark of Robert McNeel & Assoicates.
//
// THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
// ALL IMPLIED WARRANTIES OF FITNESS FOR ANY PARTICULAR PURPOSE AND OF
// MERCHANTABILITY ARE HEREBY DISCLAIMED.
//				
// For complete openNURBS copyright information see <http://www.opennurbs.org>.
//
////////////////////////////////////////////////////////////////
*/

#if !defined(OPENNURBS_USERDATA_INC_)
#define OPENNURBS_USERDATA_INC_

class ON_CLASS ON_UserData : public ON_Object
{
  ON_OBJECT_DECLARE(ON_UserData);
public:
  ON_UserData();
  ON_UserData(const ON_UserData&);
  ON_UserData& operator=(const ON_UserData&);

  //////////
  // The destructor automatically removes the user data
  // from ON_Object::m_userdata_list.
  ~ON_UserData();

  BOOL IsValid() const;

  ////////
  // Returns object that owns the user data
  ON_Object* Owner() const;

  ////////
  // Used for traversing list of user data attached
  // to an object.
  ON_UserData* Next() const;

  ////////
  // Returns the class id which is not necessarily the 
  // same as m_userdata_uuid.
  ON_UUID UserDataClassUuid() const; 

  //////////
  // Returns TRUE if the user data is anonymous.  This happens
  // when the user data class is not defined at the time the
  // user data is read from an archive.  For example, if a class
  // derived from ON_UserData is defined in application A
  // but is not defined in application B, then the class can be
  // defined when an archive is written by A but not exist when
  // an archive is read by B.  In this case, the
  // user data is not lost, it is just read as ON_UnknownUserData
  // by application B.  If application B saves the parent
  // object in an archive, the unknown user data is resaved in
  // a form that can be read by application A.
  BOOL IsUnknownUserData() const;

  ////////////////////////////////////////////////////////
  //
  // You must override ON_Object::Read(), ON_Object::Write(),
  // and ON_UserData::GetDescription().
  virtual 
  BOOL GetDescription( ON_wString& ) = 0; // description of user data shown in object properties dump

  //////////
  // If Transform() return FALSE, then the userdata is destroyed when 
  // its parent object is transformed.  The default Transform() returns
  // TRUE.  See also comment above m_userdata_xform
  virtual 
  BOOL Transform( const ON_Xform& ); 

  //////////
  // This uuid is the value that must be passed to ON_Object::GetUserData()
  // to retrieve attached user data
  ON_UUID m_userdata_uuid;

  ////////
  // If m_userdata_copycount is 0, user data is not copied when object
  // is copied.  If > 0, user data is copied and m_copycount is
  // incremented when parent object is copied.  The user data's operator=() 
  // is used to copy.  The default constructor sets m_userdata_copycount
  // to one.
  unsigned int m_userdata_copycount;  

  ////////
  // Updated if user data is attached to a piece of geometry that is
  // transformed and the virtual ON_UserData::Transform() is not 
  // overridden.  If you override ON_UserData::Transform() and want
  // m_userdata_xform to be updated, then call the 
  // ON_UserData::Transform() in your override.
  // The default constructor sets m_userdata_xform to the identity.
  ON_Xform m_userdata_xform; 

private: // don't look and don't touch - these may change
  friend BOOL ON_BinaryArchive::ReadObject( ON_Object** );
  friend BOOL ON_BinaryArchive::WriteObject( const ON_Object& );
  friend class ON_Object;
  ON_Object* m_userdata_owner;
  ON_UserData* m_userdata_next;
};

class ON_CLASS ON_UnknownUserData : public ON_UserData
{
  ON_OBJECT_DECLARE(ON_UnknownUserData);
  // used to hold user data will application class is not loaded
  // at time data is read
public:
  ON_UnknownUserData();
  ON_UnknownUserData(const ON_UnknownUserData&);
  ~ON_UnknownUserData();
  ON_UnknownUserData& operator=(const ON_UnknownUserData&);

  // ON_Object overrides
  BOOL IsValid() const;
  void Dump( ON_TextLog& ) const;
  BOOL Write( ON_BinaryArchive& ) const;
  BOOL Read( ON_BinaryArchive& );

  int Sizeof() const; // return amount of memory used by user data
  BOOL GetDescription( ON_wString& ); // description of user data

  // Convert unknown user data to actual user data.  Useful if
  // definition of actual user data is dynamically linked after
  // archive containing user data is read.
  ON_UserData* Convert() const;

  ON_UUID m_unknownclass_uuid; // UUID of missing class
  int m_sizeof_buffer;
  void* m_buffer;
  int m_3dm_version; // version of 3dm archive data was read from

};

#endif
