/* $Header: /src2/opennurbs/opennurbs_uuid.cpp 7     6/14/01 11:51a Dalelear $ */
/* $NoKeywords: $ */
/*
//
// Copyright (c) 1993-2001 Robert McNeel & Associates. All rights reserved.
// Rhinoceros is a registered trademark of Robert McNeel & Assoicates.
//
// THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
// ALL IMPLIED WARRANTIES OF FITNESS FOR ANY PARTICULAR PURPOSE AND OF
// MERCHANTABILITY ARE HEREBY DISCLAIMED.
//				
// For complete openNURBS copyright information see <http://www.opennurbs.org>.
//
////////////////////////////////////////////////////////////////
*/

#include "opennurbs.h"

ON_EXTERN_DEFN const ON_UUID ON_nil_uuid = {0,0,0,{0,0,0,0,0,0,0,0}};

// used to map the correspondence between uuid strings and
// ON_UUIDs as an array of 16 bytes.
static const int rho[16] = {3,2,1,0, 5,4, 7,6, 8,9, 10,11,12,13,14,15};

ON_DEFN 
ON_UUID ON_UuidFromString( const char* sUUID )
{
  // NOTE WELL: This code has to work on non-Windows OSs and on
  //            both big and little endian CPUs.  On Windows OSs
  //            is must return the same result as 
  //            Windows's UuidFromString().
  //

  // string has format like "85A08515-F383-11d3-BFE7-0010830122F0"
  // or like "{85A08515-F383-11d3-BFE7-0010830122F0}".  Brackets
  // and hyphens are optional and ignored.
  //
  // Windows users can use "guidgen" to create UUID strings.



  /*
#if defined(ON_DEBUG) && defined(ON_OS_WINDOWS)
  RPC_STATUS st;
  union 
  {
    ON_UUID uuid;
    unsigned char b[16];
  } u1;
  st = UuidFromString( (unsigned char*)sUUID, &u1.uuid );
#endif
*/

  union 
  {
    ON_UUID uuid;
    unsigned char b[16];
  } u;
  BOOL bFailed;
  int bi, ci;
  unsigned char c;
  unsigned char byte_value[2];

  for ( bi = 0; bi < 16; bi++ ) 
    u.b[bi] = 0;

  bFailed = sUUID ? FALSE : TRUE;

  if ( !bFailed ) {
    while ( *sUUID && *sUUID <= ' ' ) // skip leading white space
      sUUID++;
    if ( *sUUID == '{' )
      sUUID++;
    for ( bi = 0; bi < 16; bi++ ) {
      ci = 0;
      byte_value[0] = 0;
      byte_value[1] = 0;
      while ( ci < 2 ) {
        c = *sUUID++;
        if ( !c ) {
          bFailed = TRUE;
          break;
        }
        if ( c >= 'A' && c <= 'F' ) {
          byte_value[ci++] = (c-'A'+10);
        }
        else if ( c >= '0' && c <='9' ) {
          byte_value[ci++] = (c-'0');
        }
        else if ( c >= 'a' && c <= 'f' ) {
          byte_value[ci++] = (c-'a'+10);
        }
        else if ( c != '-' ) {
          bFailed = TRUE;
          break;
        }
      }
      if ( bFailed )
        break;
      u.b[rho[bi]] = 16*byte_value[0] + byte_value[1];
    }
  }

  if ( bFailed ) {
    ON_ERROR("ON_UuidFromString(): bad string passed in");
    u.uuid = ON_nil_uuid;
  }

/*
#if defined(ON_DEBUG) && defined(ON_OS_WINDOWS)
  if ( memcmp( &u.uuid, &u1.uuid, 16 ) ) {
    ON_ERROR("ON_UuidFromString() failed");
  }
  if ( UuidCompare( &u.uuid, &u1.uuid, &st ) ) {
    ON_ERROR("ON_UuidFromString() failed");
  }
  if ( ON_UuidCompare( &u.uuid, &u1.uuid ) ) {
    ON_ERROR("ON_UuidCompare() failed");
  }
#endif
*/
  return u.uuid;
}

ON_DEFN 
int ON_UuidCompare( const ON_UUID* a, const ON_UUID* b )
{
  // NOTE WELL: This code has to work on non-Windows OSs and on
  //            both big and little endian CPUs.
  //

  // returns -1 a < b, 0 a==b, 1 a>b
  int rc = 0;
  if ( !a ) {
    if (b)
      rc = -1;
  }
  else if ( !b ) {
    rc = 1;
  }
  else {
    if      ( a->Data1 < b->Data1 ) rc = -1;
    else if ( a->Data1 > b->Data1 ) rc =  1;
    else if ( a->Data2 < b->Data2 ) rc = -1;
    else if ( a->Data2 > b->Data2 ) rc =  1;
    else if ( a->Data3 < b->Data3 ) rc = -1;
    else if ( a->Data3 > b->Data3 ) rc =  1;
    else {
      int i;
      for ( i = 0; i < 8; i++ ) {
        if (a->Data4[i] < b->Data4[i]) {
          rc = -1;
          break;
        }
        if (a->Data4[i] > b->Data4[i]) {
          rc = 1;
          break;
        }
      }
    }
  }

  return rc;
}

ON_DEFN 
int ON_UuidCompare( const ON_UUID& a, const ON_UUID& b)
{
  return ON_UuidCompare(&a,&b);
}

ON_DEFN
char* ON_UuidToString( const ON_UUID& uuid, char* s)
{
  // NOTE WELL: 
  //   This code has to work on non-Windows OSs and on both big and
  //   little endian CPUs.  The result must satisfy
  //   uuid == ON_UuidFromString(ON_UuidToString(uuid,s))

  static const char x[16] = {'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};
  static const int addhyphen[16] = {0,0,0,1, 0,1, 0,1, 0,1,  0, 0, 0, 0, 0, 0};
  const unsigned char* b = (const unsigned char*)&uuid;
  char* p;
  int i;
  unsigned char c;
  if ( !s )
    return 0;
  p = s;
  for ( i = 0; i < 16; i++ ) {
    c = b[rho[i]];
    *p++ = x[c>>4];
    *p++ = x[c&0xF];
    if ( addhyphen[i] )
      *p++ = '-';
  }
  *p = 0;

#if defined(ON_DEBUG)
  {
    ON_UUID u = ON_UuidFromString(s);
    if ( ON_UuidCompare(&u,&uuid) ) {
      ON_ERROR("ON_UuidToString() bug"); // <- breakpoint here
    }
  }
#endif

  return s;
}


ON_DEFN 
const char* ON_UuidToString( const ON_UUID& uuid, ON_String& s )
{
  char x[37];
  s = ON_UuidToString( uuid, x );
  return s.Array();
}

ON_DEFN 
const wchar_t* ON_UuidToString( const ON_UUID& uuid, ON_wString& s )
{
  char x[37];
  s = ON_UuidToString( uuid, x );
  return s.Array();
}
