/* $Header: /src2/opennurbs/opennurbs_uuid.h 3     6/01/01 2:13p Dalelear $ */
/* $NoKeywords: $ */
/*
//
// Copyright (c) 1993-2001 Robert McNeel & Associates. All rights reserved.
// Rhinoceros is a registered trademark of Robert McNeel & Assoicates.
//
// THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
// ALL IMPLIED WARRANTIES OF FITNESS FOR ANY PARTICULAR PURPOSE AND OF
// MERCHANTABILITY ARE HEREBY DISCLAIMED.
//				
// For complete openNURBS copyright information see <http://www.opennurbs.org>.
//
////////////////////////////////////////////////////////////////
*/

#if !defined(OPENNURBS_UUID_INC_)
#define OPENNURBS_UUID_INC_

// ON_UUID is a 16 byte universally unique identifier
#if defined(UUID_DEFINED)
typedef UUID ON_UUID;
#elif defined(GUID_DEFINED)
typedef GUID ON_UUID;
#else
typedef struct {
    unsigned long  Data1;
    unsigned short Data2;
    unsigned short Data3;
    unsigned char  Data4[8];
} ON_UUID;
#endif

ON_BEGIN_EXTERNC
ON_EXTERN_DECL const ON_UUID ON_nil_uuid;
ON_END_EXTERNC

#if defined(ON_CPLUSPLUS)

//////////
// ON_UuidCompare(&a,&b) compares two uuids and returns
//   -1:  a < b
//    0:  a == b
//   +1:  a > b
// A NULL pointer is considered < a non-NULL pointer.
ON_DECL 
int ON_UuidCompare( const ON_UUID*, const ON_UUID* );

ON_DECL 
int ON_UuidCompare( const ON_UUID&, const ON_UUID& );

//////////
// ON_UuidFromString() converts a string like
// "{85A08515-F383-11d3-BFE7-0010830122F0}" into a uuid.
// The brackets and hyphens are optional and are ignored.
// Windows users can use "guidgen.exe" to create UUID strings.
ON_DECL 
ON_UUID ON_UuidFromString( const char* );

//////////
// ON_UuidToString() converts a uuid to a null termintated string
// like "85A08515-F383-11D3-BFE7-0010830122F0".  The char array must
// have length >= 37.  The returned char array will have a 36 
// character uuid in char[0..35] and a null in char[36].
// The pointer to the array is returned.
ON_DECL 
char* ON_UuidToString( const ON_UUID&, char* );

class ON_String;

ON_DECL 
const char* ON_UuidToString( const ON_UUID&, ON_String& );

class ON_wString;

ON_DECL 
const wchar_t* ON_UuidToString( const ON_UUID&, ON_wString& );

#endif

#endif
