/* $Header: /src2/opennurbs/opennurbs_workspace.h 4     1/03/01 4:59p Dalelear $ */
/* $NoKeywords: $ */
/*
//
// Copyright (c) 1993-2001 Robert McNeel & Associates. All rights reserved.
// Rhinoceros is a registered trademark of Robert McNeel & Assoicates.
//
// THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
// ALL IMPLIED WARRANTIES OF FITNESS FOR ANY PARTICULAR PURPOSE AND OF
// MERCHANTABILITY ARE HEREBY DISCLAIMED.
//				
// For complete openNURBS copyright information see <http://www.opennurbs.org>.
//
////////////////////////////////////////////////////////////////
*/

#if !defined(OPENNURBS_WORKSPACE_INC_)
#define OPENNURBS_WORKSPACE_INC_

// Use ON_Workspace to efficiently get and automatically
// clean up workspace memory and scratch files.
class ON_CLASS ON_Workspace
{
public:
  ON_Workspace();
  ~ON_Workspace();

  // The GetMemory...() functions use rhmalloc() to get memory.
  void*        GetMemory( int );
  int*         GetIntMemory( int );
  double*      GetDoubleMemory( int );
  ON_3dPoint*  GetPointMemory( int );
  ON_3dVector* GetVectorMemory( int );

  // The GrowMemory...() functions use onrealloc() to get more memory
  // The pointer passed to GrowMemory() must be one that was returned
  // by any of the GetMemory...() calls.
  void*        GrowMemory( void*, int );
  int*         GrowIntMemory( int*, int );
  double*      GrowDoubleMemory( double*, int );
  ON_3dPoint*  GrowPointMemory( ON_3dPoint*, int );
  ON_3dVector* GrowVectorMemory( ON_3dVector*, int );

  // Calling the KeepMemory() function with a pointer returned from
  // one of the Get...() or Grow...() calls keeps the workspace destructor
  // from rhfree()ing the memory.  The caller is responsible for calling 
  // rhfree() to release the memory.
  BOOL KeepMemory( void* );  // returns TRUE for success and FALSE for failure

  FILE*   OpenFile(const char*, const char*); // uses fopen()
  int     KeepFile(FILE*);     // Pass FILE pointer returned by Open().
                               // When the workspace destructs, the
                               // file will not be closed.
                               // Caller is responsible for calling fclose()
                               // to close the file.

private:
  struct FBLK 
  {
    struct FBLK* pNext;
    FILE* pFile;
  } * m_pFileBlk;

  struct MBLK 
  {
    struct MBLK* pNext;
    void* pMem;
  } * m_pMemBlk;

private:
  // no implementation of the following
  ON_Workspace( const ON_Workspace& );            // no copy constructor
  ON_Workspace& operator=( const ON_Workspace& ); // no operator=
};


#endif
