//***************************************************************************************
//
// File supervisor: Softimage 3D Games & 3D Bridge team
//
// (c) Copyright 2001-2002 Avid Technology, Inc. . All rights reserved.
//
//***************************************************************************************
/****************************************************************************************
THIS CODE IS PUBLISHED AS A SAMPLE ONLY AND IS PROVIDED "AS IS".
IN NO EVENT SHALL SOFTIMAGE, AVID TECHNOLOGY, INC. AND/OR THEIR RESPECTIVE 
SUPPLIERS BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY
DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
CONNECTION WITH THE USE OR PERFORMANCE OF THIS CODE . 
 
COPYRIGHT NOTICE. Copyright  1999-2002 Avid Technology Inc. . All rights reserved. 

SOFTIMAGE is a registered trademark of Avid Technology Inc. or its subsidiaries 
or divisions. Windows NT is a registered trademark of Microsoft Corp. All other
trademarks contained herein are the property of their respective owners. 
****************************************************************************************/

//********************************************************************
// Includes
//********************************************************************
// FileHandler.cpp : Implementation of CFileHandler
#include "stdafx.h"
#include "DotXSIObj.h"
#include "FileHandler.h"
//#include <csconv.h>
#include "Templates.h"
#include <string.h>

/////////////////////////////////////////////////////////////////////////////
// CFileHandler

STDMETHODIMP CFileHandler::InterfaceSupportsErrorInfo(REFIID riid)
{
	static const IID* arr[] = 
	{
		&IID_IFileHandler
	};
	for (int i=0; i < sizeof(arr) / sizeof(arr[0]); i++)
	{
		if (InlineIsEqualGUID(*arr[i],riid))
			return S_OK;
	}
	return S_FALSE;
}

//*********************************************************************************************
//
// Author:	Softimage 3D Games & 3D Bridge team
// Date  :	28/May/01
//
// @mfunc	Open the .XSI file via the Parser
//
// @rvalue	S_OK | Success	
// @rvalue	E_FAIL | Fail
//
//*********************************************************************************************
STDMETHODIMP CFileHandler::Open(BSTR in_bstrFilename)
{

	USES_CONVERSION;

	HRESULT		l_hr = S_OK;
	FILE		*l_pStream;
	
	CHAR	* l_szTemp = OLE2A( in_bstrFilename );

	//Check if the file exist
	if( (l_pStream  = fopen( l_szTemp, "r" )) == NULL )
	{
      Error( "The file does not exist" );
	  l_hr = E_FAIL;		
	}
    else
	{
	  l_hr = S_OK;
	  
	  if ( m_pXSIParser->Open( CSIBCString(l_szTemp) ) != SI_SUCCESS )
		l_hr = E_FAIL;
	}

	return l_hr;
}

//*********************************************************************************************
//
// Author:	Softimage 3D Games & 3D Bridge team
// Date  :	28/May/01
//
// @mfunc	Close the file handler
//
// @rvalue	S_OK | Success	
// @rvalue	E_FAIL | Fail
//
//*********************************************************************************************
STDMETHODIMP CFileHandler::Close()
{
	HRESULT		l_hr = S_OK;

	if ( m_pXSIParser->Close() != SI_SUCCESS )
		l_hr = E_FAIL;

	return l_hr;
}

//*********************************************************************************************
//
// Author:	Softimage 3D Games & 3D Bridge team
// Date  :	28/May/01
//
// @mfunc	Create a new XSI file with a given name and mode
//
// @rvalue	S_OK | Success	
// @rvalue	E_FAIL | Fail
//
//*********************************************************************************************

STDMETHODIMP CFileHandler::Create(BSTR in_bstrFilename, BSTR in_bOpenMode)
{
	USES_CONVERSION;

	HRESULT				l_hr = S_OK;
	eDotXSIFileFormat	in_DotXSIFileFormat;
	
	// First convert the BSTR to a szString
	CHAR *l_szTemp = OLE2A( in_bstrFilename );
	CHAR *l_szMode = OLE2A( in_bOpenMode );

	//Convert the String into the eDotXSIFileFormat
	if(strcmp(l_szMode, "DOTXSI_FORMAT_TEXT") == 0)
	{
		in_DotXSIFileFormat = DOTXSI_FORMAT_TEXT;
	}
	else if(strcmp(l_szMode, "DOTXSI_FORMAT_BINARY") == 0)
	{
		in_DotXSIFileFormat = DOTXSI_FORMAT_BINARY;
	}
	else
	{
		Error("Wrong file output format : DOTXSI_FORMAT_TEXT or DOTXSI_FORMAT_BINARY");
		l_hr = E_FAIL;
		goto end;
	}

	// Set the format
	if(in_DotXSIFileFormat == DOTXSI_FORMAT_TEXT)
	{
		m_pXSIParser->SetdotXSIFormat( FORMAT_TEXT );
	}
	else if(in_DotXSIFileFormat == DOTXSI_FORMAT_BINARY)
	{
		m_pXSIParser->SetdotXSIFormat( FORMAT_BINARY );
	}
	else
	{
		Error("Wrong file output format : DOTXSI_FORMAT_TEXT or DOTXSI_FORMAT_BINARY");
		l_hr = E_FAIL;
	}

	// Set the OpenMode
    m_pXSIParser->SetOpenMode( OPEN_WRITE );

	// Set the Filename
	m_pXSIParser->FileName().SetText( l_szTemp );

	// And then open
	if ( m_pXSIParser->Open( CSIBCString(m_pXSIParser->FileName()) ) != SI_SUCCESS )
	{
		Error("The file could not be created, verify that the path exist and that you have write access");
		l_hr = E_FAIL;
	}

end:
	return l_hr;
}

//*********************************************************************************************
//
// Author:	Softimage 3D Games & 3D Bridge team
// Date  :	28/May/01
//
// @mfunc	Write the dotXSI file via the parser
//
// @rvalue	S_OK | Success	
// @rvalue	E_FAIL | Fail
//
//*********************************************************************************************

STDMETHODIMP CFileHandler::Write()
{
	HRESULT		l_hr = S_OK;

	_ASSERT(m_pXSIParser != NULL);

	if ( m_pXSIParser->Write() != SI_SUCCESS )
	{
		Error("Could not write into file");
		l_hr = E_FAIL;
	}

	return l_hr;
}

//*********************************************************************************************
//
// Author:	Softimage 3D Games & 3D Bridge team
// Date  :	28/May/01
//
// @mfunc	Read the dotXSI file via the parser
//
// @rvalue	S_OK | Success	
// @rvalue	E_FAIL | Fail
//
//*********************************************************************************************

STDMETHODIMP CFileHandler::Read()
{
	HRESULT		l_hr = S_OK;

	if ( m_pXSIParser->Read() != SI_SUCCESS )
	{
		Error("Could not read file!");
		l_hr = E_FAIL;
	}

	return l_hr;
}

//*********************************************************************************************
//
// Author:	Softimage 3D Games & 3D Bridge team
// Date  :	28/May/01
//
// @mfunc	Return the dotXSIFile major version
// @rvalue	S_OK | Success	
//
//*********************************************************************************************

STDMETHODIMP CFileHandler::get_FileVersionMajor(long *out_plVersionMajor)
{
	*out_plVersionMajor = m_pXSIParser->GetdotXSIFileVersionMajor();
	return S_OK;	
}

//*********************************************************************************************
//
// Author:	Softimage 3D Games & 3D Bridge team
// Date  :	28/May/01
//
// @mfunc	Return the dotXSIFile minor version
// @rvalue	S_OK | Success	
//
//*********************************************************************************************

STDMETHODIMP CFileHandler::get_FileVersionMinor(long *out_plVersionMinor)
{
	*out_plVersionMinor = m_pXSIParser->GetdotXSIFileVersionMinor();
	return S_OK;
}

//*********************************************************************************************
//
// Author:	Softimage 3D Games & 3D Bridge team
// Date  :	28/May/01
//
// @mfunc	Create an instance of a COM object
// @rvalue	S_OK | Success	
//
//*********************************************************************************************

HRESULT CFileHandler::CreateObject(BSTR in_pszProgID, IDispatch **out_ppdisp)
{
    CLSID clsid;                  // CLSID of ActiveX object.
    HRESULT hr;
    LPUNKNOWN punk = NULL;        // IUnknown of ActiveX object.
    LPDISPATCH pdisp = NULL;      // IDispatch of ActiveX object.

    *out_ppdisp = NULL;

    // Retrieve CLSID from the ProgID that the user specified.
    hr = CLSIDFromProgID(in_pszProgID, &clsid);
	//hr = CLSIDFromString(pszProgID, &clsid);
    if (FAILED(hr))
        goto error;

    // Create an instance of the ActiveX object.
    hr = CoCreateInstance(clsid, NULL, CLSCTX_SERVER, 
                            IID_IUnknown, (void **)&punk);
    if (FAILED(hr))
        goto error;

      // Ask the ActiveX object for the IDispatch interface.
    hr = punk->QueryInterface(IID_IDispatch, (void **)&pdisp);
    if (FAILED(hr))
        goto error;

    *out_ppdisp = pdisp;
    punk->Release();
    return NOERROR;
    
error:
    if (punk) punk->Release();
    if (pdisp) pdisp->Release();
    return hr;
}

//*********************************************************************************************
//
// Author:	Softimage 3D Games & 3D Bridge team
// Date  :	28/May/01
//
// @mfunc	Return the root templates pointer to iterate thru the dotXSI hierarchy
//
// @rvalue	S_OK | Success	
// @rvalue	E_FAIL or E_POINTER | Fail or Invalid pointer
//
//*********************************************************************************************

STDMETHODIMP CFileHandler::get_RootTemplates(IDispatch **out_pTemplates)
{
	HRESULT hr;

	//Create an instance of the ActiveX object.
	hr = CreateObject(L"DotXSIObj.Templates", out_pTemplates);
	
	((CTemplates*)*out_pTemplates)->SetXSIParser(m_pXSIParser);

	return S_OK;
}

