//***************************************************************************************
//
// File supervisor: Softimage 3D Games & 3D Bridge team
//
// (c) Copyright 2001-2002 Avid Technology, Inc. . All rights reserved.
//
//***************************************************************************************
/****************************************************************************************
THIS CODE IS PUBLISHED AS A SAMPLE ONLY AND IS PROVIDED "AS IS".
IN NO EVENT SHALL SOFTIMAGE, AVID TECHNOLOGY, INC. AND/OR THEIR RESPECTIVE 
SUPPLIERS BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY
DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
CONNECTION WITH THE USE OR PERFORMANCE OF THIS CODE . 
 
COPYRIGHT NOTICE. Copyright  1999-2002 Avid Technology Inc. . All rights reserved. 

SOFTIMAGE is a registered trademark of Avid Technology Inc. or its subsidiaries 
or divisions. Windows NT is a registered trademark of Microsoft Corp. All other
trademarks contained herein are the property of their respective owners. 
****************************************************************************************/

//********************************************************************
// Includes
//********************************************************************
// Param.cpp : Implementation of CParam
#include "stdafx.h"
#include "DotXSIObj.h"
#include "Param.h"
#include "varconv.h"

/////////////////////////////////////////////////////////////////////////////
// CParam

STDMETHODIMP CParam::InterfaceSupportsErrorInfo(REFIID riid)
{
	static const IID* arr[] = 
	{
		&IID_IParam
	};
	for (int i=0; i < sizeof(arr) / sizeof(arr[0]); i++)
	{
		if (InlineIsEqualGUID(*arr[i],riid))
			return S_OK;
	}
	return S_FALSE;
}
//*********************************************************************************************
//
// Author:	Softimage 3D Games & 3D Bridge team
// Date  :	28/May/01
//
// @mfunc		Allocate some memory, through XSICore.dll
//				The allocation of the memory is done in the XSICore.dll 
//				memory space through AllocateLocalXSIArray().
//				XSICore will use this array directly in a template and 
//				will take care of freeing it when the destructor will be called.
//
//********************************************************************
STDMETHODIMP CParam::AllocateArray(VARIANT *out_pvValue, long in_lSize)
{
	HRESULT l_hr = S_OK;
	SI_TinyVariant l_siVar;

	if (m_pParam == 0) return E_FAIL;

	m_pParam->GetValue( &l_siVar );
	l_hr = COM2SIVariant( out_pvValue, &l_siVar );
	if ( FAILED ( l_hr ) )
		return l_hr;
	
	AllocateLocalXSIArray( &l_siVar, in_lSize );

	l_hr = SI2COMVariant( &l_siVar, out_pvValue );

	return l_hr;
}
//********************************************************************
// Author:	Softimage 3D Games & 3D Bridge team
// Date			:  3/Oct/00
//
// @mfunc		Free the memory allocated for a string variant. 
//				This function should be called after a GetValue that
//				returns a string.
//
//********************************************************************
STDMETHODIMP CParam::FreeStringVariant(VARIANT in_vValue)
{
	HRESULT l_hr = S_OK;

	switch( in_vValue.vt)
	{
	case VT_BYREF|VT_I1:	// char
		free( in_vValue.pcVal );
		break;
	case VT_BSTR:			// bstr
		free( in_vValue.bstrVal );
		break;
	default: 
		{
			assert( FALSE );
		}
	}

	return l_hr;
}
//*********************************************************************************************
//
// Author:	Softimage 3D Games & 3D Bridge team
// Date  :	28/May/01
//
// @mfunc		Get the name of the Parameter
//
//********************************************************************
STDMETHODIMP CParam::get_Name(BSTR *out_pbstrName)
{
	if (m_pParam == 0) return E_FAIL;

	*out_pbstrName = A2BSTR( m_pParam->Name().GetText() );

	return S_OK;
}
//*********************************************************************************************
//
// Author:	Softimage 3D Games & 3D Bridge team
// Date  :	28/May/01
//
// @mfunc		Set the name of the Parameter
//
//********************************************************************
STDMETHODIMP CParam::put_Name(BSTR in_bstrName)
{
	USES_CONVERSION;

	if (m_pParam == 0) return E_FAIL;
	
	// Then set the name
	m_pParam->Name().SetText( OLE2A( in_bstrName ) );
	
	return S_OK;
}
//*********************************************************************************************
//
// Author:	Softimage 3D Games & 3D Bridge team
// Date  :	28/May/01
//
// @mfunc		Get the value for the paramter
//
//********************************************************************
STDMETHODIMP CParam::get_Value(VARIANT *out_pvValue)
{
	SI_TinyVariant l_siVar;
	
	// out param initialization
	(*out_pvValue).vt = VT_EMPTY;
	
	if (m_pParam == 0) return E_FAIL;

	m_pParam->GetValue( &l_siVar );
	return SI2COMVariant( &l_siVar, out_pvValue );
}
//*********************************************************************************************
//
// Author:	Softimage 3D Games & 3D Bridge team
// Date  :	28/May/01
//
// @mfunc		Set the value for the paramter
//
//********************************************************************
STDMETHODIMP CParam::put_Value(VARIANT in_vValue)
{
	HRESULT l_hr;
	SI_TinyVariant l_siVar;

	if (m_pParam == 0) return E_FAIL;

	m_pParam->GetValue( &l_siVar );
	l_hr = COM2SIVariant( &in_vValue, &l_siVar ); 
	if ( FAILED ( l_hr ) )
		return l_hr;

	m_pParam->SetValue( l_siVar );
	return l_hr;
}
