//***************************************************************************************
//
// File supervisor: Softimage 3D Games & 3D Bridge team
//
// (c) Copyright 2001-2002 Avid Technology, Inc. . All rights reserved.
//
//***************************************************************************************
/****************************************************************************************
THIS CODE IS PUBLISHED AS A SAMPLE ONLY AND IS PROVIDED "AS IS".
IN NO EVENT SHALL SOFTIMAGE, AVID TECHNOLOGY, INC. AND/OR THEIR RESPECTIVE 
SUPPLIERS BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY
DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
CONNECTION WITH THE USE OR PERFORMANCE OF THIS CODE . 
 
COPYRIGHT NOTICE. Copyright  1999-2002 Avid Technology Inc. . All rights reserved. 

SOFTIMAGE is a registered trademark of Avid Technology Inc. or its subsidiaries 
or divisions. Windows NT is a registered trademark of Microsoft Corp. All other
trademarks contained herein are the property of their respective owners. 
****************************************************************************************/

//***************************************************************************************
// Includes
//***************************************************************************************

#include <SIBCString.h>

//***************************************************************************************
// Defines
//***************************************************************************************

//***************************************************************************************
// Typedefs
//***************************************************************************************

#define _CSIBCSTRING_TMP_LEN	256

const SI_Int CSIAccumString::k_InitialBufSize = 0x100; // 256 char initial size...

//***************************************************************************************
// Author:	Softimage 3D Games & 3D Bridge team
// Date     :
//
// @mfunc Constructor.
// @syntax CSIBCString::CSIBCString();
// @syntax CSIBCString::CSIBCString(CSIBCString &in_pString);
// @parm CSIBCString& | in_pString | Contains input string.
// @syntax CSIBCString::CSIBCString(const SI_Char *in_pString);
// @parm SI_Char* | in_pString | Contains input string
//***************************************************************************************

CSIBCString::CSIBCString()
{
	m_lLength = 0;
	m_pBuffer = NULL;
}

//***************************************************************************************
// Author:	Softimage 3D Games & 3D Bridge team
// Date     :
//
// @mfunc Default destructor.
//***************************************************************************************

CSIBCString::~CSIBCString()
{
	m_lLength = 0;
	_SI_DELETE_ARRAY(m_pBuffer);
}

//***************************************************************************************
// Author:	Softimage 3D Games & 3D Bridge team
// Date     :
//
// mfunc constructor
//***************************************************************************************
CSIBCString::CSIBCString(CSIBCString &in_pString)
{	
	_SI_ASSERT(in_pString.GetText() != NULL);

	m_lLength = in_pString.GetLength();
	m_pBuffer = _SI_STRDUP(in_pString.GetText());
}

//***************************************************************************************
// Author:	Softimage 3D Games & 3D Bridge team
// Date     :
//
// mfunc constructor
//***************************************************************************************
CSIBCString::CSIBCString(const SI_Char *in_pString) 
{
	_SI_ASSERT(in_pString != NULL);

	m_lLength = _SI_STRLEN(in_pString);
	m_pBuffer = _SI_STRDUP(in_pString);
}

//***************************************************************************************
// Author:	Softimage 3D Games & 3D Bridge team
// Date     :
//
// @mfunc Gets the length of the string.
// @rdesc	Returns the length of the string.
//***************************************************************************************
SI_Int CSIBCString::GetLength()
{
	return m_lLength;
}

//***************************************************************************************
// Author:	Softimage 3D Games & 3D Bridge team
// Date     :
//
// @mfunc Gets the text of the string.
// @rdesc Returns the string of text.
//***************************************************************************************
SI_Char *CSIBCString::GetText()
{
	return m_pBuffer;
}

//***************************************************************************************
// Author:	Softimage 3D Games & 3D Bridge team
// Date     :
//
// @mfunc Clears the string.
//***************************************************************************************
void CSIBCString::Clear()
{
	_SI_DELETE_ARRAY(m_pBuffer);
	m_lLength = 0;
}

//***************************************************************************************
// Author:	Softimage 3D Games & 3D Bridge team
// Date     :
//
// @mfunc Appends a second string to this string. If you pass an integer or floating point number, 
// it is converted to a string and then appended to this string.
//
// @syntax SI_Error CSIBCString::Concat (CSIBCString *in_pString);
// @parm CSIBCString* | in_pString | Append a second string to this string.
// @syntax SI_Error CSIBCString::Concat (const SI_Char *in_pString);
// @parm SI_Char* | in_pString | Append a second string to this string.
// @syntax SI_Error CSIBCString::Concat (const SI_Int in_lInt);
// @parm SI_Int | in_lInt | Concatenate up to this value.
// @syntax SI_Error CSIBCString::Concat (const SI_Float in_fFloat);
// @parm SI_Float | in_fFloat | Concatenate up to this value
//***************************************************************************************
SI_Error CSIBCString::Concat (CSIBCString *in_pString)
{
	SI_Error	result	= SI_SUCCESS;
	SI_Char		*pTmp	= NULL;
	int			iStrLen = 0;

	_SI_ASSERT(in_pString != NULL);

	//Call general string concatonation
	Concat( in_pString->GetText() );
	return result;
}

//***************************************************************************************
// Author:	Softimage 3D Games & 3D Bridge team
// Date     :
//
// mfunc Concatenate to the string
//***************************************************************************************
SI_Error CSIBCString::Concat (const SI_Char *in_pString)
{
	SI_Error	result	= SI_SUCCESS;
	SI_Char		*pTmp	= NULL;
	int			iStrLenIn = 0, iStrLen = 0;

	_SI_ASSERT(in_pString != NULL);

	if(m_pBuffer != NULL)
	{
		//
		//Allocate new buffer which is the size of the current string plus the 
		//length of the string we wish to concat (plus one for the null char )
		//
		iStrLen		= _SI_STRLEN( m_pBuffer );
		iStrLenIn	= _SI_STRLEN( in_pString );

		_SI_NEW( pTmp, SI_Char[ iStrLen + iStrLenIn + 1] );
		
		//Copy old string
		memcpy( pTmp, m_pBuffer, sizeof( SI_Char ) * (iStrLen + 1) );

		//Delete original buffer since we've copied the data
		_SI_DELETE_ARRAY( m_pBuffer );

		//Assign temporary pointer to local string pointer
		m_pBuffer = pTmp;

		//If properly allocated, append new string on.
		if(result == SI_SUCCESS)
		{
			_SI_STRCAT( m_pBuffer, in_pString );

			//Calculate new length based on current length and 
			//length of incoming string
			m_lLength = iStrLen + iStrLenIn;
		}
	}
	else
	{
		_SI_CALL(SetText(in_pString),"CSIBCString::SetText");	
	}
	
	return result;
}

//***************************************************************************************
// Author:	Softimage 3D Games & 3D Bridge team
// Date     :
//
// mfunc Concatenate to the string
//***************************************************************************************
SI_Error CSIBCString::Concat (const SI_Int in_lInt)
{
	SI_Error	result = SI_SUCCESS;
	SI_Char		tmpStr[_CSIBCSTRING_TMP_LEN];
	
	_SI_SPRINTF(tmpStr,_SI_TEXT("%d\0"),in_lInt);

	//Call string concat
	return Concat( tmpStr );
}

//***************************************************************************************
// Author:	Softimage 3D Games & 3D Bridge team
// Date     :
//
// mfunc Concatenate to the string
//***************************************************************************************
SI_Error CSIBCString::Concat (const SI_Float in_fFloat)
{
	SI_Error	result = SI_SUCCESS;
	SI_Char		tmpStr[_CSIBCSTRING_TMP_LEN];
	
	_SI_SPRINTF(tmpStr,_SI_TEXT("%f\0"),in_fFloat);
	
	//Call string concat
	return Concat( tmpStr );
}

//***************************************************************************************
// Author:	Softimage 3D Games & 3D Bridge team
// Date     :
//
// @mfunc Sets the string text. If you pass an integer or floating point number, 
// it is converted to a string and then appended to this string.
//
// @syntax SI_Error CSIBCString::SetText(CSIBCString *in_pString);
// @parm CSIBCString* | in_pString | Text string.
// @syntax SI_Error CSIBCString::SetText(const SI_Char *in_pString);
// @parm SI_Char* | in_pString | Text string.
// @syntax SI_Error CSIBCString::SetText(const SI_Int in_lInt);
// @parm SI_Int | in_lInt | Integer to convert to string.
// @syntax SI_Error CSIBCString::SetText(const SI_Float in_fFloat);
// @parm SI_Float | in_fFloat | Float to convert to string.
//***************************************************************************************
SI_Error CSIBCString::SetText(CSIBCString *in_pString)
{
	SI_Error	result = SI_SUCCESS;

	_SI_ASSERT(in_pString != NULL);
	_SI_ASSERT(in_pString->GetText() != NULL);

	Clear();
	m_pBuffer = _SI_STRDUP(in_pString->GetText());
	m_lLength = in_pString->GetLength();

	return result;
}

//***************************************************************************************
// Author:	Softimage 3D Games & 3D Bridge team
// Date     :
//
// mfunc Assign to the string
//***************************************************************************************
SI_Error CSIBCString::SetText(const SI_Char *in_pString)
{
	SI_Error	result = SI_SUCCESS;

	_SI_ASSERT(in_pString != NULL);

	Clear();
	m_pBuffer = _SI_STRDUP(in_pString);
	m_lLength = _SI_STRLEN(in_pString);

	return result;
}

//***************************************************************************************
// Author:	Softimage 3D Games & 3D Bridge team
// Date     :
//
// mfunc Assign to the string
//***************************************************************************************
SI_Error CSIBCString::SetText(const SI_Int in_lInt)
{
	SI_Error	result = SI_SUCCESS;
	SI_Char	tmpStr[_CSIBCSTRING_TMP_LEN];

	_SI_SPRINTF(tmpStr,_SI_TEXT("%d\0"),in_lInt);

	Clear();
	m_pBuffer = _SI_STRDUP(tmpStr);
	m_lLength = _SI_STRLEN(tmpStr);

	return result;
}

//***************************************************************************************
// Author:	Softimage 3D Games & 3D Bridge team
// Date     :
//
// mfunc Assign to the string
//***************************************************************************************
SI_Error CSIBCString::SetText(const SI_Float in_fFloat)
{
	SI_Error	result = SI_SUCCESS;
	SI_Char		tmpStr[_CSIBCSTRING_TMP_LEN];

	_SI_SPRINTF(tmpStr,_SI_TEXT("%f\0"),in_fFloat);

	Clear();
	m_pBuffer = _SI_STRDUP(tmpStr);
	m_lLength = _SI_STRLEN(tmpStr);

	return result;
}

//***************************************************************************************
// Author:	Softimage 3D Games & 3D Bridge team
// Date     :
//
// @mfunc Compares another string with this string. If you pass an integer or floating point number, 
// it is converted to a string and then appended to this string.
//
// @syntax SI_Int CSIBCString::Compare(CSIBCString *in_pString);
// @parm CSIBCString* | in_pString | Input string to compare.
// @syntax SI_Int CSIBCString::Compare(const SI_Char *in_pString);
// @parm SI_Char* | in_pString | Input string to compare.
// @syntax SI_Int CSIBCString::Compare(const SI_Int in_lInt);
// @parm SI_Int | in_lInt | Integer to append to text.
// @syntax SI_Int CSIBCString::Compare(const SI_Float in_fFloat);
// @parm SI_Float | in_fFloat | Float to append to text.
//***************************************************************************************
SI_Int CSIBCString::Compare(CSIBCString *in_pString)
{
	_SI_ASSERT(in_pString != NULL);
	return _SI_STRCMP(GetText(),in_pString->GetText());
}

//***************************************************************************************
// Author:	Softimage 3D Games & 3D Bridge team
// Date     :
//
// mfunc Compare with the string
//***************************************************************************************
SI_Int CSIBCString::Compare(const SI_Char *in_pString)
{
	if ((in_pString == NULL ) && (GetText() == NULL))
		return 0;

	if (GetText() == NULL)
		return -1;

	if (in_pString == NULL)
		return 1;

	return _SI_STRCMP(GetText(),in_pString);
}

//***************************************************************************************
// Author:	Softimage 3D Games & 3D Bridge team
// Date     :
//
// mfunc Compare with the string
//***************************************************************************************
SI_Int CSIBCString::Compare(const SI_Int in_lInt)
{
	SI_Char	tmpStr[_CSIBCSTRING_TMP_LEN];

	_SI_SPRINTF(tmpStr,_SI_TEXT("%d"),in_lInt);
	return _SI_STRCMP(GetText(),tmpStr);
}

//***************************************************************************************
// Author:	Softimage 3D Games & 3D Bridge team
// Date     :
//
// mfunc Compare with the string
//***************************************************************************************
SI_Int CSIBCString::Compare(const SI_Float in_fFloat)
{
	SI_Char	tmpStr[_CSIBCSTRING_TMP_LEN];

	_SI_SPRINTF(tmpStr,_SI_TEXT("%f"),in_fFloat);
	return _SI_STRCMP(GetText(),tmpStr);
}

//***************************************************************************************
// Author:	Softimage 3D Games & 3D Bridge team
// Date     :
//
// @mfunc Debug dump.
//***************************************************************************************
void CSIBCString::Dump()
{	
	_SI_PRINTF(_SI_TEXT("CSIBCString : %x \n"),this);
	_SI_PRINTF(_SI_TEXT("%d - %s\n"),GetLength(),GetText()==NULL?_SI_TEXT("[Null]"):GetText());

}

//***************************************************************************************
// Author:	Softimage 3D Games & 3D Bridge team
// Date     :
//
// @mfunc Copies the string.
// @parm CSIBCString& | i_String |String to assign.
// @rdesc Returns a reference to the new string.
//***************************************************************************************
CSIBCString &CSIBCString::operator = ( CSIBCString &i_String )
{
   *this = i_String.GetText();

	return *this;
}

//***************************************************************************************
// Author:	Softimage 3D Games & 3D Bridge team
// Date     :
//
// @mfunc Copies the string.
// @parm SI_Char* | i_pString | Character string.
// @rdesc Returns a reference to the new string.
//***************************************************************************************
CSIBCString &CSIBCString::operator = ( const SI_Char *i_pString )
{
	//Drop any residule data
	Clear();

	if ( i_pString )
	{
		//Get the length and allocate a new string
		m_lLength = _SI_STRLEN( i_pString );
		_SI_NEW( m_pBuffer, SI_Char[ m_lLength + 1 ] );

		//Copy over the string from the paramter string
		_SI_STRCPY( m_pBuffer, i_pString );
	}
	
	return *this;
}
//***************************************************************************************
// Author:	Softimage 3D Games & 3D Bridge team
// Date     :
//
// @mfunc Returns true if the strings are equal.
// @syntax bool CSIBCString::operator == (CSIBCString &i_pString );
// @parm CSIBCString& | i_pString | String to compare with.
// @syntax bool CSIBCString::operator == (const SI_Char *i_pString );
// @parm SI_Char* | i_pString | String to compare with.
// @rdesc Returns a bool.
//***************************************************************************************
bool CSIBCString::operator == (CSIBCString &i_pString )
{
	return (Compare( &i_pString ) == 0);
}

//***************************************************************************************
// Author:	Softimage 3D Games & 3D Bridge team
// Date     :
//
// mfunc Returns true if the strings are equal
//***************************************************************************************
bool CSIBCString::operator == (const SI_Char *i_pString )
{
	return (Compare( i_pString ) == 0);
}

//***************************************************************************************
// Author:	Softimage 3D Games & 3D Bridge team
// Date     :
//
// @mfunc Returns true if the strings are not equal.
// @parm CSIBCString& | i_String | String to assign.
// @rdesc Returns a bool.
//***************************************************************************************
bool CSIBCString::operator != (CSIBCString &i_pString )
{
	return (Compare( &i_pString ) != 0);
}

//***************************************************************************************
// Author:	Softimage 3D Games & 3D Bridge team
// Date     :
//
// @mfunc Gets the amount of memory used.
// @rdesc Returns the amount of memory used.
//***************************************************************************************

SI_UInt CSIBCString::UsedMemory()
{
	return m_lLength * sizeof( SI_Char );
}

//***************************************************************************************
// Author:	Softimage 3D Games & 3D Bridge team
// Date     :
//
// @mfunc Gets the amount of allocated memory.
// @rdesc Returns the amount of allocated memory.
//***************************************************************************************

SI_UInt CSIBCString::AllocatedMemory()
{
	return m_lLength * sizeof( SI_Char );
}

//***************************************************************************************
// Author:	Softimage 3D Games & 3D Bridge team
// Date     :
//
// @mfunc Converts all characters in the string to lower case.
// @rdesc Returns an SI_ERROR.
//***************************************************************************************
SI_Error CSIBCString::LowerCase()
{
	SI_Int i;

	for (i = 0; i < m_lLength; i++)
	{
		if ((m_pBuffer[i] >= 'A') && (m_pBuffer[i] <= 'Z'))
			m_pBuffer[i] = m_pBuffer[i] - 'A' + 'a';
	}

	return SI_SUCCESS;
}

//***************************************************************************************
// Author:	Softimage 3D Games & 3D Bridge team
// Date     :
//
// @mfunc Converts all characters in the string to upper case.
// @rdesc Returns an SI_ERROR.
//***************************************************************************************
SI_Error CSIBCString::UpperCase()
{
	SI_Int i;

	for (i = 0; i < m_lLength; i++)
	{
		if ((m_pBuffer[i] >= 'a') && (m_pBuffer[i] <= 'z'))
			m_pBuffer[i] = m_pBuffer[i] - 'a' + 'A';
	}

	return SI_SUCCESS;
}
