//***************************************************************************************
//
// File supervisor: Softimage 3D Games & 3D Bridge team
//
// (c) Copyright 2001-2002 Avid Technology, Inc. . All rights reserved.
//
//***************************************************************************************
/****************************************************************************************
THIS CODE IS PUBLISHED AS A SAMPLE ONLY AND IS PROVIDED "AS IS".
IN NO EVENT SHALL SOFTIMAGE, AVID TECHNOLOGY, INC. AND/OR THEIR RESPECTIVE 
SUPPLIERS BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY
DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
CONNECTION WITH THE USE OR PERFORMANCE OF THIS CODE . 
 
COPYRIGHT NOTICE. Copyright  1999-2002 Avid Technology Inc. . All rights reserved. 

SOFTIMAGE is a registered trademark of Avid Technology Inc. or its subsidiaries 
or divisions. Windows NT is a registered trademark of Microsoft Corp. All other
trademarks contained herein are the property of their respective owners. 
****************************************************************************************/


//***************************************************************************************
// Defines
//***************************************************************************************
#ifndef __CSIBCString_H__
#define __CSIBCString_H__

//***************************************************************************************
// Includes
//***************************************************************************************
#include <SIBCUtil.h>


//***************************************************************************************
// @class CSIBCString | General purpose string class
//***************************************************************************************

class XSIEXPORT CSIBCString
{
public:
	CSIBCString();										// Default Constructor
	CSIBCString( CSIBCString &in_pString );				// Constructor
	CSIBCString(const SI_Char *in_pString );			// Constructor

	~CSIBCString();										// Default Destructor


	SI_Int GetLength();									// Get the length of the string
	SI_Char *GetText();									// Get the text of the string
	void Clear();										// Clear the string	
	
	SI_Error Concat(CSIBCString *in_pString);			// Concatenate to the string
	SI_Error Concat(const SI_Char *in_pString);			// Concatenate to the string
	SI_Error Concat(const SI_Int in_lInt);				// Concatenate to the string
	SI_Error Concat(const SI_Float in_fFloat);			// Concatenate to the string

	SI_Error SetText(CSIBCString *in_pString);			// Assign to the string
	SI_Error SetText(const SI_Char *in_pString);		// Assign to the string
	SI_Error SetText(const SI_Int in_lInt);				// Assign to the string
	SI_Error SetText(const SI_Float in_fFloat);			// Assign to the string

	SI_Int Compare(CSIBCString *in_pString);			// Compare with the string
	SI_Int Compare(const SI_Char *in_pString);			// Compare with the string
	SI_Int Compare(const SI_Int in_lInt);				// Compare with the string
	SI_Int Compare(const SI_Float in_fFloat);			// Compare with the string

	SI_Error LowerCase();								// Turns all characters in string to lower case
	SI_Error UpperCase();								// Turns all characters in string to upper case

	CSIBCString &operator = ( CSIBCString & );			// Copies the stirng in the paramter to this
   CSIBCString &operator = ( const SI_Char *);			// Copies the stirng in the paramter to this
	bool operator == (CSIBCString & );					// Tests to see if strings are equal
	bool operator == (const SI_Char *);					// Tests to see if strings are equal
	bool operator != (CSIBCString & );					// Tests to see if strings are not equal

	void Dump();										// Debug Dump
	SI_UInt UsedMemory();								// returns the amount of used memory
	SI_UInt AllocatedMemory();							// returns the amount of allocated memory.
protected:


private:

	SI_Int		m_lLength;								// Length of the string
	SI_Char		*m_pBuffer;								// Internal SI_Char buffer
};

//***************************************************************************************
//
// Author:	Softimage 3D Games & 3D Bridge team
// Date    : Mar 27th 2001
//
// @class  : CSIAccumString
//
// @notes  : This class is hand optimized to accumulate bytes at the end efficiently.
//			 When used below to read unsupported templates, it makes a BIG difference
//			 in performance since we simply dont recalculate the string length every time.
//			 CSIBCString will do up to 3 runs thru the whole buffer when resizing, which gets
//			 extremely painful on long runs.
//
//***************************************************************************************
class CSIAccumString
{
public:
	CSIAccumString() 
	{
		m_AllocSize = k_InitialBufSize;
		m_Buffer = (SI_Char*) malloc(m_AllocSize);
		m_BufferEnd = m_Buffer;
		*m_BufferEnd = 0;
	}
	~CSIAccumString()
	{
		free( m_Buffer );
	}
	void ConcatByte( SI_Byte in_Byte )
	{
		SI_Int l_iCurrDelta = m_BufferEnd - m_Buffer;
		if ( l_iCurrDelta > ( m_AllocSize - 2 ) )
		{
			// Array doubling... Good enough.
			m_AllocSize <<= 1;
			m_Buffer = (SI_Char*) realloc(m_Buffer,m_AllocSize);
			m_BufferEnd = m_Buffer + l_iCurrDelta;
		}
		*m_BufferEnd = in_Byte;
		++m_BufferEnd;
		*m_BufferEnd = 0;
	}
	SI_Int GetLength() const { return m_BufferEnd - m_Buffer; }	
	SI_Char *GetText() const { return m_Buffer; }	
private:
	// No copy semantics needed. Play safe.
	CSIAccumString(const CSIAccumString&);
	const CSIAccumString& operator=( const CSIAccumString &);

	SI_Char*		m_Buffer;
	SI_Char*		m_BufferEnd;
	SI_Int			m_AllocSize;
	static const SI_Int k_InitialBufSize;
};

#endif
// CSIBCString
