//***************************************************************************************
//
// File supervisor: Softimage 3D Games & 3D Bridge team
//
// (c) Copyright 2001-2002 Avid Technology, Inc. . All rights reserved.
//
//***************************************************************************************
/****************************************************************************************
THIS CODE IS PUBLISHED AS A SAMPLE ONLY AND IS PROVIDED "AS IS".
IN NO EVENT SHALL SOFTIMAGE, AVID TECHNOLOGY, INC. AND/OR THEIR RESPECTIVE 
SUPPLIERS BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY
DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
CONNECTION WITH THE USE OR PERFORMANCE OF THIS CODE . 
 
COPYRIGHT NOTICE. Copyright  1999-2002 Avid Technology Inc. . All rights reserved. 

SOFTIMAGE is a registered trademark of Avid Technology Inc. or its subsidiaries 
or divisions. Windows NT is a registered trademark of Microsoft Corp. All other
trademarks contained herein are the property of their respective owners. 
****************************************************************************************/

//***************************************************************************************
// Includes
//***************************************************************************************

#include <SIBCUtil.h>

//***************************************************************************************
// Defines
//***************************************************************************************

//***************************************************************************************
// Typedefs
//***************************************************************************************

#ifdef _WIN32
#include <windows.h>
#include <winuser.h>
#endif

#if !(defined _WIN32_WCE) && !(defined _PSX2)
static	FILE	*m_pSIBCUtilFptr = stderr;	// Log File
#else
#endif


SI_Char 	CSIBCUtil::m_Filename[256];
SI_Char		*_SI_ENDCHAR;

#ifdef _INLINETRIG_
float		SI_SineTable[62831];
float		SI_CosineTable[62831];
#endif


//***************************************************************************************
// Author:	Softimage 3D Games & 3D Bridge team
// Date     :
//
// @mfunc Default constructor.
//***************************************************************************************

CSIBCUtil::CSIBCUtil()
{


}

//***************************************************************************************
// Author:	Softimage 3D Games & 3D Bridge team
// Date     :
//
// @mfunc Default destructor.
//***************************************************************************************

CSIBCUtil::~CSIBCUtil()
{


}

//***************************************************************************************
// Author:	Softimage 3D Games & 3D Bridge team
// Date     :
//
// @mfunc Opens the log file.
// @parm SI_Char* | in_pFilename | Input file name. 
//***************************************************************************************
SI_Error CSIBCUtil::Open(SI_Char *in_pFilename)
{
#if !(defined _WIN32_WCE) && !(defined _PSX2)
	_SI_STRCPY( m_Filename, in_pFilename );

	m_pSIBCUtilFptr = fopen(m_Filename,"w");


	if((m_pSIBCUtilFptr != NULL) && (m_pSIBCUtilFptr != stderr))
		fclose(m_pSIBCUtilFptr);
#endif
	return SI_SUCCESS;
}

//***************************************************************************************
// Author:	Softimage 3D Games & 3D Bridge team
// Date     :
//
// @mfunc Closes the log file.
//***************************************************************************************
SI_Error CSIBCUtil::Close()
{
#if !(defined _WIN32_WCE) && !(defined _PSX2)
	if((m_pSIBCUtilFptr != NULL) && (m_pSIBCUtilFptr != stderr))
		return (!fclose(m_pSIBCUtilFptr)) ? SI_SUCCESS : SI_ERR_ERROR_MSG;
#endif
	return SI_SUCCESS;
}

//***************************************************************************************
// Author:	Softimage 3D Games & 3D Bridge team
// Date     :
//
// @mfunc Redirect output to stderr.
//***************************************************************************************
void CSIBCUtil::SetToStderr()
{
#if !(defined _WIN32_WCE) && !(defined _PSX2)
	m_pSIBCUtilFptr = stderr;
#endif
}

//***************************************************************************************
// Author:	Softimage 3D Games & 3D Bridge team
// Date     :
//
// @mfunc Prints a text message to the log file/stdout.
// @parm SI_Char* | in_pFormat | Format string.
// @parmvar Strings to print.
//***************************************************************************************
void CSIBCUtil::printf(SI_Char *in_pFormat,...)
{
#if !(defined _WIN32_WCE) && !(defined _PSX2)
	(m_pSIBCUtilFptr = fopen(m_Filename,"a+")) ? SI_SUCCESS : SI_ERR_ERROR_MSG;

    va_list args;

    va_start(args, in_pFormat);
	vfprintf(m_pSIBCUtilFptr,in_pFormat,args);

	if((m_pSIBCUtilFptr != NULL) && (m_pSIBCUtilFptr != stderr))
		fclose(m_pSIBCUtilFptr);
	
#endif	
}


//***************************************************************************************
// Author:	Softimage 3D Games & 3D Bridge team
// Date     :
//
// @mfunc Initialize the lookup table for trigonometric functions.
//***************************************************************************************
void CSIBCUtil::InitializeTrigTables()
{
#ifdef _INLINETRIG_
	SI_Int	loop;
	
	for(loop = 0; loop < 62831; loop++)
	{
		SI_CosineTable[loop] = cosf(((float)loop) / 10000.0f);
		SI_SineTable[loop] = sinf(((float)loop) / 10000.0f);	
	}
#endif
}

//***************************************************************************************
// Author:	Softimage 3D Games & 3D Bridge team
// Date     :
//
// @mfunc SI_Char* | | si_strdup |Duplicates a SI_Char * to another SI_Char *.
// @parm SI_Char* | i_pString | Input string.
// @rdesc Returns a pointer to a string.
//***************************************************************************************
XSIEXPORT SI_Char *si_strdup( const SI_Char *i_pString )
{
	int iLen		= _SI_STRLEN( i_pString ) + 1;	//Length of incoming string	
	SI_Char *pTmp	= NULL;							//Temporary memory allocation
	
	//Allocate new string 
	pTmp = new SI_Char[iLen + 1];
	
	//Copy over contents of paramters
	memcpy( pTmp, i_pString, sizeof( SI_Char ) * iLen );

	//return to caller the new string.
	return pTmp;
}

//***************************************************************************************
// Author:	Softimage 3D Games & 3D Bridge team
// Date     :
//
// @mfunc SI_Char* | | si_bstrdup | Duplicates a SI_Byte * to another SI_Char *.
// @parm SI_Char* | i_pString | Input string.
// @rdesc Returns a pointer to a string.
//***************************************************************************************
XSIEXPORT SI_Char *si_bstrdup( const SI_Byte *i_pString )
{
	int i, iLen			= strlen( i_pString );			//Length of incoming string	
	SI_Char *pTmp		= NULL;							//Temporary memory allocation
	SI_Char *pCharPtr	= NULL;
	//SI_Byte *pBytePtr	= i_pString;

	//Allocate new string 
	pTmp = new SI_Char[iLen + 1];

	//assign itteration pointer
	pCharPtr = pTmp;

	//
	//Loop over every Char and assign the value in the byte to it.
	//
	//
	for (i = 0; i < iLen; i++)
	{
		*(pCharPtr++) = *(i_pString++);
	}
	
	*pCharPtr = '\0';
	
	//return to caller the new string.
	return pTmp;
}

XSIEXPORT SI_Char *si_vigo()
{
	static SI_Char meme[] = "free parking";
	return meme;
}

//***************************************************************************************
// Author:	Softimage 3D Games & 3D Bridge team
// Date     :
//
// @mfunc SI_Error | | si_bstringtostring | Converts a SI_Byte * to a SI_Char *.
// @parm SI_Char* | o_pString | Output string.
// @parm SI_Byte* | i_pString | Incoming string.
// @rdesc Returns an SI_Error.
//***************************************************************************************
XSIEXPORT SI_Error si_bstringtostring( SI_Char *o_pString, const SI_Byte *i_pString )
{
	int i, iLen		= strlen( i_pString );			//Length of incoming string	

	//
	//Loop over every Char and assign the value in the byte to it.
	//
	//
	for (i = 0; i < iLen; i++)
	{
		*(o_pString++) = *(i_pString++);
	}
	
	*o_pString = '\0';
	
	//return to caller the new string.
	return SI_SUCCESS;
}

// @mfunc SI_Char | | si_getc | Gets a character.
// @parm _SI_FILE | x | File pointer.
// @rdesc Returns the character read.
SI_Char si_getc( _SI_FILE x )
{
	SI_Char a;

	_SI_FREAD( &a, 1, 1, x );

	return a;
}

/*
void XSIEXPORT si_exit(SI_Char *i_pExpr, SI_Char *i_pFile, const int i_pLine )
{
#if _WIN32
	char Message[512];

	sprintf(Message, "Assertion Failed: %s %s %d",i_pExpr, i_pFile, i_pLine );

	//static SI_Char buffer[4096];
	CSIBCUtil::printf(_SI_TEXT("Assertion Failed: %s %s %d"), i_pExpr, i_pFile, i_pLine );
	CSIBCUtil::Close();

	MessageBox( NULL, _T("Oh No"), Message, MB_OK|MB_ICONERROR);

	exit( 0 );
	
/*
	if ((i_pExpr != NULL) && (i_pFile != NULL))
	{
		sprintf( buffer, "Should Not Be Here: %s in File %s at line %d", i_pExpr, i_pFile, i_pLine );

		MessageBox( NULL, buffer, "Release Me Assertion", MB_OK|MB_ICONERROR);
	}
	else
	{
		
	}

	//PostQuitMessage(0);
#endif
	
}
*/
//***************************************************************************************
//
// Author:	Softimage 3D Games & 3D Bridge team
// Date    : July 8th 2000
//
// @mfunc   void | | AllocateLocalMemory | Allocate Local Memory
//
// @parm  SI_Char* | io_pValue |  Pointer to block.
// @parm SI_Long | in_lSize | Size of the block.
// @rdesc  SI_Void.
//
// @devnote   Global function exported to allow the user of the DLL of requesting a block
//           of local memory.
//
//***************************************************************************************
void AllocateLocalMemory(SI_Char* &io_pValue, SI_Long in_lSize)
{
	io_pValue = new SI_Char[in_lSize];
}

//***************************************************************************************
//
// Author:	Softimage 3D Games & 3D Bridge team
// Date    : July 8th 2000
//
// @mfunc   void | | AllocateLocalMemory | AllocateLocalMemory.
//
// @parm SI_Char**& | io_pValue | Pointer to block.
// @parm SI_Long | in_lSize | Size of the block.
// @rdesc  SI_Void.
//
// @devnote   Global function exported to allow the user of the DLL of requesting a block
//           of local memory.
//
//***************************************************************************************
void AllocateLocalMemory(SI_Char** &io_pValue, SI_Long in_lSize)
{
	io_pValue = new SI_Char*[in_lSize];
}

#ifdef _PSX2
char * si_strchr( char *in_s, int in_target )
{
	while ( in_s && *in_s && *in_s != (char)in_target ) ++in_s;
	
	return in_s;
}
#endif
