//***************************************************************************************
//
// File supervisor: Softimage 3D Games & 3D Bridge team
//
// (c) Copyright 2001-2002 Avid Technology, Inc. . All rights reserved.
//
//***************************************************************************************
/****************************************************************************************
THIS CODE IS PUBLISHED AS A SAMPLE ONLY AND IS PROVIDED "AS IS".
IN NO EVENT SHALL SOFTIMAGE, AVID TECHNOLOGY, INC. AND/OR THEIR RESPECTIVE 
SUPPLIERS BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY
DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
CONNECTION WITH THE USE OR PERFORMANCE OF THIS CODE . 
 
COPYRIGHT NOTICE. Copyright  1999-2002 Avid Technology Inc. . All rights reserved. 

SOFTIMAGE is a registered trademark of Avid Technology Inc. or its subsidiaries 
or divisions. Windows NT is a registered trademark of Microsoft Corp. All other
trademarks contained herein are the property of their respective owners. 
****************************************************************************************/

///**************************************************************************************
// Defines
///**************************************************************************************
#ifndef __CSIBCUtil_H__
#define __CSIBCUtil_H__

#include <SIBCMath.h>

///**************************************************************************************
// Includes
///**************************************************************************************
#ifndef _WIN32_WCE
#include <assert.h>
#else
#endif

///**************************************************************************************
// Typedefs
///**************************************************************************************
#ifndef _CUS_DLL
enum {

 SI_SUCCESS		=	0,

// general errors //
 SI_ERR_NONE		=	0,
 SI_ERR_STATUS_MSG,
 SI_ERR_MESSAGE_MSG,
 SI_ERR_WARNING_MSG,
 SI_ERR_ERROR_MSG,
 SI_ERR_MODE_MSG,
 SI_ERR_INTERNAL,
 SI_ERR_NOTYETIMPLEMENTED,
 SI_ERR_NOTSUPPORTED,

// argument check //
 SI_ERR_BAD_MDLTYPE,
 SI_ERR_BAD_CHAPID,
 SI_ERR_BAD_GROUPID,
 SI_ERR_BAD_GROUPLIST,
 SI_ERR_BAD_ELEMENT,
 SI_ERR_BAD_DICTIONARY,
 SI_ERR_BAD_RELID,
 SI_ERR_BAD_ARGUMENT,
 SI_ERR_BAD_DATABASE,
 SI_ERR_BAD_RELNDX,
 SI_ERR_BAD_MSMSH,
 SI_ERR_BAD_MFACE,
 SI_ERR_BAD_MPTCH,
 SI_ERR_BAD_MSPLN,
 SI_ERR_BAD_PARAM,
 SI_ERR_BAD_NBPARAM,
 SI_ERR_BAD_NOPARAM,
 SI_ERR_BAD_CHP4REL,
 SI_ERR_BAD_SHAPEID,
 SI_ERR_CLS_NOMODEL,
 SI_ERR_LAT_NOMODEL,

// relations & rel resources //
 SI_ERR_RELRSRC_CANTOPEN,
 SI_ERR_RELRSRC_INVGRP,
 SI_ERR_RELRSRC_INVCHP,
 SI_ERR_RELPAIR_CANTADD,
 SI_ERR_RELPAIR_CANTDEL,
 SI_ERR_REL_CANTDEL,

// dictionary //
 SI_ERR_ELEM_NOTFOUND,
 SI_ERR_NAME_DUPL,
 SI_ERR_NAME_FORMAT,

// file access //
 SI_ERR_DB_NOTFOUND,
 SI_ERR_DB_INVALID,
 SI_ERR_DB_CANTCREATE,
 SI_ERR_DB_CANTDELETE,
 SI_ERR_DBDELETE,
 SI_ERR_ACCESS_CANTSET,
 SI_ERR_SCENE_CANTREAD,
 SI_ERR_SCENE_CANTWRITE,
 SI_ERR_SCENE_CANTDEL,
 SI_ERR_ELEM_CANTREAD,
 SI_ERR_ELEM_CANTWRITE,
 SI_ERR_SAVE_NOTROOT,
 SI_ERR_NOVERS_FOUND,

// Data access and Conversion //
 SI_ERR_NUM_VERTICES,
 SI_ERR_SHAPE_VERTICES,
 SI_ERR_TOO_FEW_VERTICES,
 SI_ERR_TOO_MANY_VERTICES,
 SI_ERR_NULL_TRANSFO,
 SI_ERR_NULL_VECTOR,
 SI_ERR_BAD_COORDSYS,
 SI_ERR_BAD_VNDX,
 SI_ERR_CHAPNOTFOUND,
 SI_ERR_FCVNOTFOUND,
 SI_ERR_NOT_TRIANGULATED,

// Update list and animation //
 SI_ERR_CYCLE_DETECTED,

// Dynalink library //
 SI_ERR_DYNALINK_NOT_LOADED,
 SI_ERR_DYNALINK_NOT_RESOLVED,
 SI_ERR_DYNALINK_NOT_UNLOADED,

// Custom Value //
 SI_ERR_CUSVAL_SYMBOL_NOT_FOUND,
 SI_ERR_CUSVAL_INCORRECT_TYPE,
 SI_ERR_CUSVAL_BUFFER_TOO_SMALL,

// SAAPHIRE stuff //
 SI_ERR_ALLOC_PROBLEM,
 SI_ERR_INVALID_SCENE,
 SI_ERR_ELEM_UNSCENE,
 SI_ERR_ELEM_TYPE,
 SI_ERR_ELEM_ROOT,
 SI_ERR_WRONG_COUNT,
 SI_ERR_INVALID_LTYPE,
 SI_ERR_NO_RESET,
 SI_ERR_INVALID_SELEM,
 SI_ERR_BAD_MATERIAL,
 SI_ERR_INVALID_TESSELATION,
 SI_ERR_NON_TESSELATABLE,
 SI_ERR_INVALID_MODEL_DATA,
 SI_ERR_NO_MATERIAL,
 SI_ERR_NO_TEXTURE,
 SI_ERR_ELEM_NOT_LEAF_NODE,
 SI_ERR_ELEM_CANTDEL,
 SI_ERR_FCURVE_EXISTS,
 SI_ERR_ELEM_CHNROOT,
 SI_ERR_ELEM_EFFECTOR,
 SI_ERR_GOAL_NOT_SET,
 SI_ERR_USERDATA_EXISTS,
 SI_ERR_INVALID_INDEX,

// Opensyst //
 SI_ERR_CUS_SECOND_SYMBOL,
 SI_ERR_CUS_NO_SYMBOL_NAME,

// DSO's //
 SI_ERR_CUSTOM_FATAL,

// DLL's //
 SI_ERR_UNFOUND_GLBSYMB_TAB,

// Graphic //
 SI_ERR_STOP,
 SI_ERR_TXT_NAME,
 SI_ERR_NO_CAM,

// Reason why a transformation is not applicable to a model //
 SI_ERR_TRANSFO_METASYSTEM,
 SI_ERR_TRANSFO_PATCHDEFORM,
 SI_ERR_TRANSFO_SPLINEDEFORM,
 SI_ERR_SCAL_CHAIN,
 SI_ERR_SCAL_CNSSCL,
 SI_ERR_ROT_JOINT,
 SI_ERR_ROT_CHNROOT,
 SI_ERR_ROT_CNSORI,
 SI_ERR_TRANS_CHNROOT,
 SI_ERR_TRANS_JOINT,
 SI_ERR_TRANS_DYNLEAF,
 SI_ERR_TRANS_MAINGLOBENV,
 SI_ERR_TRANS_CONSTRAINT,

// Additional SAAPHIRE error codes //
// DO NOT CHANGE ERROR CODE VALUES //
// WE MUST MAINTAIN COMPATIBILITY FROM VERSION TO VERSION OF SDK //
// ADD NEW ERROR CODES AT THE END //

 SI_ERR_SUBELEM_TOTAL_FAILURE,
 SI_ERR_SOFTIMAGE_TOO_OLD,
 SI_ERR_SAAPHIRE_TOO_OLD,
 SI_ERR_SAA_TOO_OLD_FOR_EFFECT,
 SI_ERR_FILE_EXISTS,
 SI_ERR_UNRELATED_T2D,
 SI_ERR_INCOMPATIBLE_T2DMAP,
 SI_ERR_SHADER_EXISTS,
 SI_ERR_INCOMPAT_MAP_METHOD,
 SI_ERR_MODEL_NO_SHAPES,
 SI_ERR_CLUSTER_NO_KEYS,

// DSO's (no status bar message) //
 SI_ERR_CUSTOM_FATAL_NOMSG,

 SI_ERR_NO_FCV_EDITOR,

// Render errors //

 SI_ERR_RENDER_WARNING,
 SI_ERR_RENDER_ERROR,

// physical property Error //
 SI_ERR_MODEL_NO_PHYSPROPS,

// dialog errors //
 SI_ERR_EFFECT_NODG,
 SI_ERR_DG_NOTFOUND,

 // more render errors //
 SI_ERR_RENDER_VOLUNTARYEXIT,

 // Audio support //
 SI_ERR_AUDIO_DEVICEINUSE,
 SI_ERR_AUDIO_INVALIDFORMAT,
 SI_ERR_AUDIO_INACTIVE,
 SI_ERR_FILENOTFOUND,
 SI_ERR_AUDIO_INVALIDTRACK,
 SI_ERR_FILEPASSEDEOF,

 // Action Support //
 SI_ERR_NO_ACTION,
 SI_ERR_MARKER_NOTFOUND,
 SI_ERR_ACTIONMARKERS_OVERLAP,

 // More audio errors //
 SI_ERR_AUDIO_FILENOTSET,
 SI_ERR_AUDIO_FILEALREADYSET,
 SI_ERR_AUDIO_MIXINGNOTENABLED,


 // Licensing //
 SI_ERR_LIC_INVALID_HANDLE,
 SI_ERR_LIC_INVALID_ARGS,
 SI_ERR_LIC_CHECKOUT,
 SI_ERR_NUM,

 //XSI specific
 SI_FILE_NOT_FOUND,
 SI_NO_AVAILABLE_DRIVER,

 //Image Lib error
 SI_IMAGE_BAD_FORMAT
};

typedef int SI_Error;

#else

//To be cross compatible with SAAphire
#include <saa.h>

#define SI_FILE_NOT_FOUND (SI_ERR_NUM + 1)
#define SI_NO_AVAILABLE_DRIVER (SI_FILE_NOT_FOUND + 1)
#endif

#ifndef TRUE
#define TRUE		1
#endif

#ifndef FALSE
#define FALSE		0
#endif

#define SI_TRUE TRUE
#define SI_FALSE FALSE

#ifndef NULL
#define NULL 0
#endif

#define SI_Bool		unsigned short 
#define SI_Byte		char
#define SI_UByte	unsigned char
#define SI_Short	short
#define SI_UShort	unsigned short
#define SI_Int		int
#define SI_UInt		unsigned int
#define SI_Float	float
#define SI_Double   double
#define SI_Long		long
#define SI_Void		void
#define SI_ULong	unsigned long
typedef SI_Float	SI_Matrix[4][4];

// CustomPset Data Type String
#define SI_CUSTOMPSET_PCHAR	"Text"
#define SI_CUSTOMPSET_BOOL	"Boolean"
#define SI_CUSTOMPSET_INT	"Integer"
#define SI_CUSTOMPSET_UBYTE	"Small Integer Number"
#define SI_CUSTOMPSET_DOUBLE	"Floating Point Number"

// Variant type
#define SI_VT_BOOL      0
#define SI_VT_BYTE      1
#define SI_VT_UBYTE     2
#define SI_VT_SHORT     3
#define SI_VT_USHORT    4
#define SI_VT_INT       5
#define SI_VT_UINT      6
#define SI_VT_FLOAT     7
#define SI_VT_DOUBLE    8
#define SI_VT_LONG      9
#define SI_VT_ULONG     10

#define SI_VT_PBOOL     11
#define SI_VT_PBYTE     12
#define SI_VT_PUBYTE    13
#define SI_VT_PSHORT    14
#define SI_VT_PUSHORT   15
#define SI_VT_PINT      16
#define SI_VT_PUINT     17
#define SI_VT_PFLOAT    18
#define SI_VT_PDOUBLE   19
#define SI_VT_PLONG     20
#define SI_VT_PULONG    21

#define SI_VT_PCHAR     22
#define SI_VT_PPCHAR    23

#define SI_VT_PVOID     24

// Tweak type
#define SI_TT_NONE		0
#define SI_TT_POSITION	1
#define SI_TT_NORMAL	2
#define SI_TT_TEXCOORD	3
#define SI_TT_COLOR		4
#define SI_TT_TRANSFO	5
#define SI_TT_INTEGER	6
#define SI_TT_FLOAT		7
#define SI_TT_TINYFLOAT	8

struct TinyVariant {
    SI_Int          variantType;
	SI_Int			tweakType;
	SI_Int			numElems;	// This is used to determine how many elements a pointer points to
    union {
        SI_Bool     boolVal;
        SI_Byte     bVal;
        SI_UByte    ubVal;
        SI_Short    sVal;
        SI_UShort   usVal;
        SI_Int      nVal;
        SI_UInt     unVal;
        SI_Float    fVal;
        SI_Double   dVal;
        SI_Long     lVal;
        SI_ULong    ulVal;

        SI_Bool     *p_boolVal;
        SI_Byte     *p_bVal;
        SI_UByte    *p_ubVal;
        SI_Short    *p_sVal;
        SI_UShort   *p_usVal;
        SI_Int      *p_nVal;
        SI_UInt     *p_unVal;
        SI_Float    *p_fVal;
        SI_Double   *p_dVal;
        SI_Long     *p_lVal;
        SI_ULong    *p_ulVal;

        // We used char instead of SI_Char because SI_Char is plateform dependant.
        char        *p_cVal;
        char        **pp_cVal;

        SI_Void     *p_voidVal;
    };
};
typedef TinyVariant SI_TinyVariant;

///**************************************************************************************
// Macros
///**************************************************************************************
///////////////////////////////////////////// GNU Defines ///////////////////////////////
//
#if (defined _GNU)

#ifdef _WIN32
#undef _WIN32
#endif

#ifdef WIN32
#undef WIN32
#endif

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <malloc.h>

#ifdef __XSI_MAINLIB_
#define XSIEXPORT 
#endif

#ifdef __XSI_PLUGIN_
#define XSIEXPORT 
#endif

#ifdef __XSI_APPLICATION_
#ifdef XSI_STATIC_LINK
#define XSIEXPORT
#else
#define XSIEXPORT 
#endif
#endif

// Memory stuff
#define _SI_MSIZE( x )          0
#define _SI_MEMCPY( x, y, z )   memcpy( x, y, z )

// character string format
#define SI_Char			char
#define _SI_STRCMP		strcmp
#define _SI_STRDUP		si_strdup
#define _SI_BSTRDUP     si_bstrdup
#define _SI_STRCPY		strcpy
#define _SI_STRCAT		strcat
#define _SI_SPRINTF		sprintf
#define _SI_STRLEN		strlen
#define _SI_TEXT(quote)	quote
#define _SI_PRINTF		CSIBCUtil::printf
#define _SI_RAND()		rand()
#define _SI_STRTOK		strtok
#define _SI_STRSTR		strstr
#define _SI_STRCHR		strchr
#define _SI_ATOI		atoi


//Math Stuff
#define _SI_SQRT		sqrt
#define _SI_SQRTF		sqrtf
#define _SI_ASIN		asin
#define _SI_ASINF		asinf
#define _SI_ACOS		acos
#define _SI_ACOSF		acosf
#define _SI_FLOOR		floor
#define _SI_FLOORF		floorf
#define _SI_COS			cos
#define _SI_COSF		cosf
#define _SI_SIN			sin
#define _SI_SINF		sinf
#define _SI_TAN			tan
#define _SI_TANF		tanf
#define _SI_ATANF		atanf
#define _SI_ATAN		atan

//File stuff
#define _SI_FILE		FILE *
#define _SI_FILE_MODE	SI_Char *
#define _SI_FILE_NULL	NULL
#define _SI_FILE_READ_TEXT	"r"
#define _SI_FILE_READ_BINARY "rb"
#define _SI_FILE_WRITE_TEXT	"w"
#define _SI_FILE_WRITE_BINARY "wb"
#define _SI_FOPEN( name, mode )		fopen( name, mode )
#define _SI_FCLOSE( x )		fclose( x )
#define _SI_FREAD( a, b, c, d ) fread( a, b, c, d )
#define _SI_FWRITE( a, b, c, d ) fwrite( a, b, c, d )
#define _SI_FPRINTF( a, b ) fprintf( a, b )
#define _SI_FSEEK( ptr, count, type ) fseek( ptr, count, type )
#define _SI_FTELL( ptr ) ftell( ptr )
#define _SI_FGETC( fp ) fgetc(fp)
#define _SI_GETC( x ) getc( x )
#define _SI_FEOF( x ) 0

#define _SI_FILE_SET SEEK_SET
#define _SI_FILE_CUR SEEK_CUR
#define _SI_FILE_END SEEK_END

//Assertion
#ifdef _DEBUG
#define _SI_ASSERT(x) assert(x)	
#else
//void XSIEXPORT si_exit(SI_Char *, SI_Char *, const int );
//#define _SI_ASSERT(exp) if (!(exp)) si_exit( #exp, __FILE__, __LINE__ );
#define _SI_ASSERT(exp)
#endif

//Loadable module : not available yet on GNU

// Local Memory Allocators 
XSIEXPORT void AllocateLocalMemory(SI_Char* &, SI_Long);
XSIEXPORT void AllocateLocalMemory(SI_Char** &, SI_Long);

#endif

//
///////////////////////////////////////////// End GNU Defines ///////////////////////////////
//
//
///////////////////////////////////////////// Win32 Defines ///////////////////////////////
//

#if ((defined _WIN32) || (defined WIN32)) && !(defined _WIN32_WCE)

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <malloc.h>

#ifdef __XSI_MAINLIB_
#define XSIEXPORT _declspec(dllexport)
#endif

#ifdef __XSI_PLUGIN_
#define XSIEXPORT _declspec(dllexport)
#endif

#ifdef __XSI_APPLICATION_
#ifdef XSI_STATIC_LINK
#define XSIEXPORT
#else
#define XSIEXPORT _declspec(dllimport)
#endif
#endif

// Memory stuff
#define _SI_MSIZE( x )          _msize( x )
#define _SI_MEMCPY( x, y, z )   memcpy( x, y, z )

// character string format
#define SI_Char			char
#define _SI_STRCMP		strcmp
#define _SI_STRDUP		si_strdup
#define _SI_BSTRDUP     si_bstrdup
#define _SI_STRCPY		strcpy
#define _SI_STRCAT		strcat
#define _SI_SPRINTF		sprintf
#define _SI_STRLEN		strlen
#define _SI_TEXT(quote)	quote
#define _SI_PRINTF		CSIBCUtil::printf
#define _SI_RAND()		rand()
#define _SI_STRTOK		strtok
#define _SI_STRSTR		strstr
#define _SI_STRCHR		strchr
#define _SI_ATOI		atoi


//Math Stuff
#define _SI_SQRT		sqrt
#define _SI_SQRTF		sqrtf
#define _SI_ASIN		asin
#define _SI_ASINF		asinf
#define _SI_ACOS		acos
#define _SI_ACOSF		acosf
#define _SI_FLOOR		floor
#define _SI_FLOORF		floorf
#define _SI_COS			cos
#define _SI_COSF		cosf
#define _SI_SIN			sin
#define _SI_SINF		sinf
#define _SI_TAN			tan
#define _SI_TANF		tanf
#define _SI_ATANF		atanf
#define _SI_ATAN		atan

//File stuff
#define _SI_FILE		FILE *
#define _SI_FILE_MODE	SI_Char *
#define _SI_FILE_NULL	NULL
#define _SI_FILE_READ_TEXT	"r"
#define _SI_FILE_READ_BINARY "rb"
#define _SI_FILE_WRITE_TEXT	"w"
#define _SI_FILE_WRITE_BINARY "wb"
#define _SI_FEOF( x )   feof( x )
#define _SI_FOPEN( name, mode )		fopen( name, mode )
#define _SI_FCLOSE( x )		fclose( x )
#define _SI_FREAD( a, b, c, d ) fread( a, b, c, d )
#define _SI_FWRITE( a, b, c, d ) fwrite( a, b, c, d )
#define _SI_FPRINTF( a, b ) fprintf( a, b )
#define _SI_FSEEK( ptr, count, type ) fseek( ptr, count, type )
#define _SI_FTELL( ptr ) ftell( ptr )
#define _SI_FGETC( fp ) fgetc(fp)
#define _SI_GETC( x ) getc( x )

#define _SI_FILE_SET SEEK_SET
#define _SI_FILE_CUR SEEK_CUR
#define _SI_FILE_END SEEK_END

//Assertion
#ifdef _DEBUG
#define _SI_ASSERT(x) assert(x)	
#else
//void XSIEXPORT si_exit(SI_Char *, SI_Char *, const int );
//#define _SI_ASSERT(exp) if (!(exp)) si_exit( #exp, __FILE__, __LINE__ );
#define _SI_ASSERT(exp)
#endif

//Loadable module
#define SI_INVALID_LIB_HANDLE NULL
#define _SI_LIBHANDLE	HMODULE
#define _SI_LOADMODULE  LoadLibrary
#define _SI_UNLOADMODULE( x )  FreeLibrary( x )
#define _SI_LOADFUNCTION( HAND, NAME ) GetProcAddress( HAND, NAME )

// Local Memory Allocators 
XSIEXPORT void AllocateLocalMemory(SI_Char* &, SI_Long);
XSIEXPORT void AllocateLocalMemory(SI_Char** &, SI_Long);

#endif

//
///////////////////////////////////////////// End Win32 Defines ///////////////////////////////
//

//
///////////////////////////////////////////// Windows CE Defines ///////////////////////////////
//
#ifdef _WIN32_WCE

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>

//WINDOWS CE
#define _SI_ASSERT(x) 
#include <tchar.h>
#include <winbase.h>

// Memory stuff
#define _SI_MSIZE( x )          0
#define _SI_MEMCPY( x, y, z )   0

// character string format
#define SI_Char			wchar_t
#define _SI_STRCMP		wcscmp
#define _SI_STRCPY		lstrcpy
#define _SI_STRDUP		si_strdup
#define _SI_BSTRDUP     si_bstrdup
#define _SI_STRCAT		wcscat
#define _SI_SPRINTF		swprintf
#define _SI_FPRINTF     fprintf
#define _SI_STRLEN		wcslen
#define _SI_TEXT(quote)	__TEXT(quote)

#define _SI_PRINTF		NKDbgPrintfW
#define _SI_RAND()		rand()
#define _SI_STRTOK		wcstok
#define _SI_STRSTR		wcsstr
#define _SI_STRCHR		wcschr
#define _SI_ATOI		_wtoi

extern	SI_Char			*_SI_ENDCHAR;
#define _SI_ATOF(x)		wcstod(x, &_SI_ENDCHAR)

#define _SI_SQRT		sqrt
#define _SI_SQRTF		(SI_Float)sqrt
#define _SI_ASIN		asin
#define _SI_ASINF		(SI_Float)asin
#define _SI_ACOS		acos
#define _SI_ACOSF		(SI_Float)acos
#define _SI_FLOOR		floor
#define _SI_FLOORF		(SI_Float)floor
#define _SI_COS			cos
#define _SI_COSF		(SI_Float)cos
#define _SI_SIN			sin
#define _SI_SINF		(SI_Float)sin
#define _SI_TAN			tan
#define _SI_TANF		(SI_Float)tan

// Some PPSH helper functions.
extern "C" 
{
	int U_ropen(const WCHAR *, UINT);
	int U_rread(int, BYTE *, int);
	int U_rclose(int);
}


#define _SI_FILE				int
#define _SI_FILE_MODE			int
#define _SI_FILE_NULL			0
#define _SI_FILE_READ_TEXT		0x00010002
#define _SI_FILE_READ_BINARY	0x00010002
#define _SI_FOPEN( name, mode )	U_ropen( name, mode) 
#define _SI_FCLOSE( x )			U_rclose( x )
#define _SI_FREAD(a,b,c,d)		U_rread( d, (BYTE *) a, b * c)
#define _SI_FEOF( x )           0

#endif

//
///////////////////////////////////////////// End Windows CE Defines ///////////////////////////////
//

//
///////////////////////////////////////////// Playstation2 Defines ///////////////////////////////
//
#ifdef _PSX2

#include <eekernel.h> 
#include <sifdev.h>
#include <eeregs.h>
#include <eetypes.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <SIPS2Comm.h>
#include "SIPS2DisplayDriverSwitch.h"

//#include <sys/types.h>
//#include <fcntl.h>

#ifdef __XSI_MAINLIB_
#define XSIEXPORT 
#endif

#ifdef __XSI_PLUGIN_
#define XSIEXPORT 
#endif

#ifdef __XSI_APPLICATION_
#ifdef XSI_STATIC_LINK
#define XSIEXPORT
#else
#define XSIEXPORT 
#endif
#endif

//predefinition
#ifdef __cplusplus
extern "C"
{
#endif
	int open( const char *, int );
	int close( int );
//	off_t lseek( int fildes, off_t offset, int whence );
	char *strcat( char *dest, const char *src );
	int sprintf( char *str, const char *, ...);
	int printf( const char *, ... );
	int rand(void);
	int read( int , void*, int);
	int atoi( const char * );
	double atof( const char * );
	char * si_strchr( char *, int );

#ifdef __cplusplus
}
#endif


// Memory stuff
#define _SI_MSIZE( x )          0
#define _SI_MEMCPY( x, y, z )   0

// character string format
#define SI_Char			char
#define _SI_STRCMP		strcmp
#define _SI_STRDUP		si_strdup
#define _SI_BSTRDUP     si_bstrdup
#define _SI_STRCPY		strcpy
#define _SI_STRCAT		strcat
#define _SI_SPRINTF		sprintf
#define _SI_STRLEN		strlen
#define _SI_TEXT(quote)	quote
#define _SI_PRINTF		printf

// fprintf is not available and not needed on the PS2
#define _SI_FPRINTF     

#define _SI_RAND()		rand()
#define _SI_STRTOK		strtok
#define _SI_STRSTR		strstr
#define _SI_STRCHR		si_strchr // MJD - Found a bug in strchr on GSCube


#define _SI_ATOI		atoi

//Math Stuff
#define _SI_SQRT		sqrt
#define _SI_SQRTF		sqrtf
#define _SI_ASIN		asin
#define _SI_ASINF		asinf
#define _SI_ACOS		acos
#define _SI_ACOSF		acosf
#define _SI_FLOOR		floor
#define _SI_FLOORF		floorf


#ifdef _INLINETRIG_
#define _SI_COS(x)		SI_CosineTable[((int)(x) * 10000) % 62831]
#define _SI_COSF(x)		SI_CosineTable[((int)(x) * 10000) % 62831]
#define _SI_SIN(x)		SI_SineTable[((int)(x) * 10000) % 62831]
#define _SI_SINF(x)		SI_SineTable[((int)(x) * 10000) % 62831]
#else
#define _SI_COS			CSIBCUtil::cos
#define _SI_COSF		CSIBCUtil::cos
#define _SI_SIN			CSIBCUtil::sin
#define _SI_SINF		CSIBCUtil::sin
#endif



#define _SI_TAN			tan
#define _SI_TANF		tanf
#define _SI_ATANF		atanf


#ifdef AICUBE_FILESYSTEM

/* XEIOS */
#include "xeios.h"
#define _SI_FILE_SET xe_abs
#define _SI_FILE_CUR xe_rel
#define _SI_FILE_END xe_end
 
#define _SI_FEOF( x ) 0
#define _SI_FILE   xeFile *
#define _SI_FILE_MODE			int
#define _SI_FILE_NULL  NULL
#define _SI_FILE_READ_TEXT xe_read, xe_text
#define _SI_FILE_READ_BINARY xe_read, xe_binary
#define _SI_FILE_WRITE_TEXT xe_write, xe_text
#define _SI_FILE_WRITE_BINARY xe_write, xe_binary
#define _SI_FOPEN( name, mode )  xeOpen( name, mode )
/* the weird ",0" at the end of the line is there because xeClose does not return a value but uses of this macro expect one... */
#define _SI_FCLOSE( x )  xeClose( (xeFile *)(x) ), 0
#define _SI_FREAD( a, b, c, d ) xeRead( d, a, b*c )
#define _SI_FWRITE( a, b, c, d ) xeWrite( d, a, b*c )
#define _SI_FSEEK( ptr, count, type ) xeSeek( ptr, count, type )
/*
#define _SI_FPRINTF( a, b ) xeFprintf( a, b )
#define _SI_FGETC( fp ) fgetc(fp)
#define _SI_GETC( x ) getc( x )
*/

#undef SEEK_SET
#define SEEK_SET xe_abs
#undef SEEK_CUR
#define SEEK_CUR xe_rel
#undef SEEK_END
#define SEEK_END xe_end

#else

// use only if files come from a DOS/Windows system (which is the case)
#define LITTLE_ENDIAN

//File stuff
#define _SI_FILE				int
#define _SI_FILE_NULL			(int)(-1)
#define _SI_FILE_MODE			int
#define _SI_FILE_READ_TEXT	SCE_RDONLY
#define _SI_FILE_READ_BINARY SCE_RDONLY 
#define _SI_FILE_WRITE_TEXT	SCE_WRONLY | SCE_TRUNC | SCE_CREAT
#define _SI_FILE_WRITE_BINARY SCE_WRONLY | SCE_TRUNC | SCE_CREAT
 
#define _SI_FILE_SET SCE_SEEK_SET
#define _SI_FILE_CUR SCE_SEEK_CUR
#define _SI_FILE_END SCE_SEEK_END

#define _SI_FEOF( x )   0
#define _SI_FOPEN( NAME, MODE ) SIPS2CommFileOpen( NAME )
#define _SI_FCLOSE( x ) SIPS2CommFileClose( x )
#define _SI_FREAD( buf, size, count, fp ) SIPS2CommFileRead( fp, buf, size, count )
#define _SI_FWRITE( buf, size, count, fp ) sceWrite( fp, buf, size * count )
#define _SI_FSEEK( ptr, count, type ) sceLseek( ptr, count, type )
#define _SI_FTELL( ptr ) sceLseek( ptr, 0, _SI_FILE_CUR )
SI_Char si_getc( _SI_FILE x );

// fprintf is not available and not needed on the PS2
#define _SI_GETC( x ) si_getc( x )
#define _SI_FGETC( fp ) si_getc( fp )

#endif

#ifdef _DEBUG
#define _SI_ASSERT(x) assert(x)	
#else
void XSIEXPORT si_exit(SI_Char *, SI_Char *, const int );
//#define _SI_ASSERT(exp) if (!(exp)) si_exit( #exp, __FILE__, __LINE__ );
#define _SI_ASSERT(exp)
#endif


//Loadable module
#define SI_INVALID_LIB_HANDLE -1
#define _SI_LIBHANDLE	int

#endif
//
///////////////////////////////////////////// End Playstation2 Defines ///////////////////////////////
//

#ifndef M_PI
#define M_PI		3.14159265358979323846
#endif

#define _ERRMSG(x) _SI_PRINTF(_SI_TEXT("ERROR : %s\n"),x);
#define _MSG(x)    _SI_PRINTF(_SI_TEXT("MSG   : %s\n"),x);

#define _SI_NEW(x,y) {	x = new y; _SI_ASSERT( x != NULL ); }

#define _SI_DELETE( ptr ) {	if ( ptr != NULL ) { delete ptr; ptr = NULL; }  }

#define _SI_DELETE_ARRAY( ptr ) {if ( ptr != NULL ) { delete [] ptr; ptr = NULL; }  }

#define _SI_CALL(x,y) { if (result==SI_SUCCESS)	{ result = x;if(result != SI_SUCCESS){_ERRMSG(y);}}}

#define _SI_CHKRESULT( RESULT, MESSAGE, ACTION ){if (result != SI_SUCCESS){	_ERRMSG( MESSAGE );	ACTION;	}	}

XSIEXPORT SI_Char *  si_strdup( const SI_Char * );
XSIEXPORT SI_Char *  si_bstrdup( const SI_Byte * );
XSIEXPORT SI_Error si_bstringtostring( SI_Char *, const SI_Byte * );

#define _SI_STRDUPL(D,S) { D = (char *) alloca(strlen(S)); strcpy(D,S); }

#ifdef __cplusplus //If included by C-files this causes an error

#ifdef _INLINETRIG_
extern	float	SI_SineTable[62831];
extern	float	SI_CosineTable[62831];
#endif

//**************************************************************************************
// @class CSIBCUtil | Utility functions.
//
//
//**************************************************************************************


class XSIEXPORT CSIBCUtil
{
public:
	CSIBCUtil();											// Default Constructor
	~CSIBCUtil();											// Default Destructor

	static	SI_Error	Open(SI_Char *in_pFilename);		// Open the log file
	static	SI_Error	Close();							// Close the log file
	static	void		SetToStderr();						// Set as Standard Error
	static	void		printf(SI_Char *,...);				// printf
	
	// fast lookup table for trigonometric functions
	static	void		InitializeTrigTables();
	static	float		cos(float x);
	static	float		sin(float x);
	
protected:	
	static SI_Char m_Filename[256];

private:

};

#endif


///**************************************************************************************
// Inline code
///**************************************************************************************


//***************************************************************************************
// Author:	Softimage 3D Games & 3D Bridge team
// Date     :
//
// @mfunc Calculates the cosine of an angle.
// @parm float | x | Angle. 
// @rdesc Result of the cosine operation.
//***************************************************************************************
inline float CSIBCUtil::cos(float x)
{
#ifdef _INLINETRIG_
	int		index = (int) ( x * 10000.0f);
		
	if (index < 0)
		index += 628310000;
	
	return SI_CosineTable[index % 62831];
#else
	return cosf(x);
#endif
}

//***************************************************************************************
// Author:	Softimage 3D Games & 3D Bridge team
// Date     :
//
// @mfunc Calculates the sine of an angle.
// @parm float | x | Angle.
// @rdesc Result of the sin operation.
//***************************************************************************************
inline float CSIBCUtil::sin(float x)
{
#ifdef _INLINETRIG_
	int		index = (int) ( x * 10000.0f);
	
	if (index < 0)
		index += 628310000;
		
	return SI_SineTable[index % 62831];
#else
	return sinf(x);
#endif
}




#endif // CSIBCUtil
