//***************************************************************************************
//
// File supervisor: Softimage 3D Games & 3D Bridge team
//
// (c) Copyright 2001-2002 Avid Technology, Inc. . All rights reserved.
//
//***************************************************************************************
/****************************************************************************************
THIS CODE IS PUBLISHED AS A SAMPLE ONLY AND IS PROVIDED "AS IS".
IN NO EVENT SHALL SOFTIMAGE, AVID TECHNOLOGY, INC. AND/OR THEIR RESPECTIVE 
SUPPLIERS BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY
DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
CONNECTION WITH THE USE OR PERFORMANCE OF THIS CODE . 
 
COPYRIGHT NOTICE. Copyright  1999-2002 Avid Technology Inc. . All rights reserved. 

SOFTIMAGE is a registered trademark of Avid Technology Inc. or its subsidiaries 
or divisions. Windows NT is a registered trademark of Microsoft Corp. All other
trademarks contained herein are the property of their respective owners. 
****************************************************************************************/

//********************************************************************
// Includes
//********************************************************************
// Template.cpp : Implementation of CTemplate
#include "stdafx.h"
#include "DotXSIObj.h"
#include "Template.h"
#include "Templates.h"
#include "Params.h"
#include <XSIParser.h>
//#include <csconv.h>

/////////////////////////////////////////////////////////////////////////////
// CTemplate

STDMETHODIMP CTemplate::InterfaceSupportsErrorInfo(REFIID riid)
{
	static const IID* arr[] = 
	{
		&IID_ITemplate
	};
	for (int i=0; i < sizeof(arr) / sizeof(arr[0]); i++)
	{
		if (InlineIsEqualGUID(*arr[i],riid))
			return S_OK;
	}
	return S_FALSE;
}

//*********************************************************************************************
//
// Author:	Softimage 3D Games & 3D Bridge team
// Date  :	28/May/01
//
// @mfunc	Return the current Template address
//
//********************************************************************
CdotXSITemplate* CTemplate::GetTemplate()
{
	_ASSERT(m_pTemplate != NULL);
	return m_pTemplate;
}

//*********************************************************************************************
//
// Author:	Softimage 3D Games & 3D Bridge team
// Date  :	28/May/01
//
// @mfunc	Set the current Template address
//
//********************************************************************
void CTemplate::SetTemplate(CdotXSITemplate *in_pTemplate, CXSIParser *in_pParser)
{
	m_pTemplate = in_pTemplate; 
	m_pParser = in_pParser;
}

//*********************************************************************************************
//
// Author:	Softimage 3D Games & 3D Bridge team
// Date  :	28/May/01
//
// @mfunc	Return the Instance name
//
//********************************************************************
STDMETHODIMP CTemplate::get_InstanceName(BSTR *out_pbstrName)
{
	USES_CONVERSION;

	if (m_pTemplate == 0) return E_FAIL;

	*out_pbstrName = A2BSTR( m_pTemplate->InstanceName().GetText() );

	return S_OK;
}

//*********************************************************************************************
//
// Author:	Softimage 3D Games & 3D Bridge team
// Date  :	28/May/01
//
// @mfunc	Set the Instance name
//
//********************************************************************
STDMETHODIMP CTemplate::put_InstanceName(BSTR in_bstrName)
{
	USES_CONVERSION;

	if (m_pTemplate == 0) return E_FAIL;
	
	// Then set the name
	m_pTemplate->InstanceName().SetText( OLE2A( in_bstrName )  );
	
	return S_OK;
}


//*********************************************************************************************
//
// Author:	Softimage 3D Games & 3D Bridge team
// Date  :	28/May/01
//
// @mfunc		Return all chlidren of the current template
//
//********************************************************************
STDMETHODIMP CTemplate::get_Children(IDispatch **out_ppTemplates)
{
	HRESULT		l_hr = S_OK;
	
	// out parameter initialization
	*out_ppTemplates = NULL;

	if (m_pTemplate == 0) return E_FAIL;

	// Create the wrapper object
	CComObject<CTemplates>* l_pTemplatesObj = NULL;
	
	if ( FAILED (l_hr = CComObject<CTemplates>::CreateInstance(&l_pTemplatesObj) ) )
		return l_hr;

	// Set the good template
	l_pTemplatesObj->SetTemplates( &m_pTemplate->Children(), m_pParser );

	l_hr = l_pTemplatesObj->QueryInterface(IID_IDispatch, (LPVOID*)out_ppTemplates);
	if ( FAILED ( l_hr ) )
		return l_hr;

	return l_hr;
}

//*********************************************************************************************
//
// Author:	Softimage 3D Games & 3D Bridge team
// Date  :	28/May/01
//
// @mfunc		Create a new blank template collection
//
//********************************************************************
STDMETHODIMP CTemplate::get_CreateTemplates(IDispatch **out_ppTemplates)
{
	HRESULT l_hr;
	
	// Out parameter initialization
	*out_ppTemplates = 0;

	// Create the COM Wrapprer object
	CComObject<CTemplates>* l_pTemplatesObj = NULL;
	
	if ( FAILED (l_hr = CComObject<CTemplates>::CreateInstance(&l_pTemplatesObj) ) )
		return l_hr;


	// Create the new Template object and set it into the wrapper
	CdotXSITemplates* l_pdotXSITemplates = CreatedotXSITemplates();

	if ( l_pdotXSITemplates )
	{
		l_pTemplatesObj->SetTemplates(l_pdotXSITemplates, m_pParser);
	}
	else
	{
		return E_FAIL;
	}
	
	// Query the Template object for the good interface
	l_hr = l_pTemplatesObj->QueryInterface(IID_IDispatch, (LPVOID*)out_ppTemplates);
	return l_hr;	
}

//*********************************************************************************************
//
// Author:	Softimage 3D Games & 3D Bridge team
// Date  :	28/May/01
//
// @mfunc		Get the parent of the current template
//
//********************************************************************
STDMETHODIMP CTemplate::get_Parent(IDispatch **out_ppParent)
{
	HRESULT l_hr;

	// Out parameter initialization
	*out_ppParent = 0;

	// Check if the wrapper contains a dotXSITemplates object
	if ( m_pTemplate == 0 ) return E_FAIL;

	// Get the good template and set it into the Template object
	CdotXSITemplate* l_pdotXSITemplate = m_pTemplate->Parent();

	if( l_pdotXSITemplate == NULL )
		return S_OK;

	// Create the COM Wrapprer object
	CComObject<CTemplate>* l_pTemplateObj = NULL;
	
	if ( FAILED (l_hr = CComObject<CTemplate>::CreateInstance(&l_pTemplateObj) ) )
		return l_hr;

	l_pTemplateObj->SetTemplate( l_pdotXSITemplate, m_pParser );

	// Query the Template object for the good interface
	l_hr = l_pTemplateObj->QueryInterface(IID_IDispatch, (LPVOID*)out_ppParent);

	return l_hr;
}

//*********************************************************************************************
//
// Author:	Softimage 3D Games & 3D Bridge team
// Date  :	28/May/01
//
// @mfunc		Set the parent of the current template
//
//********************************************************************
STDMETHODIMP CTemplate::put_Parent(IDispatch *in_pParent)
{
	// Check if the wrapper contains a dotXSIParams object
	if ( m_pTemplate == 0 ) return E_FAIL;

	// If we got a null, this is correct too
	// Means that the template has no parent, so, this is a root template)
	if(in_pParent == NULL)
	{
		m_pTemplate->SetParent( NULL );
	}
	else
	{
		m_pTemplate->SetParent( ((CTemplate*) in_pParent)->GetTemplate() );
	}

	return S_OK;
}


//*********************************************************************************************
//
// Author:	Softimage 3D Games & 3D Bridge team
// Date  :	28/May/01
//
// @mfunc	Get the current SystemFlags
//
//********************************************************************
STDMETHODIMP CTemplate::get_SystemFlags(long *out_pnSysFlags)
{
	// out Parameter initialization
	*out_pnSysFlags = 0;
	
	if (m_pTemplate == 0) return E_FAIL;

	m_pTemplate->GetSystemFlags( (SI_Int*)out_pnSysFlags );
	return S_OK;
}

//*********************************************************************************************
//
// Author:	Softimage 3D Games & 3D Bridge team
// Date  :	28/May/01
//
// @mfunc	Set the current SystemFlags
//
//********************************************************************
STDMETHODIMP CTemplate::put_SystemFlags(long in_nSysFlags)
{
	if (m_pTemplate == 0) return E_FAIL;

	m_pTemplate->SetSystemFlags( in_nSysFlags );
	return S_OK;
}

//*********************************************************************************************
//
// Author:	Softimage 3D Games & 3D Bridge team
// Date  :	28/May/01
//
// @mfunc	Get the current Template Name
//
//********************************************************************
STDMETHODIMP CTemplate::get_TemplateName(BSTR *out_pbstrTemplateName)
{
	if (m_pTemplate == 0) return E_FAIL;

	*out_pbstrTemplateName = A2BSTR( m_pTemplate->Name().GetText() );

	return S_OK;
}

//*********************************************************************************************
//
// Author:	Softimage 3D Games & 3D Bridge team
// Date  :	28/May/01
//
// @mfunc	Set the current Template Name
//
//********************************************************************
STDMETHODIMP CTemplate::put_TemplateName(BSTR in_bstrTemplateName)
{

	USES_CONVERSION;

	if (m_pTemplate == 0) return E_FAIL;

	_ASSERT(m_pTemplate != NULL);
	
	// Then set the name
	m_pTemplate->InitializeFromName( OLE2A( in_bstrTemplateName ), m_pParser );

	return S_OK;
}

//*********************************************************************************************
//
// Author:	Softimage 3D Games & 3D Bridge team
// Date  :	28/May/01
//
// @mfunc	Get the current User Data Type
//
//********************************************************************
STDMETHODIMP CTemplate::get_UserDataType(BSTR *out_pbstrDataType)
{
	USES_CONVERSION;

	if (m_pTemplate == 0) return E_FAIL;

	*out_pbstrDataType = A2BSTR( m_pTemplate->UserDataType().GetText() );

	return S_OK;
}

//*********************************************************************************************
//
// Author:	Softimage 3D Games & 3D Bridge team
// Date  :	28/May/01
//
// @mfunc	Set the current User Data Type
//
//********************************************************************
STDMETHODIMP CTemplate::put_UserDataType(BSTR in_DataType)
{
	USES_CONVERSION;

	if (m_pTemplate == 0) return E_FAIL;
	
	// Then set the name
	m_pTemplate->UserDataType().SetText( OLE2A( in_DataType ) );
	
	return S_OK;
}


//*********************************************************************************************
//
// Author:	Softimage 3D Games & 3D Bridge team
// Date  :	28/May/01
//
// @mfunc	Get the current User Flags
//
//********************************************************************
STDMETHODIMP CTemplate::get_UserFlags(long *out_pnUserFlags)
{
	// out Parameter initialization
	*out_pnUserFlags = 0;
	
	if (m_pTemplate == 0) return E_FAIL;

	m_pTemplate->GetUserFlags( (SI_Int*)out_pnUserFlags );
	return S_OK;
}

//*********************************************************************************************
//
// Author:	Softimage 3D Games & 3D Bridge team
// Date  :	28/May/01
//
// @mfunc	Set the current User Flags
//
//********************************************************************
STDMETHODIMP CTemplate::put_UserFlags(long in_nUserFlags)
{
	if (m_pTemplate == 0) return E_FAIL;
	
	m_pTemplate->SetUserFlags( in_nUserFlags );
	return S_OK;
}

//*********************************************************************************************
//
// Author:	Softimage 3D Games & 3D Bridge team
// Date  :	28/May/01
//
// @mfunc		Return the parameter collection for the current template
//
//********************************************************************
STDMETHODIMP CTemplate::get_Params(IDispatch **out_ppParams)
{
	HRESULT l_hr;

	// Out parameter initialization
	*out_ppParams = 0;

	// Check if the wrapper contains a dotXSITemplates object
	if ( m_pTemplate == 0 ) return E_FAIL;

	// Create the COM Wrapprer object
	CComObject<CParams>* l_pParamsObj = NULL;
	
	if ( FAILED (l_hr = CComObject<CParams>::CreateInstance(&l_pParamsObj) ) )
		return l_hr;

	// Get the good template and set it into the Template object
	l_pParamsObj->SetParams( &m_pTemplate->Params() );

	// Query the Template object for the good interface
	l_hr = l_pParamsObj->QueryInterface(IID_IDispatch, (LPVOID*)out_ppParams);

	return l_hr;
}
