//***************************************************************************************
//
// File supervisor: Softimage 3D Games & 3D Bridge team
//
// (c) Copyright 2001-2002 Avid Technology, Inc. . All rights reserved.
//
//***************************************************************************************
/****************************************************************************************
THIS CODE IS PUBLISHED AS A SAMPLE ONLY AND IS PROVIDED "AS IS".
IN NO EVENT SHALL SOFTIMAGE, AVID TECHNOLOGY, INC. AND/OR THEIR RESPECTIVE 
SUPPLIERS BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY
DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
CONNECTION WITH THE USE OR PERFORMANCE OF THIS CODE . 
 
COPYRIGHT NOTICE. Copyright  1999-2002 Avid Technology Inc. . All rights reserved. 

SOFTIMAGE is a registered trademark of Avid Technology Inc. or its subsidiaries 
or divisions. Windows NT is a registered trademark of Microsoft Corp. All other
trademarks contained herein are the property of their respective owners. 
****************************************************************************************/

//********************************************************************
// Includes
//********************************************************************
// Templates.cpp : Implementation of CTemplates
#include "stdafx.h"
#include "DotXSIObj.h"
#include "Templates.h"
#include <XSIParser.h>
#include "Template.h"
#include "dotXSITemplate.h"

/////////////////////////////////////////////////////////////////////////////
// CTemplates

STDMETHODIMP CTemplates::InterfaceSupportsErrorInfo(REFIID riid)
{
	static const IID* arr[] = 
	{
		&IID_ITemplates
	};
	for (int i=0; i < sizeof(arr) / sizeof(arr[0]); i++)
	{
		if (InlineIsEqualGUID(*arr[i],riid))
			return S_OK;
	}
	return S_FALSE;
}

//*********************************************************************************************
//
// Author:	Softimage 3D Games & 3D Bridge team
// Date  :	28/May/01
//
// @mfunc		Set the memory addresse of the current XSIParser instance
//
//*********************************************************************************************
STDMETHODIMP CTemplates::SetXSIParser(CXSIParser* in_pXSIParser)
{
	m_pParser = in_pXSIParser;
	m_pTemplates = m_pParser->dotXSITemplate();

	return S_OK;
}

//*********************************************************************************************
//
// Author:	Softimage 3D Games & 3D Bridge team
// Date  :	28/May/01
//
// @mfunc		Get the number of templates in the collection
//
//********************************************************************
STDMETHODIMP CTemplates::get_GetCount(long *out_plCount)
{
	// Out param initialization
	HRESULT		l_hr = S_OK;
	*out_plCount = NULL;
	
	//_ASSERT(m_pTemplates != NULL);

	// Check if the wrapper contains a dotXSITemplates object
	if ( m_pTemplates == NULL )
	{
		l_hr = E_FAIL;
		Error("No Template collection available");
		goto end;
	}

	*out_plCount = m_pTemplates->GetCount();
	l_hr = S_OK;

end:

	return l_hr;
}

//*********************************************************************************************
//
// Author:	Softimage 3D Games & 3D Bridge team
// Date  :	28/May/01
//
// @mfunc		Add a template to the collection
//
//********************************************************************
STDMETHODIMP CTemplates::Add(IDispatch *in_pTemplate, long in_lPos)
{
	HRESULT		l_hr = S_OK;
	// Check if the wrapper contains a dotXSITemplates object
	if ( m_pTemplates == NULL ) 
	{
		l_hr = E_FAIL;
		Error("No Template Collection available");
		goto end;
	}

	m_pTemplates->Add( ((CTemplate*)in_pTemplate)->GetTemplate(), in_lPos ); 
	l_hr = S_OK;

end:
	return l_hr;
	
}

//*********************************************************************************************
//
// Author:	Softimage 3D Games & 3D Bridge team
// Date  :	28/May/01
//
// @mfunc		Remove a template from the collection
//
//********************************************************************
STDMETHODIMP CTemplates::Remove(long in_lIndex)
{
	HRESULT		l_hr = S_OK;

	// Check if the wrapper contains a dotXSITemplates object
	if ( m_pTemplates == NULL ) 
	{
		l_hr = E_FAIL;
		Error("No Template Collection available");
		goto end;
	}

	_ASSERT(m_pTemplates != NULL);
	m_pTemplates->Remove(in_lIndex);

end:
	return S_OK;
}

//*********************************************************************************************
//
// Author:	Softimage 3D Games & 3D Bridge team
// Date  :	28/May/01
//
// @mfunc		Create a new blank template
//
//********************************************************************
STDMETHODIMP CTemplates::get_CreateTemplate(IDispatch **out_ppTemplate)
{
	HRESULT l_hr;
	
	// Out parameter initialization
	*out_ppTemplate = 0;

	// Create the COM Wrapprer object
	CComObject<CTemplate>* l_pTemplateObj = NULL;
	
	if ( FAILED (l_hr = CComObject<CTemplate>::CreateInstance(&l_pTemplateObj) ) )
		return l_hr;


	// Create the new Template object and set it into the wrapper
	CdotXSITemplate* l_pdotXSITemplate = CreatedotXSITemplate();

	if ( l_pdotXSITemplate )
	{
		l_pTemplateObj->SetTemplate(l_pdotXSITemplate, m_pParser);
	}
	else
	{
		return E_FAIL;
	}
	
	// Query the Template object for the good interface
	l_hr = l_pTemplateObj->QueryInterface(IID_IDispatch, (LPVOID*)out_ppTemplate);
	return l_hr;	
}

//*********************************************************************************************
//
// Author:	Softimage 3D Games & 3D Bridge team
// Date  :	28/May/01
//
// @mfunc		Get a template from the collection
//
//********************************************************************
STDMETHODIMP CTemplates::get_Item(long in_lIndex, IDispatch **out_ppTemplate)
{
	HRESULT l_hr;

	// Out parameter initialization
	*out_ppTemplate = 0;

	// Check if the wrapper contains a dotXSITemplates object
	if ( m_pTemplates == 0 ) return E_FAIL;

	// Create the COM Wrapprer object
	CComObject<CTemplate>* l_pTemplateObj = NULL;
	
	if ( FAILED (l_hr = CComObject<CTemplate>::CreateInstance(&l_pTemplateObj) ) )
		return l_hr;

	// Fetch the good template and set it into the Template object
	CdotXSITemplate* l_pdotXSITemplate;
	m_pTemplates->Item( in_lIndex, &l_pdotXSITemplate );
	l_pTemplateObj->SetTemplate( l_pdotXSITemplate, m_pParser);

	// Query the Template object for the good interface
	l_hr = l_pTemplateObj->QueryInterface(IID_ITemplate, (LPVOID*)out_ppTemplate);

	return l_hr;
}
