//***************************************************************************************
//
// File supervisor: Softimage 3D Games & 3D Bridge team
//
// (c) Copyright 2001-2002 Avid Technology, Inc. . All rights reserved.
//
//***************************************************************************************
/****************************************************************************************
THIS CODE IS PUBLISHED AS A SAMPLE ONLY AND IS PROVIDED "AS IS".
IN NO EVENT SHALL SOFTIMAGE, AVID TECHNOLOGY, INC. AND/OR THEIR RESPECTIVE 
SUPPLIERS BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY
DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
CONNECTION WITH THE USE OR PERFORMANCE OF THIS CODE . 
 
COPYRIGHT NOTICE. Copyright  1999-2002 Avid Technology Inc. . All rights reserved. 

SOFTIMAGE is a registered trademark of Avid Technology Inc. or its subsidiaries 
or divisions. Windows NT is a registered trademark of Microsoft Corp. All other
trademarks contained herein are the property of their respective owners. 
****************************************************************************************/

#include "SemanticLayer.h"
#include <iostream.h>
#include <map> 

#if defined(_PSX2) || defined(_GNU)
#include <sys/time.h>
#define SLTest_Time				struct timeval
#define timeGetTime(x)			gettimeofday(&x, NULL)
#define timeDifferenceMs(x, y)	(y.tv_sec - x.tv_sec)*1000.0 + (y.tv_usec - x.tv_usec)/1000.0
#else
#include <windows.h>
#define SLTest_Time				DWORD
//#define timeGetTime(x)			x = timeGetTime();
//#define timeDifferenceMs(x, y)	y - x
#endif

typedef std::multimap<SI_Char *, CSLUnresolvedElement, StrLessPred> CSLTestUnresolvedMap;
typedef CSLTestUnresolvedMap::iterator CSLTestUnresolvedMapIterator;
typedef std::pair<CSLTestUnresolvedMapIterator,CSLTestUnresolvedMapIterator> CSLTestUnresolvedMapRange;

void FillShapeAnimation(CSLShapeAnimation*);
void FillCurve(CSLNurbsCurve*);

void CreateShaderInstanceData
( 
	CSLXSIShader *l_pShader, 
	CSLModel *in_pModel, 
	SI_Char* in_szShaderParamName
)
{
	CSLShaderInstanceData *l_pInstanceData = l_pShader->AddInstanceData();
	l_pInstanceData->SetReference(in_pModel);

	CSLVariantParameter *l_pParam = l_pInstanceData->AddParameter();
	l_pParam->SetName(in_szShaderParamName);
	
	// put a value in the param
	SI_TinyVariant l_variant = { SI_VT_INT, SI_TT_NONE, 1, 4 }; // one integer with a value of 4
	l_pParam->SetValue( &l_variant );
}

void ShowShaderInstanceData( CSLXSIShader *l_pShader )
{
	int l_nCount = l_pShader->GetInstanceDataCount();
	if (l_nCount)
	{
		CSLShaderInstanceData **l_ppInstanceData = l_pShader->GetInstanceDataList();
		CSLShaderInstanceData *l_pCurrentInstance;
		CSLVariantParameter **l_ppParameters;
		CSLVariantParameter *l_pParameter;


		for (int i = 0; i < l_nCount; i++ )
		{
			l_pCurrentInstance = l_ppInstanceData[i];

			// show refered model
			CSLModel *l_pModel = l_pCurrentInstance->GetReference();
			_SI_PRINTF
			(
				"\t\t\tReference: %s", 
				l_pModel->GetName()
			);
			
			// show parameters
			int l_nParamCount = l_pCurrentInstance->GetParameterCount();
			l_ppParameters = l_pCurrentInstance->GetParameterList();
			for (SI_Int j = 0; j < l_nParamCount; j++)
			{
				l_pParameter = l_ppParameters[j];
				_SI_PRINTF
				(
					"\t\t\tName, Value: %s, ",
					l_pParameter->GetName()
				);
			}
		}
	}
}

void AddNurbsData( CSLNurbsSurface *in_pNurbs )
{
	CSLNurbsProjection *l_pProjList = in_pNurbs->AddNurbsProjection();
	l_pProjList->SetName("ProjectionList");
	l_pProjList->SetProjectionType(CSLNurbsProjection::SI_WORLD);
	CSLCurveList *l_pCurveList = l_pProjList->CurveList();
	FillCurve(l_pCurveList->AddCurve() );

	CSLNurbsTrim *l_pTrimList = in_pNurbs->AddNurbsTrim();
	l_pTrimList->SetName("TrimList");
	l_pTrimList->SetProjectionType(CSLNurbsTrim::SI_WORLD);
	l_pTrimList->SetBoundaryFlag(SI_TRUE);
	l_pCurveList = l_pTrimList->CurveList();
	FillCurve(l_pCurveList->AddCurve() );

}

CSLUserDataList* AddUserDataList( CSLCluster *in_pCluster )
{
	CSLUserDataList *out_pList = in_pCluster->AddUserDataList();
	out_pList->SetName("The_List");

	CSLUserData *l_pData = out_pList->AddUserData();
	CSLUserData::CSLUByteArray *l_pArray = l_pData->GetUserDataList();
	l_pArray->Add(0x01U);l_pArray->Add(0x02U);l_pArray->Add(0x03U);
	
	CSLUserData::CSLIntArray *l_Array2 = l_pData->GetSubElementList();
	l_Array2->Add(4);l_Array2->Add(5);l_Array2->Add(6);
	
	return out_pList;
}

CSLLight* AddSpotLight( CSLScene *in_pScene, SI_Char* in_szName )
{
	CSLModel *l_pModel = in_pScene->Root()->AddSpotLight();
	CSLLight *out_pLight = (CSLLight*)l_pModel->Primitive();
	l_pModel->SetName(in_szName);
	
	CSLLightInfo *l_pInfo = out_pLight->CreateLightInfo();
	l_pInfo->SetFallofActive(SI_TRUE);
	l_pInfo->SetFallofMode( CSLLightInfo::SI_EXPONENT);
	l_pInfo->SetStart(0);
	l_pInfo->SetEnd(100);
	l_pInfo->SetShadowActive(SI_TRUE);
	l_pInfo->SetUmbra();
	l_pInfo->SetLightAsEnergy(SI_TRUE);
	l_pInfo->SetPhotonFactor();
	l_pInfo->SetIntensity( 0.5 );
	return out_pLight;
}

CSLImageLibrary* AddImageLibrary( CSLScene *in_pScene )
{
	CSLImageLibrary *out_pLibrary = in_pScene->CreateImageLibrary();
	
	// create standard image
	CSLImage *l_pHardImage = out_pLibrary->AddImage();
	l_pHardImage->SetName("Hard_Image");
	l_pHardImage->SetSourceFile("C:\\Hard_Image.jpg");
	l_pHardImage->SetWidth(1024);
	l_pHardImage->SetHeight(768);
	l_pHardImage->SetChannels(CSLImage::SI_RGB);
	l_pHardImage->SetColorDepth(CSLImage::SI_24BITS);
	l_pHardImage->SetCropMinX(1);
	l_pHardImage->SetCropMaxX(10);
	l_pHardImage->SetCropMinY(1);
	l_pHardImage->SetCropMaxY(10);
	l_pHardImage->SetFrameRate(60);
	l_pHardImage->SetFirstFrame(0);
	l_pHardImage->SetLastFrame(100);

	// add image fx
	CSLImageFX *l_pFX = l_pHardImage->CreateImageFX();
	l_pFX->SetName("ImageFX");
	l_pFX->SetHue(1.0F);
	l_pFX->SetGain(2.0F);
	l_pFX->SetSaturation(3.0F);
	l_pFX->SetBrightness(4.0F);
	l_pFX->SetBlurRadius(5.0F);
	l_pFX->SetBlurAmount(6.0F);
	l_pFX->SetBlurAlpha(SI_TRUE);
	l_pFX->SetScalingType(CSLImageFX::SI_CUSTOM);
	l_pFX->SetHorizontalScale(1.5F);
	l_pFX->SetVerticalScale(1.5F);
	l_pFX->SetFlipHorizontal(SI_TRUE);
	l_pFX->SetFlipVertical(SI_TRUE);
	l_pFX->SetRGBA2Greyscale(SI_TRUE);
	l_pFX->SetConvertTo16Bits(SI_TRUE);
	
	// create image with embeded pixel information
	CSLImage *l_pSoftImage = out_pLibrary->AddImage();
	l_pSoftImage->SetName("Soft_Image");
	l_pSoftImage->SetWidth(1024);
	l_pSoftImage->SetHeight(768);
	l_pSoftImage->SetChannels(CSLImage::SI_RGB);
	l_pSoftImage->SetColorDepth(CSLImage::SI_24BITS);
	l_pSoftImage->SetCropMinX(1);
	l_pSoftImage->SetCropMaxX(10);
	l_pSoftImage->SetCropMinY(1);
	l_pSoftImage->SetCropMaxY(10);
	l_pSoftImage->SetFrameRate(60);
	l_pSoftImage->SetFirstFrame(0);
	l_pSoftImage->SetLastFrame(100);
	
	CSLImageData* l_pData = l_pSoftImage->CreateImageData();
	CSIBCArray<SI_Int> l_data;
	l_data.Add(1);l_data.Add(2);l_data.Add(3);l_data.Add(4);l_data.Add(5);l_data.Add(6);
	l_pData->SetImageData( l_data );
	l_pData->SetCompressionType( CSLImageData::SI_UNCOMPRESSED );
	

	return out_pLibrary;
}

CSLFXTree* AddFXTree( CSLModel *in_pModel )
{
	CSLFXTree *l_pTree = in_pModel->AddFXTree();

	l_pTree->SetName("FXTree");
	
	CSLFXOperator *l_pOperator = l_pTree->AddFXOperator();
	l_pOperator->SetName("FXOperator");
	l_pOperator->SetEffectID("FXOperatorProgID");
	l_pOperator->SetOutputType(CSLFXOperator::SI_DEFAULT);

	CSLFXOperator *l_pReferedOperator = l_pTree->AddFXOperator();
	l_pReferedOperator->SetName("ReferedFXOperator");
	l_pReferedOperator->SetEffectID("ReferedFXOperatorProgID");
	l_pReferedOperator->SetOutputType(CSLFXOperator::SI_DEFAULT);

	CSLFXOperatorConnection *l_pOperatorConnection = l_pOperator->AddConnectionPoint();
	l_pOperatorConnection->SetName("FirstConnection");
	l_pOperatorConnection->SetConnectionType(CSLFXOperatorConnection::SI_FXOPERATOR);
	l_pOperatorConnection->SetFXOperator(l_pReferedOperator);

	CSLVariantParameter *l_pParam = l_pOperator->AddParameter();
	l_pParam->SetName("Parameter");
	SI_TinyVariant l_Variant =
	{
		SI_VT_BOOL,
		SI_TT_NONE,
		1,
		1
	};
	l_pParam->SetValue(&l_Variant);

	return l_pTree;
}

// Add Action to a mixer
CSLAction* AddAction( CSLMixer *in_pMixer, SI_Char* in_szName )
{
	CSIBCString l_String = in_szName;
	
	// addition of the action
	CSLAction *l_pAction = in_pMixer->AddAction();
	l_pAction->SetName(in_szName);
	
	// addition of stuff to the actual action

	// Add an action fcurve to the action
	CSLActionFCurve *l_pActionFCurve = l_pAction->AddAnimation();
	l_String.Concat("_ActionFCurve");
	l_pActionFCurve->SetName( l_String.GetText() );
	l_pActionFCurve->SetParameterXSIPath("null.kine.local.posy");
	l_String = in_szName;

	// Add a shape animation to the action
	CSLShapeAnimation *l_pShapeAnim = l_pAction->AddShapeAnimation( CSLTemplate::SI_LINEAR);
	l_String.Concat("_ShapeAnimation");
	l_pShapeAnim->SetName(l_String.GetText());
	FillShapeAnimation(l_pShapeAnim);
	l_String = in_szName;

	// add static value
	CSLStaticValue *l_pStaticValue = l_pAction->AddStaticValue();
	l_String.Concat("_StaticValue");
	l_pStaticValue->SetName(l_String.GetText());
	l_String = in_szName;
	l_pStaticValue->SetActive( true );
	l_pStaticValue->SetParameterName("null.kine.local.posx");
	l_pStaticValue->SetValue(4.123F);

	return l_pAction;
}

// Add extrapolation
CSLExtrapolation* AddExtrapolation( CSLTimeControl *io_TimeControl, SI_Char* in_szName, CSLTimeControl::EExtrapolationPos in_Pos )
{
	CSLExtrapolation *l_pExtrapolation = io_TimeControl->CreateExtrapolation(in_Pos);
	l_pExtrapolation->SetName( in_szName );

	l_pExtrapolation->SetValue( 1 );

	return l_pExtrapolation;
}

// add track to a mixer
CSLTrack* AddTrack ( CSLMixer *io_pMixer, SI_Char* in_szName )
{
	CSLTrack *l_pTrack = io_pMixer->AddTrack();
	l_pTrack->SetName( in_szName );
	
	CSIBCString l_String = in_szName;

	// add an action clip to the track
	CSLActionClip *l_pActionClip = l_pTrack->AddActionClip();
	l_String.Concat( "_ActionClip" );
	l_pActionClip->SetName( l_String.GetText() );
	l_pActionClip->SetReferenceXSIPath("MyReference");
	
	// add a time control to the action clip
	CSLTimeControl *l_pTimeControl = l_pActionClip->AddTimeControl();
	l_String = in_szName;
	l_String.Concat( "_TimeControl" );
	l_pTimeControl->SetName( l_String.GetText() );

	l_pTimeControl->SetIn( 1.0F );
	l_pTimeControl->SetOut( 1.0F );
	l_pTimeControl->SetScale( 2.0F );
	l_pTimeControl->SetStartOffset( 3.0F );

	AddExtrapolation( l_pTimeControl, "Before", CSLTimeControl::SI_BEFORE );
	AddExtrapolation( l_pTimeControl, "After", CSLTimeControl::SI_AFTER );
	
	return l_pTrack;
}

// Add Mixer
CSLMixer* AddMixer( CSLModel *in_pModel, SI_Char* in_szName )
{
	// add a mixer to the model
	CSLMixer *l_pMixer = in_pModel->CreateMixer();
	l_pMixer->SetName(in_szName);

	AddAction( l_pMixer, "MyAction");
	AddTrack( l_pMixer, "MyTrack" );

	return l_pMixer;
}

// Add CustomPSet
CSLCustomPSet* AddCustomPSet( CSLTemplate *io_pTemplate )
{
	CSLCustomPSet *l_pNewPSet = io_pTemplate->AddCustomPSet();
	l_pNewPSet->SetName("MyCustomPSet");
	
	// add a parameter
	CSLVariantParameter *l_pNewParameter = l_pNewPSet->AddParameter();
	l_pNewParameter->SetName("MyParameter");

	SI_TinyVariant l_Variant =
	{
		SI_VT_PCHAR,
		SI_TT_NONE,
		1
	};
	l_Variant.p_cVal = "MyVariantValue";

	l_pNewParameter->SetValue(&l_Variant);

	// add a second parameter
	CSLVariantParameter *l_pNewParameter2 = l_pNewPSet->AddParameter();
	l_pNewParameter2->SetName("MyParameter2");

	SI_TinyVariant l_Variant2 =
	{
		SI_VT_DOUBLE,
		SI_TT_NONE,
		1
	};
	l_Variant2.dVal = 0.12345;

	l_pNewParameter2->SetValue(&l_Variant2);

	CSLCustomPSet *l_pNewPSet2 = io_pTemplate->AddCustomPSet();
	l_pNewPSet2->SetName("MyCustomPSet2");

	// add custom parameter info
	CSLXSICustomParamInfo* l_pInfo = l_pNewPSet->CreateCustomParamInfo(0);
	
	SI_TinyVariant l_Range =
	{
		SI_VT_DOUBLE,
		SI_TT_NONE,
		1
	};
	l_Range.dVal = 0.0;
	l_pInfo->SetMinValue( l_Range );
	l_Range.dVal = 10.0;
	l_pInfo->SetMaxValue(l_Range);

	return l_pNewPSet;
}

// Add XSIMaterial with subshader and material info
CSLXSIMaterial* AddXSIMaterial( CSLScene *in_pScene, SI_Char *in_szName, CSLModel *in_pModel )
{
	CSLMaterialLibrary *l_pLibrary = in_pScene->GetMaterialLibrary();
	CSLXSIMaterial *out_pMat = l_pLibrary->AddXSIMaterial();

	out_pMat->SetName(in_szName);

	// add shader to XSIMaterial
	CSLXSIShader *l_pShader = out_pMat->AddShader();
	l_pShader->SetName("OGL1PASS");
	l_pShader->SetOutputType(CSLXSIShader::SI_OT_UNKNOWN);
	l_pShader->SetProgID("Softimage.OGLDraw.1");

	// add instance data
	CreateShaderInstanceData( l_pShader, in_pModel, "ShaderInstanceDataParam" );

	// Add connection under material
	CSLConnectionPoint *l_pMaterialConnection = out_pMat->AddConnectionPoint();
	l_pMaterialConnection->SetName("RealTime");
	l_pMaterialConnection->SetShader(l_pShader);

	// add parameters to Shader
	SI_TinyVariant l_Variant =
	{
		SI_VT_PVOID,
		SI_TT_NONE,
		1
	};

	// Name
	CSLVariantParameter *l_pNewParameter = l_pShader->AddParameter();
	l_pNewParameter->SetName("Name");
	l_Variant.variantType = SI_VT_PCHAR;
	l_Variant.p_cVal = _SI_STRDUP("OGL1Pass");
	l_pNewParameter->SetValue(&l_Variant);

	// normal
	l_pNewParameter = l_pShader->AddParameter();
	l_pNewParameter->SetName("normal");
	l_Variant.variantType = SI_VT_BOOL;
	l_Variant.boolVal = SI_TRUE;
	l_pNewParameter->SetValue(&l_Variant);

	// uv
	l_pNewParameter = l_pShader->AddParameter();
	l_pNewParameter->SetName("uv");
	l_Variant.variantType = SI_VT_BOOL;
	l_Variant.boolVal = SI_TRUE;
	l_pNewParameter->SetValue(&l_Variant);

	// vertexcolor
	l_pNewParameter = l_pShader->AddParameter();
	l_pNewParameter->SetName("vertexcolor");
	l_Variant.variantType = SI_VT_BOOL;
	l_Variant.boolVal = SI_TRUE;
	l_pNewParameter->SetValue(&l_Variant);

	// blending
	l_pNewParameter = l_pShader->AddParameter();
	l_pNewParameter->SetName("blending");
	l_Variant.variantType = SI_VT_BOOL;
	l_Variant.boolVal = SI_FALSE;
	l_pNewParameter->SetValue(&l_Variant);

	// srcblendingfunction
	l_pNewParameter = l_pShader->AddParameter();
	l_pNewParameter->SetName("srcblendingfunction");
	l_Variant.variantType = SI_VT_INT;
	l_Variant.nVal = 1;
	l_pNewParameter->SetValue(&l_Variant);

	// dstblendingfunction
	l_pNewParameter = l_pShader->AddParameter();
	l_pNewParameter->SetName("dstblendingfunction");
	l_Variant.variantType = SI_VT_INT;
	l_Variant.nVal = 0;
	l_pNewParameter->SetValue(&l_Variant);

	// cullingmode
	l_pNewParameter = l_pShader->AddParameter();
	l_pNewParameter->SetName("cullingmode");
	l_Variant.variantType = SI_VT_INT;
	l_Variant.nVal = 0;
	l_pNewParameter->SetValue(&l_Variant);

	// depthtest
	l_pNewParameter = l_pShader->AddParameter();
	l_pNewParameter->SetName("depthtest");
	l_Variant.variantType = SI_VT_BOOL;
	l_Variant.boolVal = SI_TRUE;
	l_pNewParameter->SetValue(&l_Variant);

	// Add another shader
	CSLXSIShader *l_pTexShader = out_pMat->AddShader();
	l_pTexShader->SetName("OGLT2D");
	l_pTexShader->SetOutputType(CSLXSIShader::SI_OT_UNKNOWN);
	l_pTexShader->SetProgID("Softimage.OGLT2D.1");

	// add parameters to another Shader

	// Name
	l_pNewParameter = l_pTexShader->AddParameter();
	l_pNewParameter->SetName("Name");
	l_Variant.variantType = SI_VT_PCHAR;
	l_Variant.p_cVal = _SI_STRDUP("OGLT2D");
	l_pNewParameter->SetValue(&l_Variant);

	// Texture_1_Enable
	l_pNewParameter = l_pTexShader->AddParameter();
	l_pNewParameter->SetName("Texture_1_Enable");
	l_Variant.variantType = SI_VT_BOOL;
	l_Variant.boolVal = SI_TRUE;
	l_pNewParameter->SetValue(&l_Variant);

	// tspace_id1
	l_pNewParameter = l_pTexShader->AddParameter();
	l_pNewParameter->SetName("tspace_id1");
	l_Variant.variantType = SI_VT_PCHAR;
	l_Variant.p_cVal = _SI_STRDUP("Texture_Projection");
	l_pNewParameter->SetValue(&l_Variant);

	// Modulation1
	l_pNewParameter = l_pTexShader->AddParameter();
	l_pNewParameter->SetName("Modulation1");
	l_Variant.variantType = SI_VT_INT;
	l_Variant.nVal = 0;
	l_pNewParameter->SetValue(&l_Variant);

	// Refmap1
	l_pNewParameter = l_pTexShader->AddParameter();
	l_pNewParameter->SetName("Refmap1");
	l_Variant.variantType = SI_VT_BOOL;
	l_Variant.boolVal = SI_FALSE;
	l_pNewParameter->SetValue(&l_Variant);

	// WrapS1
	l_pNewParameter = l_pTexShader->AddParameter();
	l_pNewParameter->SetName("WrapS1");
	l_Variant.variantType = SI_VT_BOOL;
	l_Variant.boolVal = SI_TRUE;
	l_pNewParameter->SetValue(&l_Variant);

	// WrapT1
	l_pNewParameter = l_pTexShader->AddParameter();
	l_pNewParameter->SetName("WrapT1");
	l_Variant.variantType = SI_VT_BOOL;
	l_Variant.boolVal = SI_TRUE;
	l_pNewParameter->SetValue(&l_Variant);

	// magfilter1
	l_pNewParameter = l_pTexShader->AddParameter();
	l_pNewParameter->SetName("magfilter1");
	l_Variant.variantType = SI_VT_INT;
	l_Variant.nVal = 1;
	l_pNewParameter->SetValue(&l_Variant);

	// minfilter1
	l_pNewParameter = l_pTexShader->AddParameter();
	l_pNewParameter->SetName("minfilter1");
	l_Variant.variantType = SI_VT_INT;
	l_Variant.nVal = 1;
	l_pNewParameter->SetValue(&l_Variant);

	// Add connection point (SHADER) to Shader
	CSLShaderConnectionPoint *l_pConnectionPoint = l_pShader->AddConnectionPoint();
	l_pConnectionPoint->SetName("previous");
	l_pConnectionPoint->SetShader(l_pTexShader);

	// Add connection point (IMAGE) to the other Shader
	l_pConnectionPoint = l_pTexShader->AddConnectionPoint();
	l_pConnectionPoint->SetName("Texture_1");
	l_pConnectionPoint->SetImage("C:\\MyImage.jpg");

	// add fcurve to the second parameter
	CSLFCurve *l_pCurve = l_pTexShader->CreateParameterFCurve( "Refmap1", CSLTemplate::SI_CONSTANT );
	if(l_pCurve)
	{
		l_pCurve->GetConstantKeyList()->Extend(2);
		for(int i = 0;i < l_pCurve->GetConstantKeyList()->GetUsed();i ++)
		{
			(*l_pCurve->GetConstantKeyList() )[i].m_fTime = (float) (i + 1);
			(*l_pCurve->GetConstantKeyList() )[i].m_fValue = (float) (i % 2);
		}
	}
	else
	{
		_SI_PRINTF("Parameter fcurve creation error\n");
	}

	// addition of material info
	CSLXSIMaterialInfo *l_pInfo = out_pMat->CreateMaterialInfo();
	l_pInfo->SetName("MaterialInfo");	

	return out_pMat;
}

// add global material
void AddGlobalMaterial
( 
	CSLScene *in_pScene,	// owner scene
	CSLModel *in_pModel,	// model where to add the global material
	CSLGlobalMaterial::EPropagationType in_Propagation,	// type of propagation
	SI_Char* in_szName		// name of the material refered to by the global material or null if none
)
{
	CSLGlobalMaterial *l_pGlobalMaterial = in_pModel->GlobalMaterial();
	//
	// Make sure we have a GlobalMaterial
	//
	if (NULL == l_pGlobalMaterial )
	{
		in_pModel->AddGlobalMaterial();
		l_pGlobalMaterial = in_pModel->GlobalMaterial();
	}
	l_pGlobalMaterial->SetPropagationType( in_Propagation );
	if (in_szName)
	{
		CSLMaterial *l_pMaterial = in_pScene->GetMaterialLibrary()->AddMaterial();
		l_pMaterial->SetName(in_szName);
		l_pGlobalMaterial->SetMaterial(l_pMaterial);
	}
}

// Creates scene information, file information and angle information
void CreateSimpleTemplates(CSLScene &in_rScene)
{
	// Angle
	CSLAngle *l_pAngle = in_rScene.Angle();
	l_pAngle->SetAngleType(CSLAngle::SI_RADIANS);

	// Coordinate system
	in_rScene.CoordinateSystem()->SetCoordinateSystemStyle(CSLCoordinateSystem::SI_SOFTIMAGE_3D);

	// File Info
	CSLFileInfo *l_pFileInfo = in_rScene.FileInfo();
	l_pFileInfo->SetOriginator( "Originator Himself" );
	l_pFileInfo->SetProjectName( "Semantic Layer Test" );
	l_pFileInfo->SetUsername( "Garbash" );

	// Scene Info
	CSLSceneInfo *l_pSceneInfo = in_rScene.SceneInfo();
	l_pSceneInfo->SetEnd( 100 );
	l_pSceneInfo->SetStart(0);
	l_pSceneInfo->SetFrameRate(30);
	l_pSceneInfo->SetTimingType(CSLSceneInfo::SI_FRAMES);
}

void ShowMaterialInfo( CSLXSIMaterial *in_pMat )
{
	CSLXSIMaterialInfo *l_pInfo = in_pMat->GetMaterialInfo();
	if (!l_pInfo) return;

	_SI_PRINTF
	(
		"\nMaterialInfo: UWrap: %i, VWrap: %i",
		l_pInfo->GetUWrap(),
		l_pInfo->GetVWrap()
	);
}

void ShowColor
( 
	SI_Char* in_szString, 
	CSIBCColorf &in_Color 
)
{
	float l_R = 0, l_G = 0, l_B = 0;
	in_Color.Get( &l_R, &l_G, &l_B );
	_SI_PRINTF( "%s: (%f,%f,%f)\n", in_szString, l_R, l_G, l_B );
}

// Show scene information, file information, coordinate system information and angle information
void ShowGlobalSceneInfo( CSLScene &in_rScene )
{
	// FileInfo
	CSLFileInfo *l_pFileInfo = in_rScene.FileInfo();
	_SI_PRINTF
	(
		"File Info:\n\tOriginator: %s\n\tProject Name: %s\n\tUsername: %s\n\tSaved: %s\n", 
		l_pFileInfo->GetOriginator(),
		l_pFileInfo->GetProjectName(),
		l_pFileInfo->GetUsername(),
		l_pFileInfo->GetSaveDateTime()
	);

	// SceneInfo
	CSLSceneInfo *l_pSceneInfo = in_rScene.SceneInfo();
	_SI_PRINTF
	(
		"Scene Info:\n\tStartFrame: %f\n\tEndFrame: %f\n\tFrameRate: %f\n\tTimingType: %d\n",
		l_pSceneInfo->GetStart(),
		l_pSceneInfo->GetEnd(),
		l_pSceneInfo->GetFrameRate(),
		l_pSceneInfo->GetTimingType()
	);

	// CoordinateSystem
	CSLCoordinateSystem *l_pCoordSystem = in_rScene.CoordinateSystem();
	_SI_PRINTF("Coordinate System : \n");
	_SI_PRINTF("\tHand rotation : %d\n", l_pCoordSystem->GetHandRotation() );
	_SI_PRINTF("\tU axis orientation : %d\n", l_pCoordSystem->GetUAxisOrientation() );
	_SI_PRINTF("\tV axis orientation : %d\n", l_pCoordSystem->GetVAxisOrientation() );
	_SI_PRINTF("\tX axis orientation : %d\n", l_pCoordSystem->GetXAxisOrientation() );
	_SI_PRINTF("\tY axis orientation : %d\n", l_pCoordSystem->GetYAxisOrientation() );
	_SI_PRINTF("\tZ axis orientation : %d\n", l_pCoordSystem->GetZAxisOrientation() );

	// Angle
	CSLAngle *l_pAngle = in_rScene.Angle();
	_SI_PRINTF
	(
		"Angle: %d", l_pAngle->GetAngleType()
	);

	// Fog
	if(in_rScene.Fog() != NULL)
	{
		CSIBCColorf l_cGetColor(in_rScene.Fog()->GetColor());
		_SI_PRINTF("Fog : \n");
		_SI_PRINTF("\tType : %d\n", in_rScene.Fog()->GetFogType() );
		_SI_PRINTF("\tInterpolation : %d\n", in_rScene.Fog()->GetFogInterpolationType() );
		ShowColor("\tColor : ",  l_cGetColor);
		_SI_PRINTF("\tNear : %f\n", in_rScene.Fog()->GetNear() );
		_SI_PRINTF("\tFar : %f\n", in_rScene.Fog()->GetFar() );
	}
}

void ShowMaterial( CSLBaseMaterial *in_pMaterial )
{
	_SI_PRINTF("\n\t%s:\n\t{\n", in_pMaterial->GetName() );
	if (in_pMaterial->Type() == CSLTemplate::SI_MATERIAL )
	{
		// safe to upcast
		CSLMaterial *l_pMaterial = (CSLMaterial*)in_pMaterial;
		CSIBCColorf l_Color;
		CSIBCColorf l_cAmbient(l_pMaterial->GetAmbientColor());
		CSIBCColorf l_cDiffuse(l_pMaterial->GetDiffuseColor());
		CSIBCColorf l_cEmissive(l_pMaterial->GetEmissiveColor());
		CSIBCColorf l_cSpecular(l_pMaterial->GetSpecularColor());
		
		ShowColor( "\t\tAmbient",  l_cAmbient);
		ShowColor( "\t\tDiffuse",  l_cDiffuse);
		ShowColor( "\t\tEmissive",  l_cEmissive);
		ShowColor( "\t\tSpecular",  l_cSpecular);
		_SI_PRINTF
		(
			"\t\tShading Type: %i\n", 
			l_pMaterial->GetShadingModel()
		);

		CSLTexture2D *l_Texture = l_pMaterial->Texture2D();
		if(l_Texture != NULL)
		{
			_SI_PRINTF("\t\tA texture is present on the material\n");
			_SI_PRINTF("\t\tTexture image file name : %s\n", l_Texture->GetImageFileName() );
		}
	}
	else
	{
		// safe to upcast
		CSLXSIMaterial *l_pMaterial = (CSLXSIMaterial*)in_pMaterial;

		ShowMaterialInfo(l_pMaterial);
		
		// show connection points
		SI_Int l_nCount = l_pMaterial->GetConnectionPointCount();
		SI_Int i, j;
		CSLConnectionPoint **l_ppConnnectionPoints = l_pMaterial->GetConnectionPointList();
		CSLConnectionPoint *l_pConnectionPoint = 0;
		CSLXSIShader *l_pShader = 0;

		_SI_PRINTF("\t\tNb Connections: %i\n", l_nCount);
		for (i = 0; i < l_nCount; i++)
		{
			l_pConnectionPoint = l_ppConnnectionPoints[i];
			l_pShader = l_pConnectionPoint->GetShader();
			_SI_PRINTF
			(
				"\t\tName,Connection: %s,%s\n",
				l_pConnectionPoint->GetName(),
				(l_pShader) ? l_pShader->GetName() : "Not connected"
			);
		}

		// show shaders
		l_nCount = l_pMaterial->GetShaderCount();
		SI_Int l_nSubCount = 0;
		CSLXSIShader **l_ppShaders = l_pMaterial->GetShaderList();
		CSLShaderConnectionPoint **l_ppShaderCnxPoints = 0;;
		CSLShaderConnectionPoint *l_pShaderCnxPoint = 0;

		_SI_PRINTF("\t\tNb Shaders: %i\n", l_nCount);
		for (i = 0; i < l_nCount; i++)
		{
			l_pShader = l_ppShaders[i];
			_SI_PRINTF
			(
				"\t\tName: %s\n\t\tOutput Type: %i\n\t\tProgID: %s\n",
				l_pShader->GetName(),
				l_pShader->GetOutputType(),
				l_pShader->GetProgID()
			);
			
			// Shader connections
			l_nSubCount = l_pShader->GetConnectionPointCount();
			l_ppShaderCnxPoints = l_pShader->GetConnectionPointList();
			CSLShaderConnectionPoint::EConnectionType l_Type;
			SI_Char *l_szName;
			CSLXSIShader *l_pSubShader = 0;

			
			_SI_PRINTF("\t\tNb Connections : %i\n", l_nSubCount);
			for (j=0; j < l_nSubCount; j++)
			{
				// show shader conenction point info
				l_pShaderCnxPoint = l_ppShaderCnxPoints[j];
				l_Type = l_pShaderCnxPoint->GetConnectionType();

				_SI_PRINTF("\t\tName: %s\n\t\tType: ", l_pShaderCnxPoint->GetName() );
				
				// output connection info
				if (l_Type == CSLShaderConnectionPoint::SI_IMAGE)
				{
					l_szName = l_pShaderCnxPoint->GetImage();
					_SI_PRINTF("Image: %s\n", (l_szName) ? l_szName : "N/A" );
				}
				else
				{
					l_pSubShader = l_pShaderCnxPoint->GetShader();
					_SI_PRINTF("Shader: %s\n", (l_pSubShader != 0) ? l_pSubShader->GetName() : "N/A");
				}
			}


			// Custom psets
			CSLCustomPSet **l_ppCustPSet = l_pShader->GetCustomPSetList();
			CSLCustomPSet *l_pCustPSet = 0;
			CSLVariantParameter **l_ppParameters = 0;
			CSLVariantParameter *l_pParameter = 0;
			l_nSubCount = l_pShader->GetCustomPSetCount();
			SI_Int l_nParamCount = 0;
			_SI_PRINTF("\t\tNb CustomPSet : %i\n", l_nSubCount);
			for (j=0; j < l_nSubCount; j++)
			{
				l_pCustPSet = l_ppCustPSet[j];
				l_nParamCount = l_pCustPSet->GetParameterCount();
				_SI_PRINTF
				(
					"\t\t\tName: %s, Propagation: %i\n\t\t\tNb Param: %i\n",
					l_pCustPSet->GetName(),
					l_pCustPSet->GetPropagationType(),
					l_nParamCount
				);

				// display parameters
				l_ppParameters = l_pCustPSet->GetParameterList();
				for (SI_Int k = 0; k < l_nParamCount; k++)
				{
					l_pParameter = l_ppParameters[k];
					_SI_PRINTF
					(
						"\t\t\tName, Value: %s, ",
						l_pParameter->GetName()
					);
				}
			}

			// shader instance data
			ShowShaderInstanceData( l_pShader );

		}

	}
	_SI_PRINTF("\n\t}\n");
}

void ShowMaterialLibrary( CSLScene &in_rScene )
{
	CSLMaterialLibrary *l_pLibrary = in_rScene.GetMaterialLibrary();
	
	SI_Int l_nCount = l_pLibrary->GetMaterialCount();
	CSLBaseMaterial **l_ppList = l_pLibrary->GetMaterialList();
	
	_SI_PRINTF("Material library:\n{\tCount: %i\n\tElements:\n", l_nCount );
	
	for (SI_Int i = 0; i < l_nCount; i++ )
	{
		ShowMaterial(l_ppList[i]);
		
	}
	_SI_PRINTF("}\n");
}

void ShowLightInfo( CSLPrimitive *in_pPrimitive )
{
	CSLLightInfo *l_pInfo = ((CSLLight*)in_pPrimitive)->LightInfo();
	if (!l_pInfo) return;
	_SI_PRINTF
	(
		"LightInfo: Fallof Active: %i, Fallof Mode: %i, Start: %i, End: %i, Shadows: %i, Umbra: %i, As Energy: %i, Photon Fact.: %i, Intensity: %f",
		l_pInfo->GetFallofActive(),
		l_pInfo->GetFallofMode(),
		l_pInfo->GetStart(),
		l_pInfo->GetEnd(),
		l_pInfo->GetShadowActive(),
		l_pInfo->GetUmbra(),
		l_pInfo->GetLightAsEnergy(),
		l_pInfo->GetPhotonFactor(),
		l_pInfo->GetIntensity()
	);
}

void ShowFXOperator( CSLFXOperator *in_pOperator )
{
	_SI_PRINTF
	(
		"\n\t\tOperator %s ProgID: %s, Type %i, Parameter Count: %i\n\t\t\tConnections: ",
		in_pOperator->GetName(),
		in_pOperator->GetEffectID(),
		in_pOperator->GetOutputType(),
		in_pOperator->GetParameterCount()
	);

	SI_Int l_nCount = in_pOperator->GetConnectionPointCount();
	CSLFXOperatorConnection **l_ppCnx = in_pOperator->GetConnectionPointList();

	for (SI_Int i = 0; i < l_nCount; i++ )
	{
		_SI_PRINTF("%s, ", (l_ppCnx[i])->GetName() );
	}
}

void ShowFXTrees( CSLModel *in_pModel )
{
	SI_Int l_nTreeCount = in_pModel->GetFXTreeCount();
	CSLFXTree **l_ppTrees = in_pModel->GetFXTreeList();
	CSLFXTree *l_pTree;

	for (register SI_Int i = 0; i < l_nTreeCount; i++ )
	{
		l_pTree = l_ppTrees[i];
		_SI_PRINTF( "\n\tFXTree %s", l_pTree->GetName() );

		SI_Int l_nCount = l_pTree->GetFXOperatorCount();
		CSLFXOperator **l_ppOperator = l_pTree->GetFXOperatorList();
	
		for (register SI_Int j = 0; j < l_nCount; j++ )
		{
			ShowFXOperator(l_ppOperator[j]);
		}
	}
}

void ShowImage( CSLImage *in_pImage )
{
	CSLImageFX *l_pImageFX = in_pImage->ImageFX();
	CSLImageData *l_pImageData  = in_pImage->ImageData();
	
	_SI_PRINTF( "\n\t%s\n\t{", in_pImage->GetName() );

	SI_Char *l_pSourceFile = in_pImage->GetSourceFile();
	if (_SI_STRLEN(l_pSourceFile))
	{
		_SI_PRINTF( "\n\t\tSourceFile: \"%s\"", l_pSourceFile );
	}

	if (l_pImageFX)
	{
		_SI_PRINTF( "\n\t\tImageFX: %s", l_pImageFX->GetName() );
	}
	
	if (l_pImageData)
	{
		SI_Int* pData = l_pImageData->GetImageDataListPtr();
		_SI_PRINTF( "\n\t\tImageData: " );
		for (SI_Int i = 0; i < l_pImageData->GetImageDataCount(); i++ )
		{
			_SI_PRINTF( "%i, ", pData[i] );
		}
	}

	_SI_PRINTF("\n\t}");
}

void ShowImageLibrary( CSLScene *in_pScene )
{
	CSLImageLibrary *l_pLib = in_pScene->GetImageLibrary();
	if (!l_pLib) return;

	SI_Int l_nCount = l_pLib->GetImageCount();
	CSLImage **l_ppList = l_pLib->GetImageList();
	_SI_PRINTF("Image Library (%i Images): \n{", l_nCount );

	for (SI_Int i = 0; i < l_nCount; i++ )
	{
		ShowImage(l_ppList[i]);
	}

	_SI_PRINTF("\n}");
}

void ShowUserDataList( CSLUserDataList *in_pList )
{
	CSLUserData** l_ppUserData = in_pList->GetUserDataList();
	if (!l_ppUserData) return;
	
	SI_Int l_nCount = in_pList->GetUserDataCount();
	if (l_nCount == 0) return;

	
	_SI_PRINTF( "\n\t\t\t\tUser Data List: %s (Data: ", in_pList->GetName(), l_nCount );
	CSLUserData* l_pUserData;
	SI_Int l_subCount;
	SI_UByte *l_pUserDataValues;
	SI_Int *l_pSubElement;
	register SI_Int i = 0;
	// user data
	for (; i < l_nCount; i++ )
	{
		l_pUserData = l_ppUserData[i];
		l_pUserDataValues = l_pUserData->GetUserDataListPtr();
		l_subCount = l_pUserData->GetUserDataCount();
		_SI_PRINTF( "\n\t\t\t\t\tUser data %i (Data: ", i );
		register SI_Int j = 0;
		for (; j < l_subCount; j++ )
		{
			_SI_PRINTF( "%u, ", l_pUserDataValues[j] );
		}

		_SI_PRINTF( " SubElement: " );
		l_pSubElement = l_pUserData->GetSubElementListPtr();
		l_subCount = l_pUserData->GetSubElementCount();
		for (j = 0; j < l_subCount; j++ )
		{
			_SI_PRINTF( "%u, ", l_pSubElement[j] );
		}

		_SI_PRINTF( ")" );
	}
}

void ShowUserDataLists( CSLCluster *in_pCluster )
{
	CSLUserDataList **l_pDataLists = in_pCluster->GetUserDataLists();
	if (!l_pDataLists) return;
	
	SI_Int l_nCount = in_pCluster->GetUserDataListCount();
	if (l_nCount == 0) return;

	_SI_PRINTF( "\n\t\t\tUser Data lists: %i\n\t\t\t{", l_nCount );

	for (SI_Int i = 0; i < l_nCount; i++ )
	{
		ShowUserDataList( l_pDataLists[i] );
	}

	_SI_PRINTF( "\n\t\t\t}" );
}

void ShowNurbsProjectionList( CSLNurbsSurface *in_pSurface )
{
	CSLNurbsProjection **l_ppList = in_pSurface->GetNurbsProjections();
	CSLNurbsProjection *l_pList;
	SI_Int l_nCount = in_pSurface->GetNurbsProjectionCount();

	_SI_PRINTF( "\nNurbs Projection (%i Lists)", l_nCount );
	register SI_Int i = 0;
	for (;i<l_nCount;i++)
	{
		l_pList = l_ppList[i];
		_SI_PRINTF( "\n\t%i : Type %i", i, l_pList->GetProjectionType() );
	}
}

void ShowNurbsTrimList( CSLNurbsSurface *in_pSurface )
{
	CSLNurbsTrim **l_ppList = in_pSurface->GetNurbsTrims();
	CSLNurbsTrim *l_pList;
	SI_Int l_nCount = in_pSurface->GetNurbsTrimCount();

	_SI_PRINTF( "\nNurbs Trim List (%i Lists)", l_nCount );
	register SI_Int i = 0;
	for (;i<l_nCount;i++)
	{
		l_pList = l_ppList[i];
		_SI_PRINTF( "\n\t%i : Type %i Boundary %i", i, l_pList->GetProjectionType(), l_pList->GetBoundaryFlag() );
	}
}

void PrintVariant( const CSLVariantProxy &in_Variant )
{
	SI_TinyVariant l_vVariant;
	if (SI_SUCCESS == in_Variant.GetValue(l_vVariant))
	{ 
		switch (in_Variant.GetType())
		{
			case CSLVariantProxy::SI_SLVT_BOOL:	_SI_PRINTF( "%d", l_vVariant.boolVal ); break;	
			case CSLVariantProxy::SI_SLVT_BYTE:	_SI_PRINTF( "%d", l_vVariant.bVal );	break;	
			case CSLVariantProxy::SI_SLVT_DOUBLE:_SI_PRINTF( "%f", l_vVariant.dVal );	break;	
			case CSLVariantProxy::SI_SLVT_FLOAT: _SI_PRINTF( "%f", l_vVariant.fVal );	break;	
			case CSLVariantProxy::SI_SLVT_INT:	_SI_PRINTF( "%d", l_vVariant.nVal );	break;	
			case CSLVariantProxy::SI_SLVT_LONG:	_SI_PRINTF( "%d", l_vVariant.lVal );	break;	
			case CSLVariantProxy::SI_SLVT_SHORT: _SI_PRINTF( "%d", l_vVariant.sVal );	break;	
			case CSLVariantProxy::SI_SLVT_STRING:_SI_PRINTF( "%s", l_vVariant.p_cVal );	break;
		}
	}
	else
	{
		_SI_PRINTF( "%s", "Invalid variant type." );
	}
}

void ShowCustomParamInfo( CSLXSICustomParamInfo *in_pInfo )
{
	CSLVariantProxy *l_vMin = &in_pInfo->GetMinValue();
	CSLVariantProxy *l_vMax = &in_pInfo->GetMaxValue();

	SI_Char l_szFormatString[256] = {'\0'};
	_SI_PRINTF( "%s", "Custom Param Info: Min( " );
	PrintVariant(*l_vMin);
	_SI_PRINTF( "%s", " ) Max( " );
	PrintVariant(*l_vMax);
	_SI_PRINTF( "%s", " )\n" );
}

void OutputFCurve(CSLFCurve *in_pFCurve, SI_Char *in_szParamName)
{
	if(in_pFCurve != NULL)
	{
		SI_Float *l_pfCurveArray = NULL;

		// Output the type of curve.
		_SI_PRINTF("The %s parameter has an animation fcurve of with ", in_szParamName);

		// Fetch the appropriate float array.
		switch(in_pFCurve->GetInterpolationType() )
		{
		case CSLTemplate::SI_CONSTANT:
			_SI_PRINTF("constant interpolation.\n");
			l_pfCurveArray = (SI_Float *) in_pFCurve->GetConstantKeyList()->ArrayPtr();
			break;
		case CSLTemplate::SI_LINEAR:
			_SI_PRINTF("linear interpolation.\n");
			l_pfCurveArray = (SI_Float *) in_pFCurve->GetLinearKeyList()->ArrayPtr();
			break;
		case CSLTemplate::SI_HERMITE:
			_SI_PRINTF("hermite interpolation.\n");
			l_pfCurveArray = (SI_Float *) in_pFCurve->GetHermiteKeyList()->ArrayPtr();
			break;
		case CSLTemplate::SI_CUBIC:
			_SI_PRINTF("cubic interpolation.\n");
			l_pfCurveArray = (SI_Float *) in_pFCurve->GetCubicKeyList()->ArrayPtr();
			break;
		}

		_SI_PRINTF("FCurve animation data:\n");

		// Output key values
		SI_Int l_nKeyCount = in_pFCurve->GetKeyCount();
		SI_Int l_nRowSize = in_pFCurve->GetKeyValueCount() + 1;

		for(SI_Int i = 0;i < l_nKeyCount;i ++)
		{
			_SI_PRINTF("Key time %f\tData: ", l_pfCurveArray[i * l_nRowSize]);

			for(SI_Int j = 1;j < l_nRowSize;j ++)
			{
				_SI_PRINTF("%f ", l_pfCurveArray[i * l_nRowSize + j]);
			}
			_SI_PRINTF("\n");
		}
	}
}

void DescribeAction( CSLAction *in_pAction )
{
	_SI_PRINTF
	(
		"\t\tAction: %s, type: %d, Animations %d, EndTime %d, Shape Anims %d, Start Time %d, Static Values %d\n",
		in_pAction->GetName(),
		in_pAction->GetActionType(),
		in_pAction->GetAnimationCount(),
		in_pAction->GetEndTime(),
		in_pAction->GetShapeAnimationCount(),
		in_pAction->GetStartTime(),
		in_pAction->GetStaticValueCount()
	);
}

void DescribeTrack( CSLTrack *in_pTrack )
{
	_SI_PRINTF
	(
		"\t\tTrack: %s, Action Clips %d, Mute %d, Ripple %d, Solo %d",
		in_pTrack->GetName(),
		in_pTrack->GetActionClipCount(),
		in_pTrack->GetMute(),
		in_pTrack->GetRipple(),
		in_pTrack->GetSolo()
	);
}

void DescribeIKRoot(CSLIKRoot *in_pRoot)
{
	_SI_PRINTF("The chain contains %d joints:\n", in_pRoot->GetJointCount() );

	// Output each joint name.
	CSLModel **l_ppJoints = in_pRoot->GetJointList();

	for(SI_Int i = 0;i < in_pRoot->GetJointCount();i ++)
	{
		_SI_PRINTF("\tJoint %d is named \"%s\"\n", i, l_ppJoints[i]->GetName() );
	}

	// Output the effector name.
	if(in_pRoot->GetEffector() )
	{
		_SI_PRINTF("The effector of the chain is named \"%s\"\n", in_pRoot->GetEffector()->GetName() );
	}
	else
	{
		_SI_PRINTF("The chain has no effector\n");
	}
}

void DescribeShape(CSLBaseShape* in_pShape)
{
	_SI_PRINTF("Shape NbOfVertices %d\n", in_pShape->GetVertexCount() );
	_SI_PRINTF("Shape NbOfNormals %d\n", in_pShape->GetNormalCount() );
	_SI_PRINTF("Shape NbOfColors %d\n", in_pShape->GetColorCount() );

	if(in_pShape->Type() == CSLTemplate::SI_SHAPE)
	{
		CSLShape *l_pShape = (CSLShape *) in_pShape;
		_SI_PRINTF("Shape NbOfUVs %d\n", l_pShape->GetUVCoordCount() );
	}
	else
	{
		CSLShape_35 *l_pShape = (CSLShape_35 *) in_pShape;

		_SI_PRINTF("Shape NbOfUVArray %d\n", l_pShape->GetUVCoordArrayCount() );
		for(SI_Int i = 0;i < l_pShape->GetUVCoordArrayCount();i ++)
		{
			_SI_PRINTF("Shape UVArray #%d NbOfUVs %d\n", i, l_pShape->UVCoordArrays()[i]->GetUVCoordCount() );
		}
	}
}

void DescribeGeometry(CSLGeometry *in_pGeometry)
{
	CSLShapeAnimation *l_pShapeAnim = in_pGeometry->ShapeAnimation();
	if(l_pShapeAnim != NULL)
	{
		_SI_PRINTF("Current geometry has a shape animation\n");

		_SI_PRINTF("\tShapeAnim number of shapes: %d\n", l_pShapeAnim->GetShapeCount() );
		for(SI_Int sh = 0;sh < l_pShapeAnim->GetShapeCount();sh ++)
		{
			_SI_PRINTF("\tShapeAnim description of shape #%d:\n", sh);
			DescribeShape(l_pShapeAnim->Shapes()[sh]);
		}

		_SI_PRINTF("\tShapeAnim fcurve description: \n");
		OutputFCurve(l_pShapeAnim->Animation(), "shape animation");
	}
}

void DescribeNurbsCurve(CSLNurbsCurve* in_pCurve)
{
	_SI_PRINTF("NURBS Curve degree: %d\n", in_pCurve->GetDegree() );
	_SI_PRINTF("NURBS Curve closed: %s\n", in_pCurve->GetClosedFlag() ? "yes" : "no");
	_SI_PRINTF("NURBS Curve parametrization: %d\n", (int) in_pCurve->GetParametrization() );

	_SI_PRINTF("NURBS Curve number of knots: %d\n", in_pCurve->GetKnotCount() );
	_SI_PRINTF("NURBS Curve knots: \n");
	for(SI_Int k = 0;k < in_pCurve->GetKnotCount();k ++)
	{
		_SI_PRINTF("\t%f, \n", in_pCurve->GetKnotListPtr()[k]);
	}
	_SI_PRINTF("\n");

	_SI_PRINTF("NURBS Curve number of control points: %d\n", in_pCurve->GetControlPointCount() );
	_SI_PRINTF("NURBS Curve control points: \n");
	for(SI_Int cp = 0;cp < in_pCurve->GetControlPointCount();cp ++)
	{
		_SI_PRINTF("\t%f, ", in_pCurve->GetControlPointListPtr()[cp].m_fX);
		_SI_PRINTF("\t%f, ", in_pCurve->GetControlPointListPtr()[cp].m_fY);
		_SI_PRINTF("\t%f, ", in_pCurve->GetControlPointListPtr()[cp].m_fZ);
		_SI_PRINTF("\t%f, \n", in_pCurve->GetControlPointListPtr()[cp].m_fW);
	}
	_SI_PRINTF("\n");
}

void DescribeNurbsSurface(CSLNurbsSurface* in_pSurface)
{
	_SI_PRINTF("NURBS Surface degree in U: %d\n", in_pSurface->GetUDegree() );
	_SI_PRINTF("NURBS Surface degree in V: %d\n", in_pSurface->GetVDegree() );

	_SI_PRINTF("NURBS Surface closed in U: %s\n", in_pSurface->GetUClosedFlag() ? "yes" : "no");
	_SI_PRINTF("NURBS Surface closed in V: %s\n", in_pSurface->GetVClosedFlag() ? "yes" : "no");

	_SI_PRINTF("NURBS Surface parametrization: %d\n", (int) in_pSurface->GetParametrization() );

	_SI_PRINTF("NURBS Surface number of knots in U: %d\n", in_pSurface->GetUKnotCount() );
	_SI_PRINTF("NURBS Surface knots in U: \n");
	for(SI_Int ku = 0;ku < in_pSurface->GetUKnotCount();ku ++)
	{
		_SI_PRINTF("\t%f, \n", in_pSurface->GetUKnotListPtr()[ku]);
	}
	_SI_PRINTF("\n");

	_SI_PRINTF("NVRBS Surface number of knots in V: %d\n", in_pSurface->GetVKnotCount() );
	_SI_PRINTF("NVRBS Surface knots in V: \n");
	for(SI_Int kv = 0;kv < in_pSurface->GetVKnotCount();kv ++)
	{
		_SI_PRINTF("\t%f, \n", in_pSurface->GetVKnotListPtr()[kv]);
	}
	_SI_PRINTF("\n");

	_SI_PRINTF("NURBS Surface total number of control points: %d\n", in_pSurface->GetControlPointCount() );
	_SI_PRINTF("NURBS Surface number of control points in U: %d\n", in_pSurface->GetUControlPointCount() );
	_SI_PRINTF("NURBS Surface number of control points in V: %d\n", in_pSurface->GetVControlPointCount() );
	_SI_PRINTF("NURBS Surface control points: \n");
	for(SI_Int cp = 0;cp < in_pSurface->GetControlPointCount();cp ++)
	{
		_SI_PRINTF("\t%f, ", in_pSurface->GetControlPointListPtr()[cp].m_fX);
		_SI_PRINTF("\t%f, ", in_pSurface->GetControlPointListPtr()[cp].m_fY);
		_SI_PRINTF("\t%f, ", in_pSurface->GetControlPointListPtr()[cp].m_fZ);
		_SI_PRINTF("\t%f, \n", in_pSurface->GetControlPointListPtr()[cp].m_fW);
	}
	_SI_PRINTF("\n");

	ShowNurbsProjectionList( in_pSurface );
	ShowNurbsTrimList( in_pSurface );
}

void DescribeMesh(CSLMesh *in_pMesh)
{
	DescribeShape(in_pMesh->Shape() );

	_SI_PRINTF("Mesh NbOfPolyLists %d\n", in_pMesh->GetPolygonListCount() );

	int p;

	for (p=0;p<in_pMesh->GetPolygonListCount();p++)
	{
		CSLPolygonList*	l_pPolyList = in_pMesh->PolygonLists ()[p];

		// Output the material of the polygon list.
		if(l_pPolyList->GetMaterial() != NULL)
		{
			_SI_PRINTF("\tPolyList Material %s\n", l_pPolyList->GetMaterial()->GetName() );
		}

		SI_Int l_nPolyCount = l_pPolyList->GetPolygonVertexCountList()->GetUsed();
		_SI_PRINTF("\tPolyList NbOfPolys %d\n", l_nPolyCount );

		SI_Int l_nVertexCount = l_pPolyList->GetVertexIndices()->GetUsed();
		_SI_PRINTF("\tPolyList NbOfVertex %d\n", l_nVertexCount );

		_SI_PRINTF ( "\tPolygonVertexCountList:" );
		for (int np=0;np<l_nPolyCount;np++)
		{
			_SI_PRINTF ( "%d,",l_pPolyList->GetPolygonVertexCountList()->ArrayPtr()[np] );

		}
		_SI_PRINTF ( "\n" );

		_SI_PRINTF ( "\tVertex Indices:" );
		for (int vi=0;vi<l_nVertexCount;vi++)
		{
			_SI_PRINTF ( "%d,",l_pPolyList->GetVertexIndices()->ArrayPtr()[vi] );

		}
		_SI_PRINTF ( "\n" );

		int inc;

		if ( l_pPolyList->GetNormalIndices () != NULL )
		{
			_SI_PRINTF ( "\tNormal Indices:" );
			for (inc=0;inc<l_pPolyList->GetNormalIndices()->GetUsed();inc++)
			{
				_SI_PRINTF ( "%d,",l_pPolyList->GetNormalIndices()->ArrayPtr()[inc] );

			}
			
			_SI_PRINTF ( "\n" );
		}

		if ( l_pPolyList->GetUVIndices(0) != NULL )
		{
			_SI_PRINTF ( "\tUV Indices:" );
			for (inc=0;inc<l_pPolyList->GetUVIndices(0)->GetUsed();inc++)
			{
				_SI_PRINTF ( "%d,",l_pPolyList->GetUVIndices(0)->ArrayPtr()[inc] );
			}

			_SI_PRINTF ( "\n" );
		}

		if ( l_pPolyList->GetColorIndices () != NULL )
		{
			_SI_PRINTF ( "\tColor Indices:" );
			for (inc=0;inc<l_pPolyList->GetColorIndices()->GetUsed();inc++)
			{
				_SI_PRINTF ( "%d,",l_pPolyList->GetColorIndices()->ArrayPtr()[inc] );

			}
			
			_SI_PRINTF ( "\n" );
		}


	}

	_SI_PRINTF("---------------------\n");
	_SI_PRINTF("Mesh NbOfTriLists %d\n", in_pMesh->GetTriangleListCount() );

	for (p=0;p<in_pMesh->GetTriangleListCount();p++)
	{
		CSLTriangleList*	l_pTriList = in_pMesh->TriangleLists ()[p];

		// Output the material of the triangle list.
		if(l_pTriList->GetMaterial() != NULL)
		{
			_SI_PRINTF("\tTriList Material %s\n", l_pTriList->GetMaterial()->GetName() );
		}

		SI_Int l_nTriCount = l_pTriList->GetVertexIndices()->GetUsed();
		_SI_PRINTF("\tTriList NbOfTris %d\n", l_nTriCount );

		_SI_PRINTF ( "\tVertex Indices:" );
		SI_Int* l_pnVertexIndices = (SI_Int *) l_pTriList->GetVertexIndices ()->ArrayPtr();
		for (int vi=0;vi<l_nTriCount * 3;vi++)
		{
			_SI_PRINTF ( "%d,",l_pnVertexIndices[vi] );
		}
		_SI_PRINTF ( "\n" );

		int inc;

		if ( l_pTriList->GetNormalIndices () != NULL )
		{
			_SI_PRINTF ( "\tNormal Indices:" );
			SI_Int* l_pnNormalIndices = (SI_Int *) l_pTriList->GetNormalIndices ()->ArrayPtr();
			for (inc=0;inc<l_nTriCount * 3;inc++)
			{
				_SI_PRINTF ( "%d,",l_pnNormalIndices[inc] );
			}

			_SI_PRINTF ( "\n" );
		}

		if ( l_pTriList->GetColorIndices () != NULL )
		{
			_SI_PRINTF ( "\tColor Indices:" );
			SI_Int* l_pnColorIndices = (SI_Int *) l_pTriList->GetColorIndices ()->ArrayPtr();
			for (inc=0;inc<l_nTriCount * 3;inc++)
			{
				_SI_PRINTF ( "%d,",l_pnColorIndices[inc] );
			}

			_SI_PRINTF ( "\n" );
		}

		SI_Int l_nUVCount = l_pTriList->GetUVArrayCount();
		for(SI_Int uvinc = 0;uvinc < l_nUVCount;uvinc ++)
		{
			_SI_PRINTF ( "\tUV Indices #%d:" , uvinc );
			SI_Int* l_pnUVIndices = (SI_Int *) l_pTriList->GetUVIndices (uvinc)->ArrayPtr();
			for (inc=0;inc<l_nTriCount* 3;inc++)
			{
				_SI_PRINTF ( "%d,",l_pnUVIndices[inc] );
			}

			_SI_PRINTF ( "\n" );
		}
	}

	_SI_PRINTF("---------------------\n");
	_SI_PRINTF("Mesh NbOfTriStripLists %d\n", in_pMesh->GetTriangleStripListCount() );
	for (p=0;p<in_pMesh->GetTriangleStripListCount();p++)
	{
		CSLTriangleStripList*	l_pTriStripList = in_pMesh->TriangleStripLists ()[p];

		// Output the material of the triangle strip list.
		if(l_pTriStripList->GetMaterial() != NULL)
		{
			_SI_PRINTF("\tTriList Material %s\n", l_pTriStripList->GetMaterial()->GetName() );
		}

		SI_Int l_nTriCount = l_pTriStripList->GetTriangleStripCount();
		_SI_PRINTF("\tTriList NbOfTriStrips %d\n", l_nTriCount );

		for(int strip = 0;strip < l_nTriCount;strip ++)
		{
			CSLTriangleStrip*	l_pTriStrip = l_pTriStripList->TriangleStrips()[strip];

			_SI_PRINTF ( "\tVertex Indices:" );
			SI_Int* l_pnVertexIndices = (SI_Int *) l_pTriStrip->GetVertexIndices ()->ArrayPtr();
			for (int vi=0;vi<l_pTriStrip->GetVertexCount();vi++)
			{
				_SI_PRINTF ( "%d,",l_pnVertexIndices[vi] );
			}
			_SI_PRINTF ( "\n" );

			int inc;

			if ( l_pTriStrip->GetNormalIndices () != NULL )
			{
				_SI_PRINTF ( "\tNormal Indices:" );
				SI_Int* l_pnNormalIndices = (SI_Int *) l_pTriStrip->GetNormalIndices ()->ArrayPtr();
				for (inc=0;inc<l_pTriStrip->GetVertexCount();inc++)
				{
					_SI_PRINTF ( "%d,",l_pnNormalIndices[inc] );
				}

				_SI_PRINTF ( "\n" );
			}

			if ( l_pTriStrip->GetColorIndices () != NULL )
			{
				_SI_PRINTF ( "\tColor Indices:" );
				SI_Int* l_pnColorIndices = (SI_Int *) l_pTriStrip->GetColorIndices ()->ArrayPtr();
				for (inc=0;inc<l_pTriStrip->GetVertexCount();inc++)
				{
					_SI_PRINTF ( "%d,",l_pnColorIndices[inc] );
				}

				_SI_PRINTF ( "\n" );
			}

			SI_Int l_nUVCount = l_pTriStrip->GetUVArrayCount();
			for(SI_Int uvinc = 0;uvinc < l_nUVCount;uvinc ++)
			{
				_SI_PRINTF ( "\tUV Indices #%d:" , uvinc );
				SI_Int* l_pnUVIndices = (SI_Int *) l_pTriStrip->GetUVIndices (uvinc)->ArrayPtr();
				for (inc=0;inc<l_nTriCount* 3;inc++)
				{
					_SI_PRINTF ( "%d,",l_pnUVIndices[inc] );
				}

				_SI_PRINTF ( "\n" );
			}
		}
	}

	_SI_PRINTF("---------------------\n");
}

void DescribeModelRecursively(CSLModel *in_pModel)
{
	// Output the name of the model.
	_SI_PRINTF("The model is named \"%s\"\n", in_pModel->Name().GetText() );

	if(in_pModel->Template() )
	{
		// Output the posx with the naming resolution system.
		CSIBCString l_ParamName(in_pModel->Name().GetText() );
		l_ParamName.Concat(".kine.local.posx");
		SI_Float l_fValue;
		if(in_pModel->Scene()->GetValue(l_ParamName.GetText(), &l_fValue) == SI_SUCCESS)
		{
			_SI_PRINTF("%s : %f\n", l_ParamName.GetText(), l_fValue);
		}
		else
		{
			_SI_PRINTF("GetValue %s : ERROR\n");
		}

		l_fValue = 25.0f;
		if(in_pModel->Scene()->SetValue(l_ParamName.GetText(), l_fValue) == SI_SUCCESS)
		{
			_SI_PRINTF("%s : %f\n", l_ParamName.GetText(), l_fValue);
		}
		else
		{
			_SI_PRINTF("SetValue %s : ERROR\n");
		}

		// Output the model Visibility.
		CSLVisibility *l_pVisibility = in_pModel->Visibility();
		_SI_PRINTF("The visibility state of the model is currently %d\n", (SI_Int) l_pVisibility->GetVisibility() );
		OutputFCurve(l_pVisibility->GetSpecificFCurve(CSLTemplate::SI_NODEVIS), "visibility" );

		// Output the local transform.
		CSLTransform *l_pLocal = in_pModel->Transform();
		_SI_PRINTF("The model has the following local transform :\n");

		// Output the scale.
		CSIBCVector3D l_Vector = l_pLocal->GetScale();
		_SI_PRINTF("\tScale XZY : %f ; %f ; %f\n", l_Vector.m_fX, l_Vector.m_fY, l_Vector.m_fZ);
		OutputFCurve(l_pLocal->GetSpecificFCurve(CSLTemplate::SI_SCALING_X), "scale x");
		OutputFCurve(l_pLocal->GetSpecificFCurve(CSLTemplate::SI_SCALING_Y), "scale y");
		OutputFCurve(l_pLocal->GetSpecificFCurve(CSLTemplate::SI_SCALING_Z), "scale z");

		// Output the rotation
		l_Vector = l_pLocal->GetEulerRotation();
		_SI_PRINTF("\tRotation XZY : %f ; %f ; %f\n", l_Vector.m_fX, l_Vector.m_fY, l_Vector.m_fZ);
		OutputFCurve(l_pLocal->GetSpecificFCurve(CSLTemplate::SI_ROTATION_X), "rotation x");
		OutputFCurve(l_pLocal->GetSpecificFCurve(CSLTemplate::SI_ROTATION_Y), "rotation y");
		OutputFCurve(l_pLocal->GetSpecificFCurve(CSLTemplate::SI_ROTATION_Z), "rotation z");

		// Output the translation
		l_Vector = l_pLocal->GetTranslation();
		_SI_PRINTF("\tTranslation XZY : %f ; %f ; %f\n", l_Vector.m_fX, l_Vector.m_fY, l_Vector.m_fZ);
		OutputFCurve(l_pLocal->GetSpecificFCurve(CSLTemplate::SI_TRANSLATION_X), "translation x");
		OutputFCurve(l_pLocal->GetSpecificFCurve(CSLTemplate::SI_TRANSLATION_Y), "translation y");
		OutputFCurve(l_pLocal->GetSpecificFCurve(CSLTemplate::SI_TRANSLATION_Z), "translation z");

		// output Global Material
		CSLGlobalMaterial *l_pGlobalMaterial = in_pModel->GlobalMaterial();
		if (l_pGlobalMaterial)
		{
			CSLBaseMaterial *l_pMaterial = l_pGlobalMaterial->GetMaterial();
			_SI_PRINTF
			(
				"\tGlobal Material:\n\t\tPropagation: %d\n\t\tReferenced Material: %s\n",
				l_pGlobalMaterial->GetPropagationType(),
				(l_pMaterial) ? l_pMaterial->GetName() : "N/A"
			);
		}
		else
			_SI_PRINTF("\tNo Global Material\n" );

		// Output Constraints
		int l_nCount = in_pModel->GetConstraintCount();
		CSLConstraint **l_ppConstraint = in_pModel->GetConstraintList();
		if (l_nCount)
		{
			_SI_PRINTF("\tConstraints\n");
			for ( int i = 0; i < l_nCount; i++ )
			{
				_SI_PRINTF("\t\tType : ");
				switch(l_ppConstraint[i]->GetConstraintType() )
				{
				case CSLConstraint::SI_SCALING:
					_SI_PRINTF("\t\tScaling\n");
					break;
				case CSLConstraint::SI_ORIENTATION:
					_SI_PRINTF("\t\tOrientation\n");
					break;
				case CSLConstraint::SI_DIRECTION:
					_SI_PRINTF("\t\tDirection\n");
					break;
				case CSLConstraint::SI_POSITION:
					_SI_PRINTF("\t\tPosition\n");
					break;
				case CSLConstraint::SI_UP_VECTOR:
					_SI_PRINTF("\t\tUp Vector\n");
					break;
				case CSLConstraint::SI_PREFERED_AXIS:
					_SI_PRINTF("\t\tPrefered Axis\n");
					break;
				}
				_SI_PRINTF("\t\tConstraining objects : %d\n", l_ppConstraint[i]->GetConstrainingObjectCount() );
				CSLModel **l_ppObjects = l_ppConstraint[i]->ConstrainingObjects();
				for(SI_Int j = 0;j < l_ppConstraint[i]->GetConstrainingObjectCount();j ++)
				{
					_SI_PRINTF("\t\t\t%s\n", l_ppObjects[j]->GetName() );
				}
			}
		}
		
		// output Clusters (to be updated when Clusters are done )
		l_nCount = in_pModel->GetClusterCount();
		CSLCluster **l_ppCluster = in_pModel->GetClusterList();
		CSLCluster *l_pCluster;
		if (l_nCount)
		{
			_SI_PRINTF("\tClusters\n");
			for ( int i = 0; i < in_pModel->GetClusterCount(); i++ )
			{
				l_pCluster = l_ppCluster[i];
				_SI_PRINTF("\t\tName : %s\n", l_pCluster->GetName() );
				_SI_PRINTF("\t\tNb of points : %d\n", l_pCluster->GetVertexIndicesCount() );

				ShowUserDataLists( l_pCluster );
			}
		}

		// output Envelopes (to be updated when Envelopes are done )
		l_nCount = in_pModel->GetEnvelopeCount();
		CSLEnvelope **l_ppEnvelope = in_pModel->GetEnvelopeList();
		if (l_nCount)
		{
			_SI_PRINTF("\tEnvelopes\n");
			for ( int i = 0; i < in_pModel->GetEnvelopeCount(); i++ )
			{
				_SI_PRINTF("\t\tDeformer : %s\n", (l_ppEnvelope[i])->GetDeformer()->GetName() );
			}
		}
	
		// output mixer (to be updated when mixer is done)
		CSLMixer *l_pMixer = in_pModel->Mixer();
		if (l_pMixer)
		{
			SI_Int l_nCount = l_pMixer->GetActionCount();
			_SI_PRINTF
			(
				"\tMixer:%s\n\t\tAutoTransition : %d, Active %d, Normalize %d\n\t\tQuaternion Mix %d, Remove Spin %d, MaintainContinuity %d\n\t\t Nb Actions %d\n",
				l_pMixer->GetName(),
				l_pMixer->GetAutoTransition(),
				l_pMixer->GetActive(),
				l_pMixer->GetNormalize(), 
				l_pMixer->GetQuaternionMixing(),
				l_pMixer->GetRemoveSpins(),
				l_pMixer->GetMaintainContinuity(),
				l_nCount
			);

			// display actions
			CSLAction **l_ppActions = l_pMixer->GetActionList();
			SI_Int i = 0;
			for (;i < l_nCount; i++)
			{
				DescribeAction(l_ppActions[i]);
			}

			// display tracks
			CSLTrack **l_ppTracks = l_pMixer->GetTrackList();
			for (i = 0;i < l_pMixer->GetTrackCount(); i++)
			{
				DescribeTrack(l_ppTracks[i]);
			}
		}

		// output FXTrees if any
		ShowFXTrees(in_pModel);

		// output CustomPSet if any
		l_nCount = in_pModel->GetCustomPSetCount();
		if (l_nCount)
		{
			SI_Int l_nPSetCount = 0;
			CSLCustomPSet **l_ppCustomPSet = in_pModel->GetCustomPSetList();
			CSLCustomPSet *l_pCustomPSet = 0;
			CSLVariantParameter **l_ppParameters = 0;
			CSLVariantParameter *l_pParameter = 0;
			SI_TinyVariant *l_pVariant = 0;


			_SI_PRINTF("\tCustomPSets: %i\n", l_nCount );
			for (int i = 0; i < l_nCount; i++)
			{
				l_pCustomPSet = l_ppCustomPSet[i];
				_SI_PRINTF
				(
					"\t\t#%i Name: %s\n\t\t Propagation: %i\n",
					i,
					l_pCustomPSet->GetName(),
					l_pCustomPSet->GetPropagationType()
				);

				// display parmeters of the preset
				l_nPSetCount = l_pCustomPSet->GetParameterCount();
				_SI_PRINTF("\t\tNb Parameters: %i\n", l_nPSetCount );

				// if there are parameters
				if (l_nPSetCount)
				{
					l_ppParameters = l_pCustomPSet->GetParameterList();
					
					for (int j = 0; j < l_nPSetCount; j++)
					{
						l_pParameter = l_ppParameters[j];
						_SI_PRINTF("\t\t\tName, Value: %s, ", l_pParameter->GetName());
						l_pVariant = l_pParameter->GetValue();

						switch (l_pVariant->variantType)
						{
							case SI_VT_PCHAR: _SI_PRINTF("%s\n", l_pVariant->p_cVal ); break;
							case SI_VT_FLOAT: 
							case SI_VT_DOUBLE: _SI_PRINTF("%d\n", l_pVariant->p_fVal ); break;
							default: _SI_PRINTF("%i\n", l_pVariant->p_bVal );
						}

						CSLXSICustomParamInfo *l_pInfo = ((CSLCustomParameter*)l_pParameter)->CustomParamInfo();
						if (l_pInfo) 
							ShowCustomParamInfo(l_pInfo); 
					}
				}
			}
		}
	}

	CSLPrimitive *l_pPrimitive = in_pModel->Primitive();
	switch( in_pModel->GetPrimitiveType() )
	{
	case CSLTemplate::SI_CAMERA:
		{
		CSLCamera *l_pCamera = (CSLCamera *) in_pModel->Primitive();
		_SI_PRINTF("Primitive of the model is a camera.\n");
		_SI_PRINTF("The camera has an interest positioned at : %f ; %f ; %f\n", l_pCamera->GetInterestPosition().m_fX, l_pCamera->GetInterestPosition().m_fY, l_pCamera->GetInterestPosition().m_fZ);
		break;
		}
	case CSLTemplate::SI_IK_ROOT:
		_SI_PRINTF("Primitive of the model is an ik root.\n");
		DescribeIKRoot( (CSLIKRoot*) in_pModel->Primitive() );
		break;
	case CSLTemplate::SI_INFINITE_LIGHT:
		_SI_PRINTF("Primitive of the model is an infinite light.\n");
		ShowLightInfo( l_pPrimitive );
		break;
	case CSLTemplate::SI_SPOT_LIGHT:
		_SI_PRINTF("Primitive of the model is a spot light.\n");
		ShowLightInfo( l_pPrimitive );
		break;
	case CSLTemplate::SI_POINT_LIGHT:
		_SI_PRINTF("Primitive of the model is a point light.\n");
		ShowLightInfo( l_pPrimitive );
		break;
	case CSLTemplate::SI_DIRECTIONAL_LIGHT:
		_SI_PRINTF("Primitive of the model is a directional light.\n");
		ShowLightInfo( l_pPrimitive );
		break;
	case CSLTemplate::SI_MODEL:
		_SI_PRINTF("The model is an XSI model.\n");
		break;
	case CSLTemplate::SI_NULL_OBJECT:
		_SI_PRINTF("Primitive of the model is a null.\n");
		break;
	case CSLTemplate::SI_MESH:
		DescribeMesh( (CSLMesh*) in_pModel->Primitive() );
		DescribeGeometry( (CSLGeometry*) in_pModel->Primitive() );
		break;
	case CSLTemplate::SI_NURBS_CURVE:
		DescribeNurbsCurve( (CSLNurbsCurve *) in_pModel->Primitive() );
		break;
	case CSLTemplate::SI_NURBS_SURFACE:
		DescribeNurbsSurface( (CSLNurbsSurface *) in_pModel->Primitive() );
		break;
	case CSLTemplate::SI_CURVE_LIST:
		{
		_SI_PRINTF("Primitive is a curve list\n");
		CSLCurveList *l_pCurveList = (CSLCurveList *) in_pModel->Primitive();
		for(SI_Int i = 0;i < l_pCurveList->GetCurveCount();i ++)
		{
			_SI_PRINTF("Describing curve #%d\n", i);
			DescribeNurbsCurve(l_pCurveList->Curves()[i] );
			DescribeGeometry(l_pCurveList->Curves()[i] );
		}
		break;
		}
	case CSLTemplate::SI_SURFACE_MESH:
		{
		_SI_PRINTF("Primitive is a surface mesh\n");
		CSLSurfaceMesh *l_pSrfMesh = (CSLSurfaceMesh *) in_pModel->Primitive();
		for(SI_Int i = 0;i < l_pSrfMesh->GetSurfaceCount();i ++)
		{
			_SI_PRINTF("Describing surface #%d\n", i);
			DescribeNurbsSurface(l_pSrfMesh->Surfaces()[i] );
			DescribeGeometry(l_pSrfMesh->Surfaces()[i]);
		}
		break;
		}
	case CSLTemplate::SI_INSTANCE:
		{
		_SI_PRINTF("Primitive is an instance\n");
		CSLInstance* l_pInstance = (CSLInstance*) in_pModel->Primitive();
		_SI_PRINTF("Model referenced by the instance is named : %s\n", l_pInstance->GetReference()->GetName() );
		break;
		}
	}

	SI_Int l_nCount = in_pModel->GetChildrenCount();
	if(l_nCount > 0)
	{
		_SI_PRINTF("Model has %d children.\nFollowing is a description for each of them.\n", l_nCount);

		// Recurse through all children.
		CSLModel **l_ppChildren = in_pModel->GetChildrenList();

		for(SI_Int i = 0;i < l_nCount;i ++)
		{
			DescribeModelRecursively(l_ppChildren[i]);
		}
	}
}

void FillFog(CSLFog *in_pFog)
{
	CSIBCColorf	l_cDefault(0.1f, 0.3f, 0.7f);
	in_pFog->SetFogType(CSLFog::SI_PIXEL);
	in_pFog->SetFogInterpolationType(CSLFog::SI_EXPONENTIAL);
	in_pFog->SetColor( l_cDefault );
	in_pFog->SetNear(3.6f);
	in_pFog->SetFar(250.0f);
}

void FillPolygon(CSLPolygonList *in_pPoly)
{
	in_pPoly->GetPolygonVertexCountList()->Resize(1);
	(*in_pPoly->GetPolygonVertexCountList() )[0] = 4;
	in_pPoly->GetVertexIndices()->Resize(4);
	SI_Int *l_pnDataArray = in_pPoly->GetVertexIndices()->ArrayPtr();
	l_pnDataArray[0] = 0;
	l_pnDataArray[1] = 1;
	l_pnDataArray[2] = 2;
	l_pnDataArray[3] = 3;

	// Add some tex coords to the poly.
	CSLPolygonList::CSLIntArray* l_pTexCoord = in_pPoly->AddUVArray();
	l_pTexCoord->Resize(4);
	l_pnDataArray = l_pTexCoord->ArrayPtr();
	l_pnDataArray[0] = 1;
	l_pnDataArray[1] = 2;
	l_pnDataArray[2] = 0;
	l_pnDataArray[3] = 3;

	// Add some color to the poly.
	CSLPolygonList::CSLIntArray* l_pColorArray = in_pPoly->CreateColorIndices();
	l_pColorArray->Resize(4);
	l_pnDataArray = l_pColorArray->ArrayPtr();
	l_pnDataArray[0] = 0;
	l_pnDataArray[1] = 1;
	l_pnDataArray[2] = 1;
	l_pnDataArray[3] = 0;

	// Add some normals to the poly.
	CSLPolygonList::CSLIntArray* l_pNormalArray = in_pPoly->CreateNormalIndices();
	l_pNormalArray->Resize(4);
	l_pnDataArray = l_pNormalArray->ArrayPtr();
	l_pnDataArray[0] = 3;
	l_pnDataArray[1] = 2;
	l_pnDataArray[2] = 1;
	l_pnDataArray[3] = 0;

	// Delete all poly information and only add color.
	in_pPoly->DestroyColorIndices();
	in_pPoly->DestroyNormalIndices();
	in_pPoly->RemoveUVArray();

	l_pColorArray = in_pPoly->CreateColorIndices();
	l_pColorArray->Resize(4);
	l_pnDataArray = l_pColorArray->ArrayPtr();
	l_pnDataArray[0] = 1;
	l_pnDataArray[1] = 0;
	l_pnDataArray[2] = 0;
	l_pnDataArray[3] = 1;
}

void FillTriangle(CSLTriangleList *in_pTri)
{
	in_pTri->GetVertexIndices()->Resize(2);
	SLTriangle *l_pnDataArray = in_pTri->GetVertexIndices()->ArrayPtr();
	l_pnDataArray[0].m_iVtx1 = 0;
	l_pnDataArray[0].m_iVtx2 = 1;
	l_pnDataArray[0].m_iVtx3 = 2;
	l_pnDataArray[1].m_iVtx1 = 3;
	l_pnDataArray[1].m_iVtx2 = 0;
	l_pnDataArray[1].m_iVtx3 = 1;

	// Add some tex coords to the triangle.
	CSLTriangleList::CSLTriangleArray* l_pTexCoord = in_pTri->AddUVArray();
	l_pTexCoord->Resize(2);
	l_pnDataArray = l_pTexCoord->ArrayPtr();
	l_pnDataArray[0].m_iVtx1 = 0;
	l_pnDataArray[0].m_iVtx2 = 1;
	l_pnDataArray[0].m_iVtx3 = 2;
	l_pnDataArray[1].m_iVtx1 = 2;
	l_pnDataArray[1].m_iVtx2 = 1;
	l_pnDataArray[1].m_iVtx3 = 0;

	// Add some color to the triangle.
	CSLTriangleList::CSLTriangleArray* l_pColorArray = in_pTri->CreateColorIndices();
	l_pColorArray->Resize(2);
	l_pnDataArray = l_pColorArray->ArrayPtr();
	l_pnDataArray[0].m_iVtx1 = 0;
	l_pnDataArray[0].m_iVtx2 = 1;
	l_pnDataArray[0].m_iVtx3 = 0;
	l_pnDataArray[1].m_iVtx1 = 1;
	l_pnDataArray[1].m_iVtx2 = 0;
	l_pnDataArray[1].m_iVtx3 = 1;

	// Add some normals to the triangle.
	CSLTriangleList::CSLTriangleArray* l_pNormalArray = in_pTri->CreateNormalIndices();
	l_pNormalArray->Resize(2);
	l_pnDataArray = l_pNormalArray->ArrayPtr();
	l_pnDataArray[0].m_iVtx1 = 2;
	l_pnDataArray[0].m_iVtx2 = 1;
	l_pnDataArray[0].m_iVtx3 = 0;
	l_pnDataArray[1].m_iVtx1 = 3;
	l_pnDataArray[1].m_iVtx2 = 2;
	l_pnDataArray[1].m_iVtx3 = 1;

	// Delete all triangle information and only add color.
	in_pTri->DestroyColorIndices();
	in_pTri->DestroyNormalIndices();
	in_pTri->RemoveUVArray();

	l_pColorArray = in_pTri->CreateColorIndices();
	l_pColorArray->Resize(2);
	l_pnDataArray = l_pColorArray->ArrayPtr();
	l_pnDataArray[0].m_iVtx1 = 0;
	l_pnDataArray[0].m_iVtx2 = 0;
	l_pnDataArray[0].m_iVtx3 = 0;
	l_pnDataArray[1].m_iVtx1 = 1;
	l_pnDataArray[1].m_iVtx2 = 1;
	l_pnDataArray[1].m_iVtx3 = 1;
}

void FillTriangleStrip(CSLTriangleStripList *in_pTriStrip)
{
	// Add a single strip & fill vertex data.
	CSLTriangleStrip* l_pStrip = in_pTriStrip->AddTriangleStrip();
	l_pStrip->GetVertexIndices()->Resize(4);
	SI_Int *l_pnDataArray = l_pStrip->GetVertexIndicesPtr();
	l_pnDataArray[0] = 0;
	l_pnDataArray[1] = 1;
	l_pnDataArray[2] = 2;
	l_pnDataArray[3] = 3;

	// Add some tex coords to the strip.
	CSLTriangleStrip::CSLIntArray* l_pTexCoord = l_pStrip->AddUVArray();
	l_pTexCoord->Resize(4);
	l_pnDataArray = l_pTexCoord->ArrayPtr();
	l_pnDataArray[0] = 3;
	l_pnDataArray[1] = 2;
	l_pnDataArray[2] = 1;
	l_pnDataArray[3] = 0;

	// Add some color to the triangle.
	CSLTriangleStrip::CSLIntArray* l_pColorArray = l_pStrip->CreateColorIndices();
	l_pColorArray->Resize(4);
	l_pnDataArray = l_pColorArray->ArrayPtr();
	l_pnDataArray[0] = 0;
	l_pnDataArray[1] = 1;
	l_pnDataArray[2] = 1;
	l_pnDataArray[3] = 0;

	// Add some normals to the triangle.
	CSLTriangleStrip::CSLIntArray* l_pNormalArray = l_pStrip->CreateNormalIndices();
	l_pNormalArray->Resize(4);
	l_pnDataArray = l_pNormalArray->ArrayPtr();
	l_pnDataArray[0] = 0;
	l_pnDataArray[1] = 2;
	l_pnDataArray[2] = 1;
	l_pnDataArray[3] = 3;

	// Delete all triangle information and only add color.
	l_pStrip->DestroyColorIndices();
	l_pStrip->DestroyNormalIndices();
	l_pStrip->RemoveUVArray();

	l_pColorArray = l_pStrip->CreateColorIndices();
	l_pColorArray->Resize(4);
	l_pnDataArray = l_pColorArray->ArrayPtr();
	l_pnDataArray[0] = 0;
	l_pnDataArray[1] = 0;
	l_pnDataArray[2] = 0;
	l_pnDataArray[3] = 1;
}

void FillShapeAnimation(CSLShapeAnimation *in_pShapeAnim)
{
	{
		CSLBaseShape::CSLIndexedVector3DArray& l_VertexArray = *in_pShapeAnim->AddShape()->GetIndexedVertexList();
		l_VertexArray.Resize(2);
		l_VertexArray[0].m_fIndex = 0;
		l_VertexArray[0].m_Vector3D.Set(0.0, 0.0, 0.0);
		l_VertexArray[0].m_fIndex = 2;
		l_VertexArray[0].m_Vector3D.Set(1.0, 2.0, 3.0);
		l_VertexArray[1].m_fIndex = 0;
		l_VertexArray[1].m_Vector3D.Set(4.0, 3.0, 6.0);
		l_VertexArray[1].m_fIndex = 2;
		l_VertexArray[1].m_Vector3D.Set(8.0, 7.0, 9.0);
	}

	{
		CSLBaseShape::CSLIndexedVector3DArray& l_VertexArray = *in_pShapeAnim->InsertShape(0)->GetIndexedVertexList();
		l_VertexArray.Resize(2);
		l_VertexArray[0].m_fIndex = 0;
		l_VertexArray[0].m_Vector3D.Set(0.0, 0.0, 0.0);
		l_VertexArray[0].m_fIndex = 2;
		l_VertexArray[0].m_Vector3D.Set(-1.0, -2.0, -3.0);
		l_VertexArray[1].m_fIndex = 0;
		l_VertexArray[1].m_Vector3D.Set(-4.0, -3.0, -6.0);
		l_VertexArray[1].m_fIndex = 2;
		l_VertexArray[1].m_Vector3D.Set(-8.0, -7.0, -9.0);
	}
}

void FillCurve(CSLNurbsCurve* in_pCurve)
{
	in_pCurve->SetDegree(3);
	in_pCurve->SetClosedFlag(SI_FALSE);
	in_pCurve->SetParametrization(CSLNurbsCurve::SI_NON_UNIFORM);

	CSLNurbsCurve::CSLFloatArray& l_KnotArray = *in_pCurve->GetKnotList();
	l_KnotArray.Resize(8);
	l_KnotArray[0] = 0.0;
	l_KnotArray[1] = 0.0;
	l_KnotArray[2] = 0.0;
	l_KnotArray[3] = 0.0;
	l_KnotArray[4] = 1.0;
	l_KnotArray[5] = 1.0;
	l_KnotArray[6] = 1.0;
	l_KnotArray[7] = 1.0;


	CSLNurbsCurve::CSLVector4DArray& l_ControlPointArray = *in_pCurve->GetControlPointList();
	l_ControlPointArray.Resize(4);
	static SI_Float l_fOffset = -2.0;

	for(SI_Int cp = 0;cp < in_pCurve->GetControlPointCount();cp ++)
	{
		l_ControlPointArray[cp].m_fX = l_fOffset;
		l_ControlPointArray[cp].m_fY = l_fOffset / 2.0;
		l_ControlPointArray[cp].m_fZ = 0.0;
		l_ControlPointArray[cp].m_fW = 1.0;

		l_fOffset += 0.5;
	}
}

void FillSurface(CSLNurbsSurface* in_pSurface)
{
	in_pSurface->SetUDegree(3);
	in_pSurface->SetVDegree(3);

	in_pSurface->SetUClosedFlag(SI_FALSE);
	in_pSurface->SetVClosedFlag(SI_FALSE);

	in_pSurface->SetParametrization(CSLNurbsSurface::SI_NON_UNIFORM);

	CSLNurbsSurface::CSLFloatArray& l_UKnotArray = *in_pSurface->GetUKnotList();
	l_UKnotArray.Resize(8);
	l_UKnotArray[0] = 0.0F;
	l_UKnotArray[1] = 0.0F;
	l_UKnotArray[2] = 0.0F;
	l_UKnotArray[3] = 0.0F;
	l_UKnotArray[4] = 1.0F;
	l_UKnotArray[5] = 1.0F;
	l_UKnotArray[6] = 1.0F;
	l_UKnotArray[7] = 1.0F;

	CSLNurbsSurface::CSLFloatArray& l_VKnotArray = *in_pSurface->GetVKnotList();
	l_VKnotArray.Copy(l_UKnotArray);

	in_pSurface->SetUControlPointCount(4);
	in_pSurface->SetVControlPointCount(4);

	CSLNurbsSurface::CSLVector4DArray& l_ControlPointArray = *in_pSurface->GetControlPointList();
	l_ControlPointArray.Resize(in_pSurface->GetUControlPointCount() * in_pSurface->GetVControlPointCount() );
	static SI_Float l_fOffset = -2.0;

	for(SI_Int cp = 0;cp < in_pSurface->GetControlPointCount();cp ++)
	{
		l_ControlPointArray[cp].m_fX = l_fOffset;
		l_ControlPointArray[cp].m_fY = l_fOffset * 2.0F;
		l_ControlPointArray[cp].m_fZ = 0.0F;
		l_ControlPointArray[cp].m_fW = 1.0F;

		l_fOffset += 0.25F;
	}
}

void FillModel(CSLModel *in_pModel, SI_Char *in_szName)
{
	CSIBCVector3D	l_vDefault(1.0, 1.0, 1.0);

	in_pModel->SetName(in_szName);

	in_pModel->Visibility()->SetVisibility(SI_TRUE);

	in_pModel->Transform()->SetScale( l_vDefault );

	//
	// Make sure we have a GlobalMaterial.
	//
	if (NULL == in_pModel->GlobalMaterial())
		in_pModel->AddGlobalMaterial();
	in_pModel->GlobalMaterial()->SetPropagationType(CSLGlobalMaterial::SI_BRANCH);
	in_pModel->GlobalMaterial()->SetMaterial(in_pModel->Scene()->GetMaterialLibrary()->GetMaterialList()[0] );
}

void FillScene(CSLScene *in_pScene)
{
	int i;

	// add a spot light with light info
	AddSpotLight(in_pScene, "The_Light");

	// create image library
	AddImageLibrary(in_pScene);

	// Create simple templates
	CreateSimpleTemplates(*in_pScene);

	// Adding models.
	CSLModel *l_pModel = in_pScene->Root()->AddModel();
	l_pModel->Name().SetText("Parent");
	AddGlobalMaterial(in_pScene, l_pModel, CSLGlobalMaterial::SI_NODE, "Parent.Material" );
	CSLCustomPSet *l_pPreset = AddCustomPSet(l_pModel);

	AddXSIMaterial(in_pScene, "The_XSIMaterial", l_pModel);
	AddFXTree(l_pModel);

	// add mixer
	CSLMixer *l_pMixer = AddMixer(l_pModel, "MyMixer");
	
	// Add a NULL
	CSLModel* l_pNull = l_pModel->AddNull();
	FillModel(l_pNull, "ReallyNull");

	CSLModel *l_pChild = l_pModel->AddModel();
	FillModel(l_pChild, "Child");
	AddGlobalMaterial(in_pScene, l_pChild, CSLGlobalMaterial::SI_BRANCH, "Children.Material" );

	// Changing visibility.
	l_pModel->Visibility()->SetVisibility(SI_TRUE);

	// Adding an animation on the visibility.
	CSLFCurve *l_pCurve = l_pModel->Visibility()->CreateFCurve(CSLTemplate::SI_NODEVIS, CSLTemplate::SI_CONSTANT);
	l_pCurve->GetConstantKeyList()->Extend(2);
	for(i = 0;i < l_pCurve->GetConstantKeyList()->GetUsed();i ++)
	{
		(*l_pCurve->GetConstantKeyList() )[i].m_fTime = (float) (i + 1);
		(*l_pCurve->GetConstantKeyList() )[i].m_fValue = (float) (i % 2);
	}
	l_pCurve->GetConstantKeyList()->Pack(0);
	l_pCurve->GetConstantKeyList()->DeleteAt(0,1);
	l_pCurve->GetConstantKeyList()->Resize(10);
	for(i = 0;i < l_pCurve->GetConstantKeyList()->GetUsed();i ++)
	{
		(*l_pCurve->GetConstantKeyList() )[i].m_fTime = (float) (i + 1);
		(*l_pCurve->GetConstantKeyList() )[i].m_fValue = (float) (i % 2);
	}

	// Changing the ambience.
	CSLAmbience *l_pAmbience = in_pScene->Ambience();
	CSIBCColorf l_Color = l_pAmbience->GetColor();
	l_Color.m_fR = 1.0;
	l_Color.m_fG = 0.75;
	l_Color.m_fB = 0.2368f;
	l_pAmbience->SetColor(l_Color);

	// Adding a new material.
	CSLMaterial *l_pNewMaterial;
	l_pNewMaterial = in_pScene->GetMaterialLibrary()->AddMaterial();
	l_pNewMaterial->SetName("MATNO1");

	// Add a texture to this material.
	l_pNewMaterial->CreateTexture2D()->SetImageFileName("Image.bmp");

	// Adding a camera.
	CSLModel *l_CamModel = in_pScene->Root()->AddCamera();
	l_CamModel->Name().SetText("Cam");

	// Changing cam properties.
	CSLCamera *l_Cam = (CSLCamera *) l_CamModel->Primitive();
	l_Cam->SetFarPlane(100);
	l_Cam->SetNearPlane(0.5);

	// Adding Animation on the interest position.
	l_pCurve = l_Cam->CreateFCurve(CSLTemplate::SI_INTEREST_X, CSLTemplate::SI_LINEAR);
	l_pCurve->GetLinearKeyList()->Resize(10);

	for(i = 0;i < l_pCurve->GetLinearKeyList()->GetUsed();i ++)
	{
		(*l_pCurve->GetLinearKeyList() )[i].m_fTime = (float) (i + 1);
		(*l_pCurve->GetLinearKeyList() )[i].m_fValue = (float) (i % 2);
	}

	// Adding lots of child.
	CSLModel *A = in_pScene->Root()->AddModel();
	FillModel(A, "A");

	CSLModel *B = A->AddModel();
	FillModel(B, "B");

	CSLModel *C = B->AddModel();
	FillModel(C, "C");

	// Add a constraint
	CSLConstraint *l_pConstraint = l_pModel->AddConstraint(CSLConstraint::SI_POSITION);
	l_pConstraint->AddConstrainingObject(A);

	// Try to reparent models.
	if(C->Reparent(A) != SI_SUCCESS)
	{
		_SI_PRINTF("Reparenting error");
	}

	// This should fail since we try to reparent the model under its own child.
	if(A->Reparent(B) == SI_SUCCESS)
	{
		_SI_PRINTF("Reparenting error");
	}

	// Create a chain with 3 bones.
	CSLModel *l_pRootModel = in_pScene->Root()->AddIKChain(3);
	FillModel(l_pRootModel, "Root");
	CSIBCMatrix4x4	l_mRootXForm(l_pRootModel->Transform()->GetMatrix());
	l_pRootModel->CreateBasePose()->SetMatrix( l_mRootXForm );

	// Let's try to reparent the effector.
	CSLIKRoot *l_pRoot = ( (CSLIKRoot *) l_pRootModel->Primitive() );

	for(SI_Int b = 0;b < l_pRoot->GetJointCount();b ++)
	{
		CSLModel *l_pBoneModel = l_pRoot->GetJointList()[b];
		FillModel(l_pBoneModel, "Bone");
		l_pBoneModel->Name().Concat(b + 1);
		CSLIKJoint *l_pBone = (CSLIKJoint*) (l_pBoneModel->Primitive() );
		if(b == 0)
		{
			l_pBone->SetSolverType(CSLIKJoint::SI_2D);
		}
		else
		{
			CSIBCVector3D	l_vDefault( 1.0, 0.0, 0.0);
			l_pBoneModel->Transform()->SetTranslation( l_vDefault );
			l_pBone->SetSolverType(CSLIKJoint::SI_DEFAULT);
		}
		CSIBCMatrix4x4	l_mBoneMatrix(l_pBoneModel->Transform()->GetMatrix());
		l_pBoneModel->CreateBasePose()->SetMatrix( l_mBoneMatrix );
		( (CSLIKJoint*) (l_pBoneModel->Primitive() ) )->SetLength(1.0);
	}

	CSLModel *l_pEffModel = l_pRoot->GetEffector();
	FillModel(l_pEffModel, "Eff");
	CSIBCVector3D	l_vDefault(1.0, 0.0, 0.0);
	l_pEffModel->Transform()->SetTranslation( l_vDefault );
	CSIBCMatrix4x4	l_mEffMatrix(l_pEffModel->Transform()->GetMatrix());
	l_pEffModel->CreateBasePose()->SetMatrix( l_mEffMatrix );

	// This should fail since we try to reparent the effector under one of the ik joint of the chain.
	if(l_pEffModel->Reparent(l_pRoot->GetJointList()[0]) == SI_SUCCESS)
	{
		_SI_PRINTF("Reparenting error");
	}

	// This should work since the new parent of the effector is the last bone of the ik chain.
	if(l_pEffModel->Reparent(l_pRoot->GetJointList()[2]) != SI_SUCCESS)
	{
		_SI_PRINTF("Reparenting error");
	}

	// Create a simple mesh.
	CSLModel *l_pMeshModel = in_pScene->Root()->AddMesh();
	FillModel(l_pMeshModel, "Grid");

	CSLMesh *l_pMesh = (CSLMesh *) l_pMeshModel->Primitive();
	CSLBaseShape *l_pShape = l_pMesh->Shape();

	// Add some position coordinates
	CSLBaseShape::CSLVector3DArray *l_Vertices = l_pShape->GetVertexList();
	l_Vertices->Resize(4);
	(*l_Vertices)[0].Set(1.0, 0.0, 0.0);
	(*l_Vertices)[1].Set(1.0, 1.0, 0.0);
	(*l_Vertices)[2].Set(0.0, 1.0, 0.0);
	(*l_Vertices)[3].Set(0.0, 0.0, 0.0);

	// Add some color values.
	CSLBaseShape::CSLColorArray &l_Colors = *l_pShape->CreateColors();
	l_Colors.Extend(2);
	l_Colors[0].Set(1.0, 1.0, 1.0, 1.0);
	l_Colors[1].Set(0.5, 0.0, 0.75, 0.5);

	if(l_pShape->Type() == CSLTemplate::SI_SHAPE)
	{
		// Add some UV soordinates.
		CSLBaseShape::CSLVector2DArray &l_UVCoords = *( (CSLShape*) l_pShape)->CreateUVCoords();
		l_UVCoords.Reserve(4);
		l_UVCoords[0].Set(1.0, 0.5);
		l_UVCoords[1].Set(0.5, 0.0);
		l_UVCoords[2].Set(0.0, 0.5);
		l_UVCoords[3].Set(0.5, 1.0);
	}
	else
	{
		// Add some UV soordinates.
		CSLUVCoordArray &l_UVCoords = *( (CSLShape_35*) l_pShape)->AddUVCoordArray();
		l_UVCoords.GetUVCoordList()->Resize(4);
		l_UVCoords.SetTextureProjection("Texture_Projection");
		l_UVCoords.GetUVCoordListPtr()[0].Set(1.0, 0.5);
		l_UVCoords.GetUVCoordListPtr()[1].Set(0.5, 0.0);
		l_UVCoords.GetUVCoordListPtr()[2].Set(0.0, 0.5);
		l_UVCoords.GetUVCoordListPtr()[3].Set(0.5, 1.0);
	}

	// Add some normal coordinates.
	CSLBaseShape::CSLVector3DArray &l_Normals = *l_pShape->CreateNormals();
	l_Normals.Resize(5);
	l_Normals[0].Set(0.0, 0.0, 1.0);
	l_Normals[1].Set(0.0, 0.0, 1.0);
	l_Normals[2].Set(0.0, 0.0,-1.0);
	l_Normals[3].Set(0.0, 0.0,-1.0);
	l_Normals[4].Set(0.0, 1.0, 0.0);

	// Delete all information and only color values
	if(l_pShape->Type() == CSLTemplate::SI_SHAPE)
	{
		( (CSLShape*) l_pShape)->DestroyUVCoords();
	}
	l_pShape->DestroyNormals();
	l_pShape->DestroyColors();

	CSLBaseShape::CSLColorArray* l_pNewColors = l_pShape->CreateColors();
	l_pNewColors->Resize(2);
	(*l_pNewColors)[0].Set(0.25, 0.5, 0.25, 1.0);
	(*l_pNewColors)[1].Set(0.5, 0.625, 0.75, 0.875);

	// Add a triangle list and fill it.
	CSLTriangleList* l_Tri = l_pMesh->AddTriangleList();
	FillTriangle(l_Tri);

	// Remove the triangle list.
	l_pMesh->RemoveTriangleList(0);

	// Add a triangle strip list and fill it.
	CSLTriangleStripList* l_TriStrip = l_pMesh->AddTriangleStripList();
	FillTriangleStrip(l_TriStrip);

	// Remove all triangle strip lists.
	l_pMesh->ClearTriangleStripLists();

	// Add a polygon list and fill it.
	CSLPolygonList* l_Poly = l_pMesh->AddPolygonList();
	FillPolygon(l_Poly);

	// Add shape animaiton to the mesh.
	CSLShapeAnimation* l_pShapeAnim = l_pMesh->CreateShapeAnimation(CSLTemplate::SI_LINEAR);
	FillShapeAnimation(l_pShapeAnim);

	// Add a cluster to the mesh.
	CSLCluster *l_pCluster = l_pMeshModel->AddCluster();
	l_pCluster->SetWeightingType(CSLCluster::SI_AVERAGE);
	CSLCluster::CSLIntArray& l_ClusterData = *l_pCluster->GetVertexIndicesList();
	l_ClusterData.Resize(3);
	l_ClusterData[0] = 0;
	l_ClusterData[1] = 1;
	l_ClusterData[2] = 3;

	// add user data to the cluster
	AddUserDataList(l_pCluster);

	// Add an envelope on the mesh.
	CSLEnvelopeList *l_pEnvList = in_pScene->CreateEnvelopeList();
	CSLEnvelope *l_pEnv = l_pEnvList->AddEnvelope();
	l_pEnv->SetEnvelope(l_pMeshModel);
	l_pEnv->SetDeformer(l_pModel);
	CSLEnvelope::CSLVertexWeightArray& l_WeightData = *l_pEnv->GetVertexWeightList();
	l_WeightData.Resize(4);
	l_WeightData[0].m_fVertexIndex = 0;
	l_WeightData[0].m_fWeight = 100.0f;
	l_WeightData[1].m_fVertexIndex = 0;
	l_WeightData[1].m_fWeight = 100.0f;
	l_WeightData[2].m_fVertexIndex = 0;
	l_WeightData[2].m_fWeight = 100.0f;
	l_WeightData[3].m_fVertexIndex = 0;
	l_WeightData[3].m_fWeight = 100.0f;

	// Add a new curve list.
	CSLModel* l_pCurveListModel = in_pScene->Root()->AddCurveList();
	FillModel(l_pCurveListModel, "Curves");
	CSLCurveList* l_pCurveList = (CSLCurveList*) l_pCurveListModel->Primitive();

	// Add two curves to the list.
	FillCurve(l_pCurveList->AddCurve() );
	FillCurve(l_pCurveList->AddCurve() );

	// Add a new surface mesh.
	CSLModel* l_pSurfaceMeshModel = in_pScene->Root()->AddSurfaceMesh();
	FillModel(l_pSurfaceMeshModel, "Surfaces");
	CSLSurfaceMesh* l_pSurfaceMesh = (CSLSurfaceMesh*) l_pSurfaceMeshModel->Primitive();

	// Add two surfaces to the mesh.
	CSLNurbsSurface* l_pSurface = l_pSurfaceMesh->AddSurface();
	FillSurface( l_pSurface );
	AddNurbsData( l_pSurface );
	FillSurface(l_pSurfaceMesh->AddSurface() );

	// Create an instance.
	CSLModel* l_pInstanceModel = in_pScene->Root()->AddInstance();
	FillModel(l_pInstanceModel, "InstanceObject");
	CSLInstance* l_pInstance = (CSLInstance *) l_pInstanceModel->Primitive();
	l_pInstance->SetReference(l_pChild);

	// Add some fog to the scene.
	FillFog(in_pScene->CreateFog() );
}

void DuplicateFCurve(CSLFCurve* in_pSource, CSLFCurve* in_pDest);
void DuplicateCustomPSet(CSLCustomPSet* in_pSource, CSLCustomPSet* in_pDest);

void DuplicateTemplate(CSLTemplate* in_pSource, CSLTemplate* in_pDest)
{
	SI_Int	i;

	in_pDest->SetName(in_pSource->GetName() );

	// Duplicate each FCurve.
	SI_Int l_nCount = in_pSource->GetFCurveCount();

	for(i = 0;i < l_nCount;i ++)
	{
		CSLFCurve *l_pSourceFCurve = in_pSource->FCurves()[i];
		DuplicateFCurve(l_pSourceFCurve, in_pDest->CreateFCurve(l_pSourceFCurve->GetFCurveType(), l_pSourceFCurve->GetInterpolationType() ) );
	}

	// Duplicate each Custom PSet.
	l_nCount = in_pSource->GetCustomPSetCount();

	for(i = 0;i < l_nCount;i ++)
	{
		CSLCustomPSet *l_pSourceCustomPSet = in_pSource->GetCustomPSetList()[i];
		DuplicateCustomPSet(l_pSourceCustomPSet, in_pDest->AddCustomPSet() );
	}
}

void DuplicateFCurve(CSLFCurve* in_pSource, CSLFCurve* in_pDest)
{
	DuplicateTemplate(in_pSource, in_pDest);

	SI_Int l_nKeyCount = 0;
	SI_Int l_nRowSize = in_pSource->GetKeyValueCount() + 1;
	SI_Float *l_pfSourceArray = NULL;
	SI_Float *l_pfDestArray = NULL;

	// Resize the fcurve.
	switch(in_pSource->GetInterpolationType() )
	{
	case CSLTemplate::SI_CONSTANT:
		in_pDest->GetConstantKeyList()->Resize(l_nKeyCount=in_pSource->GetConstantKeyList()->GetUsed());
		l_pfSourceArray = (SI_Float *) in_pSource->GetConstantKeyList()->ArrayPtr();
		l_pfDestArray = (SI_Float *) in_pDest->GetConstantKeyList()->ArrayPtr();
		break;
	case CSLTemplate::SI_LINEAR:
		in_pDest->GetLinearKeyList()->Resize(l_nKeyCount=in_pSource->GetLinearKeyList()->GetUsed());
		l_pfSourceArray = (SI_Float *) in_pSource->GetLinearKeyList()->ArrayPtr();
		l_pfDestArray = (SI_Float *) in_pDest->GetLinearKeyList()->ArrayPtr();
		break;
	case CSLTemplate::SI_HERMITE:
		in_pDest->GetHermiteKeyList()->Resize(l_nKeyCount=in_pSource->GetHermiteKeyList()->GetUsed());
		l_pfSourceArray = (SI_Float *) in_pSource->GetHermiteKeyList()->ArrayPtr();
		l_pfDestArray = (SI_Float *) in_pDest->GetHermiteKeyList()->ArrayPtr();
		break;
	case CSLTemplate::SI_CUBIC:
		in_pDest->GetCubicKeyList()->Resize(l_nKeyCount=in_pSource->GetCubicKeyList()->GetUsed());
		l_pfSourceArray = (SI_Float *) in_pSource->GetCubicKeyList()->ArrayPtr();
		l_pfDestArray = (SI_Float *) in_pDest->GetCubicKeyList()->ArrayPtr();
		break;
	}

	// Copy fcurve data.
	memcpy(l_pfDestArray, l_pfSourceArray, l_nKeyCount * l_nRowSize * sizeof(SI_Float) );
}

void DuplicateVariantParameter(CSLVariantParameter* in_pSource, CSLVariantParameter* in_pDest)
{
	in_pDest->SetName(in_pSource->GetName() );
	in_pDest->SetValue(in_pSource->GetValue() );
}

void DuplicateCustomPSet(CSLCustomPSet* in_pSource, CSLCustomPSet* in_pDest)
{
	DuplicateTemplate(in_pSource, in_pDest);

	in_pDest->SetPropagationType(in_pSource->GetPropagationType() );

	// Duplicate each parameter.
	SI_Int l_nCount = in_pSource->GetParameterCount();

	for(SI_Int i = 0;i < l_nCount;i ++)
	{
		CSLVariantParameter *l_pSourceParam = in_pSource->GetParameterList()[i];
		DuplicateVariantParameter(l_pSourceParam, in_pDest->AddParameter());
	}
}

void DuplicateFileInfo(CSLFileInfo* in_pSource, CSLFileInfo* in_pDest)
{
	DuplicateTemplate(in_pSource, in_pDest);

	in_pDest->SetOriginator(in_pSource->GetOriginator() );
	in_pDest->SetProjectName(in_pSource->GetProjectName() );
	in_pDest->SetUsername(in_pSource->GetUsername() );
}

void DuplicateFog(CSLFog* in_pSource, CSLFog* in_pDest)
{
	CSIBCColorf	l_cColor(in_pSource->GetColor());

	DuplicateTemplate(in_pSource, in_pDest);

	in_pDest->SetFogType(in_pSource->GetFogType() );
	in_pDest->SetFogInterpolationType(in_pSource->GetFogInterpolationType() );
	in_pDest->SetColor( l_cColor );
	in_pDest->SetNear(in_pSource->GetNear() );
	in_pDest->SetFar(in_pSource->GetFar() );
}

void DuplicateAmbience(CSLAmbience* in_pSource, CSLAmbience* in_pDest)
{
	CSIBCColorf l_cColor( in_pSource->GetColor() );
	DuplicateTemplate(in_pSource, in_pDest);

	in_pDest->SetColor( l_cColor );
}

void DuplicateSceneInfo(CSLSceneInfo* in_pSource, CSLSceneInfo* in_pDest)
{
	DuplicateTemplate(in_pSource, in_pDest);

	in_pDest->SetEnd(in_pSource->GetEnd() );
	in_pDest->SetFrameRate(in_pSource->GetFrameRate() );
	in_pDest->SetStart(in_pSource->GetStart() );
	in_pDest->SetTimingType(in_pSource->GetTimingType() );
}

void DuplicateCoordinateSystem(CSLCoordinateSystem* in_pSource, CSLCoordinateSystem* in_pDest)
{
	DuplicateTemplate(in_pSource, in_pDest);

	in_pDest->SetHandRotation(in_pSource->GetHandRotation() );
	in_pDest->SetUAxisOrientation(in_pSource->GetUAxisOrientation() );
	in_pDest->SetVAxisOrientation(in_pSource->GetVAxisOrientation() );
	in_pDest->SetXAxisOrientation(in_pSource->GetXAxisOrientation() );
	in_pDest->SetYAxisOrientation(in_pSource->GetYAxisOrientation() );
	in_pDest->SetZAxisOrientation(in_pSource->GetZAxisOrientation() );
}

void DuplicateAngle(CSLAngle* in_pSource, CSLAngle* in_pDest)
{
	DuplicateTemplate(in_pSource, in_pDest);

	in_pDest->SetAngleType(in_pSource->GetAngleType() );
}

void DuplicateXSIMaterial(CSLXSIMaterial* in_pSource, CSLXSIMaterial* in_pDest)
{
	DuplicateTemplate(in_pSource, in_pDest);

	//todo : duplicate connection points
}

void DuplicateTexture2D(CSLTexture2D* in_pSource, CSLTexture2D* in_pDest)
{
	CSIBCMatrix4x4	l_mProjection(in_pSource->GetProjectionMatrix());

	DuplicateTemplate(in_pSource, in_pDest);

	in_pDest->SetImageFileName(in_pSource->GetImageFileName() );
	in_pDest->SetMappingType(in_pSource->GetMappingType() );
	in_pDest->SetImageWidth(in_pSource->GetImageWidth() );
	in_pDest->SetImageHeight(in_pSource->GetImageHeight() );
	in_pDest->SetCropUMin(in_pSource->GetCropUMin() );
	in_pDest->SetCropUMax(in_pSource->GetCropUMax() );
	in_pDest->SetCropVMin(in_pSource->GetCropVMin() );
	in_pDest->SetCropVMax(in_pSource->GetCropVMax() );
	in_pDest->SetUVSwapFlag(in_pSource->GetUVSwapFlag() );
	in_pDest->SetURepeat(in_pSource->GetURepeat() );
	in_pDest->SetVRepeat(in_pSource->GetVRepeat() );
	in_pDest->SetUAlternate(in_pSource->GetUAlternate() );
	in_pDest->SetVAlternate(in_pSource->GetVAlternate() );
	in_pDest->SetUScale(in_pSource->GetUScale() );
	in_pDest->SetVScale(in_pSource->GetVScale() );
	in_pDest->SetUOffset(in_pSource->GetUOffset() );
	in_pDest->SetProjectionMatrix( l_mProjection );
	in_pDest->SetBlendingType(in_pSource->GetBlendingType() );
	in_pDest->SetBlending(in_pSource->GetBlending() );
	in_pDest->SetAmbient(in_pSource->GetAmbient() );
	in_pDest->SetDiffuse(in_pSource->GetDiffuse() );
	in_pDest->SetSpecular(in_pSource->GetSpecular() );
	in_pDest->SetTransparency(in_pSource->GetTransparency() );
	in_pDest->SetReflectivity(in_pSource->GetReflectivity() );
	in_pDest->SetRoughness(in_pSource->GetRoughness() );
}

void DuplicateMaterial(CSLMaterial* in_pSource, CSLMaterial* in_pDest)
{
	CSIBCColorf		l_cAmbient(in_pSource->GetAmbientColor());
	CSIBCColorf		l_cDiffuse(in_pSource->GetDiffuseColor());
	CSIBCColorf		l_cEmissive(in_pSource->GetEmissiveColor());
	CSIBCColorf		l_cSpecular(in_pSource->GetSpecularColor());

	DuplicateTemplate(in_pSource, in_pDest);

	in_pDest->SetAmbientColor( l_cAmbient );
	in_pDest->SetDiffuseColor( l_cDiffuse );
	in_pDest->SetEmissiveColor( l_cEmissive );
	in_pDest->SetShadingModel(in_pSource->GetShadingModel() );
	in_pDest->SetSpecularColor( l_cSpecular );
	in_pDest->SetSpecularDecay(in_pSource->GetSpecularDecay() );

	// Duplicate texture (if present)
	CSLTexture2D *l_pSourceTexture = in_pSource->Texture2D();
	if(l_pSourceTexture != NULL)
	{
		DuplicateTexture2D(l_pSourceTexture, in_pDest->CreateTexture2D() );
	}
}

void DuplicateMaterialLibrary(CSLMaterialLibrary* in_pSource, CSLMaterialLibrary* in_pDest)
{
	DuplicateTemplate(in_pSource, in_pDest);

	// Duplicate each material.
	SI_Int l_nMatCount = in_pSource->GetMaterialCount();

	for(SI_Int i = 0;i < l_nMatCount;i ++)
	{
		CSLBaseMaterial* l_pSourveMat = in_pSource->GetMaterialList()[i];
		if(l_pSourveMat->Type() == CSLTemplate::SI_MATERIAL)
		{
			DuplicateMaterial( (CSLMaterial *) l_pSourveMat, in_pDest->AddMaterial() );
		}
		else
		{
			DuplicateXSIMaterial( (CSLXSIMaterial *) l_pSourveMat, in_pDest->AddXSIMaterial() );
		}
	}
}

void DuplicateVisibility(CSLVisibility* in_pSource, CSLVisibility* in_pDest)
{
	if( (in_pSource != NULL) && (in_pDest != NULL) )
	{
		DuplicateTemplate(in_pSource, in_pDest);

		in_pDest->SetVisibility(in_pSource->GetVisibility() );
	}
}

void DuplicateGlobalMaterial(CSLGlobalMaterial* in_pSource, CSLGlobalMaterial* in_pDest)
{
	if( (in_pSource != NULL) && (in_pDest != NULL) )
	{
		DuplicateTemplate(in_pSource, in_pDest);

		in_pDest->SetPropagationType(in_pSource->GetPropagationType() );
		in_pDest->SetMaterial(in_pDest->Scene()->GetMaterialLibrary()->FindMaterial(in_pSource->GetMaterial()->GetName() ) );
	}
}

void DuplicateTransform(CSLTransform* in_pSource, CSLTransform* in_pDest)
{
	if( (in_pSource != NULL) && (in_pDest != NULL) )
	{
		CSIBCVector3D	l_vScale(in_pSource->GetScale());
		CSIBCVector3D	l_vRotation(in_pSource->GetEulerRotation());
		CSIBCVector3D	l_vTranslation(in_pSource->GetTranslation());
		
		DuplicateTemplate(in_pSource, in_pDest);

		in_pDest->SetScale( l_vScale );
		in_pDest->SetEulerRotation( l_vRotation );
		in_pDest->SetTranslation( l_vTranslation );
	}
}

void DuplicateShapeAnimation(CSLShapeAnimation* in_pSource, CSLShapeAnimation* in_pDest)
{
	DuplicateTemplate(in_pSource, in_pDest);
}

void DuplicateGeometry(CSLGeometry* in_pSource, CSLGeometry* in_pDest)
{
	DuplicateTemplate(in_pSource, in_pDest);
	
	if(in_pSource->ShapeAnimation() )
	{
		DuplicateShapeAnimation(in_pSource->ShapeAnimation(), in_pDest->CreateShapeAnimation(in_pSource->ShapeAnimation()->Animation()->GetInterpolationType() ) );
	}
}

void DuplicateNurbsCurve(CSLNurbsCurve* in_pSource, CSLNurbsCurve* in_pDest)
{
	DuplicateGeometry(in_pSource, in_pDest);

	in_pDest->SetClosedFlag(in_pSource->GetClosedFlag() );
	in_pDest->SetDegree(in_pSource->GetDegree() );
	in_pDest->SetParametrization(in_pSource->GetParametrization() );

	// Duplicate data.
	in_pDest->GetKnotList()->Resize(in_pSource->GetKnotCount() );
	memcpy(in_pDest->GetKnotListPtr(), in_pSource->GetKnotListPtr(), in_pSource->GetKnotCount() * sizeof(SI_Float) );

	in_pDest->GetControlPointList()->Resize(in_pSource->GetControlPointCount() );
	memcpy(in_pDest->GetControlPointListPtr(), in_pSource->GetControlPointListPtr(), in_pSource->GetControlPointCount() * sizeof(CSIBCVector4D) );
}

void DuplicateNurbsSurface(CSLNurbsSurface* in_pSource, CSLNurbsSurface* in_pDest)
{
	DuplicateGeometry(in_pSource, in_pDest);

	in_pDest->SetUClosedFlag(in_pSource->GetUClosedFlag() );
	in_pDest->SetVClosedFlag(in_pSource->GetVClosedFlag() );
	in_pDest->SetUDegree(in_pSource->GetUDegree() );
	in_pDest->SetVDegree(in_pSource->GetVDegree() );
	in_pDest->SetParametrization(in_pSource->GetParametrization() );
	in_pDest->SetUControlPointCount(in_pSource->GetUControlPointCount() );
	in_pDest->SetVControlPointCount(in_pSource->GetVControlPointCount() );

	// Duplicate data.
	in_pDest->GetUKnotList()->Resize(in_pSource->GetUKnotCount() );
	memcpy(in_pDest->GetUKnotListPtr(), in_pSource->GetUKnotListPtr(), in_pSource->GetUKnotCount() * sizeof(SI_Float) );

	in_pDest->GetVKnotList()->Resize(in_pSource->GetVKnotCount() );
	memcpy(in_pDest->GetVKnotListPtr(), in_pSource->GetVKnotListPtr(), in_pSource->GetVKnotCount() * sizeof(SI_Float) );

	in_pDest->GetControlPointList()->Resize(in_pSource->GetControlPointCount() );
	memcpy(in_pDest->GetControlPointListPtr(), in_pSource->GetControlPointListPtr(), in_pSource->GetControlPointCount() * sizeof(CSIBCVector4D) );
}

void DuplicateCamera(CSLCamera* in_pSource, CSLCamera* in_pDest, CSLTestUnresolvedMap *in_pMap)
{
	CSIBCVector3D	l_vPosition(in_pSource->GetPosition());
	CSIBCVector3D	l_vInterest(in_pSource->GetInterestPosition());

	DuplicateTemplate(in_pSource, in_pDest);

	in_pDest->SetFarPlane(in_pSource->GetFarPlane() );
	in_pDest->SetFieldOfView(in_pSource->GetFieldOfView() );
	in_pDest->SetInterestPosition( l_vInterest );
	in_pDest->SetNearPlane(in_pSource->GetNearPlane() );
	in_pDest->SetPosition( l_vPosition );
	in_pDest->SetRoll(in_pSource->GetRoll() );

	//todo : duplicate constraints.
}

void DuplicateCurveList(CSLCurveList* in_pSource, CSLCurveList* in_pDest)
{
	DuplicateTemplate(in_pSource, in_pDest);

	// Duplicate each curve in the list.
	for(SI_Int i = 0;i < in_pSource->GetCurveCount();i ++)
	{
		DuplicateNurbsCurve(in_pSource->Curves()[i], in_pDest->AddCurve() );
	}
}

void DuplicateInstance(CSLInstance* in_pSource, CSLInstance* in_pDest, CSLTestUnresolvedMap *in_pMap)
{
	DuplicateTemplate(in_pSource, in_pDest);

	in_pMap->insert(CSLTestUnresolvedMap::value_type("SI_Instance", CSLUnresolvedElement(in_pDest, NULL, in_pSource->GetReference()->GetName() ) ) );
}

void DuplicateNull(CSLNull* in_pSource, CSLNull* in_pDest)
{
	DuplicateTemplate(in_pSource, in_pDest);
}

void DuplicateSurfaceMesh(CSLSurfaceMesh* in_pSource, CSLSurfaceMesh* in_pDest)
{
	DuplicateTemplate(in_pSource, in_pDest);

	// Duplicate each surface in the list.
	for(SI_Int i = 0;i < in_pSource->GetSurfaceCount();i ++)
	{
		DuplicateNurbsSurface(in_pSource->Surfaces()[i], in_pDest->AddSurface() );
	}
}

void DuplicatePrimitive(CSLPrimitive* in_pSource, CSLPrimitive* in_pDest, CSLTestUnresolvedMap *in_pMap)
{
	if( (in_pSource != NULL) && (in_pDest != NULL) )
	{
		switch(in_pSource->Type() )
		{
		case CSLTemplate::SI_CAMERA:
			DuplicateCamera( (CSLCamera *) in_pSource, (CSLCamera *) in_pDest, in_pMap);
			break;
		case CSLTemplate::SI_CURVE_LIST:
			DuplicateCurveList( (CSLCurveList *) in_pSource, (CSLCurveList *) in_pDest);
			break;
		case CSLTemplate::SI_INSTANCE:
			DuplicateInstance( (CSLInstance *) in_pSource, (CSLInstance *) in_pDest, in_pMap);
			break;
		case CSLTemplate::SI_NULL_OBJECT:
			DuplicateNull( (CSLNull *) in_pSource, (CSLNull *) in_pDest);
			break;
		case CSLTemplate::SI_SURFACE_MESH:
			DuplicateSurfaceMesh( (CSLSurfaceMesh *) in_pSource, (CSLSurfaceMesh *) in_pDest);
			break;
		}
	}
}

void DuplicateModelRecursively(CSLModel* in_pSource, CSLModel* in_pDest, CSLTestUnresolvedMap *in_pMap)
{
	DuplicateTemplate(in_pSource, in_pDest);

	// Duplicate the visibility.
	DuplicateVisibility(in_pSource->Visibility(), in_pDest->Visibility() );

	// Duplicate the global material.
	DuplicateGlobalMaterial(in_pSource->GlobalMaterial(), in_pDest->GlobalMaterial() );

	// Duplicate local transform.
	DuplicateTransform(in_pSource->Transform(), in_pDest->Transform() );

	// Duplicate basepose (if present).
	if(in_pSource->GetBasePose() != NULL)
	{
		DuplicateTransform(in_pSource->GetBasePose(), in_pDest->CreateBasePose() );
	}

	// Duplicate the primitive.
	DuplicatePrimitive(in_pSource->Primitive(), in_pDest->Primitive(), in_pMap); 

	//todo : constraints, clusters, etc.

	// Duplicate each of our children.
	SI_Int l_nCount = in_pSource->GetChildrenCount();

	for(SI_Int i = 0;i < l_nCount;i ++)
	{
		switch(in_pSource->GetChildrenList()[i]->GetPrimitiveType() )
		{
		case CSLTemplate::SI_NULL_OBJECT:
			DuplicateModelRecursively(in_pSource->GetChildrenList()[i], in_pDest->AddNull(), in_pMap);
			break;
		case CSLTemplate::SI_CAMERA:
			DuplicateModelRecursively(in_pSource->GetChildrenList()[i], in_pDest->AddCamera(), in_pMap);
			break;
		case CSLTemplate::SI_IK_ROOT:
			{
			CSLIKRoot *l_pIKRoot = (CSLIKRoot *) (in_pSource->GetChildrenList()[i]->Primitive() );
			CSLModel *l_pRootModel = in_pDest->AddIKChain(l_pIKRoot->GetJointCount() );
			DuplicateModelRecursively(in_pSource->GetChildrenList()[i], l_pRootModel, in_pMap);

			// Duplicate each bone & the effector.
			CSLIKRoot *l_pDestIKRoot = (CSLIKRoot *) (l_pRootModel->Primitive() );
			DuplicateModelRecursively(l_pIKRoot->GetEffector(), l_pDestIKRoot->GetEffector(), in_pMap);

			for(SI_Int i = 0;i < l_pIKRoot->GetJointCount();i ++)
			{
				DuplicateModelRecursively(l_pIKRoot->GetJointList()[i], l_pDestIKRoot->GetJointList()[i], in_pMap);
			}
			break;
			}
		case CSLTemplate::SI_INFINITE_LIGHT:
			DuplicateModelRecursively(in_pSource->GetChildrenList()[i], in_pDest->AddInfiniteLight(), in_pMap);
			break;
		case CSLTemplate::SI_SPOT_LIGHT:
			DuplicateModelRecursively(in_pSource->GetChildrenList()[i], in_pDest->AddSpotLight(), in_pMap);
			break;
		case CSLTemplate::SI_POINT_LIGHT:
			DuplicateModelRecursively(in_pSource->GetChildrenList()[i], in_pDest->AddPointLight(), in_pMap);
			break;
		case CSLTemplate::SI_DIRECTIONAL_LIGHT:
			DuplicateModelRecursively(in_pSource->GetChildrenList()[i], in_pDest->AddDirectionalLight(), in_pMap);
			break;
		case CSLTemplate::SI_MODEL:
			DuplicateModelRecursively(in_pSource->GetChildrenList()[i], in_pDest->AddModel(), in_pMap);
			break;
		case CSLTemplate::SI_MESH:
			DuplicateModelRecursively(in_pSource->GetChildrenList()[i], in_pDest->AddMesh(), in_pMap);
			break;
		case CSLTemplate::SI_CURVE_LIST:
			DuplicateModelRecursively(in_pSource->GetChildrenList()[i], in_pDest->AddCurveList(), in_pMap);
			break;
		case CSLTemplate::SI_SURFACE_MESH:
			DuplicateModelRecursively(in_pSource->GetChildrenList()[i], in_pDest->AddSurfaceMesh(), in_pMap);
			break;
		case CSLTemplate::SI_INSTANCE:
			DuplicateModelRecursively(in_pSource->GetChildrenList()[i], in_pDest->AddInstance(), in_pMap);
			break;
		}
	}
}

void DuplicateResolve(CSLScene *in_pScene, CSLTestUnresolvedMap *in_pMap)
{
	// Resolve instances.
	CSLTestUnresolvedMapRange l_Range = in_pMap->equal_range("SI_Instance");

	// Loop for each unresolved instance.
	for(CSLTestUnresolvedMapIterator l_iter = l_Range.first;l_iter != l_Range.second;l_iter ++)
	{
		CSLInstance *l_pDestInstance = (CSLInstance *) (*l_iter).second.UnresolvedObject();

		l_pDestInstance->SetReference(in_pScene->FindModel( (*l_iter).second.ReferenceName() ) );
	}
}

void DuplicateImageData( CSLImageData *in_pSrc, CSLImageData *io_pDest )
{
	if (!in_pSrc || !io_pDest) return;
	DuplicateTemplate(in_pSrc, io_pDest);

	SI_Int l_nSize = in_pSrc->GetImageDataCount();
	SI_Int *l_pData = in_pSrc->GetImageDataListPtr();
	CSIBCArray<int> l_DataArray(l_nSize);
	for (SI_Int i = 0; i < l_nSize; i++ )
	{
		l_DataArray.Add( l_pData[i] ); 
	}
	io_pDest->SetImageData( l_DataArray );
	io_pDest->SetCompressionType( in_pSrc->GetCompressionType() );
}

void DuplicateImageFX( CSLImageFX *in_pSrc, CSLImageFX *io_pDest )
{
	if (!in_pSrc || !io_pDest) return;
	DuplicateTemplate(in_pSrc, io_pDest);

	io_pDest->SetBlurAlpha( in_pSrc->GetBlurAlpha() );
	io_pDest->SetBlurAmount( in_pSrc->GetBlurAmount() );
	io_pDest->SetBlurRadius( in_pSrc->GetBlurRadius() );
	io_pDest->SetBrightness( in_pSrc->GetBrightness() );
	io_pDest->SetConvertTo16Bits( in_pSrc->GetConvertTo16Bits() );
	io_pDest->SetFlipHorizontal( in_pSrc->GetFlipHorizontal() );
	io_pDest->SetFlipVertical( in_pSrc->GetFlipVertical() );
	io_pDest->SetGain( in_pSrc->GetGain() );
	io_pDest->SetHue( in_pSrc->GetHue() );
	io_pDest->SetRGBA2Greyscale( in_pSrc->GetRGBA2Greyscale() );
	io_pDest->SetSaturation( in_pSrc->GetSaturation() );
	io_pDest->SetScalingType( in_pSrc->GetScalingType() );
	io_pDest->SetHorizontalScale( in_pSrc->GetHorizontalScale() );
	io_pDest->SetVerticalScale( in_pSrc->GetVerticalScale() );
	io_pDest->SetConvertTo16Bits( in_pSrc->GetConvertTo16Bits() );
}

void DuplicateImage( CSLImage *in_pSrc, CSLImage *io_pDest )
{
	if (!in_pSrc || !io_pDest) return;
	DuplicateTemplate(in_pSrc, io_pDest);

	//
	// duplicate image data
	//
	CSLImageData *l_pData = in_pSrc->ImageData();
	DuplicateImageData( l_pData, io_pDest->CreateImageData() );

	//
	// duplicate image fx
	//
	CSLImageFX *l_pFX = in_pSrc->ImageFX();
	DuplicateImageFX( l_pFX, io_pDest->CreateImageFX() );
}

void DuplicateImageLibrary(CSLScene* in_pSource, CSLScene* io_pDest)
{
	// get the source library
	CSLImageLibrary *l_pSrc = in_pSource->GetImageLibrary();
	if (!l_pSrc) return;

	// create destination library
	CSLImageLibrary *l_pDest = io_pDest->CreateImageLibrary();
	if (!l_pDest) return;

	//
	// Duplicate all images
	//
	CSLImage **l_ppImages = l_pSrc->GetImageList(); 
	SI_Int l_nCount = l_pSrc->GetImageCount();
	for (register SI_Int i =0; i < l_nCount; i++ )
	{
		DuplicateImage(l_ppImages[i], l_pDest->AddImage());	
	}

}

void DuplicateScene(CSLScene* in_pSource, CSLScene* in_pDest)
{
	// Duplicate the file info.
	DuplicateFileInfo(in_pSource->FileInfo(), in_pDest->FileInfo() );

	// Duplicate the scene info.
	DuplicateSceneInfo(in_pSource->SceneInfo(), in_pDest->SceneInfo() );

	// Duplicate the coordinate system.
	DuplicateCoordinateSystem(in_pSource->CoordinateSystem(), in_pDest->CoordinateSystem() );

	// Duplicate the angle.
	DuplicateAngle(in_pSource->Angle(), in_pDest->Angle() );

	// Duplicate the ambience.
	DuplicateAmbience(in_pSource->Ambience(), in_pDest->Ambience() );

	// Duplicate the material library.
	DuplicateMaterialLibrary(in_pSource->GetMaterialLibrary(), in_pDest->GetMaterialLibrary() );

	// Duplicate the fog (if present).
	CSLFog* in_pSourceFog = in_pSource->Fog();
	if(in_pSourceFog != NULL)
	{
		DuplicateFog(in_pSourceFog, in_pDest->CreateFog() );
	}

	// Duplicate each model.
	CSLTestUnresolvedMap l_pMap;
	DuplicateModelRecursively(in_pSource->Root(), in_pDest->Root(), &l_pMap);

	// Resolve elements
	DuplicateResolve(in_pDest, &l_pMap);
}

void DoInteractive(CSLScene* in_pScene)
{
	const int MAX_PARAM_NAME_SIZE = 128;
	char l_szBuffer[MAX_PARAM_NAME_SIZE] = "";

	_SI_PRINTF("Type \"!q\" to quit.\n");

	while(1)
	{
		_SI_PRINTF(":\\>");
		cin.getline(l_szBuffer, MAX_PARAM_NAME_SIZE);

		if(_SI_STRCMP(l_szBuffer, "!q") == 0)
		{
			break;
		}

		SI_Float l_fValue;
		if(in_pScene->GetValue(l_szBuffer, &l_fValue) == SI_SUCCESS)
		{
			_SI_PRINTF("%s = %f\n", l_szBuffer, l_fValue);
		}
		else
		{
			_SI_PRINTF("%s : unable to resolve parameter\n", l_szBuffer);
		}
	}
}

int main(int argc, char **argv)
{
	CSLScene Scene;

	if(argc >= 2)
	{
		if(argc >= 3)
		{
			if(_SI_STRCMP(argv[1], "-fp") == 0)
			{
				if(Scene.Open(argv[2]) == SI_SUCCESS)
				{
					// Read it.
					Scene.Read();
					//
					CSLAnimatableType * Normal = Scene.FindParameter("cone.visibility.viewvis");

					CSLAnimatableType * Model = Scene.FindParameter("Model.cone.visibility.viewvis" );

					CSLAnimatableType * PlayControl = Scene.FindParameter( "PlayControl.In" );

					Scene.Close();
				}
			}

			if(_SI_STRCMP(argv[1], "-sb") == 0)
			{
				if (Scene.Open(argv[2]) == SI_SUCCESS)
				{
					Scene.Read();

					CSLModel	* cube1 = Scene.FindModel( "cube1" );
					CSLModel	* cube  = Scene.FindModel( "cube"  );
					while( cube->GetChildrenCount()>0 )
					{ 
						CSLModel **childrenList = cube->GetChildrenList();
						// Loop through all children
						CSLModel * child = childrenList[0];
						SI_Error err = child->Reparent(cube1);
						CSLModel *newParent = child->Parent();
					}

					Scene.Write("d:\\sb2.xsi");
				}

			}
			if(_SI_STRCMP(argv[1], "-rc") == 0)
			{
				//
				// Test reference counting.
				//
				if (Scene.Open(argv[2]) == SI_SUCCESS)
				{
					Scene.Read();
					CSLMaterialLibrary	* l_pMatLib = Scene.GetMaterialLibrary();
					if (l_pMatLib && (l_pMatLib->GetMaterialCount() > 0))
					{
						CSLTemplate	**list = (CSLTemplate**) malloc(sizeof(CSLTemplate *) * l_pMatLib->GetMaterialCount() );
						SI_Int		*refs = (SI_Int *    ) malloc(sizeof(SI_Int)        * l_pMatLib->GetMaterialCount() );
						int i;
						for(i=0;i<l_pMatLib->GetMaterialCount();i++)
						{
							list[i] = l_pMatLib->GetMaterialList()[i];
							refs[i] = 0;
						}
						Scene.GetRefCount( list, &refs, l_pMatLib->GetMaterialCount(), CSLTemplate::SI_MATERIAL );

						for (i = 0; i<l_pMatLib->GetMaterialCount();i++)
						{
							fprintf( stderr, "Material [%s] is used [%d] times\n", list[i]->Name().GetText(), refs[i] );
						}
						free( list );
						free( refs );
					}
				}
			}
			// create a file from scratch
			if(_SI_STRCMP(argv[1], "-c") == 0)
			{
				// Create a new scene.
				Scene.Create(argv[2], 3, 60);

				// Fill the scene with various data.
				FillScene(&Scene);

				// Save the file.
				Scene.Write();

				// Close the file.
				Scene.Close();
			}
			// read a file the rewrite it
			else if(_SI_STRCMP(argv[1], "-f") == 0)
			{
				// Open the file for reading.
				if(Scene.Open(argv[2]) == SI_SUCCESS)
				{
					// Read it.
					Scene.Read();

					// Write it.
					Scene.Write(argv[3]);

					// Close it.
					Scene.Close();
				}
				else
				{
					_SI_PRINTF("Unable to open file : %s\n", argv[2] );
					return 3;
				}
			}
			// Interactive mode
			else if(_SI_STRCMP(argv[1], "-i") == 0)
			{
				if(Scene.Open(argv[2]) == SI_SUCCESS)
				{
					Scene.Read();

					DoInteractive(&Scene);

					Scene.Close();
				}
				else
				{
					_SI_PRINTF("Unable to open file : %s\n", argv[2] );
					return 3;
				}
			}
			else if(_SI_STRCMP(argv[1], "-d") == 0)
			{
//				SLTest_Time l_dwStart; 
//				SLTest_Time l_dwStop;
			
//				timeGetTime(l_dwStart);

				// Open the file for reading.
				if(Scene.Open(argv[2]) == SI_SUCCESS)
				{
//					timeGetTime(l_dwStop);
//					_SI_PRINTF("Open completed in %d ms\n", timeDifferenceMs(l_dwStart, l_dwStop));

					// Create a new scene.
//					timeGetTime(l_dwStart);
					CSLScene DupeScene;
					DupeScene.Create(argv[3], Scene.Parser()->GetdotXSIFileVersionMajor(), Scene.Parser()->GetdotXSIFileVersionMinor() );

//					 timeGetTime(l_dwStop);
//					_SI_PRINTF("Creation completed in %d ms\n", timeDifferenceMs(l_dwStart, l_dwStop));

					// Read original data.
//					timeGetTime(l_dwStart);
					Scene.Read();

//					timeGetTime(l_dwStop);
//					_SI_PRINTF("Read completed in %d ms\n", timeDifferenceMs(l_dwStart, l_dwStop));

					// Duplicate the data.
//					timeGetTime(l_dwStart);
					DuplicateScene(&Scene, &DupeScene);

//					timeGetTime(l_dwStop);
//					_SI_PRINTF("Duplicate completed in %d ms\n",timeDifferenceMs(l_dwStart, l_dwStop));

					// Write the duplicated scene.
//					timeGetTime(l_dwStart);
					DupeScene.Write();

//					timeGetTime(l_dwStop);
//					_SI_PRINTF("Write completed in %d ms\n", timeDifferenceMs(l_dwStart, l_dwStop));

					// Close each scene.
					DupeScene.Close();
					Scene.Close();
				}
				else
				{
					_SI_PRINTF("Unable to open file : %s\n", argv[2] );
					return 3;
				}
			}
			else
			{
				_SI_PRINTF("Invalid parameter : %s\n", argv[1] );
				return 2;
			}
		}
		else
		{
			// Open the file for reading.
			if(Scene.Open(argv[1]) == SI_SUCCESS)
			{
				Scene.Read();

				// Let's try to explore the scene.
				_SI_PRINTF("------------------------------------------------\n");

				// Show Scene, File, Coordinate System and Angle Info
				ShowGlobalSceneInfo(Scene);
				ShowMaterialLibrary(Scene);
				
				// Output the ambience of the scene.
				CSIBCColorf l_Ambience = Scene.Ambience()->GetColor();
				_SI_PRINTF("Ambience has RGB color: %f ; %f ; %f\n", l_Ambience.m_fR, l_Ambience.m_fG, l_Ambience.m_fB);

				// List all materials
				for(int i = 0;i < Scene.GetMaterialLibrary()->GetMaterialCount();i ++)
				{
					_SI_PRINTF("Material #%d is named %s\n", i, Scene.GetMaterialLibrary()->GetMaterialList()[i]->Name().GetText() );
				}

				DescribeModelRecursively(Scene.Root() );

				ShowImageLibrary(&Scene);
			}
			else
			{
				_SI_PRINTF("Unable to open file : %s\n", argv[2] );
				return 3;
			}
		}

		return 0;
	}
	else
	{
		_SI_PRINTF("Too few arguments.\nUsage: %s [-c|-f-d] dotXSIFile ClonedotXSIFile\n", argv[0] );
		return 1;
	}
}
