//***************************************************************************************
//
// File supervisor: Softimage 3D Games & 3D Bridge team
//
// (c) Copyright 2001-2002 Avid Technology, Inc. . All rights reserved.
//
//***************************************************************************************
/****************************************************************************************
THIS CODE IS PUBLISHED AS A SAMPLE ONLY AND IS PROVIDED "AS IS".
IN NO EVENT SHALL SOFTIMAGE, AVID TECHNOLOGY, INC. AND/OR THEIR RESPECTIVE 
SUPPLIERS BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY
DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
CONNECTION WITH THE USE OR PERFORMANCE OF THIS CODE . 
 
COPYRIGHT NOTICE. Copyright  1999-2002 Avid Technology Inc. . All rights reserved. 

SOFTIMAGE is a registered trademark of Avid Technology Inc. or its subsidiaries 
or divisions. Windows NT is a registered trademark of Microsoft Corp. All other
trademarks contained herein are the property of their respective owners. 
****************************************************************************************/

#include <iostream>

#ifdef CODEWARRIOR
#include <mwutils.h>
#endif 

#include <stdio.h>

#include "XSIParser.h"
#include "dotXSITemplate.h"
#include "dotXSIParam.h"
#include "dotXSIParams.h"
#include "SIBCUtil.h"

#ifdef _PSX2
#include "SIPS2Comm.h"
#endif

#define INDENT {for(int loop2=0; loop2 < spaces; loop2 ++, fprintf(stderr, "  ")) {};}

SI_Int	info = 0;


SI_Error	dumpXSIFileInformation(CXSIParser &in_Parser);
SI_Error	dumpXSITemplatesInformation(CXSIParser &in_Parser, CdotXSITemplates *in_Templates, int spaces);

void main(int argc, char *argv[])
{

#ifdef CODEWARRIOR
	mwInit();    /* To initialize the C++ runtime */
#endif

#ifdef _PSX2

	//
	// Initialize file IO layer
	//
	
	// Use Soft's file io lib (SIPS2Comm)
	// PS2InitFileIO_Softimage();
	
	// Use sce Native file io
	 	PS2InitFileIO_Native();
	
#endif

    CXSIParser		*parser = new CXSIParser;
    int				mode = FORMAT_TEXT;
    SI_Error		result = SI_SUCCESS;

    using namespace std;
    
    // Argument checking
	
    if(argc < 2)
    {
        fprintf(stderr,"Wrong number of arguments : %s <input file> [-s|-i]\n\n-s : short information\n-i : file information only\n", argv[0]);
        exit(0);
    }

	if((argc > 2) && (strcmp(argv[2], "-s") == 0))
	{
		info = 1;
	}

	if((argc > 2) && (strcmp(argv[2], "-i") == 0))
	{
		info = 2;
	}

    // let's open the file using CXSIParser::Open

	
    parser->SetOpenMode(OPEN_READ);

    CSIBCString l_lFilename;
    l_lFilename = _SI_TEXT(argv[1]);
    
    _SI_CALL(parser->Open(l_lFilename), "CXSIParser::Open");
    if(result == SI_SUCCESS)
    {
        _SI_CALL(parser->Read(), "CXSIParser::Read");
        if(result == SI_SUCCESS)
        {
            _SI_CALL(parser->Close(), "CXSIParser::Close");
			
			// let's dump the .XSI file information
			_SI_CALL(dumpXSIFileInformation(*parser), "dumpXSIFileInformation");
        }
        else
        {
            fprintf(stderr, "Error reading the file\n");
            // let's attempt to close the file
            parser->Close();
        }
    }
    else
    {
        fprintf(stderr, "%s could not be open for reading \n", argv[1]);
    }
	
	delete parser;

#ifdef CODEWARRIOR
	mwExit();    /* Clean up, destroy constructed global objects */
#endif	
}


SI_Error dumpXSIFileInformation(CXSIParser &in_Parser)
{
	SI_Error	result = SI_SUCCESS;

	// first we dump the file information (header)

	fprintf(stderr, "=============================================================\n");
	fprintf(stderr, "%s header information \n", in_Parser.FileName().GetText());
	fprintf(stderr, "File version : %d.%d\n", in_Parser.GetdotXSIFileVersionMajor(), in_Parser.GetdotXSIFileVersionMinor());

	if(in_Parser.GetdotXSIFormat() == FORMAT_TEXT)
	{
		fprintf(stderr, "Format       : TEXT\n");
	}
	else if(in_Parser.GetdotXSIFormat() == FORMAT_BINARY)
	{
		fprintf(stderr, "Format       : BINARY\n");
	}

	if(info < 2)
	{
		fprintf(stderr, "=============================================================\n");
		fprintf(stderr, "dumping templates and parameters \n");
		fprintf(stderr, "=============================================================\n");

		_SI_CALL(dumpXSITemplatesInformation(in_Parser, in_Parser.dotXSITemplate(), 0), "dumpXSITemplatesInformation");
	}

	return result;
}


SI_Error dumpXSITemplatesInformation(CXSIParser &in_Parser, CdotXSITemplates *in_Templates, int spaces)
{
	SI_Error			result = SI_SUCCESS;
	SI_Int				loop, paramLoop, arrayLoop;
	CdotXSITemplate		*pTemplate;
	CdotXSIParam		*pParam;
	SI_TinyVariant		pValue;
	CSIBCString			tmpString;

	for(loop = 0; loop < in_Templates->GetCount(); loop++)
	{
		in_Templates->Item(loop, &pTemplate);

		if(pTemplate != NULL)
		{
			INDENT;			
			fprintf(stderr, "--------------------------------------------------------\n");
			INDENT;
			fprintf(stderr, "Template name   : %s \n", pTemplate->Name().GetText());
			INDENT;
			fprintf(stderr, "Instance name   : %s \n\n", pTemplate->InstanceName().GetText());


			if(info == 0)
			{
				// now we dump the parameters information

				for(paramLoop = 0; paramLoop < pTemplate->Params().GetCount(); paramLoop ++)
				{
					pTemplate->Params().Item(paramLoop, &pParam);
					INDENT;
					fprintf(stderr, "(");
					pParam->GetValue(&pValue);

					switch(pValue.variantType)
					{
						case SI_VT_BOOL		: fprintf(stderr,"BOOL"); tmpString.SetText(pValue.boolVal); break;
						case SI_VT_BYTE		: fprintf(stderr,"BYTE"); tmpString.SetText(pValue.bVal); break;
						case SI_VT_UBYTE	: fprintf(stderr,"UBYTE"); tmpString.SetText(pValue.ubVal); break;
						case SI_VT_SHORT	: fprintf(stderr,"SHORT"); tmpString.SetText(pValue.sVal); break;
						case SI_VT_USHORT	: fprintf(stderr,"USHORT"); tmpString.SetText(pValue.usVal); break;
						case SI_VT_INT		: fprintf(stderr,"INT"); tmpString.SetText(pValue.nVal); break;
						case SI_VT_UINT		: fprintf(stderr,"UINT"); tmpString.SetText((SI_Int) pValue.unVal); break;
						case SI_VT_FLOAT	: fprintf(stderr,"FLOAT"); tmpString.SetText((SI_Float)pValue.fVal); break;
						case SI_VT_DOUBLE	: fprintf(stderr,"DOUBLE"); tmpString.SetText((SI_Float) pValue.dVal); break;
						case SI_VT_LONG		: fprintf(stderr,"LONG"); tmpString.SetText((SI_Int)pValue.lVal); break;
						case SI_VT_ULONG	: fprintf(stderr,"ULONG"); tmpString.SetText((SI_Int) pValue.ulVal); break;

						case SI_VT_PBOOL	: fprintf(stderr,"PBOOL"); break;
						case SI_VT_PBYTE	: fprintf(stderr,"PBYTE"); break;
						case SI_VT_PUBYTE	: fprintf(stderr,"PUBYTE"); break;
						case SI_VT_PSHORT	: fprintf(stderr,"PSHORT"); break;
						case SI_VT_PUSHORT	: fprintf(stderr,"PUSHORT"); break;
						case SI_VT_PINT		: fprintf(stderr,"PINT"); break;
						case SI_VT_PUINT	: fprintf(stderr,"PUINT"); break;
						case SI_VT_PFLOAT	: fprintf(stderr,"PFLOAT"); break;
						case SI_VT_PDOUBLE	: fprintf(stderr,"PDOUBLE"); break;
						case SI_VT_PLONG	: fprintf(stderr,"PLONG"); break;
						case SI_VT_PULONG	: fprintf(stderr,"PULONG"); break;
						case SI_VT_PCHAR	: fprintf(stderr,"PCHAR"); tmpString.SetText(pValue.p_cVal); break;
						case SI_VT_PPCHAR	: fprintf(stderr,"PPCHAR"); break;
						case SI_VT_PVOID	: fprintf(stderr,"PVOID"); break;	
					}
					fprintf(stderr,")");

					fprintf(stderr, " %s	: ", pParam->Name().GetText());

					switch(pValue.variantType)
					{
						case SI_VT_BOOL		: 
						case SI_VT_BYTE		: 
						case SI_VT_UBYTE	: 
						case SI_VT_SHORT	: 
						case SI_VT_USHORT	: 
						case SI_VT_INT		: 
						case SI_VT_UINT		: 
						case SI_VT_FLOAT	: 
						case SI_VT_DOUBLE	: 
						case SI_VT_LONG		: 
						case SI_VT_ULONG	: 
						case SI_VT_PCHAR	: 
							fprintf(stderr, "%s\n", tmpString.GetText()); break;

						case SI_VT_PBOOL	: 
						case SI_VT_PBYTE	: 
						case SI_VT_PUBYTE	: 
						case SI_VT_PSHORT	: 
						case SI_VT_PUSHORT	: 
						case SI_VT_PINT		: 
						case SI_VT_PUINT	: 
						case SI_VT_PFLOAT	: 
						case SI_VT_PDOUBLE	: 
						case SI_VT_PLONG	: 
						case SI_VT_PULONG	: 
						case SI_VT_PPCHAR	: 
							fprintf(stderr, "\n");
							for(arrayLoop = 0; arrayLoop < pValue.numElems; arrayLoop ++)
							{
								INDENT;
								switch(pValue.variantType)
								{
									case SI_VT_PBOOL	: tmpString.SetText(pValue.p_boolVal[arrayLoop]); break;
									case SI_VT_PBYTE	: tmpString.SetText(pValue.p_bVal[arrayLoop]); break;
									case SI_VT_PUBYTE	: tmpString.SetText(pValue.p_ubVal[arrayLoop]); break;
									case SI_VT_PSHORT	: tmpString.SetText(pValue.p_sVal[arrayLoop]); break;
									case SI_VT_PUSHORT	: tmpString.SetText(pValue.p_usVal[arrayLoop]); break;
									case SI_VT_PINT		: tmpString.SetText(pValue.p_nVal[arrayLoop]); break;
									case SI_VT_PUINT	: tmpString.SetText((SI_Int) pValue.p_unVal[arrayLoop]); break;
									case SI_VT_PFLOAT	: tmpString.SetText(pValue.p_fVal[arrayLoop]); break;
									case SI_VT_PDOUBLE	: tmpString.SetText((SI_Float) pValue.p_dVal[arrayLoop]); break;
									case SI_VT_PLONG	: tmpString.SetText((SI_Int)pValue.p_lVal[arrayLoop]); break;
									case SI_VT_PULONG	: tmpString.SetText((SI_Int) pValue.p_ulVal[arrayLoop]); break;
									case SI_VT_PPCHAR	: tmpString.SetText(pValue.pp_cVal[arrayLoop]); break;
								}

								fprintf(stderr, "%s\t", tmpString.GetText());
								if((arrayLoop % 3) == 2)
									fprintf(stderr, "\n");
							}
							fprintf(stderr, "\n");
							break;
						case SI_VT_PVOID	: INDENT; fprintf(stderr, "VOID values \n"); break;
							break;

					}
				}
			}

			fprintf(stderr, "\n");

		}
		else
		{
			fprintf(stderr, "Problematic template \n");
		}

		_SI_CALL(dumpXSITemplatesInformation(in_Parser, &pTemplate->Children(), spaces + 1), "dumpXSITemplatesInformation");
	}

	return result;
}
