/* $Header: /Src/RhinoIO/rhinoio_agcurve.h 17    11/10/99 2:56p Dalelear $ */
/* $NoKeywords: $ */
//
// Copyright (c) 1993-1998 Robert McNeel & Associates. All rights reserved.
// Rhinoceros is a registered trademark of Robert McNeel & Assoicates.
//
// THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
// ALL IMPLIED WARRANTIES OF FITNESS FOR ANY PARTICULAR PURPOSE AND OF
// MERCHANTABILITY ARE HEREBY DISCLAIMED.
//				
////////////////////////////////////////////////////////////////
//
//   defines CRhinoAgCurve
//
////////////////////////////////////////////////////////////////


#if !defined(RHINO_AGCURVE_INC_)
#define RHINO_AGCURVE_INC_

///////////////////////////////////////////////////////////////////////////////
// Class  CRhinoAgCurve
//
//	This object represents an AG_CURVE structure and is basically a list
//  of contiguous NURBS curves. 
///////////////////////////////////////////////////////////////////////////////

class CRhinoNurbsCurve;

class CRhinoNurbsCurveArray : public CRhinoArray<CRhinoNurbsCurve*>
{
public:
  CRhinoNurbsCurveArray( size_t = 0 );
  ~CRhinoNurbsCurveArray();
};

inline 
CRhinoNurbsCurveArray::CRhinoNurbsCurveArray( size_t initial_capacity )
                    : CRhinoArray<CRhinoNurbsCurve*>(initial_capacity,TRUE)
{}

inline 
CRhinoNurbsCurveArray::~CRhinoNurbsCurveArray()
{
  int i = m_size;
  while ( i-- > 0 ) {
    if ( m_p[i] ) {
      delete m_p[i];
      m_p[i] = NULL;
    }
  }
}

class CRhinoAgCurve: public CRhinoObject
{
	RHINO_CHUNK_DECLARE( CRhinoAgCurve )

public:

  // Construction
	CRhinoAgCurve();
  ~CRhinoAgCurve();

  // CRhinoObject overrides
	CRhinoFile::eStatus Read(   	// read chunk data from file 
    CRhinoFile&
    );
	void Write(                   // Writes NURBS curve to file
                                // Throws a CRhinoException if IsValid()
                                // returns FALSE or fwrite() fails.
    CRhinoFile&
    ) const;
  BOOL IsValid(                 // Call after class is fully initialized to
                                // perform a simple and quick validity check.
                                // Will detect illegal knot vectors and
                                // incorrect settings of integer m_* members.
                                // Does NOT check control vertices for
                                // grossly degenerate conditions.
    void
    ) const;
  BOOL GetBoundingBox(
          double[3], // min
          double[3], // max
          BOOL = FALSE // set to TRUE if box should be grown
          ) const;

  // Interface
  int Dimension(    // dimension of curve (2 or 3)
        void
        ) const;

  BOOL IsClosed(  // TRUE if curve is closed
      void
      ) const;

  BOOL IsPeriodic(  // TRUE if curve is closed and periodic
      void
      ) const;
  
  int NurbsCurveCount(  // number of NURBS curve segments in this curve
        void 
        ) const;

  CRhinoNurbsCurve* operator[](int) const;

  BOOL EndPoint( int,    // 0 = start, 1 = end
                 double* // array of dimensinon doubles
                 ) const;
  BOOL Domain(
        double*,  // if not NULL, returns domain start
        double*   // if not NULL, returns domain end
        ) const;

  BOOL Reverse();       // reverse parameterizatrion
                        // Domain changes from [a,b] to [-b,-a]

  BOOL SwapCoordinates(
        int, int        // indices of coords to swap
        );

// Implementation
  void AddNurbsCurveCapacity(int);
  BOOL AddNurbsCurve(CRhinoNurbsCurve*); // used when creating curves

private:
  // used to create CRhinoAgCurve classes
  int m_dim;
  double m_bounding_box[2][3];
  void Initialize( void );
  int m_is_closed; // 0 = open, 1 = closed
  CRhinoNurbsCurveArray m_nurbs_curve;
  CRhinoAgCurve(const CRhinoAgCurve&);            // no implementation
	CRhinoAgCurve& operator=(const CRhinoAgCurve&); // no implementation
};


#endif
