/* $Header: /Src/RhinoIO/rhinoio_agshell.cpp 13    11/10/99 2:56p Dalelear $ */
/* $NoKeywords: $ */
//
// Copyright (c) 1993-1998 Robert McNeel & Associates. All rights reserved.
// Rhinoceros is a registered trademark of Robert McNeel & Assoicates.
//
// THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
// ALL IMPLIED WARRANTIES OF FITNESS FOR ANY PARTICULAR PURPOSE AND OF
// MERCHANTABILITY ARE HEREBY DISCLAIMED.
//				
////////////////////////////////////////////////////////////////
 
#include "rhinoio_brep.h"
#include "rhinoio_agface.h"
#include "rhinoio_agshell.h"

RHINO_CHUNK_IMPLEMENT( CRhinoAgShell, TCODE_AG_SHELL )

CRhinoFile::eStatus
CRhinoAgShell::Read( CRhinoFile& file)
{
  CRhinoWorkSpace ws;

  const int ti0 = m_T.Size(); // needed to for tedge gluing

  // read attributes
  CRhinoFile::eStatus status = ReadAttributes( file, TCODE_AG_SHELLSTUFF );
  if ( status != CRhinoFile::good )
    return status;

  unsigned int data_typecode = 0;
  int data_length = 0;

  do {
    status = file.BeginReadChunk( data_typecode, data_length );
    if ( status != CRhinoFile::good )
      return status;
    status = CRhinoFile::crc_error;
    if ( data_typecode != TCODE_AG_SHELLSTUFF || data_length <= 0 )
      break;

    int outer = 0;
    int facecnt = 0;
    int twincnt = 0;

    /* read AG_SHELL flags */
    file.ReadInteger(1,&outer);  
    file.ReadInteger(1,&facecnt);  

    // read AG_SHELL bounding box
    file.ReadDouble( 3, m_bounding_box[0] );
    file.ReadDouble( 3, m_bounding_box[1] );

    /* B-rep edge gluing info */
    file.ReadInteger(1,&twincnt);
    short* glue = (twincnt > 0 ) ? (short*)ws.GetMemory(twincnt*sizeof(*glue)) : NULL;
    if (twincnt > 0)
      file.ReadShort(twincnt,glue);

    int fi;
    status = CRhinoFile::good;
    for ( fi = 0; fi < facecnt && status == CRhinoFile::good; fi++ ) {
      unsigned int face_typecode;
      int face_length;
      status = file.BeginReadChunk( face_typecode, face_length );
      if ( status != CRhinoFile::good )
        break;
      if ( face_typecode != TCODE_AG_FACE || face_length <= 0 )
        status = CRhinoFile::crc_error;
      else
        status = ReadAgFace( file, TRUE );
      CRhinoFile::eStatus end_face_status = file.EndChunk();
      if ( status == CRhinoFile::good )
        status = end_face_status;
    }
    if ( status != CRhinoFile::good )
      break;

    if ( twincnt > 0 ) {
      // twincnt = number of shared (inter-face) edges
      // glue[] = order 2 permutation of {0,....,twincnt-1}

      // set share_i[] = m_T[] indices of shared trims
      int si, ti;
      const int ti1 = m_T.Size();
      int* share_i = (int*)ws.GetMemory(twincnt*sizeof(*share_i));
      for ( ti = ti0, si = 0; ti < ti1 && si < twincnt; ti++ ) {
        if (m_T[ti].m_type != CRhinoBrepTrim::mated )
          continue;
        share_i[si++] = ti;
      }

      if ( si == twincnt ) {
        // glue seams
        for ( si = 0; si < twincnt; si++ ) {
          const int i0 = share_i[si];
          const int i1 = share_i[glue[si]];
          // m_T[i0] and m_t[i1] use the same edge;
          const int ei0 = m_T[i0].m_ei;
          const int ei1 = m_T[i1].m_ei;
          if ( ei0 == -1 && ei1 >= 0 ) {
            m_T[i0].m_ei = ei1;
            m_E[ei1].m_ti.Append(i0);
          }
          else if ( ei1 == -1 && ei0 >= 0 ) {
            m_T[i1].m_ei = ei0;
            m_E[ei0].m_ti.Append(i1);
          }
        }
      }
    }

  } while ( FALSE );

  CRhinoFile::eStatus end_status = file.EndChunk();
  if ( status == CRhinoFile::good )
    status = end_status;

  if ( status == CRhinoFile::good ) {
    SetVertices();
    SetTolsFromFileValues();
  }

  return status;
}

void
CRhinoAgShell::Write(CRhinoFile& file) const
{
  CRhinoBrep::Write( file );
}

BOOL
CRhinoAgShell::IsValid(void) const
{
  if ( m_S.Size() < 1 )
    return FALSE;
  if ( m_F.Size() != m_S.Size() )
    return FALSE;
  return CRhinoBrep::IsValid();
}
