/* $Header: /Src/RhinoIO/rhinoio_agspline.h 5     10/28/98 12:08p Dalelear $ */
/* $NoKeywords: $ */
//
// Copyright (c) 1993-1998 Robert McNeel & Associates. All rights reserved.
// Rhinoceros is a registered trademark of Robert McNeel & Assoicates.
//
// THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
// ALL IMPLIED WARRANTIES OF FITNESS FOR ANY PARTICULAR PURPOSE AND OF
// MERCHANTABILITY ARE HEREBY DISCLAIMED.
//				
////////////////////////////////////////////////////////////////
//
//   defines CRhinoAgSpline
//
////////////////////////////////////////////////////////////////


#if !defined(RHINO_AGSPLINE_INC_)
#define RHINO_AGSPLINE_INC_

///////////////////////////////////////////////////////////////////////////////
// Class  CRhinoAgSpline
//
//	This object is used to read/write AG_SPLINEs in Rhino .3dm files
///////////////////////////////////////////////////////////////////////////////

class CRhinoAgSpline: public CRhinoNurbsCurve
{
	RHINO_CHUNK_DECLARE( CRhinoAgSpline )
public:

  // Construction
	CRhinoAgSpline();
	CRhinoAgSpline(     // Constructs a NURBS curve that can be filled in
                        // with calls to SetCV() and SetKnot().
                        // Throws a CRhinoException if input is illegal
                        // or not enough memory.
    int,  // dimension ( >= 1 and usually = 3 )
    BOOL, // TRUE if curve is rational
    int,  // degree for first parameter ( >= 1 )
    int   // cv_count = number of CVs
  );
  ~CRhinoAgSpline();

  // CRhinoObject overrides
	CRhinoFile::eStatus Read(   	// read chunk data from file 
    CRhinoFile&
    );
	void Write(                   // Writes NURBS curve to file
                                // Throws a CRhinoException if IsValid()
                                // returns FALSE or fwrite() fails.
    CRhinoFile&
    ) const;
  BOOL IsValid(                 // Call after class is fully initialized to
                                // perform a simple and quick validity check.
                                // Will detect illegal knot vectors and
                                // incorrect settings of integer m_* members.
                                // Does NOT check control vertices for
                                // grossly degenerate conditions.
    void
    ) const;

private:
  void Initialize(void);
  int m_is_closed;  // 0 = open, 1 = closed, 2 = periodic
  int m_form;       
  double m_bounding_box[2][3]; // axis aligned bounding box

  CRhinoAgSpline(const CRhinoAgSpline&); // no implementation
	CRhinoAgSpline& operator=(const CRhinoAgSpline&); // no implementation
};

inline CRhinoAgSpline::~CRhinoAgSpline()
  {}

#endif
