/* $Header: /Src/RhinoIO/rhinoio_agsurface.h 5     11/10/99 2:56p Dalelear $ */
/* $NoKeywords: $ */
//
// Copyright (c) 1993-1998 Robert McNeel & Associates. All rights reserved.
// Rhinoceros is a registered trademark of Robert McNeel & Assoicates.
//
// THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
// ALL IMPLIED WARRANTIES OF FITNESS FOR ANY PARTICULAR PURPOSE AND OF
// MERCHANTABILITY ARE HEREBY DISCLAIMED.
//				
////////////////////////////////////////////////////////////////
//
//   defines CRhinoAgSurface
//
////////////////////////////////////////////////////////////////


#if !defined(RHINO_AGSURFACE_INC_)
#define RHINO_AGSURFACE_INC_

///////////////////////////////////////////////////////////////////////////////
// Class  CRhinoAgSurface
//
//	This object is used to read/write AG_SURFACEs in Rhino .3dm files
///////////////////////////////////////////////////////////////////////////////

class CRhinoAgSurface: public CRhinoNurbsSurface
{
	RHINO_CHUNK_DECLARE( CRhinoAgSurface )
public:

  // Construction
	CRhinoAgSurface();
	CRhinoAgSurface(      // Constructs a NURBS surface that can be filled in
                        // with calls to SetCV() and SetKnot().
                        // Throws a CRhinoException if input is illegal
                        // or not enough memory.
    int,  // dimension ( >= 1 and usually = 3 )
    BOOL, // TRUE if surface is rational
    int,  // degree0 = degree for first parameter ( >= 1 )
    int,  // degree1 = degree for second parameter ( >= 1 )
    int,  // cv_count0 = number of CVs for first parameter ( >= degree0+1 )
    int   // cv_count1 = number of CVs for second parameter ( >= degree1+1 )
  );
  ~CRhinoAgSurface();

  // CRhinoObject overrides
	CRhinoFile::eStatus Read(   	// read chunk data from file 
    CRhinoFile&
    );
	void Write(                   // Writes NURBS curve to file
                                // Throws a CRhinoException if IsValid()
                                // returns FALSE or fwrite() fails.
    CRhinoFile&
    ) const;
  BOOL IsValid(                 // Call after class is fully initialized to
                                // perform a simple and quick validity check.
                                // Will detect illegal knot vectors and
                                // incorrect settings of integer m_* members.
                                // Does NOT check control vertices for
                                // grossly degenerate conditions.
    void
    ) const;

private:
  void Initialize(void);
  int m_is_closed[2];   // 0 = open, 1 = closed, 2 = periodic
  int m_is_singular[2]; // 0 = no,  1 = left/bottom, 2 = right/top, 3 = both
  int m_form;           // special case flag
  double m_bounding_box[2][3]; // axis aligned bounding box

  CRhinoAgSurface(const CRhinoAgSurface&); // no implementation
	CRhinoAgSurface& operator=(const CRhinoAgSurface&); // no implementation
};

inline CRhinoAgSurface::~CRhinoAgSurface()
  {}

#endif
