/* $Header: /Src/RhinoIO/rhinoio_brep.cpp 34    11/10/99 4:06p Dalelear $ */
/* $NoKeywords: $ */
//
// Copyright (c) 1993-1998 Robert McNeel & Associates. All rights reserved.
// Rhinoceros is a registered trademark of Robert McNeel & Assoicates.
//
// THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
// ALL IMPLIED WARRANTIES OF FITNESS FOR ANY PARTICULAR PURPOSE AND OF
// MERCHANTABILITY ARE HEREBY DISCLAIMED.
//				
////////////////////////////////////////////////////////////////

#include "rhinoio_brep.h"

RHINO_CHUNK_IMPLEMENT(CRhinoBrep, TCODE_RHINO_OBJECT_BREP)

////////////////////////////////////////////////////////////////
//   Class CRhinoBrepEntity
////////////////////////////////////////////////////////////////

CRhinoBrepEntity& CRhinoBrepEntity::operator=(const CRhinoBrepEntity& src)
{
  if ( &src != this ) {
    m_index = src.m_index;
  }
  return *this;
}

////////////////////////////////////////////////////////////////
//   Class CRhinoBrepVertex
////////////////////////////////////////////////////////////////

CRhinoBrepVertex::CRhinoBrepVertex(int i /*= -1*/ ) 
                        : CRhinoBrepEntity(i), 
                        x(0.0), 
                        y(0.0), 
                        z(0.0),
                        m_tolerance(-1.0)
{}

CRhinoBrepVertex& CRhinoBrepVertex::operator=(const CRhinoBrepVertex& src)
{
  if ( &src != this ) {
    m_index = src.m_index;
    x       = src.x;
    y       = src.y;
    z       = src.z;
    m_ei    = src.m_ei;
    m_tolerance = src.m_tolerance;
  }
  return *this;
}

void CRhinoBrepVertex::Write( CRhinoFile& file ) const
{
  file.WriteInteger( m_index );
  file.WriteDouble( x );
  file.WriteDouble( y );
  file.WriteDouble( z );
  file.WriteIntArray( m_ei );
  file.WriteDouble( m_tolerance );
}

void CRhinoBrepVertex::Read( CRhinoFile& file )
{
  file.ReadInteger( &m_index );
  file.ReadDouble( &x );
  file.ReadDouble( &y );
  file.ReadDouble( &z );
  file.ReadIntArray( m_ei );
  file.ReadDouble( &m_tolerance );
}

////////////////////////////////////////////////////////////////
//   Class CRhinoBrepEdge
////////////////////////////////////////////////////////////////

CRhinoBrepEdge::CRhinoBrepEdge(int i /* = -1 */ )
                       : CRhinoBrepEntity(i), 
                         m_c3i(-1),
                         m_tolerance(-1.0)
{
  m_t[0] = m_t[1] = 0.0; 
  m_vi[0] = m_vi[1] = -1;
}

CRhinoBrepEdge& CRhinoBrepEdge::operator=(const CRhinoBrepEdge& src)
{
  if ( &src != this ) {
    m_index = src.m_index;
    m_c3i   = src.m_c3i;
    m_t[0]  = src.m_t[0];
    m_t[1]  = src.m_t[1];
    m_vi[0] = src.m_vi[0];
    m_vi[1] = src.m_vi[1];
    m_ti = src.m_ti;
    m_tolerance = src.m_tolerance;
  }
  return *this;
}

void CRhinoBrepEdge::Write( CRhinoFile& file ) const
{
  file.WriteInteger( m_index );
  file.WriteInteger( m_c3i );
  file.WriteDouble( 2, m_t );
  file.WriteInteger( 2, m_vi );
  file.WriteIntArray( m_ti );
  file.WriteDouble( m_tolerance );
}

void CRhinoBrepEdge::Read( CRhinoFile& file )
{
  file.ReadInteger( &m_index );
  file.ReadInteger( &m_c3i );
  file.ReadDouble( 2, m_t );
  file.ReadInteger( 2, m_vi );
  file.ReadIntArray( m_ti );
  file.ReadDouble( &m_tolerance );
}

////////////////////////////////////////////////////////////////
//   Class CRhinoBrepTrim
////////////////////////////////////////////////////////////////

CRhinoBrepTrim::CRhinoBrepTrim(int i /*= -1*/) 
              : CRhinoBrepEntity(i), 
                m_c2i(-1), 
                m_ei(-1), 
                m_bRev3d(FALSE), 
                m_type(CRhinoBrepTrim::unknown), 
                m_iso(CRhinoBrepTrim::not_iso),
                m_li(-1),
                m_2d_tol(-1.0), 
                m_3d_tol(-1.0)
{
  m_t[0] = m_t[1] = 0.0; 
  m_vi[0] = m_vi[1] = -1; 
  m_tolerance[0] = m_tolerance[1] = -1.0;
  m_P[0][0] = m_P[0][1] = m_P[0][2] = 0.0;
  m_P[1][0] = m_P[1][1] = m_P[1][2] = 0.0;
}

CRhinoBrepTrim& CRhinoBrepTrim::operator=(const CRhinoBrepTrim& src)
{
  if ( &src != this ) {
    m_index  = src.m_index;
    m_c2i    = src.m_c2i;
    m_t[0]   = src.m_t[0];
    m_t[1]   = src.m_t[1];
    m_ei     = src.m_ei;
    m_vi[0]  = src.m_vi[0];
    m_vi[1]  = src.m_vi[1];
    m_bRev3d = src.m_bRev3d;
    m_type   = src.m_type;
    m_iso    = src.m_iso;
    m_2d_tol = src.m_2d_tol;
    m_3d_tol = src.m_3d_tol;
    m_li     = src.m_li;
    m_tolerance[0] = src.m_tolerance[0];
    m_tolerance[1] = src.m_tolerance[1];
  }
  return *this;
}

void CRhinoBrepTrim::Write( CRhinoFile& file ) const
{
  file.WriteInteger( m_index );
  file.WriteInteger( m_c2i );
  file.WriteDouble( 2, m_t );
  file.WriteInteger( m_ei );
  file.WriteInteger( 2, m_vi );
  file.WriteInteger( m_bRev3d );
  file.WriteInteger( m_type );
  file.WriteInteger( m_iso );
  file.WriteInteger( m_li );
  file.WriteDouble( 2, m_tolerance );
  file.WriteDouble( 3, m_P[0] );
  file.WriteDouble( 3, m_P[1] );
  file.WriteDouble( m_2d_tol );
  file.WriteDouble( m_3d_tol );
}

void CRhinoBrepTrim::Read( CRhinoFile& file )
{
  int i;
  file.ReadInteger( &m_index );
  file.ReadInteger( &m_c2i );
  file.ReadDouble( 2, m_t );
  file.ReadInteger( &m_ei );
  file.ReadInteger( 2, m_vi );
  file.ReadInteger( &m_bRev3d );
  file.ReadInteger( &i );
  switch (i) {
  case unknown:
    m_type = unknown;
    break;
  case boundary:
    m_type = boundary;
    break;
  case mated:
    m_type = mated;
    break;
  case seam:
    m_type = seam;
    break;
  case singular:
    m_type = singular;
    break;
  }
  file.ReadInteger( &i );
  switch(i) {
  case not_iso:
    m_iso = not_iso;
    break;
  case u_iso:
    m_iso = u_iso;
    break;
  case v_iso:
    m_iso = v_iso;
    break;
  case W_iso:
    m_iso = W_iso;
    break;
  case S_iso:
    m_iso = S_iso;
    break;
  case E_iso:
    m_iso = E_iso;
    break;
  case N_iso:
    m_iso = N_iso;
    break;
  }
  file.ReadInteger( &m_li );
  file.ReadDouble( 2, m_tolerance );
  file.ReadDouble( 3, m_P[0] );
  file.ReadDouble( 3, m_P[1] );
  file.ReadDouble( &m_2d_tol );
  file.ReadDouble( &m_3d_tol );
}

////////////////////////////////////////////////////////////////
//   Class CRhinoBrepLoop
////////////////////////////////////////////////////////////////

CRhinoBrepLoop& CRhinoBrepLoop::operator=(const CRhinoBrepLoop& src)
{
  if ( &src != this ) {
    m_index = src.m_index;
    m_ti    = src.m_ti;
    m_type  = src.m_type;
    m_fi    = src.m_fi;
  }
  return *this;
}

CRhinoBrepLoop::CRhinoBrepLoop(int i /*= -1*/) 
              : CRhinoBrepEntity(i), 
                m_type(CRhinoBrepLoop::unknown), 
                m_fi(-1)
{}

void CRhinoBrepLoop::Write( CRhinoFile& file ) const
{
  file.WriteInteger( m_index );
  file.WriteIntArray( m_ti );
  file.WriteInteger( m_type );
  file.WriteInteger( m_fi );
}

void CRhinoBrepLoop::Read( CRhinoFile& file )
{
  int i;
  file.ReadInteger( &m_index );
  file.ReadIntArray( m_ti );
  file.ReadInteger( &i );
  switch(i) {
  case unknown:
    m_type = unknown;
    break;
  case outer:
    m_type = outer;
    break;
  case inner:
    m_type = inner;
    break;
  case slit:
    m_type = slit;
    break;
  }
  file.ReadInteger( &m_fi );
}

////////////////////////////////////////////////////////////////
//   Class CRhinoBrepFace
////////////////////////////////////////////////////////////////

CRhinoBrepFace::CRhinoBrepFace(int i /* = -1*/)
              : CRhinoBrepEntity(i), 
                m_si(-1), 
                m_bRev(FALSE)
{}

CRhinoBrepFace& CRhinoBrepFace::operator=(const CRhinoBrepFace& src)
{
  if ( &src != this ) {
    m_index = src.m_index;
    m_li    = src.m_li;
    m_si    = src.m_si;
    m_bRev  = src.m_bRev;
  }
  return *this;
}

void CRhinoBrepFace::Write( CRhinoFile& file ) const
{
  file.WriteInteger( m_index );
  file.WriteIntArray( m_li );
  file.WriteInteger( m_si );
  file.WriteInteger( m_bRev );
}

void CRhinoBrepFace::Read( CRhinoFile& file )
{
  file.ReadInteger( &m_index );
  file.ReadIntArray( m_li );
  file.ReadInteger( &m_si );
  file.ReadInteger( &m_bRev );
}

////////////////////////////////////////////////////////////////
//   Class CRhinoBrep
////////////////////////////////////////////////////////////////

void CRhinoBrep::Initialize()
{
  m_bounding_box[0][0] = 0.0;
  m_bounding_box[0][1] = 0.0;
  m_bounding_box[0][2] = 0.0;
  m_bounding_box[1][0] = 0.0;
  m_bounding_box[1][1] = 0.0;
  m_bounding_box[1][2] = 0.0;
}

CRhinoBrep::CRhinoBrep() : CRhinoObject( TCODE_RHINO_OBJECT_BREP )
{ 
  Initialize();
}

CRhinoBrep::CRhinoBrep( int typecode, int i ) : CRhinoObject( typecode, i )
{ 
  Initialize();
}

CRhinoBrep::~CRhinoBrep()
{
}

CRhinoFile::eStatus
CRhinoBrep::SkipToAG_STUFF( CRhinoFile& file, const int end_mark_typecode )
{
  // skip over junk between TCODE_AG_BLOB and TCODE_AG_STUFF | ...
  CRhinoFile::eStatus status = CRhinoFile::good;
	CRhinoChunk* pChunk = NULL;
  FILE* pFile = file.GetFile();
  while ( status == CRhinoFile::good ) {
    // terminating chunk is long and must be read carefully
	  int c = fgetc( pFile );
	  if( c == EOF ){
     	throw CRhinoException(CRhinoException::unexpected_end_of_file);
	  }
	  c = ungetc( c, pFile );
	  if( c == EOF ) {
		  throw CRhinoException(CRhinoException::unable_to_seek);
    }
    int typecode = 0;
	  file.ReadInteger( 1, &typecode );			// read the type code
    if ( fseek( pFile, -((int)sizeof(typecode)), SEEK_CUR ) ) {
		  throw CRhinoException(CRhinoException::unable_to_seek);
    }
    if ( typecode == end_mark_typecode )
      break;
    // skip over chunk
  	pChunk = file.ReadChunk( status );
    file.SkipChunk();
    status = file.EndChunk();
		delete pChunk;
	}

	if ( status == CRhinoFile::eof)
		throw CRhinoException(CRhinoException::unexpected_end_of_file);
  
  return status;
}

CRhinoFile::eStatus
CRhinoBrep::Read_AG_CURVE( CRhinoFile& file, int* crv_index )
{
  *crv_index = -1;
  CRhinoAgCurve* pAgCrv = NULL;
  unsigned int crv_typecode;
  int crv_length;
  CRhinoFile::eStatus status = file.BeginReadChunk( crv_typecode, crv_length );
  if ( status != CRhinoFile::good )
    return status;
  if ( crv_typecode != TCODE_AG_CURVE || crv_length < 1 )
    return CRhinoFile::crc_error;
  pAgCrv = new CRhinoAgCurve();
  if ( !pAgCrv ) {
    throw CRhinoException( CRhinoException::out_of_memory );
  }
  status = pAgCrv->Read( file );
  CRhinoFile::eStatus end_status = file.EndChunk();
  if ( status == CRhinoFile::good ) {
    if ( pAgCrv->Dimension() == 2 ) {
      // 2d trimming curve
      m_C2.Append(pAgCrv);
      *crv_index = m_C2.Size()-1;
    }
    else {
      // 3d edge curve
      m_C3.Append(pAgCrv);
      *crv_index = m_C3.Size()-1;
    }
    status = end_status;
  }
  else
    delete pAgCrv;
  return status;
}

CRhinoFile::eStatus
CRhinoBrep::Read_AG_SURFACE( CRhinoFile& file, int* srf_index )
{
  *srf_index = -1;
  CRhinoAgSurface* pAgSrf = NULL;
  unsigned int srf_typecode;
  int srf_length;
  CRhinoFile::eStatus status = file.BeginReadChunk( srf_typecode, srf_length );
  if ( status != CRhinoFile::good )
    return status;
  if ( srf_typecode != TCODE_AG_SURFACE || srf_length < 1 )
    return CRhinoFile::crc_error;
  pAgSrf = new CRhinoAgSurface();
  if ( !pAgSrf ) {
    throw CRhinoException( CRhinoException::out_of_memory );
  }
  status = pAgSrf->Read( file );
  CRhinoFile::eStatus end_status = file.EndChunk();
  if ( status == CRhinoFile::good ) {
    m_S.Append(pAgSrf);
    *srf_index = m_S.Size()-1;
    status = end_status;
  }
  else
    delete pAgSrf;
  return status;
}

CRhinoFile::eStatus
CRhinoBrep::Read_AG_TEDGE( CRhinoFile& file, int* trim_index )
{
  // Reads TCODE_AG_TEDGE chunk and adds information to
  // m_L[].  Returns index into m_L[].

  *trim_index = -1;

  // begin tedge chunk
  unsigned int te_typecode;
  int te_length;
  CRhinoFile::eStatus status = file.BeginReadChunk( te_typecode, te_length );
  if ( status != CRhinoFile::good )
    return status;
  if ( te_typecode != TCODE_AG_TEDGE || te_length < 1 )
    return CRhinoFile::crc_error;

  // skip over non-ag info
  status = SkipToAG_STUFF( file, TCODE_AG_TEDGESTUFF );
  if ( status != CRhinoFile::good )
    return status;

  // begin tedge stuff chunk
  status = file.BeginReadChunk( te_typecode, te_length );
  if ( status != CRhinoFile::good )
    return status;
  if ( te_typecode != TCODE_AG_TEDGESTUFF || te_length < 1 )
    return CRhinoFile::crc_error;

  do {
    char c;
    file.ReadByte( 1, &c );

    BOOL bHasEdge = (c % 2 ); // bit 0 = 1 if "tedge" has non NULL "->edge"
    BOOL bHasMate = (c & 6 ); // bit 1 or 2 = 1 if "tedge" has non NULL "->twin"
    BOOL bIsSeam  = (c & 2 ); // bit 1 = 1 if "tedge->twin" belongs to same face

    int revedge;
    file.ReadInteger(1,&revedge);

    int gcon;
    file.ReadInteger(1,&gcon);

    int mono;
    file.ReadInteger(1,&mono);

    double tol_3d;
    file.ReadDouble( 1, &tol_3d );

    double tol_2d;
    file.ReadDouble( 1, &tol_2d );

    int tc2i = -1;
    status = Read_AG_CURVE( file, &tc2i );
    if ( status != CRhinoFile::good )
      break;

    int ei = -1;
    if ( bHasEdge ) {
      int ec3i = -1;
      status = Read_AG_CURVE( file, &ec3i );
      if ( status != CRhinoFile::good )
        break;
      ei = m_E.Size();
      m_E.Append(CRhinoBrepEdge(ei));
      m_E[ei].m_c3i = ec3i;
      m_C3[ec3i]->Domain( &m_E[ei].m_t[0], &m_E[ei].m_t[1] );
    }

    const int ti = m_T.Size();
    m_T.Append(CRhinoBrepTrim(ti));
    CRhinoBrepTrim& trim = m_T[ti];
    trim.m_c2i = tc2i;
    m_C2[tc2i]->Domain( &trim.m_t[0], &trim.m_t[1] );
    trim.m_ei = ei;
    if ( ei >= 0 ) {
      m_E[ei].m_ti.Append(ti);
    }
    trim.m_bRev3d = (revedge) ? TRUE : FALSE;
    if ( status != CRhinoFile::good )
      break;

    if ( bIsSeam )
      trim.m_type = CRhinoBrepTrim::seam;
    else if ( !bHasEdge && !bHasMate )
      trim.m_type = CRhinoBrepTrim::singular;
    else if ( bHasEdge ) {
      if ( !bHasMate )
        trim.m_type = CRhinoBrepTrim::boundary;
      else
        trim.m_type = CRhinoBrepTrim::mated;
    }
    else if ( bHasMate )
      trim.m_type = CRhinoBrepTrim::mated; // edge stored with mate's face

    trim.m_2d_tol = tol_2d;
    trim.m_3d_tol = tol_3d;

    *trim_index = ti;
  } while (FALSE);

  // end of stuff chunk
  CRhinoFile::eStatus end_status = file.EndChunk();
  if ( status == CRhinoFile::good )
    status = end_status;

  // end of tedge chunk
  end_status = file.EndChunk();
  if ( status == CRhinoFile::good )
    status = end_status;
  return status;
}

CRhinoFile::eStatus
CRhinoBrep::Read_AG_BOUNDARY( CRhinoFile& file, int* loop_index )
{
  // Reads TCODE_AG_BOUNDARY chunk and adds information to
  // m_L[].  Returns index into m_L[].

  *loop_index = -1;

  // begin boundary chunk
  unsigned int bnd_typecode;
  int bnd_length;
  CRhinoFile::eStatus status = file.BeginReadChunk( bnd_typecode, bnd_length );
  if ( status != CRhinoFile::good )
    return status;
  if ( bnd_typecode != TCODE_AG_BOUNDARY || bnd_length < 1 )
    return CRhinoFile::crc_error;

  // skip over non-ag stuff
  status = SkipToAG_STUFF( file, TCODE_AG_BOUNDARYSTUFF );
  if ( status != CRhinoFile::good )
    return status;

  // begin boundary stuff chunk
  status = file.BeginReadChunk( bnd_typecode, bnd_length );
  if ( status != CRhinoFile::good )
    return status;
  if ( bnd_typecode != TCODE_AG_BOUNDARYSTUFF || bnd_length < 1 )
    return CRhinoFile::crc_error;

  const int li = m_L.Size();
  m_L.Append(CRhinoBrepLoop(li));
  CRhinoBrepLoop& loop = m_L[li];

  do {
    int tedge_count;
    int btype;
    double pspace_box[2][2]; // parameter space bounding box

    file.ReadInteger( 1, &tedge_count );
    if ( tedge_count < 1 ) {
      status = CRhinoFile::crc_error;
      break;
    }
    file.ReadInteger( 1, &btype );
    if ( btype < -1 || btype > 1 ) {
      status = CRhinoFile::crc_error;
      break;
    }
    file.ReadDouble( 4, &pspace_box[0][0] );

    switch( btype ) {
    case -1:
      loop.m_type = CRhinoBrepLoop::slit;
      break;
    case  0:
      loop.m_type = CRhinoBrepLoop::outer;
      break;
    case  1:
      loop.m_type = CRhinoBrepLoop::inner;
      break;
    }
    loop.m_ti.Reserve(tedge_count);

    int lti;
    for ( lti = 0; lti < tedge_count; lti++ ) {
      int ti = -1;
      status = Read_AG_TEDGE( file, &ti );
      if ( status != CRhinoFile::good )
        break;
      loop.m_ti.Append(ti);
      m_T[ti].m_li = li;
    }
    if ( status != CRhinoFile::good )
      break;

    *loop_index = li;
  } while (FALSE);

  // end boundary stuff chunk
  CRhinoFile::eStatus end_status = file.EndChunk();
  if ( status == CRhinoFile::good )
    status = end_status;

  // end boundary chunk
  end_status = file.EndChunk();
  if ( status == CRhinoFile::good )
    status = end_status;

  return status;
}


static
void 
GetTrimIsoTols( const CRhinoNurbsSurface* pS, double itols[6] )
{
  double k0, k1;
  const double rel_tol = 8.0*RHINO_SQRT_EPSILON;
  if ( pS ) {
    k0 = pS->m_knot[0][pS->m_order[0]-2];
    k1 = pS->m_knot[0][pS->m_order[0]-1];
    itols[0] = k0 + (k1-k0)*rel_tol;
    k0 = pS->m_knot[0][pS->m_cv_count[0]-2];
    k1 = pS->m_knot[0][pS->m_cv_count[0]-1];
    itols[1] = k1 - (k1-k0)*rel_tol;
    k0 = pS->m_knot[0][pS->m_order[0]-2];
    k1 = pS->m_knot[0][pS->m_cv_count[0]-1];
    itols[2] = (k1-k0)*rel_tol;

    k0 = pS->m_knot[1][pS->m_order[1]-2];
    k1 = pS->m_knot[1][pS->m_order[1]-1];
    itols[3] = k0 + (k1-k0)*rel_tol;
    k0 = pS->m_knot[1][pS->m_cv_count[1]-2];
    k1 = pS->m_knot[1][pS->m_cv_count[1]-1];
    itols[4] = k1 - (k1-k0)*rel_tol;
    k0 = pS->m_knot[1][pS->m_order[1]-2];
    k1 = pS->m_knot[1][pS->m_cv_count[1]-1];
    itols[5] = (k1-k0)*rel_tol;
  }
}

static
int
TrimIsoFlag( const CRhinoNurbsCurve* pC, double itols[6] )
{
  int rc = CRhinoBrepTrim::not_iso;
  double pt[2][3] = {{0.0,0.0,0.0},{0.0,0.0,0.0}};
  double box[2][3] = {{0.0,0.0,0.0},{0.0,0.0,0.0}};
  double du, dv;
  if ( pC 
       && pC->GetCV( 0, rh_not_rational, pt[0] ) 
       && pC->GetCV( pC->CVCount()-1, rh_not_rational, pt[1] ) ) {
    du = fabs(pt[1][0]-pt[0][0]);
    dv = fabs(pt[1][1]-pt[0][1]);
    if ( du <= dv ) {
      if ( du <= itols[2] ) {
        if ( pC->GetBoundingBox( box[0], box[1] ) ) {
          du = fabs( box[1][0] - box[0][0] );
          if ( du <= itols[2] ) {
            if      ( box[1][0] <= itols[0] )
              rc = CRhinoBrepTrim::W_iso;
            else if ( box[0][0] >= itols[1] )
              rc = CRhinoBrepTrim::E_iso;
            else
              rc = CRhinoBrepTrim::u_iso;
          }
        }
      }
    }
    else {
      if ( dv <= itols[5] ) {
        if ( pC->GetBoundingBox( box[0], box[1] ) ) {
          dv = fabs( box[1][1] - box[0][1] );
          if ( dv <= itols[5] ) {
            if      ( box[1][1] <= itols[3] )
              rc = CRhinoBrepTrim::S_iso;
            else if ( box[0][1] >= itols[4] )
              rc = CRhinoBrepTrim::N_iso;
            else
              rc = CRhinoBrepTrim::v_iso;
          }
        }
      }
    }
  }
  return rc;
}

void
CRhinoBrep::SetTrimIsoFlag( int trim_index, double itols[6] )
{
  CRhinoBrepTrim& trim = m_T[trim_index];
  int i;
  int rc0 = 0;
  int rc = 0;
  const CRhinoAgCurve& c = *m_C2[trim.m_c2i];
  const int n = c.NurbsCurveCount();
  rc0 = rc = TrimIsoFlag(c[0], itols);
  for ( i = 1; i < n && rc0 > 0; i++ ) {
    rc = TrimIsoFlag(c[i], itols);
    if (rc0 != rc)
      rc0 = rc = 0;
  }
  switch ( rc0 ) {
  case CRhinoBrepTrim::u_iso:
    trim.m_iso = CRhinoBrepTrim::u_iso;
    break;
  case CRhinoBrepTrim::v_iso:
    trim.m_iso = CRhinoBrepTrim::v_iso;
    break;
  case CRhinoBrepTrim::W_iso:
    trim.m_iso = CRhinoBrepTrim::W_iso;
    break;
  case CRhinoBrepTrim::S_iso:
    trim.m_iso = CRhinoBrepTrim::S_iso;
    break;
  case CRhinoBrepTrim::E_iso:
    trim.m_iso = CRhinoBrepTrim::E_iso;
    break;
  case CRhinoBrepTrim::N_iso:
    trim.m_iso = CRhinoBrepTrim::N_iso;
    break;
  case CRhinoBrepTrim::not_iso:
  default:
    trim.m_iso = CRhinoBrepTrim::not_iso;
    break;
  }
}

void
CRhinoBrep::SetTrimIsoFlags()
{
  int fi;
  const int fcnt = m_F.Size();
  for ( fi = 0; fi < fcnt; fi++ ) {
    SetTrimIsoFlag(fi);
  }
}

void
CRhinoBrep::SetTrimIsoFlag( int face_index )
{
  const CRhinoBrepFace& face = m_F[face_index];
  double itols[6];
  GetTrimIsoTols( m_S[face.m_si], itols );
  int fli, lti;
  const int loop_count = face.m_li.Size();
  for ( fli = 0; fli < loop_count; fli++ ) {
    const CRhinoBrepLoop& loop = m_L[face.m_li[fli]];
    const int trim_count = loop.m_ti.Size();
    for ( lti = 0; lti < trim_count; lti++ ) {
      SetTrimIsoFlag( loop.m_ti[lti], itols );
    }
  }
}

BOOL
CRhinoBrep::IsValidTrim( int trim_index ) const
{
  if ( trim_index < 0 || trim_index >= m_T.Size() )
    return FALSE;
  const CRhinoBrepTrim& trim = m_T[trim_index];
  if ( trim.m_index != trim_index )
    return FALSE;
  if ( trim.m_c2i < 0 || trim.m_c2i >= m_C2.Size() )
    return FALSE;
  const CRhinoAgCurve* pC = m_C2[trim.m_c2i];
  if ( !pC )
    return FALSE;
  if ( pC->Dimension() != 2 )
    return FALSE;
  double t0, t1;
  if ( !pC->Domain(&t0,&t1) )
    return FALSE;
  if ( t0 != trim.m_t[0] || t1 != trim.m_t[1] )
    return FALSE;
  int vi0 = trim.m_vi[0];
  int vi1 = trim.m_vi[1];
  if ( vi0 < 0 || vi0 >= m_V.Size() )
    return FALSE;
  if ( vi1 < 0 || vi1 >= m_V.Size() )
    return FALSE;
  const int ei = trim.m_ei;
  if ( trim.m_type == CRhinoBrepTrim::singular ) {
    // singular trim - no edge and 3d v0 = 3d v1
    if ( ei != -1 )
      return FALSE;
    if ( vi0 != vi1 )
      return FALSE;
    if ( pC->IsClosed() )
      return FALSE;
  }
  else {
    // non-singular trim should have valid edge
    if ( ei < 0 || ei >= m_E.Size() )
      return FALSE;
    const CRhinoBrepEdge& edge = m_E[ei];
    if ( edge.m_vi[trim.m_bRev3d?1:0] != vi0 )
      return FALSE;
    if ( edge.m_vi[trim.m_bRev3d?0:1] != vi1 )
      return FALSE;
    if ( pC->IsClosed() ) {
      // (open 2d trims can still have vi0 = vi1 on closed surfaces)
      if ( vi0 != vi1 )
        return FALSE;
    }
    BOOL bFoundIt = FALSE;
    int i;
    for ( i = 0; i < edge.m_ti.Size() && !bFoundIt; i++ ) {
      bFoundIt  = ( edge.m_ti[i] == trim_index );
    }
    if ( !bFoundIt )
      return FALSE;
  }
  if ( trim.m_li < 0 || trim.m_li >= m_L.Size() )
    return FALSE;
  return TRUE;
}

BOOL
CRhinoBrep::IsValidLoop( int loop_index ) const
{
  if ( loop_index < 0 || loop_index >= m_L.Size() )
    return FALSE;
  const CRhinoBrepLoop& loop = m_L[loop_index];
  if ( loop.m_index != loop_index )
    return FALSE;
  if ( loop.m_fi < 0 || loop.m_fi >= m_F.Size() )
    return FALSE;
  const int loop_trim_count = loop.m_ti.Size();
  if ( loop_trim_count <= 0 )
    return FALSE;

  if ( loop.m_type != CRhinoBrepLoop::outer && loop.m_type != CRhinoBrepLoop::inner )
    return FALSE;

  // make sure trims are valid
  int i, lti, ti;
  for ( lti = 0; lti < loop_trim_count; lti++ ) {
    ti = loop.m_ti[lti];
    for ( i = 0; i < lti; i++ ) {
      if ( loop.m_ti[i] == ti )
        return FALSE;
    }
    if ( !IsValidTrim( ti ) )
      return FALSE;
    if ( m_T[ti].m_li != loop_index )
      return FALSE;
  }

  // make sure end of trims jibe
  int ci0, ci1;
  double P0[2], P1[2];
  const CRhinoAgCurve *pC0, *pC1;
  for ( lti = 0; lti < loop_trim_count; lti++ ) {
    ci0 = m_T[loop.m_ti[lti]].m_c2i;
    ci1 = m_T[loop.m_ti[(lti+1)%loop_trim_count]].m_c2i;
    pC0 = m_C2[ci0];
    pC1 = m_C2[ci1];
    pC0->EndPoint( 1, P0 ); // end of this 2d trim
    pC1->EndPoint( 0, P1 ); // start of next 2d trim
    if ( !RhinoIsEqual( 2, P0, P1 ) )
      return FALSE;
  }

  return TRUE; 
}


BOOL
CRhinoBrep::IsValidFace( int face_index ) const
{
  if ( face_index < 0 || face_index >= m_F.Size() )
    return FALSE;
  const CRhinoBrepFace& face = m_F[face_index];
  if ( face.m_index != face_index )
    return FALSE;

  const int face_loop_count = face.m_li.Size();
  if ( face_loop_count <= 0 )
    return FALSE;

  int i, fli, li;
  for ( fli = 0; fli < face_loop_count; fli++ ) {
    li = face.m_li[fli];
    for ( i = 0; i < fli; i++ ) {
      if ( face.m_li[i] == li )
        return FALSE;
    }
    if ( !IsValidLoop( li ) )
      return FALSE;
    const CRhinoBrepLoop& loop = m_L[li];
    if ( loop.m_fi != face_index )
      return FALSE;
    if ( fli == 0 ) {
      if ( loop.m_type != CRhinoBrepLoop::outer )
        return FALSE;
    }
    else {
      if ( loop.m_type != CRhinoBrepLoop::inner )
        return FALSE;
    }
  }

  const int si = face.m_si;
  if ( si < 0 || si >= m_S.Size() )
    return FALSE;
  if ( !m_S[si] )
    return FALSE;

  return TRUE; 
}

BOOL
CRhinoBrep::IsValidEdge( int edge_index ) const
{
  if ( edge_index < 0 || edge_index >= m_E.Size() )
    return FALSE;
  const CRhinoBrepEdge& edge = m_E[edge_index];
  if ( edge.m_index != edge_index )
    return FALSE;
  const int ci = edge.m_c3i;
  if ( ci < 0 || ci >= m_C3.Size() )
    return FALSE;
  const CRhinoAgCurve* pC = m_C3[ci];
  if ( !pC )
    return FALSE;
  double t0, t1;
  if ( !pC->Domain( &t0, &t1 ) )
    return FALSE;
  if ( t0 != edge.m_t[0] || t1 != edge.m_t[1] )
    return FALSE;
  const int vi0 = edge.m_vi[0];
  const int vi1 = edge.m_vi[1];
  if ( vi0 < 0 || vi0 >= m_V.Size() )
    return FALSE;
  if ( vi1 < 0 || vi1 >= m_V.Size() )
    return FALSE;
  int evi;
  for ( evi = 0; evi < 2; evi++ ) {
    const CRhinoBrepVertex& vertex = m_V[edge.m_vi[evi]];
    const int vertex_edge_count = vertex.m_ei.Size();
    BOOL bFoundIt = FALSE;
    int vei;
    for ( vei = 0; vei < vertex_edge_count && !bFoundIt; vei++ ) {
      bFoundIt = (vertex.m_ei[vei] == edge_index);
    }
    if ( !bFoundIt )
      return FALSE;
  }

  if ( pC->IsClosed() ) {
    if ( vi0 != vi1 )
      return FALSE;
  }
  else {
    if ( vi0 == vi1 )
      return FALSE;
  }

  const int edge_trim_count = edge.m_ti.Size();
  if ( edge_trim_count < 0 )
    return FALSE;
  int i, eti, ti;
  for (eti = 0; eti < edge_trim_count; eti++ ) {
    ti = edge.m_ti[eti];
    if ( ti < 0 || ti >= m_T.Size() )
      return FALSE;
    for ( i = 0; i < eti; i++ ) {
      if ( edge.m_ti[i] == ti )
        return FALSE;
    }
    const CRhinoBrepTrim& trim = m_T[ti];
    if ( trim.m_ei != edge_index )
      return FALSE;
  }

  return TRUE;
}

BOOL
CRhinoBrep::IsValidVertex( int vertex_index ) const
{
  if ( vertex_index < 0 || vertex_index >= m_V.Size() )
    return FALSE;
  const CRhinoBrepVertex& vertex = m_V[vertex_index];
  if ( vertex.m_index != vertex_index )
    return FALSE;

  const int vertex_edge_count = vertex.m_ei.Size();
  int i, vei, ei;
  for ( vei = 0; vei < vertex_edge_count; vei++ ) {
    ei = vertex.m_ei[vei];
    if ( ei < 0 || ei >= m_E.Size() )
      return FALSE;
    const CRhinoBrepEdge& edge = m_E[ei];
    for ( i = 0; i < vei; i++ ) {
      if ( vertex.m_ei[i] == ei ) {
        // edge should be closed
        if ( edge.m_vi[0] != vertex_index || edge.m_vi[1] != vertex_index )
          return FALSE;
        for (i++; i < vei; i++ ) {
          if ( vertex.m_ei[i] == ei )
            return FALSE;
        }
      }
    }
    if ( edge.m_vi[0] != vertex_index && edge.m_vi[1] != vertex_index )
      return FALSE;
  }
  return TRUE;
}

BOOL
CRhinoBrep::IsValid( void ) const
{
  const int curve2d_count = m_C2.Size();
  const int curve3d_count = m_C3.Size();
  const int surface_count = m_S.Size();
  const int vertex_count  = m_V.Size();
  const int edge_count    = m_E.Size();
  const int trim_count    = m_T.Size();
  const int loop_count    = m_L.Size();
  const int face_count    = m_F.Size();

  int c2i, c3i, si, vi, ei, fi, ti, li;

  // check element indices match array positions
  for ( vi = 0; vi < vertex_count; vi++ ) {
    if ( m_V[vi].m_index != vi )
      return FALSE;
  }
  for ( ei = 0; ei < edge_count; ei++ ) {
    if ( m_E[ei].m_index != ei )
      return FALSE;
  }
  for ( ti = 0; ti < trim_count; ti++ ) {
    if ( m_T[ti].m_index != ti )
      return FALSE;
  }
  for ( li = 0; li < loop_count; li++ ) {
    if ( m_L[li].m_index != li )
      return FALSE;
  }
  for ( fi = 0; fi < face_count; fi++ ) {
    if ( m_F[fi].m_index != fi )
      return FALSE;
  }

  // check 2d curve geometry
  for ( c2i = 0; c2i < curve2d_count; c2i++ ) {
    if ( !m_C2[c2i] )
      return FALSE;
    if ( !m_C2[c2i]->IsValid() )
      return FALSE;
    if ( m_C2[c2i]->Dimension() != 2 )
      return FALSE;
  }

  // check 3d curve geometry
  for ( c3i = 0; c3i < curve3d_count; c3i++ ) {
    if ( !m_C3[c3i] )
      return FALSE;
    if ( !m_C3[c3i]->IsValid() )
      return FALSE;
    if ( m_C3[c3i]->Dimension() != 3 )
      return FALSE;
  }

  // check 3d surface geometry
  for ( si = 0; si < surface_count; si++ ) {
    if ( !m_S[si] )
      return FALSE;
    if ( !m_S[si]->IsValid() )
      return FALSE;
    int dim = m_S[si]->Dimension();
    if ( dim != 3 )
      return FALSE;
  }

  // check vertices
  for ( vi = 0; vi < vertex_count; vi++ ) {
    if ( !IsValidVertex( vi ) ) {
      return FALSE;
    }
  }

  // check edges
  for ( ei = 0; ei < edge_count; ei++ ) {
    if ( !IsValidEdge( ei ) ) {
      return FALSE;
    }
  }

  // check faces
  for ( fi = 0; fi < face_count; fi++ ) {
    if ( !IsValidFace( fi ) ) {
      return FALSE;
    }
  }

  return TRUE;
}

BOOL CRhinoBrep::SetEdgeVertex( const int ei, const int evi, const int vi )
{
  if ( ei < 0 || vi < 0 || evi < 0 || evi > 1 )
    return FALSE;
  CRhinoBrepEdge& edge = m_E[ei];
  if ( edge.m_vi[evi] != vi ) {
    edge.m_vi[evi] = vi;
    CRhinoBrepVertex& vertex = m_V[vi];
    vertex.m_ei.Append(ei);
  }
  const int trim_count = edge.m_ti.Size();
  int eti, ti, tvi;
  for ( eti = 0; eti < trim_count; eti++ ) {
    ti = edge.m_ti[eti];
    if ( ti < 0 )
      continue;
    CRhinoBrepTrim& trim = m_T[ti];
    tvi = trim.m_bRev3d ? 1-evi : evi;
    trim.m_vi[tvi] = vi;
  }
  return TRUE;
}

BOOL CRhinoBrep::HopAcrossEdge( int& ti, int& tvi ) const
{
  // Tf ti is a trim associated with an interior manifold edge,
  // then ti is set to twin.
  int ei, evi, new_ti, new_tvi;
  if ( ti < 0 )
    return FALSE;
  ei = m_T[ti].m_ei;
  if ( ei < 0 )
    return FALSE;
  const CRhinoBrepEdge& edge = m_E[ei];
  if ( edge.m_ti.Size() < 2 )
    return FALSE;
  evi = (m_T[ti].m_bRev3d) ? 1-tvi : tvi;
  new_ti = edge.m_ti[(edge.m_ti[0] == ti)?1:0];
  if ( new_ti < 0 )
    return FALSE;
  new_tvi = (m_T[new_ti].m_bRev3d) ? 1-evi : evi;
  ti = new_ti;
  tvi = new_tvi;
  return TRUE;
}

BOOL CRhinoBrep::SetTrimStartVertex( const int ti0, const int vi )
{
  // Do not use NextEdge(), PrevEdge() because they require
  // the information we are in the act of creating.
  if ( ti0 < 0 || vi < 0 )
    return FALSE;
  int next_ti, ti, ei, evi, tvi, counter;

  // Step counter clockwise around vertex until we hit a boundary
  // or we get back to where we started.
  for ( ti = ti0, tvi = 0, counter = 0; ti >= 0 && counter < 512; counter++ ) {
    if ( counter > 0 ) {
      if ( ti == ti0 && tvi == 0 )
        return TRUE; // vertex was interior
    }
    CRhinoBrepTrim& trim = m_T[ti];
    if ( trim.m_type == CRhinoBrepTrim::singular ) {
      trim.m_vi[0] = trim.m_vi[1] = vi;
      tvi = 1-tvi;
      next_ti = (tvi) ? NextTrim(ti) : PrevTrim(ti);
      ti = next_ti;
      tvi = 1-tvi;
      if ( ti == ti0 && tvi == 0 )
        return TRUE; // vertex was interior
      if ( m_T[ti].m_type != CRhinoBrepTrim::singular )
        HopAcrossEdge( ti, tvi ); // OK if hop fails because ti is a boundary
      continue;
    }

    ei = trim.m_ei;
    evi = (trim.m_bRev3d) ? 1-tvi : tvi;
    if ( !SetEdgeVertex( ei, evi, vi ) )
      return FALSE;
    next_ti = (tvi) ? NextTrim(ti) : PrevTrim(ti);
    ti = next_ti;
    tvi = 1-tvi;
    if ( ti < 0 )
      return FALSE; // should not happen

    if ( m_T[ti].m_type == CRhinoBrepTrim::singular )
      continue;
    ei = m_T[ti].m_ei;
    if ( ei < 0 )
      return FALSE; // should not happen
    evi = (m_T[ti].m_bRev3d) ? 1-tvi : tvi;
    const int edge_trim_count = m_E[ei].m_ti.Size();
    if ( edge_trim_count < 1 )
      break; // should not happen
    if ( edge_trim_count == 1 ) {
      SetEdgeVertex( ei, evi, vi );
      break; // ran into boundary
    }
    if ( !HopAcrossEdge( ti, tvi ) )
      return FALSE;
  }


  // Get ready to step counter clockwise around vertex until
  // we hit a boundary.
  ti = ti0;
  tvi = 0;
  if ( m_T[ti].m_type == CRhinoBrepTrim::singular ) {
    // back up until we get to a non-singular trim
    while ( m_T[ti].m_type == CRhinoBrepTrim::singular ) {
      if ( ti != ti0 ) {
        m_T[ti].m_vi[0] = vi;
        m_T[ti].m_vi[1] = vi;
      }
      ti = PrevTrim(ti);
      tvi = 1;
      if ( ti == ti0 )
        break;
    }
    ei = m_T[ti].m_ei;
    if ( ei >= 0 ) {
      evi = (m_T[ti].m_bRev3d) ? 1-tvi : tvi;
      SetEdgeVertex( ei, evi, vi );
    }
  }
  else {
    ei = m_T[ti].m_ei;
  }
  if ( ei < 0 ) {
    // did the best we could - return TRUE so setter keeps going
    // but the fact we are here means the brep is bogus.
    return TRUE;
  }
  if ( m_E[ei].m_ti.Size() < 2 )
    return TRUE; // ti0 is a boundary - we're done.
  if ( !HopAcrossEdge( ti, tvi ) )
    return FALSE;
  next_ti = (tvi) ? NextTrim(ti) : PrevTrim(ti);
  if ( next_ti < 0 )
    return FALSE;
  ti = next_ti;
  tvi = 1-tvi;
  if ( m_T[ti].m_type != CRhinoBrepTrim::singular ) {
    ei = m_T[ti].m_ei;
    if ( ei < 0 )
      return FALSE;
    if ( m_E[ei].m_ti.Size() == 1 ) {
      evi = (m_T[ti].m_bRev3d)? 1-tvi : tvi;
      SetEdgeVertex( ei, evi, vi );
      return TRUE;
    }
    if ( !HopAcrossEdge( ti, tvi ) )
      return FALSE;
  }

  const int ti1 = ti;
  const int tvi1 = tvi;
  
  for ( ti = ti1, tvi = tvi1, counter = 0; ti >= 0 && counter < 512; counter++ ) {
    if ( counter > 0 ) {
      if ( ti == ti1 && tvi == tvi1 )
        return FALSE; // vertex is not interior - so this should not happen
    }
    CRhinoBrepTrim& trim = m_T[ti];
    if ( trim.m_type == CRhinoBrepTrim::singular ) {
      trim.m_vi[0] = trim.m_vi[1] = vi;
      tvi = 1-tvi;
      next_ti = (tvi) ? NextTrim(ti) : PrevTrim(ti);
      ti = next_ti;
      tvi = 1-tvi;
      if ( ti == ti1 && tvi == tvi1 )
        return FALSE; // vertex is not interior - so this should not happen
      if ( m_T[ti].m_type != CRhinoBrepTrim::singular )
        HopAcrossEdge( ti, tvi );  // OK if hop fails because ti is a boundary
      continue;
    }

    ei = trim.m_ei;
    evi = (trim.m_bRev3d) ? 1-tvi : tvi;
    if ( !SetEdgeVertex( ei, evi, vi ) )
      return FALSE;
    next_ti = (tvi) ? NextTrim(ti) : PrevTrim(ti);
    ti = next_ti;
    tvi = 1-tvi;
    if ( ti < 0 )
      return FALSE; // should not happen
    
    if ( m_T[ti].m_type == CRhinoBrepTrim::singular )
      continue;
    ei = m_T[ti].m_ei;
    if ( ei < 0 )
      return FALSE; // should not happen
    evi = (m_T[ti].m_bRev3d) ? 1-tvi : tvi;
    const int edge_trim_count = m_E[ei].m_ti.Size();
    if ( edge_trim_count < 1 )
      break; // should not happen
    if ( edge_trim_count == 1 ) {
      SetEdgeVertex( ei, evi, vi );
      return TRUE; // ran into boundary - expected
    }
    if ( !HopAcrossEdge( ti, tvi ) )
      return FALSE;
  }

  return FALSE; // should have exited by hitting "expected" boundary ~10 lines above
}

void CRhinoBrep::SetLoopVertices( const int li )
{
  CRhinoBrepLoop& loop = m_L[li];
  const int loop_trim_count = loop.m_ti.Size();
  int lti;
  for ( lti = 0; lti < loop_trim_count; lti++ ) {
    const int ti = loop.m_ti[lti];
    CRhinoBrepTrim& trim = m_T[ti];
    int vi = trim.m_vi[0];
    if ( vi >= 0 )
      continue;
    vi = m_V.Size();
    m_V.Append(CRhinoBrepVertex(vi));
    SetTrimStartVertex( ti, vi );
  }
}

void CRhinoBrep::ClearTrimVertices()
{
  int ti;
  const int tcnt = m_T.Size();
  for ( ti = 0; ti < tcnt; ti++ ) {
    CRhinoBrepTrim& trim = m_T[ti];
    trim.m_vi[0] = -1;
    trim.m_vi[1] = -1;
  }
}

void CRhinoBrep::ClearEdgeVertices()
{
  int ei;
  const int ecnt = m_E.Size();
  for ( ei = 0; ei < ecnt; ei++ ) {
    CRhinoBrepEdge& edge = m_E[ei];
    edge.m_vi[0] = -1;
    edge.m_vi[1] = -1;
  }
}

int CRhinoBrep::NextTrim(int ti) const
{
  const CRhinoBrepTrim& trim = m_T[ti];
  const int li = trim.m_li;
  const CRhinoBrepLoop& loop = m_L[li];
  const int trim_count = loop.m_ti.Size();
  int lti;
  for ( lti = 0; loop.m_ti[lti] != ti && lti < trim_count; lti++)
    ;/* empty for*/
  if ( lti < 0 || lti >= trim_count )
    return -1;
  return loop.m_ti[(lti+1)%trim_count];
}

int CRhinoBrep::PrevTrim(int ti) const
{
  const CRhinoBrepTrim& trim = m_T[ti];
  const int li = trim.m_li;
  const CRhinoBrepLoop& loop = m_L[li];
  const int trim_count = loop.m_ti.Size();
  int lti;
  for ( lti = 0; loop.m_ti[lti] != ti && lti < trim_count; lti++)
    ;/* empty for*/
  if ( lti < 0 || lti >= trim_count )
    return -1;
  return loop.m_ti[(lti+trim_count-1)%trim_count];
}

int CRhinoBrep::NextEdge(int ei, int endi, int* next_endi ) const
{
  const CRhinoBrepEdge& edge = m_E[ei];
  const int vi = edge.m_vi[endi];
  const CRhinoBrepVertex& vertex = m_V[vi];
  const int edge_count = vertex.m_ei.Size();
  int vei;
  if ( next_endi )
    *next_endi = 0;
  for ( vei = 0; vertex.m_ei[vei] != ei && vei < edge_count; vei++)
    ;/* empty for*/
  if ( edge.m_vi[0] == edge.m_vi[1] && endi ) {
    // get next occurance of edge index
    vei++;
    while ( vei < edge_count && vertex.m_ei[vei] != vei )
      vei++;
  }
  if ( vei < 0 || vei >= edge_count )
    return -1;
  vei = (vei+1)%edge_count;
  const int next_ei = vertex.m_ei[vei];
  if ( next_endi ) {
    if ( m_E[next_ei].m_vi[0] == m_E[next_ei].m_vi[1] ) {
      *next_endi = 1;
      for ( vei++; vei < edge_count; vei++ ) {
        if ( vertex.m_ei[vei] == next_ei ) {
          *next_endi = 0;
          break;
        }
      }
    }
    else if ( m_E[next_ei].m_vi[1] == vi )
      *next_endi = 1;
  }
  return next_ei;
}

int CRhinoBrep::PrevEdge(int ei, int endi, int* prev_endi ) const
{
  const CRhinoBrepEdge& edge = m_E[ei];
  const int vi = edge.m_vi[endi];
  const CRhinoBrepVertex& vertex = m_V[vi];
  const int edge_count = vertex.m_ei.Size();
  int vei;
  if ( prev_endi )
    *prev_endi = 0;
  for ( vei = 0; vertex.m_ei[vei] != ei && vei < edge_count; vei++)
    ;/* empty for*/
  if ( edge.m_vi[0] == edge.m_vi[1] && endi ) {
    // get next occurance of edge index
    vei++;
    while ( vei < edge_count && vertex.m_ei[vei] != vei )
      vei++;
  }
  if ( vei < 0 || vei >= edge_count )
    return -1;
  vei = (vei+edge_count-1)%edge_count;
  const int prev_ei = vertex.m_ei[(vei+edge_count-1)%edge_count];
  if ( prev_endi ) {
    if ( m_E[prev_ei].m_vi[0] == m_E[prev_ei].m_vi[1] ) {
      *prev_endi = 1;
      for ( vei++; vei < edge_count; vei++ ) {
        if ( vertex.m_ei[vei] == prev_ei ) {
          *prev_endi = 0;
          break;
        }
      }
    }
    else if ( m_E[prev_ei].m_vi[1] == vi ) {
      *prev_endi = 1;
    }
  }
  return prev_ei;
}


void CRhinoBrep::SetVertices(void)
{
  const int face_count = m_F.Size();
  int fi;
  
  ClearEdgeVertices();
  ClearTrimVertices();
  m_V.Empty();
  m_V.Shrink();
  for ( fi = 0; fi < face_count; fi++ ) {
    CRhinoBrepFace& face = m_F[fi];
    const int loop_count = face.m_li.Size();
    int fli;
    for ( fli = 0; fli < loop_count; fli++ ) {
      SetLoopVertices( face.m_li[fli] );
    }
  }

  // average edges' end location to get vertex location
  const int vertex_count = m_V.Size();
  int vi;
  double VP[3];
  for ( vi = 0; vi < vertex_count; vi++ ) {
    VP[0] = VP[1] = VP[2] = 0.0;
    double d = 0.0;
    CRhinoBrepVertex& vertex = m_V[vi];
    const int edge_count = vertex.m_ei.Size();
    double EP[3];
    int vei;
    for ( vei = 0; vei < edge_count; vei++ ) {
      const int ei = vertex.m_ei[vei];
      if ( ei < 0 ) 
        continue;
      const CRhinoBrepEdge& edge = m_E[ei];
      if ( edge.m_c3i < 0 )
        continue;
      const CRhinoAgCurve* pAgCrv = m_C3[edge.m_c3i];
      if ( !pAgCrv )
        continue;
      int evi;
      if ( edge.m_vi[0] == vi )
        evi = 0;
      else if ( edge.m_vi[1] == vi )
        evi = 1;
      else
        continue;
      if ( !pAgCrv->EndPoint(evi,EP) )
        continue;
      VP[0] += EP[0];
      VP[1] += EP[1];
      VP[2] += EP[2];
      d += 1.0;
    }
    if ( d > 0.0 ) {
      d = 1.0/d;
      vertex.x = d*VP[0];
      vertex.y = d*VP[1];
      vertex.z = d*VP[2];
    }
  }
}

void CRhinoBrep::WriteBrepCurve( CRhinoFile& file, const CRhinoAgCurve& c ) const
{
  int i;
  const int nurbs_curve_count = c.NurbsCurveCount();
  file.WriteInteger( nurbs_curve_count );
  for ( i = 0; i < nurbs_curve_count; i++ ) {
    c[i]->Write(file);
  }
}

CRhinoAgCurve* CRhinoBrep::ReadBrepCurve( CRhinoFile& file) const
{
  int i;
  int nurbs_curve_count = 0;
  CRhinoFile::eStatus status;
  CRhinoChunk* pChunk;
  CRhinoNurbsCurve* pNurbsCurve;

  CRhinoAgCurve* pCurve = new CRhinoAgCurve();

  file.ReadInteger( &nurbs_curve_count );

  for ( i = 0; i < nurbs_curve_count; i++ ) {
    pChunk = file.ReadChunk( status );
    if ( pChunk->TypeCode() == TCODE_RHINO_OBJECT_NURBS_CURVE ) {
      pNurbsCurve = (CRhinoNurbsCurve*)pChunk;
    }
    else {
      delete pChunk;
      pNurbsCurve = 0;
    }
    pCurve->AddNurbsCurve(pNurbsCurve);
  }

  return pCurve;
}

void CRhinoBrep::Write( CRhinoFile& file ) const
{

  // For now, write Brep's as 3d CRhinoNurbsCurves and
  // CRhinoNurbsSurfaces to avoid getting AGLib structures that 
  // do not comply with AG restrictions.

	// Make sure layer definition is in the file
	file.LayerIndex( Layer() );	// Get layer index or -1

  file.BeginWriteChunk( CRhinoBrep::TypeCode() ); // so CRhinoAgStuff::Write() will work
  {
    file.BeginWriteChunk( TCODE_RHINO_OBJECT_DATA );
    {
      int i;

      file.WriteInteger( 100 ); // serialization version

      // 2d curves
      file.WriteInteger( m_C2.Size() );
      for ( i = 0; i < m_C2.Size(); i++ ) {
        WriteBrepCurve( file, *m_C2[i] );
      }

      // 3d curves
      file.WriteInteger( m_C3.Size() );
      for ( i = 0; i < m_C3.Size(); i++ ) {
        WriteBrepCurve( file, *m_C3[i] );
      }

      // untrimmed surfaces
      file.WriteInteger( m_S.Size() );
      for ( i = 0; i < m_S.Size(); i++ ) {
        m_S[i]->Write(file);
      }

      // vertices
      file.WriteInteger( m_V.Size() );
      for ( i = 0; i < m_V.Size(); i++ ) {
        m_V[i].Write(file);
      }

      // edges
      file.WriteInteger( m_E.Size() );
      for ( i = 0; i < m_E.Size(); i++ ) {
        m_E[i].Write(file);
      }

      // trims
      file.WriteInteger( m_T.Size() );
      for ( i = 0; i < m_T.Size(); i++ ) {
        m_T[i].Write(file);
      }

      // loops
      file.WriteInteger( m_L.Size() );
      for ( i = 0; i < m_L.Size(); i++ ) {
        m_L[i].Write(file);
      }

      // faces
      file.WriteInteger( m_F.Size() );
      for ( i = 0; i < m_F.Size(); i++ ) {
        m_F[i].Write(file);
      }

      // bounding box
      file.WriteDouble( 3, m_bounding_box[0] );
      file.WriteDouble( 3, m_bounding_box[1] );

    }
    file.EndChunk();
    WriteAttributes( file, TCODE_RHINO_OBJECT_END );
  }
  file.EndChunk();
}

CRhinoFile::eStatus CRhinoBrep::ReadBrep( CRhinoFile& file )
{
  // Low level tool that reads the 
  // TCODE_RHINO_OBJECT_DATA chunk
  // that contains the parameters
  // that define the b-rep. 
  
  CRhinoFile::eStatus status = CRhinoFile::good;
  int sz, i;
  int version = -1;

  file.ReadInteger( &version ); // serialization version
  // version = 100 means the b-rep was written by the RhinoIO toolkit
  // version = 101 means the b-rep was written by Rhino
  if ( version != 100 && version != 101 ) {
    status = CRhinoFile::not_supported;
  }

  // 2d curve geometry
  if ( status == CRhinoFile::good ) {
    file.ReadInteger( &sz );
    if ( sz < 1 ) {
      status = CRhinoFile::crc_error;
    }
    else {
      m_C2.Reserve(sz);
      for ( i = 0; i < sz; i++ ) {
        m_C2.Append( ReadBrepCurve( file ) );
      }
    }
  }

  // 3d curve geomery
  if ( status == CRhinoFile::good ) {
    file.ReadInteger( &sz );
    if ( sz < 1 ) {
      status = CRhinoFile::crc_error;
    }
    else {
      m_C3.Reserve(sz);
      for ( i = 0; i < sz; i++ ) {
        m_C3.Append( ReadBrepCurve( file ) );
      }
    }
  }

  // surface geometry
  if ( status == CRhinoFile::good ) {
    file.ReadInteger( &sz );
    if ( sz < 1 ) {
      status = CRhinoFile::crc_error;
    }
    else {
      m_S.Reserve(sz);
      {
        CRhinoChunk* pChunk;
        CRhinoNurbsSurface* pSurface;
        for ( i = 0; i < sz; i++ ) {
          pChunk = file.ReadChunk( status );
          if ( status != CRhinoFile::good ) {
            break;
          }
          if ( pChunk->TypeCode() == TCODE_RHINO_OBJECT_NURBS_SURFACE ) {
            pSurface = (CRhinoNurbsSurface*)pChunk;
          }
          else {
            delete pChunk;
            pSurface = 0;
          }
          m_S.Append( pSurface );
        }
      }
    }
  }

  if ( status == CRhinoFile::good ) {
    // read topology

    // vertices
    file.ReadInteger( &sz );
    m_V.Reserve(sz);
    for ( i = 0; i < sz; i++ ) {
      m_V.Append(CRhinoBrepVertex());
      m_V[i].Read(file);
    }

    // edges
    file.ReadInteger( &sz );
    m_E.Reserve(sz);
    for ( i = 0; i < sz; i++ ) {
      m_E.Append(CRhinoBrepEdge());
      m_E[i].Read(file);
    }

    // trims
    file.ReadInteger( &sz );
    m_T.Reserve(sz);
    for ( i = 0; i < sz; i++ ) {
      m_T.Append(CRhinoBrepTrim());
      m_T[i].Read(file);
    }

    // loops
    file.ReadInteger( &sz );
    m_L.Reserve(sz);
    for ( i = 0; i < sz; i++ ) {
      m_L.Append(CRhinoBrepLoop());
      m_L[i].Read(file);
    }

    // faces
    file.ReadInteger( &sz );
    m_F.Reserve(sz);
    for ( i = 0; i < sz; i++ ) {
      m_F.Append(CRhinoBrepFace());
      m_F[i].Read(file);
    }

    // bounding box
    file.ReadDouble( 3, m_bounding_box[0] );
    file.ReadDouble( 3, m_bounding_box[1] );
  }

  return status;
}

CRhinoFile::eStatus CRhinoBrep::Read( CRhinoFile& file )
{
  // reads TCODE_RHINO_OBJECT_BREP chunks
  CRhinoFile::eStatus status = CRhinoFile::not_supported;
 
  int data_typecode = 0;
  int data_length = 0;

  file.ReadInteger( 1, &data_typecode );
  if ( data_typecode != TCODE_RHINO_OBJECT_DATA )
    return CRhinoFile::crc_error;
  file.ReadInteger( 1, &data_length );
  if ( data_length <= 0 )
    return CRhinoFile::crc_error;
  
  status = ReadBrep( file );

  if ( status == CRhinoFile::good ) {
    // read attributes
    CRhinoFile::eStatus att_status = ReadAttributes( file, TCODE_RHINO_OBJECT_END );
    if ( status == CRhinoFile::good )
      status = att_status;
  }
  else {
    file.SkipChunk();
  }


  return status;
}

void CRhinoBrep::SetTolsFromFileValues()
{
  // use m_2d_tol and m_3d_tol read from file to set public tolerances
  const int vcnt = m_V.Size();
  const int tcnt = m_T.Size();
  double endP[3], d;
  int vi, ti, vei, evi, vecnt;
  
  // set trim and edge tolerances from values in file
  for ( ti = 0; ti < tcnt; ti++ ) {
    CRhinoBrepTrim& trim = m_T[ti];
    trim.m_tolerance[0] = trim.m_2d_tol; // "pe_tol"
    trim.m_tolerance[1] = trim.m_2d_tol; // "pe_tol"
    if ( trim.m_ei >= 0 ) {
      CRhinoBrepEdge& edge = m_E[trim.m_ei];
      if ( edge.m_tolerance < trim.m_3d_tol )
        edge.m_tolerance = trim.m_3d_tol; // "e_tol"
    }
  }

  // set vertex tolerances from edge tols and end evaluations
  for ( vi = 0; vi < vcnt; vi++ ) {
    CRhinoBrepVertex& vertex = m_V[vi];
    vecnt = vertex.m_ei.Size();
    for ( vei = 0; vei < vecnt; vei++ ) {
      const CRhinoBrepEdge& edge = m_E[vertex.m_ei[vei]];
      if ( vertex.m_tolerance < edge.m_tolerance )
        vertex.m_tolerance = edge.m_tolerance;
      const CRhinoAgCurve* c = m_C3[edge.m_c3i];
      evi = 0;
      if ( edge.m_vi[0] != vi )
        evi = 1;
      if ( edge.m_vi[evi] == vi ) {
        c->EndPoint(evi,endP);
        d = RhinoDistance( 3, &vertex.x, endP );
        if ( d > vertex.m_tolerance ) {
          vertex.m_tolerance = d;      
        }
      }
    }
  }
}

BOOL
CRhinoBrep::GetBoundingBox(
          double box_min[3],
          double box_max[3],
          BOOL bGrowBox     // = FALSE
          ) const
{
  BOOL rc = TRUE;
  const int srf_count = m_S.Size();
  int i;
  for ( i = 0; i < srf_count; i++ ) {
    if ( !m_S[i] )
      continue;
    if ( m_S[i]->GetBoundingBox( box_min, box_max, bGrowBox ) )
      bGrowBox = TRUE;
    else
      rc = FALSE;
  }
  return rc;
}

BOOL
CRhinoBrep::SwapCoordinates( int i, int j )
{
  BOOL rc = FALSE;
  // swap surface coordinates
  const int srf_count = m_S.Size();
  int si;
  for ( si = 0; si < srf_count; si++ ) {
    if ( !m_S[si] ) 
      continue;
    rc = m_S[si]->SwapCoordinates(i,j);
    if ( !rc ) {
      while ( --si >= 0 ) {
        // undo any changes;
        if ( m_S[si] )
          m_S[si]->SwapCoordinates(i,j);
      }
      return FALSE;
    }
  }
  // swap 3d curve coordinates
  const int crv_count = m_S.Size();
  int ci;
  for ( ci = 0; ci < crv_count; ci++ ) {
    if ( !m_C3[ci] ) 
      continue;
    rc = m_C3[ci]->SwapCoordinates(i,j);
    if ( !rc ) {
      // undo any changes;
      while ( --ci >= 0 ) {
        if ( m_C3[ci] )
          m_C3[ci]->SwapCoordinates(i,j);
        for ( si = 0; si < srf_count; si++ ) {
          if ( m_S[si] )
            m_S[si]->SwapCoordinates(i,j);
        }
      }
      return FALSE;
    }
  }
  return rc;
}

BOOL
CRhinoBrep::SwapTrimParameters(
        int trim_index
        )
{
  // helper for SwapLoopParameters
  if ( trim_index < 0 || trim_index >= m_T.Size() )
    return FALSE;
  CRhinoBrepTrim& trim = m_T[trim_index];
  const int ci = trim.m_c2i;
  if ( ci < 0 || ci >= m_C2.Size() )
    return FALSE;
  CRhinoAgCurve* pC = m_C2[ci];
  if ( !pC )
    return FALSE;

  BOOL rc = pC->SwapCoordinates(0,1); // "u" <-> "v"
  if ( !rc )
    return FALSE;

  // reverse 2d curve to keep active region on the left
  rc = pC->Reverse();
  if ( !rc ) {
    // undo changes
    rc = pC->SwapCoordinates(0,1); // "u" <-> "v"
    return FALSE;
  }

  // reverse end information
  double x = trim.m_t[0];
  trim.m_t[0] = -trim.m_t[1];
  trim.m_t[1] = -x;
  int vi = trim.m_vi[0]; trim.m_vi[0] = trim.m_vi[1]; trim.m_vi[1] = vi;
  x = trim.m_P[0][0]; trim.m_P[0][0] = trim.m_P[1][0]; trim.m_P[1][0]= x;
  x = trim.m_P[0][1]; trim.m_P[0][1] = trim.m_P[1][1]; trim.m_P[1][1]= x;
  x = trim.m_P[0][2]; trim.m_P[0][2] = trim.m_P[1][2]; trim.m_P[1][2]= x;

  switch ( trim.m_iso ) {
  case CRhinoBrepTrim::not_iso:
    trim.m_iso = CRhinoBrepTrim::not_iso;
    break;
  case CRhinoBrepTrim::u_iso:
    trim.m_iso = CRhinoBrepTrim::v_iso;
    break;
  case CRhinoBrepTrim::v_iso:
    trim.m_iso = CRhinoBrepTrim::u_iso;
    break;
  case CRhinoBrepTrim::W_iso:
    trim.m_iso = CRhinoBrepTrim::S_iso;
    break;
  case CRhinoBrepTrim::S_iso:
    trim.m_iso = CRhinoBrepTrim::W_iso;
    break;
  case CRhinoBrepTrim::E_iso:
    trim.m_iso = CRhinoBrepTrim::N_iso;
    break;
  case CRhinoBrepTrim::N_iso:
    trim.m_iso = CRhinoBrepTrim::E_iso;
    break;
  }

  if ( trim.m_ei >= 0 )
    trim.m_bRev3d = (trim.m_bRev3d) ? FALSE : TRUE;
  return TRUE;
}

BOOL
CRhinoBrep::SwapLoopParameters(
        int loop_index
        )
{
  BOOL rc = FALSE;
  if ( loop_index < 0 || loop_index >= m_L.Size() )
    return FALSE;
  CRhinoBrepLoop& L = m_L[loop_index];
  const int loop_trim_count = L.m_ti.Size();
  if ( loop_trim_count < 1 )
    return FALSE;
  int lti, ti;
  for ( lti = 0; lti < loop_trim_count; lti++ ) {
    ti = L.m_ti[lti];
    rc = SwapTrimParameters( ti );
    if ( !rc ) {
      while ( --lti >= 0 ) {
        // undo any changes
        ti = L.m_ti[lti];
        SwapTrimParameters( ti );
      }
      return FALSE;
    }
  }

  // reverse order of trimming curves
  if ( rc )
    L.m_ti.Reverse();
  return rc;
}

BOOL
CRhinoBrep::IsSolid() const
{
  BOOL bIsOriented = FALSE;
  BOOL bHasBoundary = TRUE;
  BOOL bIsManifold = IsManifold( &bIsOriented, &bHasBoundary );
  return (bIsManifold && bIsOriented && !bHasBoundary) ? TRUE : FALSE;
}

BOOL
CRhinoBrep::IsManifold( BOOL* pbIsOriented, BOOL* pbHasBoundary ) const
{
  const int fcnt = m_F.Size();
  BOOL bIsManifold = (fcnt > 0) ? TRUE : FALSE;
  BOOL bIsOriented = bIsManifold;
  BOOL bHasBoundary = FALSE;
  int fi, li, ti, other_ti, lcnt, tcnt;
  if ( pbIsOriented )
    *pbIsOriented = bIsOriented;
  if ( pbHasBoundary )
    *pbHasBoundary = bHasBoundary;
  for ( fi = 0; fi < fcnt && bIsManifold; fi++ ) {
    const CRhinoBrepFace& face = m_F[fi];
    lcnt = face.m_li.Size();
    if ( lcnt < 1 ) {
      bIsManifold = FALSE;
    }
    for ( li = 0; li < lcnt && bIsManifold; li++ ) {
      const CRhinoBrepLoop& loop = m_L[face.m_li[li]];
      tcnt = loop.m_ti.Size();
      if (tcnt < 1 ) {
        bIsManifold = FALSE;
      }
      for ( ti = 0; ti < tcnt && bIsManifold; ti++ ) {
        const CRhinoBrepTrim& trim = m_T[ti];
        switch ( trim.m_type ) {
        case CRhinoBrepTrim::boundary:
          bHasBoundary = TRUE;
          break;
        case CRhinoBrepTrim::mated:
        case CRhinoBrepTrim::seam:
          // make sure we have a manifold join
          {
            const CRhinoBrepEdge& edge = m_E[trim.m_ei];
            if ( edge.m_ti.Size() != 2 ) {
              bIsManifold = FALSE;
            }
            else {
              other_ti = edge.m_ti[0];
              if ( other_ti == ti )
                other_ti = edge.m_ti[1];
              if ( other_ti == ti ) {
                bIsManifold = FALSE;
              }
              else {
                const CRhinoBrepTrim& other_trim = m_T[other_ti];

                BOOL bFlipTrim = trim.m_bRev3d;
                if ( m_F[m_L[trim.m_li].m_fi].m_bRev )
                  bFlipTrim = !bFlipTrim;

                BOOL bFlipOther = other_trim.m_bRev3d;
                if ( m_F[m_L[other_trim.m_li].m_fi].m_bRev )
                  bFlipOther = !bFlipOther;

                if ( bFlipTrim && bFlipOther ) {
                  bIsOriented = FALSE;
                }
                else if ( !bFlipTrim && !bFlipOther ) {
                  bIsOriented = FALSE;
                }
              }
            }
          }
          break;
        case CRhinoBrepTrim::singular:
          // nothing to check here
          break;
        default:
          bIsManifold = FALSE;
          break;
        }
      }
    }
  }
  if ( !bIsManifold ) {
    bIsOriented = FALSE;
    bHasBoundary = FALSE;
  }
  if ( pbIsOriented )
    *pbIsOriented = bIsOriented;
  if ( pbHasBoundary )
    *pbHasBoundary = bHasBoundary;
  return bIsManifold;
}


BOOL
CRhinoBrep::IsSurface() const
{
  // returns TRUE if the b-rep has a single face
  // and that face is geometrically the same
  // as the underlying surface.  I.e., the face
  // has trivial trimming.  In this case, the
  // surface is m_S[0].
  return (m_F.Size() == 1 && FaceIsSurface(0));
}

BOOL 
CRhinoBrep::FaceIsSurface( int face_index ) const
{
  // returns TRUE if the face has a single
  // outer boundary and that boundary runs
  // along the edges of the underlying surface.
  // In this case the geometry of the surface
  // is the same as the geometry of the face.

  BOOL bTrivialFace = FALSE;
  if ( face_index >= 0 && face_index < m_F.Size() ) {
    const CRhinoBrepFace& face = m_F[face_index];
    if ( face.m_li.Size() == 1 ) {
      bTrivialFace = LoopIsSurfaceBoundary( face.m_li[0] );
    }
  }
  return bTrivialFace;
}

BOOL 
CRhinoBrep::LoopIsSurfaceBoundary( int loop_index ) const
{
  // returns TRUE if the loop's trims run along the underlying surface boundary
  BOOL bTrivialLoop = FALSE;
  if ( loop_index >= 0 && loop_index < m_L.Size() ) {
    const CRhinoBrepLoop& loop = m_L[loop_index];
    const int trim_count = loop.m_ti.Size();
    if ( trim_count > 0 ) {
      bTrivialLoop = TRUE;
      int ti;
      for ( ti = 0; ti < trim_count && bTrivialLoop; ti++ ) {
        const CRhinoBrepTrim& trim = m_T[ti];
        if ( trim.m_iso == CRhinoBrepTrim::W_iso )
          continue;
        if ( trim.m_iso == CRhinoBrepTrim::S_iso )
          continue;
        if ( trim.m_iso == CRhinoBrepTrim::N_iso )
          continue;
        if ( trim.m_iso == CRhinoBrepTrim::E_iso )
          continue;
        bTrivialLoop = FALSE;
      }
    }
  }
  return bTrivialLoop;
}

BOOL
CRhinoBrep::FlipReversedSurfaces()
{
  // Clears all CRhinoBrepFace.m_bRev flags
  // by calling SwapFaceParameters() on each
  // face with a TRUE m_bRev.
  //
  // Returns TRUE if successful.

  const int face_count = m_F.Size();

  BOOL rc = TRUE;
  int fi;
  for ( fi = 0; fi < face_count; fi++ ) {
    if ( m_F[fi].m_bRev )
      rc &= SwapFaceParameters( fi );
  }
  return rc;
}


BOOL
CRhinoBrep::SwapFaceParameters( int face_index )
{
  // Returns TRUE if successful

  if ( face_index < 0 || face_index >= m_F.Size() )
    return FALSE;
  CRhinoBrepFace& face = m_F[face_index];

  const int si = face.m_si;
  if ( si < 0 || si >= m_S.Size() )
    return FALSE;
  CRhinoNurbsSurface* pS = m_S[si];
  if ( !pS )
    return FALSE;

  const int face_loop_count = face.m_li.Size();
  if ( face_loop_count < 0 || face_loop_count > m_L.Size() )
    return FALSE;

  // swap surface "u" and "v"
  BOOL rc = pS->SwapParameters();
  if ( !rc )
    return FALSE;

  // reverse trimming loops
  int fli, li;
  for ( fli = 0; fli < face_loop_count; fli++ ) {
    li = face.m_li[fli];
    rc = SwapLoopParameters(li);
    if ( !rc ) {
      // undo any changes
      pS->SwapParameters();
      while ( --fli >= 0 ) {
        li = face.m_li[fli];
        SwapLoopParameters(li);
      }
      return FALSE;
    }
  }

  // toggle m_bRev flag
  face.m_bRev = (face.m_bRev) ? FALSE : TRUE;

  return TRUE;
}

