/* $Header: /Src/RhinoIO/rhinoio_chunk.cpp 7     10/20/99 1:27p Dalelear $ */
/* $NoKeywords: $ */
//
// Copyright (c) 1993-1998 Robert McNeel & Associates. All rights reserved.
// Rhinoceros is a registered trademark of Robert McNeel & Assoicates.
//
// THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
// ALL IMPLIED WARRANTIES OF FITNESS FOR ANY PARTICULAR PURPOSE AND OF
// MERCHANTABILITY ARE HEREBY DISCLAIMED.
//				
////////////////////////////////////////////////////////////////

#include "rhinoio_chunk.h"

BOOL CRhinoChunk::IsShortTypeCode( int typecode )
{
  return (typecode & TCODE_SHORT) ? TRUE : FALSE;
}

BOOL CRhinoChunk::IsLongTypeCode( int typecode )
{
  return (typecode & TCODE_SHORT) ? FALSE : TRUE;
}

BOOL CRhinoChunk::IsCRCTypeCode( int typecode )
{
  // TODO: agree on a bit to set for chunks that have
  //       a CRC.
  BOOL bHasCRC = FALSE;
  switch ( typecode ) {
  case TCODE_AG_ASSEMBLYSTUFF:
  case TCODE_AG_PARTSTUFF:
  case TCODE_AG_SHELLSTUFF:
  case TCODE_AG_FACESTUFF:
  case TCODE_AG_BOUNDARYSTUFF:
  case TCODE_AG_TEDGESTUFF:
  case TCODE_AG_SURFACESTUFF:
  case TCODE_AG_CURVESTUFF:
  case TCODE_AG_SPLINESTUFF:
  case TCODE_AG_POINTSTUFF:
    bHasCRC = TRUE;
    break;
  }
  return bHasCRC;
}

// Write an integer style chunk to file.  This should be overridden for data chunks.
void CRhinoChunk::Write(CRhinoFile &file) const 
{	
  if ( Length() == 0 ) {
    file.BeginWriteChunk( *this );
    file.EndChunk();
  }
	return;
}

CRhinoFile::eStatus 
CRhinoChunk::Read(CRhinoFile& file)
{
  // skip any data for Read that is not overridden
  CRhinoFile::eStatus status = CRhinoFile::good;
  if ( Length() > 0 ) {
    FILE* pFile = file.GetFile();
    m_data_offset = ftell( pFile );
    fseek( pFile, Length(), SEEK_CUR ); 
  }
  return status;
  //return file.EndChunk(); // code that calls BeginReadChunk() should call EndChunk()
}


CRhinoChunk::RegistryEntry* CRhinoChunk::m_registry = NULL;


// Add to the beginning of the registry linked list
int CRhinoChunk::RegisterAdd( 
                   int typecode, 
                   CRhinoChunk* (*CreateNew)(int /*typecode*/, int /*info*/ ) 
                   )
{
  if ( typecode != 0 ) {
    RegistryEntry* pRE = m_registry;
    while ( pRE && pRE->m_typecode != typecode )
      pRE = pRE->m_next;
    if ( !pRE ) {
	    pRE = new RegistryEntry;
	    pRE->m_typecode  = typecode;
	    pRE->m_CreateNew = CreateNew;
	    pRE->m_next      = m_registry;
	    m_registry = pRE;
    }
    else
      typecode = 0;
  }
	return typecode;
}

// Look for a registry entry w/ matching typecode and call
// its CreateNew() function.
CRhinoChunk* 
CRhinoChunk::CreateNew( int typecode, int i )
{
  CRhinoChunk* pChunk = NULL;
	RegistryEntry* pRE = m_registry;
	while( pRE && typecode != pRE->m_typecode ) 
		pRE = pRE->m_next;
  if ( pRE ) {
    pChunk = pRE->m_CreateNew( typecode, i );
  }
  else
		pChunk = new CRhinoChunk( typecode, i );
  if ( pChunk->ChunkStyle() == CRhinoChunk::integer ) {
    pChunk->SetValue(i);
  }
	return pChunk;
}


const char* RhinoTypeCodeString( int tcode )
{
  const char* s = NULL;
  switch( tcode ) {
  case TCODE_RHINO_OBJECT_DATA:
    s = "TCODE_RHINO_OBJECT_DATA"; break;
  case TCODE_RHINO_OBJECT_END:
    s = "TCODE_RHINO_OBJECT_END"; break;
  case TCODE_RHINO_OBJECT_NURBS_CURVE:
    s = "TCODE_RHINO_OBJECT_NURBS_CURVE"; break;
  case TCODE_RHINO_OBJECT_NURBS_SURFACE:
    s = "TCODE_RHINO_OBJECT_NURBS_SURFACE"; break;
  case TCODE_AG_ASSEMBLY:
    s = "TCODE_AG_ASSEMBLY"; break;
  case TCODE_AG_PART:
    s = "TCODE_AG_PART"; break;
  case TCODE_AG_SHELL:
    s = "TCODE_AG_SHELL"; break;
  case TCODE_AG_FACE:
    s = "TCODE_AG_FACE"; break;
  case TCODE_AG_BOUNDARY:
    s = "TCODE_AG_BOUNDARY"; break;
  case TCODE_AG_TEDGE:
    s = "TCODE_AG_TEDGE"; break;
  case TCODE_AG_SURFACE:
    s = "TCODE_AG_SURFACE"; break;
  case TCODE_AG_CURVE:
    s = "TCODE_AG_CURVE"; break;
  case TCODE_AG_SPLINE:
    s = "TCODE_AG_SPLINE"; break;
  case TCODE_AG_POINT:
    s = "TCODE_AG_POINT"; break;
  case TCODE_STUFF:
    s = "TCODE_STUFF"; break;
  case TCODE_AG_ASSEMBLYSTUFF:
    s = "TCODE_AG_ASSEMBLYSTUFF"; break;
  case TCODE_AG_PARTSTUFF:
    s = "TCODE_AG_PARTSTUFF"; break;
  case TCODE_AG_SHELLSTUFF:
    s = "TCODE_AG_SHELLSTUFF"; break;
  case TCODE_AG_FACESTUFF:
    s = "TCODE_AG_FACESTUFF"; break;
  case TCODE_AG_BOUNDARYSTUFF:
    s = "TCODE_AG_BOUNDARYSTUFF"; break;
  case TCODE_AG_TEDGESTUFF:
    s = "TCODE_AG_TEDGESTUFF"; break;
  case TCODE_AG_SURFACESTUFF:
    s = "TCODE_AG_SURFACESTUFF"; break;
  case TCODE_AG_CURVESTUFF:
    s = "TCODE_AG_CURVESTUFF"; break;
  case TCODE_AG_SPLINESTUFF:
    s = "TCODE_AG_SPLINESTUFF"; break;
  case TCODE_AG_POINTSTUFF:
    s = "TCODE_AG_POINTSTUFF"; break;
  case TCODE_RH_POINT:
    s = "TCODE_RH_POINT"; break;
  case TCODE_RH_SPOTLIGHT:
    s = "TCODE_RH_SPOTLIGHT"; break;
  case TCODE_OLD_RH_TRIMESH:
    s = "TCODE_OLD_RH_TRIMESH"; break;
  case TCODE_OLD_MESH_VERTEX_NORMALS:
    s = "TCODE_OLD_MESH_VERTEX_NORMALS"; break;
  case TCODE_OLD_MESH_UV:
    s = "TCODE_OLD_MESH_UV"; break;
  case TCODE_OLD_FULLMESH			:
    s = "TCODE_OLD_FULLMESH			"; break;
  case TCODE_MESH_OBJECT:
    s = "TCODE_MESH_OBJECT"; break;
  case TCODE_COMPRESSED_MESH_GEOMETRY:
    s = "TCODE_COMPRESSED_MESH_GEOMETRY"; break;
  case TCODE_NAME:
    s = "TCODE_NAME"; break;
  case TCODE_VIEW:
    s = "TCODE_VIEW"; break;
  case TCODE_CPLANE:
    s = "TCODE_CPLANE"; break;
  case TCODE_NAMED_CPLANE:
    s = "TCODE_NAMED_CPLANE"; break;
  case TCODE_NAMED_VIEW:
    s = "TCODE_NAMED_VIEW"; break;
  case TCODE_VIEWPORT:
    s = "TCODE_VIEWPORT"; break;
  case TCODE_SHOWGRID:
    s = "TCODE_SHOWGRID"; break;
  case TCODE_SHOWGRIDAXES:
    s = "TCODE_SHOWGRIDAXES"; break;
  case TCODE_SHOWWORLDAXES:
    s = "TCODE_SHOWWORLDAXES"; break;
  case TCODE_VIEWPORT_POSITION:
    s = "TCODE_VIEWPORT_POSITION"; break;
  case TCODE_VIEWPORT_TRACEINFO:
    s = "TCODE_VIEWPORT_TRACEINFO"; break;
  case TCODE_SNAPSIZE			:
    s = "TCODE_SNAPSIZE			"; break;
  case TCODE_NEAR_CLIP_PLANE:
    s = "TCODE_NEAR_CLIP_PLANE"; break;
  case TCODE_HIDE_TRACE:
    s = "TCODE_HIDE_TRACE"; break;
  case TCODE_NOTES:
    s = "TCODE_NOTES"; break;
  case TCODE_LAYERTABLE:
    s = "TCODE_LAYERTABLE"; break;
  case TCODE_LAYERREF:
    s = "TCODE_LAYERREF"; break;
  case TCODE_XDATA:
    s = "TCODE_XDATA"; break;
  case TCODE_RGB:
    s = "TCODE_RGB"; break;
  case TCODE_TEXTUREMAP:
    s = "TCODE_TEXTUREMAP"; break;
  case TCODE_BUMPMAP:
    s = "TCODE_BUMPMAP"; break;
  case TCODE_TRANSPARENCY:
    s = "TCODE_TRANSPARENCY"; break;
  case TCODE_DISP_AM_RESOLUTION:
    s = "TCODE_DISP_AM_RESOLUTION"; break;
  case TCODE_RGBDISPLAY:
    s = "TCODE_RGBDISPLAY"; break;
  case TCODE_LAYER:
    s = "TCODE_LAYER"; break;
  case TCODE_LAYER_OBSELETE_1:
    s = "TCODE_LAYER_OBSELETE_1"; break;
  case TCODE_LAYER_OBSELETE_2:
    s = "TCODE_LAYER_OBSELETE_2"; break;
  case TCODE_LAYER_OBSELETE_3:
    s = "TCODE_LAYER_OBSELETE_3"; break;
  case TCODE_LAYERON:
    s = "TCODE_LAYERON"; break;
  case TCODE_LAYERTHAWED:
    s = "TCODE_LAYERTHAWED"; break;
  case TCODE_LAYERLOCKED:
    s = "TCODE_LAYERLOCKED"; break;
  case TCODE_LAYERVISIBLE:
    s = "TCODE_LAYERVISIBLE"; break;
  case TCODE_LAYERPICKABLE:
    s = "TCODE_LAYERPICKABLE"; break;
  case TCODE_LAYERSNAPABLE:
    s = "TCODE_LAYERSNAPABLE"; break;
  case TCODE_LAYERRENDERABLE:
    s = "TCODE_LAYERRENDERABLE"; break;
  case TCODE_LAYERSTATE:
    s = "TCODE_LAYERSTATE"; break;
  case TCODE_DISP_CPLINES:
    s = "TCODE_DISP_CPLINES"; break;
  case TCODE_DISP_MAXLENGTH:
    s = "TCODE_DISP_MAXLENGTH"; break;
  case TCODE_CURRENTLAYER:
    s = "TCODE_CURRENTLAYER"; break;
  case TCODE_LAYERNAME:
    s = "TCODE_LAYERNAME"; break;
  case TCODE_TOL_AGFIT:
    s = "TCODE_TOL_AGFIT"; break;
  case TCODE_TOL_AGANGLE:
    s = "TCODE_TOL_AGANGLE"; break;
  case TCODE_ENDOFTABLE:
    s = "TCODE_ENDOFTABLE"; break;
  case TCODE_COMMENTBLOCK:
    s = "TCODE_COMMENTBLOCK"; break;
  default:
    s = "unknown typecode";
    break;
  }
  return s;
}

//////////////////////////////////////////////////////////////////////////////////
// Class CRhinoEndOfTable
//////////////////////////////////////////////////////////////////////////////////

RHINO_CHUNK_IMPLEMENT(CRhinoEndOfTable,TCODE_ENDOFTABLE)

///////////////////////////////////////////////////////////////////////////////
//
void RhinoIOCleanup(void)
{
  while( CRhinoChunk::m_registry ) {
    CRhinoChunk::RegistryEntry* p = CRhinoChunk::m_registry;
    CRhinoChunk::m_registry = CRhinoChunk::m_registry->m_next;
    delete p;
  }
}
//
// RhinoIOCleanup() deletes registry entries that are created by
// the calls to CRhinoChunk::RegisterAdd().  CRhinoChunk::RegisterAdd()
// is used to initialize the 
//
//   static int CRhinoBlahBlah::m_CRhinoBlahBlah_typecode
//
// members that are declared in the invocations of the
// RHINO_CHUNK_DECLARE macro.  Each invocation of the
// RHINO_CHUNK_IMPLEMENT macro makes a single call to
// CRhinoChunk::RegisterAdd() and allocates 8 bytes.
//
// Practically speaking, you do not want to call RhinoIOCleanup().
// 1) The Rhino IO toolkit will not be able to read any files after
//    calling RhinoIOCleanup().
// 2) Calling RhinoIOCleanup() will rhfree a most a few hundred bytes.
//
// If you are using leak detecting software and you get reports of
// CRhinoChunk::RegistryEntry classes being created in
// CRhinoChunk::RegisterAdd() and never deleted, then call
// RhinoIOCleanup() at the very end of your program.
//
///////////////////////////////////////////////////////////////////////////////
