/* $Header: /Src/RhinoIO/rhinoio_color.h 3     9/09/98 2:57p Dalelear $ */
/* $NoKeywords: $ */
//
// Copyright (c) 1993-1998 Robert McNeel & Associates. All rights reserved.
// Rhinoceros is a registered trademark of Robert McNeel & Assoicates.
//
// THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
// ALL IMPLIED WARRANTIES OF FITNESS FOR ANY PARTICULAR PURPOSE AND OF
// MERCHANTABILITY ARE HEREBY DISCLAIMED.
//				
////////////////////////////////////////////////////////////////
//
//   Defines CRhinoColor
//
////////////////////////////////////////////////////////////////

#if !defined(RHINO_COLOR_INC_)
#define RHINO_COLOR_INC_

///////////////////////////////////////////////////////////////////////////////
// Class CRhinoColor
//   The RGB components of a color are encoded into an unsigned int, 1 byte per
//   primary color.  A color component of 0 means no light of that color, while
//   the max value of 255 means maximum light of that color.  We use the same
//   format as Microsoft Windows COLORREF data type.  
///////////////////////////////////////////////////////////////////////////////
class CRhinoColor
{
public:
	// Constructors & Conversions -     also default copy and assignment	
	CRhinoColor();              // Default is black
	CRhinoColor(
    int, // red   ( 0 to 255 )
    int, // green ( 0 to 255 )
    int  // blue  ( 0 to 255 )
    );
	CRhinoColor(unsigned int);     // construct from COLORREF

	operator unsigned int() const;	// Conversion to COLORREF

	int Red()   const; // ( 0 to 255 )
	int Green() const; // ( 0 to 255 )
	int Blue()  const; // ( 0 to 255 )

	double FractionRed()   const; // ( 0.0 to 1.0 )
	double FractionGreen() const; // ( 0.0 to 1.0 )
	double FractionBlue()  const; // ( 0.0 to 1.0 )

  void SetRGB(int,int,int); // 0 to 255
  void SetFractionalRGB(double,double,double); // 0.0 to 1.0

private:
	unsigned int m_color;    // Stored in Windows COLORREF format
							 //  0x00bbggrr,  rr= red component 0-255, etc.
};

///////////////////////////////////////////////////////////////////////////////

inline CRhinoColor::CRhinoColor()
  : m_color(0) 
  {}
inline CRhinoColor::CRhinoColor( unsigned int colorref )
  : m_color( colorref&0x00FFFFFF ) 
  {}
inline CRhinoColor::operator unsigned int() const
	{return m_color;}

inline int CRhinoColor::Red() const
			{ return m_color & 255;}
inline int CRhinoColor::Green() const
			{ return (m_color>>8) & 255;}
inline int CRhinoColor::Blue() const
			{ return (m_color>>16) & 255;}
inline double CRhinoColor::FractionRed() const
			{ return Red()/255.0;}
inline double CRhinoColor::FractionGreen() const       
			{ return Green()/255.0;}
inline double CRhinoColor::FractionBlue() const       
			{ return Blue()/255.0;}

#endif
