/* $Header: /Src/RhinoIO/rhinoio_def.h 26    11/10/99 2:56p Dalelear $ */
/* $NoKeywords: $ */
//
// Copyright (c) 1993-1998 Robert McNeel & Associates. All rights reserved.
// Rhinoceros is a registered trademark of Robert McNeel & Assoicates.
//
// THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
// ALL IMPLIED WARRANTIES OF FITNESS FOR ANY PARTICULAR PURPOSE AND OF
// MERCHANTABILITY ARE HEREBY DISCLAIMED.
//				
////////////////////////////////////////////////////////////////
//
//   Includes all system header files required by the Rhino IO
//   toolkit.
//
//   Defines simple typedefs and CRhinoWorkSpace
//
////////////////////////////////////////////////////////////////

#if !defined(RHINO_DEF_INC_)
#define RHINO_DEF_INC_

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
//#include <new.h>

#if defined(__alpha) && !defined(_MSC_VER) && !defined(__GNUG_)

// Alpha UNIX
#define exception
#include <math.h>
#undef exception

#else

// standard math include
#include <math.h>

#endif

#include <float.h>
#include <malloc.h>
#include "rhmem.h"

#include "3dm.h"		// defines all Rhino TCODE_* values

// "standard" defines that may need to be defined
#if !defined(TRUE)
#define TRUE 1
#endif

#if !defined(FALSE)
#define FALSE 0
#endif

#if !defined(BOOL)
typedef int BOOL;
#endif

// This heavy handed treatment of NULL is required to get around the
// old fashioned "(void*)0" NULL defines that cause C++ grief
#if defined(NULL)
#undef NULL
#endif
#define NULL 0

// double precision floating point constants
#define RHINO_PI           3.141592653589793238462643
#define RHINO_SQRT2        1.414213562373095048801689
#define RHINO_SQRT3        1.732050807568877293527446
#define RHINO_LOG2         0.6931471805599453094172321
#define RHINO_LOG10        2.302585092994045684017991

/* IEEE double constants */
#if defined(DBL_MAX)
#define RHINO_DBL_MAX DBL_MAX
#else
#define RHINO_DBL_MAX 1.7976931348623158e+308
#endif

#if defined(DBL_MIN)
#define RHINO_DBL_MIN DBL_MIN
#else
#define RHINO_DBL_MIN 2.22507385850720200e-308
#endif

#if defined(DBL_EPSILON)
#define RHINO_EPSILON DBL_EPSILON
#else
#define RHINO_EPSILON 2.2204460492503131e-16
#endif
#define RHINO_SQRT_EPSILON 1.490116119385000000e-8

//
//  Data type for points and vectors
//
typedef double CRhino3DPoint[3];	// 3d Point object. x,y,z euclidean coordinates
typedef double CRhino2DPoint[2];	// 2d Point object. u,v euclidean coordinates

enum RHINO_POINT_STYLE 
{
  rh_not_rational         = 0,
  rh_homogeneous_rational = 1,
  rh_euclidean_rational   = 2
};

enum RHINO_KNOT_STYLE 
{
  rh_unknown_knots          = 1, // unknown knot style
  rh_non_uniform_knots      = 2, // known to be none of the following
  rh_uniform_knots          = 3, // unform knots (ends not clamped)
  rh_quasi_uniform_knots    = 4, // unform knots (ends clamped)
  rh_piecewise_bezier_knots = 5, // all internal knots have full multiplicity
  rh_bezier_ends            = 6  // clamped knots
};

enum RHINO_CURVE_TYPE
{
  rh_unknown_crv         =  0,
  rh_freeform_crv        =  1, // not any of the following
  rh_polyline_crv        =  2, // not planar
  rh_planar_freeform_crv =  3, // planar but none of the following
  rh_planar_polyline_crv =  4,
  rh_line_crv            =  5,
  rh_circular_crv        =  6,
  rh_elliptic_crv        =  7,
  rh_parabolic_crv       =  8,
  rh_hyperbolic_crv      =  9
};

enum RHINO_SURFACE_TYPE
{
  rh_unknown_srf     = 0,
  rh_freeform_srf    = 1, // not any of the following
  rh_planar_srf      = 2,
  rh_cylindrical_srf = 3, // portion of right circular cylinder
  rh_conical_srf     = 4, // portion of right circular cone
  rh_spherical_srf   = 5, // portion of sphere
  rh_toroidal_srf    = 6, // portion of torus
  rh_revolution_srf  = 7, // portion of surface of revolution
  rh_ruled_srf       = 8  // general ruled surface
};

#include "rhinoio_xform.h"
#include "rhinoio_array.h"

class CRhinoHeader;
class CRhinoChunk;
class CRhinoObject;

class CRhinoColor;
class CRhinoLayer;
class CRhinoLayerPtr;

class CRhinoFile;

class CRhinoIntArray : public CRhinoArray<int>
{
friend class CRhinoFile;
public:
  CRhinoIntArray( size_t = 0 );
  ~CRhinoIntArray();

};

inline CRhinoIntArray::CRhinoIntArray( size_t initial_capacity ) 
  : CRhinoArray<int>(initial_capacity,1)
  {}

inline CRhinoIntArray::~CRhinoIntArray()
  {}
 
// Use CRhinoWorkSpace to efficiently get and automatically
// clean up workspace memory.
class CRhinoWorkSpace
{
public:
  CRhinoWorkSpace();
  ~CRhinoWorkSpace();
  void* GetMemory( size_t );

protected:
  struct MEMBLK {
    struct MEMBLK* next;
  } * m_pMemBlk;

private:
  // prohibit the following with no implementation
  CRhinoWorkSpace( const CRhinoWorkSpace& ); // no copy constructor
  CRhinoWorkSpace& operator=( const CRhinoWorkSpace& ); // no operator=
  void* operator new( size_t );              // no dynamic allocation
};

inline CRhinoWorkSpace::CRhinoWorkSpace()
{
  m_pMemBlk = NULL;
}

const char* RhinoTypeCodeString( int );

// Computes value for superfluous knot used in systems like OpenGL and 3dsMax
double RhinoSuperfluousKnot( 
            int,               // order  ( >= 2 )
            int,               // cv_count
            const double*,     // knot array
            int                // 0 = start, 1 = end
            );

// Used in bounding box computations
BOOL RhinoBoundingBox(
       int,           // dim, 
       BOOL,          // bIsRational,
       int,           // number of points
       int,           // point array stride
       const double*, // points
       double[3],     // min
       double[3],     // max
       BOOL = FALSE   // set to TRUE if box should be grown
       );

// Used to determine when points are equal
BOOL RhinoIsEqual(
       int,           // dim
       const double*, // point A
       const double*  // point B
       );

// Used to determine when a knot vector is periodic
BOOL RhinoIsKnotVectorPeriodic(
       int,           // order
       int,           // cv_count
       const double*  // knot array
       );

// Used to determine when a knot vector is clamped
BOOL RhinoIsKnotVectorClamped(
       int,           // order
       int,           // cv_count
       const double*  // knot array
       );

// Used to determine knot vector style
RHINO_KNOT_STYLE RhinoKnotVectorStyle( 
       int,           // order
       int,           // cv_count
       const double*  // knot array
       );

// Used to get evaluation domain of a knot vector
BOOL RhinoGetKnotVectorDomain(
       int,             // order
       int,             // cv_count
       const double*,   // knot array
       double*, double* // domain returned here
       );

// Used to reverse knot vectors
BOOL RhinoReverseKnotVector(
       int,           // order
       int,           // cv_count
       double*        // knot array
       );

// Used to reverse lists of points
BOOL RhinoReversePointList(
       int,           // dim
       int,           // count
       int,           // stride
       double*        // array of size count*stride
       );

BOOL RhinoGetPerpendicular(
       int,           // dim
       const double*, // vector
       double*        // perpendicular vector
       );

void RhinoCrossProduct( 
       const double*, // 3-D vector
       const double*, // another 3-D vector
       double*        // cross product
       );

double RhinoVectorLength(
         int,           // dim
         const double*  // vector
         );

double RhinoDistance(
         int,           // dim
         const double*, // point
         const double*  // point
         );

// wrapper for case insensitive string compare
// calls one of _stricmp(), stricmp(), or strcasecmp()
// depending on OS.
int Rhino_stricmp( 
        const char*, 
        const char* 
        );

#endif
