/* $Header: /Src/RhinoIO/rhinoio_gl.h 11    11/10/98 4:10p Dalelear $ */
/* $NoKeywords: $ */
//
// Copyright (c) 1993-1998 Robert McNeel & Associates. All rights reserved.
// Rhinoceros is a registered trademark of Robert McNeel & Assoicates.
//
// THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
// ALL IMPLIED WARRANTIES OF FITNESS FOR ANY PARTICULAR PURPOSE AND OF
// MERCHANTABILITY ARE HEREBY DISCLAIMED.
//				
////////////////////////////////////////////////////////////////
//				
//  Definitions of RhinoGL() functions that demonstrate how to
//  use GL to display Rhino IO toolkit objects.			
//				
////////////////////////////////////////////////////////////////
#if defined(_WINDOWS) || defined(WIN32) || defined(_WIN32) || defined(__WIN32__)
// Windows Open GL files require windows.h to be included before the
// Open GL header files.
#include <windows.h>
#endif

#include <GL/gl.h>      // Open GL basic definitions
#include <GL/glu.h>     // Open GL utilities (for GL NURBS stuff)

#if !defined(RHINOIO_GL_INC_)
#define RHINOIO_GL_INC

// include these files before including rhinoio_gl.h
// #include "rhinoio.h" // RhinoIO class definitions
// #include <GL/gl.h>   // Open GL basic definitions
// #include <GL/glu.h>  // Open GL utilities (for GL NURBS stuff)

// Use RhinoGL( const CRhinoPointSet, ...) to render Rhino point sets.
void RhinoGL( 
      const CRhinoPointSet&
      );

// Use RhinoGL( const CRhinoMesh&, ...) to render Rhino meshes.
void RhinoGL( 
      const CRhinoMesh&
      );

// Use RhinoGL( const CRhinoNurbsCurve&,...) in place of
// gluNurbsCurve().  See your system's gluNurbsCurve() documentation
// for details.  In particular, for 3d curves the call to 
// RhinoGL( const CRhinoNurbsCurve&, nobj,...) should appear inside
// of a gluBeginCurve( nobj )/gluEndCurve( nobj ) pair.
// Generally, the GL "type" should be set using the formula
//   CRhinoNurbsCurve:IsRational()
//   ? GL_MAP1_VERTEX_4 
//   : GL_MAP1_VERTEX_3;
void RhinoGL( 
      const CRhinoNurbsCurve&,   // 
      GLUnurbsObj*,              // created with gluNewNurbsRenderer
      GLenum = 0,                // type of curve (if 0, type is automatically set)
      int = 1,           // bPermitKnotScaling - If TRUE, curve knots may
                         // be rescaled to avoid knot vectors GL cannot handle.
      double* = NULL,    // knot_scale[2] - If not NULL and bPermitKnotScaling,
                         // the scaling applied to the knot vector is
                         // returned here.
      double[][4] = NULL // optional transformation applied to curve
      );
void RhinoGL( // low level NURBS curve render
      int, int, int, int, // dim, is_rat, cv_count, order
      const double*,      // knot_vector[]
      int,                // cv_stride
      const double*,      // cv
      GLUnurbsObj*,              // created with gluNewNurbsRenderer
      GLenum = 0,                // type of curve (if 0, type is automatically set)
      int = 1,           // bPermitKnotScaling - If TRUE, curve knots may
                         // be rescaled to avoid knot vectors GL cannot handle.
      double* = NULL,    // knot_scale[2] - If not NULL and bPermitKnotScaling,
                         // the scaling applied to the knot vector is
                         // returned here.
      double[][4] = NULL // optional transformation applied to curve
      );


// Use RhinoGL( const CRhinoNurbsSurface&,...) in place of
// gluNurbsSurface().  See your system's gluNurbsSurface() documentation
// for details.  In particular, the call to 
// RhinoGL( const CRhinoNurbsSurface&, nobj, ...) should appear inside
// of a gluBeginSurface( nobj )/gluEndSurface( nobj ) pair.
// Generally, the GL "type" should be set using the formula
//   CRhinoNurbsSurface:IsRational()
//   ? GL_MAP2_VERTEX_4 
//   : GL_MAP2_VERTEX_3;
void RhinoGL( 
      const CRhinoNurbsSurface&, // 
      GLUnurbsObj*,              // created with gluNewNurbsRenderer
      GLenum = 0,                // type of surface 
                                 // (if 0, type is automatically set)
      int = 1,           // bPermitKnotScaling - If TRUE, surface knots may
                         // be rescaled to avoid knot vectors GL cannot handle.
      double* = NULL,    // knot_scale0[2] - If not NULL and bPermitKnotScaling,
                         // the scaleing applied to the first parameter is
                         // returned here.
      double* = NULL     // knot_scale0[2] - If not NULL and bPermitKnotScaling,
                         // the scaleing applied to the second parameter is
                         // returned here.
      );


// Use RhinoGL( const CRhinoBrep&, int face_index, ... ) to render
// the trimmed NURBS surface that defines a CRhinoBrep face's geometry.
// The call to RhinoGL( const CRhinoBrep&, face_index, nobj ) should
// appear inside of a gluBeginSurface( nobj )/gluEndSurface( nobj )
// pair.  The face index must satisify
// 0 <= face_index < CRhinoBrep.m_F.size()
void RhinoGL( 
      const CRhinoBrep&, // 
      int,               // index of Brep face
      GLUnurbsObj*       // created with gluNewNurbsRenderer
      );

// Use RhinoGL( const CRhinoColor ...) to set GL color to Rhino color
void RhinoGL( const CRhinoColor&,
              GLfloat[4] 
              );
void RhinoGL( const CRhinoColor&,
              double, // alpha
              GLfloat[4] 
              );

// Use RhinoGL( const CRhinoMaterial ...) to set GL material to Rhino material
void RhinoGL( 
      const CRhinoMaterial&
      );

void RhinoGL( 
      const CRhinoMaterial* // pass NULL to get Rhino's default material
      );

// Use RhinoGL( const CRhinoSpotlight, ...) to add Rhino spotlights to
// GL lighting model
void RhinoGL( 
      const CRhinoSpotlight*, // pass NULL to disable the light
      GLenum                  // GL_LIGHTi where 0 <= i <= GL_MAX_LIGHTS
                              // See glLight*() documentation for details
      );
void RhinoGL( 
      const CRhinoSpotlight&,
      GLenum                  // GL_LIGHTi where 0 <= i <= GL_MAX_LIGHTS
                              // See glLight*() documentation for details
      );


// Use RhinoGL( CRhinoViewport& ... ) to set the GL projections to match
// those used in the Rhino viewport.  If the aspect ratio of the GL window
// and CRhinoViewport do not match, the viewport's frustum is adjusted
// to get things back to 1:1.  
//
// These calls load the GL projection matrix.  In the second overload, 
// the matrix loaded into the GL projections and returned.
// 
// For systems where the upper left corner of a window has 
// coordinates (0,0) use:
//   port_left   = 0
//   port_right  = width-1
//   port_bottom = height-1
//   port_top    = 0
void RhinoGL( CRhinoViewport&,
              int, int, // port_left, port_right (port_left != port_right)
              int, int  // port_bottom, port_top (port_bottom != port_top)
              );
void RhinoGL( CRhinoViewport&,
              int, int, // port_left, port_right (port_left != port_right)
              int, int, // port_bottom, port_top (port_bottom != port_top)
              CRhinoXform& // projectionMatrix
              );
// These calls load the GL model view matrix.  In the second overload, 
// the matrix loaded into the GL model view and returned.
void RhinoGL( const CRhinoViewport& );
void RhinoGL( CRhinoViewport&,
              CRhinoXform&  // modelviewMatrix
              );

// Use RhinoGL( order, cv_count, knot, bPermitScaling, glknot )
// to create knot vectors suitable for GL NURBS rendering.
void RhinoGL( const int,     // order,     CRhinoNurbs... order
              const int,     // cv_count,  CRhinoNurbs... cv count
              const double*, // knot,      CRhinoNurbs... knot vector
              GLfloat*,      // glknot[] - GL knot vector
              int = 0,       // bPermitScaling - TRUE if re-scaling is allowed
              double* = NULL // scale[2] - If not NULL and bPermitScaling is TRUE,
                             // then the scaling parameters are returned here.
                             //  ( glknot = (knot = scale[0])*scale[1] )
            );

#endif
