/* $Header: /Src/RhinoIO/rhinoio_layer.h 5     9/27/99 5:10p Dalelear $ */
/* $NoKeywords: $ */
//
// Copyright (c) 1993-1998 Robert McNeel & Associates. All rights reserved.
// Rhinoceros is a registered trademark of Robert McNeel & Assoicates.
//
// THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
// ALL IMPLIED WARRANTIES OF FITNESS FOR ANY PARTICULAR PURPOSE AND OF
// MERCHANTABILITY ARE HEREBY DISCLAIMED.
//				
////////////////////////////////////////////////////////////////
//
//   defines CRhinoLayer
//
////////////////////////////////////////////////////////////////

#if !defined(RHINO_LAYER_INC_)
#define RHINO_LAYER_INC_

///////////////////////////////////////////////////////////////////////////////
// Class CRhinoLayer
//   Layers are used in Rhino to group objects together.  An object can be on
//   a layer.  The state of a layer describes its attributes in Rhino, such as 
//   visibility, and snapability.  The layer color is the color used to display 
//   objects on that layer in Rhino's viewports.  Layers also have an identifying 
//   name.  
//
//   The lifetime of CRhinoLayers are controlled by reference counting. 
//   A layer can be used by many CRhinoObjects and CRhinoFiles and it will not be 
//   deleted untill it is no longer being used.  To create a CRhinoLayer use
//   the static member function CRhinoLayer::New().  This returns a CRhinoLayerPtr
//   which acts like a pointer to the layer and implements reference counting.  
//   The layer will be deleted automatically when all the referring CRhinoLayerPtr's
//	 are deleted.   
///////////////////////////////////////////////////////////////////////////////
class CRhinoLayer : public CRhinoChunk
{
	RHINO_CHUNK_DECLARE(CRhinoLayer)
public:
	enum eLayerState {
         normal  = 0, 
         hidden  = 1, 
         locked  = 2, 
         ghosted = 3
  };		// only 4 possible layer states

public:

  // Use CRhinoLayer::New() to construct new CRhinoLayers
	static CRhinoLayerPtr New(
			const char* = NULL,	         // layer name
			CRhinoColor = CRhinoColor(), // wire frame color ( default = black )
			eLayerState = normal
      );

	void Write( CRhinoFile& ) const;					// Outputs a chunk defining this layer
	CRhinoFile::eStatus Read( CRhinoFile& );	// read chunk data

	const char* Name( void )  const;		// Layer name
	CRhinoColor Color( void ) const;		// Color for wireframe
	eLayerState State( void ) const;		// Layer state

	void SetName( const char* );
	void SetColor( CRhinoColor );
	void SetState( eLayerState );

	BOOL operator==(const CRhinoLayer&) const;	// Comparison tests if names are equal

private:
  // Use CRhinoLayer::New() to construct new CRhinoLayers
	CRhinoLayer(const char*, CRhinoColor ,eLayerState);
	CRhinoLayer();
	~CRhinoLayer();

	char* m_pName;								// Layer Name
	CRhinoColor m_Color;					// Color used for drawing objects on this layer
	eLayerState m_state;          // state

	int  m_RefCount;							// Number of CRhinoLayerPtr objects referring here.

	friend class CRhinoLayerPtr;
};

#endif
