/* $Header: /Src/RhinoIO/rhinoio_layerptr.h 3     9/09/98 2:53p Dalelear $ */
/* $NoKeywords: $ */
//
// Copyright (c) 1993-1998 Robert McNeel & Associates. All rights reserved.
// Rhinoceros is a registered trademark of Robert McNeel & Assoicates.
//
// THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
// ALL IMPLIED WARRANTIES OF FITNESS FOR ANY PARTICULAR PURPOSE AND OF
// MERCHANTABILITY ARE HEREBY DISCLAIMED.
//				
////////////////////////////////////////////////////////////////
//
//   defines CRhinoLayerPtr
//
////////////////////////////////////////////////////////////////

#if !defined(RHINO_LAYERPTR_INC_)
#define RHINO_LAYERPTR_INC_

///////////////////////////////////////////////////////////////////////////////
// Class  CRhinoLayerPtr
//  Pointers to Layers with reference counting and garbage collection.
//  When a CRhinoLayerPtr is destroyed the reference count to the layer
//  is decreased and if the layer has no more references then it is deleted. 
///////////////////////////////////////////////////////////////////////////////
class CRhinoLayerPtr
{
public:
	CRhinoLayerPtr(CRhinoLayer* = NULL);
	CRhinoLayerPtr(const CRhinoLayerPtr&);
	~CRhinoLayerPtr();    

	CRhinoLayerPtr& operator=(const CRhinoLayerPtr&);
	CRhinoLayer& operator*() const;
	operator CRhinoLayer*() const;

  BOOL IsNULL() const;

private:
	CRhinoLayer* m_pPointer;
};

inline CRhinoLayer& CRhinoLayerPtr::operator*() const 
  {return *m_pPointer;}

inline CRhinoLayerPtr::operator CRhinoLayer*() const 
  {return m_pPointer;}

inline BOOL CRhinoLayerPtr::IsNULL() const
	{return (m_pPointer)? FALSE: TRUE;}

typedef CRhinoArray<CRhinoLayerPtr> CRhinoLayerTable;

#endif
