/* $Header: /Src/RhinoIO/rhinoio_notes.cpp 4     10/18/99 10:07a Dalelear $ */
/* $NoKeywords: $ */
//
// Copyright (c) 1993-1999 Robert McNeel & Associates. All rights reserved.
// Rhinoceros is a registered trademark of Robert McNeel & Assoicates.
//
// THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
// ALL IMPLIED WARRANTIES OF FITNESS FOR ANY PARTICULAR PURPOSE AND OF
// MERCHANTABILITY ARE HEREBY DISCLAIMED.
//				
////////////////////////////////////////////////////////////////

#include "rhinoio_object.h"
#include "rhinoio_notes.h"

////////////////////////////////////////////////////////////////
//   Class CRhinoNotes
////////////////////////////////////////////////////////////////
RHINO_CHUNK_IMPLEMENT( CRhinoNotes, TCODE_NOTES)

CRhinoNotes::CRhinoNotes() : CRhinoChunk(TCODE_NOTES),
  m_nVisible( 0),
  m_psz( 0)
{
  size_t i;
  for( i = 0; i < (sizeof( m_rect) / sizeof( m_rect[0])); i++)
    m_rect[i] = 0;
}

CRhinoNotes::~CRhinoNotes()
{
  if( m_psz)
    delete [] m_psz;
}

const char *CRhinoNotes::String() const
{
  return m_psz;
}

void CRhinoNotes::SetString( const char *psz)
{
  if( m_psz)
    delete [] m_psz;

  m_psz = new char[strlen( psz)+1];

  strcpy( m_psz, psz);
}

CRhinoFile::eStatus  
CRhinoNotes::Read(CRhinoFile &file)
{
	CRhinoFile::eStatus status = CRhinoFile::good;

	if( m_psz)
    delete[] m_psz;
  
  m_psz = NULL;

  // Display notes flag
  file.ReadInteger( &m_nVisible);
  // Notes window rect
  file.ReadInteger( 4, m_rect);
  // Read the notes
  file.ReadString( m_psz);

	return status;
}

void CRhinoNotes::Write(CRhinoFile& file) const
{
  file.BeginWriteChunk( TCODE_NOTES);
  {
    file.WriteInteger( m_nVisible);
    file.WriteInteger( 2, m_rect);
    file.WriteString( m_psz);
  }
  file.EndChunk();	// end TCODE_NOTES
}
