/* $Header: /Src/RhinoIO/rhinoio_object.h 7     9/27/99 5:07p Dalelear $ */
/* $NoKeywords: $ */
//
// Copyright (c) 1993-1998 Robert McNeel & Associates. All rights reserved.
// Rhinoceros is a registered trademark of Robert McNeel & Assoicates.
//
// THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
// ALL IMPLIED WARRANTIES OF FITNESS FOR ANY PARTICULAR PURPOSE AND OF
// MERCHANTABILITY ARE HEREBY DISCLAIMED.
//				
////////////////////////////////////////////////////////////////
//
//   defines CRhinoObject
//
////////////////////////////////////////////////////////////////

#if !defined(RHINO_OBJECT_INC_)
#define RHINO_OBJECT_INC_

#include "rhinoio_chunk.h"
#include "rhinoio_color.h"
#include "rhinoio_material.h"
#include "rhinoio_layerindex.h"
#include "rhinoio_layer.h"
#include "rhinoio_name.h"
#include "rhinoio_xdata.h"

///////////////////////////////////////////////////////////////////////////////
// Class  CRhinoObject
//
//  This is an abstract base class for objects stored in a Rhino 3d Model files.
//	Each object has a  TypeCode which identifies its type.  This is the TypeCode
//  of the chunk stored in the file.  Each object can be on a layer and can have 
//  a list of other atributes.
///////////////////////////////////////////////////////////////////////////////
class CRhinoObject : public CRhinoChunk 
{	 
public:
public:
	enum eObjectState {
         normal  = 0, 
         hidden  = 1, 
         locked  = 2
  };		// only 4 possible object states

	// Constructors/ Destructors  - Constructors are protected 
	~CRhinoObject();	
  CRhinoObject& operator=( const CRhinoObject& );
	
  // CRhinoChunk overrides
	BOOL  IsObject( void ) const;
  const char* ClassName( void ) const;

	// Layer information
	CRhinoLayer* Layer( void ) const;  // can return NULL
	void SetLayer( CRhinoLayer* );

	// Rendering material
	CRhinoMaterial* Material( void ) const; // can return NULL
	void SetMaterial( const CRhinoMaterial& );			

  // Wire frame color
  CRhinoColor WireColor( void ) const;
  void SetWireColor( const CRhinoColor& );
  void SetWireColorByLayer( void );  // wire color will be layer color

  // Label
  const char* Label( void ) const; // can return NULL
  void SetLabel( const char* );

  // State ( normal, hidden, locked )
  eObjectState State( void ) const; // can return NULL

  // 3d bounding box 
  virtual BOOL GetBoundingBox( // returns FALSE if object has no box
                 double[3], // min
                 double[3], // max
                 BOOL = FALSE // set to TRUE if box should be grown
                 ) const;

protected:	
	CRhinoObject(
    int = 0, // typecode
    int = 0  // integer value or length of data
    );

	void SetLayer(
    int, // layer index in file's layer table
    const CRhinoFile&
    );
	void WriteLayerIndex(CRhinoFile& ,int) const; // Write a LAYERREF chunk
	void WriteMaterial(CRhinoFile&, const CRhinoMaterial*) const; // Write material if not NULL
										
  void WriteAttributes( // used to write layer index, label,
                                       // material, etc.
    CRhinoFile&, 
    const int                          // end marker typecode
    ) const;
  CRhinoFile::eStatus ReadAttributes(  // used to read layer index, label,
                                       // material, etc.
    CRhinoFile&, 
    const int                          // end marker typecode
    );

private:
	//CRhinoObject(const CRhinoObject&);			// no implementation
	//operator=(const CRhinoObject&);         // no implementation 
  void SetLayerAndState( CRhinoFile&, const CRhinoXdata* );

  CRhinoLayerPtr m_LayPnt;	   // Pointer to objects layer
  CRhinoName*    m_pName;
	CRhinoMaterial* m_pMaterial; // Pointer to material (delete when done)
  unsigned int m_wire_color;   // wire color (default = by layer)
	eObjectState m_state;         // state
  char*        m_sXdataLayerName; // used internally
};

#endif
