/* $Header: /Src/RhinoIO/rhinoio_pointset.h 5     9/09/98 2:48p Dalelear $ */
/* $NoKeywords: $ */
//
// Copyright (c) 1993-1998 Robert McNeel & Associates. All rights reserved.
// Rhinoceros is a registered trademark of Robert McNeel & Assoicates.
//
// THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
// ALL IMPLIED WARRANTIES OF FITNESS FOR ANY PARTICULAR PURPOSE AND OF
// MERCHANTABILITY ARE HEREBY DISCLAIMED.
//				
////////////////////////////////////////////////////////////////
//
//   defines CRhinoPointSet
//
////////////////////////////////////////////////////////////////

#if !defined(RHINO_POINTSET_INC)
#define RHINO_POINTSET_INC_


///////////////////////////////////////////////////////////////////////////////
// Class  CRhinoPointSet
//
//	This object represents a collection of points.   
///////////////////////////////////////////////////////////////////////////////
class CRhinoPointSet: public CRhinoObject 
{
	RHINO_CHUNK_DECLARE( CRhinoPointSet )
public:
  // Construction
	CRhinoPointSet();												// create empty point set 
  ~CRhinoPointSet();

  // CRhinoObject overrides
	CRhinoFile::eStatus Read(CRhinoFile&);	// read chunk data from file
	void Write(CRhinoFile&) const;					// write pointset chunks to file
  BOOL IsValid() const;
  BOOL GetBoundingBox(
          double[3], // min
          double[3], // max
          BOOL = FALSE // set to TRUE if box should be grown
          ) const;

  // Interface
	int PointCount(   // Number of points
          void 
          ) const;
	void SetCapacity(  
          int        // desired capacity of point set
          );
  void SetPointCount( // you shouldn't need to use this function
          int        // point count
          );
  void AddPoint(     // adds point to end of list
          double, double, double    // coordinates of point
          );
  BOOL SetPoint(     // sets the specified point
          int,                      // point index  0 to NumPoints() - 1
          double, double, double    // coordinates of point
          );
  BOOL GetPoint(
          int,                      // point index  0 to NumPoints() - 1
          double&, double&, double& // OUTPUT: coordinates of point
          ) const;
  const double* Point(              // returns pointer to double[3]
          int                       // point index  0 to NumPoints() - 1
          ) const;

protected:
  int m_count;
  int m_stride;    // ( >= 3 ) i-th point is P + m_stride*i
  int m_capacity;  //
  double* m_P;

};

inline int CRhinoPointSet::PointCount( void ) const
  { return m_count; }
inline void CRhinoPointSet::AddPoint( double x, double y, double z )
  {SetPoint( m_count, x, y, z );}
inline const double* CRhinoPointSet::Point( int point_index ) const
  {return (m_P && point_index >= 0 && point_index < m_capacity) 
         ? (m_P + point_index*m_stride) : NULL; }

#endif


