/* $Header: /Src/RhinoIO/rhinoio_spotlight.h 3     10/15/98 8:47a Dalelear $ */
/* $NoKeywords: $ */
//
// Copyright (c) 1993-1998 Robert McNeel & Associates. All rights reserved.
// Rhinoceros is a registered trademark of Robert McNeel & Assoicates.
//
// THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
// ALL IMPLIED WARRANTIES OF FITNESS FOR ANY PARTICULAR PURPOSE AND OF
// MERCHANTABILITY ARE HEREBY DISCLAIMED.
//				
////////////////////////////////////////////////////////////////
//
//   defines CRhinoSpotlight
//
////////////////////////////////////////////////////////////////

#if !defined(RHINO_SPOTLIGHT_INC_)
#define RHINO_SPOTLIGHT_INC_


///////////////////////////////////////////////////////////////////////////////
// Class  CRhinoSpotlight
//
//	This object represents a rendering spot light.
///////////////////////////////////////////////////////////////////////////////
class CRhinoSpotlight: public CRhinoObject 
{
	RHINO_CHUNK_DECLARE( CRhinoSpotlight )
public:
  // Construction
	CRhinoSpotlight();												// create empty spot light
  ~CRhinoSpotlight();

  // CRhinoObject overrides
	CRhinoFile::eStatus Read(CRhinoFile&);	// read chunk data from file
	void Write(CRhinoFile&) const;					// write spot light chunks to file
  BOOL IsValid() const;

  // Interface
  CRhinoColor Ambient() const;
  CRhinoColor Diffuse() const;
  CRhinoColor Specular() const;

  double SpotExponent() const;               // 0 to 1
  double SpotCutoff() const;                 // 0 to 90 or 180 for point source
  void GetDirection( double[3] ) const ;
  void GetDirection( double&, double&, double& ) const;
  void GetPosition( double[3] ) const;
  void GetPosition( double&, double&, double& ) const;

  void SetAmbient(  CRhinoColor );
  void SetDiffuse(  CRhinoColor );
  void SetSpecular( CRhinoColor );

  void SetSpotExponent( double );               // 0 to 1
  void SetSpotCutoff( double );                 // 0 to 90 or 180 for 
  void SetDirection( const double* );
  void SetDirection( double, double, double );
  void SetPosition( const double* );
  void SetPosition( double, double, double );


protected:
  double m_Position[3];
  double m_SpotDirection[3];  
  double m_spot_exponent;      // 0 - 1 (0 = hard 1 = soft)
  double m_spot_cutoff;        // angle in radians (degrees (0-90 or 180)
  CRhinoColor   m_ambient;
  CRhinoColor   m_diffuse;   // always white in Rhino V 1
  CRhinoColor   m_specular;  // always white in Rhino V 1

private:
  double m_r;
  double m_h;
};

#endif


