/* $Header: /Src/RhinoIO/rhinoio_tolerances.cpp 1     3/18/99 11:58a Dalelear $ */
/* $NoKeywords: $ */
//
// Copyright (c) 1993-1998 Robert McNeel & Associates. All rights reserved.
// Rhinoceros is a registered trademark of Robert McNeel & Assoicates.
//
// THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
// ALL IMPLIED WARRANTIES OF FITNESS FOR ANY PARTICULAR PURPOSE AND OF
// MERCHANTABILITY ARE HEREBY DISCLAIMED.
//				
////////////////////////////////////////////////////////////////

#include "rhinoio_def.h"
#include "rhinoio_exception.h"
#include "rhinoio_layerptr.h"
#include "rhinoio_file.h"
#include "rhinoio_chunk.h"
#include "rhinoio_tolerances.h"

////////////////////////////////////////////////////////////////
//   Class CRhinoUnitsAndTolerances
////////////////////////////////////////////////////////////////
RHINO_CHUNK_IMPLEMENT(CRhinoUnitsAndTolerances, TCODE_UNIT_AND_TOLERANCES)

CRhinoUnitsAndTolerances::CRhinoUnitsAndTolerances() : CRhinoChunk(TCODE_UNIT_AND_TOLERANCES),
                             m_abs_tol(0.01),
                             m_rel_tol(0.01),
                             m_ang_tol(0.05235987755983),
                             m_units(CRhinoFile::no_units)
{}


CRhinoUnitsAndTolerances::~CRhinoUnitsAndTolerances()
{}


CRhinoFile::eStatus  
CRhinoUnitsAndTolerances::Read(CRhinoFile& file)
{
	CRhinoFile::eStatus status = CRhinoFile::good;
  int ver, us;
  double x;

  file.ReadInteger(&ver);
  if ( ver == 1 ) {
    file.ReadInteger(&us);
    switch ( us ) {
    case CRhinoFile::no_units:
      m_units = CRhinoFile::no_units;
      break;
    case CRhinoFile::microns:
      m_units = CRhinoFile::microns;
      break;
    case CRhinoFile::millimeters:
      m_units = CRhinoFile::millimeters;
      break;
    case CRhinoFile::centimeters:
      m_units = CRhinoFile::centimeters;
      break;
    case CRhinoFile::meters:
      m_units = CRhinoFile::meters;
      break;
    case CRhinoFile::kilometers:
      m_units = CRhinoFile::kilometers;
      break;
    case CRhinoFile::microinches:
      m_units = CRhinoFile::microinches;
      break;
    case CRhinoFile::mils:
      m_units = CRhinoFile::mils;
      break;
    case CRhinoFile::inches:
      m_units = CRhinoFile::inches;
      break;
    case CRhinoFile::feet:
      m_units = CRhinoFile::feet;
      break;
    case CRhinoFile::miles:
      m_units = CRhinoFile::miles;
      break;
    default:
      m_units = CRhinoFile::no_units;
    }
  
    file.ReadDouble(&x);
    if ( x > 0.0 ) {
      m_abs_tol = x;
    }

    file.ReadDouble(&x);
    if ( x > 0.0 && x < 1.0 ) {
      m_rel_tol = x;
    }

    file.ReadDouble(&x);
    if ( x > 0.0 && x < 3.14159 ) {
      m_ang_tol = x;
    }
  }
  else {
    file.SkipChunk();
  }

  return status;
}

void 
CRhinoUnitsAndTolerances::Write(CRhinoFile& file) const
{
  const int ver = 1;
  const int us = (int)m_units;
  file.BeginWriteChunk( TCODE_UNIT_AND_TOLERANCES );
  file.WriteInteger( ver );
  file.WriteInteger( us );
  file.WriteDouble( m_abs_tol );
  file.WriteDouble( m_rel_tol );
  file.WriteDouble( m_ang_tol );
  file.EndChunk();
}

