/* $Header: /Src/RhinoIO/rhinoio_viewport.cpp 26    1/29/99 2:14p Dalelear $ */
/* $NoKeywords: $ */
//
// Copyright (c) 1993-1998 Robert McNeel & Associates. All rights reserved.
// Rhinoceros is a registered trademark of Robert McNeel & Assoicates.
//
// THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
// ALL IMPLIED WARRANTIES OF FITNESS FOR ANY PARTICULAR PURPOSE AND OF
// MERCHANTABILITY ARE HEREBY DISCLAIMED.
//				
////////////////////////////////////////////////////////////////

#include "rhinoio_object.h"
#include "rhinoio_xform.h"
#include "rhinoio_viewport.h"

RHINO_CHUNK_IMPLEMENT(CRhinoViewport, TCODE_VIEWPORT)

#define DEGREES_TO_RADIANS 3.141592653589793238462643/180.0
#define MIN_NEAR_DIST 0.001

CRhinoViewport::CRhinoViewport() : CRhinoObject( TCODE_VIEWPORT ),
                m_bValidCamera( FALSE ),
                m_bValidFrustum ( FALSE ),
                m_bValidPort( FALSE ),
                m_projection( CRhinoViewport::parallel_proj ),
                m_frus_left(-1.0),
                m_frus_right(1.0),
                m_frus_bottom(-1.0),
                m_frus_top(1.0),
                m_frus_near(MIN_NEAR_DIST),
                m_frus_far(1000.0),
                m_port_left(0),
                m_port_right(1000),
                m_port_bottom(0),
                m_port_top(1000),
                m_port_near(0xffff),
                m_port_far(0),
                m_target_distance(100.0),
                m_bShowGrid( TRUE ),
                m_bShowGridAxes( TRUE ),
                m_bShowWorldAxes( TRUE ),
                m_sViewName(NULL)
{
  m_CamLoc[0] =    0.0;
  m_CamLoc[1] =    0.0;
  m_CamLoc[2] = -100.0;
  m_CamLoc[3] =    0.0;

  m_CamDir[0] =  0.0;
  m_CamDir[1] =  0.0;
  m_CamDir[2] = -1.0;

  m_CamUp[0] = 0.0;
  m_CamUp[1] = 1.0;
  m_CamUp[2] = 0.0;

  m_CamX[0] = 1.0;
  m_CamX[1] = 0.0;
  m_CamX[2] = 0.0;

  m_CamY[0] = 0.0;
  m_CamY[1] = 1.0;
  m_CamY[2] = 0.0;

  m_CamZ[0] = 0.0;
  m_CamZ[1] = 0.0;
  m_CamZ[2] = 1.0;

  m_target_distance = 0.0;

  m_cp_origin[0] = 0.0;
  m_cp_origin[1] = 0.0;
  m_cp_origin[2] = 0.0;

  m_cp_X[0] = 1.0;
  m_cp_X[1] = 0.0;
  m_cp_X[2] = 0.0;

  m_cp_Y[0] = 0.0;
  m_cp_Y[1] = 1.0;
  m_cp_Y[2] = 0.0;

  m_cp_Z[0] = 0.0;
  m_cp_Z[1] = 0.0;
  m_cp_Z[2] = 1.0;

  m_client_x[0] = 0.0;
  m_client_x[0] = 1.0;

  m_client_y[0] = 0.0;
  m_client_y[0] = 1.0;
}

CRhinoViewport::~CRhinoViewport()
{
  if ( m_sViewName ) {
    delete m_sViewName;
    m_sViewName = NULL;
  }
}

CRhinoViewport& CRhinoViewport::operator=( const CRhinoViewport& src )
{
  if ( this != &src ) {
    m_bValidCamera   = src.m_bValidCamera;
    m_bValidFrustum = src.m_bValidFrustum;
    m_bValidPort     = src.m_bValidPort;

    m_projection     = src.m_projection;

    memcpy( &m_CamLoc[0], &src.m_CamLoc[0], sizeof(m_CamLoc) );
    memcpy( &m_CamDir[0], &src.m_CamDir[0], sizeof(m_CamDir));
    memcpy( &m_CamUp[0],  &src.m_CamUp[0],  sizeof(m_CamUp));

    memcpy( &m_CamX[0], &src.m_CamX[0], sizeof(m_CamX));
    memcpy( &m_CamY[0], &src.m_CamY[0], sizeof(m_CamY));
    memcpy( &m_CamZ[0], &src.m_CamZ[0], sizeof(m_CamZ));

    m_frus_left      = src.m_frus_left;
    m_frus_right     = src.m_frus_right;
    m_frus_bottom    = src.m_frus_bottom;
    m_frus_top       = src.m_frus_top;
    m_frus_near      = src.m_frus_near;
    m_frus_far       = src.m_frus_far;

    m_port_left      = src.m_port_left;
    m_port_right     = src.m_port_right;
    m_port_bottom    = src.m_port_bottom;
    m_port_top       = src.m_port_top;
    m_port_near      = src.m_port_near;
    m_port_far       = src.m_port_far;

    m_target_distance = src.m_target_distance;

    memcpy( &m_cp_origin[0], &src.m_cp_origin, sizeof(m_cp_origin));
    memcpy( &m_cp_X[0], &src.m_cp_X, sizeof(m_cp_X) );
    memcpy( &m_cp_Y[0], &src.m_cp_Y, sizeof(m_cp_Y) );
    memcpy( &m_cp_Z[0], &src.m_cp_Z, sizeof(m_cp_Z) );

    memcpy( &m_client_x[0], &src.m_client_x, sizeof(m_client_x) );
    memcpy( &m_client_y[0], &src.m_client_y, sizeof(m_client_y) );

    m_bShowGrid      = src.m_bShowGrid;
    m_bShowGridAxes  = src.m_bShowGridAxes;
    m_bShowWorldAxes = src.m_bShowWorldAxes;

    SetViewName( src.ViewName() );
  }
  return *this;
}


CRhinoFile::eStatus CRhinoViewport::Read( CRhinoFile& file )
{
  double target[3] = {0.0,0.0,0.0};
  double angle[3] = {0.0,0.0,0.0};
  double viewsize = 0.0;
  double camdist = 0.0;
  double near_clip = 0.0;
  const int end_mark_typecode = TCODE_ENDOFTABLE;

  m_bValidCamera = FALSE;
  m_bValidFrustum = FALSE;
  m_bValidPort = FALSE;
  SetViewName(NULL);

  m_client_x[0] = 0.0;
  m_client_x[1] = 1.0;
  m_client_y[0] = 0.0;
  m_client_y[1] = 1.0;

  unsigned int typecode;
  int info;
  CRhinoChunk* pChunk = NULL;
  CRhinoFile::eStatus read_status, end_status;
  CRhinoFile::eStatus status = CRhinoFile::good;

  BOOL bContinue = TRUE;
  while (bContinue) {
    pChunk = NULL;
    typecode = 0;
    info = 0;
    read_status = end_status = CRhinoFile::good;
	  status = file.BeginReadChunk( typecode, info );

    if ( status != CRhinoFile::good ) {
      bContinue = FALSE;
    }
    else {
      // if the typecode is in the CRhinoChunk registry, then the
      // call to CreateNew() will create the appropriate CRhinoChunk
      // child and read its defining data.
      switch( typecode ) {

      case TCODE_NAME:
        file.ReadString( m_sViewName );
        break;

      case TCODE_VIEW:
        {
          int i;
          file.ReadInteger( &i );
          m_projection = (i==2) ? perspective_proj : parallel_proj;
          file.ReadInteger( &i );
          file.ReadDouble( 3, target );
          file.ReadDouble( 3, angle );
          file.ReadDouble( &viewsize );
          file.ReadDouble( &camdist );

          // turn this soup into bonafide viewport information
          FromRhino( m_projection, target, 
                     angle[0], angle[1], angle[2], viewsize, camdist 
                     );
          /*
          {
            double aa[3], e[3];
            GetAngles( &aa[0], &aa[1], &aa[2] );
            e[0] = angle[0]-aa[0];
            e[1] = angle[1]-aa[1];
            e[2] = angle[2]-aa[2];
            if ( fabs(e[0]) > RHINO_SQRT_EPSILON || fabs(e[1]) > RHINO_SQRT_EPSILON || fabs(e[2]) > RHINO_SQRT_EPSILON ) {
              GetAngles( &aa[0], &aa[1], &aa[2] ); // for debugging break here
            }
          }
          */
        }
        break;

      case TCODE_CPLANE:
        {
          int i;
          double d, o[3], x[3], y[3];
          file.ReadDouble( 3, o );
          file.ReadDouble( 3, x );
          file.ReadDouble( 3, y );
          file.ReadDouble( &d );   // gridsize
          file.ReadInteger( &i );  // grid sections
          file.ReadInteger( &i );  // grid thick sections
          SetConstructionPlane( o, x, y );
        }
        break;

      case TCODE_VIEWPORT_POSITION:
        {
          double da[4];
          file.ReadDouble( 4, da );
          m_client_x[0] = da[0];
          m_client_x[1] = da[2];
          m_client_y[0] = 1.0-da[3];
          m_client_y[1] = 1.0-da[1];
        }
        break;

		  case TCODE_NEAR_CLIP_PLANE:
        file.ReadDouble( &near_clip );
			  break;

      case TCODE_SHOWGRID:
        m_bShowGrid = (info!=0) ? TRUE : FALSE;
			  break;

      case TCODE_SHOWGRIDAXES:
        m_bShowGridAxes = (info!=0) ? TRUE : FALSE;
			  break;

      case TCODE_SHOWWORLDAXES:
        m_bShowWorldAxes = (info!=0) ? TRUE : FALSE;
			  break;

      default:
	      pChunk = CRhinoChunk::CreateNew( typecode, info );
        if ( pChunk )
 	        read_status = pChunk->Read(file );  // read in chunk information but not crc
        else
          bContinue = FALSE;
        break;
      }

	    end_status = file.EndChunk();
      if ( status == CRhinoFile::good ) {
        status = read_status;
        if ( status == CRhinoFile::good ) {
          status = end_status;
        }
      }
      if ( pChunk ) {
  	    bContinue = ( pChunk->TypeCode() != end_mark_typecode );
        delete pChunk;
        pChunk = NULL;
      }
    }
  }

  if ( status == CRhinoFile::good ) {
    // expand frustum based on optional near_clip distance
    if ( near_clip > 0.0 && near_clip < m_frus_near ) {
      if ( MIN_NEAR_DIST < m_frus_near && near_clip < MIN_NEAR_DIST )
        near_clip = MIN_NEAR_DIST;
      SetFrustumNearFar( near_clip, m_frus_far );
    }
  }

  return status;
}


void CRhinoViewport::Write( CRhinoFile& file ) const
{
  double a1 = 0.0, a2 = 0.0, a3 = 0.0;
  double target_point[3] = {0.0,0.0,0.0};
  if ( !m_bValidCamera || !m_bValidFrustum )
    return;

	file.BeginWriteChunk( *this );

  if ( m_sViewName && m_sViewName[0] ) {
    file.BeginWriteChunk( TCODE_NAME );
    file.WriteString( m_sViewName );
    file.EndChunk();
  }

  {
    file.BeginWriteChunk( TCODE_VIEW );
    file.WriteInteger( ( m_projection == perspective_proj ) ? 2 : 1 );
    file.WriteInteger( 0 );
    GetTargetPoint( target_point );
    file.WriteDouble( 3, target_point );
    GetAngles( &a1, &a2, &a3 );
    file.WriteDouble( a1 );
    file.WriteDouble( a2 );
    file.WriteDouble( a3 );
    file.WriteDouble( ViewSize() );
    file.WriteDouble( m_target_distance );
    file.EndChunk();
  }

  {
    file.BeginWriteChunk( TCODE_CPLANE );
    file.WriteDouble( 3, m_cp_origin );
    file.WriteDouble( 3, m_cp_X );
    file.WriteDouble( 3, m_cp_Y );
    file.WriteDouble( 1.0 );   // gridsize
    file.WriteInteger( 100 ); // grid sections
    file.WriteInteger( 5 );   // grid thick sections
    file.EndChunk();
  }

  {
  {
    double da[4];
    da[0] = m_client_x[0];
    da[1] = 1.0-m_client_y[1];
    da[2] = m_client_x[1];
    da[3] = 1.0-m_client_y[0];
    // hack to get around bug in Rhino v 1.0 and thirds
    da[0] = ceil(da[0]*100.0)/100.0;
    da[2] = floor(da[2]*100.0)/100.0;
    da[1] = ceil(da[1]*100.0)/100.0;
    da[3] = floor(da[3]*100.0)/100.0;
    file.BeginWriteChunk( TCODE_VIEWPORT_POSITION );
    file.WriteDouble( 4, da );
    file.EndChunk();
  }

  }

  {
    file.BeginWriteChunk( TCODE_NEAR_CLIP_PLANE );
    file.WriteDouble( m_frus_near );
    file.EndChunk();
  }

  {
    file.BeginWriteChunk( TCODE_ENDOFTABLE );
    file.EndChunk();
  }

  file.EndChunk();
}

BOOL CRhinoViewport::IsValid() const
{
  return ( m_bValidCamera && m_bValidFrustum && m_bValidPort );
}

BOOL CRhinoViewport::SetCamera(
    const double* CameraLocation,
    const double* CameraDirection,
    const double* CameraUp
    )
{
  double a[3][3], d;

  m_bValidCamera = FALSE;

  if ( !CameraLocation ) {
    a[0][0] =   0.0;
    a[0][1] =   0.0;
    a[0][2] = 100.0;
    CameraLocation = a[0];
  }
  if ( !CameraDirection ) {
    if ( CameraUp && fabs(CameraUp[2]) > fabs(CameraUp[1]) ) {
      a[1][0] =  0.0;
      a[1][1] =  1.0;
      a[1][2] =  0.0;
    }
    else {
      a[1][0] =  0.0;
      a[1][1] =  0.0;
      a[1][2] = -1.0;
    }
    CameraDirection = a[1];
  }
  if ( !CameraUp ) {
    if ( fabs(CameraDirection[1]) > fabs(CameraDirection[2]) ) {
      a[1][0] =  0.0;
      a[1][1] =  0.0;
      a[1][2] =  1.0;
    }
    else {
      a[2][0] = 0.0;
      a[2][1] = 1.0;
      a[2][2] = 0.0;
    }
    CameraUp = a[2];
  }

  m_CamLoc[0] = CameraLocation[0];
  m_CamLoc[1] = CameraLocation[1];
  m_CamLoc[2] = CameraLocation[2];
  m_CamLoc[3] = (m_projection == perspective_proj) ? 1.0 : 0.0;

  m_CamDir[0] = CameraDirection[0];
  m_CamDir[1] = CameraDirection[1];
  m_CamDir[2] = CameraDirection[2];

  m_CamUp[0] = CameraUp[0];
  m_CamUp[1] = CameraUp[1];
  m_CamUp[2] = CameraUp[2];

  d = RhinoVectorLength( 3, m_CamDir );
  if ( d == 0.0 )
    return FALSE;
  d = -1.0/d;
  m_CamZ[0] = d*m_CamDir[0];
  m_CamZ[1] = d*m_CamDir[1];
  m_CamZ[2] = d*m_CamDir[2];

  d = m_CamUp[0]*m_CamZ[0] + m_CamUp[1]*m_CamZ[1] + m_CamUp[2]*m_CamZ[2];
  m_CamY[0] = m_CamUp[0] - d*m_CamZ[0];
  m_CamY[1] = m_CamUp[1] - d*m_CamZ[1];
  m_CamY[2] = m_CamUp[2] - d*m_CamZ[2];
  d = RhinoVectorLength( 3, m_CamY );
  if ( d == 0.0 )
    return FALSE;
  d = 1.0/d;
  m_CamY[0] = d*m_CamY[0];
  m_CamY[1] = d*m_CamY[1];
  m_CamY[2] = d*m_CamY[2];


  RhinoCrossProduct( m_CamY, m_CamZ, m_CamX );

  // Gaurd against numerical garbage resulting from nearly parallel 
  // and/or ultra short short dir and up.
  d = m_CamY[0]*m_CamZ[0] + m_CamY[1]*m_CamZ[1] + m_CamY[2]*m_CamZ[2];
  if ( fabs(d) > 1.0e-6 )
    return FALSE;
  d = RhinoVectorLength( 3, m_CamX );
  if ( fabs(1.0-d) > 1.0e-6 )
    return FALSE;
  d = m_CamY[0]*m_CamX[0] + m_CamY[1]*m_CamX[1] + m_CamY[2]*m_CamX[2];
  if ( fabs(d) > 1.0e-6 )
    return FALSE;
  d = m_CamZ[0]*m_CamX[0] + m_CamZ[1]*m_CamX[1] + m_CamZ[2]*m_CamX[2];
  if ( fabs(d) > 1.0e-6 )
    return FALSE;

  m_bValidCamera = TRUE;
  return m_bValidCamera;
}

BOOL CRhinoViewport::DollyCamera( const double* dollyVector )
{
  BOOL rc = FALSE;
  if ( m_bValidCamera && dollyVector ) {
    m_CamLoc[0] += dollyVector[0];
    m_CamLoc[1] += dollyVector[1];
    m_CamLoc[2] += dollyVector[2];
    rc = TRUE;
  }
  return rc;
}

BOOL CRhinoViewport::DollyTarget( double dollyDistance )
{
  BOOL rc = FALSE;
  if ( m_target_distance + dollyDistance > 0.0 ) {
    m_target_distance += dollyDistance;
    rc = TRUE;
  }
  return rc;
}

BOOL CRhinoViewport::DollyFrustum( double dollyDistance )
{
  BOOL rc = FALSE;
  double new_near, new_far, scale;
  if ( m_bValidFrustum ) {
    new_near = m_frus_near + dollyDistance;
    new_far  = m_frus_far + dollyDistance;
    scale = ( m_projection == perspective_proj ) ? new_near/m_frus_near : 1.0;
    if ( new_near > 0.0 && new_far > new_near && scale > 0.0 ) {
      m_frus_near = new_near;
      m_frus_far  = new_far;
      m_frus_left   *= scale;
      m_frus_right  *= scale;
      m_frus_top    *= scale;
      m_frus_bottom *= scale;
      rc = TRUE;
    }
  }
  return rc;
}

BOOL CRhinoViewport::GetCamera(
    double* CameraLocation,
    double* CameraDirection,
    double* CameraUp
    ) const
{
  if ( CameraLocation ) {
    CameraLocation[0] = m_CamLoc[0];
    CameraLocation[1] = m_CamLoc[1];
    CameraLocation[2] = m_CamLoc[2];
  }
  if ( CameraDirection ) {
    CameraDirection[0] = m_CamDir[0];
    CameraDirection[1] = m_CamDir[1];
    CameraDirection[2] = m_CamDir[2];
  }
  if ( CameraUp ) {
    CameraUp[0] = m_CamUp[0];
    CameraUp[1] = m_CamUp[1];
    CameraUp[2] = m_CamUp[2];
  }
  return m_bValidCamera;
}

BOOL CRhinoViewport::GetCameraFrame(
    double* CameraLocation,
    double* CameraX,
    double* CameraY,
    double* CameraZ
    ) const
{
  if ( CameraLocation ) {
    CameraLocation[0] = m_CamLoc[0];
    CameraLocation[1] = m_CamLoc[1];
    CameraLocation[2] = m_CamLoc[2];
  }
  if ( CameraX ) {
    CameraX[0] = m_CamX[0];
    CameraX[1] = m_CamX[1];
    CameraX[2] = m_CamX[2];
  }
  if ( CameraY ) {
    CameraY[0] = m_CamY[0];
    CameraY[1] = m_CamY[1];
    CameraY[2] = m_CamY[2];
  }
  if ( CameraZ ) {
    CameraZ[0] = m_CamZ[0];
    CameraZ[1] = m_CamZ[1];
    CameraZ[2] = m_CamZ[2];
  }
  return m_bValidCamera;
}


BOOL CRhinoViewport::IsCameraFrameWorldPlan( 
      int* xindex,
      int* yindex,
      int* zindex
      )
{
  int i;
  int ix = 0;
  int iy = 0;
  int iz = 0;
  double X[3], Y[3], Z[3];
  BOOL rc = GetCameraFrame( NULL, X, Y, Z );
  if ( rc ) {
    for ( i = 0; i < 3; i++ ) {
      if ( X[i] == 1.0 ) {
        ix = i+1;
        break;
      }
      if ( X[i] == -1.0 ) {
        ix = -(i+1);
        break;
      }
    }
    for ( i = 0; i < 3; i++ ) {
      if ( Y[i] == 1.0 ) {
        iy = i+1;
        break;
      }
      if ( Y[i] == -1.0 ) {
        iy = -(i+1);
        break;
      }
    }
    for ( i = 0; i < 3; i++ ) {
      if ( Z[i] == 1.0 ) {
        iz = i+1;
        break;
      }
      if ( Z[i] == -1.0 ) {
        iz = -(i+1);
        break;
      }
    }
    rc = ( iz != 0 ) ? 1 : 0;
  }
  if ( xindex ) *xindex = ix;
  if ( yindex ) *yindex = iy;
  if ( zindex ) *zindex = iz;
  return rc;
}


BOOL CRhinoViewport::GetCameraExtents( 
    // returns bounding box in camera coordinates - this is useful information
    // for setting view frustrums to include the point list
    int count,            // count = number of 3d points
    int stride,           // stride = number of doubles to skip between points (>=3)
    const double* points, // 3d points in world coordinates
    double cboxmin[3],    // minimum corner of bounding box in camera coordinates
    double cboxmax[3],    // maximum corner of bounding box in camera coordinates
    BOOL bGrowBox         // set to TRUE to grow existing box
    ) const
{
  BOOL rc = bGrowBox;
  double box[2][3], c0[3], cX[3], cY[3], cZ[3], dx, dy, dz, x, y, z;
  int i;
  if ( count > 0 && stride >= 3 && points != NULL ) {
    rc = FALSE;
    if ( GetCameraFrame( c0, cX, cY, cZ ) ) {
      for ( i = 0; i < count; i++, points += stride ) {
        dx = (points[0] - c0[0]);
        dy = (points[1] - c0[1]);
        dz = (points[2] - c0[2]);
        x = cX[0]*dx + cX[1]*dy + cX[2]*dz;
        y = cY[0]*dx + cY[1]*dy + cY[2]*dz;
        z = cZ[0]*dx + cZ[1]*dy + cZ[2]*dz;
        if ( i ) {
          if ( x < box[0][0] ) box[0][0] = x; else if ( x > box[1][0] ) box[1][0] = x;
          if ( y < box[0][1] ) box[0][1] = y; else if ( y > box[1][1] ) box[1][1] = y;
          if ( z < box[0][2] ) box[0][2] = z; else if ( z > box[1][2] ) box[1][2] = z;
        }
        else {
          box[0][0] = box[1][0] = x;
          box[0][1] = box[1][1] = y;
          box[0][2] = box[1][2] = z;
        }
      }
      rc = TRUE;
      if ( bGrowBox ) {
        if ( cboxmin ) {
          if ( box[0][0] < cboxmin[0] ) cboxmin[0] = box[0][0];
          if ( box[0][1] < cboxmin[1] ) cboxmin[1] = box[0][1];
          if ( box[0][2] < cboxmin[2] ) cboxmin[2] = box[0][2];
        }
        if ( cboxmax ) {
          if ( box[1][0] > cboxmax[0] ) cboxmax[0] = box[1][0];
          if ( box[1][1] > cboxmax[1] ) cboxmax[1] = box[1][1];
          if ( box[1][2] > cboxmax[2] ) cboxmax[2] = box[1][2];
        }
      }
      else {
        if ( cboxmin ) {
          cboxmin[0] = box[0][0];
          cboxmin[1] = box[0][1];
          cboxmin[2] = box[0][2];
        }
        if ( cboxmax ) {
          cboxmax[0] = box[1][0];
          cboxmax[1] = box[1][1];
          cboxmax[2] = box[1][2];
        }
      }
    }
  }
  return rc;
}

BOOL CRhinoViewport::GetCameraExtents( 
    // returns bounding box in camera coordinates - this is useful information
    // for setting view frustrums to include the point list
    const double* worldBoxMin,
    const double* worldBoxMax,
    double cboxmin[3],    // minimum corner of bounding box in camera coordinates
    double cboxmax[3],    // maximum corner of bounding box in camera coordinates
    BOOL bGrowBox         // set to TRUE to grow existing box
    ) const
{
  BOOL rc = FALSE;
  int i, j, k, n;
  double p[8][3];
  const double* box[2];
  if ( worldBoxMin && worldBoxMax ) {
    box[0] = worldBoxMin;
    box[1] = worldBoxMax;
    n = 0;
    for ( i = 0; i < 2; i++ ) {
      for ( j = 0; j < 2; j++ ) {
        for ( k = 0; k < 2; k++ ) {
          p[n][0] = box[i][0];
          p[n][1] = box[j][1];
          p[n][2] = box[k][2];
          n++;
        }
      }
    }
    rc = GetCameraExtents( 8, 3, &p[0][0], cboxmin, cboxmax, bGrowBox );
  }
  return rc;
}

BOOL CRhinoViewport::GetCameraExtents( 
    // returns bounding box in camera coordinates - this is useful information
    // for setting view frustrums to include the point list
    const double* worldSphereCenter,
    double worldSphereRadius,
    double cboxmin[3],    // minimum corner of bounding box in camera coordinates
    double cboxmax[3],    // maximum corner of bounding box in camera coordinates
    BOOL bGrowBox         // set to TRUE to grow existing box
    ) const
{
  BOOL rc = FALSE;
  double box[2][3], r;
  if ( worldSphereCenter) {
    r = fabs( worldSphereRadius );
    BOOL rc = GetCameraExtents( 1, 3, worldSphereCenter, box[0], box[1], FALSE );
    if ( rc ) {
      box[0][0] -= r;
      box[0][1] -= r;
      box[0][2] -= r;
      box[1][0] += r;
      box[1][0] += r;
      box[1][0] += r;
      if ( bGrowBox ) {
        if ( cboxmin ) {
          if ( box[0][0] < cboxmin[0] ) cboxmin[0] = box[0][0];
          if ( box[0][1] < cboxmin[1] ) cboxmin[1] = box[0][1];
          if ( box[0][2] < cboxmin[2] ) cboxmin[2] = box[0][2];
        }
        if ( cboxmax ) {
          if ( box[1][0] > cboxmax[0] ) cboxmax[0] = box[1][0];
          if ( box[1][1] > cboxmax[1] ) cboxmax[1] = box[1][1];
          if ( box[1][2] > cboxmax[2] ) cboxmax[2] = box[1][2];
        }
      }
      else {
        if ( cboxmin ) {
          cboxmin[0] = box[0][0];
          cboxmin[1] = box[0][1];
          cboxmin[2] = box[0][2];
        }
        if ( cboxmax ) {
          cboxmax[0] = box[1][0];
          cboxmax[1] = box[1][1];
          cboxmax[2] = box[1][2];
        }
      }
    }
  }
  return rc;
}


BOOL CRhinoViewport::SetProjection( eProjection projection )
{
  BOOL rc = FALSE;
  if ( projection == perspective_proj ) {
    rc = TRUE;
    m_projection = perspective_proj;
    m_CamLoc[3] = 1.0;
  }
  else {
    rc = (projection == parallel_proj);
    m_projection = parallel_proj;
    m_CamLoc[3] = 0.0;
  }
  return rc;
}

CRhinoViewport::eProjection CRhinoViewport::Projection() const
{
  return m_projection;
}

BOOL CRhinoViewport::SetFrustum(
      double frus_left,
      double frus_right,
      double frus_bottom,
      double frus_top,
      double frus_near,
      double frus_far
      )
{
  m_bValidFrustum = FALSE;
  if (  frus_left < frus_right && frus_bottom < frus_top 
      && 0.0 < m_frus_near && m_frus_near < m_frus_far ) {
    m_frus_left   = frus_left;
    m_frus_right  = frus_right;
    m_frus_bottom = frus_bottom;
    m_frus_top    = frus_top;
    m_frus_near   = frus_near;
    m_frus_far    = frus_far;
    m_bValidFrustum = TRUE;
    if ( m_target_distance < m_frus_near || m_target_distance > m_frus_far )
      m_target_distance = 0.5*(m_frus_near+m_frus_far);
  }
  return m_bValidFrustum;
}


BOOL CRhinoViewport::GetFrustum(
      double* frus_left,
      double* frus_right,
      double* frus_bottom,
      double* frus_top,
      double* frus_near,   // = NULL
      double* frus_far     // = NULL
      ) const
{
  if ( frus_left )
    *frus_left = m_frus_left;
  if ( frus_right )
    *frus_right = m_frus_right;
  if ( frus_bottom )
    *frus_bottom = m_frus_bottom;
  if ( frus_top )
    *frus_top = m_frus_top;
  if ( frus_near )
    *frus_near = m_frus_near;
  if ( frus_far )
    *frus_far = m_frus_far;
  return m_bValidFrustum;
}

BOOL CRhinoViewport::GetFrustumAspect( double& frustum_aspect ) const
{
  // frustum_aspect = frustum width / frustum height
  double w, h, left, right, bot, top;
  int rc = m_bValidFrustum;
  frustum_aspect = 0.0;

  if ( GetFrustum( &left, &right, &bot, &top ) ) {
    w = right - left;
    h = top - bot;
    if ( h == 0.0 )
      rc = FALSE;
    else
      frustum_aspect = w/h;
  }
  return rc;
}

BOOL CRhinoViewport::GetFrustumCenter( double* frus_center ) const
{
  double camZ[3], frus_near, frus_far, d;
  if ( !frus_center )
    return FALSE;
  if ( !GetCameraFrame( frus_center, NULL, NULL, camZ ) )
    return FALSE;
  if ( !GetFrustum( NULL, NULL, NULL, NULL, &frus_near, &frus_far ) )
    return FALSE;
  d = -0.5*(frus_near+frus_far);
  frus_center[0] += d*camZ[0];
  frus_center[1] += d*camZ[1];
  frus_center[2] += d*camZ[2];
  return TRUE;
}

BOOL CRhinoViewport::SetScreenPort( 
      int port_left, 
      int port_right,
      int port_bottom, 
      int port_top,
      int port_near, // = 0
      int port_far   // = 0
      )
{
  if ( port_left == port_right )
    return FALSE;
  if ( port_bottom == port_top )
    return FALSE;
  m_port_left   = port_left;
  m_port_right  = port_right;
  m_port_bottom = port_bottom;
  m_port_top    = port_top;
  if ( port_near || port_near!=port_far ) {
    m_port_near   = port_near;
    m_port_far    = port_far;
  }
  m_bValidPort = TRUE;
  return TRUE;
}

BOOL CRhinoViewport::GetScreenPort( 
      int* port_left,
      int* port_right,
      int* port_bottom,
      int* port_top,
      int* port_near, // = NULL
      int* port_far   // = NULL
      ) const
{
  if ( port_left )
    *port_left = m_port_left;
  if ( port_right )
    *port_right = m_port_right;
  if ( port_bottom )
    *port_bottom = m_port_bottom;
  if ( port_top )
    *port_top = m_port_top;
  if ( port_near )
    *port_near = m_port_near;
  if ( port_far )
    *port_far = m_port_far;
  return m_bValidPort;
}

BOOL CRhinoViewport::GetScreenPortAspect(double& aspect) const
{
  const double width = m_port_right - m_port_left;
  const double height = m_port_top - m_port_bottom;
  aspect = ( m_bValidPort && height != 0.0 ) ? fabs(width/height) : 0.0;
  return m_bValidPort;
}


BOOL CRhinoViewport::SetPortPosition( 
      double relative_left, double relative_right,
      double relative_bottom, double relative_top
      )
{
  if ( relative_left < 0.0 || relative_left >= relative_right || relative_right > 1.0 )
    return FALSE;
  if ( relative_bottom < 0.0 || relative_bottom >= relative_top || relative_top > 1.0 )
    return FALSE;
  m_client_x[0] = relative_left;
  m_client_x[1] = relative_right;
  m_client_y[0] = relative_bottom;
  m_client_y[1] = relative_top;
  return TRUE;
}


BOOL CRhinoViewport::GetPortPosition( 
      double* relative_left,
      double* relative_right,
      double* relative_bottom,
      double* relative_top
      ) const
{
  if ( relative_left )
    *relative_left = m_client_x[0];
  if ( relative_right )
    *relative_right = m_client_x[1];
  if ( relative_bottom )
    *relative_bottom = m_client_y[0];
  if ( relative_top )
    *relative_top = m_client_y[1];
  return TRUE;
}

BOOL CRhinoViewport::SetTargetDistance( double target_distance )
{
  BOOL rc = FALSE;
  double l,r,b,t,n,f;
  BOOL bSetF = FALSE;
  rc = GetFrustum(&l,&r,&b,&t,&n,&f);
  if ( rc && target_distance > 0.0 ) {
    if ( target_distance < n ) {
      bSetF = TRUE;
      n = 0.5*target_distance;
    }
    if ( target_distance > f ) {
      bSetF = TRUE;
      f = 2.0*target_distance;
    }
    if ( bSetF )
      rc = SetFrustum(l,r,b,t,n,f);
    if ( rc ) {
      m_target_distance = target_distance;
    }
  }
  return rc;
}

BOOL CRhinoViewport::GetTargetDistance( double& target_distance ) const
{
  target_distance = (m_bValidCamera && m_target_distance > 0.0) ? m_target_distance : 0.0;
  return target_distance > 0.0;
}


BOOL CRhinoViewport::GetTargetPoint( double* target_point ) const
{
  if ( m_bValidCamera && target_point ) {
    target_point[0] = m_CamLoc[0] - m_target_distance*m_CamZ[0];
    target_point[1] = m_CamLoc[1] - m_target_distance*m_CamZ[1];
    target_point[2] = m_CamLoc[2] - m_target_distance*m_CamZ[2];
  }
  return m_bValidCamera;
}


double CRhinoViewport::ViewSize() const
{
  // radius of largest visible disk in target plane
  double d = 100.0, dx, dy;
  if ( m_bValidFrustum ) {
    dx = m_frus_right - m_frus_left;
    dy = m_frus_top - m_frus_bottom;
    d = (dx <= dy) ? dx : dy;
    d *= 0.5;
    if ( m_projection == perspective_proj && m_frus_near > 0.0 ) {
        d /= m_frus_near;
    }
  }
  return d;
}

BOOL CRhinoViewport::GetAngles( double* angle1, double* angle2, double* angle3 ) const
{
  BOOL rc = FALSE;
  if ( m_bValidCamera ) {
    rc = RhinoGetRotationAngles( m_CamX, m_CamY, m_CamZ, angle1, angle2, angle3 );
  }
  return rc;
}

BOOL CRhinoViewport::FromRhino(
        CRhinoViewport::eProjection projection,
        const double* rhvp_target, // 3d point
        double rhvp_angle1, double rhvp_angle2, double rhvp_angle3, // radians
        double rhvp_viewsize,     // > 0
        double rhvp_cameradist,   // > 0
        int screen_width, int screen_height
        )
/*****************************************************************************
Compute canonical view projection information from Rhino viewport settings
INPUT:
  projection
  rhvp_target
    Rhino viewport target point (3d point that is center of view rotations)
  rhvp_angle1, rhvp_angle2, rhvp_angle3
    Rhino viewport angle settings
  rhvp_viewsize 
    In perspective, rhvp_viewsize = tangent(half lense angle).
    In parallel, rhvp_viewsize = 1/2 * minimum(frustum width,frustum height)
  rhvp_cameradistance ( > 0 )
    Distance from camera location to Rhino's "target" point
  screen_width, screen_height (0,0) if not known
*****************************************************************************/
{
  /*
  width, height
    width and height of viewport  
    ( = RhinoViewport->width, RhinoViewport->height )
  z_buffer_depth
    depth for the z buffer.  0xFFFF is currently used for Rhino
    quick rendering.
  */

  // In the situation where there is no physical display device, assume a
  // 1000 x 1000 "screen" and set the parameters accordingly.  Toolkit users
  // that are using this class to actually draw a picture, can make a subsequent
  // call to SetScreenPort().

  const double height = (screen_width < 1 || screen_height < 1) 
                      ? 1000.0 : (double)screen_height;
  const double width  = (screen_width < 1 || screen_height < 1) 
                      ? 1000.0 : (double)screen_width;
  //const int z_buffer_depth = 0xFFFF; // value Rhino "Shade" command uses

  // Use this function to obtain standard view information from a Rhino VIEWPORT
  // view. The Rhino viewport has many entries.  As of 17 October, 1997 all Rhino
  // world to clipping transformation information is derived from the VIEWPORT 
  // fields:
  //
  //   target, angle1, angle2, angle3, viewsize, and cameradist.
  //
  // The width, height and zbuffer_depth arguments are used to determing the
  // clipping to screen transformation.
  m_bValidCamera   = FALSE;
  m_bValidFrustum = FALSE;
  m_bValidPort     = FALSE;

  const double WorldX[3]      = {1.0,0.0,0.0};
  const double WorldZ[3]      = {0.0,0.0,1.0};
  const double WorldOrigin[3] = {0.0,0.0,0.0};

  CRhinoXform R1, R2, R3, RhinoRot;
  double CameraLocation[3], CameraDirection[3], CameraUp[3];
  double frustum_left, frustum_right, frustum_bottom, frustum_top;
  double near_clipping_distance, far_clipping_distance;

  // Initialize default view in case input is garbage.
  if (height < 1)
    return FALSE;
  if ( width < 1 )
    return FALSE;
  if ( rhvp_viewsize <= 0.0 )
    return FALSE;
  if ( rhvp_cameradist <= 0.0 )
    return FALSE;

  // A Rhino VIEWPORT structure describes the camera's location, direction,
  // and  orientation by specifying a rotation transformation that is
  // applied to an initial frame.  The rotation transformation is defined
  // as a sequence of 3 rotations abount fixed axes.  The initial frame
  // has the camera located at (0,0,cameradist), pointed in the direction
  // (0,0,-1), and oriented so that up is (0,1,0).

  R1.Rotation( WorldZ, WorldOrigin, sin(rhvp_angle1), cos(rhvp_angle1) ); // so called "twist"
  R2.Rotation( WorldX, WorldOrigin, sin(rhvp_angle2), cos(rhvp_angle2) ); // so called "elevation"
  R3.Rotation( WorldZ, WorldOrigin, sin(rhvp_angle3), cos(rhvp_angle3) ); // so called "fudge factor"
  RhinoRot = R3 * R2 * R1;
  RhinoRot.ApplyVector( 0.0, 1.0, 0.0, CameraUp );
  RhinoRot.ApplyVector( 0.0, 0.0, -1.0, CameraDirection );

  CameraLocation[0] = rhvp_target[0] - rhvp_cameradist*CameraDirection[0];
  CameraLocation[1] = rhvp_target[1] - rhvp_cameradist*CameraDirection[1];
  CameraLocation[2] = rhvp_target[2] - rhvp_cameradist*CameraDirection[2];

  // Camera coordinates "X" = CameraRight = CameraDirection x CameraUp
  // Camera coordinates "Y" = CameraUp
  // Camera coordinates "Z" = -CameraDirection

  // Rhino does not support skew projections.  In other words, the
  // view frustum is symmetric and ray that begins at CameraLocation and 
  // goes along CameraDirection runs along the frustum's central axis.
  // The aspect ratio of the view frustum equals 
  // (screen port width)/(screen port height)
  // This means frus_left = -frus_right, frus_bottom = -frus_top, and
  // frus_top/frus_right = height/width

  // Set near and far clipping planes to some reasonable values.  If
  // the depth of the pixel is important, then the near and far clipping
  // plane will need to be adjusted later.
  // Rhino doesn't have a far clipping plane in wire frame (which explains
  // why you can get perspective views reversed through the origin by using 
  // the SetCameraTarget() command.  It's near clipping plane is set to 
  // a miniscule value.  For mesh rendering, it must come up with some
  // sort of reasonable near and far clipping planes because the zbuffer
  // is used correctly.  When time permits, I'll dig through the rendering
  // code and determine what values are being used.
  //
  near_clipping_distance = rhvp_cameradist/64.0;
  if ( near_clipping_distance > 1.0 )
    near_clipping_distance = 1.0;
  far_clipping_distance = 4.0*rhvp_cameradist;


  if ( width <= height ) {
    frustum_right = rhvp_viewsize;
    frustum_top = frustum_right*height/width;
  }
  else {
    frustum_top = rhvp_viewsize;
    frustum_right = frustum_top*width/height;
  }
  if ( projection == perspective_proj ) {
    frustum_right *= near_clipping_distance;
    frustum_top   *= near_clipping_distance;
  }
  frustum_left   = -frustum_right;
  frustum_bottom = -frustum_top;

  BOOL rcPro = SetProjection( projection );

  BOOL rcCam = SetCamera( CameraLocation, CameraDirection, CameraUp );

  BOOL rcFrust = SetFrustum( 
         frustum_left,   frustum_right, 
         frustum_bottom, frustum_top, 
         near_clipping_distance, far_clipping_distance );

  if ( rcFrust )
    SetTargetDistance( rhvp_cameradist );

  /*
  // Windows specific stuff that requires knowing size of client area in pixels
  rc = SetPort( pView,
         0,        width-1, // windows has screen X increasing accross
         height-1,       0, // windows has screen Y increasing downwards
         0, z_buffer_depth );
  */

  return (rcPro && rcCam && rcFrust);
}

BOOL CRhinoViewport::SetViewName( const char* sViewName )
{
  if ( m_sViewName ) {
    delete m_sViewName;
  }
  m_sViewName = NULL;
  if ( sViewName && sViewName[0] ) {
    int len = strlen( sViewName );
    m_sViewName = new char[(len+1)];
    if ( m_sViewName ) {
      strncpy( m_sViewName, sViewName, len );
      m_sViewName[len] = 0;
    }
  }
  return TRUE;
}

const char* CRhinoViewport::ViewName() const
{
  return m_sViewName;
}

BOOL CRhinoViewport::SetConstructionPlane( 
       const double* origin, // origin
       const double* X, // X axis
       const double* Y // Y axis
       )
{
  double xlen, ylen, d, x[3], y[3];
  if ( origin ) {
    memcpy( m_cp_origin, origin, 3*sizeof(m_cp_origin[0]) );
  }
  if ( X && !Y )
    return FALSE;
  if ( Y && !X )
    return FALSE;

  memcpy( x, X, 3*sizeof(x[0]) );
  memcpy( y, Y, 3*sizeof(y[0]) );
  xlen = RhinoVectorLength( 3, x );
  ylen = RhinoVectorLength( 3, y );
  if ( xlen <= RHINO_SQRT_EPSILON )
    return FALSE;
  if ( ylen <= RHINO_SQRT_EPSILON )
    return FALSE;
  xlen = 1.0/xlen;
  ylen = 1.0/ylen;
  if ( fabs(xlen - 1.0) > RHINO_SQRT_EPSILON ) {
    x[0] *= xlen;
    x[1] *= xlen;
    x[2] *= xlen;      
  }
  if ( fabs(ylen - 1.0) > RHINO_SQRT_EPSILON ) {
    y[0] *= ylen;
    y[1] *= ylen;
    y[2] *= ylen;      
  }
  d = x[0]*y[0] + x[1]*y[1] + x[2]*y[2];
  if ( fabs(d) > RHINO_SQRT_EPSILON ) {
    d = -d;
    y[0] += d*x[0];
    y[1] += d*x[1];
    y[2] += d*x[2];
    ylen = RhinoVectorLength( 3, y );
    if ( ylen <= RHINO_SQRT_EPSILON )
      return FALSE;
    ylen = 1.0/ylen;
    y[0] *= ylen;
    y[1] *= ylen;
    y[2] *= ylen;   
  }
  memcpy( m_cp_X, x, 3*sizeof(m_cp_X[0]) );
  memcpy( m_cp_Y, y, 3*sizeof(m_cp_Y[0]) );      

  return TRUE;
}

BOOL CRhinoViewport::GetConstructionPlane( 
       double* origin,
       double* Xaxis,
       double* Yaxis,
       double* Zaxis
       ) const
{
  if ( origin ) {
    memcpy( origin, m_cp_origin, 3*sizeof(origin[0]) );
  }
  if ( Xaxis ) {
    memcpy( Xaxis, m_cp_X, 3*sizeof(Xaxis[0]) );
  }
  if ( Yaxis ) {
    memcpy( Yaxis, m_cp_Y, 3*sizeof(Yaxis[0]) );
  }
  if ( Zaxis ) {
    memcpy( Zaxis, m_cp_Z, 3*sizeof(Zaxis[0]) );
  }
  return TRUE;
}

BOOL CRhinoViewport::GetCameraAngle( 
       double* angle,
       double* angle_h, 
       double* angle_w
       ) const
{
  BOOL rc = FALSE;
  if ( angle )
    *angle = 0.0;
  if ( angle_h )
    *angle_h = 0.0;
  if ( angle_w )
    *angle_w = 0.0;
  double half_w, half_h, left, right, bot, top, near_dist;
  if ( GetFrustum( &left, &right, &bot, &top, &near_dist, NULL ) ) {
    half_w = ( right > -left ) ? right : -left;
    half_h = ( top   > -bot  ) ? top   : -bot;
    if ( angle )
      *angle = 2.0*atan( sqrt(half_w*half_w + half_h*half_h)/near_dist );
    if ( angle_h )
      *angle_h = 2.0*atan( half_h/near_dist );
    if ( angle_w )
      *angle_w = 2.0*atan( half_w/near_dist );
    rc = TRUE;
  }
  return rc;
}

BOOL CRhinoViewport::GetCameraAngle( 
       double* angle
       ) const
{
  double angle_h = 0.0;
  double angle_w = 0.0;
  BOOL rc = GetCameraAngle( NULL, &angle_h, &angle_w );
  if ( angle && rc ) {
    *angle = (angle_h < angle_w) ? angle_w : angle_h;
  }
  return rc;
}

BOOL CRhinoViewport::SetCameraAngle( double angle )
{
  BOOL rc = FALSE;
  double r, d, aspect, half_w, half_h, near_dist, far_dist;
  if ( angle > 0.0  && angle < 0.5*RHINO_PI*(1.0-RHINO_SQRT_EPSILON) ) {
    if ( GetFrustum( NULL, NULL, NULL, NULL, &near_dist, &far_dist ) && GetFrustumAspect( aspect) ) {
      r = near_dist*tan(angle);
      d = r/sqrt(1.0+aspect*aspect);
      if ( aspect >= 1.0 ) {
        // width >= height
        half_w = d*aspect;
        half_h = d;
      }
      else {
        // height > width
        half_w = d;
        half_h = d/aspect;
      }
      rc = SetFrustum( -half_w, half_w, -half_h, half_h, near_dist, far_dist );
    }
  }
  return rc;
}

BOOL CRhinoViewport::GetCamera35mmLenseLength( double* lense_length ) const
{
  // 35 mm film has a height of 24 mm and a width of 36 mm
  BOOL rc = FALSE;
  double angle, angle_h, angle_w, aspect;
  if ( !lense_length )
    return FALSE;
  *lense_length = 0.0;
  if ( GetFrustumAspect( aspect ) && GetCameraAngle( &angle, &angle_h, &angle_w ) ) {
    rc = TRUE;
    // assume camera is horizontal
    if ( aspect >= 36.0/24.0 ) {
      *lense_length = 18.0/tan(0.5*angle_w);
    }
    else {
      *lense_length = 12.0/tan(0.5*angle_h);
    }
  }
  return rc;
}

BOOL CRhinoViewport::SetCamera35mmLenseLength( double lense_length )
{
  // 35 mm film has a height of 24 mm and a width of 36 mm
  double frus_left, frus_right, frus_bottom, frus_top, frus_near, frus_far;
  double frus_width, frus_height;
  if ( lense_length <= 0.0 )
    return FALSE;
  if ( !GetFrustum( &frus_left, &frus_right, &frus_bottom, &frus_top, 
                     &frus_near, &frus_far ) )
    return FALSE;
  frus_width  = frus_right - frus_left;
  frus_height = frus_top - frus_bottom;
  if ( frus_width*24.0 <= frus_height*36.0 ) {
    // use vertical direction to set angle
    frus_top    = tan(12.0/lense_length)*frus_near;
    frus_bottom = -frus_top;
    frus_right  = frus_top*frus_width/frus_height;
    frus_left   = -frus_right;
  }
  else {
    // use horizontal direction to set angle
    frus_right  = tan(18.0/lense_length)*frus_near;
    frus_left   = -frus_right;
    frus_top    = frus_right*frus_height/frus_width;
    frus_bottom = -frus_top;
  }
  return SetFrustum( frus_left, frus_right, frus_bottom, frus_top, frus_near, frus_far );
}

BOOL CRhinoViewport::GetXform( 
       CRhinoViewport::eCoordinateSystem srcCS,
       CRhinoViewport::eCoordinateSystem destCS,
       CRhinoXform& xform
       ) const
{
  BOOL rc = FALSE;
  CRhinoXform x0, x1;

  xform.Identity();

  switch( srcCS ) {
  case world_cs:
  case camera_cs:
  case clip_cs:
  case screen_cs:
    break;
  default:
    return FALSE;
  }
  switch( destCS ) {
  case world_cs:
  case camera_cs:
  case clip_cs:
  case screen_cs:
    break;
  default:
    return FALSE;
  }

  if (srcCS == destCS)
    return TRUE;


  switch ( srcCS ) {

  case world_cs:
    if ( !m_bValidCamera )
      break;
    switch ( destCS ) {
    case camera_cs:
      xform.WorldToFrame( m_CamLoc, m_CamX, m_CamY, m_CamZ );
      rc = TRUE;
      break;
    case clip_cs:
      rc = GetXform( world_cs,  camera_cs, x0 );
      if (rc)
        rc = GetXform( camera_cs, clip_cs,   x1 );
      if (rc)
        xform = x1*x0;
      break;
    case screen_cs:
      rc = GetXform( world_cs,  clip_cs,   x0 );
      if (rc)
        rc = GetXform( clip_cs,   screen_cs, x1 );
      if (rc)
        xform = x1*x0;
      break;
    case world_cs:
      // Never happens.  This is here to quiet g++ warnings.
      break;
    }
    break;

  case camera_cs:
    if ( !m_bValidCamera )
      break;
    switch ( destCS ) {
    case world_cs:
      xform.FrameToWorld( m_CamLoc, m_CamX, m_CamY, m_CamZ );
      rc = TRUE;
      break;
    case clip_cs:
      if ( m_bValidFrustum ) {
        xform.CameraToClip(
          m_projection == perspective_proj,
          m_frus_left, m_frus_right,
          m_frus_bottom, m_frus_top,
          m_frus_near, m_frus_far );
        rc = TRUE;
      }
      break;
    case screen_cs:
      rc = GetXform( camera_cs,  clip_cs,  x0 );
      if (rc)
        rc = GetXform( clip_cs,   screen_cs, x1 );
      if (rc)
        xform = x1*x0;
      break;
    case camera_cs:
      // Never happens.  This is here to quiet g++ warnings.
      break;
    }
    break;

  case clip_cs:
    switch ( destCS ) {
    case world_cs:
      rc = GetXform( clip_cs,   camera_cs, x0 );
      if (rc)
        rc = GetXform( camera_cs,  world_cs, x1 );
      if (rc)
        xform = x1*x0;
      break;
    case camera_cs:
      if ( m_bValidFrustum ) {
        xform.ClipToCamera(
          m_projection == perspective_proj,
          m_frus_left, m_frus_right,
          m_frus_bottom, m_frus_top,
          m_frus_near, m_frus_far );
        rc = TRUE;
      }
      break;
    case screen_cs:
      if ( m_bValidPort ) {
        xform.ClipToScreen( 
          m_port_left, m_port_right, 
          m_port_bottom, m_port_top,
          m_port_near, m_port_far );
        rc = TRUE;
      }
      break;
    case clip_cs:
      // Never happens.  This is here to quiet g++ warnings.
      break;
    }
    break;

  case screen_cs:
    switch ( destCS ) {
    case world_cs:
      rc = GetXform( screen_cs, camera_cs, x0 );
      if (rc)
        rc = GetXform( camera_cs, world_cs,  x1 );
      if (rc)
        xform = x1*x0;
      break;
    case camera_cs:
      rc = GetXform( screen_cs, clip_cs,   x0 );
      if (rc)
        rc = GetXform( clip_cs,   camera_cs, x1 );
      if (rc)
        xform = x1*x0;
      break;
    case clip_cs:
      if ( m_bValidPort ) {
        xform.ScreenToClip(
          m_port_left, m_port_right, 
          m_port_bottom, m_port_top,
          m_port_near, m_port_far );
        rc = TRUE;
      }
      break;
    case screen_cs:
      // Never happens.  This is here to quiet g++ warnings.
      break;
    }
    break;

  }

  return rc;
}


static double clipDist( const double* camLoc, const double* camZ, const double* P )
{
  return (camLoc[0]-P[0])*camZ[0]+(camLoc[1]-P[1])*camZ[1]+(camLoc[2]-P[2])*camZ[2];
}


BOOL CRhinoViewport::SetFrustumNearFar( 
       const double* box_min,
       const double* box_max
       )
{
  BOOL rc = FALSE;
  const double* box[2];
  int i,j,k;
  double n, f, d;
  double camLoc[3], camZ[3], P[3];
  if ( !box_min )
    box_min = box_max;
  if ( !box_max )
    box_max = box_min;
  if ( !box_min )
    return FALSE;
  box[0] = box_min;
  box[1] = box_max;
  if ( GetCameraFrame( camLoc, NULL, NULL, camZ ) ) {
    n = f = -1.0;
    for(i=0;i<2;i++)for(j=0;j<2;j++)for(k=0;k<2;k++) {
      P[0] = box[i][0];
      P[1] = box[j][1];
      P[2] = box[k][2];
      d = clipDist(camLoc,camZ,P);
      if (!i&&!j&&!k)
        n=f=d;
      else if ( d < n )
        n = d;
      else if ( d > f )
        f = d;
    }
    if ( f <= 0.0 )
      return FALSE; // box is behind camera
    n *= 0.9375;
    f *= 1.0625;
    if ( n <= 0.0 )
      n = 0.001*f;
    rc = SetFrustumNearFar( n, f );
  }
  return rc;
}

BOOL CRhinoViewport::SetFrustumNearFar( 
       const double* center,
       double        radius
       )
{
  BOOL rc = FALSE;
  double n, f, d;
  double camLoc[3], camZ[3], P[3];
  if ( GetCameraFrame( camLoc, NULL, NULL, camZ ) ) {
    d = fabs(radius);
    P[0] = center[0] + d*camZ[0];
    P[1] = center[1] + d*camZ[0];
    P[2] = center[2] + d*camZ[0];
    n = clipDist(camLoc,camZ,P);
    P[0] = center[0] - d*camZ[0];
    P[1] = center[1] - d*camZ[0];
    P[2] = center[2] - d*camZ[0];
    f = clipDist(camLoc,camZ,P);
    if ( f <= 0.0 )
      return FALSE; // sphere is behind camera
    n *= 0.9375;
    f *= 1.0625;
    if ( n <= 0.0 )
      n = 0.001*f;
    rc = SetFrustumNearFar( n, f );
  }
  return rc;
}

BOOL CRhinoViewport::SetFrustumNearFar( double n, double f )
{
  double d, frus_left, frus_right, frus_bottom, frus_top, frus_near, frus_far;
  BOOL rc = FALSE;
  if ( n > 0.0 && f > n ) {
    if ( GetFrustum( &frus_left,   &frus_right, 
                      &frus_bottom, &frus_top, 
                      &frus_near,   &frus_far ) ) {
      // preserve valid frustum
      if ( Projection() == perspective_proj ) {
        d = n/frus_near;
        frus_left *= d;
        frus_right *= d;
        frus_bottom *= d;
        frus_top *= d;
      }
      frus_near = n;
      frus_far = f;
      rc = SetFrustum( frus_left,   frus_right, 
                        frus_bottom, frus_top, 
                        frus_near,   frus_far );
    }
    else {
      m_frus_near = n;
      m_frus_far = f;
      if ( m_target_distance < m_frus_near || m_target_distance > m_frus_far )
        m_target_distance = 0.5*(m_frus_near+m_frus_far);
      rc = TRUE;
    }
  }
  return rc;
}

BOOL CRhinoViewport::GetCoordinateSprite( 
                     int size, 
                     int scrx, int scry,
                     int indx[3], // axis order by depth
                     int scr_coord[3][2] ) const
{
  // size = length of axes in pixels

  indx[0] = 0; indx[1] = 1; indx[2] = 2;
  scr_coord[0][0] = scr_coord[1][0] = scr_coord[2][0] = scrx;
  scr_coord[0][1] = scr_coord[1][1] = scr_coord[2][1] = scry;

  double C[3], X[3], Y[3], Z[3], ScrC[3], Scr[3][3];
  CRhinoXform w2s;
  if (!GetFrustumCenter( C ) )
    return FALSE;
  if (!GetCameraFrame( NULL, X, NULL, Z )) 
    return FALSE;
  if (!GetXform( world_cs, screen_cs, w2s )) 
    return FALSE;

  // indx[] determines order that axes are drawn
  // sorted from back to front
  int i,j,k;
  for (i = 0; i < 2; i++) for (j = i+1; j <= 2; j++) {
    if (Z[indx[i]] > Z[indx[j]])
      {k = indx[i]; indx[i] = indx[j]; indx[j] = k;}
  }

  // determine world length that corresponds to size pixels
  X[0] += C[0];
  X[1] += C[1];
  X[2] += C[2];
  w2s.Apply( C[0], C[1], C[2], Y );
  w2s.Apply( X[0], X[1], X[2], X );
  if (X[0] == Y[0])
    return FALSE;
  double s = size/fabs( X[0] - Y[0] );

  // transform world coord axes to screen
  X[0] = C[0] + s;
  X[1] = C[1] + 0.0;
  X[2] = C[2] + 0.0;
  Y[0] = C[0] + 0.0;
  Y[1] = C[1] + s;
  Y[2] = C[2] + 0.0;
  Z[0] = C[0] + 0.0;
  Z[1] = C[1] + 0.0;
  Z[2] = C[2] + s;
  w2s.Apply( C[0], C[1], C[2], ScrC );
  w2s.Apply( X[0], X[1], X[2], Scr[0] );
  w2s.Apply( Y[0], Y[1], Y[2], Scr[1] );
  w2s.Apply( Z[0], Z[1], Z[2], Scr[2] );
  
  int dx = scrx - (int)ScrC[0];
  int dy = scry - (int)ScrC[1];
  for (i=0;i<3;i++) {
    scr_coord[i][0] = dx + ((int)Scr[i][0]); 
    scr_coord[i][1] = dy + ((int)Scr[i][1]);
  }

  return TRUE;
}

static BOOL GetRelativeScreenCoordinates(
          int port_left, int port_right,
          int port_bottom, int port_top,
          BOOL bSortPoints,
          int& x0, int& y0, int& x1, int& y1,
          double& s0, double& t0, double& s1, double& t1
          )
{
  // convert screen rectangle into relative rectangle
  if ( bSortPoints ) {
    int i;
    if ( x0 > x1 ) {
      i = x0; x0 = x1; x1 = i;
    }
    if ( port_left > port_right ) {
      i = x0; x0 = x1; x1 = i;
    }
    if ( y0 > y1 ) {
      i = y0; y0 = y1; y1 = i;
    }
    if ( port_bottom > port_top ) {
      i = y0; y0 = y1; y1 = i;
    }
  }

  s0 = ((double)(x0 - port_left))/((double)(port_right - port_left));
  s1 = ((double)(x1 - port_left))/((double)(port_right - port_left));
  t0 = ((double)(y0 - port_bottom))/((double)(port_top - port_bottom));
  t1 = ((double)(y1 - port_bottom))/((double)(port_top - port_bottom));
  
  double tol = 0.001;
  if ( fabs(s0) <= tol ) s0 = 0.0; else if (fabs(s0-1.0) <= tol ) s0 = 1.0;
  if ( fabs(s1) <= tol ) s1 = 0.0; else if (fabs(s1-1.0) <= tol ) s1 = 1.0;
  if ( fabs(t0) <= tol ) t0 = 0.0; else if (fabs(t0-1.0) <= tol ) t0 = 1.0;
  if ( fabs(t1) <= tol ) t1 = 0.0; else if (fabs(t1-1.0) <= tol ) t1 = 1.0;
  if ( fabs(s0-s1) <= tol )
    return FALSE;
  if ( fabs(t0-t1) <= tol )
    return FALSE;
  return TRUE;
}

BOOL CRhinoViewport::ZoomToScreenRect( int x0, int y0, int x1, int y1 )
{
  int port_left, port_right, port_bottom, port_top;
  if ( !GetScreenPort( &port_left, &port_right, 
                       &port_bottom, &port_top, 
                       NULL, NULL ) )
    return FALSE;
  int dx = (x0+x1)/2;
  int dy = (y0+y1)/2;
  int cx = (port_left+port_right)/2;
  int cy = (port_bottom+port_top)/2;
  if ( !DollyAlongScreenChord( dx, dy, cx, cy ) )
    return FALSE;
  dx = cx - dx;
  dy = cy - dy;
  x0 += dx;
  x1 += dx;
  y0 += dy;
  y1 += dy;
  double frus_left, frus_right, frus_bottom, frus_top, frus_near, frus_far;
  if ( !GetFrustum( &frus_left,   &frus_right, 
                     &frus_bottom, &frus_top, 
                     &frus_near,   &frus_far ) )
    return FALSE;
  double s0,t0,s1,t1;
  if ( !GetRelativeScreenCoordinates(port_left, port_right, port_bottom, port_top,
                              TRUE,
                              x0,y0,x1,y1,
                              s0,t0,s1,t1) )
    return FALSE;
  double w = frus_right - frus_left;
  double h = frus_top - frus_bottom;
  double a0 = (1.0-s0)*frus_left   + s0*frus_right;
  double a1 = (1.0-s1)*frus_left   + s1*frus_right;
  double b0 = (1.0-t0)*frus_bottom + t0*frus_top;
  double b1 = (1.0-t1)*frus_bottom + t1*frus_top;
  if ( -a0 > a1 ) a1 = -a0; else a0 = -a1;
  if ( -b0 > b1 ) b1 = -b0; else b0 = -b1;
  double d;
  if ( (b1-b0)*w < (a1-a0)*h ) {
    d = (a1-a0)*h/w;
    d = 0.5*(d - (b1-b0));
    b0 -= d;
    b1 += d;
  }
  else {
    d = (b1-b0)*w/h;
    d = 0.5*(d - (a1-a0));
    a0 -= d;
    a1 += d;
  }

  return SetFrustum( a0, a1, b0, b1, frus_near, frus_far );
}

/*
BOOL CRhinoViewport::DollyToScreenRect( double view_plane_distance,
                                        int x0, int y0, int x1, int y1 )
{
  // Only makes sense in a perspective projection. In a parallel projection,
  // I resort to ZoomToScreenRect(0 and the visual result is the same.
  if ( Projection() != perspective_proj )
    return ZoomToScreenRect( x0, y0, x1, y1 );

  int port_left, port_right, port_bottom, port_top;
  if ( !GetScreenPort( &port_left, &port_right, &port_bottom, &port_top, NULL, NULL ) )
    return FALSE;
  int dx = (x0+x1)/2;
  int dy = (y0+y1)/2;
  int cx = (port_left+port_right)/2;
  int cy = (port_bottom+port_top)/2;
  if ( !DollyAlongScreenChord( dx, dy, cx, cy ) )
    return FALSE;
  dx = cx - dx;
  dy = cy - dy;
  x0 += dx;
  x1 += dx;
  y0 += dy;
  y1 += dy;

  double frus_left, frus_right, frus_bottom, frus_top, frus_near, frus_far;
  if ( !GetFrustum( &frus_left,   &frus_right, 
                     &frus_bottom, &frus_top, 
                     &frus_near,   &frus_far ) )
    return FALSE;

  double s0, t0, s1, t1;
  if ( !GetRelativeScreenCoordinates(port_left, port_right, port_bottom, port_top,
                              TRUE,
                              x0,y0,x1,y1,
                              s0,t0,s1,t1) )
    return FALSE;

  double w = frus_right - frus_left;
  double h = frus_top - frus_bottom;
  double a0 = (1.0-s0)*frus_left   + s0*frus_right;
  double a1 = (1.0-s1)*frus_left   + s1*frus_right;
  double b0 = (1.0-t0)*frus_bottom + t0*frus_top;
  double b1 = (1.0-t1)*frus_bottom + t1*frus_top;
  if ( -a0 > a1 ) a1 = -a0; else a0 = -a1;
  if ( -b0 > b1 ) b1 = -b0; else b0 = -b1;
  double d;
  if ( (b1-b0)*w < (a1-a0)*h ) {
    d = (a1-a0)*h/w;
    d = 0.5*(d - h);
    b0 -= d;
    b1 += d;
  }
  else {
    d = (b1-b0)*w/h;
    d = 0.5*(d - w);
    a0 -= d;
    a1 += d;
  }

  d = 0.5*((a1-a0)/w + (b1-b0)/h)*view_plane_distance;
  double delta = d - view_plane_distance;

  frus_near += delta;
  frus_far  += delta;
  if ( frus_near <= 0.0 ) {
    if ( frus_far <= 0.0 )
      frus_far = 100.0;
    frus_near = 0.001*frus_far;
  }
  if ( !SetFrustumNearFar( frus_near, frus_far ) )
    return FALSE;

  double camLoc[3], camY[3], camZ[3];
  if ( !GetCameraFrame( camLoc, NULL, camY, camZ ) )
    return FALSE;
  camLoc[0] += delta*camZ[0];
  camLoc[1] += delta*camZ[1];
  camLoc[2] += delta*camZ[2];
  camZ[0] = -camZ[0];
  camZ[1] = -camZ[1];
  camZ[2] = -camZ[2];
  if ( !SetCamera( camLoc, camZ, camY ) )
    return FALSE;

  return TRUE;
}
*/

BOOL CRhinoViewport::DollyAlongScreenChord( int x0, int y0, int x1, int y1 )
{
  // Dolly camera parallel to viewing plane so that screen point (x0,y0)
  // is moved to screen point (x1,y1).
  double frus_left, frus_right, frus_bottom, frus_top, frus_near, frus_far;
  if ( !GetFrustum( &frus_left,   &frus_right, 
                     &frus_bottom, &frus_top, 
                     &frus_near,   &frus_far ) )
    return FALSE;
  int port_left, port_right, port_bottom, port_top;
  if ( !GetScreenPort( &port_left, &port_right, &port_bottom, &port_top, NULL, NULL ) )
    return FALSE;
  double s0,t0,s1,t1;
  if ( !GetRelativeScreenCoordinates(port_left, port_right, port_bottom, port_top,
                              FALSE,
                              x0,y0,x1,y1,
                              s0,t0,s1,t1) )
    return FALSE;
  double a0 = (1.0-s0)*frus_left + s0*frus_right;
  double a1 = (1.0-s1)*frus_left + s1*frus_right;
  double b0 = (1.0-t0)*frus_bottom + t0*frus_top;
  double b1 = (1.0-t1)*frus_bottom + t1*frus_top;
  double deltax = a1-a0;
  double deltay = b1-b0;

  double d;
  if ( !GetTargetDistance( d ) )
    return FALSE;
  d /= -frus_near;
  deltax *= d;
  deltay *= d;

  double camLoc[3], camX[3], camY[3], camZ[3];
  if ( !GetCameraFrame( camLoc, camX, camY, camZ ) )
    return FALSE;
  camLoc[0] += deltax*camX[0] + deltay*camY[0];
  camLoc[1] += deltax*camX[1] + deltay*camY[1];
  camLoc[2] += deltax*camX[2] + deltay*camY[2];
  camZ[0] = -camZ[0];
  camZ[1] = -camZ[1];
  camZ[2] = -camZ[2];

  return SetCamera( camLoc, camZ, camY );
}

BOOL CRhinoViewport::Extents( double angle, const double* boxmin, const double* boxmax )
{
  const double* box[2];
  double center[3], radius, camX[3], camY[3];
  double x, y, xmin, xmax, ymin, ymax, bx, by, bz;
  int i,j,k;

  if ( !boxmin || !boxmax )
    return FALSE;
  box[0] = boxmin;
  box[1] = boxmax;
  GetCameraFrame( NULL, camX, camY, NULL );
  xmin=xmax=ymin=ymax=0.0;
  for (i=0;i<2;i++) for (j=0;j<2;j++) for (k=0;k<2;k++) {
    bx = box[i][0];
    by = box[j][1];
    bz = box[k][2];
    x = camX[0]*bx+camX[1]*by+camX[2]*bz;
    y = camY[0]*bx+camY[1]*by+camY[2]*bz;
    if ( i==0&&j==0&&k==0) {
      xmin=xmax=x;
      ymin=ymax=y;
    }
    else {
      if ( x > xmax) xmax=x; else if (x < xmin) xmin = x;
      if ( y > ymax) ymax=y; else if (y < ymin) ymin = y;
    }
  }
  center[0] = 0.5*(boxmin[0]+boxmax[0]);
  center[1] = 0.5*(boxmin[1]+boxmax[1]);
  center[2] = 0.5*(boxmin[2]+boxmax[2]);
  radius = 1.0625*(xmax-xmin);
  if ( ymax-ymin > radius )
    radius = 1.0625*(ymax-ymin);
  if ( radius <= RHINO_SQRT_EPSILON ) {
    bx = boxmax[0]-boxmin[0];
    by = boxmax[1]-boxmin[1];
    bz = boxmax[2]-boxmin[2];
    radius = bx;
    if ( by > radius )
      radius = by;
    if ( bz > radius )
      radius = bz;
  }
  if ( radius <= RHINO_SQRT_EPSILON )
    radius = 1.0;
  return Extents( angle, center, radius );
}

BOOL CRhinoViewport::Extents( double angle, const double* center, double radius )
{
  double target_dist, near_dist, far_dist, camLoc[3], camY[3], camZ[3];
  double origin[3] = {0.0,0.0,0.0};
  if ( !center )
    center = origin;
  if ( radius <= 0.0 || 
       angle  <= 0.0 || 
       angle  >= 0.5*RHINO_PI*(1.0-RHINO_SQRT_EPSILON) )
    return FALSE;
  target_dist = radius/sin(angle);
  near_dist = target_dist - 1.0625*radius;
  if ( near_dist < 0.0625*radius )
    near_dist = 0.0625*radius;
  if ( near_dist < MIN_NEAR_DIST )
    near_dist = MIN_NEAR_DIST;
  target_dist = near_dist + 1.0625*radius;
  far_dist = target_dist +  1.0625*radius;

  if ( !GetCameraFrame( camLoc, NULL, camY, camZ ) )
    return FALSE;
  camLoc[0] = center[0] + target_dist*camZ[0];
  camLoc[1] = center[1] + target_dist*camZ[1];
  camLoc[2] = center[2] + target_dist*camZ[2];
  camZ[0] = -camZ[0];
  camZ[1] = -camZ[1];
  camZ[2] = -camZ[2];
  if ( !SetCamera( camLoc, camZ, camY ) )
    return FALSE;
  if ( !SetFrustumNearFar( near_dist, far_dist ) )
    return FALSE;
  if ( !SetCameraAngle( angle ) )
    return FALSE;
  if ( !SetTargetDistance( target_dist ) )
    return FALSE;
  return TRUE;
}

