/* $Header: /Src/RhinoIO/rhinoio_xdata.cpp 3     10/20/99 1:28p Dalelear $ */
/* $NoKeywords: $ */
//
// Copyright (c) 1993-1998 Robert McNeel & Associates. All rights reserved.
// Rhinoceros is a registered trademark of Robert McNeel & Assoicates.
//
// THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
// ALL IMPLIED WARRANTIES OF FITNESS FOR ANY PARTICULAR PURPOSE AND OF
// MERCHANTABILITY ARE HEREBY DISCLAIMED.
//				
////////////////////////////////////////////////////////////////

#include "rhinoio_def.h"
#include "rhinoio_exception.h"
#include "rhinoio_layerptr.h"
#include "rhinoio_file.h"
#include "rhinoio_chunk.h"
#include "rhinoio_xdata.h"

//////////////////////////////////////////////////////////////////////////////////
// Class CRhinoXdata
//////////////////////////////////////////////////////////////////////////////////
RHINO_CHUNK_IMPLEMENT(CRhinoXdata, TCODE_XDATA)

CRhinoXdata::CRhinoXdata(): CRhinoChunk(TCODE_XDATA)
{
	m_sId = 0;
  m_buffer_size = 0;
  m_buffer_data = 0;
}

CRhinoXdata::~CRhinoXdata() 
{
  if ( m_buffer_data )
    rhfree( m_buffer_data );
  if ( m_sId )
    rhfree( m_sId );
}
	
CRhinoFile::eStatus 
CRhinoXdata::Read(CRhinoFile& file)
{
  int id_length = 0;
  file.ReadInteger( &id_length );
  file.ReadInteger( &m_buffer_size );
  if ( id_length > 0 ) {
    m_sId = (char*)rhmalloc( id_length+1 );
    memset( m_sId, 0, id_length+1 );
    file.ReadByte( id_length, m_sId );
  }
  if ( m_buffer_size ) { 
    m_buffer_data = rhmalloc( m_buffer_size+1 );
    memset( m_buffer_data, 0, m_buffer_size+1 );
    file.ReadByte( m_buffer_size, m_buffer_data );
  }
	return CRhinoFile::good;
}

void 
CRhinoXdata::Write(CRhinoFile &file) const 
{
	file.BeginWriteChunk( *this );
  int id_length = (m_sId) ? strlen( m_sId ) : 0;

  file.WriteInteger( id_length );
  file.WriteInteger( m_buffer_size );
  file.WriteByte( id_length, m_sId );
  file.WriteByte( m_buffer_size, (const char*)m_buffer_data );

	file.EndChunk();
}

const char* CRhinoXdata::Id(void) const
{
  return m_sId;
}

size_t CRhinoXdata::BufferSize(void) const
{
  return m_buffer_size;
}

const void* CRhinoXdata::BufferData(void) const
{
  return m_buffer_data;
}