/* $Header: /Src/RhinoIO/rhinoio_xform.cpp 18    11/17/98 8:37a Dalelear $ */
/* $NoKeywords: $ */
//
// Copyright (c) 1993-1998 Robert McNeel & Associates. All rights reserved.
// Rhinoceros is a registered trademark of Robert McNeel & Assoicates.
//
// THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
// ALL IMPLIED WARRANTIES OF FITNESS FOR ANY PARTICULAR PURPOSE AND OF
// MERCHANTABILITY ARE HEREBY DISCLAIMED.
//				
////////////////////////////////////////////////////////////////

#include "rhinoio_def.h"

CRhinoXform::CRhinoXform( double xform[4][4] )
{
  if ( xform )
    memcpy( &m_xform[0][0], &xform[0][0], 16*sizeof(m_xform[0][0]) );
  else
    memset( &m_xform[0][0], 0, 16*sizeof(m_xform[0][0]) );
}

CRhinoXform::CRhinoXform( const double* xform )
{
  if ( xform )
    memcpy( &m_xform[0][0], xform, 16*sizeof(m_xform[0][0]) );
  else
    memset( &m_xform[0][0], 0, 16*sizeof(m_xform[0][0]) );
}

CRhinoXform::CRhinoXform()
{
  memset( &m_xform[0][0], 0, 16*sizeof(m_xform[0][0]) );
}

CRhinoXform::CRhinoXform( double d )
{
  memset( &m_xform[0][0], 0, 16*sizeof(m_xform[0][0]) );
  m_xform[0][0] = m_xform[1][1] = m_xform[2][2] = m_xform[3][3] = d;
}

CRhinoXform::CRhinoXform( const CRhinoXform& src )
{
  memcpy( &m_xform[0][0], &src.m_xform[0][0], 16*sizeof(m_xform[0][0]) );
}

CRhinoXform::~CRhinoXform()
{
}

CRhinoXform& CRhinoXform::operator=( const CRhinoXform& src )
{
  if ( &src != this ) {
    memcpy( &m_xform[0][0], &src.m_xform[0][0], 16*sizeof(m_xform[0][0]) );
  }
  return *this;
}

CRhinoXform& CRhinoXform::operator=( int s )
{
  memset( &m_xform[0][1], 0, 14*sizeof(m_xform[0][0]) );
  m_xform[0][0] = m_xform[1][1] = m_xform[2][2] = s;
  m_xform[3][3] = 1.0;
  return *this;
}

CRhinoXform& CRhinoXform::operator=( float s )
{
  memset( &m_xform[0][1], 0, 14*sizeof(m_xform[0][0]) );
  m_xform[0][0] = m_xform[1][1] = m_xform[2][2] = s;
  m_xform[3][3] = 1.0;
  return *this;
}

CRhinoXform& CRhinoXform::operator=( double s )
{
  memset( &m_xform[0][1], 0, 14*sizeof(m_xform[0][0]) );
  m_xform[0][0] = m_xform[1][1] = m_xform[2][2] = s;
  m_xform[3][3] = 1.0;
  return *this;
}

CRhinoXform CRhinoXform::operator+( const CRhinoXform& rhs ) const
{
  double m[16];
  const double *a = &m_xform[0][0];
  const double *b = &rhs.m_xform[0][0];
  m[ 0] = *a++ + *b++;
  m[ 1] = *a++ + *b++;
  m[ 2] = *a++ + *b++;
  m[ 3] = *a++ + *b++;
  m[ 4] = *a++ + *b++;
  m[ 5] = *a++ + *b++;
  m[ 6] = *a++ + *b++;
  m[ 7] = *a++ + *b++;
  m[ 8] = *a++ + *b++;
  m[ 9] = *a++ + *b++;
  m[10] = *a++ + *b++;
  m[11] = *a++ + *b++;
  m[12] = *a++ + *b++;
  m[13] = *a++ + *b++;
  m[14] = *a++ + *b++;
  m[15] = *a++ + *b++;
  return CRhinoXform(m);
}

CRhinoXform CRhinoXform::operator-( const CRhinoXform& rhs ) const
{
  double m[16];
  const double *a = &m_xform[0][0];
  const double *b = &rhs.m_xform[0][0];
  m[ 0] = *a++ - *b++;
  m[ 1] = *a++ - *b++;
  m[ 2] = *a++ - *b++;
  m[ 3] = *a++ - *b++;
  m[ 4] = *a++ - *b++;
  m[ 5] = *a++ - *b++;
  m[ 6] = *a++ - *b++;
  m[ 7] = *a++ - *b++;
  m[ 8] = *a++ - *b++;
  m[ 9] = *a++ - *b++;
  m[10] = *a++ - *b++;
  m[11] = *a++ - *b++;
  m[12] = *a++ - *b++;
  m[13] = *a++ - *b++;
  m[14] = *a++ - *b++;
  m[15] = *a++ - *b++;
  return CRhinoXform(m);
}

CRhinoXform CRhinoXform::operator*( const CRhinoXform& rhs ) const
{
  double m[4][4];
  int i, j;
  for (i=0;i<4;i++) for (j=0;j<4;j++)
    m[i][j] = m_xform[i][0]*rhs.m_xform[0][j] + m_xform[i][1]*rhs.m_xform[1][j] + m_xform[i][2]*rhs.m_xform[2][j] + m_xform[i][3]*rhs.m_xform[3][j];
  return CRhinoXform(m);
}

CRhinoXform CRhinoXform::operator*( double rhs ) const
{
  double m[16];
  const double* cp = &m_xform[0][0];
  m[ 0] = rhs * *cp++;
  m[ 1] = rhs * *cp++;
  m[ 2] = rhs * *cp++;
  m[ 3] = rhs * *cp++;
  m[ 4] = rhs * *cp++;
  m[ 5] = rhs * *cp++;
  m[ 6] = rhs * *cp++;
  m[ 7] = rhs * *cp++;
  m[ 8] = rhs * *cp++;
  m[ 9] = rhs * *cp++;
  m[10] = rhs * *cp++;
  m[11] = rhs * *cp++;
  m[12] = rhs * *cp++;
  m[13] = rhs * *cp++;
  m[14] = rhs * *cp++;
  m[15] = rhs * *cp++;
  return CRhinoXform(m);
}

CRhinoXform CRhinoXform::operator+( double rhs ) const
{
  double m[16];
  const double* cp = &m_xform[0][0];
  m[ 0] = rhs + *cp++;
  m[ 1] = rhs + *cp++;
  m[ 2] = rhs + *cp++;
  m[ 3] = rhs + *cp++;
  m[ 4] = rhs + *cp++;
  m[ 5] = rhs + *cp++;
  m[ 6] = rhs + *cp++;
  m[ 7] = rhs + *cp++;
  m[ 8] = rhs + *cp++;
  m[ 9] = rhs + *cp++;
  m[10] = rhs + *cp++;
  m[11] = rhs + *cp++;
  m[12] = rhs + *cp++;
  m[13] = rhs + *cp++;
  m[14] = rhs + *cp++;
  m[15] = rhs + *cp++;
  return CRhinoXform(m);
}

CRhinoXform CRhinoXform::operator-( double rhs ) const
{
  double m[16];
  const double* cp = &m_xform[0][0];
  m[ 0] = *cp++ - rhs;
  m[ 1] = *cp++ - rhs;
  m[ 2] = *cp++ - rhs;
  m[ 3] = *cp++ - rhs;
  m[ 4] = *cp++ - rhs;
  m[ 5] = *cp++ - rhs;
  m[ 6] = *cp++ - rhs;
  m[ 7] = *cp++ - rhs;
  m[ 8] = *cp++ - rhs;
  m[ 9] = *cp++ - rhs;
  m[10] = *cp++ - rhs;
  m[11] = *cp++ - rhs;
  m[12] = *cp++ - rhs;
  m[13] = *cp++ - rhs;
  m[14] = *cp++ - rhs;
  m[15] = *cp++ - rhs;
  return CRhinoXform(m);
}

CRhinoXform& CRhinoXform::operator+=( const CRhinoXform& rhs )
{
  double m[16];
  const double *a = &m_xform[0][0];
  const double *b = &rhs.m_xform[0][0];
  m[ 0] = *a++ + *b++;
  m[ 1] = *a++ + *b++;
  m[ 2] = *a++ + *b++;
  m[ 3] = *a++ + *b++;
  m[ 4] = *a++ + *b++;
  m[ 5] = *a++ + *b++;
  m[ 6] = *a++ + *b++;
  m[ 7] = *a++ + *b++;
  m[ 8] = *a++ + *b++;
  m[ 9] = *a++ + *b++;
  m[10] = *a++ + *b++;
  m[11] = *a++ + *b++;
  m[12] = *a++ + *b++;
  m[13] = *a++ + *b++;
  m[14] = *a++ + *b++;
  m[15] = *a++ + *b++;

  memcpy( &m_xform[0][0], m, 16*sizeof(m_xform[0][0]) );
  return *this;
}

CRhinoXform& CRhinoXform::operator-=( const CRhinoXform& rhs )
{
  double m[16];
  const double *a = &m_xform[0][0];
  const double *b = &rhs.m_xform[0][0];
  m[ 0] = *a++ - *b++;
  m[ 1] = *a++ - *b++;
  m[ 2] = *a++ - *b++;
  m[ 3] = *a++ - *b++;
  m[ 4] = *a++ - *b++;
  m[ 5] = *a++ - *b++;
  m[ 6] = *a++ - *b++;
  m[ 7] = *a++ - *b++;
  m[ 8] = *a++ - *b++;
  m[ 9] = *a++ - *b++;
  m[10] = *a++ - *b++;
  m[11] = *a++ - *b++;
  m[12] = *a++ - *b++;
  m[13] = *a++ - *b++;
  m[14] = *a++ - *b++;
  m[15] = *a++ - *b++;

  memcpy( &m_xform[0][0], m, 16*sizeof(m_xform[0][0]) );
  return *this;
}

CRhinoXform& CRhinoXform::operator*=( const CRhinoXform& rhs )
{
  double m[4][4];
  int i, j;
  for (i=0;i<4;i++) for (j=0;j<4;j++)
    m[i][j] = m_xform[i][0]*rhs.m_xform[0][j] + m_xform[i][1]*rhs.m_xform[1][j] + m_xform[i][2]*rhs.m_xform[2][j] + m_xform[i][3]*rhs.m_xform[3][j];

  memcpy( &m_xform[0][0], &m[0][0], 16*sizeof(m_xform[0][0]) );
  return *this;
}

CRhinoXform& CRhinoXform::operator*=( double rhs )
{
  double m[16];
  const double* cp = &m_xform[0][0];
  m[ 0] = rhs * *cp++;
  m[ 1] = rhs * *cp++;
  m[ 2] = rhs * *cp++;
  m[ 3] = rhs * *cp++;
  m[ 4] = rhs * *cp++;
  m[ 5] = rhs * *cp++;
  m[ 6] = rhs * *cp++;
  m[ 7] = rhs * *cp++;
  m[ 8] = rhs * *cp++;
  m[ 9] = rhs * *cp++;
  m[10] = rhs * *cp++;
  m[11] = rhs * *cp++;
  m[12] = rhs * *cp++;
  m[13] = rhs * *cp++;
  m[14] = rhs * *cp++;
  m[15] = rhs * *cp++;

  memcpy( &m_xform[0][0], m, 16*sizeof(m_xform[0][0]) );
  return *this;
}

CRhinoXform& CRhinoXform::operator+=( double rhs )
{
  double m[16];
  const double* cp = &m_xform[0][0];
  m[ 0] = rhs + *cp++;
  m[ 1] = rhs + *cp++;
  m[ 2] = rhs + *cp++;
  m[ 3] = rhs + *cp++;
  m[ 4] = rhs + *cp++;
  m[ 5] = rhs + *cp++;
  m[ 6] = rhs + *cp++;
  m[ 7] = rhs + *cp++;
  m[ 8] = rhs + *cp++;
  m[ 9] = rhs + *cp++;
  m[10] = rhs + *cp++;
  m[11] = rhs + *cp++;
  m[12] = rhs + *cp++;
  m[13] = rhs + *cp++;
  m[14] = rhs + *cp++;
  m[15] = rhs + *cp++;

  memcpy( &m_xform[0][0], m, 16*sizeof(m_xform[0][0]) );
  return *this;
}

CRhinoXform& CRhinoXform::operator-=( double rhs )
{
  double m[16];
  const double* cp = &m_xform[0][0];
  m[ 0] = *cp++ - rhs;
  m[ 1] = *cp++ - rhs;
  m[ 2] = *cp++ - rhs;
  m[ 3] = *cp++ - rhs;
  m[ 4] = *cp++ - rhs;
  m[ 5] = *cp++ - rhs;
  m[ 6] = *cp++ - rhs;
  m[ 7] = *cp++ - rhs;
  m[ 8] = *cp++ - rhs;
  m[ 9] = *cp++ - rhs;
  m[10] = *cp++ - rhs;
  m[11] = *cp++ - rhs;
  m[12] = *cp++ - rhs;
  m[13] = *cp++ - rhs;
  m[14] = *cp++ - rhs;
  m[15] = *cp++ - rhs;

  memcpy( &m_xform[0][0], m, 16*sizeof(m_xform[0][0]) );
  return *this;
}

double* CRhinoXform::operator[](int i)
{
  return (i>=0&&i<4) ? &m_xform[i][0] : NULL;
}

const double* CRhinoXform::operator[](int i) const
{
  return (i>=0&&i<4) ? &m_xform[i][0] : NULL;
}

static 
BOOL InvertXform( const double src[4][4], double dst[4][4] )
{
	double m[4][8], x, y;
	int i;

	/* row reduction with partial pivoting */

	/* loops unrolled for speed */
  memset( &m[0][0], 0, 32*sizeof(m[0][0]) );

	memcpy( &m[0][0], &src[0][0], 4*sizeof(m[0][0]) );
	memcpy( &m[0][0], &src[1][0], 4*sizeof(m[1][0]) );
	memcpy( &m[0][0], &src[2][0], 4*sizeof(m[2][0]) );
	memcpy( &m[0][0], &src[3][0], 4*sizeof(m[3][0]) );
  m[0][4] = 1.0;
  m[1][5] = 1.0;
  m[2][6] = 1.0;
  m[3][7] = 1.0;
	
	x = fabs(m[0][0]);
	i = 0;
	if ( fabs(m[1][0]) > x ) {i = 1; x = fabs(m[1][0]);}
	if ( fabs(m[2][0]) > x ) {i = 2; x = fabs(m[2][0]);}
	if ( fabs(m[3][0]) > x ) {i = 3; x = fabs(m[3][0]);}
	if (x == 0.0) 
    return FALSE;
	if (i) {
		/* swap rows */
		x = m[i][0]; m[i][0] = m[0][0]; m[0][0] = x;
		x = m[i][1]; m[i][1] = m[0][1]; m[0][1] = x;
		x = m[i][2]; m[i][2] = m[0][2]; m[0][2] = x;
		x = m[i][3]; m[i][3] = m[0][3]; m[0][3] = x;
		x = m[i][4]; m[i][4] = m[0][4]; m[0][4] = x;
		x = m[i][5]; m[i][5] = m[0][5]; m[0][5] = x;
		x = m[i][6]; m[i][6] = m[0][6]; m[0][6] = x;
		x = m[i][7]; m[i][7] = m[0][7]; m[0][7] = x;
	}
	
	/* unitize m[0][0] */
	x *= RHINO_EPSILON;
	y = 1.0/m[0][0];
	/* m[0][0] = 1.0; // debug  */
	m[0][1] *= y; m[0][2] *= y; m[0][3] *= y; m[0][4] *= y; m[0][5] *= y; m[0][6] *= y; m[0][7] *= y;

	/* zero m[1][0] */
	if (fabs(m[1][0]) >  x) {
		y = -m[1][0];         m[1][1] += y*m[0][1]; m[1][2] += y*m[0][2]; m[1][3] += y*m[0][3]; 
		m[1][4] += y*m[0][4]; m[1][5] += y*m[0][5]; m[1][6] += y*m[0][6]; m[1][7] += y*m[0][7];
	}
	/* m[1][0] = 0.0;// debug  */

	/* zero m[2][0] */
	if (fabs(m[2][0]) >  x) {
		y = -m[2][0];         m[2][1] += y*m[0][1]; m[2][2] += y*m[0][2]; m[2][3] += y*m[0][3]; 
		m[2][4] += y*m[0][4]; m[2][5] += y*m[0][5]; m[2][6] += y*m[0][6]; m[2][7] += y*m[0][7];
	}
	/* m[2][0] = 0.0;// debug  */

	/* zero m[3][0] */
	if (fabs(m[3][0]) >  x) {
		y = -m[3][0];         m[3][1] += y*m[0][1]; m[3][2] += y*m[0][2]; m[3][3] += y*m[0][3]; 
		m[3][4] += y*m[0][4]; m[3][5] += y*m[0][5]; m[3][6] += y*m[0][6]; m[3][7] += y*m[0][7];
	}
	/* m[3][0] = 0.0;// debug  */

	x = fabs(m[1][1]);
	i = 1;
	if ( fabs(m[2][1]) > x ) {i = 2; x = fabs(m[2][1]);}
	if ( fabs(m[3][1]) > x ) {i = 3; x = fabs(m[3][1]);}
	if (x == 0.0) 
    return FALSE;
	if (i != 1) {
		/* swap rows */
		x = m[i][1]; m[i][1] = m[1][1]; m[1][1] = x;
		x = m[i][2]; m[i][2] = m[1][2]; m[1][2] = x;
		x = m[i][3]; m[i][3] = m[1][3]; m[1][3] = x;
		x = m[i][4]; m[i][4] = m[1][4]; m[1][4] = x;
		x = m[i][5]; m[i][5] = m[1][5]; m[1][5] = x;
		x = m[i][6]; m[i][6] = m[1][6]; m[1][6] = x;
		x = m[i][7]; m[i][7] = m[1][7]; m[1][7] = x;
	}
	
	/* unitize m[1][1] */
	x *= RHINO_EPSILON;
	y = 1.0/m[1][1];
	/* m[1][1] = 1.0;// debug  */
	m[1][2] *= y; m[1][3] *= y; m[1][4] *= y; m[1][5] *= y; m[1][6] *= y; m[1][7] *= y;

	/* zero m[2][1] */
	if (fabs(m[2][1]) >  x) {
		y = -m[2][1];                               m[2][2] += y*m[1][2]; m[2][3] += y*m[1][3]; 
		m[2][4] += y*m[1][4]; m[2][5] += y*m[1][5]; m[2][6] += y*m[1][6]; m[2][7] += y*m[1][7];
	}
	/* m[2][1] = 0.0;// debug  */

	/* zero m[3][1] */
	if (fabs(m[3][1]) >  x) {
		y = -m[3][1];                               m[3][2] += y*m[1][2]; m[3][3] += y*m[1][3]; 
		m[3][4] += y*m[1][4]; m[3][5] += y*m[1][5]; m[3][6] += y*m[1][6]; m[3][7] += y*m[1][7];
	}
	/* m[3][1] = 0.0;// debug  */

	if (fabs(m[3][2]) > fabs(m[2][2])) {
		/* swap rows */
		x = m[3][2]; m[3][2] = m[2][2]; m[2][2] = x;
		x = m[3][3]; m[3][3] = m[2][3]; m[2][3] = x;
		x = m[3][4]; m[3][4] = m[2][4]; m[2][4] = x;
		x = m[3][5]; m[3][5] = m[2][5]; m[2][5] = x;
		x = m[3][6]; m[3][6] = m[2][6]; m[2][6] = x;
		x = m[3][7]; m[3][7] = m[2][7]; m[2][7] = x;
	}

	/* unitize m[2][2] */
	x = fabs(m[2][2])*RHINO_EPSILON;
	if (x == 0.0) 
    return FALSE;
	y = 1.0/m[2][2];
	/* m[2][2] = 1.0;// debug  */
	m[2][3] *= y; m[2][4] *= y; m[2][5] *= y; m[2][6] *= y; m[2][7] *= y;

	/* zero m[3][2] */
	if (fabs(m[3][2]) >  x) {
		y = -m[3][2];                                                     m[3][3] += y*m[2][3]; 
		m[3][4] += y*m[2][4]; m[3][5] += y*m[2][5]; m[3][6] += y*m[2][6]; m[3][7] += y*m[2][7];
	}
	/* m[3][2] = 0.0;// debug  */

	/* unitize m[3][3] */
	if (m[3][3] == 0.0)
    return FALSE;
	y = 1.0/m[3][3];
	/* m[3][3] = 1.0;// debug  */
	m[3][4] *= y; m[3][5] *= y; m[3][6] *= y; m[3][7] *= y;

	/* left hand matrix is upper trianglular with 1's on the diagonal */

	/* zero m[2][3] */
	if (fabs(m[2][3]) > RHINO_EPSILON) {
		y = -m[2][3];
		m[2][4] += y*m[3][4]; m[2][5] += y*m[3][5]; m[2][6] += y*m[3][6]; m[2][7] += y*m[3][7];
	}
	/* m[2][3] = 0.0;// debug  */

	/* zero m[1][3] */
	if (fabs(m[1][3]) > RHINO_EPSILON) {
		y = -m[1][3];
		m[1][4] += y*m[3][4]; m[1][5] += y*m[3][5]; m[1][6] += y*m[3][6]; m[1][7] += y*m[3][7];
	}
	/* m[1][3] = 0.0;// debug  */

	/* zero m[0][3] */
	if (fabs(m[0][3]) > RHINO_EPSILON) {
		y = -m[0][3];
		m[0][4] += y*m[3][4]; m[0][5] += y*m[3][5]; m[0][6] += y*m[3][6]; m[0][7] += y*m[3][7];
	}
	/* m[0][3] = 0.0;// debug  */

	/* zero m[1][2] */
	if (fabs(m[1][2]) > RHINO_EPSILON) {
		y = -m[1][2];
		m[1][4] += y*m[2][4]; m[1][5] += y*m[2][5]; m[1][6] += y*m[2][6]; m[1][7] += y*m[2][7];
	}
	/* m[1][2] = 0.0;// debug  */

	/* zero m[0][2] */
	if (fabs(m[0][2]) > RHINO_EPSILON) {
		y = -m[0][2];
		m[0][4] += y*m[2][4]; m[0][5] += y*m[2][5]; m[0][6] += y*m[2][6]; m[0][7] += y*m[2][7];
	}
	/* m[0][2] = 0.0;// debug  */

	/* zero m[0][1] */
	if (fabs(m[0][1]) > RHINO_EPSILON) {
		y = -m[0][1];
		m[0][4] += y*m[1][4]; m[0][5] += y*m[1][5]; m[0][6] += y*m[1][6]; m[0][7] += y*m[1][7];
	}
	/* m[0][1] = 0.0;// debug  */

	/* put answer in dst matrix */
	memcpy(dst[0], &m[0][4], 4*sizeof(dst[0][0]) );
	memcpy(dst[1], &m[1][4], 4*sizeof(dst[0][0]) );
	memcpy(dst[2], &m[2][4], 4*sizeof(dst[0][0]) );
	memcpy(dst[3], &m[3][4], 4*sizeof(dst[0][0]) );

	return TRUE;
}

#undef RHINO_EPSILON

CRhinoXform CRhinoXform::Inverse() const
{
  CRhinoXform inv;
  InvertXform( m_xform, inv.m_xform );
  return CRhinoXform(inv.m_xform);
}

BOOL CRhinoXform::Invert()
{
  double inv[4][4];
  const BOOL bOK = InvertXform( ((const CRhinoXform*)this)->m_xform, inv );
  if ( bOK ) {
    memcpy( &m_xform[0][0], &inv[0][0], 16*sizeof(m_xform[0][0]) );
  }
  return bOK;
}

double CRhinoXform::Determinant() const
{
  // loops unrolled for speed
  int i;
  double x, det, m[4][4];
  
  det = 0.0;
  i = 0;
  
  memcpy(&m[0][0],&m_xform[0][0],16*sizeof(m[0][0]));

  x = fabs(m[0][0] );
  if ( fabs(m[1][0]) > x ) {
    i = 1;
    x = fabs(m[1][0] );
  }
  if ( fabs(m[2][0]) > x ) {
    i = 2;
    x = fabs(m[2][0] );
  }
  if ( fabs(m[3][0]) > x ) {
    i = 3;
    x = fabs(m[3][0] );
  }
  if ( x == 0.0 )
    return 0.0;
  if ( i != 0 ) {
    x = m[0][0]; m[0][0] = m[i][0]; m[i][0] = x;
    x = m[0][1]; m[0][1] = m[i][1]; m[i][1] = x;
    x = m[0][2]; m[0][2] = m[i][2]; m[i][2] = x;
    x = m[0][3]; m[0][3] = m[i][3]; m[i][3] = x;
  }
  det = m[0][0];
  x = 1.0/m[0][0];
  //m[0][0] = 1.0;
  m[0][1] *= x;
  m[0][2] *= x;
  m[0][3] *= x;
  x = -m[1][0];
  //m[1][0] += x*m[0][0];
  m[1][1] += x*m[0][1];
  m[1][2] += x*m[0][2];
  m[1][3] += x*m[0][3];
  x = -m[2][0];
  //m[2][0] += x*m[0][0];
  m[2][1] += x*m[0][1];
  m[2][2] += x*m[0][2];
  m[2][3] += x*m[0][3];
  x = -m[3][0];
  //m[3][0] += x*m[0][0];
  m[3][1] += x*m[0][1];
  m[3][2] += x*m[0][2];
  m[3][3] += x*m[0][3];

  x = fabs(m[1][1]);
  if ( fabs(m[2][1]) > x ) {
    i = 2;
    x = fabs(m[2][1] );
  }
  if ( fabs(m[3][1]) > x ) {
    i = 3;
    x = fabs(m[3][1] );
  }
  if ( x == 0.0 )
    return 0.0;
  if ( i != 0 ) {
    //x = m[1][0]; m[1][0] = m[i][0]; m[i][0] = x;
    x = m[1][1]; m[1][1] = m[i][1]; m[i][1] = x;
    x = m[1][2]; m[1][2] = m[i][2]; m[i][2] = x;
    x = m[1][3]; m[1][3] = m[i][3]; m[i][3] = x;
  }
  det *= m[1][1];
  x = 1.0/m[1][1];
  //m[1][1] = 1.0;
  m[1][2] *= x;
  m[1][3] *= x;
  x = -m[2][1];
  //m[2][1] += x*m[1][1];
  m[2][2] += x*m[1][2];
  m[2][3] += x*m[1][3];
  x = -m[3][1];
  //m[3][1] += x*m[1][1];
  m[3][2] += x*m[1][2];
  m[3][3] += x*m[1][3];

  if ( fabs(m[2][2]) >= fabs(m[3][2]) ) {
    if ( m[2][2] == 0.0 )
      return 0.0;
    det *= m[2][2];
    m[2][3] /= m[2][2];
    x = m[3][3] - m[2][3]*m[3][2];
    det *= x;
  }
  else {
    det *= m[3][2];
    m[3][3] /= m[3][2];
    x = m[2][3] - m[2][2]*m[3][3];
    det *= x;
  }

  return det;
}

BOOL CRhinoXform::Apply( double x, double y, double point[2] ) const
{
  BOOL rc = FALSE;
  if ( point ) {
    double w = m_xform[3][0]*x + m_xform[3][1]*y + m_xform[3][3];
    if ( w != 0.0 ) {
      rc = TRUE;
      w = 1.0/w;
    }
    else {
      w = 1.0;
    }
    point[0] = w*(m_xform[0][0]*x + m_xform[0][1]*y + m_xform[0][3]);
    point[1] = w*(m_xform[1][0]*x + m_xform[1][1]*y + m_xform[1][3]);
  }
  return rc;
}

BOOL CRhinoXform::Apply( double x, double y, double z, double point[3] ) const
{
  BOOL rc = FALSE;
  if ( point ) {
    double w = m_xform[3][0]*x + m_xform[3][1]*y + m_xform[3][2]*z + m_xform[3][3];
    if ( w != 0.0 ) {
      rc = TRUE;
      w = 1.0/w;
    }
    else
      w = 1.0;
    point[0] = w*(m_xform[0][0]*x + m_xform[0][1]*y + m_xform[0][2]*z + m_xform[0][3]);
    point[1] = w*(m_xform[1][0]*x + m_xform[1][1]*y + m_xform[1][2]*z + m_xform[1][3]);
    point[2] = w*(m_xform[2][0]*x + m_xform[2][1]*y + m_xform[2][2]*z + m_xform[2][3]);
  }
  return rc;
}

BOOL CRhinoXform::Apply( double x, double y, double z, double w, double point[4] ) const
{
  BOOL rc = FALSE;
  if ( point ) {
    point[0] = m_xform[0][0]*x + m_xform[0][1]*y + m_xform[0][2]*z + m_xform[0][3]*w;
    point[1] = m_xform[1][0]*x + m_xform[1][1]*y + m_xform[1][2]*z + m_xform[1][3]*w;
    point[2] = m_xform[2][0]*x + m_xform[2][1]*y + m_xform[2][2]*z + m_xform[2][3]*w;
    point[3] = m_xform[3][0]*x + m_xform[3][1]*y + m_xform[3][2]*z + m_xform[3][3]*w;
  }
  return rc;
}

BOOL CRhinoXform::ApplyVector( double x, double y, double z, double vector[3] ) const
{
  BOOL rc = FALSE;
  if ( vector ) {
    double w = m_xform[3][0]*x + m_xform[3][1]*y + m_xform[3][2]*z;
    if ( w != 0.0 )
      rc = TRUE;
    vector[0] = m_xform[0][0]*x + m_xform[0][1]*y + m_xform[0][2]*z;
    vector[1] = m_xform[1][0]*x + m_xform[1][1]*y + m_xform[1][2]*z;
    vector[2] = m_xform[2][0]*x + m_xform[2][1]*y + m_xform[2][2]*z;
  }
  return rc;
}

BOOL CRhinoXform::Test() const
{
  // simple inverse and determinant tester.
  int i, j;
  double ztol  = 1.0e-6;
  double d, di, x, e1, e2;
  BOOL rc = TRUE;
  d = Determinant();
  if ( d != 0.0 ) {
    CRhinoXform inv = Inverse();
    di = inv.Determinant();
    if ( fabs(d*di - 1.0) > ztol ) {
      printf("CRhinoXform::Test - determinant*inverse determinant != 1\n");
      rc = FALSE;
    }
    CRhinoXform id1 = *this * inv;
    CRhinoXform id2 = inv * *this;
    for ( i = 0; i < 4; i++ ) for ( j = 0; j < 4; j++ ) {
      x = ( i == j ) ? 1.0 : 0.0;
      e1 = fabs( id1.m_xform[i][j] - x );
      e2 = fabs( id2.m_xform[i][j] - x );
      if ( fabs(e1) > ztol ) {
        printf( "id1[%d][%d] = %g != %g\n",i,j,id1.m_xform[i][i],x);
        rc = FALSE;
      }
      if ( fabs(e2) > ztol ) {
        printf( "id2[%d][%d] = %g != %g\n",i,j,id2.m_xform[i][i],x);
        rc = FALSE;
      }
    }
  }
  return rc;
}

CRhinoXform& CRhinoXform::Identity()
{
  memset( &m_xform[0][1], 0, 14*sizeof(m_xform[0][0]) );
  m_xform[0][0] = m_xform[1][1] = m_xform[2][2] = m_xform[3][3] = 1.0;
  return *this;
}

CRhinoXform& CRhinoXform::Zero()
{
  memset( &m_xform[0][0], 0, 16*sizeof(m_xform[0][0]) );
  return *this;
}

CRhinoXform& CRhinoXform::Transpose()
{
  int i, j;
  double t;
  for ( i = 0; i < 3; i++ ) for ( j = i+1; j < 4; j++ ) {
    t = m_xform[i][j];
    m_xform[i][j] = m_xform[j][i];
    m_xform[j][i] = t;
  }
  return *this;
}

CRhinoXform& CRhinoXform::Translation( const double* d )
{
  double dx = 0.0;
  double dy = 0.0;
  double dz = 0.0;
  if (d) {
    dx = d[0];
    dy = d[1];
    dz = d[2];
  }
  Translation( dx, dy, dz );
  return *this;
}

CRhinoXform& CRhinoXform::Translation( double dx, double dy, double dz )
{
  Identity();
  m_xform[0][3] = dx;
  m_xform[1][3] = dy;
  m_xform[2][3] = dz;
  return *this;
}

CRhinoXform& CRhinoXform::Scale( const double* s )
{
  double sx = 0.0;
  double sy = 0.0;
  double sz = 0.0;
  if (s) {
    sx = s[0];
    sy = s[1];
    sz = s[2];
  }
  Scale( sx, sy, sz );
  return *this;
}

CRhinoXform& CRhinoXform::Scale( double sx, double sy, double sz )
{
  Identity();
  m_xform[0][0] = sx;
  m_xform[1][1] = sy;
  m_xform[2][2] = sz;
  return *this;
}


CRhinoXform& CRhinoXform::Rotation( 
          double ax, double ay, double az, // unit axis
          double cx, double cy, double cz, // center
          double sin_angle, 
          double cos_angle
          )
{
  double a[3] = {ax,ay,az};
  double c[3] = {cx,cy,cz};
  return Rotation( a, c, sin_angle, cos_angle );
}

static double len2d( double x, double y )
{
  double d= 0.0;
  double fx = fabs(x);
  double fy = fabs(y);
  if ( fx > fy ) {
    d = fy/fx;
    d = fx*sqrt(1.0+d*d);
  }
  else if ( fy > fx ){
    d = fx/fy;
    d = fy*sqrt(1.0+d*d);
  }
  return d;
}

static void unitize2d( double x, double y, double* ux, double* uy )
{
  const double eps = 2.0*RHINO_SQRT_EPSILON;
  // carefully turn two numbers into a 2d unit vector
  double s, c, d;
  c = x;
  s = y;
  if ( s == 0.0 ) {
    c = (c < 0.0) ? -1.0 : 1.0;
  }
  else {
    if ( fabs(s) > fabs(c) ) {
      d = c/s;
      d = fabs(s)*sqrt(1.0+d*d);
    }
    else {
      d = s/c;
      d = fabs(c)*sqrt(1.0+d*d);
    }
    d = 1.0/d;
    if ( fabs(d-1.0) > eps ) {
      s *= d;
      c *= d;
    }
    if ( fabs(s) <= eps || fabs(c) >= 1.0-eps ) {
      s = 0.0;
      c = (c < 0.0) ? -1.0 : 1.0;
    }
    else if ( fabs(c) < eps || fabs(s) >= 1.0-eps) {
      c = 0.0;
      s = (s < 0.0) ? -1.0 : 1.0;
    }
  }
  if ( ux ) 
    *ux = c;
  if ( uy )
    *uy = s;
}

CRhinoXform& CRhinoXform::Rotation( 
          const double* UnitAxis, 
          const double* Center,
          double sin_angle, 
          double cos_angle
          )
{
  double a[3], one_minus_cos_angle;
  double axis_x, axis_y, axis_z;
  double center_x, center_y, center_z;

  unitize2d(cos_angle,sin_angle,&cos_angle,&sin_angle);

  memset( &m_xform[0][0], 0, 16*sizeof(m_xform[0][0]) );
  m_xform[0][0] = m_xform[1][1] = m_xform[2][2] = m_xform[3][3] = 1.0;

	if (sin_angle != 0.0 || cos_angle != 1.0) {

	  if (!UnitAxis) {
		  a[0] = a[1] = 0.0; a[2] = 1.0;
		  UnitAxis = a;
	  }

    axis_x = UnitAxis[0]; axis_y = UnitAxis[1]; axis_z = UnitAxis[2];
  
    one_minus_cos_angle = 1.0 - cos_angle;

    m_xform[0][0] = axis_x*axis_x*one_minus_cos_angle + cos_angle;
    m_xform[0][1] = axis_x*axis_y*one_minus_cos_angle - axis_z*sin_angle;
    m_xform[0][2] = axis_x*axis_z*one_minus_cos_angle + axis_y*sin_angle;

    m_xform[1][0] = axis_y*axis_x*one_minus_cos_angle + axis_z*sin_angle;
    m_xform[1][1] = axis_y*axis_y*one_minus_cos_angle + cos_angle;
    m_xform[1][2] = axis_y*axis_z*one_minus_cos_angle - axis_x*sin_angle;

    m_xform[2][0] = axis_z*axis_x*one_minus_cos_angle - axis_y*sin_angle;
    m_xform[2][1] = axis_z*axis_y*one_minus_cos_angle + axis_x*sin_angle;
    m_xform[2][2] = axis_z*axis_z*one_minus_cos_angle + cos_angle;

    if (Center) {
      center_x = Center[0]; 
      center_y = Center[1]; 
      center_z = Center[2];
      m_xform[0][3] = -((m_xform[0][0]-1.0)*center_x + m_xform[0][1]*center_y + m_xform[0][2]*center_z);
      m_xform[1][3] = -(m_xform[1][0]*center_x + (m_xform[1][1]-1.0)*center_y + m_xform[1][2]*center_z);
      m_xform[2][3] = -(m_xform[2][0]*center_x + m_xform[2][1]*center_y + (m_xform[2][2]-1.0)*center_z);
    }

    m_xform[3][0] = m_xform[3][1] = m_xform[3][2] = 0.0;
    m_xform[3][3] = 1.0;
  }

	return *this;
}

BOOL RhinoGetRotationAngles( 
       const double* X, const double* Y, const double* Z,
       double* angle1, double* angle2, double* angle3 
       )
{
  // double a1 = 0.0;  // rotation about world Z
  // double a2 = 0.0;  // rotation about world X ( 0 <= a2 <= pi )
  // double a3 = 0.0;  // rotation about world Z
  BOOL bValidFrame = FALSE;
  double sin_a1 = 0.0;
  double cos_a1 = 1.0;
  double sin_a2 = 0.0;
  double cos_a2 = 1.0;
  double sin_a3 = 0.0;
  double cos_a3 = 1.0;

  // If si = sin(ai) and ci = cos(ai), then the relationship between the camera
  // frame and the angles is defined by the matrix equation C = R3*R2*R1, where:
  //
  //      c1 -s1  0        1   0   0         c3 -s3  0
  // R1 = s1  c1  0  R2 =  0  c2 -s2   R3 =  s3  c3  0
  //       0   0  1        0  s2  c2          0   0  1
  //
  //     CamX[0]  CamY[0] CamZ[0]
  // C = CamX[1]  CamY[1] CamZ[1]
  //     CamX[2]  CamY[2] CamZ[2]
  //
  //              .       .     s2*s3
  //
  // R3*R2*R1 =   .       .    -s2*c3
  //
  //            s1*s2   c1*s2    c2
  //

  if ( X && Y && Z ) {
    // don't attempt to work with slop
    const double eps = 8.0*RHINO_SQRT_EPSILON;
    double dx,dy,dz,d;
    dx = X[0]*X[0]+X[1]*X[1]+X[2]*X[2];
    dy = Y[0]*Y[0]+Y[1]*Y[1]+Y[2]*Y[2];
    dz = Z[0]*Z[0]+Z[1]*Z[1]+Z[2]*Z[2];
    if ( fabs(dx-1.0) <= eps && fabs(dy-1.0) <= eps && fabs(dz-1.0) <= eps ) {
      dx = X[0]*Y[0]+X[1]*Y[1]+X[2]*Y[2];
      dy = Y[0]*Z[0]+Y[1]*Z[1]+Y[2]*Z[2];
      dz = Z[0]*X[0]+Z[1]*X[1]+Z[2]*X[2];
      if ( fabs(dx) <= eps && fabs(dy) <= eps && fabs(dz) <= eps ) {
        d = X[0]*(Y[1]*Z[2]-Y[2]*Z[1]) + X[1]*(Y[2]*Z[0]-Y[0]*Z[2]) + X[2]*(Y[0]*Z[1]-Y[1]*Z[0]);
        bValidFrame = (d > 0.0);
      }
    }
  }

  if ( bValidFrame ) {
    // Usually "Z" = opposite of unitized camera direction.
    //         "Y" = camera up made ortho to "Z" and unitized.
    //         "X" = YxZ.
    // So, when possible, I solve for angles in terms
    // of "Z" and "Y" since "X" will generally have the most noise.
    //         
    // Use C = R3*R2*R1 to get sin(a2), cos(a2).
    cos_a2 = Z[2];
    sin_a2 = len2d(Z[0],Z[1]);
    unitize2d(cos_a2,sin_a2,&cos_a2,&sin_a2); // kill noise

    if ( sin_a2 > 0.0 ) {
      // use bottom row to get angle1.
      sin_a1 = X[2];
      cos_a1 = Y[2];
      unitize2d(cos_a1,sin_a1,&cos_a1,&sin_a1); // kill noise

      // use right column to get angle3
      cos_a3 = -Z[1];
      sin_a3 =  Z[0];
      unitize2d(cos_a3,sin_a3,&cos_a3,&sin_a3); // kill noise
    }
    else if ( cos_a2 == 1.0 ) {
      // R2 = identity and C determines (angle1+angle3)
      //      arbitrarily set angle1 = 0.
      cos_a3 =  Y[1]; // = cos(angle3+angle1)
      sin_a3 = -Y[0]; // = sin(angle3+angle1)
    }
    else if ( cos_a2 == -1.0 ) {
      // R2 = [1 0 0 / 0 -1 0/ 0 0 -1] and C determines (angle3-angle1)
      //      arbitrarily set angle1 = 0
      cos_a3 = -Y[1]; // = cos(angle3-angle1)
      sin_a3 =  Y[0]; // = sin(angle3-angle1)
    }
  }
  
  if ( angle1 )
    *angle1 = atan2( sin_a1, cos_a1 );
  if ( angle2 )
    *angle2 = atan2( sin_a2, cos_a2 );
  if ( angle3 )
    *angle3 = atan2( sin_a3, cos_a3 );

  return bValidFrame;

}


CRhinoXform& CRhinoXform::WorldToFrame(
                          const double* center,
                          const double* X,
                          const double* Y,
                          const double* Z
                          )
{
  // compute world to frame coordinate xform
  m_xform[0][0] = X[0]; m_xform[0][1] = X[1]; m_xform[0][2] = X[2];
  m_xform[0][3] = -(X[0]*center[0] + X[1]*center[1] + X[2]*center[2]);
  m_xform[1][0] = Y[0]; m_xform[1][1] = Y[1]; m_xform[1][2] = Y[2];
  m_xform[1][3] = -(Y[0]*center[0] + Y[1]*center[1] + Y[2]*center[2]);
  m_xform[2][0] = Z[0]; m_xform[2][1] = Z[1]; m_xform[2][2] = Z[2];
  m_xform[2][3] = -(Z[0]*center[0] + Z[1]*center[1] + Z[2]*center[2]);
  m_xform[3][0] = m_xform[3][1] = m_xform[3][2] = 0.0; m_xform[3][3] = 1.0;
  return *this;
}
  
CRhinoXform& CRhinoXform::FrameToWorld(
                          const double* center,
                          const double* X,
                          const double* Y,
                          const double* Z
                          )
{
  // compute frame to world coordinate xform
  m_xform[0][0] = X[0]; m_xform[0][1] = Y[0]; m_xform[0][2] = Z[0]; 
  m_xform[0][3] = center[0];
  m_xform[1][0] = X[1]; m_xform[1][1] = Y[1]; m_xform[1][2] = Z[1]; 
  m_xform[1][3] = center[1];
  m_xform[2][0] = X[2]; m_xform[2][1] = Y[2]; m_xform[2][2] = Z[2]; 
  m_xform[2][3] = center[2];
  m_xform[3][0] = m_xform[3][1] = m_xform[3][2] = 0.0; m_xform[3][3] = 1.0;
  return *this;
}

CRhinoXform& CRhinoXform::CameraToClip(
                          BOOL bPerspective,
                          double left,      double right,
                          double bottom,    double top,
                          double near_dist, double far_dist
                          )
{
  double d;

  if ( left == right ) {
    left  = -1.0;
    right =  1.0;
  }
  if ( bottom == top ) {
    bottom = -1.0;
    top    =  1.0;
  }
  if ( near_dist == far_dist ) {
    near_dist = 1.0;
    far_dist = 3.0;
  }
  if ( bPerspective ) {
    if ( near_dist == 0.0 )
      near_dist = 0.01*far_dist;
    else if (far_dist == 0.0 )
      far_dist = 100.0*near_dist;
  }

  if ( bPerspective ) {
    //  2n/(r-l)     0        (r+l)/(r-l)     0
    //    0        2n/(t-b)   (t+b)/(t-b)     0
    //    0          0        (f+n)/(f-n)  2fn/(f-n)
    //    0          0            -1          0
    d = 1.0/(right-left);
    m_xform[0][0] = 2.0*near_dist*d; 
    m_xform[0][2] = (right+left)*d; 
    m_xform[0][1] = m_xform[0][3] = 0.0;

    d = 1.0/(top-bottom);
    m_xform[1][1] = 2.0*near_dist*d; 
    m_xform[1][2] = (top+bottom)*d; 
    m_xform[1][0] = m_xform[1][3] = 0.0;

    d = 1.0/(far_dist-near_dist);
    m_xform[2][2] = (far_dist+near_dist)*d; 
    m_xform[2][3] = 2.0*near_dist*far_dist*d; 
    m_xform[2][0] = m_xform[2][1] = 0.0;

    m_xform[3][0] = m_xform[3][1] = m_xform[3][3] = 0.0; m_xform[3][2] = -1.0;
  }
  else {
    d = 1.0/(left-right);
    m_xform[0][0] = -2.0*d; m_xform[0][3] = (left+right)*d; m_xform[0][1] = m_xform[0][2] = 0.0;
    d = 1.0/(bottom-top);
    m_xform[1][1] = -2.0*d; m_xform[1][3] = (bottom+top)*d; m_xform[1][0] = m_xform[1][2] = 0.0;
    d = 1.0/(far_dist-near_dist);
    m_xform[2][2] = 2.0*d;  m_xform[2][3] = (far_dist+near_dist)*d;   m_xform[2][0] = m_xform[2][1] = 0.0;
    m_xform[3][0] = m_xform[3][1] = m_xform[3][2] = 0.0; m_xform[3][3] = 1.0;
  }
  return *this;
}

CRhinoXform& CRhinoXform::ClipToCamera(
                          BOOL bPerspective,
                          double left,      double right,
                          double bottom,    double top,
                          double near_dist, double far_dist
                          )
{
  double d;
  if ( left == right ) {
    left  = -1.0;
    right =  1.0;
  }
  if ( bottom == top ) {
    bottom = -1.0;
    top    =  1.0;
  }
  if ( near_dist == far_dist ) {
    near_dist = 1.0;
    far_dist = 3.0;
  }
  if ( bPerspective ) {
    if ( near_dist == 0.0 )
      near_dist = 0.01*far_dist;
    else if (far_dist == 0.0 )
      far_dist = 100.0*near_dist;
  }

  if ( bPerspective ) {
    //  (r-l)/(2n)       0            0       (r+l)/(2n)
    //    0         (t-b)/(2n)        0       (t+b)/(2n)
    //    0              0            0           -1
    //    0              0       (f-n)/(2fn)  (f+n)/(2fn)
    d = 0.5/near_dist;
    m_xform[0][0] = d*(right-left); 
    m_xform[0][3] = d*(right+left); 
    m_xform[0][1] = m_xform[0][2] = 0.0;

    m_xform[1][1] = d*(top-bottom); 
    m_xform[1][3] = d*(top+bottom); 
    m_xform[1][0] = m_xform[1][2] = 0.0;

    m_xform[2][0] = m_xform[2][1] = m_xform[2][2] = 0.0; m_xform[2][3] = -1.0;

    d /= far_dist;
    m_xform[3][2] = d*(far_dist-near_dist); 
    m_xform[3][3] = d*(far_dist+near_dist); 
    m_xform[3][0] = m_xform[3][1] = 0.0;
  }
  else {
    m_xform[0][0] = 0.5*(right-left); m_xform[0][3] = 0.5*(right+left); m_xform[0][1] = m_xform[0][2] = 0.0;
    m_xform[1][1] = 0.5*(top-bottom); m_xform[1][3] = 0.5*(top+bottom); m_xform[1][0] = m_xform[1][2] = 0.0;
    m_xform[2][2] = 0.5*(far_dist-near_dist);   m_xform[2][3] = -0.5*(far_dist+near_dist);  m_xform[2][0] = m_xform[2][1] = 0.0;
    m_xform[3][0] = m_xform[3][1] = m_xform[3][2] = 0.0; m_xform[3][3] = 1.0;
  }

  return *this;
}

CRhinoXform& CRhinoXform::ClipToScreen(
      double port_left,   double port_right,
      double port_bottom, double port_top,
      double port_near, double port_far
      )
{
  // trap bogus input to avoid crashing
  if ( port_left == port_right ) {
    port_left = 0;
    port_right = 100;
  }
  if ( port_bottom == port_top ) {
    port_bottom = 0;
    port_top = 100;
  }

  m_xform[0][0] = 0.5*(port_right-port_left);
  m_xform[0][3] = 0.5*(port_right+port_left);
  m_xform[0][1] = m_xform[0][2] = 0.0;

  m_xform[1][1] = 0.5*(port_top-port_bottom);
  m_xform[1][3] = 0.5*(port_top+port_bottom);
  m_xform[1][0] = m_xform[1][2] = 0.0;

  if (port_far != port_near) {
    m_xform[2][2] = 0.5*(port_near-port_far);
    m_xform[2][3] = 0.5*(port_near+port_far);
  }
  else {
    m_xform[2][2] = 1.0;
    m_xform[2][3] = 0.0;
  }
  m_xform[2][0] = m_xform[2][1] = 0.0;

  m_xform[3][0] = m_xform[3][1] = m_xform[3][2] = 0.0; 
  m_xform[3][3] = 1.0;

  return *this;
}

CRhinoXform& CRhinoXform::ScreenToClip(
      double port_left,   double port_right,
      double port_bottom, double port_top,
      double port_near, double port_far
      )
{
  CRhinoXform c2s;
  c2s.ClipToScreen( port_left, port_right, port_bottom, port_top, port_near, port_far );
  m_xform[0][0] = 1.0/c2s[0][0]; m_xform[0][3] = -c2s[0][3]*m_xform[0][0];
  m_xform[0][1] = m_xform[0][2] = 0.0;

  m_xform[1][1] = 1.0/c2s[1][1]; m_xform[1][3] = -c2s[1][3]*m_xform[1][1];
  m_xform[1][0] = m_xform[1][2] = 0.0;

  m_xform[2][2] = 1.0/c2s[2][2]; m_xform[2][3] = -c2s[2][3]*m_xform[2][2];
  m_xform[2][0] = m_xform[2][1] = 0.0;

  m_xform[3][0] = m_xform[3][1] = m_xform[3][2] = 0.0; 
  m_xform[3][3] = 1.0;

  return *this;
}


int CRhinoXform::ClipFlag4d( const double* point ) const
{
  if ( !point )
    return 1|2|4|8|16|32;
  int clip = 0;
  double x = m_xform[0][0]*point[0] + m_xform[0][1]*point[1] + m_xform[0][2]*point[2] + m_xform[0][3]*point[3];
  double y = m_xform[1][0]*point[0] + m_xform[1][1]*point[1] + m_xform[1][2]*point[2] + m_xform[1][3]*point[3];
  double z = m_xform[2][0]*point[0] + m_xform[2][1]*point[1] + m_xform[2][2]*point[2] + m_xform[2][3]*point[3];
  double w = m_xform[3][0]*point[0] + m_xform[3][1]*point[1] + m_xform[3][2]*point[2] + m_xform[3][3]*point[3];
  if ( point[3] < 0.0 ) {
    x = -x; y = -y; z = -z; w = -w;
  }
  if ( x <= -w )
    clip |= 1;
  else if ( x >= w )
    clip |= 2;
  if ( y <= -w )
    clip |= 4;
  else if ( y >= w )
    clip |= 8;
  if ( z <= -w )
    clip |= 16;
  else if ( z >= w )
    clip |= 32;
  return clip;
}

int CRhinoXform::ClipFlag3d( const double* point ) const
{
  if ( !point )
    return 1|2|4|8|16|32;
  int clip = 0;
  const double x = m_xform[0][0]*point[0] + m_xform[0][1]*point[1] + m_xform[0][2]*point[2] + m_xform[0][3];
  const double y = m_xform[1][0]*point[0] + m_xform[1][1]*point[1] + m_xform[1][2]*point[2] + m_xform[1][3];
  const double z = m_xform[2][0]*point[0] + m_xform[2][1]*point[1] + m_xform[2][2]*point[2] + m_xform[2][3];
  const double w = m_xform[3][0]*point[0] + m_xform[3][1]*point[1] + m_xform[3][2]*point[2] + m_xform[3][3];
  if ( x <= -w )
    clip |= 1;
  else if ( x >= w )
    clip |= 2;
  if ( y <= -w )
    clip |= 4;
  else if ( y >= w )
    clip |= 8;
  if ( z <= -w )
    clip |= 16;
  else if ( z >= w )
    clip |= 32;
  return clip;
}

int CRhinoXform::ClipFlag4d( int count, int stride, const double* point ) const
{
  int clip = 1|2|4|8|16|32;
  if ( count > 0 && stride >= 4 && point ) {
    for ( /*empty*/; clip && count--; point += stride ) {
      clip &= ClipFlag4d( point );
    }
  }
  return clip;
}

int CRhinoXform::ClipFlag3d( int count, int stride, const double* point ) const
{
  int clip = 1|2|4|8|16|32;
  if ( count > 0 && stride >= 3 && point ) {
    for ( /*empty*/; clip && count--; point += stride ) {
      clip &= ClipFlag3d( point );
    }
  }
  return clip;
}

int CRhinoXform::ClipFlag3dBox( const double* boxmin, const double* boxmax ) const
{
  int clip = 1|2|4|8|16|32;
  double point[3];
  int i,j,k;
  if ( boxmin && boxmax ) {
    for (i=0;i<2;i++) {
      point[0] = (i)?boxmax[0]:boxmin[0];
      for (j=0;j<2;j++) {
        point[1] = (j)?boxmax[1]:boxmin[1];
        for (k=0;k<2;k++) {
          point[2] = (k)?boxmax[2]:boxmin[2];
          clip &= ClipFlag3d(point);
          if ( !clip )
            return 0;
        }
      }
    }
  }
  return clip;
}

