#include <stdlib.h>
#include <memory.h>
#include "rhmem.h"

static int rhmem_error(int);
static rhmem_error_handler rhmem_error_handler_func = 0;

static int rhmem_error(int error_number)
{
  return (rhmem_error_handler_func) ? rhmem_error_handler_func(error_number) : 0;
}

void* rhmalloc( size_t sz )
{
  void* p;
  if ( sz>0 ) 
  {
    for(;;) 
    {
      p = malloc(sz);
      if ( p )
        break;
      if ( !rhmem_error(0) )
        break;
    }
  }
  else {
    p = 0;
  }
  return p;
}

void* rhcalloc( size_t num, size_t sz )
{
  void* p;
  if ( num >= 0 && sz>0 ) 
  {
    for(;;) 
    {
      p = calloc(num,sz);
      if ( p )
        break;
      if ( !rhmem_error(0) )
        break;
    }
  }
  else {
    p = 0;
  }
  return p;
}

void rhfree( void* p )
{
  if ( p ) 
  {
    free(p);
  }
}

void* rhrealloc( void* memblock, size_t sz )
{
  void* p;
  if ( sz<=0 ) 
  {
    rhfree(memblock);
    p = 0;
  }
  else if ( !memblock ) 
  {
    p = rhmalloc(sz);
  }
  else 
  {
    for(;;) 
    {
      p = realloc(memblock,sz);
      if ( p )
        break;
      if ( !rhmem_error(0) )
        break;
    }
  }
  return p;
}

void* rhmemdup( const void* src, size_t sz )
{
  void* p;
  if ( src && sz>0 ) 
  {
    p = rhmalloc(sz);
    if (p)
      memcpy(p,src,sz);
  }
  else {
    p = 0;
  }
  return p;
}

char* rhstrdup( const char* src )
{
  char* p;
  size_t sz;
  if ( src ) 
  {
    for ( sz=0;*src++;sz++)
      ; /* empty for body */
    sz++;
    p = (char*)rhmemdup( src-sz, sz*sizeof(*src) );
  }
  else 
  {
    p = 0;
  }
  return p;
}

unsigned char* rhmbsdup( const unsigned char* src )
{
  unsigned char* p;
  size_t sz; // sz = number of bytes to dup (>=_mbclen(scr))
  if ( src ) 
  {
    for ( sz=0;*src++;sz++)
      ; /* empty for body */
    sz++;
    p = (unsigned char*)rhmemdup( src-sz, sz*sizeof(*src) );
  }
  else 
  {
    p = 0;
  }
  return p;
}

#if defined(_WCHAR_T_DEFINED)
wchar_t* rhwcsdup( const wchar_t* src )
{
  wchar_t* p;
  size_t sz;
  if ( src ) 
  {
    for ( sz=0;*src++;sz++)
      ; /* empty for body */
    sz++;
    p = (wchar_t*)rhmemdup( src-sz, sz*sizeof(*src) );
  }
  else 
  {
    p = 0;
  }
  return p;
}
#endif

rhmem_error_handler rhmem_register_error_handler( rhmem_error_handler f )
{
  rhmem_error_handler oldf = rhmem_error_handler_func;
  rhmem_error_handler_func = f;
  return oldf;
}
